@@ -764,7 +764,8 @@ static void reset_gmdhpf_defaults ()
  
 { FILE *adp_fp;
   int i;
-  char *adprc_file;
+  char adprc_file [MAXPATHLEN];
+  char *adprc_dir;
 
 #define RC_SUFFIX "/.adprc"
 
@@ -778,7 +779,7 @@ static void reset_gmdhpf_defaults ()
 
 try_phome:
 
-  if ((adprc_file = getenv("PHOME")) == NULL)
+  if ((adprc_dir = getenv("PHOME")) == NULL)
     {
 #ifdef DEBUG
 printf("$PHOME (home directory of ADAPTOR) not set; no configuration file found
@@ -787,9 +788,11 @@ printf("$PHOME (home directory of ADAPTOR) not set; no configuration file found
      goto try_home;
     }
  
-  strcat (adprc_file,RC_SUFFIX);
+  strcpy (adprc_file, adprc_dir);
+  strcat (adprc_file, RC_SUFFIX);
 
   if ((adp_fp = fopen(adprc_file,"r")) != NULL)
+
      {
 #ifdef DEBUG
 printf("reading configuration file %s\n",adprc_file);
@@ -797,8 +800,8 @@ printf("reading configuration file %s\n",adprc_file);
        read_gmdhpf_adprc_values (adp_fp);
        fclose (adp_fp);
 
-       strcat (ADPRC_FILES,adprc_file);
-       strcat (ADPRC_FILES," ");
+       strcat (ADPRC_FILES, adprc_file);
+       strcat (ADPRC_FILES, " ");
      }
 
 
@@ -806,7 +809,8 @@ printf("reading configuration file %s\n",adprc_file);
 
 try_home: 
 
-  if ((adprc_file = getenv("HOME")) == NULL)
+  if ((adprc_dir = getenv("HOME")) == NULL)
+
     {
 #ifdef DEBUG
 printf ("$HOME (installation directory) not set\n");
@@ -814,6 +818,7 @@ printf ("$HOME (installation directory) not set\n");
      goto try_work;
     }
 
+  strcpy (adprc_file, adprc_dir);
   strcat (adprc_file, RC_SUFFIX);
 
   if ((adp_fp = fopen(adprc_file,"r")) != NULL)
@@ -824,8 +829,8 @@ printf("reading configuration file %s\n",adprc_file);
        read_gmdhpf_adprc_values (adp_fp);
        fclose (adp_fp);
 
-       strcat (ADPRC_FILES,adprc_file);
-       strcat (ADPRC_FILES," ");
+       strcat (ADPRC_FILES, adprc_file);
+       strcat (ADPRC_FILES, " ");
       }
 
 
@@ -835,6 +840,7 @@ try_work:
   strcat (adprc_file, RC_SUFFIX);
 
   if ((adp_fp = fopen(adprc_file,"r")) != NULL)
+
      {
 #ifdef DEBUG
 printf("reading configuration file %s\n",adprc_file);
