@@ -1,6 +1,13 @@
 # include "CodeDescriptors.h"
-# include "yyCodeDescriptors.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,27 +15,24 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 33 "CodeDescriptors.puma"
+/* line 33 "CodeDescriptors.puma" */
 
 
 # define  MODULE      "CodeDescriptors"
@@ -36,7 +40,7 @@
 # include <stdio.h>
 
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 # include "protocol.h"
 
 # include "Types.h"       /* ArrayFormals */
@@ -50,11 +54,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* CodeDescriptors_Exit) () = yyExit;
+# include "yyCodeDescriptors.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* CodeDescriptors_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -63,27 +81,28 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module CodeDescriptors, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module CodeDescriptors, routine %s failed\n",
+  yyFunction);
  CodeDescriptors_Exit ();
 }
 
-tTree DalibArrayOverlapUpdate ARGS((tTree var, tTree size));
-static tTree OverlapSizeParams ARGS((tTree size));
-static tTree DalibSectionCreate ARGS((int section_id, tTree var));
-tTree DalibSectionDescriptor ARGS((int section_id));
-static tTree DalibSectionFree ARGS((int section_id));
-void DalibSectionTranslation ARGS((int nr, tTree var, tTree * yyP3, tTree * yyP2, tTree * yyP1));
-static tTree DalibLocalCreate ARGS((int local_id, int pack, tTree var));
-static tTree DalibLocalDescriptor ARGS((int local_id));
-static tTree DalibLocalFree ARGS((int local_id));
-void DalibLocalTranslation ARGS((int nr, int pack, tTree var, tTree * yyP6, tTree * yyP5, tTree * yyP4));
-tTree DalibPid ARGS(());
-tTree DalibFindOwner ARGS((tTree topology, tTree specs));
-static tTree FindOwnerParams ARGS((tTree specs));
-tTree DalibDimBroadcast ARGS((tTree var, tTree on_top, tTree on_val));
-tTree DalibNodeGet ARGS((tTree var));
-tTree DalibBroadcast ARGS((tTree var, tTree from));
-tTree DalibLocExchange ARGS((tTree var));
+tTree DalibArrayOverlapUpdate ARGS ((tTree var, tTree size));
+static tTree OverlapSizeParams ARGS ((tTree size));
+static tTree DalibSectionCreate ARGS ((int section_id, tTree var));
+tTree DalibSectionDescriptor ARGS ((int section_id));
+static tTree DalibSectionFree ARGS ((int section_id));
+void DalibSectionTranslation ARGS ((int nr, tTree var, tTree * yyP3, tTree * yyP2, tTree * yyP1));
+static tTree DalibLocalCreate ARGS ((int local_id, int pack, tTree var));
+static tTree DalibLocalDescriptor ARGS ((int local_id));
+static tTree DalibLocalFree ARGS ((int local_id));
+void DalibLocalTranslation ARGS ((int nr, int pack, tTree var, tTree * yyP6, tTree * yyP5, tTree * yyP4));
+tTree DalibPid ARGS ((void));
+tTree DalibFindOwner ARGS ((tTree topology, tTree specs));
+static tTree FindOwnerParams ARGS ((tTree specs));
+tTree DalibDimBroadcast ARGS ((tTree var, tTree on_top, tTree on_val));
+tTree DalibNodeGet ARGS ((tTree var));
+tTree DalibBroadcast ARGS ((tTree var, tTree from));
+tTree DalibLocExchange ARGS ((tTree var));
 
 tTree DalibArrayOverlapUpdate
 # if defined __STDC__ | defined __cplusplus
@@ -95,48 +114,38 @@ tTree DalibArrayOverlapUpdate
 # endif
 {
   if (size->Kind == kBTE_EMPTY) {
-# line 66 "CodeDescriptors.puma"
+/* line 66 "CodeDescriptors.puma" */
    return NoTree;
 
   }
   if (var->Kind == kINDEXED_VAR) {
-# line 73 "CodeDescriptors.puma"
+/* line 73 "CodeDescriptors.puma" */
  {
   tTree param;
   tTree paramlist;
   tTree call;
   int sec_id;
   {
-# line 75 "CodeDescriptors.puma"
-
-# line 76 "CodeDescriptors.puma"
-
-# line 77 "CodeDescriptors.puma"
-
-# line 78 "CodeDescriptors.puma"
-
-# line 80 "CodeDescriptors.puma"
+/* line 80 "CodeDescriptors.puma" */
    sec_id = 1;
-# line 82 "CodeDescriptors.puma"
+/* line 82 "CodeDescriptors.puma" */
    paramlist = OverlapSizeParams (size);
-# line 84 "CodeDescriptors.puma"
+/* line 84 "CodeDescriptors.puma" */
    param = DalibSectionDescriptor (sec_id);
-# line 85 "CodeDescriptors.puma"
+/* line 85 "CodeDescriptors.puma" */
    paramlist = mBTP_LIST (param, paramlist);
-# line 87 "CodeDescriptors.puma"
+/* line 87 "CodeDescriptors.puma" */
    call = mPROC_OBJ (MakeDalibId ("overlap_update"));
-# line 88 "CodeDescriptors.puma"
+/* line 88 "CodeDescriptors.puma" */
    call = mACF_BASIC (mCALL_STMT (call, paramlist));
   }
-  {
    return mACF_LIST (DalibSectionCreate (sec_id, var), mACF_LIST (call, mACF_LIST (DalibSectionFree (sec_id), NoTree)));
-  }
  }
 
   }
-# line 95 "CodeDescriptors.puma"
+/* line 95 "CodeDescriptors.puma" */
   {
-# line 96 "CodeDescriptors.puma"
+/* line 96 "CodeDescriptors.puma" */
    failure_protocol (MODULE, "DalibArrayOverlapUpdate", var);
   }
    return NoTree;
@@ -152,23 +161,23 @@ static tTree OverlapSizeParams
 # endif
 {
   if (size->Kind == kBTE_EMPTY) {
-# line 102 "CodeDescriptors.puma"
+/* line 102 "CodeDescriptors.puma" */
    return mBTP_EMPTY ();
 
   }
   if (size->Kind == kBTE_LIST) {
-# line 106 "CodeDescriptors.puma"
+/* line 106 "CodeDescriptors.puma" */
    return ConcatParams (OverlapSizeParams (size->BTE_LIST.Elem), OverlapSizeParams (size->BTE_LIST.Next));
 
   }
   if (size->Kind == kSLICE_EXP) {
-# line 111 "CodeDescriptors.puma"
-   return mBTP_LIST (ExpToVarParam (size->SLICE_EXP.START), mBTP_LIST (ExpToVarParam (size->SLICE_EXP.STOP), NoTree));
+/* line 111 "CodeDescriptors.puma" */
+   return mBTP_LIST (ExpToVarParam (size->SLICE_EXP.FIRST), mBTP_LIST (ExpToVarParam (size->SLICE_EXP.STOP), NoTree));
 
   }
-# line 116 "CodeDescriptors.puma"
+/* line 116 "CodeDescriptors.puma" */
   {
-# line 117 "CodeDescriptors.puma"
+/* line 117 "CodeDescriptors.puma" */
    failure_protocol (MODULE, "OverlapSizeParams", size);
   }
    return NoTree;
@@ -185,42 +194,34 @@ static tTree DalibSectionCreate
 # endif
 {
   if (var->Kind == kINDEXED_VAR) {
-# line 141 "CodeDescriptors.puma"
+/* line 141 "CodeDescriptors.puma" */
  {
   tTree param;
   tTree params;
   tTree call;
   {
-# line 143 "CodeDescriptors.puma"
-
-# line 144 "CodeDescriptors.puma"
-
-# line 145 "CodeDescriptors.puma"
-
-# line 147 "CodeDescriptors.puma"
+/* line 147 "CodeDescriptors.puma" */
    params = MakeSectionParams (var->INDEXED_VAR.IND_EXPS);
-# line 148 "CodeDescriptors.puma"
+/* line 148 "CodeDescriptors.puma" */
    param = mVAR_PARAM (MakeUsedVarA (TreeVarName (var->INDEXED_VAR.IND_VAR), "_DSP"));
-# line 149 "CodeDescriptors.puma"
+/* line 149 "CodeDescriptors.puma" */
    params = mBTP_LIST (param, params);
-# line 150 "CodeDescriptors.puma"
+/* line 150 "CodeDescriptors.puma" */
    param = DalibSectionDescriptor (section_id);
-# line 151 "CodeDescriptors.puma"
+/* line 151 "CodeDescriptors.puma" */
    params = mBTP_LIST (param, params);
-# line 152 "CodeDescriptors.puma"
+/* line 152 "CodeDescriptors.puma" */
    call = mPROC_OBJ (MakeDalibId ("section_create"));
-# line 153 "CodeDescriptors.puma"
+/* line 153 "CodeDescriptors.puma" */
    call = mACF_BASIC (mCALL_STMT (call, params));
   }
-  {
    return call;
-  }
  }
 
   }
-# line 158 "CodeDescriptors.puma"
+/* line 158 "CodeDescriptors.puma" */
   {
-# line 159 "CodeDescriptors.puma"
+/* line 159 "CodeDescriptors.puma" */
    failure_protocol (MODULE, "DalibSectionCreate", var);
   }
    return NoTree;
@@ -235,20 +236,16 @@ tTree DalibSectionDescriptor
  register int section_id;
 # endif
 {
-# line 171 "CodeDescriptors.puma"
+/* line 171 "CodeDescriptors.puma" */
  {
   tTree var;
   {
-# line 172 "CodeDescriptors.puma"
-
-# line 174 "CodeDescriptors.puma"
+/* line 174 "CodeDescriptors.puma" */
    var = MakeUsedVarAn (MakeIdent ("ISEC", 4), "_DSP", section_id);
-# line 175 "CodeDescriptors.puma"
+/* line 175 "CodeDescriptors.puma" */
    DefineNewAddrVar (TreeVarName (var));
   }
-  {
    return mVAR_PARAM (var);
-  }
  }
 
 }
@@ -261,32 +258,24 @@ static tTree DalibSectionFree
  register int section_id;
 # endif
 {
-# line 187 "CodeDescriptors.puma"
+/* line 187 "CodeDescriptors.puma" */
  {
   tTree param;
   tTree params;
   tTree call;
   {
-# line 189 "CodeDescriptors.puma"
-
-# line 190 "CodeDescriptors.puma"
-
-# line 191 "CodeDescriptors.puma"
-
-# line 193 "CodeDescriptors.puma"
+/* line 193 "CodeDescriptors.puma" */
    params = mBTP_EMPTY ();
-# line 194 "CodeDescriptors.puma"
+/* line 194 "CodeDescriptors.puma" */
    param = DalibSectionDescriptor (section_id);
-# line 195 "CodeDescriptors.puma"
+/* line 195 "CodeDescriptors.puma" */
    params = mBTP_LIST (param, params);
-# line 196 "CodeDescriptors.puma"
+/* line 196 "CodeDescriptors.puma" */
    call = mPROC_OBJ (MakeDalibId ("section_free"));
-# line 197 "CodeDescriptors.puma"
+/* line 197 "CodeDescriptors.puma" */
    call = mACF_BASIC (mCALL_STMT (call, params));
   }
-  {
    return call;
-  }
  }
 
 }
@@ -304,13 +293,11 @@ void DalibSectionTranslation
 # endif
 {
   if (var->Kind == kUSED_VAR) {
-# line 211 "CodeDescriptors.puma"
+/* line 211 "CodeDescriptors.puma" */
  {
   tTree param;
   {
-# line 213 "CodeDescriptors.puma"
-
-# line 214 "CodeDescriptors.puma"
+/* line 214 "CodeDescriptors.puma" */
    param = mVAR_PARAM (MakeUsedVarA (TreeVarName (var->USED_VAR.VARNAME), "_DSP"));
   }
    * yyP3 = NoTree;
@@ -321,16 +308,14 @@ void DalibSectionTranslation
 
   }
   if (var->Kind == kINDEXED_VAR) {
-# line 217 "CodeDescriptors.puma"
+/* line 217 "CodeDescriptors.puma" */
  {
   tTree param;
   {
-# line 219 "CodeDescriptors.puma"
+/* line 219 "CodeDescriptors.puma" */
    if (! ((IsWholeVar (var)))) goto yyL2;
   {
-# line 221 "CodeDescriptors.puma"
-
-# line 222 "CodeDescriptors.puma"
+/* line 222 "CodeDescriptors.puma" */
    param = mVAR_PARAM (MakeUsedVarA (TreeVarName (var->INDEXED_VAR.IND_VAR), "_DSP"));
   }
   }
@@ -341,18 +326,18 @@ void DalibSectionTranslation
  }
 yyL2:;
 
-# line 225 "CodeDescriptors.puma"
+/* line 225 "CodeDescriptors.puma" */
    * yyP3 = DalibSectionCreate (nr, var);
    * yyP2 = DalibSectionDescriptor (nr);
    * yyP1 = DalibSectionFree (nr);
    return;
 
   }
-# line 232 "CodeDescriptors.puma"
+/* line 232 "CodeDescriptors.puma" */
   {
-# line 234 "CodeDescriptors.puma"
+/* line 234 "CodeDescriptors.puma" */
    error_protocol ("cannot create a descriptor here");
-# line 235 "CodeDescriptors.puma"
+/* line 235 "CodeDescriptors.puma" */
    tree_protocol ("this is the illegal variable : ", var);
   }
    * yyP3 = NoTree;
@@ -373,38 +358,30 @@ static tTree DalibLocalCreate
  register tTree var;
 # endif
 {
-# line 259 "CodeDescriptors.puma"
+/* line 259 "CodeDescriptors.puma" */
  {
   tTree param;
   tTree params;
   tTree call;
   {
-# line 261 "CodeDescriptors.puma"
-
-# line 262 "CodeDescriptors.puma"
-
-# line 263 "CodeDescriptors.puma"
-
-# line 265 "CodeDescriptors.puma"
+/* line 265 "CodeDescriptors.puma" */
    param = ExpToVarParam (MakeConstant (pack));
-# line 266 "CodeDescriptors.puma"
+/* line 266 "CodeDescriptors.puma" */
    params = mBTP_LIST (param, mBTP_EMPTY ());
-# line 267 "CodeDescriptors.puma"
+/* line 267 "CodeDescriptors.puma" */
    param = CopyTree (var);
-# line 268 "CodeDescriptors.puma"
+/* line 268 "CodeDescriptors.puma" */
    params = mBTP_LIST (param, params);
-# line 269 "CodeDescriptors.puma"
+/* line 269 "CodeDescriptors.puma" */
    param = DalibLocalDescriptor (local_id);
-# line 270 "CodeDescriptors.puma"
+/* line 270 "CodeDescriptors.puma" */
    params = mBTP_LIST (param, params);
-# line 271 "CodeDescriptors.puma"
+/* line 271 "CodeDescriptors.puma" */
    call = mPROC_OBJ (MakeDalibId ("local_create"));
-# line 272 "CodeDescriptors.puma"
+/* line 272 "CodeDescriptors.puma" */
    call = mACF_BASIC (mCALL_STMT (call, params));
   }
-  {
    return call;
-  }
  }
 
 }
@@ -417,20 +394,16 @@ static tTree DalibLocalDescriptor
  register int local_id;
 # endif
 {
-# line 291 "CodeDescriptors.puma"
+/* line 291 "CodeDescriptors.puma" */
  {
   tTree var;
   {
-# line 292 "CodeDescriptors.puma"
-
-# line 294 "CodeDescriptors.puma"
+/* line 294 "CodeDescriptors.puma" */
    var = MakeUsedVarAn (MakeIdent ("ILOC", 4), "_DSP", local_id);
-# line 295 "CodeDescriptors.puma"
+/* line 295 "CodeDescriptors.puma" */
    DefineNewAddrVar (TreeVarName (var));
   }
-  {
    return mVAR_PARAM (var);
-  }
  }
 
 }
@@ -443,32 +416,24 @@ static tTree DalibLocalFree
  register int local_id;
 # endif
 {
-# line 307 "CodeDescriptors.puma"
+/* line 307 "CodeDescriptors.puma" */
  {
   tTree param;
   tTree params;
   tTree call;
   {
-# line 309 "CodeDescriptors.puma"
-
-# line 310 "CodeDescriptors.puma"
-
-# line 311 "CodeDescriptors.puma"
-
-# line 313 "CodeDescriptors.puma"
+/* line 313 "CodeDescriptors.puma" */
    params = mBTP_EMPTY ();
-# line 314 "CodeDescriptors.puma"
+/* line 314 "CodeDescriptors.puma" */
    param = DalibLocalDescriptor (local_id);
-# line 315 "CodeDescriptors.puma"
+/* line 315 "CodeDescriptors.puma" */
    params = mBTP_LIST (param, params);
-# line 316 "CodeDescriptors.puma"
+/* line 316 "CodeDescriptors.puma" */
    call = mPROC_OBJ (MakeDalibId ("local_free"));
-# line 317 "CodeDescriptors.puma"
+/* line 317 "CodeDescriptors.puma" */
    call = mACF_BASIC (mCALL_STMT (call, params));
   }
-  {
    return call;
-  }
  }
 
 }
@@ -486,17 +451,17 @@ void DalibLocalTranslation
  register tTree * yyP4;
 # endif
 {
-# line 332 "CodeDescriptors.puma"
+/* line 332 "CodeDescriptors.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 334 "CodeDescriptors.puma"
+/* line 334 "CodeDescriptors.puma" */
    DalibSectionTranslation (nr, var, & yyV1, & yyV2, & yyV3);
-# line 336 "CodeDescriptors.puma"
+/* line 336 "CodeDescriptors.puma" */
    yyV1 = CombineACF (yyV1, DalibLocalCreate (nr, pack, yyV2));
-# line 338 "CodeDescriptors.puma"
+/* line 338 "CodeDescriptors.puma" */
    yyV3 = CombineACF (DalibLocalFree (nr), yyV3);
   }
    * yyP6 = yyV1;
@@ -510,26 +475,22 @@ void DalibLocalTranslation
 
 tTree DalibPid
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 351 "CodeDescriptors.puma"
+/* line 351 "CodeDescriptors.puma" */
  {
   tTree call;
   {
-# line 352 "CodeDescriptors.puma"
-
-# line 354 "CodeDescriptors.puma"
+/* line 354 "CodeDescriptors.puma" */
  call = mPROC_OBJ (MakeDalibId ("pid"));
      DefineNewHelpFn (MakeDalibId ("pid"));
      call = mFUNC_CALL_EXP (call, mBTP_EMPTY());
    
   }
-  {
    return call;
-  }
  }
 
 }
@@ -545,19 +506,13 @@ tTree DalibFindOwner
 {
   if (topology->Kind == kON_VAR) {
   if (Definitions_IsType (topology->ON_VAR.arrayid, kObject)) {
-# line 373 "CodeDescriptors.puma"
+/* line 373 "CodeDescriptors.puma" */
  {
   int nr;
   tTree params;
   tTree stmt;
   {
-# line 377 "CodeDescriptors.puma"
-
-# line 378 "CodeDescriptors.puma"
-
-# line 379 "CodeDescriptors.puma"
-
-# line 381 "CodeDescriptors.puma"
+/* line 381 "CodeDescriptors.puma" */
  params = FindOwnerParams (specs);
      nr     = TreeListLength (params) / 2;
 
@@ -569,14 +524,38 @@ tTree DalibFindOwner
 
    
   }
-  {
    return stmt;
-  }
  }
 
   }
   }
- yyAbort ("DalibFindOwner");
+  if (topology->Kind == kON_PROCS) {
+  if (topology->ON_PROCS.topid->Kind == kTopologyObject) {
+/* line 395 "CodeDescriptors.puma" */
+ {
+  tTree params;
+  tTree stmt;
+  {
+/* line 402 "CodeDescriptors.puma" */
+ params = mBTP_EMPTY ();
+     params = mBTP_LIST (mVAR_PARAM (MakeUsedVarA (topology->ON_PROCS.topid->TopologyObject.Ident, "_TOPID")),
+                         params);
+     stmt = mPROC_OBJ (MakeDalibId ("fix_owner"));
+     stmt = mACF_BASIC (mCALL_STMT (stmt, params));
+   
+  }
+   return stmt;
+ }
+
+  }
+  }
+/* line 412 "CodeDescriptors.puma" */
+  {
+/* line 414 "CodeDescriptors.puma" */
+   failure2_protocol (MODULE, "DalibFindOwner", topology, specs);
+  }
+   return NoTree;
+
 }
 
 static tTree FindOwnerParams
@@ -588,31 +567,28 @@ static tTree FindOwnerParams
 # endif
 {
   if (specs->Kind == kON_LIST) {
-# line 397 "CodeDescriptors.puma"
+/* line 421 "CodeDescriptors.puma" */
  {
   tTree new_params;
   {
-# line 399 "CodeDescriptors.puma"
-
-# line 401 "CodeDescriptors.puma"
+/* line 425 "CodeDescriptors.puma" */
  new_params = FindOwnerParams (specs->ON_LIST.Next);
     new_params = mBTP_LIST (ExpToVarParam (specs->ON_LIST.Elem->ON_SPEC.ON_VAL), new_params);
     new_params = mBTP_LIST (ExpToVarParam (MakeConstant (specs->ON_LIST.Elem->ON_SPEC.array_dim)),
                             new_params);
   
   }
-  {
    return new_params;
-  }
  }
 
   }
   if (specs->Kind == kON_EMPTY) {
-# line 410 "CodeDescriptors.puma"
+/* line 434 "CodeDescriptors.puma" */
    return mBTP_EMPTY ();
 
   }
  yyAbort ("FindOwnerParams");
+ { tTree yyDummy; return yyDummy; }
 }
 
 tTree DalibDimBroadcast
@@ -628,25 +604,22 @@ tTree DalibDimBroadcast
   if (on_top->Kind == kON_VAR) {
   if (Definitions_IsType (on_top->ON_VAR.tempid, kObject)) {
   if (on_val->Kind == kON_SPEC) {
-# line 425 "CodeDescriptors.puma"
+/* line 449 "CodeDescriptors.puma" */
  {
   tTree stmt;
   tTree paramlist;
   tTree param;
   {
-# line 428 "CodeDescriptors.puma"
-
-# line 429 "CodeDescriptors.puma"
-
-# line 430 "CodeDescriptors.puma"
-
-# line 432 "CodeDescriptors.puma"
+/* line 456 "CodeDescriptors.puma" */
  paramlist = mBTP_EMPTY();
 
      param     = ExpToVarParam (MakeSizeExp (MakeFullShape (var)));
      paramlist = mBTP_LIST (param, paramlist);
 
-     param     = mVAR_PARAM (var);
+     if (TreeRank (var) >= 1)
+        param  = mVAR_PARAM (FirstArrayElement (MakeFullShape (var)));
+      else
+        param  = mVAR_PARAM (var);
      paramlist = mBTP_LIST (param, paramlist);
 
      param     = ExpToVarParam (on_val->ON_SPEC.ON_VAL);
@@ -663,17 +636,15 @@ tTree DalibDimBroadcast
 
    
   }
-  {
    return stmt;
-  }
  }
 
   }
   }
   }
-# line 456 "CodeDescriptors.puma"
+/* line 483 "CodeDescriptors.puma" */
   {
-# line 457 "CodeDescriptors.puma"
+/* line 484 "CodeDescriptors.puma" */
    failure2_protocol (MODULE, "DalibDimBroadcast", on_top, on_val);
   }
    return NoTree;
@@ -688,25 +659,22 @@ tTree DalibNodeGet
  register tTree var;
 # endif
 {
-# line 471 "CodeDescriptors.puma"
+/* line 498 "CodeDescriptors.puma" */
  {
   tTree stmt;
   tTree param;
   tTree paramlist;
   {
-# line 473 "CodeDescriptors.puma"
-
-# line 474 "CodeDescriptors.puma"
-
-# line 475 "CodeDescriptors.puma"
-
-# line 477 "CodeDescriptors.puma"
+/* line 504 "CodeDescriptors.puma" */
  paramlist = mBTP_EMPTY();
 
      param     = ExpToVarParam (MakeSizeExp (var));
      paramlist = mBTP_LIST (param, paramlist);
 
-     param     = mVAR_PARAM (var);
+     if (TreeRank (var) >= 1)
+        param  = mVAR_PARAM (FirstArrayElement (MakeFullShape (var)));
+      else
+        param  = mVAR_PARAM (var);
      paramlist = mBTP_LIST (param, paramlist);
 
      stmt = mPROC_OBJ (MakeDalibId ("node_get"));
@@ -714,9 +682,7 @@ tTree DalibNodeGet
 
    
   }
-  {
    return stmt;
-  }
  }
 
 }
@@ -730,19 +696,13 @@ tTree DalibBroadcast
  register tTree from;
 # endif
 {
-# line 502 "CodeDescriptors.puma"
+/* line 532 "CodeDescriptors.puma" */
  {
   tTree stmt;
   tTree param;
   tTree paramlist;
   {
-# line 504 "CodeDescriptors.puma"
-
-# line 505 "CodeDescriptors.puma"
-
-# line 506 "CodeDescriptors.puma"
-
-# line 508 "CodeDescriptors.puma"
+/* line 538 "CodeDescriptors.puma" */
  paramlist = mBTP_EMPTY();
 
      param     = ExpToVarParam (from);
@@ -755,6 +715,7 @@ tTree DalibBroadcast
         param  = mVAR_PARAM (FirstArrayElement (MakeFullShape (var)));
       else
         param  = mVAR_PARAM (var);
+
      paramlist = mBTP_LIST (param, paramlist);
 
      
@@ -768,9 +729,7 @@ tTree DalibBroadcast
 
    
   }
-  {
    return stmt;
-  }
  }
 
 }
@@ -783,19 +742,13 @@ tTree DalibLocExchange
  register tTree var;
 # endif
 {
-# line 545 "CodeDescriptors.puma"
+/* line 576 "CodeDescriptors.puma" */
  {
   tTree stmt;
   tTree param;
   tTree paramlist;
   {
-# line 547 "CodeDescriptors.puma"
-
-# line 548 "CodeDescriptors.puma"
-
-# line 549 "CodeDescriptors.puma"
-
-# line 551 "CodeDescriptors.puma"
+/* line 582 "CodeDescriptors.puma" */
  paramlist = mBTP_EMPTY();
 
      param     = ExpToVarParam (MakeSizeExp (MakeFullShape (var)));
@@ -809,17 +762,15 @@ tTree DalibLocExchange
 
    
   }
-  {
    return stmt;
-  }
  }
 
 }
 
-void BeginCodeDescriptors ()
+void BeginCodeDescriptors ARGS ((void))
 {
 }
 
-void CloseCodeDescriptors ()
+void CloseCodeDescriptors ARGS ((void))
 {
 }
