@@ -1,6 +1,13 @@
 # include "CodeGlobal.h"
-# include "yyCodeGlobal.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,32 +15,29 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 23 "CodeGlobal.puma"
+/* line 23 "CodeGlobal.puma" */
 
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
@@ -56,11 +60,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* CodeGlobal_Exit) () = yyExit;
+# include "yyCodeGlobal.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* CodeGlobal_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -69,32 +87,32 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module CodeGlobal, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module CodeGlobal, routine %s failed\n",
+  yyFunction);
  CodeGlobal_Exit ();
 }
 
-tTree CodeReduction ARGS((tTree stmt));
-static bool IsFullGlobalReduction ARGS((tTree on_top, tTree on_specs));
-static void SplitOnSpecifications ARGS((tTree on_list, tTree * yyP2, tTree * yyP1));
-static bool NoValSpecification ARGS((tTree on_list));
-static tTree FullDimReductions ARGS((tTree f, tTree p, tTree top, tTree spec));
-static tTree MakeTopDimReduction ARGS((tTree topology, int dim, tTree f, tTree vars));
-static tTree MakeArrayDimReduction ARGS((tDefinitions array_obj, int dim, tTree f, tTree vars));
-static tTree MakeFullGlobalReduction ARGS((tTree f, tTree red_vars));
-static tTree GlobalLocExchange ARGS((tTree params));
-tTree CodeBroadcast ARGS((tTree stmt));
-static tTree MakeHostBroadcasts ARGS((tTree var));
-static tTree MakeNodeBroadcasts ARGS((tTree var, tTree top, tTree specs));
-static tTree MakeNodeDimBroadcasts ARGS((tTree var, tTree top, tTree specs));
-static bool IsVarSelection ARGS((tTree var, tTree on_val));
-static tTree Overlapping ARGS((tTree t));
-static tTree TakeWholeVar ARGS((tTree v));
-static tTree MakeReductionStmt ARGS((tTree top, int dim, tTree var, tTree f));
-static tTree MakeArrayReductionStmt ARGS((tDefinitions obj, int dim, tTree var, tTree f));
-static tTree MakeLocReductionStmt ARGS((tTree var, tTree f));
-static void MyDalibSectionTranslation ARGS((tTree var, tTree * yyP5, tTree * yyP4, tTree * yyP3));
-static tTree MakeLocOpParam ARGS((tTree var, tIdent funcname));
-static tTree MakeOpParam ARGS((tTree var, tIdent funcname));
+tTree CodeReduction ARGS ((tTree stmt));
+static rbool IsFullGlobalReduction ARGS ((tTree on_top, tTree on_specs));
+static void SplitOnSpecifications ARGS ((tTree on_list, tTree * yyP2, tTree * yyP1));
+static tTree FullDimReductions ARGS ((tTree f, tTree p, tTree top, tTree spec));
+static tTree MakeTopDimReduction ARGS ((tTree topology, int dim, tTree f, tTree vars));
+static tTree MakeArrayDimReduction ARGS ((tDefinitions array_obj, int dim, tTree f, tTree vars));
+static tTree MakeFullGlobalReduction ARGS ((tTree f, tTree red_vars));
+static tTree GlobalLocExchange ARGS ((tTree params));
+tTree CodeBroadcast ARGS ((tTree stmt));
+static tTree MakeHostBroadcasts ARGS ((tTree var));
+static tTree MakeNodeBroadcasts ARGS ((tTree var, tTree top, tTree specs));
+static tTree MakeNodeDimBroadcasts ARGS ((tTree var, tTree top, tTree specs));
+static rbool IsVarSelection ARGS ((tTree var, tTree on_val));
+static tTree Overlapping ARGS ((tTree t));
+static tTree TakeWholeVar ARGS ((tTree v));
+static tTree MakeReductionStmt ARGS ((tTree top, int dim, tTree var, tTree f));
+static tTree MakeArrayReductionStmt ARGS ((tDefinitions obj, int dim, tTree var, tTree f));
+static tTree MakeLocReductionStmt ARGS ((tTree var, tTree f));
+static void MyDalibSectionTranslation ARGS ((tTree var, tTree * yyP5, tTree * yyP4, tTree * yyP3));
+static tTree MakeLocOpParam ARGS ((tTree var, tIdent funcname));
+static tTree MakeOpParam ARGS ((tTree var, tIdent funcname));
 
 tTree CodeReduction
 # if defined __STDC__ | defined __cplusplus
@@ -105,12 +123,12 @@ tTree CodeReduction
 # endif
 {
   if (stmt->Kind == kREDUCTION_STMT) {
-# line 71 "CodeGlobal.puma"
+/* line 71 "CodeGlobal.puma" */
   {
-# line 73 "CodeGlobal.puma"
+/* line 73 "CodeGlobal.puma" */
    if (! ((IsFullGlobalReduction (stmt->REDUCTION_STMT.RED_TOP, stmt->REDUCTION_STMT.RED_SPEC)))) goto yyL1;
   {
-# line 75 "CodeGlobal.puma"
+/* line 75 "CodeGlobal.puma" */
    stmt_protocol ("global reduction along all processors");
   }
   }
@@ -118,36 +136,32 @@ tTree CodeReduction
 yyL1:;
 
   if (stmt->REDUCTION_STMT.RED_TOP->Kind == kON_VAR) {
-# line 80 "CodeGlobal.puma"
+/* line 80 "CodeGlobal.puma" */
  {
   tTree red_stmts;
   {
-# line 82 "CodeGlobal.puma"
-
-# line 84 "CodeGlobal.puma"
+/* line 84 "CodeGlobal.puma" */
    stmt_protocol ("global reduction along processors array dimensions");
-# line 86 "CodeGlobal.puma"
+/* line 86 "CodeGlobal.puma" */
    red_stmts = FullDimReductions (stmt->REDUCTION_STMT.REDFUNC, stmt->REDUCTION_STMT.ELEMS, stmt->REDUCTION_STMT.RED_TOP, stmt->REDUCTION_STMT.RED_SPEC);
-# line 88 "CodeGlobal.puma"
+/* line 88 "CodeGlobal.puma" */
    tree_protocol ("generated code :\n", red_stmts);
   }
-  {
    return red_stmts;
-  }
  }
 
   }
   }
-# line 93 "CodeGlobal.puma"
+/* line 93 "CodeGlobal.puma" */
   {
-# line 95 "CodeGlobal.puma"
+/* line 95 "CodeGlobal.puma" */
    failure_protocol (MODULE, "CodeReduction", stmt);
   }
    return NoTree;
 
 }
 
-static bool IsFullGlobalReduction
+static rbool IsFullGlobalReduction
 # if defined __STDC__ | defined __cplusplus
 (register tTree on_top, register tTree on_specs)
 # else
@@ -157,25 +171,25 @@ static bool IsFullGlobalReduction
 # endif
 {
   if (on_top->Kind == kON_ALL) {
-# line 108 "CodeGlobal.puma"
-   return true;
+/* line 108 "CodeGlobal.puma" */
+   return rtrue;
 
   }
   if (on_top->Kind == kON_VAR) {
-# line 111 "CodeGlobal.puma"
+/* line 111 "CodeGlobal.puma" */
   {
-# line 113 "CodeGlobal.puma"
+/* line 113 "CodeGlobal.puma" */
    if (! ((IsFullTopologyObj (on_top->ON_VAR.topid)))) goto yyL2;
   {
-# line 114 "CodeGlobal.puma"
+/* line 114 "CodeGlobal.puma" */
    if (! ((TreeListLength (on_specs) == VarRank (on_top->ON_VAR.topid)))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
-  return false;
+  return rfalse;
 }
 
 static void SplitOnSpecifications
@@ -189,7 +203,7 @@ static void SplitOnSpecifications
 # endif
 {
   if (on_list->Kind == kON_EMPTY) {
-# line 125 "CodeGlobal.puma"
+/* line 125 "CodeGlobal.puma" */
    * yyP2 = on_list;
    * yyP1 = on_list;
    return;
@@ -197,14 +211,14 @@ static void SplitOnSpecifications
   }
   if (on_list->Kind == kON_LIST) {
   if (on_list->ON_LIST.Elem->ON_SPEC.ON_VAL->Kind == kSLICE_EXP) {
-# line 137 "CodeGlobal.puma"
+/* line 137 "CodeGlobal.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 139 "CodeGlobal.puma"
+/* line 139 "CodeGlobal.puma" */
    SplitOnSpecifications (on_list->ON_LIST.Next, & yyV1, & yyV2);
-# line 140 "CodeGlobal.puma"
+/* line 140 "CodeGlobal.puma" */
  on_list->ON_LIST.Next = yyV2; 
   }
    * yyP2 = yyV1;
@@ -213,14 +227,14 @@ static void SplitOnSpecifications
  }
 
   }
-# line 143 "CodeGlobal.puma"
+/* line 143 "CodeGlobal.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 145 "CodeGlobal.puma"
+/* line 145 "CodeGlobal.puma" */
    SplitOnSpecifications (on_list->ON_LIST.Next, & yyV1, & yyV2);
-# line 146 "CodeGlobal.puma"
+/* line 146 "CodeGlobal.puma" */
  on_list->ON_LIST.Next = yyV1; 
   }
    * yyP2 = on_list;
@@ -232,34 +246,6 @@ static void SplitOnSpecifications
 ;
 }
 
-static bool NoValSpecification
-# if defined __STDC__ | defined __cplusplus
-(register tTree on_list)
-# else
-(on_list)
- register tTree on_list;
-# endif
-{
-  if (on_list->Kind == kON_EMPTY) {
-# line 159 "CodeGlobal.puma"
-   return true;
-
-  }
-  if (on_list->Kind == kON_LIST) {
-  if (on_list->ON_LIST.Elem->ON_SPEC.ON_VAL->Kind == kSLICE_EXP) {
-# line 168 "CodeGlobal.puma"
-  {
-# line 169 "CodeGlobal.puma"
-   if (! (NoValSpecification (on_list->ON_LIST.Next))) goto yyL2;
-  }
-   return true;
-yyL2:;
-
-  }
-  }
-  return false;
-}
-
 static tTree FullDimReductions
 # if defined __STDC__ | defined __cplusplus
 (register tTree f, register tTree p, register tTree top, register tTree spec)
@@ -272,33 +258,33 @@ static tTree FullDimReductions
 # endif
 {
   if (spec->Kind == kON_LIST) {
-# line 180 "CodeGlobal.puma"
+/* line 157 "CodeGlobal.puma" */
    return CombineACF (FullDimReductions (f, p, top, spec->ON_LIST.Elem), FullDimReductions (f, p, top, spec->ON_LIST.Next));
 
   }
   if (spec->Kind == kON_EMPTY) {
-# line 186 "CodeGlobal.puma"
+/* line 163 "CodeGlobal.puma" */
    return NoTree;
 
   }
   if (top->Kind == kON_VAR) {
   if (spec->Kind == kON_SPEC) {
-# line 193 "CodeGlobal.puma"
+/* line 170 "CodeGlobal.puma" */
   {
-# line 195 "CodeGlobal.puma"
+/* line 172 "CodeGlobal.puma" */
    if (! ((top->ON_VAR.topid == NoObject))) goto yyL3;
   }
    return MakeArrayDimReduction (top->ON_VAR.tempid, spec->ON_SPEC.temp_dim, f, p);
 yyL3:;
 
-# line 200 "CodeGlobal.puma"
+/* line 177 "CodeGlobal.puma" */
    return MakeTopDimReduction (MakeTopologyParam (top->ON_VAR.topid), spec->ON_SPEC.top_dim, f, p);
 
   }
   }
-# line 205 "CodeGlobal.puma"
+/* line 182 "CodeGlobal.puma" */
   {
-# line 207 "CodeGlobal.puma"
+/* line 184 "CodeGlobal.puma" */
    failure_protocol (MODULE, "FullDimReductions", spec);
   }
    return NoTree;
@@ -318,15 +304,15 @@ static tTree MakeTopDimReduction
 {
   if (vars->Kind == kBTV_LIST) {
   if (vars->BTV_LIST.Next->Kind == kBTV_EMPTY) {
-# line 221 "CodeGlobal.puma"
+/* line 198 "CodeGlobal.puma" */
    return MakeReductionStmt (topology, dim, vars->BTV_LIST.Elem, f);
 
   }
-# line 226 "CodeGlobal.puma"
+/* line 203 "CodeGlobal.puma" */
    return MakeLocReductionStmt (f, vars->BTV_LIST.Elem);
 
   }
-# line 231 "CodeGlobal.puma"
+/* line 208 "CodeGlobal.puma" */
    return MakeFullGlobalReduction (f, vars);
 
 }
@@ -344,12 +330,13 @@ static tTree MakeArrayDimReduction
 {
   if (vars->Kind == kBTV_LIST) {
   if (vars->BTV_LIST.Next->Kind == kBTV_EMPTY) {
-# line 245 "CodeGlobal.puma"
+/* line 222 "CodeGlobal.puma" */
    return MakeArrayReductionStmt (array_obj, dim, vars->BTV_LIST.Elem, f);
 
   }
   }
  yyAbort ("MakeArrayDimReduction");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static tTree MakeFullGlobalReduction
@@ -363,15 +350,16 @@ static tTree MakeFullGlobalReduction
 {
   if (red_vars->Kind == kBTV_LIST) {
   if (red_vars->BTV_LIST.Next->Kind == kBTV_EMPTY) {
-# line 260 "CodeGlobal.puma"
+/* line 237 "CodeGlobal.puma" */
    return MakeReductionStmt (NoTree, 1, red_vars->BTV_LIST.Elem, f);
 
   }
-# line 265 "CodeGlobal.puma"
+/* line 242 "CodeGlobal.puma" */
    return CombineACF (MakeLocReductionStmt (red_vars->BTV_LIST.Elem, f), GlobalLocExchange (red_vars->BTV_LIST.Next));
 
   }
  yyAbort ("MakeFullGlobalReduction");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static tTree GlobalLocExchange
@@ -383,18 +371,18 @@ static tTree GlobalLocExchange
 # endif
 {
   if (params->Kind == kBTV_EMPTY) {
-# line 290 "CodeGlobal.puma"
+/* line 267 "CodeGlobal.puma" */
    return NoTree;
 
   }
   if (params->Kind == kBTV_LIST) {
-# line 294 "CodeGlobal.puma"
+/* line 271 "CodeGlobal.puma" */
    return CombineACF (DalibLocExchange (params->BTV_LIST.Elem), GlobalLocExchange (params->BTV_LIST.Next));
 
   }
-# line 300 "CodeGlobal.puma"
+/* line 277 "CodeGlobal.puma" */
   {
-# line 301 "CodeGlobal.puma"
+/* line 278 "CodeGlobal.puma" */
    failure_protocol (MODULE, "GlobalLocExchange", params);
   }
    return NoTree;
@@ -412,63 +400,55 @@ tTree CodeBroadcast
   if (stmt->Kind == kBROADCAST_STMT) {
   if (stmt->BROADCAST_STMT.OWNER_TOP->Kind == kON_HOST) {
   if (stmt->BROADCAST_STMT.OWNER_SPEC->Kind == kON_EMPTY) {
-# line 334 "CodeGlobal.puma"
+/* line 311 "CodeGlobal.puma" */
  {
   tTree new;
   {
-# line 335 "CodeGlobal.puma"
-
-# line 336 "CodeGlobal.puma"
+/* line 314 "CodeGlobal.puma" */
    stmt_protocol ("following statement is broadcast from host/io-process");
-# line 337 "CodeGlobal.puma"
+/* line 315 "CodeGlobal.puma" */
    new = MakeHostBroadcasts (stmt->BROADCAST_STMT.ELEMS);
-# line 338 "CodeGlobal.puma"
+/* line 316 "CodeGlobal.puma" */
    tree_protocol ("generated code :\n", new);
   }
-  {
    return new;
-  }
  }
 
   }
   }
   if (stmt->BROADCAST_STMT.OWNER_TOP->Kind == kON_ALL) {
   if (stmt->BROADCAST_STMT.OWNER_SPEC->Kind == kON_EMPTY) {
-# line 342 "CodeGlobal.puma"
+/* line 320 "CodeGlobal.puma" */
   {
-# line 343 "CodeGlobal.puma"
+/* line 321 "CodeGlobal.puma" */
    error_protocol ("broadcast from all nodes makes no sense");
   }
    return NoTree;
 
   }
   }
-# line 347 "CodeGlobal.puma"
+/* line 325 "CodeGlobal.puma" */
  {
   tTree new;
   tTree yyV1;
   tTree yyV2;
   {
-# line 348 "CodeGlobal.puma"
-
-# line 349 "CodeGlobal.puma"
+/* line 327 "CodeGlobal.puma" */
    stmt_protocol ("following statement is broadcast from node processes");
-# line 351 "CodeGlobal.puma"
+/* line 329 "CodeGlobal.puma" */
    SplitOnSpecifications (stmt->BROADCAST_STMT.OWNER_SPEC, & yyV1, & yyV2);
-# line 353 "CodeGlobal.puma"
+/* line 331 "CodeGlobal.puma" */
    new = MakeNodeBroadcasts (stmt->BROADCAST_STMT.ELEMS, stmt->BROADCAST_STMT.OWNER_TOP, yyV1);
-# line 354 "CodeGlobal.puma"
+/* line 332 "CodeGlobal.puma" */
    tree_protocol ("generated code :\n", new);
   }
-  {
    return new;
-  }
  }
 
   }
-# line 358 "CodeGlobal.puma"
+/* line 336 "CodeGlobal.puma" */
   {
-# line 359 "CodeGlobal.puma"
+/* line 337 "CodeGlobal.puma" */
    failure_protocol (MODULE, "CodeBroadcast", stmt);
   }
    return NoTree;
@@ -484,41 +464,35 @@ static tTree MakeHostBroadcasts
 # endif
 {
   if (var->Kind == kBTV_LIST) {
-# line 381 "CodeGlobal.puma"
+/* line 359 "CodeGlobal.puma" */
    return CombineACF (MakeHostBroadcasts (var->BTV_LIST.Elem), MakeHostBroadcasts (var->BTV_LIST.Next));
 
   }
   if (var->Kind == kBTV_EMPTY) {
-# line 386 "CodeGlobal.puma"
+/* line 365 "CodeGlobal.puma" */
    return NoTree;
 
   }
-# line 390 "CodeGlobal.puma"
+/* line 370 "CodeGlobal.puma" */
   {
-# line 392 "CodeGlobal.puma"
+/* line 372 "CodeGlobal.puma" */
    if (! ((TreeWriteDistribution (var) == - 1))) goto yyL3;
   }
    return NoTree;
 yyL3:;
 
-# line 396 "CodeGlobal.puma"
+/* line 376 "CodeGlobal.puma" */
  {
   tTree bc_var;
   tTree from;
   {
-# line 398 "CodeGlobal.puma"
-
-# line 399 "CodeGlobal.puma"
-
-# line 401 "CodeGlobal.puma"
+/* line 381 "CodeGlobal.puma" */
  bc_var = Overlapping (var);
 
      from = MakeConstant (1);  
    
   }
-  {
    return DalibBroadcast (bc_var, from);
-  }
  }
 
 }
@@ -534,37 +508,33 @@ static tTree MakeNodeBroadcasts
 # endif
 {
   if (var->Kind == kBTV_LIST) {
-# line 426 "CodeGlobal.puma"
+/* line 406 "CodeGlobal.puma" */
    return CombineACF (MakeNodeBroadcasts (var->BTV_LIST.Elem, top, specs), MakeNodeBroadcasts (var->BTV_LIST.Next, top, specs));
 
   }
   if (var->Kind == kBTV_EMPTY) {
-# line 431 "CodeGlobal.puma"
+/* line 411 "CodeGlobal.puma" */
    return NoTree;
 
   }
-# line 435 "CodeGlobal.puma"
+/* line 415 "CodeGlobal.puma" */
   {
-# line 437 "CodeGlobal.puma"
+/* line 417 "CodeGlobal.puma" */
    if (! ((TreeWriteDistribution (var) != 0))) goto yyL3;
   }
    return MakeNodeDimBroadcasts (var, top, specs);
 yyL3:;
 
   if (top->Kind == kON_VAR) {
-# line 441 "CodeGlobal.puma"
+/* line 421 "CodeGlobal.puma" */
  {
   tTree new;
   tTree update;
   {
-# line 443 "CodeGlobal.puma"
+/* line 423 "CodeGlobal.puma" */
    if (! ((IsFullTopologyObj (top->ON_VAR.topid)))) goto yyL4;
   {
-# line 445 "CodeGlobal.puma"
-
-# line 446 "CodeGlobal.puma"
-
-# line 448 "CodeGlobal.puma"
+/* line 428 "CodeGlobal.puma" */
   
 
      new = MakeNodeDimBroadcasts (var, top, specs);
@@ -574,22 +544,21 @@ yyL3:;
   
   }
   }
-  {
    return new;
-  }
  }
 yyL4:;
 
   }
-# line 458 "CodeGlobal.puma"
+/* line 438 "CodeGlobal.puma" */
   {
-# line 460 "CodeGlobal.puma"
+/* line 440 "CodeGlobal.puma" */
    if (! ((TreeWriteDistribution (var) == 0))) goto yyL5;
   }
    return mACF_LIST (DalibFindOwner (top, specs), mACF_LIST (DalibNodeGet (var), NoTree));
 yyL5:;
 
  yyAbort ("MakeNodeBroadcasts");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static tTree MakeNodeDimBroadcasts
@@ -603,38 +572,38 @@ static tTree MakeNodeDimBroadcasts
 # endif
 {
   if (specs->Kind == kON_LIST) {
-# line 483 "CodeGlobal.puma"
+/* line 463 "CodeGlobal.puma" */
    return CombineACF (MakeNodeDimBroadcasts (var, top, specs->ON_LIST.Elem), MakeNodeDimBroadcasts (var, top, specs->ON_LIST.Next));
 
   }
   if (specs->Kind == kON_EMPTY) {
-# line 488 "CodeGlobal.puma"
+/* line 468 "CodeGlobal.puma" */
    return NoTree;
 
   }
   if (specs->Kind == kON_SPEC) {
-# line 492 "CodeGlobal.puma"
+/* line 472 "CodeGlobal.puma" */
   {
-# line 493 "CodeGlobal.puma"
+/* line 473 "CodeGlobal.puma" */
    if (! ((IsVarSelection (var, specs)))) goto yyL3;
   }
    return NoTree;
 yyL3:;
 
-# line 497 "CodeGlobal.puma"
+/* line 477 "CodeGlobal.puma" */
    return DalibDimBroadcast (CopyTree (var), top, specs);
 
   }
-# line 504 "CodeGlobal.puma"
+/* line 484 "CodeGlobal.puma" */
   {
-# line 505 "CodeGlobal.puma"
+/* line 485 "CodeGlobal.puma" */
    error_protocol ("illegal broadcast within processor topology");
   }
    return NoTree;
 
 }
 
-static bool IsVarSelection
+static rbool IsVarSelection
 # if defined __STDC__ | defined __cplusplus
 (register tTree var, register tTree on_val)
 # else
@@ -644,22 +613,16 @@ static bool IsVarSelection
 # endif
 {
   if (on_val->Kind == kON_SPEC) {
-# line 520 "CodeGlobal.puma"
+/* line 500 "CodeGlobal.puma" */
  {
   var_descriptor owner;
   tTree val1;
-  bool found;
+  rbool found;
   {
-# line 522 "CodeGlobal.puma"
-
-# line 523 "CodeGlobal.puma"
-
-# line 524 "CodeGlobal.puma"
-
-# line 526 "CodeGlobal.puma"
+/* line 506 "CodeGlobal.puma" */
    if (! ((TreeWriteDistribution (var) == 1))) goto yyL1;
   {
-# line 527 "CodeGlobal.puma"
+/* line 507 "CodeGlobal.puma" */
  GetVarDescriptor (var, &found, &owner);
      if (!found)
         tree_error_protocol
@@ -672,16 +635,16 @@ static bool IsVarSelection
             error_protocol ("illegal broadcast (something went wrong)");
        }
    
-# line 539 "CodeGlobal.puma"
+/* line 519 "CodeGlobal.puma" */
    if (! ((found))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
   }
-  return false;
+  return rfalse;
 }
 
 static tTree Overlapping
@@ -695,13 +658,13 @@ static tTree Overlapping
   if (t->Kind == kDO_VAR) {
   if (t->DO_VAR.BODY->Kind == kBTV_LIST) {
   if (t->DO_VAR.BODY->BTV_LIST.Next->Kind == kBTV_EMPTY) {
-# line 552 "CodeGlobal.puma"
+/* line 532 "CodeGlobal.puma" */
    return TakeWholeVar (t->DO_VAR.BODY->BTV_LIST.Elem);
 
   }
-# line 559 "CodeGlobal.puma"
+/* line 539 "CodeGlobal.puma" */
   {
-# line 561 "CodeGlobal.puma"
+/* line 541 "CodeGlobal.puma" */
    error_protocol ("Broadcast: Too many variables in the variable list");
   }
    return TakeWholeVar (t->DO_VAR.BODY->BTV_LIST.Elem);
@@ -709,36 +672,40 @@ static tTree Overlapping
   }
   }
   if (t->Kind == kSELECTED_VAR) {
-# line 565 "CodeGlobal.puma"
-   return Overlapping (t->SELECTED_VAR.SELEC_VAR);
+/* line 545 "CodeGlobal.puma" */
+  {
+/* line 547 "CodeGlobal.puma" */
+ t->SELECTED_VAR.SELEC_VAR = Overlapping (t->SELECTED_VAR.SELEC_VAR); 
+  }
+   return t;
 
   }
   if (t->Kind == kINDEXED_VAR) {
-# line 569 "CodeGlobal.puma"
+/* line 552 "CodeGlobal.puma" */
   {
-# line 570 "CodeGlobal.puma"
+/* line 554 "CodeGlobal.puma" */
    if (! ((IsContiguousSection (t)))) goto yyL4;
   }
    return t;
 yyL4:;
 
-# line 576 "CodeGlobal.puma"
+/* line 561 "CodeGlobal.puma" */
    return t->INDEXED_VAR.IND_VAR;
 
   }
   if (t->Kind == kUSED_VAR) {
-# line 581 "CodeGlobal.puma"
+/* line 568 "CodeGlobal.puma" */
    return t;
 
   }
   if (t->Kind == kLOOP_VAR) {
-# line 585 "CodeGlobal.puma"
+/* line 573 "CodeGlobal.puma" */
    return t;
 
   }
-# line 589 "CodeGlobal.puma"
+/* line 578 "CodeGlobal.puma" */
   {
-# line 590 "CodeGlobal.puma"
+/* line 580 "CodeGlobal.puma" */
    failure_protocol (MODULE, "Overlapping", t);
   }
    return t;
@@ -753,35 +720,38 @@ static tTree TakeWholeVar
  register tTree v;
 # endif
 {
+ yyRecursion:
   if (v->Kind == kINDEXED_VAR) {
-# line 602 "CodeGlobal.puma"
+/* line 592 "CodeGlobal.puma" */
    return v->INDEXED_VAR.IND_VAR;
 
   }
   if (v->Kind == kUSED_VAR) {
-# line 606 "CodeGlobal.puma"
+/* line 596 "CodeGlobal.puma" */
    return v;
 
   }
   if (v->Kind == kDO_VAR) {
   if (v->DO_VAR.BODY->Kind == kBTV_LIST) {
   if (v->DO_VAR.BODY->BTV_LIST.Next->Kind == kBTV_EMPTY) {
-# line 610 "CodeGlobal.puma"
-   return TakeWholeVar (v->DO_VAR.BODY->BTV_LIST.Elem);
+/* line 600 "CodeGlobal.puma" */
+   v = v->DO_VAR.BODY->BTV_LIST.Elem;
+   goto yyRecursion;
 
   }
-# line 617 "CodeGlobal.puma"
+/* line 607 "CodeGlobal.puma" */
   {
-# line 619 "CodeGlobal.puma"
+/* line 609 "CodeGlobal.puma" */
    error_protocol ("Broadcast: Too many variables in the variable list");
   }
-   return TakeWholeVar (v->DO_VAR.BODY->BTV_LIST.Elem);
+   v = v->DO_VAR.BODY->BTV_LIST.Elem;
+   goto yyRecursion;
 
   }
   }
-# line 623 "CodeGlobal.puma"
+/* line 613 "CodeGlobal.puma" */
   {
-# line 624 "CodeGlobal.puma"
+/* line 614 "CodeGlobal.puma" */
    failure_protocol (MODULE, "TakeWholeVar", v);
   }
    return v;
@@ -800,16 +770,14 @@ static tTree MakeReductionStmt
 # endif
 {
   if (f->Kind == kPROC_OBJ) {
-# line 650 "CodeGlobal.puma"
+/* line 640 "CodeGlobal.puma" */
  {
   tTree t;
   {
-# line 652 "CodeGlobal.puma"
+/* line 642 "CodeGlobal.puma" */
    if (! ((TreeRank (var) == 0))) goto yyL1;
   {
-# line 654 "CodeGlobal.puma"
-
-# line 656 "CodeGlobal.puma"
+/* line 646 "CodeGlobal.puma" */
  t = MakeOpParam (var, f->PROC_OBJ.Ident);
      t = mBTP_LIST (mVAR_PARAM (CopyTree(var)), mBTP_LIST (t, mBTP_EMPTY()));
 
@@ -832,24 +800,20 @@ static tTree MakeReductionStmt
    
   }
   }
-  {
    return t;
-  }
  }
 yyL1:;
 
-# line 680 "CodeGlobal.puma"
+/* line 670 "CodeGlobal.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   tTree t;
   {
-# line 682 "CodeGlobal.puma"
+/* line 672 "CodeGlobal.puma" */
    MyDalibSectionTranslation (var, & yyV1, & yyV2, & yyV3);
-# line 684 "CodeGlobal.puma"
-
-# line 686 "CodeGlobal.puma"
+/* line 676 "CodeGlobal.puma" */
  t = MakeOpParam (var, f->PROC_OBJ.Ident);
      t = mBTP_LIST (yyV2, mBTP_LIST (t, mBTP_EMPTY()));
  
@@ -867,15 +831,13 @@ yyL1:;
      t = mACF_BASIC (t);
    
   }
-  {
    return ComposeNewACF (yyV1, t, yyV3);
-  }
  }
 
   }
-# line 706 "CodeGlobal.puma"
+/* line 696 "CodeGlobal.puma" */
   {
-# line 707 "CodeGlobal.puma"
+/* line 697 "CodeGlobal.puma" */
    failure_protocol (MODULE, "MakeReductionStmt", f);
   }
    return NoTree;
@@ -894,16 +856,14 @@ static tTree MakeArrayReductionStmt
 # endif
 {
   if (f->Kind == kPROC_OBJ) {
-# line 724 "CodeGlobal.puma"
+/* line 714 "CodeGlobal.puma" */
  {
   tTree t;
   {
-# line 726 "CodeGlobal.puma"
+/* line 716 "CodeGlobal.puma" */
    if (! ((TreeRank (var) == 0))) goto yyL1;
   {
-# line 728 "CodeGlobal.puma"
-
-# line 730 "CodeGlobal.puma"
+/* line 720 "CodeGlobal.puma" */
  t = MakeOpParam (var, f->PROC_OBJ.Ident);
      t = mBTP_LIST (mVAR_PARAM (CopyTree(var)), mBTP_LIST (t, mBTP_EMPTY()));
 
@@ -916,16 +876,14 @@ static tTree MakeArrayReductionStmt
    
   }
   }
-  {
    return t;
-  }
  }
 yyL1:;
 
   }
-# line 744 "CodeGlobal.puma"
+/* line 734 "CodeGlobal.puma" */
   {
-# line 745 "CodeGlobal.puma"
+/* line 735 "CodeGlobal.puma" */
    failure_protocol (MODULE, "MakeArrayReductionStmt", f);
   }
    return NoTree;
@@ -942,16 +900,14 @@ static tTree MakeLocReductionStmt
 # endif
 {
   if (f->Kind == kPROC_OBJ) {
-# line 761 "CodeGlobal.puma"
+/* line 751 "CodeGlobal.puma" */
  {
   tTree t;
   {
-# line 763 "CodeGlobal.puma"
+/* line 753 "CodeGlobal.puma" */
    if (! ((TreeRank (var) == 0))) goto yyL1;
   {
-# line 765 "CodeGlobal.puma"
-
-# line 767 "CodeGlobal.puma"
+/* line 757 "CodeGlobal.puma" */
  t = MakeLocOpParam (var, f->PROC_OBJ.Ident);
      t = mBTP_LIST (mVAR_PARAM (CopyTree(var)), mBTP_LIST (t, mBTP_EMPTY()));
      t = mCALL_STMT (mPROC_OBJ (MakeDalibId ("pos_reduction")), t);
@@ -959,18 +915,16 @@ static tTree MakeLocReductionStmt
    
   }
   }
-  {
    return t;
-  }
  }
 yyL1:;
 
   }
-# line 776 "CodeGlobal.puma"
+/* line 766 "CodeGlobal.puma" */
   {
-# line 778 "CodeGlobal.puma"
+/* line 768 "CodeGlobal.puma" */
    error_protocol ("loc reductions not allowed for sections");
-# line 779 "CodeGlobal.puma"
+/* line 769 "CodeGlobal.puma" */
    tree_protocol ("no reduction for this var : ", var);
   }
    return NoTree;
@@ -988,19 +942,13 @@ static void MyDalibSectionTranslation
  register tTree * yyP3;
 # endif
 {
-# line 797 "CodeGlobal.puma"
+/* line 787 "CodeGlobal.puma" */
  {
   tTree pre_stmts;
   tTree post_stmts;
   tTree descriptor;
   {
-# line 799 "CodeGlobal.puma"
-
-# line 800 "CodeGlobal.puma"
-
-# line 801 "CodeGlobal.puma"
-
-# line 803 "CodeGlobal.puma"
+/* line 793 "CodeGlobal.puma" */
    DalibSectionTranslation (1, var, & pre_stmts, & descriptor, & post_stmts);
   }
    * yyP5 = pre_stmts;
@@ -1021,15 +969,13 @@ static tTree MakeLocOpParam
  register tIdent funcname;
 # endif
 {
-# line 814 "CodeGlobal.puma"
+/* line 804 "CodeGlobal.puma" */
  {
   int op;
   {
-# line 816 "CodeGlobal.puma"
-
-# line 818 "CodeGlobal.puma"
+/* line 808 "CodeGlobal.puma" */
    op = GetGlobalOp (TreeType (var), funcname);
-# line 820 "CodeGlobal.puma"
+/* line 810 "CodeGlobal.puma" */
  if ((op < 1) || (op > 6))
       { error_protocol ("illegal loc reduction (type, function)");
         tree_protocol ("var  : ", var);
@@ -1037,9 +983,7 @@ static tTree MakeLocOpParam
       }
    
   }
-  {
    return ExpToVarParam (MakeConstant (op));
-  }
  }
 
 }
@@ -1053,15 +997,13 @@ static tTree MakeOpParam
  register tIdent funcname;
 # endif
 {
-# line 832 "CodeGlobal.puma"
+/* line 822 "CodeGlobal.puma" */
  {
   int op;
   {
-# line 834 "CodeGlobal.puma"
-
-# line 836 "CodeGlobal.puma"
+/* line 826 "CodeGlobal.puma" */
    op = GetGlobalOp (TreeType (var), funcname);
-# line 838 "CodeGlobal.puma"
+/* line 828 "CodeGlobal.puma" */
  if (op == -1)  
       { error_protocol ("illegal reduction (type, function)");
         tree_protocol ("var  : ", var);
@@ -1069,17 +1011,15 @@ static tTree MakeOpParam
       }
    
   }
-  {
    return ExpToVarParam (MakeConstant (op));
-  }
  }
 
 }
 
-void BeginCodeGlobal ()
+void BeginCodeGlobal ARGS ((void))
 {
 }
 
-void CloseCodeGlobal ()
+void CloseCodeGlobal ARGS ((void))
 {
 }
