@@ -1,6 +1,13 @@
 # include "CodeOn.h"
-# include "yyCodeOn.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,31 +15,28 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 33 "CodeOn.puma"
+/* line 33 "CodeOn.puma" */
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
@@ -55,11 +59,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* CodeOn_Exit) () = yyExit;
+# include "yyCodeOn.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* CodeOn_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -68,38 +86,39 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module CodeOn, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module CodeOn, routine %s failed\n",
+  yyFunction);
  CodeOn_Exit ();
 }
 
-bool NeedsNoContext ARGS((tTree stmt));
-tTree ApplyOnSpecs ARGS((tTree body, tTree top, tTree specs));
-static tTree ApplyOnSpecList ARGS((tTree body, tTree top, tTree specs));
-static tTree MakeTopMask ARGS((tTree topology));
-static tTree MakeTopDimMask ARGS((tTree topology, tTree spec));
-static tTree MakeIsLocal ARGS((tDefinitions on_id, int on_dim, tTree val));
-static void TemplateDimInfo ARGS((tDefinitions template, int temp_dim, int * yyP2, tTree * yyP1));
-static tTree GetLocalRange ARGS((tTree actual_range, int kind, int base, int stride, tDefinitions template_id, int template_dim));
-static void GetTempSliceVars ARGS((tDefinitions template, int temp_dim, tTree * yyP4, tTree * yyP3));
-static void GetTempRangeVars ARGS((tDefinitions template, int temp_dim, tTree * yyP7, tTree * yyP6, tTree * yyP5));
-static tTree MakeLocalRangeStmt ARGS((tDefinitions template, int temp_dim, int base, int stride, tTree range));
-static tTree MakeLocalSliceStmt ARGS((tDefinitions template, int template_dim, tTree range));
-static bool IsBlockDistribution ARGS((int kind));
-static bool IsMappedDistribution ARGS((int kind, tTree size));
-static bool IsOne ARGS((tTree size));
-tTree InitTaskRegion ARGS((tTree task_body));
-tTree StartTasking ARGS(());
-tTree StopTasking ARGS(());
-tTree ApplyOnInfo ARGS((tTree t, bool push_flag));
-static tTree PushContext ARGS((tDefinitions on_id, int on_dim));
-static tTree PopContext ARGS((tDefinitions on_id, int on_dim));
-tTree CodeONPushContext ARGS((tTree top, tTree specs));
-tTree CodeONPopContext ARGS((tTree top, tTree specs));
-static tTree MakePushParams ARGS((tTree specs));
-static tTree MakePushHomeCall ARGS((tIdent id, tTree spec_params));
-static tTree MakePopHomeCall ARGS(());
+rbool NeedsNoContext ARGS ((tTree stmt));
+tTree ApplyOnSpecs ARGS ((tTree body, tTree top, tTree specs));
+static tTree ApplyOnSpecList ARGS ((tTree body, tTree top, tTree specs));
+static tTree MakeTopMask ARGS ((tTree topology));
+static tTree MakeTopDimMask ARGS ((tTree topology, tTree spec));
+static tTree MakeIsLocal ARGS ((tDefinitions on_id, int on_dim, tTree val));
+static void TemplateDimInfo ARGS ((tDefinitions template, int temp_dim, int * yyP2, tTree * yyP1));
+static tTree GetLocalRange ARGS ((tTree actual_range, int kind, int base, int stride, tDefinitions template_id, int template_dim));
+static void GetTempSliceVars ARGS ((tDefinitions template, int temp_dim, tTree * yyP4, tTree * yyP3));
+static void GetTempRangeVars ARGS ((tDefinitions template, int temp_dim, tTree * yyP7, tTree * yyP6, tTree * yyP5));
+static tTree MakeLocalRangeStmt ARGS ((tDefinitions template, int temp_dim, int base, int stride, tTree range));
+static tTree MakeLocalSliceStmt ARGS ((tDefinitions template, int template_dim, tTree range));
+static rbool IsBlockDistribution ARGS ((int kind));
+static rbool IsMappedDistribution ARGS ((int kind, tTree size));
+static rbool IsOne ARGS ((tTree size));
+tTree InitTaskRegion ARGS ((tTree task_body));
+tTree StartTasking ARGS ((void));
+tTree StopTasking ARGS ((void));
+tTree ApplyOnInfo ARGS ((tTree t, rbool push_flag));
+static tTree PushContext ARGS ((tDefinitions on_id, int on_dim));
+static tTree PopContext ARGS ((tDefinitions on_id, int on_dim));
+tTree CodeONPushContext ARGS ((tTree top, tTree specs));
+tTree CodeONPopContext ARGS ((tTree top, tTree specs));
+static tTree MakePushParams ARGS ((tTree specs));
+static tTree MakePushHomeCall ARGS ((tIdent id, tTree spec_params));
+static tTree MakePopHomeCall ARGS ((void));
 
-bool NeedsNoContext
+rbool NeedsNoContext
 # if defined __STDC__ | defined __cplusplus
 (register tTree stmt)
 # else
@@ -110,92 +129,96 @@ bool NeedsNoContext
 
   switch (stmt->Kind) {
   case kACF_LIST:
-# line 70 "CodeOn.puma"
+/* line 70 "CodeOn.puma" */
   {
-# line 72 "CodeOn.puma"
+/* line 72 "CodeOn.puma" */
    if (! ((NeedsNoContext (stmt->ACF_LIST.Elem)))) goto yyL1;
   {
-# line 73 "CodeOn.puma"
+/* line 73 "CodeOn.puma" */
    if (! ((NeedsNoContext (stmt->ACF_LIST.Next)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   break;
   case kACF_EMPTY:
-# line 76 "CodeOn.puma"
-   return true;
+/* line 76 "CodeOn.puma" */
+   return rtrue;
+
+  case kACF_DUMMY:
+/* line 79 "CodeOn.puma" */
+   return rtrue;
 
   case kACF_BASIC:
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kCALL_STMT) {
-# line 79 "CodeOn.puma"
+/* line 82 "CodeOn.puma" */
   {
-# line 81 "CodeOn.puma"
-   return false;
+/* line 84 "CodeOn.puma" */
+   return rfalse;
   }
 
   }
-# line 84 "CodeOn.puma"
-   return true;
+/* line 87 "CodeOn.puma" */
+   return rtrue;
 
   case kACF_DO:
-# line 87 "CodeOn.puma"
+/* line 90 "CodeOn.puma" */
   {
-# line 89 "CodeOn.puma"
-   if (! ((NeedsNoContext (stmt->ACF_DO.DO_BODY)))) goto yyL5;
+/* line 92 "CodeOn.puma" */
+   if (! ((NeedsNoContext (stmt->ACF_DO.DO_BODY)))) goto yyL6;
   }
-   return true;
-yyL5:;
-
-  break;
-  case kACF_IF:
-# line 92 "CodeOn.puma"
-  {
-# line 94 "CodeOn.puma"
-   if (! ((NeedsNoContext (stmt->ACF_IF.THEN_PART)))) goto yyL6;
-  {
-# line 95 "CodeOn.puma"
-   if (! ((NeedsNoContext (stmt->ACF_IF.ELSE_PART)))) goto yyL6;
-  }
-  }
-   return true;
+   return rtrue;
 yyL6:;
 
   break;
-  case kACF_ON:
-# line 98 "CodeOn.puma"
+  case kACF_IF:
+/* line 95 "CodeOn.puma" */
   {
-# line 100 "CodeOn.puma"
-   if (! ((NeedsNoContext (stmt->ACF_ON.ON_BODY)))) goto yyL7;
+/* line 97 "CodeOn.puma" */
+   if (! ((NeedsNoContext (stmt->ACF_IF.THEN_PART)))) goto yyL7;
+  {
+/* line 98 "CodeOn.puma" */
+   if (! ((NeedsNoContext (stmt->ACF_IF.ELSE_PART)))) goto yyL7;
   }
-   return true;
+  }
+   return rtrue;
 yyL7:;
 
   break;
-  case kACF_PARALLEL:
-# line 103 "CodeOn.puma"
+  case kACF_ON:
+/* line 101 "CodeOn.puma" */
   {
-# line 105 "CodeOn.puma"
-   if (! ((NeedsNoContext (stmt->ACF_PARALLEL.PARALLEL_BODY)))) goto yyL8;
+/* line 103 "CodeOn.puma" */
+   if (! ((NeedsNoContext (stmt->ACF_ON.ON_BODY)))) goto yyL8;
   }
-   return true;
+   return rtrue;
 yyL8:;
 
   break;
-  case kACF_CRITICAL:
-# line 108 "CodeOn.puma"
+  case kACF_PARALLEL:
+/* line 106 "CodeOn.puma" */
   {
-# line 110 "CodeOn.puma"
-   if (! ((NeedsNoContext (stmt->ACF_CRITICAL.CRITICAL_BODY)))) goto yyL9;
+/* line 108 "CodeOn.puma" */
+   if (! ((NeedsNoContext (stmt->ACF_PARALLEL.PARALLEL_BODY)))) goto yyL9;
   }
-   return true;
+   return rtrue;
 yyL9:;
 
+  break;
+  case kACF_CRITICAL:
+/* line 111 "CodeOn.puma" */
+  {
+/* line 113 "CodeOn.puma" */
+   if (! ((NeedsNoContext (stmt->ACF_CRITICAL.CRITICAL_BODY)))) goto yyL10;
+  }
+   return rtrue;
+yyL10:;
+
   break;
   }
 
-  return false;
+  return rfalse;
 }
 
 tTree ApplyOnSpecs
@@ -209,72 +232,60 @@ tTree ApplyOnSpecs
 # endif
 {
   if (top->Kind == kON_ALL) {
-# line 121 "CodeOn.puma"
+/* line 124 "CodeOn.puma" */
    return body;
 
   }
   if (top->Kind == kON_HOST) {
-# line 126 "CodeOn.puma"
+/* line 129 "CodeOn.puma" */
  {
   tTree new_acf;
   {
-# line 128 "CodeOn.puma"
-
-# line 130 "CodeOn.puma"
+/* line 133 "CodeOn.puma" */
  new_acf = mOP_EXP (mOP_EQ(), DalibPid(), MakeConstant (1));
       new_acf = mACF_IF (new_acf, MakeStmtList (body), mACF_EMPTY());
     
   }
-  {
    return new_acf;
-  }
  }
 
   }
   if (top->Kind == kON_VAR) {
   if (specs->Kind == kON_EMPTY) {
-# line 137 "CodeOn.puma"
+/* line 140 "CodeOn.puma" */
   {
-# line 139 "CodeOn.puma"
+/* line 142 "CodeOn.puma" */
    if (! ((IsFullTopologyObj (top->ON_VAR.topid)))) goto yyL3;
   }
    return body;
 yyL3:;
 
-# line 144 "CodeOn.puma"
+/* line 147 "CodeOn.puma" */
  {
   tTree new_acf;
   {
-# line 146 "CodeOn.puma"
-
-# line 148 "CodeOn.puma"
+/* line 151 "CodeOn.puma" */
  new_acf = MakeTopMask (top);
       new_acf = mACF_IF (new_acf, MakeStmtList (body), mACF_EMPTY());
     
   }
-  {
    return new_acf;
-  }
  }
 
   }
   if (specs->Kind == kON_LIST) {
-# line 155 "CodeOn.puma"
+/* line 158 "CodeOn.puma" */
    return ApplyOnSpecList (body, top, specs);
 
   }
   }
   if (top->Kind == kON_PROCS) {
-# line 160 "CodeOn.puma"
+/* line 163 "CodeOn.puma" */
  {
   tTree new_acf;
   tTree sub_top;
   {
-# line 162 "CodeOn.puma"
-
-# line 163 "CodeOn.puma"
-
-# line 165 "CodeOn.puma"
+/* line 168 "CodeOn.puma" */
  new_acf = MakeTopMask (top);
       new_acf = mACF_IF (new_acf, MakeStmtList (body), mACF_EMPTY());
  
@@ -286,15 +297,13 @@ yyL3:;
 
     
   }
-  {
    return new_acf;
-  }
  }
 
   }
-# line 179 "CodeOn.puma"
+/* line 182 "CodeOn.puma" */
   {
-# line 181 "CodeOn.puma"
+/* line 184 "CodeOn.puma" */
    failure2_protocol (MODULE, "ApplyOnSpecs", top, specs);
   }
    return body;
@@ -311,37 +320,36 @@ static tTree ApplyOnSpecList
  register tTree specs;
 # endif
 {
+ yyRecursion:
   if (specs->Kind == kON_EMPTY) {
-# line 194 "CodeOn.puma"
+/* line 197 "CodeOn.puma" */
    return body;
 
   }
   if (specs->Kind == kON_LIST) {
-# line 199 "CodeOn.puma"
-   return ApplyOnSpecList (ApplyOnSpecList (body, top, specs->ON_LIST.Next), top, specs->ON_LIST.Elem);
+/* line 202 "CodeOn.puma" */
+   body = ApplyOnSpecList (body, top, specs->ON_LIST.Next);
+   specs = specs->ON_LIST.Elem;
+   goto yyRecursion;
 
   }
   if (specs->Kind == kON_SPEC) {
-# line 204 "CodeOn.puma"
+/* line 207 "CodeOn.puma" */
  {
   tTree new_stmt;
   {
-# line 206 "CodeOn.puma"
-
-# line 208 "CodeOn.puma"
+/* line 211 "CodeOn.puma" */
  new_stmt = mACF_IF (MakeTopDimMask (top, specs), body, mACF_EMPTY ());
      new_stmt = mACF_LIST (new_stmt, mACF_EMPTY ());
    
   }
-  {
    return new_stmt;
-  }
  }
 
   }
-# line 215 "CodeOn.puma"
+/* line 218 "CodeOn.puma" */
   {
-# line 217 "CodeOn.puma"
+/* line 220 "CodeOn.puma" */
    failure_protocol (MODULE, "ApplyOnSpecs", specs);
   }
    return body;
@@ -358,16 +366,12 @@ static tTree MakeTopMask
 {
   if (topology->Kind == kON_VAR) {
   if (Definitions_IsType (topology->ON_VAR.arrayid, kObject)) {
-# line 229 "CodeOn.puma"
+/* line 232 "CodeOn.puma" */
  {
   tTree paramlist;
   tTree call;
   {
-# line 231 "CodeOn.puma"
-
-# line 232 "CodeOn.puma"
-
-# line 234 "CodeOn.puma"
+/* line 237 "CodeOn.puma" */
  paramlist = mBTP_EMPTY ();
     paramlist = mBTP_LIST (ExpToVarParam (MakeConstant (0)), paramlist);
     paramlist = mBTP_LIST (mVAR_PARAM (MakeUsedVarA (topology->ON_VAR.arrayid->Object.Ident, "_DSP")),
@@ -378,25 +382,19 @@ static tTree MakeTopMask
     call = mFUNC_CALL_EXP (call, paramlist);
   
   }
-  {
    return call;
-  }
  }
 
   }
   }
   if (topology->Kind == kON_PROCS) {
   if (topology->ON_PROCS.topid->Kind == kTopologyObject) {
-# line 247 "CodeOn.puma"
+/* line 250 "CodeOn.puma" */
  {
   tTree paramlist;
   tTree call;
   {
-# line 249 "CodeOn.puma"
-
-# line 250 "CodeOn.puma"
-
-# line 252 "CodeOn.puma"
+/* line 255 "CodeOn.puma" */
  paramlist = mBTP_EMPTY ();
     paramlist = mBTP_LIST (mVAR_PARAM (MakeUsedVarA (topology->ON_PROCS.topid->TopologyObject.Ident, "_TOPID")),
                            paramlist);
@@ -406,16 +404,14 @@ static tTree MakeTopMask
     call = mFUNC_CALL_EXP (call, paramlist);
   
   }
-  {
    return call;
-  }
  }
 
   }
   }
-# line 264 "CodeOn.puma"
+/* line 267 "CodeOn.puma" */
   {
-# line 266 "CodeOn.puma"
+/* line 269 "CodeOn.puma" */
    failure_protocol (MODULE, "MakeTopMask", topology);
   }
    return NoTree;
@@ -432,21 +428,21 @@ static tTree MakeTopDimMask
 # endif
 {
   if (topology->Kind == kON_VAR) {
-# line 281 "CodeOn.puma"
+/* line 284 "CodeOn.puma" */
   {
-# line 283 "CodeOn.puma"
+/* line 286 "CodeOn.puma" */
    if (! ((IsRaggedVarObject (topology->ON_VAR.arrayid)))) goto yyL1;
   }
    return TriDenTIsLocal (topology->ON_VAR.arrayid, spec->ON_SPEC.array_dim, spec->ON_SPEC.ON_VAL);
 yyL1:;
 
-# line 288 "CodeOn.puma"
+/* line 291 "CodeOn.puma" */
    return MakeIsLocal (topology->ON_VAR.arrayid, spec->ON_SPEC.array_dim, spec->ON_SPEC.ON_VAL);
 
   }
-# line 293 "CodeOn.puma"
+/* line 296 "CodeOn.puma" */
   {
-# line 295 "CodeOn.puma"
+/* line 298 "CodeOn.puma" */
    failure2_protocol (MODULE, "MakeTopDimMask", topology, spec);
   }
    return NoTree;
@@ -463,16 +459,12 @@ static tTree MakeIsLocal
  register tTree val;
 # endif
 {
-# line 301 "CodeOn.puma"
+/* line 304 "CodeOn.puma" */
  {
   tTree paramlist;
   tTree call;
   {
-# line 303 "CodeOn.puma"
-
-# line 304 "CodeOn.puma"
-
-# line 306 "CodeOn.puma"
+/* line 309 "CodeOn.puma" */
  paramlist = mBTP_EMPTY ();
      paramlist = mBTP_LIST (ExpToVarParam (CopyTree(val)), paramlist);
      paramlist = mBTP_LIST (ExpToVarParam (MakeConstant (on_dim)),
@@ -484,9 +476,7 @@ static tTree MakeIsLocal
      call = mFUNC_CALL_EXP (call, paramlist);
   
   }
-  {
    return call;
-  }
  }
 
 }
@@ -502,21 +492,15 @@ static void TemplateDimInfo
  register tTree * yyP1;
 # endif
 {
-# line 328 "CodeOn.puma"
+/* line 334 "CodeOn.puma" */
  {
   int kind;
   tTree size;
   var_descriptor vard;
   {
-# line 330 "CodeOn.puma"
-
-# line 331 "CodeOn.puma"
-
-# line 332 "CodeOn.puma"
-
-# line 334 "CodeOn.puma"
+/* line 340 "CodeOn.puma" */
    SetVarObjDescriptor (& vard, template, NoTree);
-# line 336 "CodeOn.puma"
+/* line 342 "CodeOn.puma" */
  kind = vard.distribution_kind [temp_dim - 1]; 
      size = vard.distribution_size [temp_dim - 1];
    
@@ -543,76 +527,68 @@ static tTree GetLocalRange
 # endif
 {
   if (actual_range->Kind == kSLICE_EXP) {
-# line 361 "CodeOn.puma"
+/* line 367 "CodeOn.puma" */
  {
   tTree stmt;
   tTree yyV1;
   tTree yyV2;
   {
-# line 363 "CodeOn.puma"
-
-# line 365 "CodeOn.puma"
+/* line 371 "CodeOn.puma" */
    if (! ((base == 0))) goto yyL1;
   {
-# line 366 "CodeOn.puma"
+/* line 372 "CodeOn.puma" */
    if (! ((stride == 1))) goto yyL1;
   {
-# line 367 "CodeOn.puma"
+/* line 373 "CodeOn.puma" */
    if (! ((IsStride1 (actual_range->SLICE_EXP.INC)))) goto yyL1;
   {
-# line 369 "CodeOn.puma"
+/* line 375 "CodeOn.puma" */
    if (! ((IsBlockDistribution (kind)))) goto yyL1;
   {
-# line 371 "CodeOn.puma"
+/* line 377 "CodeOn.puma" */
    stmt = MakeLocalSliceStmt (template_id, template_dim, actual_range);
-# line 373 "CodeOn.puma"
+/* line 379 "CodeOn.puma" */
    GetTempSliceVars (template_id, template_dim, & yyV1, & yyV2);
-# line 375 "CodeOn.puma"
-   actual_range->SLICE_EXP.START = mVAR_EXP (yyV1);
-# line 376 "CodeOn.puma"
+/* line 381 "CodeOn.puma" */
+   actual_range->SLICE_EXP.FIRST = mVAR_EXP (yyV1);
+/* line 382 "CodeOn.puma" */
    actual_range->SLICE_EXP.STOP = mVAR_EXP (yyV2);
   }
   }
   }
   }
   }
-  {
    return stmt;
-  }
  }
 yyL1:;
 
-# line 385 "CodeOn.puma"
+/* line 391 "CodeOn.puma" */
  {
   tTree stmt;
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 387 "CodeOn.puma"
-
-# line 389 "CodeOn.puma"
+/* line 395 "CodeOn.puma" */
    stmt = MakeLocalRangeStmt (template_id, template_dim, base, stride, actual_range);
-# line 392 "CodeOn.puma"
+/* line 398 "CodeOn.puma" */
    GetTempRangeVars (template_id, template_dim, & yyV1, & yyV2, & yyV3);
-# line 394 "CodeOn.puma"
-   actual_range->SLICE_EXP.START = mVAR_EXP (yyV1);
-# line 395 "CodeOn.puma"
+/* line 400 "CodeOn.puma" */
+   actual_range->SLICE_EXP.FIRST = mVAR_EXP (yyV1);
+/* line 401 "CodeOn.puma" */
    actual_range->SLICE_EXP.STOP = mVAR_EXP (yyV2);
-# line 397 "CodeOn.puma"
+/* line 403 "CodeOn.puma" */
  if (!IsBlockDistribution (kind))  
         actual_range->SLICE_EXP.INC = mVAR_EXP (yyV3);
    
   }
-  {
    return stmt;
-  }
  }
 
   }
-# line 404 "CodeOn.puma"
+/* line 410 "CodeOn.puma" */
   {
-# line 406 "CodeOn.puma"
+/* line 412 "CodeOn.puma" */
    failure_protocol (MODULE, "GetLocalRange", actual_range);
   }
    return NoTree;
@@ -630,22 +606,18 @@ static void GetTempSliceVars
  register tTree * yyP3;
 # endif
 {
-# line 418 "CodeOn.puma"
+/* line 424 "CodeOn.puma" */
  {
   tTree low;
   tTree high;
   {
-# line 420 "CodeOn.puma"
-
-# line 421 "CodeOn.puma"
-
-# line 423 "CodeOn.puma"
+/* line 429 "CodeOn.puma" */
    low = MakeUsedVarAn (template->Object.Ident, "_START", temp_dim);
-# line 424 "CodeOn.puma"
+/* line 430 "CodeOn.puma" */
    DefineNewIntVar (TreeVarName (low));
-# line 426 "CodeOn.puma"
+/* line 432 "CodeOn.puma" */
    high = MakeUsedVarAn (template->Object.Ident, "_STOP", temp_dim);
-# line 427 "CodeOn.puma"
+/* line 433 "CodeOn.puma" */
    DefineNewIntVar (TreeVarName (high));
   }
    * yyP4 = low;
@@ -668,29 +640,23 @@ static void GetTempRangeVars
  register tTree * yyP5;
 # endif
 {
-# line 433 "CodeOn.puma"
+/* line 439 "CodeOn.puma" */
  {
   tTree low;
   tTree high;
   tTree inc;
   {
-# line 435 "CodeOn.puma"
-
-# line 436 "CodeOn.puma"
-
-# line 437 "CodeOn.puma"
-
-# line 439 "CodeOn.puma"
+/* line 445 "CodeOn.puma" */
    low = MakeUsedVarAn (template->Object.Ident, "_START", temp_dim);
-# line 440 "CodeOn.puma"
+/* line 446 "CodeOn.puma" */
    DefineNewIntVar (TreeVarName (low));
-# line 442 "CodeOn.puma"
+/* line 448 "CodeOn.puma" */
    high = MakeUsedVarAn (template->Object.Ident, "_STOP", temp_dim);
-# line 443 "CodeOn.puma"
+/* line 449 "CodeOn.puma" */
    DefineNewIntVar (TreeVarName (high));
-# line 445 "CodeOn.puma"
+/* line 451 "CodeOn.puma" */
    inc = MakeUsedVarAn (template->Object.Ident, "_INC", temp_dim);
-# line 446 "CodeOn.puma"
+/* line 452 "CodeOn.puma" */
    DefineNewIntVar (TreeVarName (inc));
   }
    * yyP7 = low;
@@ -715,7 +681,7 @@ static tTree MakeLocalRangeStmt
 # endif
 {
   if (range->Kind == kSLICE_EXP) {
-# line 478 "CodeOn.puma"
+/* line 484 "CodeOn.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -723,13 +689,9 @@ static tTree MakeLocalRangeStmt
   tTree paramlist;
   tIdent id;
   {
-# line 481 "CodeOn.puma"
+/* line 487 "CodeOn.puma" */
    GetTempRangeVars (template, temp_dim, & yyV1, & yyV2, & yyV3);
-# line 483 "CodeOn.puma"
-
-# line 484 "CodeOn.puma"
-
-# line 486 "CodeOn.puma"
+/* line 492 "CodeOn.puma" */
  paramlist = mBTP_EMPTY ();
 
     paramlist = mBTP_LIST (mVAR_PARAM (yyV3), paramlist);
@@ -741,7 +703,7 @@ static tTree MakeLocalRangeStmt
      else  
        paramlist = mBTP_LIST (ExpToVarParam (range->SLICE_EXP.INC), paramlist);
     paramlist = mBTP_LIST (ExpToVarParam (range->SLICE_EXP.STOP    ), paramlist);
-    paramlist = mBTP_LIST (ExpToVarParam (range->SLICE_EXP.START    ), paramlist);
+    paramlist = mBTP_LIST (ExpToVarParam (range->SLICE_EXP.FIRST    ), paramlist);
     if ((base == 0) && (stride == 1))
         id = MakeDalibId ("array_lrange");
       else
@@ -755,15 +717,13 @@ static tTree MakeLocalRangeStmt
                            paramlist);
   
   }
-  {
    return mACF_BASIC (mCALL_STMT (mPROC_OBJ (id), paramlist));
-  }
  }
 
   }
-# line 513 "CodeOn.puma"
+/* line 519 "CodeOn.puma" */
   {
-# line 514 "CodeOn.puma"
+/* line 520 "CodeOn.puma" */
    failure_protocol (MODULE, "MakeLocalRangeStmt", range);
   }
    return NoTree;
@@ -781,25 +741,21 @@ static tTree MakeLocalSliceStmt
 # endif
 {
   if (range->Kind == kSLICE_EXP) {
-# line 538 "CodeOn.puma"
+/* line 544 "CodeOn.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree paramlist;
   tTree call;
   {
-# line 541 "CodeOn.puma"
+/* line 547 "CodeOn.puma" */
    GetTempSliceVars (template, template_dim, & yyV1, & yyV2);
-# line 543 "CodeOn.puma"
-
-# line 544 "CodeOn.puma"
-
-# line 546 "CodeOn.puma"
+/* line 552 "CodeOn.puma" */
  paramlist = mBTP_EMPTY ();
     paramlist = mBTP_LIST (mVAR_PARAM (yyV2), paramlist);
     paramlist = mBTP_LIST (mVAR_PARAM (yyV1), paramlist);
     paramlist = mBTP_LIST (ExpToVarParam (range->SLICE_EXP.STOP    ), paramlist);
-    paramlist = mBTP_LIST (ExpToVarParam (range->SLICE_EXP.START    ), paramlist);
+    paramlist = mBTP_LIST (ExpToVarParam (range->SLICE_EXP.FIRST    ), paramlist);
     paramlist = mBTP_LIST (ExpToVarParam (MakeConstant (template_dim)),
                            paramlist);
     paramlist = mBTP_LIST (mVAR_PARAM (MakeUsedVarA (template->Object.Ident, "_DSP")),
@@ -808,22 +764,20 @@ static tTree MakeLocalSliceStmt
     call = mACF_BASIC (mCALL_STMT (call, paramlist));
   
   }
-  {
    return call;
-  }
  }
 
   }
-# line 561 "CodeOn.puma"
+/* line 567 "CodeOn.puma" */
   {
-# line 563 "CodeOn.puma"
+/* line 569 "CodeOn.puma" */
    failure_protocol (MODULE, "MakeLocalSliceStmt", range);
   }
    return NoTree;
 
 }
 
-static bool IsBlockDistribution
+static rbool IsBlockDistribution
 # if defined __STDC__ | defined __cplusplus
 (register int kind)
 # else
@@ -831,42 +785,42 @@ static bool IsBlockDistribution
  register int kind;
 # endif
 {
-# line 569 "CodeOn.puma"
+/* line 575 "CodeOn.puma" */
   {
-# line 570 "CodeOn.puma"
+/* line 576 "CodeOn.puma" */
    if (! ((kind == kBLOCK_DIM))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 573 "CodeOn.puma"
+/* line 579 "CodeOn.puma" */
   {
-# line 574 "CodeOn.puma"
+/* line 580 "CodeOn.puma" */
    if (! ((kind == kGEN_BLOCK_DIM))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 577 "CodeOn.puma"
+/* line 583 "CodeOn.puma" */
   {
-# line 578 "CodeOn.puma"
+/* line 584 "CodeOn.puma" */
    if (! ((kind == kANY_BLOCK_DIM))) goto yyL3;
   }
-   return true;
+   return rtrue;
 yyL3:;
 
-# line 581 "CodeOn.puma"
+/* line 587 "CodeOn.puma" */
   {
-# line 582 "CodeOn.puma"
+/* line 588 "CodeOn.puma" */
    if (! ((kind == kANY_GEN_BLOCK_DIM))) goto yyL4;
   }
-   return true;
+   return rtrue;
 yyL4:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsMappedDistribution
+static rbool IsMappedDistribution
 # if defined __STDC__ | defined __cplusplus
 (register int kind, register tTree size)
 # else
@@ -875,38 +829,38 @@ static bool IsMappedDistribution
  register tTree size;
 # endif
 {
-# line 587 "CodeOn.puma"
+/* line 593 "CodeOn.puma" */
   {
-# line 589 "CodeOn.puma"
+/* line 595 "CodeOn.puma" */
    if (! ((IsBlockDistribution (kind)))) goto yyL1;
   {
-# line 590 "CodeOn.puma"
-   return false;
+/* line 596 "CodeOn.puma" */
+   return rfalse;
   }
   }
 yyL1:;
 
-# line 593 "CodeOn.puma"
+/* line 599 "CodeOn.puma" */
   {
-# line 595 "CodeOn.puma"
+/* line 601 "CodeOn.puma" */
    if (! ((kind == kCYCLIC_DIM))) goto yyL2;
   {
-# line 596 "CodeOn.puma"
+/* line 602 "CodeOn.puma" */
    if (! ((IsOne (size)))) goto yyL2;
   {
-# line 597 "CodeOn.puma"
-   return false;
+/* line 603 "CodeOn.puma" */
+   return rfalse;
   }
   }
   }
 yyL2:;
 
-# line 600 "CodeOn.puma"
-   return true;
+/* line 606 "CodeOn.puma" */
+   return rtrue;
 
 }
 
-static bool IsOne
+static rbool IsOne
 # if defined __STDC__ | defined __cplusplus
 (register tTree size)
 # else
@@ -914,37 +868,33 @@ static bool IsOne
  register tTree size;
 # endif
 {
-# line 605 "CodeOn.puma"
+/* line 611 "CodeOn.puma" */
   {
-# line 607 "CodeOn.puma"
+/* line 613 "CodeOn.puma" */
    if (! ((size == NoTree))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 610 "CodeOn.puma"
+/* line 616 "CodeOn.puma" */
  {
-  bool found;
+  rbool found;
   int val;
   {
-# line 612 "CodeOn.puma"
-
-# line 613 "CodeOn.puma"
-
-# line 615 "CodeOn.puma"
+/* line 621 "CodeOn.puma" */
    GetIntConstValue (size, & found, & val);
-# line 616 "CodeOn.puma"
+/* line 622 "CodeOn.puma" */
    if (! ((found))) goto yyL2;
   {
-# line 617 "CodeOn.puma"
+/* line 623 "CodeOn.puma" */
    if (! ((val == 1))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
  }
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
 tTree InitTaskRegion
@@ -956,31 +906,27 @@ tTree InitTaskRegion
 # endif
 {
   if (task_body->Kind == kACF_LIST) {
-# line 628 "CodeOn.puma"
+/* line 634 "CodeOn.puma" */
    return CombineACF (InitTaskRegion (task_body->ACF_LIST.Elem), InitTaskRegion (task_body->ACF_LIST.Next));
 
   }
   if (task_body->Kind == kACF_EMPTY) {
-# line 633 "CodeOn.puma"
+/* line 639 "CodeOn.puma" */
    return NoTree;
 
   }
   if (task_body->Kind == kACF_ON) {
   if (task_body->ACF_ON.ON_HOME->Kind == kON_VAR) {
   if (Definitions_IsType (task_body->ACF_ON.ON_HOME->ON_VAR.arrayid, kObject)) {
-# line 638 "CodeOn.puma"
+/* line 644 "CodeOn.puma" */
  {
   tTree stmt;
   tTree params;
   {
-# line 641 "CodeOn.puma"
-
-# line 642 "CodeOn.puma"
-
-# line 644 "CodeOn.puma"
+/* line 650 "CodeOn.puma" */
  params = mBTP_EMPTY ();
 
-     params = mBTP_LIST (mVAR_PARAM (MakeUsedVarA (task_body->ACF_ON.ON_HOME->ON_VAR.arrayid->Object.Ident, LOCAL_SUFFIX)),
+     params = mBTP_LIST (mVAR_PARAM (MakeUsedVarA (task_body->ACF_ON.ON_HOME->ON_VAR.arrayid->Object.Ident, DSP_SUFFIX)),
                          params);
 
      stmt = mPROC_OBJ (MakeDalibId ("set_task_home"));
@@ -988,28 +934,20 @@ tTree InitTaskRegion
      stmt = mACF_BASIC (stmt);
    
   }
-  {
    return stmt;
-  }
  }
 
   }
   }
   if (task_body->ACF_ON.ON_HOME->Kind == kON_PROCS) {
   if (Definitions_IsType (task_body->ACF_ON.ON_HOME->ON_PROCS.topid, kObject)) {
-# line 657 "CodeOn.puma"
+/* line 663 "CodeOn.puma" */
  {
   tTree create;
   tTree stmt;
   tTree params;
   {
-# line 660 "CodeOn.puma"
-
-# line 661 "CodeOn.puma"
-
-# line 662 "CodeOn.puma"
-
-# line 664 "CodeOn.puma"
+/* line 670 "CodeOn.puma" */
  create = MakeSubProcStmt (task_body->ACF_ON.ON_HOME->ON_PROCS.topid);
 
      params = mBTP_EMPTY ();
@@ -1023,35 +961,29 @@ tTree InitTaskRegion
      stmt = mACF_LIST (stmt, NoTree);
    
   }
-  {
    return CombineACF (create, stmt);
-  }
  }
 
   }
   }
   }
   if (task_body->Kind == kACF_DO) {
-# line 680 "CodeOn.puma"
+/* line 686 "CodeOn.puma" */
  {
   tTree stmt;
   {
-# line 682 "CodeOn.puma"
-
-# line 684 "CodeOn.puma"
+/* line 690 "CodeOn.puma" */
    stmt = CompleteACFs (InitTaskRegion (task_body->ACF_DO.DO_BODY));
-# line 686 "CodeOn.puma"
+/* line 692 "CodeOn.puma" */
    stmt = mACF_DO (CopyTree (task_body->ACF_DO.DO_ID), CopyTree (task_body->ACF_DO.DO_RANGE), stmt, CopyTree (task_body->ACF_DO.DO_DEP_INFO), CopyTree (task_body->ACF_DO.DO_HOME_INFO));
   }
-  {
    return mACF_LIST (stmt, NoTree);
-  }
  }
 
   }
-# line 692 "CodeOn.puma"
+/* line 698 "CodeOn.puma" */
   {
-# line 693 "CodeOn.puma"
+/* line 699 "CodeOn.puma" */
    failure_protocol (MODULE, "InitTaskRegion", task_body);
   }
    return NoTree;
@@ -1060,21 +992,17 @@ tTree InitTaskRegion
 
 tTree StartTasking
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 699 "CodeOn.puma"
+/* line 705 "CodeOn.puma" */
  {
   tTree stmt;
   tTree params;
   {
-# line 700 "CodeOn.puma"
-
-# line 701 "CodeOn.puma"
-
-# line 703 "CodeOn.puma"
+/* line 709 "CodeOn.puma" */
  params = mBTP_EMPTY ();
 
      stmt = mPROC_OBJ (MakeDalibId ("start_tasking"));
@@ -1082,30 +1010,24 @@ tTree StartTasking
      stmt = mACF_BASIC (stmt);
    
   }
-  {
    return stmt;
-  }
  }
 
 }
 
 tTree StopTasking
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 715 "CodeOn.puma"
+/* line 721 "CodeOn.puma" */
  {
   tTree stmt;
   tTree params;
   {
-# line 716 "CodeOn.puma"
-
-# line 717 "CodeOn.puma"
-
-# line 719 "CodeOn.puma"
+/* line 725 "CodeOn.puma" */
  params = mBTP_EMPTY ();
 
      stmt = mPROC_OBJ (MakeDalibId ("stop_tasking"));
@@ -1113,41 +1035,31 @@ tTree StopTasking
      stmt = mACF_BASIC (stmt);
    
   }
-  {
    return stmt;
-  }
  }
 
 }
 
 tTree ApplyOnInfo
 # if defined __STDC__ | defined __cplusplus
-(register tTree t, register bool push_flag)
+(register tTree t, register rbool push_flag)
 # else
 (t, push_flag)
  register tTree t;
- register bool push_flag;
+ register rbool push_flag;
 # endif
 {
   if (t->Kind == kACF_DO) {
   if (t->ACF_DO.DO_HOME_INFO->Kind == kON_INFO) {
-# line 760 "CodeOn.puma"
+/* line 766 "CodeOn.puma" */
  {
-  int yyV1;
-  tTree yyV2;
   tTree new_body;
   tTree val;
   {
-# line 763 "CodeOn.puma"
-   TemplateDimInfo (t->ACF_DO.DO_HOME_INFO->ON_INFO.on_id, t->ACF_DO.DO_HOME_INFO->ON_INFO.on_dim, & yyV1, & yyV2);
-# line 765 "CodeOn.puma"
-   if (! ((IsMappedDistribution (yyV1, yyV2)))) goto yyL1;
+/* line 779 "CodeOn.puma" */
+   if (! ((IsMappedDistribution (t->ACF_DO.DO_HOME_INFO->ON_INFO.kind, t->ACF_DO.DO_HOME_INFO->ON_INFO.size)))) goto yyL1;
   {
-# line 767 "CodeOn.puma"
-
-# line 768 "CodeOn.puma"
-
-# line 770 "CodeOn.puma"
+/* line 784 "CodeOn.puma" */
  val = mVAR_EXP (CopyTree (t->ACF_DO.DO_ID));
      val = MultConstant (val, t->ACF_DO.DO_HOME_INFO->ON_INFO.stride);
      val = AddConstant (val, t->ACF_DO.DO_HOME_INFO->ON_INFO.base);
@@ -1157,31 +1069,21 @@ tTree ApplyOnInfo
      t->ACF_DO.DO_BODY = MakeStmtList (new_body);
      t->ACF_DO.DO_HOME_INFO = mNO_HOME_INFO ();
    
-# line 780 "CodeOn.puma"
+/* line 794 "CodeOn.puma" */
    CodeParallelLoop (t);
   }
   }
-  {
    return t;
-  }
  }
 yyL1:;
 
-# line 784 "CodeOn.puma"
+/* line 798 "CodeOn.puma" */
  {
   tTree range_stmt;
   tTree new_stmts;
-  int yyV1;
-  tTree yyV2;
   {
-# line 787 "CodeOn.puma"
-
-# line 788 "CodeOn.puma"
-
-# line 790 "CodeOn.puma"
-   TemplateDimInfo (t->ACF_DO.DO_HOME_INFO->ON_INFO.on_id, t->ACF_DO.DO_HOME_INFO->ON_INFO.on_dim, & yyV1, & yyV2);
-# line 792 "CodeOn.puma"
- range_stmt = GetLocalRange (t->ACF_DO.DO_RANGE, yyV1, t->ACF_DO.DO_HOME_INFO->ON_INFO.base, t->ACF_DO.DO_HOME_INFO->ON_INFO.stride, t->ACF_DO.DO_HOME_INFO->ON_INFO.on_id, t->ACF_DO.DO_HOME_INFO->ON_INFO.on_dim);
+/* line 806 "CodeOn.puma" */
+ range_stmt = GetLocalRange (t->ACF_DO.DO_RANGE, t->ACF_DO.DO_HOME_INFO->ON_INFO.kind, t->ACF_DO.DO_HOME_INFO->ON_INFO.base, t->ACF_DO.DO_HOME_INFO->ON_INFO.stride, t->ACF_DO.DO_HOME_INFO->ON_INFO.on_id, t->ACF_DO.DO_HOME_INFO->ON_INFO.on_dim);
 
      new_stmts = t;
 
@@ -1200,16 +1102,14 @@ yyL1:;
 
    
   }
-  {
    return new_stmts;
-  }
  }
 
   }
   }
-# line 814 "CodeOn.puma"
+/* line 828 "CodeOn.puma" */
   {
-# line 816 "CodeOn.puma"
+/* line 830 "CodeOn.puma" */
    failure_protocol (MODULE, "ApplyOnInfo", t);
   }
    return NoTree;
@@ -1225,20 +1125,16 @@ static tTree PushContext
  register int on_dim;
 # endif
 {
-# line 829 "CodeOn.puma"
+/* line 843 "CodeOn.puma" */
  {
   tTree params;
   {
-# line 831 "CodeOn.puma"
-
-# line 833 "CodeOn.puma"
+/* line 847 "CodeOn.puma" */
  params = mBTP_EMPTY ();
      params = mBTP_LIST (ExpToVarParam (MakeConstant (on_dim)), params);
    
   }
-  {
    return MakePushHomeCall (on_id->Object.Ident, params);
-  }
  }
 
 }
@@ -1252,7 +1148,7 @@ static tTree PopContext
  register int on_dim;
 # endif
 {
-# line 842 "CodeOn.puma"
+/* line 856 "CodeOn.puma" */
    return MakePopHomeCall ();
 
 }
@@ -1268,23 +1164,19 @@ tTree CodeONPushContext
 {
   if (top->Kind == kON_VAR) {
   if (Definitions_IsType (top->ON_VAR.arrayid, kObject)) {
-# line 855 "CodeOn.puma"
+/* line 869 "CodeOn.puma" */
    return MakePushHomeCall (top->ON_VAR.arrayid->Object.Ident, MakePushParams (specs));
 
   }
   }
   if (top->Kind == kON_PROCS) {
   if (Definitions_IsType (top->ON_PROCS.topid, kObject)) {
-# line 860 "CodeOn.puma"
+/* line 874 "CodeOn.puma" */
  {
   tTree stmt;
   tTree params;
   {
-# line 862 "CodeOn.puma"
-
-# line 863 "CodeOn.puma"
-
-# line 865 "CodeOn.puma"
+/* line 879 "CodeOn.puma" */
  params = mBTP_EMPTY ();
 
       params = mBTP_LIST (mVAR_PARAM (MakeUsedVarA (top->ON_PROCS.topid->Object.Ident, "_TOPID")),
@@ -1295,24 +1187,18 @@ tTree CodeONPushContext
       stmt = mACF_BASIC (stmt);
     
   }
-  {
    return stmt;
-  }
  }
 
   }
   }
   if (top->Kind == kON_HOST) {
-# line 878 "CodeOn.puma"
+/* line 892 "CodeOn.puma" */
  {
   tTree stmt;
   tTree params;
   {
-# line 880 "CodeOn.puma"
-
-# line 881 "CodeOn.puma"
-
-# line 883 "CodeOn.puma"
+/* line 897 "CodeOn.puma" */
  params = mBTP_EMPTY ();
 
       stmt = mPROC_OBJ (MakeDalibId ("push_host_context"));
@@ -1320,15 +1206,13 @@ tTree CodeONPushContext
       stmt = mACF_BASIC (stmt);
     
   }
-  {
    return stmt;
-  }
  }
 
   }
-# line 893 "CodeOn.puma"
+/* line 907 "CodeOn.puma" */
   {
-# line 895 "CodeOn.puma"
+/* line 909 "CodeOn.puma" */
    failure_protocol (MODULE, "CodeOnContext", top);
   }
    return NoTree;
@@ -1345,59 +1229,47 @@ tTree CodeONPopContext
 # endif
 {
   if (top->Kind == kON_VAR) {
-# line 901 "CodeOn.puma"
+/* line 915 "CodeOn.puma" */
    return MakePopHomeCall ();
 
   }
   if (top->Kind == kON_PROCS) {
-# line 906 "CodeOn.puma"
+/* line 920 "CodeOn.puma" */
  {
   tTree stmt;
   tTree params;
   {
-# line 908 "CodeOn.puma"
-
-# line 909 "CodeOn.puma"
-
-# line 911 "CodeOn.puma"
+/* line 925 "CodeOn.puma" */
  params = mBTP_EMPTY ();
 
       stmt = mCALL_STMT (mPROC_OBJ (MakeDalibId ("pop_procs_context")), params);
       stmt = mACF_BASIC (stmt);
     
   }
-  {
    return stmt;
-  }
  }
 
   }
   if (top->Kind == kON_HOST) {
-# line 920 "CodeOn.puma"
+/* line 934 "CodeOn.puma" */
  {
   tTree stmt;
   tTree params;
   {
-# line 922 "CodeOn.puma"
-
-# line 923 "CodeOn.puma"
-
-# line 925 "CodeOn.puma"
+/* line 939 "CodeOn.puma" */
  params = mBTP_EMPTY ();
 
       stmt = mCALL_STMT (mPROC_OBJ (MakeDalibId ("pop_host_context")), params);
       stmt = mACF_BASIC (stmt);
     
   }
-  {
    return stmt;
-  }
  }
 
   }
-# line 934 "CodeOn.puma"
+/* line 948 "CodeOn.puma" */
   {
-# line 936 "CodeOn.puma"
+/* line 950 "CodeOn.puma" */
    failure_protocol (MODULE, "CodeOnContext", top);
   }
    return NoTree;
@@ -1413,23 +1285,23 @@ static tTree MakePushParams
 # endif
 {
   if (specs->Kind == kON_EMPTY) {
-# line 948 "CodeOn.puma"
+/* line 962 "CodeOn.puma" */
    return mBTP_EMPTY ();
 
   }
   if (specs->Kind == kON_LIST) {
-# line 953 "CodeOn.puma"
+/* line 967 "CodeOn.puma" */
    return mBTP_LIST (MakePushParams (specs->ON_LIST.Elem), MakePushParams (specs->ON_LIST.Next));
 
   }
   if (specs->Kind == kON_SPEC) {
-# line 959 "CodeOn.puma"
+/* line 973 "CodeOn.puma" */
    return ExpToVarParam (MakeConstant (specs->ON_SPEC.array_dim));
 
   }
-# line 964 "CodeOn.puma"
+/* line 978 "CodeOn.puma" */
   {
-# line 965 "CodeOn.puma"
+/* line 979 "CodeOn.puma" */
    failure_protocol (MODULE, "MakePushParams", specs);
   }
    return NoTree;
@@ -1445,23 +1317,17 @@ static tTree MakePushHomeCall
  register tTree spec_params;
 # endif
 {
-# line 977 "CodeOn.puma"
+/* line 991 "CodeOn.puma" */
  {
   int len;
   tTree params;
   tTree stmt;
   {
-# line 979 "CodeOn.puma"
-
-# line 980 "CodeOn.puma"
-
-# line 981 "CodeOn.puma"
-
-# line 983 "CodeOn.puma"
+/* line 997 "CodeOn.puma" */
  len    = TreeListLength (spec_params);
 
       params = mBTP_LIST (ExpToVarParam (MakeConstant (len)), spec_params);
-      params = mBTP_LIST (mVAR_PARAM (MakeUsedVarA (id, LOCAL_SUFFIX)),
+      params = mBTP_LIST (mVAR_PARAM (MakeUsedVarA (id, DSP_SUFFIX)),
                           params);
 
       stmt = mPROC_OBJ (MakeDalibId ("push_home_context"));
@@ -1469,47 +1335,39 @@ static tTree MakePushHomeCall
       stmt = mACF_BASIC (stmt);
     
   }
-  {
    return stmt;
-  }
  }
 
 }
 
 static tTree MakePopHomeCall
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 999 "CodeOn.puma"
+/* line 1013 "CodeOn.puma" */
  {
   tTree stmt;
   tTree params;
   {
-# line 1001 "CodeOn.puma"
-
-# line 1002 "CodeOn.puma"
-
-# line 1004 "CodeOn.puma"
+/* line 1018 "CodeOn.puma" */
  params = mBTP_EMPTY ();
 
      stmt = mCALL_STMT (mPROC_OBJ (MakeDalibId ("pop_home_context")), params);
      stmt = mACF_BASIC (stmt);
    
   }
-  {
    return stmt;
-  }
  }
 
 }
 
-void BeginCodeOn ()
+void BeginCodeOn ARGS ((void))
 {
 }
 
-void CloseCodeOn ()
+void CloseCodeOn ARGS ((void))
 {
 }
