@@ -1,6 +1,13 @@
 # include "Common.h"
-# include "yyCommon.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,33 +15,30 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 67 "Common.puma"
+/* line 67 "Common.puma" */
 
 
 # define  MODULE "Common"
 
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 # include "Objects.h"
 # include "Types.h"
 # include "protocol.h"
@@ -54,11 +58,25 @@ static int overlap_error;
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* Common_Exit) () = yyExit;
+# include "yyCommon.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* Common_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -67,59 +85,59 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module Common, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module Common, routine %s failed\n",
+  yyFunction);
  Common_Exit ();
 }
 
-void CheckCommonBlock ARGS((tTree t, bool is_main));
-void CheckCommonDescriptors ARGS((tTree t, CompareRoutine p));
-static void CompareGlobalLocal ARGS((tDefinitions t, tTree common, bool is_main));
-static int CommonEntries ARGS((tTree cd));
-static int GetCommonSize ARGS((tTree t));
-static int GetTypeSize ARGS((tTree t));
-static int GetIndexSize ARGS((tTree t));
-static void MatchCommonBlocks ARGS((tDefinitions t, tTree common, CompareRoutine p));
-static void MatchForDescriptors ARGS((tTree global, tTree local, tDefinitions scope, int diff, CompareRoutine p));
-void CheckActualDescriptors ARGS((tTree t, CompareRoutine p));
-static int GetObjSize ARGS((tDefinitions obj));
-static bool IsSequential ARGS((tDefinitions obj));
-void PrintCommonBlock ARGS((tTree decl, int dist_flag));
-static void PrintCommonData ARGS((tTree decl, tDefinitions scope, int dist_flag));
-void PrintCall ARGS((tTree call));
-static void PrintParameters ARGS((tTree actuals, tTree dummies, tDefinitions dscope));
-static void PrintParam ARGS((tTree actual, tDefinitions dummy));
-static void MatchParameterList ARGS((tTree actuals, tTree dummies, tDefinitions dscope, CompareRoutine cp));
-static void MatchSingleParameter ARGS((tTree actual, tDefinitions dummy, CompareRoutine p));
-static void GetFullVarObject ARGS((tTree param, bool * yyP2, tDefinitions * yyP1));
+void CheckCommonBlock ARGS ((tTree t, rbool is_main));
+void CheckCommonDescriptors ARGS ((tTree t, CompareRoutine p));
+static void CompareGlobalLocal ARGS ((tDefinitions t, tTree common, rbool is_main));
+static int CommonEntries ARGS ((tTree cd));
+static int GetCommonSize ARGS ((tTree t));
+static int GetTypeSize ARGS ((tTree t));
+static int GetIndexSize ARGS ((tTree t));
+static void MatchCommonBlocks ARGS ((tDefinitions t, tTree common, CompareRoutine p));
+static void MatchForDescriptors ARGS ((tTree global, tTree local, tDefinitions scope, int diff, CompareRoutine p));
+void CheckActualDescriptors ARGS ((tTree t, CompareRoutine p));
+static tTree RealActuals ARGS ((tTree actuals, tDefinitions routine));
+static int GetObjSize ARGS ((tDefinitions obj));
+static rbool IsSequential ARGS ((tDefinitions obj));
+void PrintCommonBlock ARGS ((tTree decl, int dist_flag));
+static void PrintCommonData ARGS ((tTree decl, tDefinitions scope, int dist_flag));
+void PrintCall ARGS ((tTree call));
+static void PrintParameters ARGS ((tTree actuals, tTree dummies, tDefinitions dscope));
+static void PrintParam ARGS ((tTree actual, tDefinitions dummy));
+static void MatchParameterList ARGS ((tTree actuals, tTree dummies, tDefinitions dscope, CompareRoutine cp));
+static void MatchSingleParameter ARGS ((tTree actual, tDefinitions dummy, CompareRoutine p));
+static void GetFullVarObject ARGS ((tTree param, rbool * yyP2, tDefinitions * yyP1));
 
 void CheckCommonBlock
 # if defined __STDC__ | defined __cplusplus
-(register tTree t, register bool is_main)
+(register tTree t, register rbool is_main)
 # else
 (t, is_main)
  register tTree t;
- register bool is_main;
+ register rbool is_main;
 # endif
 {
   if (t->Kind == kCOMMON_DECL) {
-# line 103 "Common.puma"
+/* line 103 "Common.puma" */
  {
   tDefinitions global;
   {
-# line 105 "Common.puma"
-
-# line 107 "Common.puma"
+/* line 107 "Common.puma" */
    global = GetDeclEntry (t->COMMON_DECL.Ident, GetCommonEntries ());
-# line 109 "Common.puma"
+/* line 109 "Common.puma" */
    CompareGlobalLocal (global, t, is_main);
   }
    return;
  }
 
   }
-# line 112 "Common.puma"
+/* line 112 "Common.puma" */
   {
-# line 113 "Common.puma"
+/* line 113 "Common.puma" */
    failure_protocol (MODULE, "CheckCommonBlock", t);
   }
    return;
@@ -137,24 +155,22 @@ void CheckCommonDescriptors
 # endif
 {
   if (t->Kind == kCOMMON_DECL) {
-# line 127 "Common.puma"
+/* line 127 "Common.puma" */
  {
   tDefinitions global;
   {
-# line 129 "Common.puma"
-
-# line 131 "Common.puma"
+/* line 131 "Common.puma" */
    global = GetDeclEntry (t->COMMON_DECL.Ident, GetCommonEntries ());
-# line 133 "Common.puma"
+/* line 133 "Common.puma" */
    MatchCommonBlocks (global, t, p);
   }
    return;
  }
 
   }
-# line 136 "Common.puma"
+/* line 136 "Common.puma" */
   {
-# line 137 "Common.puma"
+/* line 137 "Common.puma" */
    failure_protocol (MODULE, "CheckCommonDescriptors", t);
   }
    return;
@@ -164,24 +180,24 @@ void CheckCommonDescriptors
 
 static void CompareGlobalLocal
 # if defined __STDC__ | defined __cplusplus
-(register tDefinitions t, register tTree common, register bool is_main)
+(register tDefinitions t, register tTree common, register rbool is_main)
 # else
 (t, common, is_main)
  register tDefinitions t;
  register tTree common;
- register bool is_main;
+ register rbool is_main;
 # endif
 {
-# line 148 "Common.puma"
+/* line 148 "Common.puma" */
 
 char msg[150];
 
-# line 152 "Common.puma"
+/* line 152 "Common.puma" */
   {
-# line 156 "Common.puma"
+/* line 156 "Common.puma" */
    if (! ((t == NoObject))) goto yyL1;
   {
-# line 157 "Common.puma"
+/* line 157 "Common.puma" */
    tree_error_protocol ("No Object for Common", common);
   }
   }
@@ -189,12 +205,12 @@ char msg[150];
 yyL1:;
 
   if (t->Kind == kCommonObject) {
-# line 160 "Common.puma"
+/* line 160 "Common.puma" */
   {
-# line 163 "Common.puma"
+/* line 163 "Common.puma" */
    if (! ((t->CommonObject.decl == common))) goto yyL2;
   {
-# line 166 "Common.puma"
+/* line 166 "Common.puma" */
  
 
      t->CommonObject.size     = GetCommonSize     (common);
@@ -207,16 +223,14 @@ yyL1:;
    return;
 yyL2:;
 
-# line 175 "Common.puma"
+/* line 175 "Common.puma" */
  {
   int size;
   {
-# line 178 "Common.puma"
+/* line 178 "Common.puma" */
    if (! ((t->CommonObject.decl != common))) goto yyL3;
   {
-# line 182 "Common.puma"
-
-# line 184 "Common.puma"
+/* line 184 "Common.puma" */
  t->CommonObject.main = t->CommonObject.main || is_main;
 
          
@@ -225,9 +239,13 @@ yyL2:;
 
      if (size != t->CommonObject.size)
 
-       { simple_warning_protocol
+       { set_protocol_stmt (common);
+
+         serious_warning_protocol
               ("incompatible lengths for common block data");
+
          PrintCommonBlock (common, 0);
+
          sprintf (msg,"acutal has size %d , reference has size %d", 
                        size, t->CommonObject.size);
          print_protocol (msg);
@@ -251,9 +269,9 @@ yyL2:;
 yyL3:;
 
   }
-# line 213 "Common.puma"
+/* line 217 "Common.puma" */
   {
-# line 214 "Common.puma"
+/* line 219 "Common.puma" */
    failure_protocol (MODULE, "CheckGlobalLocal", common);
   }
    return;
@@ -270,11 +288,12 @@ static int CommonEntries
 # endif
 {
   if (cd->Kind == kCOMMON_DECL) {
-# line 225 "Common.puma"
+/* line 230 "Common.puma" */
    return TreeListLength (cd->COMMON_DECL.IDS);
 
   }
  yyAbort ("CommonEntries");
+ { int yyDummy; return yyDummy; }
 }
 
 static int GetCommonSize
@@ -285,40 +304,38 @@ static int GetCommonSize
  register tTree t;
 # endif
 {
+ yyRecursion:
   if (t->Kind == kCOMMON_DECL) {
-# line 242 "Common.puma"
-   return GetCommonSize (t->COMMON_DECL.IDS);
+/* line 247 "Common.puma" */
+   t = t->COMMON_DECL.IDS;
+   goto yyRecursion;
 
   }
   if (t->Kind == kDECL_LIST) {
-# line 246 "Common.puma"
+/* line 251 "Common.puma" */
    return GetCommonSize (t->DECL_LIST.Elem) + GetCommonSize (t->DECL_LIST.Next);
 
   }
   if (t->Kind == kDECL_EMPTY) {
-# line 250 "Common.puma"
+/* line 255 "Common.puma" */
    return 0;
 
   }
   if (t->Kind == kVAR_DECL) {
-# line 254 "Common.puma"
+/* line 259 "Common.puma" */
  {
   tDefinitions Obj;
   {
-# line 258 "Common.puma"
-
-# line 259 "Common.puma"
+/* line 264 "Common.puma" */
    Obj = GetLocalObject (t->VAR_DECL.Ident);
   }
-  {
    return GetTypeSize (Obj->VarObject.decl);
-  }
  }
 
   }
-# line 264 "Common.puma"
+/* line 269 "Common.puma" */
   {
-# line 265 "Common.puma"
+/* line 270 "Common.puma" */
    failure_protocol (MODULE, "GetCommonSize", t);
   }
    return 0;
@@ -333,17 +350,19 @@ static int GetTypeSize
  register tTree t;
 # endif
 {
+ yyRecursion:
   if (t->Kind == kVAR_DECL) {
-# line 275 "Common.puma"
-   return GetTypeSize (t->VAR_DECL.VAL);
+/* line 280 "Common.puma" */
+   t = t->VAR_DECL.VAL;
+   goto yyRecursion;
 
   }
   if (t->Kind == kARRAY_TYPE) {
-# line 279 "Common.puma"
+/* line 284 "Common.puma" */
    return GetIndexSize (t->ARRAY_TYPE.ARRAY_INDEX_TYPES) * TreeSize (t->ARRAY_TYPE.ARRAY_COMP_TYPE);
 
   }
-# line 283 "Common.puma"
+/* line 288 "Common.puma" */
    return TreeSize (t);
 
 }
@@ -356,62 +375,54 @@ static int GetIndexSize
  register tTree t;
 # endif
 {
+ yyRecursion:
   if (t->Kind == kSHAPE_LIST) {
   if (t->SHAPE_LIST.Next->Kind == kSHAPE_EMPTY) {
-# line 293 "Common.puma"
-   return GetIndexSize (t->SHAPE_LIST.Elem);
+/* line 298 "Common.puma" */
+   t = t->SHAPE_LIST.Elem;
+   goto yyRecursion;
 
   }
-# line 297 "Common.puma"
+/* line 302 "Common.puma" */
    return GetIndexSize (t->SHAPE_LIST.Elem) * GetIndexSize (t->SHAPE_LIST.Next);
 
   }
   if (t->Kind == kEXPLICIT_SHAPE) {
-# line 301 "Common.puma"
+/* line 306 "Common.puma" */
  {
   int lval;
   int hval;
   int size;
-  bool found;
+  rbool found;
   {
-# line 303 "Common.puma"
-
-# line 304 "Common.puma"
-
-# line 305 "Common.puma"
-
-# line 306 "Common.puma"
-
-# line 308 "Common.puma"
+/* line 313 "Common.puma" */
    GetIntConstValue (t->EXPLICIT_SHAPE.LOWER, & found, & lval);
-# line 309 "Common.puma"
+/* line 314 "Common.puma" */
    if (! (found)) goto yyL3;
   {
-# line 310 "Common.puma"
+/* line 315 "Common.puma" */
    GetIntConstValue (t->EXPLICIT_SHAPE.UPPER, & found, & hval);
-# line 311 "Common.puma"
+/* line 316 "Common.puma" */
    if (! (found)) goto yyL3;
   {
-# line 312 "Common.puma"
+/* line 317 "Common.puma" */
    size = hval - lval + 1;
   }
   }
   }
-  {
    return size;
-  }
  }
 yyL3:;
 
   }
   if (Tree_IsType (t, kSHAPE_SPEC)) {
-# line 316 "Common.puma"
+/* line 321 "Common.puma" */
    return 0;
 
   }
-# line 320 "Common.puma"
+/* line 325 "Common.puma" */
   {
-# line 321 "Common.puma"
+/* line 326 "Common.puma" */
    failure_protocol (MODULE, "GetIndexSize", t);
   }
    return 0;
@@ -428,16 +439,16 @@ static void MatchCommonBlocks
  CompareRoutine p;
 # endif
 {
-# line 333 "Common.puma"
+/* line 338 "Common.puma" */
 
 char msg[150];
 
-# line 337 "Common.puma"
+/* line 342 "Common.puma" */
   {
-# line 341 "Common.puma"
+/* line 346 "Common.puma" */
    if (! ((t == NoObject))) goto yyL1;
   {
-# line 342 "Common.puma"
+/* line 347 "Common.puma" */
    tree_error_protocol ("No Object for Common", common);
   }
   }
@@ -445,21 +456,21 @@ char msg[150];
 yyL1:;
 
   if (t->Kind == kCommonObject) {
-# line 345 "Common.puma"
+/* line 350 "Common.puma" */
   {
-# line 347 "Common.puma"
+/* line 352 "Common.puma" */
    if (! ((t->CommonObject.decl == common))) goto yyL2;
   }
    return;
 yyL2:;
 
-# line 350 "Common.puma"
+/* line 355 "Common.puma" */
   {
-# line 353 "Common.puma"
+/* line 358 "Common.puma" */
    error = NO_ERROR;
-# line 355 "Common.puma"
+/* line 360 "Common.puma" */
    MatchForDescriptors (t->CommonObject.decl, common, t->CommonObject.CommonElements, 0, p);
-# line 357 "Common.puma"
+/* line 362 "Common.puma" */
  if (error == SEQ_ERROR)
 
       { simple_error_protocol ("COMMON BLOCK mismatch of sequential data");
@@ -472,9 +483,9 @@ yyL2:;
    return;
 
   }
-# line 367 "Common.puma"
+/* line 372 "Common.puma" */
   {
-# line 368 "Common.puma"
+/* line 373 "Common.puma" */
    failure_protocol (MODULE, "MatchCommonBlocks", common);
   }
    return;
@@ -494,87 +505,90 @@ static void MatchForDescriptors
  CompareRoutine p;
 # endif
 {
+ yyRecursion:
   if (global->Kind == kCOMMON_DECL) {
   if (local->Kind == kCOMMON_DECL) {
-# line 384 "Common.puma"
+/* line 389 "Common.puma" */
   {
-# line 386 "Common.puma"
-   MatchForDescriptors (global->COMMON_DECL.IDS, local->COMMON_DECL.IDS, scope, diff, p);
+/* line 391 "Common.puma" */
+   global = global->COMMON_DECL.IDS;
+   local = local->COMMON_DECL.IDS;
+   goto yyRecursion;
   }
-   return;
 
   }
   }
   if (global->Kind == kDECL_LIST) {
-# line 389 "Common.puma"
+/* line 394 "Common.puma" */
  {
   tDefinitions global_obj;
   {
-# line 391 "Common.puma"
-
-# line 393 "Common.puma"
+/* line 398 "Common.puma" */
    global_obj = GetDeclEntry (global->DECL_LIST.Elem->DECL_NODE.Ident, scope);
-# line 395 "Common.puma"
+/* line 400 "Common.puma" */
    if (! ((IsSequential (global_obj)))) goto yyL2;
   {
-# line 397 "Common.puma"
-   MatchForDescriptors (global->DECL_LIST.Next, local, scope, diff - GetObjSize (global_obj), p);
+/* line 402 "Common.puma" */
+   global = global->DECL_LIST.Next;
+   diff = diff - GetObjSize (global_obj);
+   goto yyRecursion;
   }
   }
-   return;
  }
 yyL2:;
 
-  if (local->Kind == kDECL_LIST) {
-# line 413 "Common.puma"
-  {
-# line 416 "Common.puma"
-   if (! ((diff == 0))) goto yyL4;
-  {
-# line 420 "Common.puma"
-   p (GetLocalObject (local->DECL_LIST.Elem->DECL_NODE.Ident), GetDeclEntry (global->DECL_LIST.Elem->DECL_NODE.Ident, scope));
-# line 422 "Common.puma"
-   MatchForDescriptors (global->DECL_LIST.Next, local->DECL_LIST.Next, scope, diff, p);
-  }
-  }
-   return;
-yyL4:;
-
-  }
   }
   if (local->Kind == kDECL_LIST) {
-# line 401 "Common.puma"
+/* line 406 "Common.puma" */
  {
   tDefinitions local_obj;
   {
-# line 403 "Common.puma"
-
-# line 405 "Common.puma"
+/* line 410 "Common.puma" */
    local_obj = GetLocalObject (local->DECL_LIST.Elem->DECL_NODE.Ident);
-# line 407 "Common.puma"
+/* line 412 "Common.puma" */
    if (! ((IsSequential (local_obj)))) goto yyL3;
   {
-# line 409 "Common.puma"
-   MatchForDescriptors (global, local->DECL_LIST.Next, scope, diff + GetObjSize (local_obj), p);
+/* line 414 "Common.puma" */
+   local = local->DECL_LIST.Next;
+   diff = diff + GetObjSize (local_obj);
+   goto yyRecursion;
   }
   }
-   return;
  }
 yyL3:;
 
+  }
+  if (global->Kind == kDECL_LIST) {
+  if (local->Kind == kDECL_LIST) {
+/* line 418 "Common.puma" */
+  {
+/* line 421 "Common.puma" */
+   if (! ((diff == 0))) goto yyL4;
+  {
+/* line 425 "Common.puma" */
+   p (GetLocalObject (local->DECL_LIST.Elem->DECL_NODE.Ident), GetDeclEntry (global->DECL_LIST.Elem->DECL_NODE.Ident, scope));
+/* line 427 "Common.puma" */
+   global = global->DECL_LIST.Next;
+   local = local->DECL_LIST.Next;
+   goto yyRecursion;
+  }
+  }
+yyL4:;
+
+  }
   }
   if (global->Kind == kDECL_EMPTY) {
   if (local->Kind == kDECL_EMPTY) {
-# line 425 "Common.puma"
+/* line 430 "Common.puma" */
    return;
 
   }
   }
-# line 431 "Common.puma"
+/* line 436 "Common.puma" */
   {
-# line 437 "Common.puma"
+/* line 442 "Common.puma" */
    error = SEQ_ERROR;
-# line 438 "Common.puma"
+/* line 443 "Common.puma" */
    seq_diff = diff;
   }
    return;
@@ -592,37 +606,32 @@ void CheckActualDescriptors
 # endif
 {
   if (t->Kind == kCALL_STMT) {
-# line 451 "Common.puma"
+/* line 456 "Common.puma" */
  {
   tTree Dummies;
   tDefinitions DummyScope;
   tDefinitions RefObject;
   {
-# line 455 "Common.puma"
-
-# line 456 "Common.puma"
-
-# line 457 "Common.puma"
-
-# line 459 "Common.puma"
+/* line 464 "Common.puma" */
    RefObject = GetReferenceObject (t->CALL_STMT.CALL_ID->PROC_OBJ.Object);
-# line 461 "Common.puma"
+/* line 466 "Common.puma" */
    if (! ((RefObject != NoObject))) goto yyL1;
   {
-# line 465 "Common.puma"
+/* line 470 "Common.puma" */
    GetDummies (RefObject, & Dummies, & DummyScope);
-# line 467 "Common.puma"
+/* line 472 "Common.puma" */
    error = NO_ERROR;
-# line 469 "Common.puma"
+/* line 474 "Common.puma" */
  if ((Dummies == NoTree) || (DummyScope == NoDefinitions))
          { stmt_protocol ("illegal reference");
            obj_protocol ("call      object: ", t->CALL_STMT.CALL_ID->PROC_OBJ.Object);
            obj_protocol ("reference object: ", RefObject);
          }
        else
-         MatchParameterList (t->CALL_STMT.CALL_PARAMS, Dummies, DummyScope, p);
+         MatchParameterList (RealActuals (t->CALL_STMT.CALL_PARAMS, RefObject),
+                             Dummies, DummyScope, p);
     
-# line 478 "Common.puma"
+/* line 484 "Common.puma" */
  if (error != NO_ERROR)
          { simple_error_protocol ("MISMATCH of ACTUALS/DUMMIES");
            PrintCall (t);
@@ -634,33 +643,27 @@ void CheckActualDescriptors
  }
 yyL1:;
 
-# line 485 "Common.puma"
+/* line 491 "Common.puma" */
    return;
 
   }
   if (t->Kind == kFUNC_CALL_EXP) {
-# line 490 "Common.puma"
+/* line 496 "Common.puma" */
  {
   tTree Dummies;
   tDefinitions DummyScope;
   tDefinitions RefObject;
   {
-# line 494 "Common.puma"
-
-# line 495 "Common.puma"
-
-# line 496 "Common.puma"
-
-# line 498 "Common.puma"
+/* line 504 "Common.puma" */
    RefObject = GetReferenceObject (t->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Object);
-# line 500 "Common.puma"
+/* line 506 "Common.puma" */
    if (! ((RefObject != NoObject))) goto yyL3;
   {
-# line 502 "Common.puma"
+/* line 508 "Common.puma" */
    GetDummies (RefObject, & Dummies, & DummyScope);
-# line 504 "Common.puma"
+/* line 510 "Common.puma" */
    error = NO_ERROR;
-# line 506 "Common.puma"
+/* line 512 "Common.puma" */
  if ((Dummies == NoTree) || (DummyScope == NoDefinitions))
 
          { stmt_protocol ("illegal reference");
@@ -671,7 +674,7 @@ yyL1:;
        else
          MatchParameterList (t->FUNC_CALL_EXP.FUNC_PARAMS, Dummies, DummyScope, p);
     
-# line 517 "Common.puma"
+/* line 523 "Common.puma" */
  if (error != NO_ERROR)
          { simple_error_protocol ("MISMATCH of ACTUALS/DUMMIES");
            PrintCall (t);
@@ -683,13 +686,13 @@ yyL1:;
  }
 yyL3:;
 
-# line 524 "Common.puma"
+/* line 530 "Common.puma" */
    return;
 
   }
-# line 527 "Common.puma"
+/* line 533 "Common.puma" */
   {
-# line 528 "Common.puma"
+/* line 534 "Common.puma" */
    failure_protocol (MODULE, "CheckActualDescriptors", t);
   }
    return;
@@ -697,6 +700,36 @@ yyL3:;
 ;
 }
 
+static tTree RealActuals
+# if defined __STDC__ | defined __cplusplus
+(register tTree actuals, register tDefinitions routine)
+# else
+(actuals, routine)
+ register tTree actuals;
+ register tDefinitions routine;
+# endif
+{
+  if (actuals->Kind == kBTP_LIST) {
+  if (routine->Kind == kFuncObject) {
+/* line 548 "Common.puma" */
+   return actuals->BTP_LIST.Next;
+
+  }
+  }
+  if (routine->Kind == kFuncObject) {
+/* line 553 "Common.puma" */
+  {
+/* line 555 "Common.puma" */
+   error_protocol ("call of a function without arguments");
+  }
+   return actuals;
+
+  }
+/* line 560 "Common.puma" */
+   return actuals;
+
+}
+
 static int GetObjSize
 # if defined __STDC__ | defined __cplusplus
 (register tDefinitions obj)
@@ -705,24 +738,24 @@ static int GetObjSize
  register tDefinitions obj;
 # endif
 {
-# line 544 "Common.puma"
+/* line 578 "Common.puma" */
   {
-# line 545 "Common.puma"
+/* line 579 "Common.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   {
-# line 546 "Common.puma"
+/* line 580 "Common.puma" */
    failure_protocol (MODULE, "GetObjSize", NoTree);
   }
   }
    return 0;
 yyL1:;
 
-# line 550 "Common.puma"
+/* line 584 "Common.puma" */
    return GetTypeSize (obj->Object.decl);
 
 }
 
-static bool IsSequential
+static rbool IsSequential
 # if defined __STDC__ | defined __cplusplus
 (register tDefinitions obj)
 # else
@@ -730,27 +763,27 @@ static bool IsSequential
  register tDefinitions obj;
 # endif
 {
-# line 556 "Common.puma"
+/* line 590 "Common.puma" */
   {
-# line 557 "Common.puma"
+/* line 591 "Common.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   {
-# line 558 "Common.puma"
+/* line 592 "Common.puma" */
    failure_protocol (MODULE, "IsSequential", NoTree);
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 561 "Common.puma"
+/* line 595 "Common.puma" */
   {
-# line 562 "Common.puma"
+/* line 596 "Common.puma" */
    if (! ((VarDistribution (obj) == 0))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
 void PrintCommonBlock
@@ -763,30 +796,28 @@ void PrintCommonBlock
 # endif
 {
   if (decl->Kind == kCOMMON_DECL) {
-# line 577 "Common.puma"
+/* line 611 "Common.puma" */
  {
   tDefinitions global;
   {
-# line 579 "Common.puma"
-
-# line 581 "Common.puma"
+/* line 615 "Common.puma" */
    global = GetDeclEntry (decl->COMMON_DECL.Ident, GetCommonEntries ());
-# line 583 "Common.puma"
+/* line 617 "Common.puma" */
    tree_protocol ("actual common block    :\n", decl);
-# line 585 "Common.puma"
+/* line 619 "Common.puma" */
    PrintCommonData (decl, GetCurrentScope (), dist_flag);
-# line 587 "Common.puma"
+/* line 621 "Common.puma" */
    tree_protocol ("reference common block :\n", global->CommonObject.decl);
-# line 589 "Common.puma"
+/* line 623 "Common.puma" */
    PrintCommonData (global->CommonObject.decl, global->CommonObject.CommonElements, dist_flag);
   }
    return;
  }
 
   }
-# line 592 "Common.puma"
+/* line 626 "Common.puma" */
   {
-# line 594 "Common.puma"
+/* line 628 "Common.puma" */
    failure_protocol (MODULE, "PrintCommonBlock", decl);
   }
    return;
@@ -804,39 +835,34 @@ static void PrintCommonData
  register int dist_flag;
 # endif
 {
-# line 599 "Common.puma"
+/* line 633 "Common.puma" */
 
 char msg[150], string[100];
 
+ yyRecursion:
   if (decl->Kind == kCOMMON_DECL) {
-# line 603 "Common.puma"
+/* line 637 "Common.puma" */
   {
-# line 605 "Common.puma"
+/* line 639 "Common.puma" */
    PrintCommonData (decl->COMMON_DECL.IDS, scope, dist_flag);
-# line 606 "Common.puma"
+/* line 640 "Common.puma" */
    print_protocol ("");
   }
    return;
 
   }
   if (decl->Kind == kDECL_LIST) {
-# line 609 "Common.puma"
+/* line 643 "Common.puma" */
  {
   tDefinitions obj;
   int size;
   int is_dist;
   {
-# line 611 "Common.puma"
-
-# line 612 "Common.puma"
-
-# line 613 "Common.puma"
-
-# line 615 "Common.puma"
+/* line 649 "Common.puma" */
    obj = GetDeclEntry (decl->DECL_LIST.Elem->DECL_NODE.Ident, scope);
-# line 616 "Common.puma"
+/* line 650 "Common.puma" */
    size = GetObjSize (obj);
-# line 618 "Common.puma"
+/* line 652 "Common.puma" */
  GetString (decl->DECL_LIST.Elem->DECL_NODE.Ident, string);
   
      is_dist = 0;     
@@ -851,21 +877,21 @@ char msg[150], string[100];
 
      print_protocol (msg);
    
-# line 633 "Common.puma"
-   PrintCommonData (decl->DECL_LIST.Next, scope, dist_flag);
+/* line 667 "Common.puma" */
+   decl = decl->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
  }
 
   }
   if (decl->Kind == kDECL_EMPTY) {
-# line 636 "Common.puma"
+/* line 670 "Common.puma" */
    return;
 
   }
-# line 639 "Common.puma"
+/* line 673 "Common.puma" */
   {
-# line 641 "Common.puma"
+/* line 675 "Common.puma" */
    failure_protocol (MODULE, "PrintCommonData", decl);
   }
    return;
@@ -882,23 +908,19 @@ void PrintCall
 # endif
 {
   if (call->Kind == kCALL_STMT) {
-# line 654 "Common.puma"
+/* line 688 "Common.puma" */
  {
   tTree Dummies;
   tDefinitions DummyScope;
   {
-# line 656 "Common.puma"
+/* line 690 "Common.puma" */
    tree_protocol ("subroutine call is : ", call);
-# line 658 "Common.puma"
-
-# line 659 "Common.puma"
-
-# line 661 "Common.puma"
+/* line 695 "Common.puma" */
    if (! ((call->CALL_STMT.CALL_ID->PROC_OBJ.Object == GetDeclEntry (call->CALL_STMT.CALL_ID->PROC_OBJ.Ident, GetUnitEntries ())))) goto yyL1;
   {
-# line 663 "Common.puma"
+/* line 697 "Common.puma" */
    GetDummies (call->CALL_STMT.CALL_ID->PROC_OBJ.Object, & Dummies, & DummyScope);
-# line 665 "Common.puma"
+/* line 699 "Common.puma" */
    PrintParameters (call->CALL_STMT.CALL_PARAMS, Dummies, DummyScope);
   }
   }
@@ -906,34 +928,30 @@ void PrintCall
  }
 yyL1:;
 
-# line 668 "Common.puma"
+/* line 702 "Common.puma" */
   {
-# line 670 "Common.puma"
+/* line 704 "Common.puma" */
    tree_protocol ("subroutine call is : ", call);
-# line 671 "Common.puma"
+/* line 705 "Common.puma" */
    print_protocol ("no dummies are available");
   }
    return;
 
   }
   if (call->Kind == kFUNC_CALL_EXP) {
-# line 674 "Common.puma"
+/* line 708 "Common.puma" */
  {
   tTree Dummies;
   tDefinitions DummyScope;
   {
-# line 676 "Common.puma"
+/* line 710 "Common.puma" */
    tree_protocol ("function call is : ", call);
-# line 678 "Common.puma"
-
-# line 679 "Common.puma"
-
-# line 681 "Common.puma"
+/* line 715 "Common.puma" */
    if (! ((call->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Object == GetDeclEntry (call->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Ident, GetUnitEntries ())))) goto yyL3;
   {
-# line 683 "Common.puma"
+/* line 717 "Common.puma" */
    GetDummies (call->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Object, & Dummies, & DummyScope);
-# line 685 "Common.puma"
+/* line 719 "Common.puma" */
    PrintParameters (call->FUNC_CALL_EXP.FUNC_PARAMS, Dummies, DummyScope);
   }
   }
@@ -941,19 +959,19 @@ yyL1:;
  }
 yyL3:;
 
-# line 688 "Common.puma"
+/* line 722 "Common.puma" */
   {
-# line 690 "Common.puma"
+/* line 724 "Common.puma" */
    tree_protocol ("function call is : ", call);
-# line 691 "Common.puma"
+/* line 725 "Common.puma" */
    print_protocol ("no dummies are available");
   }
    return;
 
   }
-# line 694 "Common.puma"
+/* line 728 "Common.puma" */
   {
-# line 695 "Common.puma"
+/* line 729 "Common.puma" */
    failure_protocol (MODULE, "PrintCall", call);
   }
    return;
@@ -971,55 +989,57 @@ static void PrintParameters
  register tDefinitions dscope;
 # endif
 {
+ yyRecursion:
   if (actuals->Kind == kBTP_LIST) {
   if (dummies->Kind == kDECL_LIST) {
   if (dummies->DECL_LIST.Elem->Kind == kVAR_PARAM_DECL) {
-# line 701 "Common.puma"
+/* line 735 "Common.puma" */
   {
-# line 704 "Common.puma"
+/* line 738 "Common.puma" */
    PrintParam (actuals->BTP_LIST.Elem, GetDeclEntry (dummies->DECL_LIST.Elem->VAR_PARAM_DECL.Ident, dscope));
-# line 705 "Common.puma"
-   PrintParameters (actuals->BTP_LIST.Next, dummies->DECL_LIST.Next, dscope);
+/* line 739 "Common.puma" */
+   actuals = actuals->BTP_LIST.Next;
+   dummies = dummies->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   }
   if (dummies->Kind == kDECL_EMPTY) {
-# line 714 "Common.puma"
+/* line 748 "Common.puma" */
   {
-# line 716 "Common.puma"
+/* line 750 "Common.puma" */
    PrintParam (actuals->BTP_LIST.Elem, NoObject);
-# line 717 "Common.puma"
-   PrintParameters (actuals->BTP_LIST.Next, dummies, dscope);
+/* line 751 "Common.puma" */
+   actuals = actuals->BTP_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   }
   if (actuals->Kind == kBTP_EMPTY) {
   if (dummies->Kind == kDECL_LIST) {
   if (dummies->DECL_LIST.Elem->Kind == kVAR_PARAM_DECL) {
-# line 708 "Common.puma"
+/* line 742 "Common.puma" */
   {
-# line 710 "Common.puma"
+/* line 744 "Common.puma" */
    PrintParam (NoTree, GetDeclEntry (dummies->DECL_LIST.Elem->VAR_PARAM_DECL.Ident, dscope));
-# line 711 "Common.puma"
-   PrintParameters (actuals, dummies->DECL_LIST.Next, dscope);
+/* line 745 "Common.puma" */
+   dummies = dummies->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   }
   if (dummies->Kind == kDECL_EMPTY) {
-# line 720 "Common.puma"
+/* line 754 "Common.puma" */
    return;
 
   }
   }
-# line 723 "Common.puma"
+/* line 757 "Common.puma" */
   {
-# line 724 "Common.puma"
+/* line 758 "Common.puma" */
    failure2_protocol (MODULE, "PrintParameters", actuals, dummies);
   }
    return;
@@ -1036,27 +1056,27 @@ static void PrintParam
  register tDefinitions dummy;
 # endif
 {
-# line 729 "Common.puma"
+/* line 763 "Common.puma" */
   {
-# line 731 "Common.puma"
+/* line 765 "Common.puma" */
    if (! ((dummy == NoObject))) goto yyL1;
   {
-# line 733 "Common.puma"
+/* line 767 "Common.puma" */
    tree_protocol ("  actual : ", actual);
-# line 734 "Common.puma"
+/* line 768 "Common.puma" */
    print_protocol ("  dummy  : --");
-# line 735 "Common.puma"
+/* line 769 "Common.puma" */
    print_protocol ("");
   }
   }
    return;
 yyL1:;
 
-# line 738 "Common.puma"
+/* line 772 "Common.puma" */
   {
-# line 739 "Common.puma"
+/* line 773 "Common.puma" */
    tree_protocol ("  actual : ", actual);
-# line 740 "Common.puma"
+/* line 774 "Common.puma" */
    tree_protocol ("  dummy  : ", dummy->Object.decl);
   }
    return;
@@ -1075,16 +1095,17 @@ static void MatchParameterList
  CompareRoutine cp;
 # endif
 {
-# line 754 "Common.puma"
+ yyRecursion:
+/* line 788 "Common.puma" */
   {
-# line 756 "Common.puma"
+/* line 790 "Common.puma" */
    if (! ((TreeListLength (actuals) != TreeListLength (dummies)))) goto yyL1;
   {
-# line 758 "Common.puma"
+/* line 792 "Common.puma" */
    error_protocol ("mismatch of actual parameters with their dummies");
-# line 759 "Common.puma"
+/* line 793 "Common.puma" */
    tree_protocol ("actuals : ", actuals);
-# line 760 "Common.puma"
+/* line 794 "Common.puma" */
    tree_protocol ("dummies : ", dummies);
   }
   }
@@ -1094,28 +1115,29 @@ yyL1:;
   if (actuals->Kind == kBTP_LIST) {
   if (dummies->Kind == kDECL_LIST) {
   if (dummies->DECL_LIST.Elem->Kind == kVAR_PARAM_DECL) {
-# line 763 "Common.puma"
+/* line 797 "Common.puma" */
   {
-# line 766 "Common.puma"
+/* line 800 "Common.puma" */
    MatchSingleParameter (actuals->BTP_LIST.Elem, GetDeclEntry (dummies->DECL_LIST.Elem->VAR_PARAM_DECL.Ident, dscope), cp);
-# line 767 "Common.puma"
-   MatchParameterList (actuals->BTP_LIST.Next, dummies->DECL_LIST.Next, dscope, cp);
+/* line 801 "Common.puma" */
+   actuals = actuals->BTP_LIST.Next;
+   dummies = dummies->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   }
   }
   if (actuals->Kind == kBTP_EMPTY) {
   if (dummies->Kind == kDECL_EMPTY) {
-# line 770 "Common.puma"
+/* line 804 "Common.puma" */
    return;
 
   }
   }
-# line 774 "Common.puma"
+/* line 808 "Common.puma" */
   {
-# line 778 "Common.puma"
+/* line 812 "Common.puma" */
    failure2_protocol (MODULE, "MatchParameterList", actuals, dummies);
   }
    return;
@@ -1134,16 +1156,16 @@ static void MatchSingleParameter
 # endif
 {
   if (actual->Kind == kNO_PARAM) {
-# line 793 "Common.puma"
+/* line 827 "Common.puma" */
    return;
 
   }
-# line 796 "Common.puma"
+/* line 830 "Common.puma" */
   {
-# line 798 "Common.puma"
+/* line 832 "Common.puma" */
    if (! ((dummy == NoObject))) goto yyL2;
   {
-# line 800 "Common.puma"
+/* line 834 "Common.puma" */
  error_protocol ("dummy not found");
       tree_protocol ("acual argument is : ", actual);
     
@@ -1153,17 +1175,17 @@ static void MatchSingleParameter
 yyL2:;
 
   if (actual->Kind == kVAR_PARAM) {
-# line 805 "Common.puma"
+/* line 839 "Common.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   tDefinitions yyV2;
   {
-# line 807 "Common.puma"
+/* line 841 "Common.puma" */
    GetFullVarObject (actual->VAR_PARAM.V, & yyV1, & yyV2);
-# line 808 "Common.puma"
+/* line 842 "Common.puma" */
    if (! ((yyV1))) goto yyL3;
   {
-# line 810 "Common.puma"
+/* line 844 "Common.puma" */
    p (yyV2, dummy);
   }
   }
@@ -1172,9 +1194,9 @@ yyL2:;
 yyL3:;
 
   }
-# line 813 "Common.puma"
+/* line 847 "Common.puma" */
   {
-# line 815 "Common.puma"
+/* line 849 "Common.puma" */
    tree_protocol ("not applied for this actual argument : ", actual);
   }
    return;
@@ -1184,21 +1206,21 @@ yyL3:;
 
 static void GetFullVarObject
 # if defined __STDC__ | defined __cplusplus
-(register tTree param, register bool * yyP2, register tDefinitions * yyP1)
+(register tTree param, register rbool * yyP2, register tDefinitions * yyP1)
 # else
 (param, yyP2, yyP1)
  register tTree param;
- register bool * yyP2;
+ register rbool * yyP2;
  register tDefinitions * yyP1;
 # endif
 {
   if (param->Kind == kVAR_PARAM) {
-# line 826 "Common.puma"
+/* line 860 "Common.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   tDefinitions yyV2;
   {
-# line 828 "Common.puma"
+/* line 862 "Common.puma" */
    GetFullVarObject (param->VAR_PARAM.V, & yyV1, & yyV2);
   }
    * yyP2 = yyV1;
@@ -1208,22 +1230,22 @@ static void GetFullVarObject
 
   }
   if (param->Kind == kUSED_VAR) {
-# line 831 "Common.puma"
-   * yyP2 = true;
+/* line 865 "Common.puma" */
+   * yyP2 = rtrue;
    * yyP1 = param->USED_VAR.VARNAME->VAR_OBJ.Object;
    return;
 
   }
   if (param->Kind == kINDEXED_VAR) {
-# line 834 "Common.puma"
+/* line 868 "Common.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   tDefinitions yyV2;
   {
-# line 836 "Common.puma"
+/* line 870 "Common.puma" */
    if (! ((IsWholeVar (param)))) goto yyL3;
   {
-# line 838 "Common.puma"
+/* line 872 "Common.puma" */
    GetFullVarObject (param->INDEXED_VAR.IND_VAR, & yyV1, & yyV2);
   }
   }
@@ -1234,18 +1256,18 @@ static void GetFullVarObject
 yyL3:;
 
   }
-# line 841 "Common.puma"
-   * yyP2 = false;
+/* line 875 "Common.puma" */
+   * yyP2 = rfalse;
    * yyP1 = NoObject;
    return;
 
 ;
 }
 
-void BeginCommon ()
+void BeginCommon ARGS ((void))
 {
 }
 
-void CloseCommon ()
+void CloseCommon ARGS ((void))
 {
 }
