@@ -1,6 +1,13 @@
 # include "DefTable.h"
-# include "yyDefTable.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,33 +15,30 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 49 "DefTable.puma"
+/* line 49 "DefTable.puma" */
 
 
 # include <string.h>
 
 # include "Idents.h"     
-# include "StringMem.h"    /* GetString, ... */
+# include "StringM.h"    /* GetString, ... */
 # include "protocol.h"
 
 static tScopes ScopeList;
@@ -60,7 +64,7 @@ static tObject   HPFLocalLibModule;
 static tEntries  HPFTaskLibEntries;
 static tObject   HPFTaskLibModule;
 
-static tObject   DefaultTopologies [MAX_TORUS_RANK+2];
+static tObject   DefaultTopologies [MAX_DIMENSIONS+2];
 
 /* globally used for intrinsics */
 
@@ -725,6 +729,7 @@ static void InitIntrinsicEntries ()
   MakeHPFTaskLibProcEntry ("HPF_TASK_SIZE");
   MakeHPFTaskLibProcEntry ("HPF_SEND");
   MakeHPFTaskLibProcEntry ("HPF_RECV");
+  MakeHPFTaskLibProcEntry ("HPF_BCAST");
   MakeHPFTaskLibProcEntry ("HPF_SEND_INIT");
   MakeHPFTaskLibProcEntry ("HPF_RECV_INIT");
   MakeHPFTaskLibProcEntry ("HPF_TASK_COMM");
@@ -733,11 +738,25 @@ static void InitIntrinsicEntries ()
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* DefTable_Exit) () = yyExit;
+# include "yyDefTable.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* DefTable_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -746,38 +765,39 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module DefTable, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module DefTable, routine %s failed\n",
+  yyFunction);
  DefTable_Exit ();
 }
 
-static tDefinitions GetFirstScope ARGS((tDefinitions s));
-static tDefinitions GetTailScopes ARGS((tDefinitions s));
-void NewScope ARGS(());
-void OpenScope ARGS((tDefinitions Scope));
-void CloseScope ARGS(());
-tDefinitions GetCurrentScope ARGS(());
-tDefinitions GetExternalEntries ARGS(());
-tDefinitions GetIntrinsicEntries ARGS(());
-tDefinitions GetUnitEntries ARGS(());
-tDefinitions GetCommonEntries ARGS(());
-tDefinitions GetHPFLibrary ARGS(());
-tDefinitions GetHPFLocalLibrary ARGS(());
-tDefinitions GetHPFTaskLibrary ARGS(());
-void InsertUnitEntry ARGS((tDefinitions Obj));
-void InsertExternalEntry ARGS((tDefinitions Obj));
-void InsertCommonEntry ARGS((tDefinitions Obj));
-static void InsertEntryScopes ARGS((tDefinitions Obj, tDefinitions sl));
-void InsertEntry ARGS((tDefinitions Obj));
-static void ReplaceEntry ARGS((tIdent Name, tDefinitions New, tDefinitions scope));
-void ChangeEntry ARGS((tIdent Name, tDefinitions Obj));
-tDefinitions GetDeclEntry ARGS((tIdent Ident, tDefinitions Decls));
-static tDefinitions GetDecl ARGS((tIdent Ident, tDefinitions sl));
-tDefinitions GetLocalObject ARGS((tIdent Ident));
-tDefinitions GetGlobalObject ARGS((tIdent Ident));
-tDefinitions GetIntrinsicObject ARGS((tIdent Ident));
-tDefinitions GetDefaultTopology ARGS((int rank));
-tDefinitions RemoveHostAssociation ARGS(());
-void ResetHostAssociation ARGS((tDefinitions HostAssociation));
+static tDefinitions GetFirstScope ARGS ((tDefinitions s));
+static tDefinitions GetTailScopes ARGS ((tDefinitions s));
+void NewScope ARGS ((void));
+void OpenScope ARGS ((tDefinitions Scope));
+void CloseScope ARGS ((void));
+tDefinitions GetCurrentScope ARGS ((void));
+tDefinitions GetExternalEntries ARGS ((void));
+tDefinitions GetIntrinsicEntries ARGS ((void));
+tDefinitions GetUnitEntries ARGS ((void));
+tDefinitions GetCommonEntries ARGS ((void));
+tDefinitions GetHPFLibrary ARGS ((void));
+tDefinitions GetHPFLocalLibrary ARGS ((void));
+tDefinitions GetHPFTaskLibrary ARGS ((void));
+void InsertUnitEntry ARGS ((tDefinitions Obj));
+void InsertExternalEntry ARGS ((tDefinitions Obj));
+void InsertCommonEntry ARGS ((tDefinitions Obj));
+static void InsertEntryScopes ARGS ((tDefinitions Obj, tDefinitions sl));
+void InsertEntry ARGS ((tDefinitions Obj));
+static void ReplaceEntry ARGS ((tIdent Name, tDefinitions New, tDefinitions scope));
+void ChangeEntry ARGS ((tIdent Name, tDefinitions Obj));
+tDefinitions GetDeclEntry ARGS ((tIdent Ident, tDefinitions Decls));
+static tDefinitions GetDecl ARGS ((tIdent Ident, tDefinitions sl));
+tDefinitions GetLocalObject ARGS ((tIdent Ident));
+tDefinitions GetGlobalObject ARGS ((tIdent Ident));
+tDefinitions GetIntrinsicObject ARGS ((tIdent Ident));
+tDefinitions GetDefaultTopology ARGS ((int rank));
+tDefinitions RemoveHostAssociation ARGS ((void));
+void ResetHostAssociation ARGS ((tDefinitions HostAssociation));
 
 static tDefinitions GetFirstScope
 # if defined __STDC__ | defined __cplusplus
@@ -787,12 +807,12 @@ static tDefinitions GetFirstScope
  register tDefinitions s;
 # endif
 {
-# line 804 "DefTable.puma"
+/* line 805 "DefTable.puma" */
   {
-# line 805 "DefTable.puma"
+/* line 806 "DefTable.puma" */
    if (! ((s == NoDefinitions))) goto yyL1;
   {
-# line 806 "DefTable.puma"
+/* line 807 "DefTable.puma" */
    DefTableError ("GetFirstScope (NULL)");
   }
   }
@@ -800,20 +820,21 @@ static tDefinitions GetFirstScope
 yyL1:;
 
   if (s->Kind == kSCOPE_EMPTY) {
-# line 810 "DefTable.puma"
+/* line 811 "DefTable.puma" */
   {
-# line 812 "DefTable.puma"
+/* line 813 "DefTable.puma" */
    DefTableError ("GetFirstScope (no scopes)");
   }
    return NoEntries;
 
   }
   if (s->Kind == kSCOPE_LIST) {
-# line 816 "DefTable.puma"
+/* line 817 "DefTable.puma" */
    return s->SCOPE_LIST.Table;
 
   }
  yyAbort ("GetFirstScope");
+ { tDefinitions yyDummy; return yyDummy; }
 }
 
 static tDefinitions GetTailScopes
@@ -824,12 +845,12 @@ static tDefinitions GetTailScopes
  register tDefinitions s;
 # endif
 {
-# line 823 "DefTable.puma"
+/* line 824 "DefTable.puma" */
   {
-# line 824 "DefTable.puma"
+/* line 825 "DefTable.puma" */
    if (! ((s == NoDefinitions))) goto yyL1;
   {
-# line 825 "DefTable.puma"
+/* line 826 "DefTable.puma" */
    DefTableError ("GetTailScopes (NULL)");
   }
   }
@@ -837,32 +858,33 @@ static tDefinitions GetTailScopes
 yyL1:;
 
   if (s->Kind == kSCOPE_EMPTY) {
-# line 829 "DefTable.puma"
+/* line 830 "DefTable.puma" */
   {
-# line 831 "DefTable.puma"
+/* line 832 "DefTable.puma" */
    DefTableError ("GetTailScopes (no scopes)");
   }
    return NoDefinitions;
 
   }
   if (s->Kind == kSCOPE_LIST) {
-# line 835 "DefTable.puma"
+/* line 836 "DefTable.puma" */
    return s->SCOPE_LIST.Hidden;
 
   }
  yyAbort ("GetTailScopes");
+ { tDefinitions yyDummy; return yyDummy; }
 }
 
 void NewScope
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 848 "DefTable.puma"
+/* line 849 "DefTable.puma" */
   {
-# line 850 "DefTable.puma"
+/* line 851 "DefTable.puma" */
    ScopeList = mSCOPE_LIST (mENTRY_EMPTY (), ScopeList);
   }
    return;
@@ -878,12 +900,12 @@ void OpenScope
  register tDefinitions Scope;
 # endif
 {
-# line 855 "DefTable.puma"
+/* line 856 "DefTable.puma" */
   {
-# line 856 "DefTable.puma"
+/* line 857 "DefTable.puma" */
    if (! ((Scope == NoDefinitions))) goto yyL1;
   {
-# line 858 "DefTable.puma"
+/* line 859 "DefTable.puma" */
    DefTableError ("OpenScope: illegal structure");
   }
   }
@@ -891,26 +913,26 @@ void OpenScope
 yyL1:;
 
   if (Scope->Kind == kENTRY_LIST) {
-# line 861 "DefTable.puma"
+/* line 862 "DefTable.puma" */
   {
-# line 863 "DefTable.puma"
+/* line 864 "DefTable.puma" */
    ScopeList = mSCOPE_LIST (Scope, ScopeList);
   }
    return;
 
   }
   if (Scope->Kind == kENTRY_EMPTY) {
-# line 866 "DefTable.puma"
+/* line 867 "DefTable.puma" */
   {
-# line 868 "DefTable.puma"
+/* line 869 "DefTable.puma" */
    ScopeList = mSCOPE_LIST (Scope, ScopeList);
   }
    return;
 
   }
-# line 871 "DefTable.puma"
+/* line 872 "DefTable.puma" */
   {
-# line 872 "DefTable.puma"
+/* line 873 "DefTable.puma" */
    DefTableError ("OpenScope: illegal structure");
   }
    return;
@@ -920,14 +942,14 @@ yyL1:;
 
 void CloseScope
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 877 "DefTable.puma"
+/* line 878 "DefTable.puma" */
   {
-# line 878 "DefTable.puma"
+/* line 879 "DefTable.puma" */
    ScopeList = GetTailScopes (ScopeList);
   }
    return;
@@ -937,104 +959,104 @@ void CloseScope
 
 tDefinitions GetCurrentScope
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 883 "DefTable.puma"
+/* line 884 "DefTable.puma" */
   {
-# line 884 "DefTable.puma"
+/* line 885 "DefTable.puma" */
    if (! ((ScopeList == EmptyScopeList))) goto yyL1;
   }
    return UnitEntries;
 yyL1:;
 
-# line 889 "DefTable.puma"
+/* line 890 "DefTable.puma" */
    return GetFirstScope (ScopeList);
 
 }
 
 tDefinitions GetExternalEntries
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 896 "DefTable.puma"
+/* line 897 "DefTable.puma" */
    return ExternalEntries;
 
 }
 
 tDefinitions GetIntrinsicEntries
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 902 "DefTable.puma"
+/* line 903 "DefTable.puma" */
    return IntrinsicEntries;
 
 }
 
 tDefinitions GetUnitEntries
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 908 "DefTable.puma"
+/* line 909 "DefTable.puma" */
    return UnitEntries;
 
 }
 
 tDefinitions GetCommonEntries
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 914 "DefTable.puma"
+/* line 915 "DefTable.puma" */
    return CommonEntries;
 
 }
 
 tDefinitions GetHPFLibrary
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 920 "DefTable.puma"
+/* line 921 "DefTable.puma" */
    return HPFLibModule;
 
 }
 
 tDefinitions GetHPFLocalLibrary
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 926 "DefTable.puma"
+/* line 927 "DefTable.puma" */
    return HPFLocalLibModule;
 
 }
 
 tDefinitions GetHPFTaskLibrary
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 932 "DefTable.puma"
+/* line 933 "DefTable.puma" */
    return HPFTaskLibModule;
 
 }
@@ -1047,9 +1069,9 @@ void InsertUnitEntry
  register tDefinitions Obj;
 # endif
 {
-# line 944 "DefTable.puma"
+/* line 945 "DefTable.puma" */
   {
-# line 946 "DefTable.puma"
+/* line 947 "DefTable.puma" */
    UnitEntries = mENTRY_LIST (Obj, UnitEntries);
   }
    return;
@@ -1065,9 +1087,9 @@ void InsertExternalEntry
  register tDefinitions Obj;
 # endif
 {
-# line 951 "DefTable.puma"
+/* line 952 "DefTable.puma" */
   {
-# line 953 "DefTable.puma"
+/* line 954 "DefTable.puma" */
    ExternalEntries = mENTRY_LIST (Obj, ExternalEntries);
   }
    return;
@@ -1083,9 +1105,9 @@ void InsertCommonEntry
  register tDefinitions Obj;
 # endif
 {
-# line 958 "DefTable.puma"
+/* line 959 "DefTable.puma" */
   {
-# line 960 "DefTable.puma"
+/* line 961 "DefTable.puma" */
    CommonEntries = mENTRY_LIST (Obj, CommonEntries);
   }
    return;
@@ -1103,9 +1125,9 @@ static void InsertEntryScopes
 # endif
 {
   if (sl->Kind == kSCOPE_LIST) {
-# line 965 "DefTable.puma"
+/* line 966 "DefTable.puma" */
   {
-# line 967 "DefTable.puma"
+/* line 968 "DefTable.puma" */
    sl->SCOPE_LIST.Table = mENTRY_LIST (Obj, sl->SCOPE_LIST.Table);
   }
    return;
@@ -1122,21 +1144,21 @@ void InsertEntry
  register tDefinitions Obj;
 # endif
 {
-# line 972 "DefTable.puma"
+/* line 973 "DefTable.puma" */
   {
-# line 974 "DefTable.puma"
+/* line 975 "DefTable.puma" */
    if (! ((ScopeList == EmptyScopeList))) goto yyL1;
   {
-# line 976 "DefTable.puma"
+/* line 977 "DefTable.puma" */
    InsertUnitEntry (Obj);
   }
   }
    return;
 yyL1:;
 
-# line 979 "DefTable.puma"
+/* line 980 "DefTable.puma" */
   {
-# line 981 "DefTable.puma"
+/* line 982 "DefTable.puma" */
    InsertEntryScopes (Obj, ScopeList);
   }
    return;
@@ -1154,12 +1176,13 @@ static void ReplaceEntry
  register tDefinitions scope;
 # endif
 {
-# line 992 "DefTable.puma"
+ yyRecursion:
+/* line 993 "DefTable.puma" */
   {
-# line 994 "DefTable.puma"
+/* line 995 "DefTable.puma" */
    if (! ((scope == NoEntries))) goto yyL1;
   {
-# line 996 "DefTable.puma"
+/* line 997 "DefTable.puma" */
    DefTableError ("ReplaceEntry: NULL");
   }
   }
@@ -1167,38 +1190,38 @@ static void ReplaceEntry
 yyL1:;
 
   if (scope->Kind == kENTRY_LIST) {
-# line 999 "DefTable.puma"
+/* line 1000 "DefTable.puma" */
   {
-# line 1001 "DefTable.puma"
+/* line 1002 "DefTable.puma" */
    if (! ((scope->ENTRY_LIST.Elem->Object.Ident == Name))) goto yyL2;
   {
-# line 1003 "DefTable.puma"
+/* line 1004 "DefTable.puma" */
    scope->ENTRY_LIST.Elem = New;
   }
   }
    return;
 yyL2:;
 
-# line 1006 "DefTable.puma"
+/* line 1007 "DefTable.puma" */
   {
-# line 1008 "DefTable.puma"
-   ReplaceEntry (Name, New, scope->ENTRY_LIST.Next);
+/* line 1009 "DefTable.puma" */
+   scope = scope->ENTRY_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (scope->Kind == kENTRY_EMPTY) {
-# line 1011 "DefTable.puma"
+/* line 1012 "DefTable.puma" */
   {
-# line 1013 "DefTable.puma"
+/* line 1014 "DefTable.puma" */
    DefTableError ("ReplaceEntry: old entry not found");
   }
    return;
 
   }
-# line 1016 "DefTable.puma"
+/* line 1017 "DefTable.puma" */
   {
-# line 1018 "DefTable.puma"
+/* line 1019 "DefTable.puma" */
    DefTableError ("ReplaceEntry: illegal structure");
   }
    return;
@@ -1215,9 +1238,9 @@ void ChangeEntry
  register tDefinitions Obj;
 # endif
 {
-# line 1023 "DefTable.puma"
+/* line 1024 "DefTable.puma" */
   {
-# line 1025 "DefTable.puma"
+/* line 1026 "DefTable.puma" */
    ReplaceEntry (Name, Obj, GetFirstScope (ScopeList));
   }
    return;
@@ -1234,12 +1257,13 @@ tDefinitions GetDeclEntry
  register tDefinitions Decls;
 # endif
 {
-# line 1038 "DefTable.puma"
+ yyRecursion:
+/* line 1039 "DefTable.puma" */
   {
-# line 1040 "DefTable.puma"
+/* line 1041 "DefTable.puma" */
    if (! ((Decls == NoEntries))) goto yyL1;
   {
-# line 1042 "DefTable.puma"
+/* line 1043 "DefTable.puma" */
    DefTableError ("GetDeclEntry (NULL)");
   }
   }
@@ -1247,27 +1271,28 @@ tDefinitions GetDeclEntry
 yyL1:;
 
   if (Decls->Kind == kENTRY_EMPTY) {
-# line 1046 "DefTable.puma"
+/* line 1047 "DefTable.puma" */
    return NoObject;
 
   }
   if (Decls->Kind == kENTRY_LIST) {
-# line 1051 "DefTable.puma"
+/* line 1052 "DefTable.puma" */
   {
-# line 1053 "DefTable.puma"
+/* line 1054 "DefTable.puma" */
    if (! ((Ident == Decls->ENTRY_LIST.Elem->Object.Ident))) goto yyL3;
   }
    return Decls->ENTRY_LIST.Elem;
 yyL3:;
 
-# line 1058 "DefTable.puma"
-   return GetDeclEntry (Ident, Decls->ENTRY_LIST.Next);
+/* line 1059 "DefTable.puma" */
+   Decls = Decls->ENTRY_LIST.Next;
+   goto yyRecursion;
 
   }
-# line 1063 "DefTable.puma"
+/* line 1064 "DefTable.puma" */
   {
-# line 1065 "DefTable.puma"
- char name[MAXID_LENGTH];
+/* line 1066 "DefTable.puma" */
+ char name[MAX_ID_LENGTH];
      GetString (Ident, name);
      printf ("internal error for definition table when finding %s\n", name);
      printf ("Decls has kind = %d\n", Decls->Kind);
@@ -1287,36 +1312,34 @@ static tDefinitions GetDecl
  register tDefinitions sl;
 # endif
 {
+ yyRecursion:
   if (sl->Kind == kSCOPE_LIST) {
-# line 1083 "DefTable.puma"
+/* line 1086 "DefTable.puma" */
  {
   tDefinitions obj;
   {
-# line 1085 "DefTable.puma"
-
-# line 1087 "DefTable.puma"
+/* line 1090 "DefTable.puma" */
    obj = GetDeclEntry (Ident, sl->SCOPE_LIST.Table);
-# line 1089 "DefTable.puma"
+/* line 1092 "DefTable.puma" */
    if (! ((obj != NoObject))) goto yyL1;
   }
-  {
    return obj;
-  }
  }
 yyL1:;
 
-# line 1094 "DefTable.puma"
-   return GetDecl (Ident, sl->SCOPE_LIST.Hidden);
+/* line 1097 "DefTable.puma" */
+   sl = sl->SCOPE_LIST.Hidden;
+   goto yyRecursion;
 
   }
   if (sl->Kind == kSCOPE_EMPTY) {
-# line 1099 "DefTable.puma"
+/* line 1102 "DefTable.puma" */
    return NoObject;
 
   }
-# line 1104 "DefTable.puma"
+/* line 1107 "DefTable.puma" */
   {
-# line 1106 "DefTable.puma"
+/* line 1109 "DefTable.puma" */
    DefTableError ("GetDecl (illegal scope list)");
   }
    return NoObject;
@@ -1331,7 +1354,7 @@ tDefinitions GetLocalObject
  register tIdent Ident;
 # endif
 {
-# line 1120 "DefTable.puma"
+/* line 1123 "DefTable.puma" */
    return GetDeclEntry (Ident, GetFirstScope (ScopeList));
 
 }
@@ -1344,7 +1367,7 @@ tDefinitions GetGlobalObject
  register tIdent Ident;
 # endif
 {
-# line 1126 "DefTable.puma"
+/* line 1137 "DefTable.puma" */
    return GetDecl (Ident, ScopeList);
 
 }
@@ -1357,13 +1380,11 @@ tDefinitions GetIntrinsicObject
  register tIdent Ident;
 # endif
 {
-# line 1132 "DefTable.puma"
+/* line 1151 "DefTable.puma" */
  {
   tDefinitions Obj;
   {
-# line 1134 "DefTable.puma"
-
-# line 1136 "DefTable.puma"
+/* line 1155 "DefTable.puma" */
  Obj = GetDeclEntry (Ident, IntrinsicEntries);
      if (Obj == NoObject)
         Obj = GetDeclEntry (Ident, HPFLibEntries);
@@ -1371,9 +1392,7 @@ tDefinitions GetIntrinsicObject
         Obj = GetDeclEntry (Ident, HPFLocalLibEntries);
    
   }
-  {
    return Obj;
-  }
  }
 
 }
@@ -1386,40 +1405,36 @@ tDefinitions GetDefaultTopology
  register int rank;
 # endif
 {
-# line 1148 "DefTable.puma"
+/* line 1167 "DefTable.puma" */
   {
-# line 1149 "DefTable.puma"
-   if (! (((rank < - 1) || (rank > MAX_TORUS_RANK)))) goto yyL1;
+/* line 1168 "DefTable.puma" */
+   if (! (((rank < - 1) || (rank > MAX_DIMENSIONS)))) goto yyL1;
   }
    return NoObject;
 yyL1:;
 
-# line 1153 "DefTable.puma"
+/* line 1172 "DefTable.puma" */
    return DefaultTopologies [rank + 1];
 
 }
 
 tDefinitions RemoveHostAssociation
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 1165 "DefTable.puma"
+/* line 1184 "DefTable.puma" */
  {
   tDefinitions sl;
   {
-# line 1166 "DefTable.puma"
-
-# line 1168 "DefTable.puma"
+/* line 1187 "DefTable.puma" */
  sl = ScopeList;
       ScopeList = EmptyScopeList;
     
   }
-  {
    return sl;
-  }
  }
 
 }
@@ -1432,9 +1447,9 @@ void ResetHostAssociation
  register tDefinitions HostAssociation;
 # endif
 {
-# line 1177 "DefTable.puma"
+/* line 1196 "DefTable.puma" */
   {
-# line 1178 "DefTable.puma"
+/* line 1197 "DefTable.puma" */
    ScopeList = HostAssociation;
   }
    return;
@@ -1442,9 +1457,9 @@ void ResetHostAssociation
 ;
 }
 
-void BeginDefTable ()
+void BeginDefTable ARGS ((void))
 {
-# line 753 "DefTable.puma"
+/* line 754 "DefTable.puma" */
 
 
   UnitEntries        = mENTRY_EMPTY();
@@ -1461,7 +1476,7 @@ void BeginDefTable ()
   EmptyScopeList = ScopeList;    /* used for comparisons */
 
   { int i;
-    for (i=-1; i<=MAX_TORUS_RANK; i++)
+    for (i=-1; i<=MAX_DIMENSIONS; i++)
      DefaultTopologies[i+1] =
       mTopologyObject (DefaultId(), NoTree, Public, NoObject, i, NoTree, 0, 0);
   }
@@ -1481,9 +1496,9 @@ void BeginDefTable ()
 
 }
 
-void CloseDefTable ()
+void CloseDefTable ARGS ((void))
 {
-# line 788 "DefTable.puma"
+/* line 789 "DefTable.puma" */
 
 
 if (ScopeList != EmptyScopeList)
