@@ -1,6 +1,13 @@
 # include "Descriptor.h"
-# include "yyDescriptor.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,34 +15,31 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 56 "Descriptor.puma"
+/* line 56 "Descriptor.puma" */
 
 
-# undef DEBUG
+#undef DEBUG
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 # include "protocol.h"
 
 # include "DefTable.h"
@@ -46,6 +50,7 @@
 # include "Transform.h"    /* VarParamToExp, .. */
 # include "Distributions.h"
 # include "TreeOps.h"
+# include "Unparse.h"
 
 # include "HomeDescriptor.h" 
 # include "VarDescriptor.h"
@@ -73,11 +78,25 @@ int *t1, *t2;
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* Descriptor_Exit) () = yyExit;
+# include "yyDescriptor.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* Descriptor_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -86,43 +105,44 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module Descriptor, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module Descriptor, routine %s failed\n",
+  yyFunction);
  Descriptor_Exit ();
 }
 
-bool IsTemplateDescriptor ARGS((pvar vard));
-void MakeTemplateDescriptor ARGS((pvar vard));
-static void ExpandDescriptorDimension ARGS((pvar vard, int dim));
-void ReplicateDescriptorDimension ARGS((pvar vard, int dim));
-void IntersectDescriptor ARGS((pvar vard1, pvar vard2));
-static void IntersectTopDims ARGS((pvar vard1, pvar vard2, int top_dim));
-void AddDescriptorDimension ARGS((pvar vard, tTree actual, tTree formal, int kind));
-static void SetFormalShape ARGS((pvar vard, int dim, tTree formal));
-static void GetFormalSize ARGS((tTree formal, tTree * yyP3, tTree * yyP2, tTree * yyP1));
-static void SetActualShape ARGS((pvar vard, int dim, tTree actual));
-void MakeSliceDescriptor ARGS((tTree exp, pvar vard));
-void MakeShapeDescriptor ARGS((tTree exp, pvar vard));
-static void MakeListShapeDescriptor ARGS((tTree exp, pvar vard));
-void MakeLoopVarDescriptor ARGS((tTree exp, pvar vard));
-static void CorrectTemplateDescriptor ARGS((pvar vard));
-void ProductDescriptors ARGS((pvar d1, pvar d2));
-void CombineDescriptors ARGS((pvar d1, pvar d2));
-void ExpandDescriptor ARGS((pvar d, tTree loopid, bool * yyP4));
-int FindDescriptorDim ARGS((pvar d, tTree loopid));
-static bool ActualIsDependent ARGS((pvar d, int dim, tTree loopid));
-void RedVarDescriptor ARGS((pvar vard, int dim));
-void RemDimDescriptor ARGS((pvar vard, int index_dim));
-void MakeRepArray1Descriptor ARGS((pvar vard, int size));
-static void MakeNewSerialDim ARGS((pvar vard, int dim, tTree exp));
-static void MoveIndexData ARGS((pvar vard, int old_dim, int new_dim));
-static void SwitchIndexData ARGS((pvar vard, int index_dim1, int index_dim2));
-void TransposeDescriptor ARGS((pvar vard));
-static void SwitchDimDescriptor ARGS((pvar vard, int idim1, int idim2));
-void SpreadVarDescriptor ARGS((pvar vard, int dim, tTree ncopies));
-bool SameDescriptorTopology ARGS((pvar vard1, pvar vard2));
-tTree PrintableDescriptorVar ARGS((pvar vard));
+rbool IsTemplateDescriptor ARGS ((pvar vard));
+void MakeTemplateDescriptor ARGS ((pvar vard));
+static void ExpandDescriptorDimension ARGS ((pvar vard, int dim));
+void ReplicateDescriptorDimension ARGS ((pvar vard, int dim));
+void IntersectDescriptor ARGS ((pvar vard1, pvar vard2));
+static void IntersectTopDims ARGS ((pvar vard1, pvar vard2, int top_dim));
+void AddDescriptorDimension ARGS ((pvar vard, tTree actual, tTree formal, int kind));
+static void SetFormalShape ARGS ((pvar vard, int dim, tTree formal));
+static void GetFormalSize ARGS ((tTree formal, tTree * yyP3, tTree * yyP2, tTree * yyP1));
+static void SetActualShape ARGS ((pvar vard, int dim, tTree actual));
+void MakeSliceDescriptor ARGS ((tTree exp, pvar vard));
+void MakeShapeDescriptor ARGS ((tTree exp, pvar vard));
+static void MakeListShapeDescriptor ARGS ((tTree exp, pvar vard));
+void MakeLoopVarDescriptor ARGS ((tTree exp, pvar vard));
+static void CorrectTemplateDescriptor ARGS ((pvar vard));
+void ProductDescriptors ARGS ((pvar d1, pvar d2));
+void CombineDescriptors ARGS ((pvar d1, pvar d2));
+void ExpandDescriptor ARGS ((pvar d, tTree loopid, rbool * yyP4));
+int FindDescriptorDim ARGS ((pvar d, tTree loopid));
+static rbool ActualIsDependent ARGS ((pvar d, int dim, tTree loopid));
+void RedVarDescriptor ARGS ((pvar vard, int dim));
+void RemDimDescriptor ARGS ((pvar vard, int index_dim));
+void MakeRepArray1Descriptor ARGS ((pvar vard, int size));
+static void MakeNewSerialDim ARGS ((pvar vard, int dim, tTree exp));
+static void MoveIndexData ARGS ((pvar vard, int old_dim, int new_dim));
+static void SwitchIndexData ARGS ((pvar vard, int index_dim1, int index_dim2));
+void TransposeDescriptor ARGS ((pvar vard));
+static void SwitchDimDescriptor ARGS ((pvar vard, int idim1, int idim2));
+void SpreadVarDescriptor ARGS ((pvar vard, int dim, tTree ncopies));
+rbool SameDescriptorTopology ARGS ((pvar vard1, pvar vard2));
+tTree PrintableDescriptorVar ARGS ((pvar vard));
 
-bool IsTemplateDescriptor
+rbool IsTemplateDescriptor
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -130,15 +150,15 @@ bool IsTemplateDescriptor
  pvar vard;
 # endif
 {
-# line 102 "Descriptor.puma"
+/* line 103 "Descriptor.puma" */
   {
-# line 103 "Descriptor.puma"
+/* line 104 "Descriptor.puma" */
    if (! ((vard -> type_kind == kDUMMY_TYPE))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
 void MakeTemplateDescriptor
@@ -149,13 +169,13 @@ void MakeTemplateDescriptor
  pvar vard;
 # endif
 {
-# line 122 "Descriptor.puma"
+/* line 123 "Descriptor.puma" */
   {
-# line 124 "Descriptor.puma"
+/* line 125 "Descriptor.puma" */
  int pdim;
-     bool new;
+     rbool new;
 
-     new = false;
+     new = rfalse;
 
      for (pdim = 0; pdim < vard->topology_rank; pdim++)
 
@@ -190,7 +210,7 @@ void MakeTemplateDescriptor
                
 
                vard->on_index_dim [pdim] = rank + 1;
-               new = true;
+               new = rtrue;
 
                vard->formal_rank = rank + 1;
              }
@@ -214,17 +234,17 @@ static void ExpandDescriptorDimension
  register int dim;
 # endif
 {
-# line 184 "Descriptor.puma"
+/* line 185 "Descriptor.puma" */
   {
-# line 186 "Descriptor.puma"
+/* line 187 "Descriptor.puma" */
    if (! ((dim < 1))) goto yyL1;
   }
    return;
 yyL1:;
 
-# line 189 "Descriptor.puma"
+/* line 190 "Descriptor.puma" */
   {
-# line 191 "Descriptor.puma"
+/* line 192 "Descriptor.puma" */
  tTree lb, ub;
      int   pdim;
 
@@ -263,17 +283,17 @@ void ReplicateDescriptorDimension
  register int dim;
 # endif
 {
-# line 229 "Descriptor.puma"
+/* line 230 "Descriptor.puma" */
   {
-# line 231 "Descriptor.puma"
+/* line 232 "Descriptor.puma" */
    if (! ((dim < 1))) goto yyL1;
   }
    return;
 yyL1:;
 
-# line 234 "Descriptor.puma"
+/* line 235 "Descriptor.puma" */
   {
-# line 236 "Descriptor.puma"
+/* line 237 "Descriptor.puma" */
  tTree lb, ub;
      int   pdim;
 
@@ -315,9 +335,9 @@ void IntersectDescriptor
  pvar vard2;
 # endif
 {
-# line 278 "Descriptor.puma"
+/* line 279 "Descriptor.puma" */
   {
-# line 280 "Descriptor.puma"
+/* line 281 "Descriptor.puma" */
 
 #ifdef DEBUG
    printf ("Intersection of two descriptors : ");
@@ -329,38 +349,36 @@ void IntersectDescriptor
    PrintVarDescriptor (vard2);
 #endif 
    
-# line 292 "Descriptor.puma"
+/* line 293 "Descriptor.puma" */
    if (! ((VDIsSubSet (vard1, vard2)))) goto yyL1;
   }
    return;
 yyL1:;
 
-# line 295 "Descriptor.puma"
+/* line 296 "Descriptor.puma" */
   {
-# line 297 "Descriptor.puma"
+/* line 298 "Descriptor.puma" */
    if (! ((VDIsSubSet (vard2, vard1)))) goto yyL2;
   {
-# line 299 "Descriptor.puma"
+/* line 300 "Descriptor.puma" */
  *vard1 = *vard2; 
   }
   }
    return;
 yyL2:;
 
-# line 308 "Descriptor.puma"
+/* line 309 "Descriptor.puma" */
  {
   int top_dim;
   {
-# line 310 "Descriptor.puma"
-
-# line 312 "Descriptor.puma"
+/* line 313 "Descriptor.puma" */
    if (! ((SameDescriptorTopology (vard1, vard2)))) goto yyL3;
   {
-# line 314 "Descriptor.puma"
+/* line 315 "Descriptor.puma" */
    MakeTemplateDescriptor (vard1);
-# line 315 "Descriptor.puma"
+/* line 316 "Descriptor.puma" */
    MakeTemplateDescriptor (vard2);
-# line 317 "Descriptor.puma"
+/* line 318 "Descriptor.puma" */
  for (top_dim = 1; top_dim <= VarRank (vard1->topology_obj); top_dim ++)
 
         IntersectTopDims (vard1, vard2, top_dim);
@@ -381,9 +399,9 @@ yyL2:;
  }
 yyL3:;
 
-# line 333 "Descriptor.puma"
+/* line 334 "Descriptor.puma" */
   {
-# line 335 "Descriptor.puma"
+/* line 336 "Descriptor.puma" */
    MakeNoDescriptor (vard1);
   }
    return;
@@ -401,24 +419,20 @@ static void IntersectTopDims
  register int top_dim;
 # endif
 {
-# line 346 "Descriptor.puma"
+/* line 347 "Descriptor.puma" */
   {
-# line 348 "Descriptor.puma"
+/* line 349 "Descriptor.puma" */
    if (! ((IsNoDescriptor (vard1)))) goto yyL1;
   }
    return;
 yyL1:;
 
-# line 351 "Descriptor.puma"
+/* line 352 "Descriptor.puma" */
  {
   int idim1;
   int idim2;
   {
-# line 353 "Descriptor.puma"
-
-# line 354 "Descriptor.puma"
-
-# line 356 "Descriptor.puma"
+/* line 357 "Descriptor.puma" */
  idim1 = vard1->on_index_dim [top_dim-1] - 1;
      idim2 = vard2->on_index_dim [top_dim-1] - 1;
 
@@ -496,19 +510,13 @@ void AddDescriptorDimension
 # endif
 {
   if (formal->Kind == kSLICE_EXP) {
-# line 426 "Descriptor.puma"
+/* line 427 "Descriptor.puma" */
  {
   int idim;
   int tdim;
   int pdim;
   {
-# line 428 "Descriptor.puma"
-
-# line 429 "Descriptor.puma"
-
-# line 430 "Descriptor.puma"
-
-# line 432 "Descriptor.puma"
+/* line 433 "Descriptor.puma" */
  idim = vard->formal_rank;   
      tdim = vard->template_rank;
      pdim = VarRank (vard->topology_obj);
@@ -533,10 +541,10 @@ void AddDescriptorDimension
        { vard->topology_dim[idim] = pdim + 1;
          vard->topology_rank += 1;
 
-         if (vard->topology_rank > MAX_TORUS_RANK)
+         if (vard->topology_rank > MAX_DIMENSIONS)
 
             { printf ("too many distributed dimensions, max = %d\n",
-                       MAX_TORUS_RANK);
+                       MAX_DIMENSIONS);
               failure_protocol (MODULE, "AddDescriptorDimension", NoTree);
             }
 
@@ -558,9 +566,9 @@ void AddDescriptorDimension
  }
 
   }
-# line 479 "Descriptor.puma"
+/* line 480 "Descriptor.puma" */
   {
-# line 481 "Descriptor.puma"
+/* line 482 "Descriptor.puma" */
    failure_protocol (MODULE, "AddDescriptorDimension", formal);
   }
    return;
@@ -578,15 +586,15 @@ static void SetFormalShape
  register tTree formal;
 # endif
 {
-# line 486 "Descriptor.puma"
+/* line 487 "Descriptor.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 488 "Descriptor.puma"
+/* line 489 "Descriptor.puma" */
    GetFormalSize (formal, & yyV1, & yyV2, & yyV3);
-# line 490 "Descriptor.puma"
+/* line 491 "Descriptor.puma" */
  vard->formal_rank ++;
       vard->formal_shape[dim][0] = yyV1;
       vard->formal_shape[dim][1] = yyV2;
@@ -612,17 +620,17 @@ static void GetFormalSize
 # endif
 {
   if (formal->Kind == kSLICE_EXP) {
-# line 500 "Descriptor.puma"
-   * yyP3 = formal->SLICE_EXP.START;
+/* line 501 "Descriptor.puma" */
+   * yyP3 = formal->SLICE_EXP.FIRST;
    * yyP2 = formal->SLICE_EXP.STOP;
    * yyP1 = formal->SLICE_EXP.INC;
    return;
 
   }
   if (Tree_IsType (formal, kSHAPE_SPEC)) {
-# line 503 "Descriptor.puma"
+/* line 504 "Descriptor.puma" */
   {
-# line 504 "Descriptor.puma"
+/* line 506 "Descriptor.puma" */
    failure_protocol (MODULE, "GetFormalSize", formal);
   }
    * yyP3 = NoTree;
@@ -645,11 +653,11 @@ static void SetActualShape
 # endif
 {
   if (actual->Kind == kSLICE_EXP) {
-# line 509 "Descriptor.puma"
+/* line 511 "Descriptor.puma" */
   {
-# line 511 "Descriptor.puma"
+/* line 513 "Descriptor.puma" */
  vard->actual_rank ++;
-      vard->actual_shape[dim][0] = actual->SLICE_EXP.START;
+      vard->actual_shape[dim][0] = actual->SLICE_EXP.FIRST;
       vard->actual_shape[dim][1] = actual->SLICE_EXP.STOP;
       vard->actual_shape[dim][2] = actual->SLICE_EXP.INC;
     
@@ -657,9 +665,9 @@ static void SetActualShape
    return;
 
   }
-# line 518 "Descriptor.puma"
+/* line 520 "Descriptor.puma" */
   {
-# line 520 "Descriptor.puma"
+/* line 522 "Descriptor.puma" */
  if (TreeRank (actual) == 1)
          vard->actual_rank ++;
       vard->actual_shape[dim][0] = actual;
@@ -682,24 +690,24 @@ void MakeSliceDescriptor
 # endif
 {
   if (exp->Kind == kSLICE_EXP) {
-# line 538 "Descriptor.puma"
+/* line 540 "Descriptor.puma" */
   {
-# line 540 "Descriptor.puma"
+/* line 542 "Descriptor.puma" */
    MakeReplicatedDescriptor (vard);
-# line 542 "Descriptor.puma"
+/* line 544 "Descriptor.puma" */
 
 #ifdef DEBUG
      printf ("MakeSliceDescriptor (before) : \n");
      PrintVarDescriptor (vard);
 #endif
    
-# line 549 "Descriptor.puma"
+/* line 551 "Descriptor.puma" */
  if (ddefault_kind == DDEFAULT_REPLICATED)
        AddDescriptorDimension (vard, exp, exp, kSERIAL_DIM);
       else
        AddDescriptorDimension (vard, exp, exp, kBLOCK_DIM);
    
-# line 555 "Descriptor.puma"
+/* line 557 "Descriptor.puma" */
 
 #ifdef DEBUG
      printf ("MakeSliceDescriptor (after) : \n");
@@ -710,9 +718,9 @@ void MakeSliceDescriptor
    return;
 
   }
-# line 563 "Descriptor.puma"
+/* line 565 "Descriptor.puma" */
   {
-# line 564 "Descriptor.puma"
+/* line 566 "Descriptor.puma" */
    failure_protocol (MODULE, "MakeSliceDescriptor", exp);
   }
    return;
@@ -730,25 +738,23 @@ void MakeShapeDescriptor
 # endif
 {
   if (exp->Kind == kARRAY_EXP) {
-# line 578 "Descriptor.puma"
+/* line 580 "Descriptor.puma" */
   {
-# line 580 "Descriptor.puma"
+/* line 582 "Descriptor.puma" */
    MakeListShapeDescriptor (exp, vard);
   }
    return;
 
   }
-# line 583 "Descriptor.puma"
+/* line 585 "Descriptor.puma" */
  {
   tTree slice;
   {
-# line 585 "Descriptor.puma"
-
-# line 587 "Descriptor.puma"
+/* line 589 "Descriptor.puma" */
    MakeReplicatedDescriptor (vard);
-# line 589 "Descriptor.puma"
+/* line 591 "Descriptor.puma" */
    slice = mSLICE_EXP (MakeConstant (1), exp, mDUMMY_EXP ());
-# line 591 "Descriptor.puma"
+/* line 593 "Descriptor.puma" */
    AddDescriptorDimension (vard, slice, slice, kSERIAL_DIM);
   }
    return;
@@ -766,37 +772,36 @@ static void MakeListShapeDescriptor
  pvar vard;
 # endif
 {
+ yyRecursion:
   if (exp->Kind == kARRAY_EXP) {
-# line 596 "Descriptor.puma"
+/* line 598 "Descriptor.puma" */
   {
-# line 598 "Descriptor.puma"
+/* line 600 "Descriptor.puma" */
    MakeReplicatedDescriptor (vard);
-# line 599 "Descriptor.puma"
-   MakeListShapeDescriptor (exp->ARRAY_EXP.ELEMENTS, vard);
+/* line 601 "Descriptor.puma" */
+   exp = exp->ARRAY_EXP.ELEMENTS;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (exp->Kind == kBTE_LIST) {
-# line 602 "Descriptor.puma"
+/* line 604 "Descriptor.puma" */
  {
   tTree slice;
   {
-# line 604 "Descriptor.puma"
-
-# line 606 "Descriptor.puma"
+/* line 608 "Descriptor.puma" */
    slice = mSLICE_EXP (MakeConstant (1), exp->BTE_LIST.Elem, mDUMMY_EXP ());
-# line 608 "Descriptor.puma"
+/* line 610 "Descriptor.puma" */
    AddDescriptorDimension (vard, slice, slice, kSERIAL_DIM);
-# line 609 "Descriptor.puma"
-   MakeListShapeDescriptor (exp->BTE_LIST.Next, vard);
+/* line 611 "Descriptor.puma" */
+   exp = exp->BTE_LIST.Next;
+   goto yyRecursion;
   }
-   return;
  }
 
   }
   if (exp->Kind == kBTE_EMPTY) {
-# line 612 "Descriptor.puma"
+/* line 614 "Descriptor.puma" */
    return;
 
   }
@@ -814,15 +819,13 @@ void MakeLoopVarDescriptor
 {
   if (exp->Kind == kVAR_EXP) {
   if (exp->VAR_EXP.V->Kind == kLOOP_VAR) {
-# line 625 "Descriptor.puma"
+/* line 627 "Descriptor.puma" */
  {
   int i;
   {
-# line 627 "Descriptor.puma"
+/* line 629 "Descriptor.puma" */
    SetVarDescriptor (exp->VAR_EXP.V, vard);
-# line 629 "Descriptor.puma"
-
-# line 631 "Descriptor.puma"
+/* line 633 "Descriptor.puma" */
  
  
      for (i=1; i<= OuterLoops(); i++)
@@ -852,9 +855,9 @@ void MakeLoopVarDescriptor
 
   }
   }
-# line 656 "Descriptor.puma"
+/* line 658 "Descriptor.puma" */
   {
-# line 658 "Descriptor.puma"
+/* line 660 "Descriptor.puma" */
    failure_protocol (MODULE, "MakeLoopVarDescriptor", exp);
   }
    return;
@@ -870,9 +873,9 @@ static void CorrectTemplateDescriptor
  pvar vard;
 # endif
 {
-# line 669 "Descriptor.puma"
+/* line 671 "Descriptor.puma" */
   {
-# line 671 "Descriptor.puma"
+/* line 673 "Descriptor.puma" */
  int topology_rank;
     int template_rank;
     int idim, kind;
@@ -956,29 +959,29 @@ void ProductDescriptors
  pvar d2;
 # endif
 {
-# line 754 "Descriptor.puma"
+/* line 756 "Descriptor.puma" */
   {
-# line 756 "Descriptor.puma"
+/* line 758 "Descriptor.puma" */
    if (! ((d2 -> formal_rank <= 0))) goto yyL1;
   }
    return;
 yyL1:;
 
-# line 759 "Descriptor.puma"
+/* line 761 "Descriptor.puma" */
   {
-# line 761 "Descriptor.puma"
+/* line 763 "Descriptor.puma" */
    if (! ((d1 -> formal_rank <= 0))) goto yyL2;
   {
-# line 762 "Descriptor.puma"
+/* line 764 "Descriptor.puma" */
  *d1 = *d2; 
   }
   }
    return;
 yyL2:;
 
-# line 765 "Descriptor.puma"
+/* line 767 "Descriptor.puma" */
   {
-# line 767 "Descriptor.puma"
+/* line 769 "Descriptor.puma" */
  int i, rank1, rank2;
 
 #ifdef DEBUG
@@ -1038,27 +1041,27 @@ void CombineDescriptors
  pvar d2;
 # endif
 {
-# line 834 "Descriptor.puma"
+/* line 836 "Descriptor.puma" */
   {
-# line 836 "Descriptor.puma"
+/* line 838 "Descriptor.puma" */
    if (! ((d1 -> actual_rank > 0))) goto yyL1;
   {
-# line 837 "Descriptor.puma"
+/* line 839 "Descriptor.puma" */
    if (! ((d2 -> actual_rank == 0))) goto yyL1;
   }
   }
    return;
 yyL1:;
 
-# line 840 "Descriptor.puma"
+/* line 842 "Descriptor.puma" */
   {
-# line 842 "Descriptor.puma"
+/* line 844 "Descriptor.puma" */
    if (! ((d1 -> actual_rank == 0))) goto yyL2;
   {
-# line 843 "Descriptor.puma"
+/* line 845 "Descriptor.puma" */
    if (! ((d2 -> actual_rank > 0))) goto yyL2;
   {
-# line 845 "Descriptor.puma"
+/* line 847 "Descriptor.puma" */
  *d1 = *d2; 
   }
   }
@@ -1066,40 +1069,40 @@ yyL1:;
    return;
 yyL2:;
 
-# line 848 "Descriptor.puma"
+/* line 850 "Descriptor.puma" */
   {
-# line 850 "Descriptor.puma"
+/* line 852 "Descriptor.puma" */
    if (! ((d2 -> var_tree == NoTree))) goto yyL3;
   }
    return;
 yyL3:;
 
-# line 853 "Descriptor.puma"
+/* line 855 "Descriptor.puma" */
   {
-# line 855 "Descriptor.puma"
+/* line 857 "Descriptor.puma" */
    if (! ((d1 -> var_tree == NoTree))) goto yyL4;
   {
-# line 857 "Descriptor.puma"
+/* line 859 "Descriptor.puma" */
  *d1 = *d2; 
   }
   }
    return;
 yyL4:;
 
-# line 860 "Descriptor.puma"
+/* line 862 "Descriptor.puma" */
   {
-# line 862 "Descriptor.puma"
+/* line 864 "Descriptor.puma" */
    if (! ((d2 -> topology_rank == 0))) goto yyL5;
   }
    return;
 yyL5:;
 
-# line 865 "Descriptor.puma"
+/* line 867 "Descriptor.puma" */
   {
-# line 867 "Descriptor.puma"
+/* line 869 "Descriptor.puma" */
    if (! ((d1 -> topology_rank == 0))) goto yyL6;
   {
-# line 869 "Descriptor.puma"
+/* line 871 "Descriptor.puma" */
  *d1 = *d2; 
   }
   }
@@ -1111,51 +1114,49 @@ yyL6:;
 
 void ExpandDescriptor
 # if defined __STDC__ | defined __cplusplus
-(pvar d, register tTree loopid, register bool * yyP4)
+(pvar d, register tTree loopid, register rbool * yyP4)
 # else
 (d, loopid, yyP4)
  pvar d;
  register tTree loopid;
- register bool * yyP4;
+ register rbool * yyP4;
 # endif
 {
-# line 889 "Descriptor.puma"
+/* line 891 "Descriptor.puma" */
   {
-# line 891 "Descriptor.puma"
+/* line 893 "Descriptor.puma" */
    if (! ((IsNoDescriptor (d)))) goto yyL1;
   }
-   * yyP4 = true;
+   * yyP4 = rtrue;
    return;
 yyL1:;
 
-# line 894 "Descriptor.puma"
+/* line 896 "Descriptor.puma" */
   {
-# line 896 "Descriptor.puma"
+/* line 898 "Descriptor.puma" */
    if (! ((IsReplicatedDescriptor (d)))) goto yyL2;
   }
-   * yyP4 = true;
+   * yyP4 = rtrue;
    return;
 yyL2:;
 
-# line 899 "Descriptor.puma"
+/* line 901 "Descriptor.puma" */
   {
-# line 903 "Descriptor.puma"
+/* line 905 "Descriptor.puma" */
    if (! ((IsHostDescriptor (d)))) goto yyL3;
   }
-   * yyP4 = true;
+   * yyP4 = rtrue;
    return;
 yyL3:;
 
-# line 906 "Descriptor.puma"
+/* line 908 "Descriptor.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 908 "Descriptor.puma"
-
-# line 910 "Descriptor.puma"
+/* line 912 "Descriptor.puma" */
  int i, rank;
 
-     okay = true;
+     okay = rtrue;
 
 #ifdef DEBUG
      printf ("expand the descriptor for ");
@@ -1172,7 +1173,7 @@ yyL3:;
          { ExpandDescriptorDimension (d, i);
 
            if (ActualIsDependent (d, i, loopid))    
-              okay = false;
+              okay = rfalse;
 
          } 
 
@@ -1199,13 +1200,11 @@ int FindDescriptorDim
  register tTree loopid;
 # endif
 {
-# line 943 "Descriptor.puma"
+/* line 945 "Descriptor.puma" */
  {
   int dim;
   {
-# line 945 "Descriptor.puma"
-
-# line 947 "Descriptor.puma"
+/* line 949 "Descriptor.puma" */
  int idim, rank;
 
      dim = 0;                     
@@ -1220,14 +1219,12 @@ int FindDescriptorDim
 
    
   }
-  {
    return dim;
-  }
  }
 
 }
 
-static bool ActualIsDependent
+static rbool ActualIsDependent
 # if defined __STDC__ | defined __cplusplus
 (pvar d, register int dim, register tTree loopid)
 # else
@@ -1237,13 +1234,33 @@ static bool ActualIsDependent
  register tTree loopid;
 # endif
 {
-# line 966 "Descriptor.puma"
- {
-  bool is;
+/* line 968 "Descriptor.puma" */
   {
-# line 968 "Descriptor.puma"
+/* line 970 "Descriptor.puma" */
+   if (! ((d -> actual_shape [dim - 1] [0] == NoTree))) goto yyL1;
+  {
+/* line 972 "Descriptor.puma" */
+   return rfalse;
+  }
+  }
+yyL1:;
 
-# line 970 "Descriptor.puma"
+/* line 975 "Descriptor.puma" */
+  {
+/* line 977 "Descriptor.puma" */
+   if (! ((IsSelector (d->actual_shape[dim-1][0])))) goto yyL2;
+  {
+/* line 979 "Descriptor.puma" */
+   return rfalse;
+  }
+  }
+yyL2:;
+
+/* line 982 "Descriptor.puma" */
+ {
+  rbool is;
+  {
+/* line 986 "Descriptor.puma" */
  tTree lb, ub, inc;
      tIdent name;
 
@@ -1260,14 +1277,14 @@ static bool ActualIsDependent
         is = is || IsVarInExp (name, inc);
  
    
-# line 987 "Descriptor.puma"
-   if (! ((is))) goto yyL1;
+/* line 1003 "Descriptor.puma" */
+   if (! ((is))) goto yyL3;
   }
-   return true;
+   return rtrue;
  }
-yyL1:;
+yyL3:;
 
-  return false;
+  return rfalse;
 }
 
 void RedVarDescriptor
@@ -1279,13 +1296,11 @@ void RedVarDescriptor
  register int dim;
 # endif
 {
-# line 1001 "Descriptor.puma"
+/* line 1017 "Descriptor.puma" */
  {
   int index_dim;
   {
-# line 1003 "Descriptor.puma"
-
-# line 1005 "Descriptor.puma"
+/* line 1021 "Descriptor.puma" */
  
 
      index_dim = GetFormalDim (vard, dim);
@@ -1315,7 +1330,7 @@ void RemDimDescriptor
  register int index_dim;
 # endif
 {
-# line 1022 "Descriptor.puma"
+/* line 1038 "Descriptor.puma" */
  {
   int i;
   int rank;
@@ -1323,17 +1338,7 @@ void RemDimDescriptor
   tTree index_ub;
   int act_rank;
   {
-# line 1024 "Descriptor.puma"
-
-# line 1025 "Descriptor.puma"
-
-# line 1026 "Descriptor.puma"
-
-# line 1027 "Descriptor.puma"
-
-# line 1028 "Descriptor.puma"
-
-# line 1030 "Descriptor.puma"
+/* line 1046 "Descriptor.puma" */
  
  
 #ifdef DEBUG
@@ -1395,11 +1400,11 @@ void MakeRepArray1Descriptor
  register int size;
 # endif
 {
-# line 1088 "Descriptor.puma"
+/* line 1104 "Descriptor.puma" */
   {
-# line 1090 "Descriptor.puma"
+/* line 1106 "Descriptor.puma" */
    MakeReplicatedDescriptor (vard);
-# line 1091 "Descriptor.puma"
+/* line 1107 "Descriptor.puma" */
    MakeNewSerialDim (vard, 1, MakeConstant (size));
   }
    return;
@@ -1417,9 +1422,9 @@ static void MakeNewSerialDim
  register tTree exp;
 # endif
 {
-# line 1096 "Descriptor.puma"
+/* line 1112 "Descriptor.puma" */
   {
-# line 1098 "Descriptor.puma"
+/* line 1114 "Descriptor.puma" */
   int i, rank;
 
       
@@ -1446,11 +1451,13 @@ static void MakeNewSerialDim
       vard->alloc_shape[dim-1][0] = vard->formal_shape[dim-1][0];
       vard->alloc_shape[dim-1][1] = vard->formal_shape[dim-1][1];
 
-      vard->overlap[dim-1][0] = 0;
-      vard->overlap[dim-1][1] = 0;
+      vard->shadow[dim-1][0] = 0;
+      vard->shadow[dim-1][1] = 0;
  
       vard->distribution_kind[dim-1] = kSERIAL_DIM;
       vard->distribution_size[dim-1] = 0;
+      vard->align_mult[dim-1] = 1;
+      vard->align_add[dim-1] = 1;
       vard->template_dim[dim-1] = 0;
 
       vard->formal_rank += 1;
@@ -1488,16 +1495,12 @@ static void MoveIndexData
  register int new_dim;
 # endif
 {
-# line 1157 "Descriptor.puma"
+/* line 1175 "Descriptor.puma" */
  {
   int new;
   int old;
   {
-# line 1159 "Descriptor.puma"
-
-# line 1160 "Descriptor.puma"
-
-# line 1162 "Descriptor.puma"
+/* line 1180 "Descriptor.puma" */
  old = old_dim - 1; new = new_dim - 1;
 
     vard->formal_shape[new][0] = vard->formal_shape[old][0];
@@ -1510,8 +1513,8 @@ static void MoveIndexData
     vard->alloc_shape [new][0] = vard->alloc_shape [old][0];
     vard->alloc_shape [new][1] = vard->alloc_shape [old][1];
 
-    vard->overlap     [new][0] = vard->overlap     [old][0];
-    vard->overlap     [new][1] = vard->overlap     [old][1];
+    vard->shadow     [new][0] = vard->shadow     [old][0];
+    vard->shadow     [new][1] = vard->shadow     [old][1];
 
     vard->distribution_kind[new] = vard->distribution_kind[old];
     vard->distribution_size[new] = vard->distribution_size[old];
@@ -1539,9 +1542,9 @@ static void SwitchIndexData
  register int index_dim2;
 # endif
 {
-# line 1190 "Descriptor.puma"
+/* line 1208 "Descriptor.puma" */
   {
-# line 1192 "Descriptor.puma"
+/* line 1210 "Descriptor.puma" */
  int dim1, dim2;
 
     dim1 = index_dim1 - 1; dim2 = index_dim2 - 1;
@@ -1556,8 +1559,8 @@ static void SwitchIndexData
     switch_tree (vard->alloc_shape[dim1]+1, vard->alloc_shape[dim2]+1);
     switch_tree (vard->alloc_shape[dim1]+2, vard->alloc_shape[dim2]+2);
 
-    switch_int (vard->overlap[dim1]+0, vard->overlap[dim2]+0);
-    switch_int (vard->overlap[dim1]+1, vard->overlap[dim2]+1);
+    switch_int (vard->shadow[dim1]+0, vard->shadow[dim2]+0);
+    switch_int (vard->shadow[dim1]+1, vard->shadow[dim2]+1);
 
     switch_int (vard->distribution_kind+dim1, vard->distribution_kind+dim2);
     switch_int (vard->distribution_size+dim1, vard->distribution_size+dim2);
@@ -1581,20 +1584,16 @@ void TransposeDescriptor
  pvar vard;
 # endif
 {
-# line 1229 "Descriptor.puma"
+/* line 1247 "Descriptor.puma" */
  {
   int index_dim1;
   int index_dim2;
   {
-# line 1231 "Descriptor.puma"
-
-# line 1232 "Descriptor.puma"
-
-# line 1234 "Descriptor.puma"
+/* line 1252 "Descriptor.puma" */
  if (vard->actual_rank != 2)
         failure_protocol (MODULE, "TransposeDescriptor", vard->var_tree);
    
-# line 1239 "Descriptor.puma"
+/* line 1257 "Descriptor.puma" */
  index_dim1 = GetFormalDim (vard, 1);
      index_dim2 = GetFormalDim (vard, 2);
      SwitchDimDescriptor (vard, index_dim1, index_dim2);
@@ -1616,9 +1615,9 @@ static void SwitchDimDescriptor
  register int idim2;
 # endif
 {
-# line 1247 "Descriptor.puma"
+/* line 1265 "Descriptor.puma" */
   {
-# line 1249 "Descriptor.puma"
+/* line 1267 "Descriptor.puma" */
  int i, rank;
 
      SwitchIndexData (vard, idim1, idim2);
@@ -1656,19 +1655,13 @@ void SpreadVarDescriptor
  register tTree ncopies;
 # endif
 {
-# line 1286 "Descriptor.puma"
+/* line 1304 "Descriptor.puma" */
  {
   int i;
   int rank;
   int index_dim;
   {
-# line 1288 "Descriptor.puma"
-
-# line 1289 "Descriptor.puma"
-
-# line 1290 "Descriptor.puma"
-
-# line 1292 "Descriptor.puma"
+/* line 1310 "Descriptor.puma" */
  
 
      rank  = vard->actual_rank;
@@ -1692,7 +1685,7 @@ void SpreadVarDescriptor
 ;
 }
 
-bool SameDescriptorTopology
+rbool SameDescriptorTopology
 # if defined __STDC__ | defined __cplusplus
 (pvar vard1, pvar vard2)
 # else
@@ -1701,34 +1694,34 @@ bool SameDescriptorTopology
  pvar vard2;
 # endif
 {
-# line 1324 "Descriptor.puma"
+/* line 1342 "Descriptor.puma" */
   {
-# line 1326 "Descriptor.puma"
+/* line 1344 "Descriptor.puma" */
    if (! ((IsNoDescriptor (vard1) || IsNoDescriptor (vard2)))) goto yyL1;
   {
-# line 1327 "Descriptor.puma"
-   return false;
+/* line 1345 "Descriptor.puma" */
+   return rfalse;
   }
   }
 yyL1:;
 
-# line 1330 "Descriptor.puma"
+/* line 1348 "Descriptor.puma" */
   {
-# line 1332 "Descriptor.puma"
+/* line 1350 "Descriptor.puma" */
    if (! ((SameTopologyObject (vard1 -> topology_obj, vard2 -> topology_obj)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 1335 "Descriptor.puma"
+/* line 1353 "Descriptor.puma" */
   {
-# line 1337 "Descriptor.puma"
+/* line 1355 "Descriptor.puma" */
    if (! ((vard1 -> template_obj != NoObject))) goto yyL3;
   {
-# line 1338 "Descriptor.puma"
+/* line 1356 "Descriptor.puma" */
    if (! ((vard1 -> template_obj == vard2 -> template_obj))) goto yyL3;
   {
-# line 1340 "Descriptor.puma"
+/* line 1358 "Descriptor.puma" */
 
 #ifdef DEBUG
       printf ("descriptors use same template (but not same topology)\n");
@@ -1739,10 +1732,10 @@ yyL2:;
   }
   }
   }
-   return true;
+   return rtrue;
 yyL3:;
 
-  return false;
+  return rfalse;
 }
 
 tTree PrintableDescriptorVar
@@ -1753,43 +1746,43 @@ tTree PrintableDescriptorVar
  pvar vard;
 # endif
 {
-# line 1357 "Descriptor.puma"
+/* line 1375 "Descriptor.puma" */
   {
-# line 1359 "Descriptor.puma"
+/* line 1377 "Descriptor.puma" */
    if (! ((IsNoDescriptor (vard)))) goto yyL1;
   }
    return mUSED_VAR (mVAR_OBJ (0, MakeIdent ("$FREE", 5)));
 yyL1:;
 
-# line 1364 "Descriptor.puma"
+/* line 1382 "Descriptor.puma" */
   {
-# line 1366 "Descriptor.puma"
+/* line 1384 "Descriptor.puma" */
    if (! ((IsReplicatedDescriptor (vard)))) goto yyL2;
   {
-# line 1367 "Descriptor.puma"
+/* line 1385 "Descriptor.puma" */
    if (! ((vard -> formal_rank == 0))) goto yyL2;
   }
   }
    return mUSED_VAR (mVAR_OBJ (0, MakeIdent ("$ALL", 4)));
 yyL2:;
 
-# line 1372 "Descriptor.puma"
+/* line 1390 "Descriptor.puma" */
   {
-# line 1374 "Descriptor.puma"
+/* line 1392 "Descriptor.puma" */
    if (! ((IsHostDescriptor (vard)))) goto yyL3;
   }
    return mUSED_VAR (mVAR_OBJ (0, MakeIdent ("$HOST", 5)));
 yyL3:;
 
-# line 1379 "Descriptor.puma"
+/* line 1397 "Descriptor.puma" */
    return MakeDescriptorVar (vard);
 
 }
 
-void BeginDescriptor ()
+void BeginDescriptor ARGS ((void))
 {
 }
 
-void CloseDescriptor ()
+void CloseDescriptor ARGS ((void))
 {
 }
