@@ -1,6 +1,13 @@
 # include "FindHome.h"
-# include "yyFindHome.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,44 +15,42 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 42 "FindHome.puma"
+/* line 42 "FindHome.puma" */
 
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
 # include "ParNest.h"        /* IsNewVariable   */
 # include "Descriptor.h"     
-# include "Objects.h"
+# include "Objects.h"         /* IsGlobalVar    */
 # include "HomeDescriptor.h"
 # include "MoveControl.h"
 # include "Accepted.h"
 # include "Rank.h"
 # include "Distributions.h"
 # include "ArrayData.h"
+# include "Nesting.h"
 
 # define MODULE "FindHome"
 
@@ -53,11 +58,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* FindHome_Exit) () = yyExit;
+# include "yyFindHome.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* FindHome_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -66,25 +85,26 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module FindHome, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module FindHome, routine %s failed\n",
+  yyFunction);
  FindHome_Exit ();
 }
 
-void GetHomeDescriptor ARGS((tTree t, pvar good_home));
-void FindHome ARGS((tTree t, pvar min_home, pvar good_home));
-static void FindParamHome ARGS((tTree t, pvar min_home, pvar good_home));
-static void FindParamListHome ARGS((tTree t, pvar min_home, pvar good_home));
-static void FindExpHome ARGS((tTree exp, pvar min_home, pvar good_home));
-static void FindVarHome ARGS((tTree var, pvar min_home, pvar good_home));
-static void FindReadVarHome ARGS((tTree var, pvar min_home, pvar good_home));
-static void FindWriteVarHome ARGS((tTree var, pvar min_home, pvar good_home));
-static void FindDescriptorHome ARGS((tTree t, pvar min_home, pvar good_home));
-static void ChooseHome ARGS((pvar home1, pvar home2));
-static void ChooseBestHome ARGS((pvar home1, pvar home2));
-static void UnionHome ARGS((pvar home1, pvar home2));
-static void ExpandHomes ARGS((pvar min_home, pvar good_home, tTree id));
-static void PrintHomes ARGS((tTree t, pvar min_home, pvar good_home));
-void FindSerialHome ARGS((tTree stmt, pvar home, bool * yyP1));
+void GetHomeDescriptor ARGS ((tTree t, pvar good_home));
+void FindHome ARGS ((tTree t, pvar min_home, pvar good_home));
+static void FindParamHome ARGS ((tTree t, pvar min_home, pvar good_home));
+static void FindParamListHome ARGS ((tTree t, pvar min_home, pvar good_home));
+static void FindExpHome ARGS ((tTree exp, pvar min_home, pvar good_home));
+static void FindVarHome ARGS ((tTree var, pvar min_home, pvar good_home));
+static void FindReadVarHome ARGS ((tTree var, pvar min_home, pvar good_home));
+static void FindWriteVarHome ARGS ((tTree var, pvar min_home, pvar good_home));
+static void FindDescriptorHome ARGS ((tTree t, pvar min_home, pvar good_home));
+static void ChooseHome ARGS ((pvar home1, pvar home2));
+static void ChooseBestHome ARGS ((pvar home1, pvar home2));
+static void UnionHome ARGS ((pvar home1, pvar home2));
+static void ExpandHomes ARGS ((pvar min_home, pvar good_home, tTree id));
+static void PrintHomes ARGS ((tTree t, pvar min_home, pvar good_home));
+void FindSerialHome ARGS ((tTree stmt, pvar home, rbool * yyP1));
 
 void GetHomeDescriptor
 # if defined __STDC__ | defined __cplusplus
@@ -95,15 +115,13 @@ void GetHomeDescriptor
  pvar good_home;
 # endif
 {
-# line 78 "FindHome.puma"
+/* line 79 "FindHome.puma" */
  {
   var_descriptor min_home;
   {
-# line 80 "FindHome.puma"
-
-# line 82 "FindHome.puma"
+/* line 83 "FindHome.puma" */
    FindHome (t, & min_home, good_home);
-# line 84 "FindHome.puma"
+/* line 85 "FindHome.puma" */
 
 #ifdef DEBUG
      printf ("GetHomeDescriptor of this tree is : \n");
@@ -128,233 +146,227 @@ void FindHome
  pvar good_home;
 # endif
 {
+ yyRecursion:
 
   switch (t->Kind) {
   case kACF_LIST:
-# line 108 "FindHome.puma"
+/* line 109 "FindHome.puma" */
  {
   var_descriptor min_home1;
   var_descriptor good_home1;
   {
-# line 110 "FindHome.puma"
-
-# line 111 "FindHome.puma"
-
-# line 113 "FindHome.puma"
+/* line 114 "FindHome.puma" */
    set_protocol_stmt (t->ACF_LIST.Elem);
-# line 115 "FindHome.puma"
+/* line 116 "FindHome.puma" */
    FindHome (t->ACF_LIST.Elem, min_home, good_home);
-# line 116 "FindHome.puma"
+/* line 117 "FindHome.puma" */
    FindHome (t->ACF_LIST.Next, & min_home1, & good_home1);
-# line 118 "FindHome.puma"
+/* line 119 "FindHome.puma" */
    UnionHome (min_home, & min_home1);
-# line 119 "FindHome.puma"
+/* line 120 "FindHome.puma" */
    ChooseHome (good_home, & good_home1);
   }
    return;
  }
 
   case kACF_EMPTY:
-# line 122 "FindHome.puma"
+/* line 123 "FindHome.puma" */
   {
-# line 124 "FindHome.puma"
+/* line 125 "FindHome.puma" */
    MakeNoDescriptor (min_home);
-# line 125 "FindHome.puma"
+/* line 126 "FindHome.puma" */
    MakeNoDescriptor (good_home);
   }
    return;
 
   case kACF_FORALL:
-# line 128 "FindHome.puma"
+/* line 129 "FindHome.puma" */
   {
-# line 130 "FindHome.puma"
+/* line 131 "FindHome.puma" */
  t->Kind = kACF_DO; 
-# line 132 "FindHome.puma"
-   FindHome (t, min_home, good_home);
+/* line 133 "FindHome.puma" */
+   goto yyRecursion;
   }
-   return;
 
   case kACF_DO:
-# line 135 "FindHome.puma"
+/* line 136 "FindHome.puma" */
   {
-# line 137 "FindHome.puma"
+/* line 138 "FindHome.puma" */
    IncParNesting (t);
-# line 139 "FindHome.puma"
+/* line 140 "FindHome.puma" */
    FindHome (t->ACF_DO.DO_BODY, min_home, good_home);
-# line 141 "FindHome.puma"
+/* line 142 "FindHome.puma" */
    set_protocol_stmt (t);
-# line 143 "FindHome.puma"
+/* line 144 "FindHome.puma" */
    ExpandHomes (min_home, good_home, t->ACF_DO.DO_ID);
-# line 145 "FindHome.puma"
+/* line 146 "FindHome.puma" */
    DecParNesting (t);
   }
    return;
 
   case kACF_IF:
-# line 148 "FindHome.puma"
+/* line 149 "FindHome.puma" */
  {
   var_descriptor min_home1;
   var_descriptor good_home1;
   {
-# line 150 "FindHome.puma"
-
-# line 151 "FindHome.puma"
-
-# line 153 "FindHome.puma"
+/* line 154 "FindHome.puma" */
    IncParNesting (t);
-# line 155 "FindHome.puma"
+/* line 156 "FindHome.puma" */
    FindHome (t->ACF_IF.THEN_PART, min_home, good_home);
-# line 156 "FindHome.puma"
+/* line 157 "FindHome.puma" */
    FindHome (t->ACF_IF.ELSE_PART, & min_home1, & good_home1);
-# line 158 "FindHome.puma"
+/* line 159 "FindHome.puma" */
    DecParNesting (t);
-# line 160 "FindHome.puma"
+/* line 161 "FindHome.puma" */
    UnionHome (min_home, & min_home1);
-# line 161 "FindHome.puma"
+/* line 162 "FindHome.puma" */
    ChooseHome (good_home, & good_home1);
-# line 163 "FindHome.puma"
+/* line 164 "FindHome.puma" */
    FindExpHome (t->ACF_IF.IF_EXP, & min_home1, & good_home1);
-# line 165 "FindHome.puma"
+/* line 166 "FindHome.puma" */
    UnionHome (min_home, & min_home1);
-# line 166 "FindHome.puma"
+/* line 167 "FindHome.puma" */
    ChooseHome (good_home, & good_home1);
   }
    return;
  }
 
   case kACF_WHERE:
-# line 169 "FindHome.puma"
+/* line 170 "FindHome.puma" */
  {
   var_descriptor min_home1;
   var_descriptor good_home1;
   {
-# line 171 "FindHome.puma"
-
-# line 172 "FindHome.puma"
-
-# line 174 "FindHome.puma"
+/* line 175 "FindHome.puma" */
    FindHome (t->ACF_WHERE.TRUE_PART, min_home, good_home);
-# line 175 "FindHome.puma"
+/* line 176 "FindHome.puma" */
    FindHome (t->ACF_WHERE.FALSE_PART, & min_home1, & good_home1);
-# line 177 "FindHome.puma"
+/* line 178 "FindHome.puma" */
    UnionHome (min_home, & min_home1);
-# line 178 "FindHome.puma"
+/* line 179 "FindHome.puma" */
    ChooseHome (good_home, & good_home1);
-# line 180 "FindHome.puma"
+/* line 181 "FindHome.puma" */
    FindExpHome (t->ACF_WHERE.WHERE_EXP, & min_home1, & good_home1);
-# line 182 "FindHome.puma"
+/* line 183 "FindHome.puma" */
    UnionHome (min_home, & min_home1);
-# line 183 "FindHome.puma"
+/* line 184 "FindHome.puma" */
    ChooseHome (good_home, & good_home1);
   }
    return;
  }
 
   case kACF_WHILE:
-# line 186 "FindHome.puma"
+/* line 187 "FindHome.puma" */
  {
   var_descriptor min_home1;
   var_descriptor good_home1;
   {
-# line 188 "FindHome.puma"
-
-# line 189 "FindHome.puma"
-
-# line 191 "FindHome.puma"
+/* line 192 "FindHome.puma" */
    FindHome (t->ACF_WHILE.WHILE_BODY, min_home, good_home);
-# line 192 "FindHome.puma"
+/* line 193 "FindHome.puma" */
    FindExpHome (t->ACF_WHILE.WHILE_EXP, & min_home1, & good_home1);
-# line 194 "FindHome.puma"
+/* line 195 "FindHome.puma" */
    UnionHome (min_home, & min_home1);
-# line 195 "FindHome.puma"
+/* line 196 "FindHome.puma" */
    ChooseHome (good_home, & good_home1);
   }
    return;
  }
 
   case kACF_NEW:
-# line 198 "FindHome.puma"
+/* line 199 "FindHome.puma" */
   {
-# line 200 "FindHome.puma"
+/* line 201 "FindHome.puma" */
    IncParNesting (t);
-# line 201 "FindHome.puma"
+/* line 202 "FindHome.puma" */
    FindHome (t->ACF_NEW.NEW_BODY, min_home, good_home);
-# line 202 "FindHome.puma"
+/* line 203 "FindHome.puma" */
    DecParNesting (t);
   }
    return;
 
   case kACF_RESIDENT:
-# line 205 "FindHome.puma"
+/* line 206 "FindHome.puma" */
   {
-# line 207 "FindHome.puma"
+/* line 208 "FindHome.puma" */
    IncParNesting (t);
-# line 208 "FindHome.puma"
+/* line 209 "FindHome.puma" */
    FindHome (t->ACF_RESIDENT.RESIDENT_BODY, min_home, good_home);
-# line 209 "FindHome.puma"
+/* line 210 "FindHome.puma" */
    DecParNesting (t);
   }
    return;
 
   case kACF_TASK_REGION:
-# line 212 "FindHome.puma"
+/* line 213 "FindHome.puma" */
   {
-# line 214 "FindHome.puma"
+/* line 215 "FindHome.puma" */
    IncParNesting (t);
-# line 215 "FindHome.puma"
+/* line 216 "FindHome.puma" */
    FindHome (t->ACF_TASK_REGION.TASK_BODY, min_home, good_home);
-# line 216 "FindHome.puma"
+/* line 217 "FindHome.puma" */
    DecParNesting (t);
   }
    return;
 
   case kACF_REDUCTION:
-# line 219 "FindHome.puma"
+/* line 220 "FindHome.puma" */
   {
-# line 221 "FindHome.puma"
+/* line 222 "FindHome.puma" */
    IncParNesting (t);
-# line 222 "FindHome.puma"
+/* line 223 "FindHome.puma" */
    FindHome (t->ACF_REDUCTION.REDUCTION_BODY, min_home, good_home);
-# line 223 "FindHome.puma"
+/* line 224 "FindHome.puma" */
    DecParNesting (t);
   }
    return;
 
   case kACF_HOME:
   if (t->ACF_HOME.HOME_VAR->Kind == kON_VAR_CLAUSE) {
-# line 232 "FindHome.puma"
+/* line 233 "FindHome.puma" */
   {
-# line 234 "FindHome.puma"
+/* line 235 "FindHome.puma" */
    IncParNesting (t);
-# line 236 "FindHome.puma"
+/* line 237 "FindHome.puma" */
    FindHome (t->ACF_HOME.HOME_BODY, min_home, good_home);
-# line 238 "FindHome.puma"
+/* line 239 "FindHome.puma" */
    DecParNesting (t);
-# line 242 "FindHome.puma"
+/* line 243 "FindHome.puma" */
    SetVarDescriptor (t->ACF_HOME.HOME_VAR->ON_VAR_CLAUSE.ON_VAR, good_home);
   }
    return;
 
   }
   break;
-  case kACF_BASIC:
-# line 245 "FindHome.puma"
+  case kACF_DUMMY:
+/* line 250 "FindHome.puma" */
   {
-# line 247 "FindHome.puma"
-   SetActualData (t);
-# line 249 "FindHome.puma"
-   FindHome (t->ACF_BASIC.BASIC_STMT, min_home, good_home);
+/* line 252 "FindHome.puma" */
+   MakeNoDescriptor (min_home);
+/* line 253 "FindHome.puma" */
+   MakeNoDescriptor (good_home);
   }
    return;
 
+  case kACF_BASIC:
+/* line 256 "FindHome.puma" */
+  {
+/* line 258 "FindHome.puma" */
+   SetActualData (t);
+/* line 260 "FindHome.puma" */
+   t = t->ACF_BASIC.BASIC_STMT;
+   goto yyRecursion;
+  }
+
   case kREDUCE_STMT:
   if (t->REDUCE_STMT.RED_PARAMS->Kind == kBTP_LIST) {
   if (t->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->Kind == kBTP_LIST) {
-# line 258 "FindHome.puma"
+/* line 269 "FindHome.puma" */
   {
-# line 260 "FindHome.puma"
+/* line 271 "FindHome.puma" */
    FindParamHome (t->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->BTP_LIST.Elem, min_home, good_home);
-# line 265 "FindHome.puma"
+/* line 276 "FindHome.puma" */
  if (IsNoDescriptor (good_home))
       MakeParLoopNestDescriptor (good_home);
 
@@ -370,24 +382,20 @@ void FindHome
   }
   break;
   case kASSIGN_STMT:
-# line 277 "FindHome.puma"
+/* line 288 "FindHome.puma" */
  {
   var_descriptor min_home1;
   var_descriptor good_home1;
   {
-# line 279 "FindHome.puma"
-
-# line 280 "FindHome.puma"
-
-# line 284 "FindHome.puma"
+/* line 295 "FindHome.puma" */
    FindWriteVarHome (t->ASSIGN_STMT.ASSIGN_VAR, min_home, good_home);
-# line 285 "FindHome.puma"
+/* line 296 "FindHome.puma" */
    FindExpHome (t->ASSIGN_STMT.ASSIGN_EXP, & min_home1, & good_home1);
-# line 287 "FindHome.puma"
+/* line 298 "FindHome.puma" */
    UnionHome (min_home, & min_home1);
-# line 288 "FindHome.puma"
+/* line 299 "FindHome.puma" */
    ChooseHome (good_home, & good_home1);
-# line 290 "FindHome.puma"
+/* line 301 "FindHome.puma" */
 
 #ifdef DEBUG
     PrintHomes (t, min_home, good_home);
@@ -398,24 +406,24 @@ void FindHome
  }
 
   case kIO_STMT:
-# line 299 "FindHome.puma"
+/* line 310 "FindHome.puma" */
   {
-# line 303 "FindHome.puma"
+/* line 314 "FindHome.puma" */
    MakeHostDescriptor (good_home);
-# line 304 "FindHome.puma"
+/* line 315 "FindHome.puma" */
    MakeNoDescriptor (min_home);
   }
    return;
 
   case kCALL_STMT:
-# line 309 "FindHome.puma"
+/* line 320 "FindHome.puma" */
   {
-# line 311 "FindHome.puma"
-   if (! ((IsPureCall (t)))) goto yyL17;
+/* line 322 "FindHome.puma" */
+   if (! ((IsPureCall (t)))) goto yyL18;
   {
-# line 313 "FindHome.puma"
+/* line 324 "FindHome.puma" */
    FindParamListHome (t->CALL_STMT.CALL_PARAMS, min_home, good_home);
-# line 315 "FindHome.puma"
+/* line 326 "FindHome.puma" */
 
 #ifdef DEBUG
     PrintHomes (t, min_home, good_home);
@@ -424,130 +432,128 @@ void FindHome
   }
   }
    return;
-yyL17:;
+yyL18:;
 
-# line 322 "FindHome.puma"
+/* line 333 "FindHome.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   {
-# line 324 "FindHome.puma"
-   if (! ((IsSerialCall (t)))) goto yyL18;
+/* line 335 "FindHome.puma" */
+   if (! ((IsSerialCall (t)))) goto yyL19;
   {
-# line 326 "FindHome.puma"
+/* line 337 "FindHome.puma" */
    stmt_protocol ("check for serial home");
-# line 328 "FindHome.puma"
+/* line 339 "FindHome.puma" */
    FindSerialHome (t->CALL_STMT.CALL_PARAMS, min_home, & yyV1);
-# line 329 "FindHome.puma"
-   if (! ((yyV1))) goto yyL18;
+/* line 340 "FindHome.puma" */
+   if (! ((yyV1))) goto yyL19;
   {
-# line 331 "FindHome.puma"
+/* line 342 "FindHome.puma" */
    tree_protocol ("serial home found : ", PrintableDescriptorVar (min_home));
-# line 333 "FindHome.puma"
+/* line 344 "FindHome.puma" */
  *good_home = *min_home; 
   }
   }
   }
    return;
  }
-yyL18:;
+yyL19:;
 
-# line 336 "FindHome.puma"
+/* line 347 "FindHome.puma" */
   {
-# line 338 "FindHome.puma"
-   if (! ((IsSerialCall (t)))) goto yyL19;
+/* line 349 "FindHome.puma" */
+   if (! ((IsSerialCall (t)))) goto yyL20;
   {
-# line 340 "FindHome.puma"
+/* line 351 "FindHome.puma" */
    MakeHostDescriptor (min_home);
-# line 341 "FindHome.puma"
+/* line 352 "FindHome.puma" */
    MakeHostDescriptor (good_home);
   }
   }
    return;
-yyL19:;
+yyL20:;
 
-# line 344 "FindHome.puma"
+/* line 355 "FindHome.puma" */
  {
   tTree home_clause;
   {
-# line 346 "FindHome.puma"
-   if (! ((HasUserHome ()))) goto yyL20;
+/* line 357 "FindHome.puma" */
+   if (! ((HasUserHome ()))) goto yyL21;
   {
-# line 348 "FindHome.puma"
-
-# line 350 "FindHome.puma"
+/* line 361 "FindHome.puma" */
    home_clause = GetUserHome ();
-# line 352 "FindHome.puma"
+/* line 363 "FindHome.puma" */
    MakeClauseDescriptor (home_clause, min_home);
-# line 353 "FindHome.puma"
+/* line 364 "FindHome.puma" */
    MakeClauseDescriptor (home_clause, good_home);
   }
   }
    return;
  }
-yyL20:;
+yyL21:;
 
-# line 356 "FindHome.puma"
+/* line 367 "FindHome.puma" */
   {
-# line 358 "FindHome.puma"
-   if (! ((UserIndependent ()))) goto yyL21;
+/* line 369 "FindHome.puma" */
+   if (! ((UserIndependent ()))) goto yyL22;
   {
-# line 360 "FindHome.puma"
+/* line 371 "FindHome.puma" */
    FindParamListHome (t->CALL_STMT.CALL_PARAMS, min_home, good_home);
   }
   }
    return;
-yyL21:;
+yyL22:;
 
-# line 365 "FindHome.puma"
+/* line 376 "FindHome.puma" */
   {
-# line 367 "FindHome.puma"
+/* line 378 "FindHome.puma" */
    MakeReplicatedDescriptor (min_home);
-# line 368 "FindHome.puma"
+/* line 379 "FindHome.puma" */
    MakeReplicatedDescriptor (good_home);
   }
    return;
 
   case kALLOCATE_STMT:
-# line 374 "FindHome.puma"
+/* line 385 "FindHome.puma" */
   {
-# line 376 "FindHome.puma"
+/* line 387 "FindHome.puma" */
    FindDescriptorHome (t->ALLOCATE_STMT.PARAMS, min_home, good_home);
   }
    return;
 
   case kDEALLOCATE_STMT:
-# line 379 "FindHome.puma"
+/* line 390 "FindHome.puma" */
   {
-# line 381 "FindHome.puma"
+/* line 392 "FindHome.puma" */
    FindDescriptorHome (t->DEALLOCATE_STMT.PARAMS, min_home, good_home);
   }
    return;
 
   case kCREATE_DSP_STMT:
-# line 384 "FindHome.puma"
+/* line 395 "FindHome.puma" */
   {
-# line 386 "FindHome.puma"
+/* line 397 "FindHome.puma" */
    FindDescriptorHome (t->CREATE_DSP_STMT.VAR, min_home, good_home);
   }
    return;
 
   case kFREE_DSP_STMT:
-# line 389 "FindHome.puma"
+/* line 400 "FindHome.puma" */
   {
-# line 391 "FindHome.puma"
+/* line 402 "FindHome.puma" */
    FindDescriptorHome (t->FREE_DSP_STMT.VAR, min_home, good_home);
   }
    return;
 
   }
 
-# line 394 "FindHome.puma"
+/* line 405 "FindHome.puma" */
   {
-# line 396 "FindHome.puma"
+/* line 407 "FindHome.puma" */
    stmt_protocol ("executed by ALL");
-# line 398 "FindHome.puma"
+/* line 409 "FindHome.puma" */
    MakeReplicatedDescriptor (good_home);
-# line 399 "FindHome.puma"
+/* line 410 "FindHome.puma" */
    MakeReplicatedDescriptor (min_home);
   }
    return;
@@ -567,39 +573,39 @@ static void FindParamHome
 {
   if (t->Kind == kVAR_PARAM) {
   if (t->VAR_PARAM.V->Kind == kADDR) {
-# line 410 "FindHome.puma"
+/* line 421 "FindHome.puma" */
   {
-# line 412 "FindHome.puma"
+/* line 423 "FindHome.puma" */
    FindExpHome (t->VAR_PARAM.V->ADDR.E, min_home, good_home);
   }
    return;
 
   }
-# line 415 "FindHome.puma"
+/* line 426 "FindHome.puma" */
   {
-# line 417 "FindHome.puma"
+/* line 428 "FindHome.puma" */
    if (! (((t->VAR_PARAM.intent == IntentOut) || (t->VAR_PARAM.intent == IntentInOut)))) goto yyL2;
   {
-# line 418 "FindHome.puma"
+/* line 429 "FindHome.puma" */
    FindWriteVarHome (t->VAR_PARAM.V, min_home, good_home);
   }
   }
    return;
 yyL2:;
 
-# line 421 "FindHome.puma"
+/* line 432 "FindHome.puma" */
   {
-# line 423 "FindHome.puma"
+/* line 434 "FindHome.puma" */
    FindReadVarHome (t->VAR_PARAM.V, min_home, good_home);
   }
    return;
 
   }
-# line 428 "FindHome.puma"
+/* line 439 "FindHome.puma" */
   {
-# line 430 "FindHome.puma"
+/* line 441 "FindHome.puma" */
    MakeNoDescriptor (min_home);
-# line 431 "FindHome.puma"
+/* line 442 "FindHome.puma" */
    MakeNoDescriptor (good_home);
   }
    return;
@@ -618,33 +624,29 @@ static void FindParamListHome
 # endif
 {
   if (t->Kind == kBTP_LIST) {
-# line 442 "FindHome.puma"
+/* line 453 "FindHome.puma" */
  {
   var_descriptor min_home1;
   var_descriptor good_home1;
   {
-# line 444 "FindHome.puma"
-
-# line 445 "FindHome.puma"
-
-# line 447 "FindHome.puma"
+/* line 458 "FindHome.puma" */
    FindParamHome (t->BTP_LIST.Elem, min_home, good_home);
-# line 448 "FindHome.puma"
+/* line 459 "FindHome.puma" */
    FindParamListHome (t->BTP_LIST.Next, & min_home1, & good_home1);
-# line 450 "FindHome.puma"
+/* line 461 "FindHome.puma" */
    UnionHome (min_home, & min_home1);
-# line 451 "FindHome.puma"
+/* line 462 "FindHome.puma" */
    ChooseHome (good_home, & good_home1);
   }
    return;
  }
 
   }
-# line 454 "FindHome.puma"
+/* line 465 "FindHome.puma" */
   {
-# line 456 "FindHome.puma"
+/* line 467 "FindHome.puma" */
    MakeNoDescriptor (min_home);
-# line 457 "FindHome.puma"
+/* line 468 "FindHome.puma" */
    MakeNoDescriptor (good_home);
   }
    return;
@@ -662,23 +664,20 @@ static void FindExpHome
  pvar good_home;
 # endif
 {
+ yyRecursion:
   if (exp->Kind == kBTE_LIST) {
-# line 468 "FindHome.puma"
+/* line 479 "FindHome.puma" */
  {
   var_descriptor min_home1;
   var_descriptor good_home1;
   {
-# line 470 "FindHome.puma"
-
-# line 471 "FindHome.puma"
-
-# line 473 "FindHome.puma"
+/* line 484 "FindHome.puma" */
    FindExpHome (exp->BTE_LIST.Elem, min_home, good_home);
-# line 474 "FindHome.puma"
+/* line 485 "FindHome.puma" */
    FindExpHome (exp->BTE_LIST.Next, & min_home1, & good_home1);
-# line 476 "FindHome.puma"
+/* line 487 "FindHome.puma" */
    UnionHome (min_home, & min_home1);
-# line 477 "FindHome.puma"
+/* line 488 "FindHome.puma" */
    ChooseHome (good_home, & good_home1);
   }
    return;
@@ -686,22 +685,18 @@ static void FindExpHome
 
   }
   if (exp->Kind == kOP_EXP) {
-# line 480 "FindHome.puma"
+/* line 491 "FindHome.puma" */
  {
   var_descriptor min_home1;
   var_descriptor good_home1;
   {
-# line 482 "FindHome.puma"
-
-# line 483 "FindHome.puma"
-
-# line 485 "FindHome.puma"
+/* line 496 "FindHome.puma" */
    FindExpHome (exp->OP_EXP.OPND1, min_home, good_home);
-# line 486 "FindHome.puma"
+/* line 497 "FindHome.puma" */
    FindExpHome (exp->OP_EXP.OPND2, & min_home1, & good_home1);
-# line 488 "FindHome.puma"
+/* line 499 "FindHome.puma" */
    UnionHome (min_home, & min_home1);
-# line 489 "FindHome.puma"
+/* line 500 "FindHome.puma" */
    ChooseHome (good_home, & good_home1);
   }
    return;
@@ -709,67 +704,67 @@ static void FindExpHome
 
   }
   if (exp->Kind == kOP1_EXP) {
-# line 492 "FindHome.puma"
+/* line 503 "FindHome.puma" */
   {
-# line 494 "FindHome.puma"
-   FindExpHome (exp->OP1_EXP.OPND, min_home, good_home);
+/* line 505 "FindHome.puma" */
+   exp = exp->OP1_EXP.OPND;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (exp->Kind == kVAR_EXP) {
-# line 497 "FindHome.puma"
+/* line 508 "FindHome.puma" */
   {
-# line 499 "FindHome.puma"
+/* line 510 "FindHome.puma" */
    FindReadVarHome (exp->VAR_EXP.V, min_home, good_home);
   }
    return;
 
   }
   if (exp->Kind == kFUNC_CALL_EXP) {
-# line 502 "FindHome.puma"
+/* line 513 "FindHome.puma" */
   {
-# line 504 "FindHome.puma"
+/* line 515 "FindHome.puma" */
    if (! ((IsPureCall (exp)))) goto yyL5;
   {
-# line 506 "FindHome.puma"
+/* line 517 "FindHome.puma" */
    FindParamListHome (exp->FUNC_CALL_EXP.FUNC_PARAMS, min_home, good_home);
   }
   }
    return;
 yyL5:;
 
-# line 509 "FindHome.puma"
+/* line 520 "FindHome.puma" */
   {
-# line 511 "FindHome.puma"
+/* line 522 "FindHome.puma" */
    if (! ((IsSerialCall (exp)))) goto yyL6;
   {
-# line 513 "FindHome.puma"
+/* line 524 "FindHome.puma" */
    FindParamListHome (exp->FUNC_CALL_EXP.FUNC_PARAMS, min_home, good_home);
-# line 515 "FindHome.puma"
+/* line 526 "FindHome.puma" */
    MakeNoDescriptor (min_home);
   }
   }
    return;
 yyL6:;
 
-# line 520 "FindHome.puma"
+/* line 531 "FindHome.puma" */
   {
-# line 522 "FindHome.puma"
+/* line 533 "FindHome.puma" */
    tree_protocol ("function call requires replicated execution : ", exp);
-# line 524 "FindHome.puma"
+/* line 535 "FindHome.puma" */
    MakeReplicatedDescriptor (min_home);
-# line 525 "FindHome.puma"
+/* line 536 "FindHome.puma" */
    MakeReplicatedDescriptor (good_home);
   }
    return;
 
   }
-# line 528 "FindHome.puma"
+/* line 539 "FindHome.puma" */
   {
-# line 530 "FindHome.puma"
+/* line 541 "FindHome.puma" */
    MakeNoDescriptor (min_home);
-# line 531 "FindHome.puma"
+/* line 542 "FindHome.puma" */
    MakeNoDescriptor (good_home);
   }
    return;
@@ -787,34 +782,52 @@ static void FindVarHome
  pvar good_home;
 # endif
 {
-# line 554 "FindHome.puma"
+/* line 565 "FindHome.puma" */
   {
-# line 556 "FindHome.puma"
-   if (! ((IsDescriptorVar (var)))) goto yyL1;
+/* line 567 "FindHome.puma" */
+   if (! ((GetCurrentModel () == HPF_TASK))) goto yyL1;
   {
-# line 558 "FindHome.puma"
-   SetVarDescriptor (var, good_home);
-# line 559 "FindHome.puma"
+/* line 569 "FindHome.puma" */
+   if (! ((IsGlobalVar (var, GetCurrentUnitObject ())))) goto yyL1;
+  {
+/* line 571 "FindHome.puma" */
    MakeNoDescriptor (min_home);
+/* line 572 "FindHome.puma" */
+   MakeParLoopNestDescriptor (good_home);
+  }
   }
   }
    return;
 yyL1:;
 
-  if (var->Kind == kINDEXED_VAR) {
-# line 562 "FindHome.puma"
+/* line 575 "FindHome.puma" */
   {
-# line 566 "FindHome.puma"
+/* line 577 "FindHome.puma" */
+   if (! ((IsDescriptorVar (var)))) goto yyL2;
+  {
+/* line 579 "FindHome.puma" */
+   SetVarDescriptor (var, good_home);
+/* line 580 "FindHome.puma" */
+   MakeNoDescriptor (min_home);
+  }
+  }
+   return;
+yyL2:;
+
+  if (var->Kind == kINDEXED_VAR) {
+/* line 583 "FindHome.puma" */
+  {
+/* line 587 "FindHome.puma" */
    FindExpHome (var->INDEXED_VAR.IND_EXPS, min_home, good_home);
   }
    return;
 
   }
-# line 569 "FindHome.puma"
+/* line 590 "FindHome.puma" */
   {
-# line 571 "FindHome.puma"
+/* line 592 "FindHome.puma" */
    MakeNoDescriptor (min_home);
-# line 572 "FindHome.puma"
+/* line 593 "FindHome.puma" */
    MakeNoDescriptor (good_home);
   }
    return;
@@ -832,41 +845,41 @@ static void FindReadVarHome
  pvar good_home;
 # endif
 {
-# line 594 "FindHome.puma"
+/* line 615 "FindHome.puma" */
   {
-# line 596 "FindHome.puma"
+/* line 617 "FindHome.puma" */
    if (! ((IsNewVariable (var)))) goto yyL1;
   {
-# line 597 "FindHome.puma"
+/* line 619 "FindHome.puma" */
    MakeNoDescriptor (min_home);
-# line 598 "FindHome.puma"
+/* line 620 "FindHome.puma" */
    MakeNoDescriptor (good_home);
   }
   }
    return;
 yyL1:;
 
-# line 601 "FindHome.puma"
+/* line 623 "FindHome.puma" */
   {
-# line 603 "FindHome.puma"
+/* line 625 "FindHome.puma" */
    FindVarHome (var, min_home, good_home);
-# line 605 "FindHome.puma"
+/* line 627 "FindHome.puma" */
    if (! ((IsParallelHomeDescriptor (good_home)))) goto yyL2;
   }
    return;
 yyL2:;
 
-# line 608 "FindHome.puma"
+/* line 630 "FindHome.puma" */
   {
-# line 610 "FindHome.puma"
+/* line 632 "FindHome.puma" */
    FindVarHome (var, min_home, good_home);
-# line 612 "FindHome.puma"
+/* line 634 "FindHome.puma" */
    if (! ((! IsNoDescriptor (good_home)))) goto yyL3;
   {
-# line 613 "FindHome.puma"
+/* line 635 "FindHome.puma" */
    if (! ((TreeWriteDistribution (var) == 1))) goto yyL3;
   {
-# line 614 "FindHome.puma"
+/* line 636 "FindHome.puma" */
    MakeNoDescriptor (min_home);
   }
   }
@@ -874,11 +887,11 @@ yyL2:;
    return;
 yyL3:;
 
-# line 617 "FindHome.puma"
+/* line 639 "FindHome.puma" */
   {
-# line 619 "FindHome.puma"
+/* line 641 "FindHome.puma" */
    MakeParLoopNestDescriptor (good_home);
-# line 620 "FindHome.puma"
+/* line 642 "FindHome.puma" */
    MakeNoDescriptor (min_home);
   }
    return;
@@ -896,14 +909,14 @@ static void FindWriteVarHome
  pvar good_home;
 # endif
 {
-# line 639 "FindHome.puma"
+/* line 661 "FindHome.puma" */
   {
-# line 641 "FindHome.puma"
+/* line 663 "FindHome.puma" */
    if (! ((IsNewVariable (var)))) goto yyL1;
   {
-# line 643 "FindHome.puma"
+/* line 665 "FindHome.puma" */
    MakeNoDescriptor (min_home);
-# line 644 "FindHome.puma"
+/* line 666 "FindHome.puma" */
    MakeNoDescriptor (good_home);
   }
   }
@@ -911,15 +924,15 @@ static void FindWriteVarHome
 yyL1:;
 
   if (var->Kind == kUSED_VAR) {
-# line 647 "FindHome.puma"
+/* line 669 "FindHome.puma" */
   {
-# line 651 "FindHome.puma"
+/* line 673 "FindHome.puma" */
    if (! ((TreeRank (var->USED_VAR.VARNAME) == 0))) goto yyL2;
   {
-# line 652 "FindHome.puma"
+/* line 674 "FindHome.puma" */
    if (! ((TreeWriteDistribution (var->USED_VAR.VARNAME) == 0))) goto yyL2;
   {
-# line 654 "FindHome.puma"
+/* line 676 "FindHome.puma" */
  if (IsParallelMasked ())
 
         { MakeNoDescriptor (min_home);
@@ -939,21 +952,21 @@ yyL1:;
 yyL2:;
 
   }
-# line 668 "FindHome.puma"
+/* line 690 "FindHome.puma" */
   {
-# line 670 "FindHome.puma"
+/* line 692 "FindHome.puma" */
    FindVarHome (var, min_home, good_home);
-# line 672 "FindHome.puma"
+/* line 694 "FindHome.puma" */
    if (! ((IsParallelHomeDescriptor (good_home)))) goto yyL3;
   }
    return;
 yyL3:;
 
-# line 675 "FindHome.puma"
+/* line 697 "FindHome.puma" */
   {
-# line 677 "FindHome.puma"
+/* line 699 "FindHome.puma" */
    MakeNoDescriptor (min_home);
-# line 678 "FindHome.puma"
+/* line 700 "FindHome.puma" */
    MakeReplicatedDescriptor (good_home);
   }
    return;
@@ -971,81 +984,78 @@ static void FindDescriptorHome
  pvar good_home;
 # endif
 {
+ yyRecursion:
   if (t->Kind == kVAR_PARAM) {
-# line 694 "FindHome.puma"
+/* line 716 "FindHome.puma" */
   {
-# line 696 "FindHome.puma"
-   FindDescriptorHome (t->VAR_PARAM.V, min_home, good_home);
+/* line 718 "FindHome.puma" */
+   t = t->VAR_PARAM.V;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kINDEXED_VAR) {
-# line 699 "FindHome.puma"
+/* line 721 "FindHome.puma" */
   {
-# line 701 "FindHome.puma"
-   FindDescriptorHome (t->INDEXED_VAR.IND_VAR, min_home, good_home);
+/* line 723 "FindHome.puma" */
+   t = t->INDEXED_VAR.IND_VAR;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kSELECTED_VAR) {
-# line 704 "FindHome.puma"
+/* line 726 "FindHome.puma" */
   {
-# line 706 "FindHome.puma"
+/* line 728 "FindHome.puma" */
    MakeReplicatedDescriptor (min_home);
-# line 707 "FindHome.puma"
+/* line 729 "FindHome.puma" */
    MakeReplicatedDescriptor (good_home);
   }
    return;
 
   }
   if (t->Kind == kUSED_VAR) {
-# line 710 "FindHome.puma"
+/* line 732 "FindHome.puma" */
   {
-# line 712 "FindHome.puma"
+/* line 734 "FindHome.puma" */
    if (! ((IsNewVariable (t)))) goto yyL4;
   {
-# line 714 "FindHome.puma"
+/* line 736 "FindHome.puma" */
    MakeNoDescriptor (min_home);
-# line 715 "FindHome.puma"
+/* line 737 "FindHome.puma" */
    MakeNoDescriptor (good_home);
   }
   }
    return;
 yyL4:;
 
-# line 718 "FindHome.puma"
+/* line 740 "FindHome.puma" */
   {
-# line 720 "FindHome.puma"
+/* line 742 "FindHome.puma" */
    stmt_protocol ("ALLOCATE/DEALLOCATE replicated (not NEW)");
-# line 721 "FindHome.puma"
+/* line 743 "FindHome.puma" */
    tree_protocol ("due to this variable : ", t);
-# line 723 "FindHome.puma"
+/* line 745 "FindHome.puma" */
    MakeReplicatedDescriptor (min_home);
-# line 724 "FindHome.puma"
+/* line 746 "FindHome.puma" */
    MakeReplicatedDescriptor (good_home);
   }
    return;
 
   }
   if (t->Kind == kBTP_LIST) {
-# line 727 "FindHome.puma"
+/* line 749 "FindHome.puma" */
  {
   var_descriptor min_home1;
   var_descriptor good_home1;
   {
-# line 729 "FindHome.puma"
-
-# line 730 "FindHome.puma"
-
-# line 732 "FindHome.puma"
+/* line 754 "FindHome.puma" */
    FindDescriptorHome (t->BTP_LIST.Elem, min_home, good_home);
-# line 733 "FindHome.puma"
+/* line 755 "FindHome.puma" */
    FindDescriptorHome (t->BTP_LIST.Next, & min_home1, & good_home1);
-# line 735 "FindHome.puma"
+/* line 757 "FindHome.puma" */
    UnionHome (min_home, & min_home1);
-# line 736 "FindHome.puma"
+/* line 758 "FindHome.puma" */
    ChooseHome (good_home, & good_home1);
   }
    return;
@@ -1053,23 +1063,23 @@ yyL4:;
 
   }
   if (t->Kind == kBTP_EMPTY) {
-# line 739 "FindHome.puma"
+/* line 761 "FindHome.puma" */
   {
-# line 741 "FindHome.puma"
+/* line 763 "FindHome.puma" */
    MakeNoDescriptor (min_home);
-# line 742 "FindHome.puma"
+/* line 764 "FindHome.puma" */
    MakeNoDescriptor (good_home);
   }
    return;
 
   }
-# line 745 "FindHome.puma"
+/* line 767 "FindHome.puma" */
   {
-# line 747 "FindHome.puma"
+/* line 769 "FindHome.puma" */
    tree_error_protocol ("unknown structure for ALLOCATE/..", t);
-# line 749 "FindHome.puma"
+/* line 771 "FindHome.puma" */
    MakeReplicatedDescriptor (min_home);
-# line 750 "FindHome.puma"
+/* line 772 "FindHome.puma" */
    MakeReplicatedDescriptor (good_home);
   }
    return;
@@ -1086,9 +1096,9 @@ static void ChooseHome
  pvar home2;
 # endif
 {
-# line 769 "FindHome.puma"
+/* line 791 "FindHome.puma" */
   {
-# line 771 "FindHome.puma"
+/* line 793 "FindHome.puma" */
 
 #ifdef DEBUG
      printf ("CHOOSE of ");
@@ -1118,47 +1128,47 @@ static void ChooseBestHome
  pvar home2;
 # endif
 {
-# line 795 "FindHome.puma"
+/* line 817 "FindHome.puma" */
   {
-# line 797 "FindHome.puma"
+/* line 819 "FindHome.puma" */
    if (! ((IsNoDescriptor (home1)))) goto yyL1;
   {
-# line 798 "FindHome.puma"
+/* line 820 "FindHome.puma" */
  *home1 = *home2; 
   }
   }
    return;
 yyL1:;
 
-# line 801 "FindHome.puma"
+/* line 823 "FindHome.puma" */
   {
-# line 803 "FindHome.puma"
+/* line 825 "FindHome.puma" */
    if (! ((IsNoDescriptor (home2)))) goto yyL2;
   }
    return;
 yyL2:;
 
-# line 814 "FindHome.puma"
+/* line 836 "FindHome.puma" */
   {
-# line 816 "FindHome.puma"
+/* line 838 "FindHome.puma" */
    if (! ((home1 -> var_tree != NoTree))) goto yyL3;
   {
-# line 817 "FindHome.puma"
+/* line 839 "FindHome.puma" */
    if (! ((home2 -> var_tree == NoTree))) goto yyL3;
   }
   }
    return;
 yyL3:;
 
-# line 820 "FindHome.puma"
+/* line 842 "FindHome.puma" */
   {
-# line 822 "FindHome.puma"
+/* line 844 "FindHome.puma" */
    if (! ((home1 -> var_tree == NoTree))) goto yyL4;
   {
-# line 823 "FindHome.puma"
+/* line 845 "FindHome.puma" */
    if (! ((home2 -> var_tree != NoTree))) goto yyL4;
   {
-# line 825 "FindHome.puma"
+/* line 847 "FindHome.puma" */
  *home1 = *home2; 
   }
   }
@@ -1166,27 +1176,27 @@ yyL3:;
    return;
 yyL4:;
 
-# line 843 "FindHome.puma"
+/* line 865 "FindHome.puma" */
   {
-# line 845 "FindHome.puma"
+/* line 867 "FindHome.puma" */
    if (! ((IsParallelHomeDescriptor (home1)))) goto yyL5;
   {
-# line 846 "FindHome.puma"
+/* line 868 "FindHome.puma" */
    if (! ((! IsParallelHomeDescriptor (home2)))) goto yyL5;
   }
   }
    return;
 yyL5:;
 
-# line 849 "FindHome.puma"
+/* line 871 "FindHome.puma" */
   {
-# line 851 "FindHome.puma"
+/* line 873 "FindHome.puma" */
    if (! ((! IsParallelHomeDescriptor (home1)))) goto yyL6;
   {
-# line 852 "FindHome.puma"
+/* line 874 "FindHome.puma" */
    if (! ((IsParallelHomeDescriptor (home2)))) goto yyL6;
   {
-# line 854 "FindHome.puma"
+/* line 876 "FindHome.puma" */
  *home1 = *home2; 
   }
   }
@@ -1194,27 +1204,27 @@ yyL5:;
    return;
 yyL6:;
 
-# line 863 "FindHome.puma"
+/* line 885 "FindHome.puma" */
   {
-# line 865 "FindHome.puma"
+/* line 887 "FindHome.puma" */
    if (! ((home1 -> topology_rank > 0))) goto yyL7;
   {
-# line 866 "FindHome.puma"
+/* line 888 "FindHome.puma" */
    if (! ((home2 -> topology_rank == 0))) goto yyL7;
   }
   }
    return;
 yyL7:;
 
-# line 869 "FindHome.puma"
+/* line 891 "FindHome.puma" */
   {
-# line 871 "FindHome.puma"
+/* line 893 "FindHome.puma" */
    if (! ((home1 -> topology_rank == 0))) goto yyL8;
   {
-# line 872 "FindHome.puma"
+/* line 894 "FindHome.puma" */
    if (! ((home2 -> topology_rank > 0))) goto yyL8;
   {
-# line 874 "FindHome.puma"
+/* line 896 "FindHome.puma" */
  *home1 = *home2; 
   }
   }
@@ -1222,7 +1232,7 @@ yyL7:;
    return;
 yyL8:;
 
-# line 877 "FindHome.puma"
+/* line 899 "FindHome.puma" */
    return;
 
 ;
@@ -1237,9 +1247,9 @@ static void UnionHome
  pvar home2;
 # endif
 {
-# line 897 "FindHome.puma"
+/* line 919 "FindHome.puma" */
   {
-# line 899 "FindHome.puma"
+/* line 921 "FindHome.puma" */
 
 #ifdef DEBUG
      printf ("UNION of ");
@@ -1270,13 +1280,11 @@ static void ExpandHomes
  register tTree id;
 # endif
 {
-# line 923 "FindHome.puma"
+/* line 945 "FindHome.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 925 "FindHome.puma"
-
-# line 927 "FindHome.puma"
+/* line 949 "FindHome.puma" */
 
 #ifdef DEBUG
      printf ("EXPAND of min_home ");
@@ -1337,9 +1345,9 @@ static void PrintHomes
  pvar good_home;
 # endif
 {
-# line 980 "FindHome.puma"
+/* line 1002 "FindHome.puma" */
   {
-# line 982 "FindHome.puma"
+/* line 1004 "FindHome.puma" */
  
 #ifdef DEBUG
       printf ("HOME of ");
@@ -1359,20 +1367,20 @@ static void PrintHomes
 
 void FindSerialHome
 # if defined __STDC__ | defined __cplusplus
-(register tTree stmt, pvar home, register bool * yyP1)
+(register tTree stmt, pvar home, register rbool * yyP1)
 # else
 (stmt, home, yyP1)
  register tTree stmt;
  pvar home;
- register bool * yyP1;
+ register rbool * yyP1;
 # endif
 {
   if (stmt->Kind == kCALL_STMT) {
-# line 1005 "FindHome.puma"
+/* line 1027 "FindHome.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   {
-# line 1007 "FindHome.puma"
+/* line 1029 "FindHome.puma" */
    FindSerialHome (stmt->CALL_STMT.CALL_PARAMS, home, & yyV1);
   }
    * yyP1 = yyV1;
@@ -1382,30 +1390,28 @@ void FindSerialHome
   }
   if (stmt->Kind == kASSIGN_STMT) {
   if (stmt->ASSIGN_STMT.ASSIGN_EXP->Kind == kFUNC_CALL_EXP) {
-# line 1010 "FindHome.puma"
+/* line 1032 "FindHome.puma" */
  {
   var_descriptor min_home;
   {
-# line 1012 "FindHome.puma"
-
-# line 1014 "FindHome.puma"
+/* line 1036 "FindHome.puma" */
    FindVarHome (stmt->ASSIGN_STMT.ASSIGN_VAR, & min_home, home);
-# line 1016 "FindHome.puma"
+/* line 1038 "FindHome.puma" */
    if (! ((IsNoDescriptor (& min_home)))) goto yyL2;
   {
-# line 1017 "FindHome.puma"
+/* line 1039 "FindHome.puma" */
    if (! ((IsSerialDescriptor (home)))) goto yyL2;
   {
-# line 1018 "FindHome.puma"
+/* line 1040 "FindHome.puma" */
    if (! ((! CountCommunication (home, stmt->ASSIGN_STMT.ASSIGN_VAR)))) goto yyL2;
   {
-# line 1019 "FindHome.puma"
+/* line 1041 "FindHome.puma" */
    if (! ((! CountCommunication (home, stmt->ASSIGN_STMT.ASSIGN_EXP)))) goto yyL2;
   }
   }
   }
   }
-   * yyP1 = true;
+   * yyP1 = rtrue;
    return;
  }
 yyL2:;
@@ -1413,42 +1419,40 @@ yyL2:;
   }
   }
   if (stmt->Kind == kBTP_LIST) {
-# line 1022 "FindHome.puma"
+/* line 1044 "FindHome.puma" */
  {
   var_descriptor min_home;
   {
-# line 1024 "FindHome.puma"
-
-# line 1026 "FindHome.puma"
+/* line 1048 "FindHome.puma" */
    FindParamListHome (stmt, & min_home, home);
-# line 1028 "FindHome.puma"
+/* line 1050 "FindHome.puma" */
    if (! ((IsNoDescriptor (& min_home)))) goto yyL3;
   {
-# line 1029 "FindHome.puma"
+/* line 1051 "FindHome.puma" */
    if (! ((IsSerialDescriptor (home)))) goto yyL3;
   {
-# line 1030 "FindHome.puma"
+/* line 1052 "FindHome.puma" */
    if (! ((! CountCommunication (home, stmt)))) goto yyL3;
   }
   }
   }
-   * yyP1 = true;
+   * yyP1 = rtrue;
    return;
  }
 yyL3:;
 
   }
-# line 1033 "FindHome.puma"
-   * yyP1 = false;
+/* line 1055 "FindHome.puma" */
+   * yyP1 = rfalse;
    return;
 
 ;
 }
 
-void BeginFindHome ()
+void BeginFindHome ARGS ((void))
 {
 }
 
-void CloseFindHome ()
+void CloseFindHome ARGS ((void))
 {
 }
