@@ -1,6 +1,13 @@
 # include "GlobalComm.h"
-# include "yyGlobalComm.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,34 +15,31 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 28 "GlobalComm.puma"
+/* line 28 "GlobalComm.puma" */
 
 
 # undef DEBUG
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
@@ -43,9 +47,13 @@
 # include "Transform.h"
 
 # include "VarDescriptor.h"
+# include "HomeDescriptor.h"
 # include "Distributions.h"
 # include "Descriptor.h"
 
+# include "IndShadow.h"   /* IsIndShadowVariable              */
+# include "VectorMove.h"  /* MakeVectorVar                    */
+
 # include "Ownership.h"   /* BetterOnSelection                */
 # include "VarComm.h"     /* IsLocalVarRead, IsLocalVarWrite  */
 # include "FArguments.h"  /* IsOnlyReadParam                  */
@@ -58,11 +66,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* GlobalComm_Exit) () = yyExit;
+# include "yyGlobalComm.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* GlobalComm_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -71,23 +93,24 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module GlobalComm, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module GlobalComm, routine %s failed\n",
+  yyFunction);
  GlobalComm_Exit ();
 }
 
-tTree GlobalCommunication ARGS((tTree stmt));
-static tTree ExtractGlobalComm ARGS((tTree stmt, pvar home));
-static tTree ExtractParamComm ARGS((tTree params, pvar home));
-static tTree ExtractWriteVarComm ARGS((tTree var, pvar home));
-static void ApplyHomeChange ARGS((tTree stmts, pvar inner_home, pvar outer_home));
-static tTree MakeBroadcast ARGS((tTree bc_vars));
-static tTree ParameterBroadcast ARGS((tTree p));
-static tTree ControlSpecBroadcast ARGS((tTree p));
-static tTree ReadSpecBroadcast ARGS((tTree p));
-static tTree NameListVars ARGS((tTree names));
-static void VectorizeGlobalComm ARGS((tTree stmts, pvar home, tTree id, tTree slice));
-static void VectorizeVarList ARGS((tTree vars, tTree id, tTree slice, bool * yyP1));
-static void SummarizeGlobalComm ARGS((tTree stmts, pvar home, tTree loop));
+tTree GlobalCommunication ARGS ((tTree stmt));
+static tTree ExtractGlobalComm ARGS ((tTree stmt, pvar home));
+static tTree ExtractParamComm ARGS ((int line, tTree params, pvar home));
+static tTree ExtractWriteVarComm ARGS ((int line, tTree var, pvar home));
+static void ApplyHomeChange ARGS ((tTree stmts, pvar inner_home, pvar outer_home));
+static tTree MakeBroadcast ARGS ((int line, tTree bc_vars));
+static tTree ParameterBroadcast ARGS ((tTree p));
+static tTree ControlSpecBroadcast ARGS ((tTree p));
+static tTree ReadSpecBroadcast ARGS ((tTree p));
+static tTree NameListVars ARGS ((tTree names));
+static void VectorizeGlobalComm ARGS ((tTree stmts, pvar home, tTree id, tTree slice));
+static void VectorizeVarList ARGS ((tTree vars, tTree id, tTree slice, rbool * yyP1));
+static void SummarizeGlobalComm ARGS ((tTree stmts, pvar home, tTree loop));
 
 tTree GlobalCommunication
 # if defined __STDC__ | defined __cplusplus
@@ -98,24 +121,20 @@ tTree GlobalCommunication
 # endif
 {
   if (stmt->Kind == kACF_HOME) {
-# line 68 "GlobalComm.puma"
+/* line 72 "GlobalComm.puma" */
  {
   var_descriptor outer_home;
   {
-# line 70 "GlobalComm.puma"
-
-# line 72 "GlobalComm.puma"
+/* line 76 "GlobalComm.puma" */
    MakeReplicatedDescriptor (& outer_home);
   }
-  {
    return ExtractGlobalComm (stmt, & outer_home);
-  }
  }
 
   }
-# line 77 "GlobalComm.puma"
+/* line 81 "GlobalComm.puma" */
   {
-# line 79 "GlobalComm.puma"
+/* line 83 "GlobalComm.puma" */
    failure_protocol (MODULE, "GlobalCommunication", stmt);
   }
    return NoTree;
@@ -135,113 +154,93 @@ static tTree ExtractGlobalComm
   switch (stmt->Kind) {
   case kACF_HOME:
   if (stmt->ACF_HOME.HOME_VAR->Kind == kON_HOST_CLAUSE) {
-# line 98 "GlobalComm.puma"
+/* line 102 "GlobalComm.puma" */
  {
   var_descriptor on_home;
   tTree global_comm;
   {
-# line 100 "GlobalComm.puma"
-
-# line 102 "GlobalComm.puma"
+/* line 106 "GlobalComm.puma" */
    MakeHostDescriptor (& on_home);
-# line 104 "GlobalComm.puma"
-
-# line 106 "GlobalComm.puma"
+/* line 110 "GlobalComm.puma" */
    IncParNesting (stmt);
-# line 108 "GlobalComm.puma"
+/* line 112 "GlobalComm.puma" */
    global_comm = ExtractGlobalComm (stmt->ACF_HOME.HOME_BODY, & on_home);
-# line 110 "GlobalComm.puma"
+/* line 114 "GlobalComm.puma" */
    DecParNesting (stmt);
-# line 114 "GlobalComm.puma"
+/* line 118 "GlobalComm.puma" */
    ApplyHomeChange (global_comm, & on_home, home);
   }
-  {
    return global_comm;
-  }
  }
 
   }
   if (stmt->ACF_HOME.HOME_VAR->Kind == kON_PROC_CLAUSE) {
-# line 125 "GlobalComm.puma"
+/* line 129 "GlobalComm.puma" */
  {
   var_descriptor on_home;
   tTree global_comm;
   {
-# line 127 "GlobalComm.puma"
-
-# line 129 "GlobalComm.puma"
+/* line 133 "GlobalComm.puma" */
    SetProcDescriptor (stmt->ACF_HOME.HOME_VAR->ON_PROC_CLAUSE.ON_PROC, & on_home);
-# line 131 "GlobalComm.puma"
-
-# line 133 "GlobalComm.puma"
+/* line 137 "GlobalComm.puma" */
    IncParNesting (stmt);
-# line 135 "GlobalComm.puma"
+/* line 139 "GlobalComm.puma" */
    global_comm = ExtractGlobalComm (stmt->ACF_HOME.HOME_BODY, & on_home);
-# line 137 "GlobalComm.puma"
+/* line 141 "GlobalComm.puma" */
    DecParNesting (stmt);
-# line 141 "GlobalComm.puma"
+/* line 145 "GlobalComm.puma" */
    ApplyHomeChange (global_comm, & on_home, home);
   }
-  {
    return global_comm;
-  }
  }
 
   }
   if (stmt->ACF_HOME.HOME_VAR->Kind == kON_VAR_CLAUSE) {
-# line 152 "GlobalComm.puma"
+/* line 156 "GlobalComm.puma" */
  {
   var_descriptor on_home;
   tTree global_comm;
   {
-# line 154 "GlobalComm.puma"
-
-# line 156 "GlobalComm.puma"
+/* line 160 "GlobalComm.puma" */
    SetVarDescriptor (stmt->ACF_HOME.HOME_VAR->ON_VAR_CLAUSE.ON_VAR, & on_home);
-# line 158 "GlobalComm.puma"
-
-# line 160 "GlobalComm.puma"
+/* line 164 "GlobalComm.puma" */
    IncParNesting (stmt);
-# line 162 "GlobalComm.puma"
+/* line 166 "GlobalComm.puma" */
    global_comm = ExtractGlobalComm (stmt->ACF_HOME.HOME_BODY, & on_home);
-# line 164 "GlobalComm.puma"
+/* line 168 "GlobalComm.puma" */
    DecParNesting (stmt);
-# line 168 "GlobalComm.puma"
+/* line 172 "GlobalComm.puma" */
    ApplyHomeChange (global_comm, & on_home, home);
   }
-  {
    return global_comm;
-  }
  }
 
   }
   break;
   case kACF_LIST:
-# line 179 "GlobalComm.puma"
+/* line 183 "GlobalComm.puma" */
    return (CombineACF (ExtractGlobalComm (stmt->ACF_LIST.Elem, home), ExtractGlobalComm (stmt->ACF_LIST.Next, home)));
 
   case kACF_EMPTY:
-# line 185 "GlobalComm.puma"
+/* line 189 "GlobalComm.puma" */
    return NoTree;
 
   case kACF_DUMMY:
-# line 190 "GlobalComm.puma"
+/* line 194 "GlobalComm.puma" */
    return NoTree;
 
   case kACF_IF:
-# line 195 "GlobalComm.puma"
+/* line 199 "GlobalComm.puma" */
    return (CombineACF (ExtractGlobalComm (stmt->ACF_IF.THEN_PART, home), ExtractGlobalComm (stmt->ACF_IF.ELSE_PART, home)));
 
   case kACF_WHILE:
-# line 207 "GlobalComm.puma"
+/* line 211 "GlobalComm.puma" */
  {
   tTree communication;
   {
-# line 209 "GlobalComm.puma"
-
-# line 211 "GlobalComm.puma"
+/* line 215 "GlobalComm.puma" */
    communication = ExtractGlobalComm (stmt->ACF_WHILE.WHILE_BODY, home);
-# line 213 "GlobalComm.puma"
+/* line 217 "GlobalComm.puma" */
  if (communication != NoTree)
 
          { set_protocol_stmt (stmt);
@@ -251,201 +250,175 @@ static tTree ExtractGlobalComm
          }
     
   }
-  {
    return NoTree;
-  }
  }
 
   case kACF_DO:
   if (stmt->ACF_DO.DO_HOME_INFO->Kind == kCOMM_INFO) {
-# line 234 "GlobalComm.puma"
+/* line 238 "GlobalComm.puma" */
  {
   var_descriptor body_home;
   tTree communication;
   {
-# line 236 "GlobalComm.puma"
-
-# line 238 "GlobalComm.puma"
+/* line 242 "GlobalComm.puma" */
    SetVarDescriptor (stmt->ACF_DO.DO_HOME_INFO->COMM_INFO.home_var, & body_home);
-# line 240 "GlobalComm.puma"
-
-# line 242 "GlobalComm.puma"
+/* line 246 "GlobalComm.puma" */
    communication = ExtractGlobalComm (stmt->ACF_DO.DO_BODY, & body_home);
-# line 244 "GlobalComm.puma"
+/* line 248 "GlobalComm.puma" */
    VectorizeGlobalComm (communication, & body_home, stmt->ACF_DO.DO_ID, stmt->ACF_DO.DO_RANGE);
   }
-  {
    return communication;
-  }
  }
 
   }
   if (stmt->ACF_DO.DO_DEP_INFO->Kind == kSERIAL_INFO) {
-# line 255 "GlobalComm.puma"
+/* line 259 "GlobalComm.puma" */
  {
   tTree communication;
   {
-# line 257 "GlobalComm.puma"
-
-# line 259 "GlobalComm.puma"
+/* line 263 "GlobalComm.puma" */
    communication = ExtractGlobalComm (stmt->ACF_DO.DO_BODY, home);
-# line 261 "GlobalComm.puma"
+/* line 265 "GlobalComm.puma" */
    SummarizeGlobalComm (communication, home, stmt);
   }
-  {
    return communication;
-  }
  }
 
   }
   if (stmt->ACF_DO.DO_DEP_INFO->Kind == kPARDO_INFO) {
-# line 266 "GlobalComm.puma"
+/* line 270 "GlobalComm.puma" */
  {
   tTree communication;
   {
-# line 268 "GlobalComm.puma"
-
-# line 270 "GlobalComm.puma"
+/* line 274 "GlobalComm.puma" */
    communication = ExtractGlobalComm (stmt->ACF_DO.DO_BODY, home);
-# line 272 "GlobalComm.puma"
+/* line 276 "GlobalComm.puma" */
    SummarizeGlobalComm (communication, home, stmt);
   }
-  {
    return communication;
-  }
  }
 
   }
   break;
   case kACF_TASK_REGION:
-# line 283 "GlobalComm.puma"
+/* line 287 "GlobalComm.puma" */
  {
   tTree comm_stmts;
   {
-# line 285 "GlobalComm.puma"
-
-# line 287 "GlobalComm.puma"
+/* line 291 "GlobalComm.puma" */
    IncParNesting (stmt);
-# line 289 "GlobalComm.puma"
+/* line 293 "GlobalComm.puma" */
    comm_stmts = ExtractGlobalComm (stmt->ACF_TASK_REGION.TASK_BODY, home);
-# line 291 "GlobalComm.puma"
+/* line 295 "GlobalComm.puma" */
    DecParNesting (stmt);
   }
-  {
    return comm_stmts;
-  }
  }
 
   case kACF_NEW:
-# line 302 "GlobalComm.puma"
+/* line 306 "GlobalComm.puma" */
  {
   tTree comm_stmts;
   {
-# line 304 "GlobalComm.puma"
-
-# line 306 "GlobalComm.puma"
+/* line 310 "GlobalComm.puma" */
    IncParNesting (stmt);
-# line 308 "GlobalComm.puma"
+/* line 312 "GlobalComm.puma" */
    comm_stmts = ExtractGlobalComm (stmt->ACF_NEW.NEW_BODY, home);
-# line 310 "GlobalComm.puma"
+/* line 314 "GlobalComm.puma" */
    DecParNesting (stmt);
   }
-  {
    return comm_stmts;
-  }
  }
 
   case kACF_REDUCTION:
-# line 325 "GlobalComm.puma"
+/* line 329 "GlobalComm.puma" */
  {
   tTree comm_stmts;
   {
-# line 327 "GlobalComm.puma"
-
-# line 329 "GlobalComm.puma"
+/* line 334 "GlobalComm.puma" */
    IncParNesting (stmt);
-# line 331 "GlobalComm.puma"
+/* line 336 "GlobalComm.puma" */
    comm_stmts = ExtractGlobalComm (stmt->ACF_REDUCTION.REDUCTION_BODY, home);
-# line 333 "GlobalComm.puma"
+/* line 338 "GlobalComm.puma" */
    DecParNesting (stmt);
   }
-  {
-   return CombineACF (comm_stmts, ExtractWriteVarComm (stmt->ACF_REDUCTION.REDUCTION_VAR, home));
-  }
+   return CombineACF (comm_stmts, ExtractWriteVarComm (stmt->ACF_REDUCTION.Line, stmt->ACF_REDUCTION.REDUCTION_VAR, home));
  }
 
   case kACF_RESIDENT:
-# line 344 "GlobalComm.puma"
+/* line 349 "GlobalComm.puma" */
  {
   tTree comm_stmts;
   {
-# line 346 "GlobalComm.puma"
-
-# line 348 "GlobalComm.puma"
+/* line 353 "GlobalComm.puma" */
    IncParNesting (stmt);
-# line 350 "GlobalComm.puma"
+/* line 355 "GlobalComm.puma" */
    comm_stmts = ExtractGlobalComm (stmt->ACF_RESIDENT.RESIDENT_BODY, home);
-# line 352 "GlobalComm.puma"
+/* line 357 "GlobalComm.puma" */
    DecParNesting (stmt);
   }
-  {
    return comm_stmts;
-  }
  }
 
   case kACF_BASIC:
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
-# line 363 "GlobalComm.puma"
-   return ExtractWriteVarComm (stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR, home);
+/* line 368 "GlobalComm.puma" */
+   return ExtractWriteVarComm (stmt->ACF_BASIC.Line, stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR, home);
 
   }
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kREDUCTION_STMT) {
-# line 370 "GlobalComm.puma"
-   return ExtractWriteVarComm (stmt->ACF_BASIC.BASIC_STMT->REDUCTION_STMT.ELEMS, home);
+/* line 375 "GlobalComm.puma" */
+   return ExtractWriteVarComm (stmt->ACF_BASIC.Line, stmt->ACF_BASIC.BASIC_STMT->REDUCTION_STMT.ELEMS, home);
 
   }
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kCALL_STMT) {
-# line 381 "GlobalComm.puma"
-   return ExtractParamComm (stmt->ACF_BASIC.BASIC_STMT->CALL_STMT.CALL_PARAMS, home);
+/* line 387 "GlobalComm.puma" */
+   return ExtractParamComm (stmt->ACF_BASIC.Line, stmt->ACF_BASIC.BASIC_STMT->CALL_STMT.CALL_PARAMS, home);
 
   }
-  if (stmt->ACF_BASIC.BASIC_STMT->Kind == kIO_STMT) {
-# line 396 "GlobalComm.puma"
+/* line 403 "GlobalComm.puma" */
   {
-# line 398 "GlobalComm.puma"
-   if (! ((stmt->ACF_BASIC.BASIC_STMT->IO_STMT.ID->PROC_OBJ.Ident == MakeIdent ("INQUIRE", 7)))) goto yyL19;
+/* line 405 "GlobalComm.puma" */
+   if (stmt->ACF_BASIC.BASIC_STMT->Kind != kIO_STMT) goto yyL19;
+/* line 407 "GlobalComm.puma" */
+   if (! ((stmt->ACF_BASIC.BASIC_STMT->IO_STMT.ID->PROC_OBJ.Ident == IsIdent ("INQUIRE")))) goto yyL19;
   }
-   return MakeBroadcast (ControlSpecBroadcast (stmt->ACF_BASIC.BASIC_STMT->IO_STMT.IO_SPECS));
+   return MakeBroadcast (stmt->ACF_BASIC.Line, ControlSpecBroadcast (stmt->ACF_BASIC.BASIC_STMT->IO_STMT.IO_SPECS));
 yyL19:;
 
-# line 403 "GlobalComm.puma"
+/* line 412 "GlobalComm.puma" */
   {
-# line 405 "GlobalComm.puma"
-   if (! ((stmt->ACF_BASIC.BASIC_STMT->IO_STMT.ID->PROC_OBJ.Ident == MakeIdent ("OPEN", 4)))) goto yyL20;
+/* line 414 "GlobalComm.puma" */
+   if (stmt->ACF_BASIC.BASIC_STMT->Kind != kIO_STMT) goto yyL20;
+/* line 416 "GlobalComm.puma" */
+   if (! ((stmt->ACF_BASIC.BASIC_STMT->IO_STMT.ID->PROC_OBJ.Ident == IsIdent ("OPEN")))) goto yyL20;
   }
-   return MakeBroadcast (ControlSpecBroadcast (stmt->ACF_BASIC.BASIC_STMT->IO_STMT.IO_SPECS));
+   return MakeBroadcast (stmt->ACF_BASIC.Line, ControlSpecBroadcast (stmt->ACF_BASIC.BASIC_STMT->IO_STMT.IO_SPECS));
 yyL20:;
 
-# line 416 "GlobalComm.puma"
+/* line 427 "GlobalComm.puma" */
   {
-# line 418 "GlobalComm.puma"
-   if (! ((stmt->ACF_BASIC.BASIC_STMT->IO_STMT.ID->PROC_OBJ.Ident == MakeIdent ("READ", 4)))) goto yyL21;
+/* line 429 "GlobalComm.puma" */
+   if (stmt->ACF_BASIC.BASIC_STMT->Kind != kIO_STMT) goto yyL21;
+/* line 431 "GlobalComm.puma" */
+   if (! ((stmt->ACF_BASIC.BASIC_STMT->IO_STMT.ID->PROC_OBJ.Ident == IsIdent ("READ")))) goto yyL21;
   }
-   return MakeBroadcast (CombineBTV (ReadSpecBroadcast (stmt->ACF_BASIC.BASIC_STMT->IO_STMT.IO_SPECS), ParameterBroadcast (stmt->ACF_BASIC.BASIC_STMT->IO_STMT.IO_ITEMS)));
+   return MakeBroadcast (stmt->ACF_BASIC.Line, CombineBTV (ReadSpecBroadcast (stmt->ACF_BASIC.BASIC_STMT->IO_STMT.IO_SPECS), ParameterBroadcast (stmt->ACF_BASIC.BASIC_STMT->IO_STMT.IO_ITEMS)));
 yyL21:;
 
-# line 430 "GlobalComm.puma"
+  if (stmt->ACF_BASIC.BASIC_STMT->Kind == kIO_STMT) {
+/* line 443 "GlobalComm.puma" */
    return NoTree;
 
   }
-# line 441 "GlobalComm.puma"
+/* line 454 "GlobalComm.puma" */
    return NoTree;
 
   }
 
-# line 446 "GlobalComm.puma"
+/* line 459 "GlobalComm.puma" */
   {
-# line 448 "GlobalComm.puma"
+/* line 461 "GlobalComm.puma" */
    failure_protocol (MODULE, "ExtractGlobalComm", stmt);
   }
    return NoTree;
@@ -454,82 +427,84 @@ yyL21:;
 
 static tTree ExtractParamComm
 # if defined __STDC__ | defined __cplusplus
-(register tTree params, pvar home)
+(register int line, register tTree params, pvar home)
 # else
-(params, home)
+(line, params, home)
+ register int line;
  register tTree params;
  pvar home;
 # endif
 {
   if (params->Kind == kBTP_LIST) {
-# line 460 "GlobalComm.puma"
-   return CombineACF (ExtractParamComm (params->BTP_LIST.Elem, home), ExtractParamComm (params->BTP_LIST.Next, home));
+/* line 473 "GlobalComm.puma" */
+   return CombineACF (ExtractParamComm (line, params->BTP_LIST.Elem, home), ExtractParamComm (line, params->BTP_LIST.Next, home));
 
   }
   if (params->Kind == kBTP_EMPTY) {
-# line 466 "GlobalComm.puma"
+/* line 479 "GlobalComm.puma" */
    return NoTree;
 
   }
   if (params->Kind == kVAR_PARAM) {
   if (params->VAR_PARAM.V->Kind == kADDR) {
-# line 471 "GlobalComm.puma"
+/* line 484 "GlobalComm.puma" */
    return NoTree;
 
   }
-# line 476 "GlobalComm.puma"
+/* line 489 "GlobalComm.puma" */
   {
-# line 478 "GlobalComm.puma"
+/* line 491 "GlobalComm.puma" */
    if (! ((params->VAR_PARAM.intent == IntentIn))) goto yyL4;
   }
    return NoTree;
 yyL4:;
 
-# line 483 "GlobalComm.puma"
-   return ExtractWriteVarComm (params->VAR_PARAM.V, home);
+/* line 496 "GlobalComm.puma" */
+   return ExtractWriteVarComm (line, params->VAR_PARAM.V, home);
 
   }
-# line 488 "GlobalComm.puma"
+/* line 501 "GlobalComm.puma" */
    return NoTree;
 
 }
 
 static tTree ExtractWriteVarComm
 # if defined __STDC__ | defined __cplusplus
-(register tTree var, pvar home)
+(register int line, register tTree var, pvar home)
 # else
-(var, home)
+(line, var, home)
+ register int line;
  register tTree var;
  pvar home;
 # endif
 {
   if (var->Kind == kBTV_LIST) {
-# line 501 "GlobalComm.puma"
-   return CombineACF (ExtractWriteVarComm (var->BTV_LIST.Elem, home), ExtractWriteVarComm (var->BTV_LIST.Next, home));
+/* line 516 "GlobalComm.puma" */
+   return CombineACF (ExtractWriteVarComm (line, var->BTV_LIST.Elem, home), ExtractWriteVarComm (line, var->BTV_LIST.Next, home));
 
   }
   if (var->Kind == kBTV_EMPTY) {
-# line 507 "GlobalComm.puma"
+/* line 522 "GlobalComm.puma" */
    return NoTree;
 
   }
-# line 518 "GlobalComm.puma"
+/* line 533 "GlobalComm.puma" */
   {
-# line 520 "GlobalComm.puma"
+/* line 535 "GlobalComm.puma" */
    tree_protocol ("broadcast variable is : ", var);
-# line 521 "GlobalComm.puma"
+/* line 536 "GlobalComm.puma" */
    tree_protocol ("broadcast home is : ", PrintableDescriptorVar (home));
-# line 523 "GlobalComm.puma"
+/* line 538 "GlobalComm.puma" */
    goto yyL3;
   }
 yyL3:;
 
-# line 540 "GlobalComm.puma"
+/* line 541 "GlobalComm.puma" */
   {
-# line 542 "GlobalComm.puma"
+/* line 543 "GlobalComm.puma" */
    if (! ((IsReplicatedDescriptor (home)))) goto yyL4;
   {
-# line 544 "GlobalComm.puma"
+/* line 545 "GlobalComm.puma" */
    tree_protocol ("no broadcast, replicated home : ", var);
   }
   }
@@ -537,63 +512,59 @@ yyL3:;
 yyL4:;
 
   if (var->Kind == kREMOTE_VAR) {
-# line 555 "GlobalComm.puma"
+/* line 556 "GlobalComm.puma" */
    return NoTree;
 
   }
-# line 566 "GlobalComm.puma"
+/* line 567 "GlobalComm.puma" */
   {
-# line 568 "GlobalComm.puma"
+/* line 569 "GlobalComm.puma" */
    if (! ((IsLocalVarWrite (var, home)))) goto yyL6;
   {
-# line 570 "GlobalComm.puma"
+/* line 571 "GlobalComm.puma" */
    tree_protocol ("no broadcast, local write is : ", var);
   }
   }
    return NoTree;
 yyL6:;
 
-# line 575 "GlobalComm.puma"
+/* line 576 "GlobalComm.puma" */
   {
-# line 577 "GlobalComm.puma"
+/* line 578 "GlobalComm.puma" */
    if (! ((IsIndShadowVariable (var, home)))) goto yyL7;
   {
-# line 579 "GlobalComm.puma"
+/* line 580 "GlobalComm.puma" */
    tree_protocol ("no broadcast, ind shadow is : ", var);
   }
   }
    return NoTree;
 yyL7:;
 
-# line 584 "GlobalComm.puma"
+/* line 585 "GlobalComm.puma" */
  {
   var_descriptor vard;
   {
-# line 586 "GlobalComm.puma"
-
-# line 588 "GlobalComm.puma"
+/* line 589 "GlobalComm.puma" */
    SetVarDescriptor (var, & vard);
-# line 589 "GlobalComm.puma"
+/* line 590 "GlobalComm.puma" */
    if (! ((NoReplicationDescriptor (& vard)))) goto yyL8;
   {
-# line 591 "GlobalComm.puma"
+/* line 592 "GlobalComm.puma" */
    tree_protocol ("no broadcast, single incarnation: ", var);
   }
   }
-  {
    return NoTree;
-  }
  }
 yyL8:;
 
-# line 603 "GlobalComm.puma"
+/* line 604 "GlobalComm.puma" */
  {
-  tTree bc_vars;
+  tTree new_stmt;
   {
-# line 605 "GlobalComm.puma"
+/* line 606 "GlobalComm.puma" */
    if (! ((IsLocalVarRead (var, home)))) goto yyL9;
   {
-# line 607 "GlobalComm.puma"
+/* line 608 "GlobalComm.puma" */
 
 #ifdef DEBUG
      printf ("broadcast needed for : ");
@@ -603,25 +574,25 @@ yyL8:;
      printf ("\n");
 #endif 
    
-# line 617 "GlobalComm.puma"
-
-# line 619 "GlobalComm.puma"
-   bc_vars = mBTV_LIST (CopyTree (var), mBTV_EMPTY ());
+/* line 620 "GlobalComm.puma" */
+   new_stmt = mBTV_LIST (CopyTree (var), mBTV_EMPTY ());
+/* line 625 "GlobalComm.puma" */
+   new_stmt = mACF_BASIC (mBROADCAST_STMT (new_stmt, NoTree, mON_EMPTY ()));
+/* line 627 "GlobalComm.puma" */
+   LineACFNode (new_stmt, line);
   }
   }
-  {
-   return mACF_BASIC (mBROADCAST_STMT (bc_vars, NoTree, mON_EMPTY ()));
-  }
+   return new_stmt;
  }
 yyL9:;
 
-# line 627 "GlobalComm.puma"
+/* line 632 "GlobalComm.puma" */
   {
-# line 629 "GlobalComm.puma"
+/* line 634 "GlobalComm.puma" */
    error_protocol ("broadcast on non-local variables");
-# line 630 "GlobalComm.puma"
+/* line 635 "GlobalComm.puma" */
    tree_protocol ("illegal variable is : ", var);
-# line 631 "GlobalComm.puma"
+/* line 636 "GlobalComm.puma" */
    tree_protocol ("broadcast home is : ", PrintableDescriptorVar (home));
   }
    return NoTree;
@@ -638,12 +609,13 @@ static void ApplyHomeChange
  pvar outer_home;
 # endif
 {
-# line 646 "GlobalComm.puma"
+ yyRecursion:
+/* line 651 "GlobalComm.puma" */
   {
-# line 648 "GlobalComm.puma"
+/* line 653 "GlobalComm.puma" */
    if (! ((stmts == NoTree))) goto yyL1;
   {
-# line 650 "GlobalComm.puma"
+/* line 655 "GlobalComm.puma" */
 
 #ifdef DEBUG
     printf ("call of ApllyHomeChange\n");
@@ -661,26 +633,26 @@ static void ApplyHomeChange
 yyL1:;
 
   if (stmts->Kind == kACF_LIST) {
-# line 663 "GlobalComm.puma"
+/* line 668 "GlobalComm.puma" */
   {
-# line 665 "GlobalComm.puma"
+/* line 670 "GlobalComm.puma" */
    ApplyHomeChange (stmts->ACF_LIST.Next, inner_home, outer_home);
-# line 666 "GlobalComm.puma"
-   ApplyHomeChange (stmts->ACF_LIST.Elem, inner_home, outer_home);
+/* line 671 "GlobalComm.puma" */
+   stmts = stmts->ACF_LIST.Elem;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (stmts->Kind == kACF_EMPTY) {
-# line 669 "GlobalComm.puma"
+/* line 674 "GlobalComm.puma" */
    return;
 
   }
   if (stmts->Kind == kACF_BASIC) {
   if (stmts->ACF_BASIC.BASIC_STMT->Kind == kBROADCAST_STMT) {
-# line 672 "GlobalComm.puma"
+/* line 677 "GlobalComm.puma" */
   {
-# line 674 "GlobalComm.puma"
+/* line 679 "GlobalComm.puma" */
    BetterOnSelection (stmts->ACF_BASIC.BASIC_STMT, inner_home, outer_home);
   }
    return;
@@ -692,14 +664,26 @@ yyL1:;
 
 static tTree MakeBroadcast
 # if defined __STDC__ | defined __cplusplus
-(register tTree bc_vars)
+(register int line, register tTree bc_vars)
 # else
-(bc_vars)
+(line, bc_vars)
+ register int line;
  register tTree bc_vars;
 # endif
 {
-# line 684 "GlobalComm.puma"
-   return mACF_BASIC (mBROADCAST_STMT (CompleteBTVs (bc_vars), NoTree, mON_EMPTY ()));
+/* line 692 "GlobalComm.puma" */
+ {
+  tTree new_stmt;
+  {
+/* line 696 "GlobalComm.puma" */
+   new_stmt = mBROADCAST_STMT (CompleteBTVs (bc_vars), NoTree, mON_EMPTY ());
+/* line 697 "GlobalComm.puma" */
+   new_stmt = mACF_BASIC (new_stmt);
+/* line 699 "GlobalComm.puma" */
+   LineACFNode (new_stmt, line);
+  }
+   return new_stmt;
+ }
 
 }
 
@@ -712,62 +696,62 @@ static tTree ParameterBroadcast
 # endif
 {
   if (p->Kind == kBTP_LIST) {
-# line 701 "GlobalComm.puma"
+/* line 715 "GlobalComm.puma" */
    return CombineBTV (ParameterBroadcast (p->BTP_LIST.Elem), ParameterBroadcast (p->BTP_LIST.Next));
 
   }
   if (p->Kind == kBTP_EMPTY) {
-# line 707 "GlobalComm.puma"
+/* line 721 "GlobalComm.puma" */
    return NoTree;
 
   }
   if (p->Kind == kVAR_PARAM) {
-# line 712 "GlobalComm.puma"
+/* line 726 "GlobalComm.puma" */
   {
-# line 714 "GlobalComm.puma"
+/* line 728 "GlobalComm.puma" */
    if (! ((IsOnlyReadParam (p)))) goto yyL3;
   }
    return NoTree;
 yyL3:;
 
-# line 719 "GlobalComm.puma"
+/* line 733 "GlobalComm.puma" */
   {
-# line 721 "GlobalComm.puma"
+/* line 735 "GlobalComm.puma" */
    if (! ((TreeWriteDistribution (p->VAR_PARAM.V) == 2))) goto yyL4;
   }
    return NoTree;
 yyL4:;
 
-# line 725 "GlobalComm.puma"
+/* line 739 "GlobalComm.puma" */
   {
-# line 727 "GlobalComm.puma"
+/* line 741 "GlobalComm.puma" */
    if (! ((TreeWriteDistribution (p->VAR_PARAM.V) == 0))) goto yyL5;
   }
    return CopyTree (p->VAR_PARAM.V);
 yyL5:;
 
-# line 731 "GlobalComm.puma"
+/* line 745 "GlobalComm.puma" */
    return NoTree;
 
   }
   if (p->Kind == kNO_PARAM) {
-# line 736 "GlobalComm.puma"
+/* line 750 "GlobalComm.puma" */
    return NoTree;
 
   }
   if (p->Kind == kFUNC_PARAM) {
-# line 740 "GlobalComm.puma"
+/* line 754 "GlobalComm.puma" */
    return NoTree;
 
   }
   if (p->Kind == kPROC_PARAM) {
-# line 744 "GlobalComm.puma"
+/* line 758 "GlobalComm.puma" */
    return NoTree;
 
   }
-# line 748 "GlobalComm.puma"
+/* line 762 "GlobalComm.puma" */
   {
-# line 749 "GlobalComm.puma"
+/* line 763 "GlobalComm.puma" */
    failure_protocol (MODULE, "ParameterBroadcast", p);
   }
    return NoTree;
@@ -783,44 +767,44 @@ static tTree ControlSpecBroadcast
 # endif
 {
   if (p->Kind == kBTP_LIST) {
-# line 763 "GlobalComm.puma"
+/* line 777 "GlobalComm.puma" */
    return CombineBTV (ControlSpecBroadcast (p->BTP_LIST.Elem), ControlSpecBroadcast (p->BTP_LIST.Next));
 
   }
   if (p->Kind == kBTP_EMPTY) {
-# line 768 "GlobalComm.puma"
+/* line 782 "GlobalComm.puma" */
    return NoTree;
 
   }
   if (p->Kind == kNAMED_PARAM) {
-# line 772 "GlobalComm.puma"
+/* line 786 "GlobalComm.puma" */
   {
-# line 773 "GlobalComm.puma"
+/* line 787 "GlobalComm.puma" */
    if (! ((p->NAMED_PARAM.Name == MakeIdent ("EXISTS", 6)))) goto yyL3;
   }
    return ParameterBroadcast (p->NAMED_PARAM.VAL);
 yyL3:;
 
-# line 777 "GlobalComm.puma"
+/* line 791 "GlobalComm.puma" */
   {
-# line 778 "GlobalComm.puma"
+/* line 792 "GlobalComm.puma" */
    if (! ((p->NAMED_PARAM.Name == MakeIdent ("IOSTAT", 6)))) goto yyL4;
   }
    return ParameterBroadcast (p->NAMED_PARAM.VAL);
 yyL4:;
 
-# line 782 "GlobalComm.puma"
+/* line 796 "GlobalComm.puma" */
    return NoTree;
 
   }
   if (p->Kind == kVAR_PARAM) {
-# line 786 "GlobalComm.puma"
+/* line 800 "GlobalComm.puma" */
    return NoTree;
 
   }
-# line 790 "GlobalComm.puma"
+/* line 804 "GlobalComm.puma" */
   {
-# line 791 "GlobalComm.puma"
+/* line 805 "GlobalComm.puma" */
    failure_protocol (MODULE, "ControlSpecBroadcast", p);
   }
    return NoTree;
@@ -835,33 +819,35 @@ static tTree ReadSpecBroadcast
  register tTree p;
 # endif
 {
+ yyRecursion:
   if (p->Kind == kBTP_LIST) {
-# line 807 "GlobalComm.puma"
+/* line 821 "GlobalComm.puma" */
    return CombineBTV (ReadSpecBroadcast (p->BTP_LIST.Elem), ReadSpecBroadcast (p->BTP_LIST.Next));
 
   }
   if (p->Kind == kBTP_EMPTY) {
-# line 812 "GlobalComm.puma"
+/* line 826 "GlobalComm.puma" */
    return NoTree;
 
   }
   if (p->Kind == kNAMED_PARAM) {
-# line 816 "GlobalComm.puma"
+/* line 830 "GlobalComm.puma" */
   {
-# line 817 "GlobalComm.puma"
+/* line 831 "GlobalComm.puma" */
    if (! ((p->NAMED_PARAM.Name == MakeIdent ("IOSTAT", 6)))) goto yyL3;
   }
    return ParameterBroadcast (p->NAMED_PARAM.VAL);
 yyL3:;
 
-# line 821 "GlobalComm.puma"
-   return ReadSpecBroadcast (p->NAMED_PARAM.VAL);
+/* line 835 "GlobalComm.puma" */
+   p = p->NAMED_PARAM.VAL;
+   goto yyRecursion;
 
   }
   if (p->Kind == kVAR_PARAM) {
-# line 825 "GlobalComm.puma"
+/* line 839 "GlobalComm.puma" */
   {
-# line 826 "GlobalComm.puma"
+/* line 840 "GlobalComm.puma" */
    if (! ((GetCurrentModel () == HPF_SERIAL))) goto yyL5;
   }
    return NoTree;
@@ -870,19 +856,19 @@ yyL5:;
   if (p->VAR_PARAM.V->Kind == kUSED_VAR) {
   if (p->VAR_PARAM.V->USED_VAR.VARNAME->VAR_OBJ.Object->Kind == kNameListObject) {
   if (p->VAR_PARAM.V->USED_VAR.VARNAME->VAR_OBJ.Object->NameListObject.decl->Kind == kNAMELIST_DECL) {
-# line 830 "GlobalComm.puma"
+/* line 844 "GlobalComm.puma" */
    return NameListVars (p->VAR_PARAM.V->USED_VAR.VARNAME->VAR_OBJ.Object->NameListObject.decl->NAMELIST_DECL.IDS);
 
   }
   }
   }
-# line 838 "GlobalComm.puma"
+/* line 852 "GlobalComm.puma" */
    return NoTree;
 
   }
-# line 842 "GlobalComm.puma"
+/* line 856 "GlobalComm.puma" */
   {
-# line 843 "GlobalComm.puma"
+/* line 857 "GlobalComm.puma" */
    failure_protocol (MODULE, "ReadSpecBroadcast", p);
   }
    return NoTree;
@@ -898,59 +884,53 @@ static tTree NameListVars
 # endif
 {
   if (names->Kind == kDECL_LIST) {
-# line 855 "GlobalComm.puma"
+/* line 869 "GlobalComm.puma" */
    return CombineBTV (NameListVars (names->DECL_LIST.Elem), NameListVars (names->DECL_LIST.Next));
 
   }
   if (names->Kind == kDECL_EMPTY) {
-# line 860 "GlobalComm.puma"
+/* line 874 "GlobalComm.puma" */
    return NoTree;
 
   }
   if (names->Kind == kVAR_DECL) {
-# line 864 "GlobalComm.puma"
+/* line 878 "GlobalComm.puma" */
   {
-# line 866 "GlobalComm.puma"
+/* line 880 "GlobalComm.puma" */
    if (! ((VarDistribution (GetLocalObject (names->VAR_DECL.Ident)) == - 1))) goto yyL3;
   }
    return NoTree;
 yyL3:;
 
-# line 870 "GlobalComm.puma"
+/* line 884 "GlobalComm.puma" */
  {
   tTree newvar;
   tDefinitions Obj;
   {
-# line 872 "GlobalComm.puma"
-
-# line 873 "GlobalComm.puma"
-
-# line 875 "GlobalComm.puma"
+/* line 889 "GlobalComm.puma" */
    Obj = GetLocalObject (names->VAR_DECL.Ident);
-# line 877 "GlobalComm.puma"
+/* line 891 "GlobalComm.puma" */
    if (! ((VarDistribution (Obj) == 0))) goto yyL4;
   {
-# line 879 "GlobalComm.puma"
+/* line 893 "GlobalComm.puma" */
  newvar = mVAR_OBJ (names->VAR_DECL.Line, names->VAR_DECL.Ident);
       newvar->VAR_OBJ.Object = Obj;
       newvar = mUSED_VAR (newvar);
     
   }
   }
-  {
    return newvar;
-  }
  }
 yyL4:;
 
-# line 887 "GlobalComm.puma"
+/* line 901 "GlobalComm.puma" */
   {
-# line 889 "GlobalComm.puma"
+/* line 903 "GlobalComm.puma" */
    if (! ((VarDistribution (GetLocalObject (names->VAR_DECL.Ident)) == 1))) goto yyL5;
   {
-# line 891 "GlobalComm.puma"
+/* line 905 "GlobalComm.puma" */
    error_protocol ("distributed variable in namelist");
-# line 892 "GlobalComm.puma"
+/* line 906 "GlobalComm.puma" */
    tree_protocol ("illegal variable is ", names);
   }
   }
@@ -958,9 +938,9 @@ yyL4:;
 yyL5:;
 
   }
-# line 896 "GlobalComm.puma"
+/* line 910 "GlobalComm.puma" */
   {
-# line 897 "GlobalComm.puma"
+/* line 911 "GlobalComm.puma" */
    failure_protocol (MODULE, "NameListVars", names);
   }
    return NoTree;
@@ -978,49 +958,50 @@ static void VectorizeGlobalComm
  register tTree slice;
 # endif
 {
-# line 917 "GlobalComm.puma"
+ yyRecursion:
+/* line 931 "GlobalComm.puma" */
   {
-# line 919 "GlobalComm.puma"
+/* line 933 "GlobalComm.puma" */
    if (! ((stmts == NoTree))) goto yyL1;
   }
    return;
 yyL1:;
 
   if (stmts->Kind == kACF_LIST) {
-# line 922 "GlobalComm.puma"
+/* line 936 "GlobalComm.puma" */
   {
-# line 924 "GlobalComm.puma"
+/* line 938 "GlobalComm.puma" */
    set_protocol_stmt (stmts->ACF_LIST.Elem);
-# line 925 "GlobalComm.puma"
+/* line 939 "GlobalComm.puma" */
    VectorizeGlobalComm (stmts->ACF_LIST.Elem, home, id, slice);
-# line 926 "GlobalComm.puma"
-   VectorizeGlobalComm (stmts->ACF_LIST.Next, home, id, slice);
+/* line 940 "GlobalComm.puma" */
+   stmts = stmts->ACF_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (stmts->Kind == kACF_EMPTY) {
-# line 929 "GlobalComm.puma"
+/* line 943 "GlobalComm.puma" */
    return;
 
   }
   if (stmts->Kind == kACF_BASIC) {
   if (stmts->ACF_BASIC.BASIC_STMT->Kind == kBROADCAST_STMT) {
-# line 932 "GlobalComm.puma"
+/* line 946 "GlobalComm.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   {
-# line 934 "GlobalComm.puma"
+/* line 948 "GlobalComm.puma" */
    tree_protocol ("vectorize bc : ", stmts->ACF_BASIC.BASIC_STMT);
-# line 935 "GlobalComm.puma"
+/* line 949 "GlobalComm.puma" */
    tree_protocol ("id           : ", id);
-# line 936 "GlobalComm.puma"
+/* line 950 "GlobalComm.puma" */
    tree_protocol ("slice        : ", slice);
-# line 937 "GlobalComm.puma"
+/* line 951 "GlobalComm.puma" */
    tree_protocol ("home         : ", PrintableDescriptorVar (home));
-# line 939 "GlobalComm.puma"
+/* line 953 "GlobalComm.puma" */
    VectorizeVarList (stmts->ACF_BASIC.BASIC_STMT->BROADCAST_STMT.ELEMS, id, slice, & yyV1);
-# line 941 "GlobalComm.puma"
+/* line 955 "GlobalComm.puma" */
  if (yyV1 != 1)
 
         { 
@@ -1034,9 +1015,9 @@ yyL1:;
 
   }
   }
-# line 950 "GlobalComm.puma"
+/* line 964 "GlobalComm.puma" */
   {
-# line 952 "GlobalComm.puma"
+/* line 966 "GlobalComm.puma" */
    failure_protocol (MODULE, "VectorizeGlobalComm", stmts);
   }
    return;
@@ -1046,32 +1027,30 @@ yyL1:;
 
 static void VectorizeVarList
 # if defined __STDC__ | defined __cplusplus
-(register tTree vars, register tTree id, register tTree slice, register bool * yyP1)
+(register tTree vars, register tTree id, register tTree slice, register rbool * yyP1)
 # else
 (vars, id, slice, yyP1)
  register tTree vars;
  register tTree id;
  register tTree slice;
- register bool * yyP1;
+ register rbool * yyP1;
 # endif
 {
   if (vars->Kind == kBTV_EMPTY) {
-# line 963 "GlobalComm.puma"
+/* line 977 "GlobalComm.puma" */
    * yyP1 = 0;
    return;
 
   }
   if (vars->Kind == kBTV_LIST) {
-# line 966 "GlobalComm.puma"
+/* line 980 "GlobalComm.puma" */
  {
   int done;
-  bool yyV1;
+  rbool yyV1;
   {
-# line 968 "GlobalComm.puma"
-
-# line 970 "GlobalComm.puma"
+/* line 984 "GlobalComm.puma" */
    MakeVectorVar (vars->BTV_LIST.Elem, id, slice, & done);
-# line 977 "GlobalComm.puma"
+/* line 991 "GlobalComm.puma" */
  if (done == -1)
 
          { error_protocol ("could not vectorize global communication");
@@ -1079,7 +1058,7 @@ static void VectorizeVarList
            tree_protocol  ("loop id :          ", id);
          }
     
-# line 985 "GlobalComm.puma"
+/* line 999 "GlobalComm.puma" */
    VectorizeVarList (vars->BTV_LIST.Next, id, slice, & yyV1);
   }
    * yyP1 = done;
@@ -1087,9 +1066,9 @@ static void VectorizeVarList
  }
 
   }
-# line 988 "GlobalComm.puma"
+/* line 1002 "GlobalComm.puma" */
   {
-# line 990 "GlobalComm.puma"
+/* line 1004 "GlobalComm.puma" */
    failure_protocol (MODULE, "VectorizeVarList", vars);
   }
    * yyP1 = 0;
@@ -1108,48 +1087,49 @@ static void SummarizeGlobalComm
  register tTree loop;
 # endif
 {
-# line 1002 "GlobalComm.puma"
+ yyRecursion:
+/* line 1016 "GlobalComm.puma" */
   {
-# line 1004 "GlobalComm.puma"
+/* line 1018 "GlobalComm.puma" */
    if (! ((stmts == NoTree))) goto yyL1;
   }
    return;
 yyL1:;
 
   if (stmts->Kind == kACF_LIST) {
-# line 1007 "GlobalComm.puma"
+/* line 1021 "GlobalComm.puma" */
   {
-# line 1009 "GlobalComm.puma"
+/* line 1023 "GlobalComm.puma" */
    set_protocol_stmt (stmts->ACF_LIST.Elem);
-# line 1011 "GlobalComm.puma"
+/* line 1025 "GlobalComm.puma" */
    SummarizeGlobalComm (stmts->ACF_LIST.Elem, home, loop);
-# line 1012 "GlobalComm.puma"
-   SummarizeGlobalComm (stmts->ACF_LIST.Next, home, loop);
+/* line 1026 "GlobalComm.puma" */
+   stmts = stmts->ACF_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (stmts->Kind == kACF_EMPTY) {
-# line 1015 "GlobalComm.puma"
+/* line 1029 "GlobalComm.puma" */
    return;
 
   }
   if (stmts->Kind == kACF_BASIC) {
   if (stmts->ACF_BASIC.BASIC_STMT->Kind == kBROADCAST_STMT) {
-# line 1018 "GlobalComm.puma"
+/* line 1032 "GlobalComm.puma" */
   {
-# line 1020 "GlobalComm.puma"
+/* line 1034 "GlobalComm.puma" */
    error_protocol ("cannot summarize global broadcast in serial loop");
-# line 1021 "GlobalComm.puma"
+/* line 1035 "GlobalComm.puma" */
    tree_protocol ("serial loop : \n", loop);
   }
    return;
 
   }
   }
-# line 1024 "GlobalComm.puma"
+/* line 1038 "GlobalComm.puma" */
   {
-# line 1026 "GlobalComm.puma"
+/* line 1040 "GlobalComm.puma" */
    failure_protocol (MODULE, "SummarizeGlobalComm", stmts);
   }
    return;
@@ -1157,10 +1137,10 @@ yyL1:;
 ;
 }
 
-void BeginGlobalComm ()
+void BeginGlobalComm ARGS ((void))
 {
 }
 
-void CloseGlobalComm ()
+void CloseGlobalComm ARGS ((void))
 {
 }
