@@ -1,6 +1,13 @@
 # include "HomeDescriptor.h"
-# include "yyHomeDescriptor.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,34 +15,31 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 47 "HomeDescriptor.puma"
+/* line 51 "HomeDescriptor.puma" */
 
 
-# undef DEBUG
+#undef DEBUG
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 # include "protocol.h"
 
 # include "DefTable.h"
@@ -50,16 +54,31 @@
 # include "Distributions.h"
 # include "TreeOps.h"
 # include "Descriptor.h"
+# include "Unparse.h"
 
 # define MODULE "HomeDescriptor"
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* HomeDescriptor_Exit) () = yyExit;
+# include "yyHomeDescriptor.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* HomeDescriptor_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -68,31 +87,35 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module HomeDescriptor, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module HomeDescriptor, routine %s failed\n",
+  yyFunction);
  HomeDescriptor_Exit ();
 }
 
-void MakeNoDescriptor ARGS((pvar vard));
-void MakeReplicatedDescriptor ARGS((pvar vard));
-void MakeHostDescriptor ARGS((pvar vard));
-void MakeClauseDescriptor ARGS((tTree home_clause, pvar vard));
-void MakeParLoopNestDescriptor ARGS((pvar vard));
-static void AddParallelDimension ARGS((pvar vard, tTree loop));
-bool IsNoDescriptor ARGS((pvar vard));
-bool IsReplicatedDescriptor ARGS((pvar vard));
-bool IsHostDescriptor ARGS((pvar vard));
-bool IsProcDescriptor ARGS((pvar vard));
-bool NoReplicationDescriptor ARGS((pvar vard));
-bool IsFullLoopDescriptor ARGS((pvar vard));
-bool IsParallelLoopDescriptor ARGS((pvar home, tTree loop));
-bool IsParallelHomeDescriptor ARGS((pvar d1));
-static bool DescriptorLinearDependency ARGS((pvar d, tTree loop));
-static bool IsLinearDependent ARGS((tTree exp, tTree id));
-static bool ResolveIsVariable ARGS((tTree var));
-void UnionHomeDescriptor ARGS((pvar vard1, pvar vard2));
-static void UnionTopDims ARGS((pvar vard1, pvar vard2, int top_dim));
-void MakeSerialDescriptor ARGS((pvar home, bool * yyP1));
-bool IsSerialDescriptor ARGS((pvar home));
+void MakeNoDescriptor ARGS ((pvar vard));
+void MakeReplicatedDescriptor ARGS ((pvar vard));
+void MakeHostDescriptor ARGS ((pvar vard));
+void MakeClauseDescriptor ARGS ((tTree home_clause, pvar vard));
+void MakeParLoopNestDescriptor ARGS ((pvar vard));
+static void AddParallelDimension ARGS ((pvar vard, tTree loop));
+rbool IsNoDescriptor ARGS ((pvar vard));
+rbool IsReplicatedDescriptor ARGS ((pvar vard));
+rbool IsHostDescriptor ARGS ((pvar vard));
+rbool IsProcDescriptor ARGS ((pvar vard));
+rbool IsFictiveParDescriptor ARGS ((pvar vard));
+rbool NoReplicationDescriptor ARGS ((pvar vard));
+rbool IsFullLoopDescriptor ARGS ((pvar vard));
+rbool IsParallelLoopDescriptor ARGS ((pvar home, tTree loop));
+rbool IsParallelHomeDescriptor ARGS ((pvar d1));
+static rbool DescriptorLinearDependency ARGS ((pvar d, tTree loop));
+static rbool IsLinearDependent ARGS ((tTree exp, tTree id));
+static rbool ResolveIsVariable ARGS ((tTree var));
+rbool IsLoopInvariantDescriptor ARGS ((pvar vd, tTree loop));
+static rbool AnyDependency ARGS ((tTree id, tTree exp));
+void UnionHomeDescriptor ARGS ((pvar vard1, pvar vard2));
+static void UnionTopDims ARGS ((pvar vard1, pvar vard2, int top_dim));
+void MakeSerialDescriptor ARGS ((pvar home, rbool * yyP1));
+rbool IsSerialDescriptor ARGS ((pvar home));
 
 void MakeNoDescriptor
 # if defined __STDC__ | defined __cplusplus
@@ -102,9 +125,9 @@ void MakeNoDescriptor
  pvar vard;
 # endif
 {
-# line 83 "HomeDescriptor.puma"
+/* line 88 "HomeDescriptor.puma" */
   {
-# line 85 "HomeDescriptor.puma"
+/* line 90 "HomeDescriptor.puma" */
  vard->formal_rank    = -1;
      vard->actual_rank    = 0;
      vard->var_tree       = NoTree;
@@ -117,7 +140,7 @@ void MakeNoDescriptor
      vard->topology_rank  = 0;
      vard->type_kind      = kDUMMY_TYPE;
      vard->type_size      = 0;
-     vard->expanded       = false;
+     vard->expanded       = rfalse;
    
   }
    return;
@@ -133,9 +156,9 @@ void MakeReplicatedDescriptor
  pvar vard;
 # endif
 {
-# line 111 "HomeDescriptor.puma"
+/* line 116 "HomeDescriptor.puma" */
   {
-# line 113 "HomeDescriptor.puma"
+/* line 118 "HomeDescriptor.puma" */
  vard->formal_rank    = 0;
      vard->actual_rank    = 0;
      vard->var_tree       = NoTree;
@@ -148,7 +171,7 @@ void MakeReplicatedDescriptor
      vard->reach_info     = NoTree;
      vard->type_kind      = kDUMMY_TYPE;
      vard->type_size      = 0;
-     vard->expanded       = false;
+     vard->expanded       = rfalse;
    
   }
    return;
@@ -164,9 +187,9 @@ void MakeHostDescriptor
  pvar vard;
 # endif
 {
-# line 139 "HomeDescriptor.puma"
+/* line 144 "HomeDescriptor.puma" */
   {
-# line 141 "HomeDescriptor.puma"
+/* line 146 "HomeDescriptor.puma" */
  vard->formal_rank    = 0;
      vard->actual_rank    = 0;
      vard->shared         = 0;
@@ -179,7 +202,7 @@ void MakeHostDescriptor
      vard->reach_info     = NoTree;
      vard->type_kind      = kDUMMY_TYPE;
      vard->type_size      = 0;
-     vard->expanded       = false;
+     vard->expanded       = rfalse;
    
   }
    return;
@@ -197,44 +220,53 @@ void MakeClauseDescriptor
 # endif
 {
   if (home_clause->Kind == kON_VAR_CLAUSE) {
-# line 168 "HomeDescriptor.puma"
+  if (home_clause->ON_VAR_CLAUSE.ON_VAR->Kind == kDUMMY_VAR) {
+/* line 175 "HomeDescriptor.puma" */
   {
-# line 170 "HomeDescriptor.puma"
+/* line 177 "HomeDescriptor.puma" */
+   MakeReplicatedDescriptor (vard);
+  }
+   return;
+
+  }
+/* line 180 "HomeDescriptor.puma" */
+  {
+/* line 182 "HomeDescriptor.puma" */
    SetVarDescriptor (home_clause->ON_VAR_CLAUSE.ON_VAR, vard);
   }
    return;
 
   }
   if (home_clause->Kind == kON_PROC_CLAUSE) {
-# line 173 "HomeDescriptor.puma"
+/* line 185 "HomeDescriptor.puma" */
   {
-# line 175 "HomeDescriptor.puma"
+/* line 187 "HomeDescriptor.puma" */
    SetProcDescriptor (home_clause->ON_PROC_CLAUSE.ON_PROC, vard);
   }
    return;
 
   }
   if (home_clause->Kind == kON_HOST_CLAUSE) {
-# line 178 "HomeDescriptor.puma"
+/* line 190 "HomeDescriptor.puma" */
   {
-# line 180 "HomeDescriptor.puma"
+/* line 192 "HomeDescriptor.puma" */
    MakeHostDescriptor (vard);
   }
    return;
 
   }
   if (home_clause->Kind == kON_ALL_CLAUSE) {
-# line 183 "HomeDescriptor.puma"
+/* line 195 "HomeDescriptor.puma" */
   {
-# line 185 "HomeDescriptor.puma"
+/* line 197 "HomeDescriptor.puma" */
    MakeReplicatedDescriptor (vard);
   }
    return;
 
   }
-# line 188 "HomeDescriptor.puma"
+/* line 200 "HomeDescriptor.puma" */
   {
-# line 190 "HomeDescriptor.puma"
+/* line 202 "HomeDescriptor.puma" */
    failure_protocol (MODULE, "MakeClauseDescriptor", home_clause);
   }
    return;
@@ -250,21 +282,21 @@ void MakeParLoopNestDescriptor
  pvar vard;
 # endif
 {
-# line 213 "HomeDescriptor.puma"
+/* line 225 "HomeDescriptor.puma" */
   {
-# line 215 "HomeDescriptor.puma"
+/* line 227 "HomeDescriptor.puma" */
    if (! ((OuterLoops () <= 0))) goto yyL1;
   {
-# line 217 "HomeDescriptor.puma"
+/* line 229 "HomeDescriptor.puma" */
    MakeNoDescriptor (vard);
   }
   }
    return;
 yyL1:;
 
-# line 220 "HomeDescriptor.puma"
+/* line 232 "HomeDescriptor.puma" */
   {
-# line 222 "HomeDescriptor.puma"
+/* line 234 "HomeDescriptor.puma" */
  int i;
 
      MakeReplicatedDescriptor (vard);
@@ -289,15 +321,15 @@ static void AddParallelDimension
  register tTree loop;
 # endif
 {
-# line 235 "HomeDescriptor.puma"
+/* line 247 "HomeDescriptor.puma" */
   {
-# line 237 "HomeDescriptor.puma"
+/* line 249 "HomeDescriptor.puma" */
    if (! ((IsParallelLoop (loop)))) goto yyL1;
   {
-# line 238 "HomeDescriptor.puma"
+/* line 250 "HomeDescriptor.puma" */
    if (! ((IsExtractableRange (GetLoopSlice (loop))))) goto yyL1;
   {
-# line 240 "HomeDescriptor.puma"
+/* line 252 "HomeDescriptor.puma" */
    AddDescriptorDimension (vard, GetLoopId (loop), GetLoopSlice (loop), kBLOCK_DIM);
   }
   }
@@ -308,7 +340,7 @@ yyL1:;
 ;
 }
 
-bool IsNoDescriptor
+rbool IsNoDescriptor
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -316,18 +348,18 @@ bool IsNoDescriptor
  pvar vard;
 # endif
 {
-# line 257 "HomeDescriptor.puma"
+/* line 270 "HomeDescriptor.puma" */
   {
-# line 258 "HomeDescriptor.puma"
+/* line 271 "HomeDescriptor.puma" */
    if (! ((vard -> formal_rank == - 1))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-bool IsReplicatedDescriptor
+rbool IsReplicatedDescriptor
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -335,18 +367,18 @@ bool IsReplicatedDescriptor
  pvar vard;
 # endif
 {
-# line 263 "HomeDescriptor.puma"
+/* line 276 "HomeDescriptor.puma" */
   {
-# line 265 "HomeDescriptor.puma"
+/* line 278 "HomeDescriptor.puma" */
    if (! ((SameTopologyObject (vard -> topology_obj, GetDefaultTopology (0))))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-bool IsHostDescriptor
+rbool IsHostDescriptor
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -354,22 +386,22 @@ bool IsHostDescriptor
  pvar vard;
 # endif
 {
-# line 270 "HomeDescriptor.puma"
+/* line 283 "HomeDescriptor.puma" */
   {
-# line 271 "HomeDescriptor.puma"
+/* line 284 "HomeDescriptor.puma" */
    if (! ((vard -> formal_rank == 0))) goto yyL1;
   {
-# line 272 "HomeDescriptor.puma"
+/* line 285 "HomeDescriptor.puma" */
    if (! ((SameTopologyObject (vard -> topology_obj, GetDefaultTopology (- 1))))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-bool IsProcDescriptor
+rbool IsProcDescriptor
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -377,18 +409,18 @@ bool IsProcDescriptor
  pvar vard;
 # endif
 {
-# line 277 "HomeDescriptor.puma"
+/* line 290 "HomeDescriptor.puma" */
   {
-# line 278 "HomeDescriptor.puma"
+/* line 291 "HomeDescriptor.puma" */
    if (! ((vard -> topology_obj == vard -> template_obj))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-bool NoReplicationDescriptor
+rbool IsFictiveParDescriptor
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -396,60 +428,98 @@ bool NoReplicationDescriptor
  pvar vard;
 # endif
 {
-# line 291 "HomeDescriptor.puma"
+/* line 296 "HomeDescriptor.puma" */
   {
-# line 293 "HomeDescriptor.puma"
-   if (! ((IsReplicatedDescriptor (vard)))) goto yyL1;
+/* line 298 "HomeDescriptor.puma" */
+   if (! ((vard -> template_obj == NoObject))) goto yyL1;
   {
-# line 294 "HomeDescriptor.puma"
-   return false;
+/* line 299 "HomeDescriptor.puma" */
+   if (! ((vard -> formal_rank > 0))) goto yyL1;
   }
   }
+   return rtrue;
 yyL1:;
 
-# line 297 "HomeDescriptor.puma"
+  return rfalse;
+}
+
+rbool NoReplicationDescriptor
+# if defined __STDC__ | defined __cplusplus
+(pvar vard)
+# else
+(vard)
+ pvar vard;
+# endif
+{
+/* line 312 "HomeDescriptor.puma" */
+ {
+  int target_model;
   {
-# line 298 "HomeDescriptor.puma"
-   if (! ((IsHostDescriptor (vard)))) goto yyL2;
+/* line 314 "HomeDescriptor.puma" */
+   if (! ((IsReplicatedDescriptor (vard)))) goto yyL1;
+  {
+/* line 318 "HomeDescriptor.puma" */
+   target_model = GetCurrentModel ();
+/* line 320 "HomeDescriptor.puma" */
+   if (! (((target_model == HPF_SERIAL) || (target_model == HPF_LOCAL)))) goto yyL1;
+  }
+  }
+   return rtrue;
+ }
+yyL1:;
+
+/* line 323 "HomeDescriptor.puma" */
+  {
+/* line 325 "HomeDescriptor.puma" */
+   if (! ((IsReplicatedDescriptor (vard)))) goto yyL2;
+  {
+/* line 326 "HomeDescriptor.puma" */
+   return rfalse;
+  }
   }
-   return true;
 yyL2:;
 
-# line 301 "HomeDescriptor.puma"
+/* line 329 "HomeDescriptor.puma" */
   {
-# line 302 "HomeDescriptor.puma"
-   if (! ((vard -> topology_obj == NoObject))) goto yyL3;
+/* line 330 "HomeDescriptor.puma" */
+   if (! ((IsHostDescriptor (vard)))) goto yyL3;
   }
-   return true;
+   return rtrue;
 yyL3:;
 
-# line 305 "HomeDescriptor.puma"
- {
-  bool no_replication;
+/* line 333 "HomeDescriptor.puma" */
   {
-# line 307 "HomeDescriptor.puma"
+/* line 334 "HomeDescriptor.puma" */
+   if (! ((vard -> topology_obj == NoObject))) goto yyL4;
+  }
+   return rtrue;
+yyL4:;
 
-# line 309 "HomeDescriptor.puma"
+/* line 337 "HomeDescriptor.puma" */
+ {
+  rbool no_replication;
+  {
+/* line 341 "HomeDescriptor.puma" */
  int  i;
 
-     no_replication = true;
+     no_replication = rtrue;
 
      for (i=0; i < vard->topology_rank; i++)
 
-       if (vard->on_val[i] == NoTree) no_replication = false;
+       if (vard->on_val[i] == NoTree) no_replication = rfalse;
 
    
-# line 319 "HomeDescriptor.puma"
-   if (! ((no_replication))) goto yyL4;
+/* line 351 "HomeDescriptor.puma" */
+   if (! ((no_replication))) goto yyL5;
   }
-   return true;
+   return rtrue;
  }
-yyL4:;
+yyL5:;
 
-  return false;
+  return rfalse;
 }
 
-bool IsFullLoopDescriptor
+rbool IsFullLoopDescriptor
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -457,48 +527,44 @@ bool IsFullLoopDescriptor
  pvar vard;
 # endif
 {
-# line 327 "HomeDescriptor.puma"
+/* line 359 "HomeDescriptor.puma" */
  {
-  bool okay;
+  rbool okay;
   int dim;
   {
-# line 329 "HomeDescriptor.puma"
+/* line 361 "HomeDescriptor.puma" */
    if (! ((vard -> var_tree == NoTree))) goto yyL1;
   {
-# line 330 "HomeDescriptor.puma"
+/* line 362 "HomeDescriptor.puma" */
    if (! ((vard -> template_obj == NoObject))) goto yyL1;
   {
-# line 332 "HomeDescriptor.puma"
-
-# line 333 "HomeDescriptor.puma"
-
-# line 335 "HomeDescriptor.puma"
- okay = true;
+/* line 367 "HomeDescriptor.puma" */
+ okay = rtrue;
 
      for (dim = 1; dim <= vard->formal_rank; dim++)
 
         { if (vard->actual_shape[dim-1][0] != vard->formal_shape[dim-1][0])
-             okay = false;
+             okay = rfalse;
           if (vard->actual_shape[dim-1][1] != vard->formal_shape[dim-1][1])
-             okay = false;
+             okay = rfalse;
           if (vard->actual_shape[dim-1][2] != NoTree)
-             okay = false;
+             okay = rfalse;
 
         } 
    
-# line 349 "HomeDescriptor.puma"
+/* line 381 "HomeDescriptor.puma" */
    if (! ((okay))) goto yyL1;
   }
   }
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-bool IsParallelLoopDescriptor
+rbool IsParallelLoopDescriptor
 # if defined __STDC__ | defined __cplusplus
 (pvar home, register tTree loop)
 # else
@@ -507,18 +573,18 @@ bool IsParallelLoopDescriptor
  register tTree loop;
 # endif
 {
-# line 369 "HomeDescriptor.puma"
+/* line 401 "HomeDescriptor.puma" */
   {
-# line 371 "HomeDescriptor.puma"
+/* line 403 "HomeDescriptor.puma" */
    if (! ((DescriptorLinearDependency (home, loop)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-bool IsParallelHomeDescriptor
+rbool IsParallelHomeDescriptor
 # if defined __STDC__ | defined __cplusplus
 (pvar d1)
 # else
@@ -526,16 +592,12 @@ bool IsParallelHomeDescriptor
  pvar d1;
 # endif
 {
-# line 376 "HomeDescriptor.puma"
+/* line 408 "HomeDescriptor.puma" */
  {
-  bool okay;
+  rbool okay;
   int i;
   {
-# line 378 "HomeDescriptor.puma"
-
-# line 379 "HomeDescriptor.puma"
-
-# line 381 "HomeDescriptor.puma"
+/* line 413 "HomeDescriptor.puma" */
  
 
 #ifdef DEBUG
@@ -543,22 +605,22 @@ bool IsParallelHomeDescriptor
      PrintVarDescriptor (d1);
 #endif
 
-     okay = true;
+     okay = rtrue;
      for (i=1; i <= OuterLoops(); i++)
        okay = okay && DescriptorLinearDependency (d1, GetOuterLoop(i));
 
    
-# line 394 "HomeDescriptor.puma"
+/* line 426 "HomeDescriptor.puma" */
    if (! ((okay))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool DescriptorLinearDependency
+static rbool DescriptorLinearDependency
 # if defined __STDC__ | defined __cplusplus
 (pvar d, register tTree loop)
 # else
@@ -567,34 +629,27 @@ static bool DescriptorLinearDependency
  register tTree loop;
 # endif
 {
-# line 399 "HomeDescriptor.puma"
+/* line 431 "HomeDescriptor.puma" */
   {
-# line 400 "HomeDescriptor.puma"
+/* line 433 "HomeDescriptor.puma" */
    if (! ((! IsParallelLoop (loop)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 403 "HomeDescriptor.puma"
+/* line 436 "HomeDescriptor.puma" */
  {
-  bool okay;
+  rbool okay;
   int i;
   tTree id;
   tTree index;
   {
-# line 405 "HomeDescriptor.puma"
-
-# line 406 "HomeDescriptor.puma"
-
-# line 407 "HomeDescriptor.puma"
-
-# line 408 "HomeDescriptor.puma"
-
-# line 410 "HomeDescriptor.puma"
- okay = false;
+/* line 443 "HomeDescriptor.puma" */
+ okay = rfalse;
      id   = GetLoopId (loop);
 
      for (i=0; i<d->formal_rank; i++)
+
        { index = d->actual_shape[i][0];
          okay = okay || (IsLinearDependent (index, id));
 #ifdef DEBUG
@@ -604,17 +659,17 @@ yyL1:;
 #endif
        }
    
-# line 423 "HomeDescriptor.puma"
+/* line 457 "HomeDescriptor.puma" */
    if (! ((okay))) goto yyL2;
   }
-   return true;
+   return rtrue;
  }
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsLinearDependent
+static rbool IsLinearDependent
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp, register tTree id)
 # else
@@ -623,46 +678,57 @@ static bool IsLinearDependent
  register tTree id;
 # endif
 {
-# line 436 "HomeDescriptor.puma"
+/* line 471 "HomeDescriptor.puma" */
+  {
+/* line 473 "HomeDescriptor.puma" */
+   if (! ((exp == NoTree))) goto yyL1;
+  {
+/* line 475 "HomeDescriptor.puma" */
+   return rfalse;
+  }
+  }
+yyL1:;
+
+  if (exp->Kind == kREC_COMP) {
+/* line 480 "HomeDescriptor.puma" */
+  {
+/* line 482 "HomeDescriptor.puma" */
+   return rfalse;
+  }
+
+  }
+/* line 485 "HomeDescriptor.puma" */
  {
   tTree var;
   int stride;
   int base;
-  bool found;
+  rbool found;
   {
-# line 438 "HomeDescriptor.puma"
-
-# line 439 "HomeDescriptor.puma"
-
-# line 440 "HomeDescriptor.puma"
-
-# line 441 "HomeDescriptor.puma"
-
-# line 443 "HomeDescriptor.puma"
+/* line 492 "HomeDescriptor.puma" */
    ResolveExpression (exp, & found, & stride, & base, & var);
-# line 444 "HomeDescriptor.puma"
-   if (! ((found))) goto yyL1;
+/* line 493 "HomeDescriptor.puma" */
+   if (! ((found))) goto yyL3;
   {
-# line 445 "HomeDescriptor.puma"
-   if (! ((stride != 0))) goto yyL1;
+/* line 494 "HomeDescriptor.puma" */
+   if (! ((stride != 0))) goto yyL3;
   {
-# line 447 "HomeDescriptor.puma"
-   if (! ((ResolveIsVariable (var)))) goto yyL1;
+/* line 496 "HomeDescriptor.puma" */
+   if (! ((ResolveIsVariable (var)))) goto yyL3;
   {
-# line 449 "HomeDescriptor.puma"
-   if (! ((TreeVarName (var) == TreeVarName (id)))) goto yyL1;
+/* line 498 "HomeDescriptor.puma" */
+   if (! ((TreeVarName (var) == TreeVarName (id)))) goto yyL3;
   }
   }
   }
   }
-   return true;
+   return rtrue;
  }
-yyL1:;
+yyL3:;
 
-  return false;
+  return rfalse;
 }
 
-static bool ResolveIsVariable
+static rbool ResolveIsVariable
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -671,24 +737,110 @@ static bool ResolveIsVariable
 # endif
 {
   if (Tree_IsType (var, kBT_VAR)) {
-# line 454 "HomeDescriptor.puma"
-   return true;
+/* line 503 "HomeDescriptor.puma" */
+   return rtrue;
 
   }
   if (var->Kind == kVAR_EXP) {
-# line 457 "HomeDescriptor.puma"
-   return true;
+/* line 506 "HomeDescriptor.puma" */
+   return rtrue;
 
   }
   if (var->Kind == kFUNC_CALL_EXP) {
-# line 460 "HomeDescriptor.puma"
+/* line 509 "HomeDescriptor.puma" */
   {
-# line 461 "HomeDescriptor.puma"
-   return false;
+/* line 510 "HomeDescriptor.puma" */
+   return rfalse;
   }
 
   }
-  return false;
+  return rfalse;
+}
+
+rbool IsLoopInvariantDescriptor
+# if defined __STDC__ | defined __cplusplus
+(pvar vd, register tTree loop)
+# else
+(vd, loop)
+ pvar vd;
+ register tTree loop;
+# endif
+{
+/* line 532 "HomeDescriptor.puma" */
+  {
+/* line 534 "HomeDescriptor.puma" */
+   if (! ((! IsCountLoop (loop)))) goto yyL1;
+  }
+   return rtrue;
+yyL1:;
+
+/* line 537 "HomeDescriptor.puma" */
+ {
+  rbool okay;
+  int i;
+  tTree id;
+  tTree index;
+  {
+/* line 544 "HomeDescriptor.puma" */
+ okay = rtrue;
+
+     id   = GetLoopId (loop);
+
+     for (i=0; i < vd->formal_rank; i++)
+
+       { index = vd->actual_shape[i][0];
+
+         okay = okay && (!AnyDependency (id, index));
+       }
+
+   
+/* line 557 "HomeDescriptor.puma" */
+   if (! ((okay))) goto yyL2;
+  }
+   return rtrue;
+ }
+yyL2:;
+
+  return rfalse;
+}
+
+static rbool AnyDependency
+# if defined __STDC__ | defined __cplusplus
+(register tTree id, register tTree exp)
+# else
+(id, exp)
+ register tTree id;
+ register tTree exp;
+# endif
+{
+/* line 562 "HomeDescriptor.puma" */
+  {
+/* line 564 "HomeDescriptor.puma" */
+   if (! ((exp == NoTree))) goto yyL1;
+  {
+/* line 566 "HomeDescriptor.puma" */
+   return rfalse;
+  }
+  }
+yyL1:;
+
+  if (exp->Kind == kREC_COMP) {
+/* line 569 "HomeDescriptor.puma" */
+  {
+/* line 571 "HomeDescriptor.puma" */
+   return rfalse;
+  }
+
+  }
+/* line 574 "HomeDescriptor.puma" */
+  {
+/* line 576 "HomeDescriptor.puma" */
+   if (! ((IsVarInExp (TreeVarName (id), exp)))) goto yyL3;
+  }
+   return rtrue;
+yyL3:;
+
+  return rfalse;
 }
 
 void UnionHomeDescriptor
@@ -700,33 +852,31 @@ void UnionHomeDescriptor
  pvar vard2;
 # endif
 {
-# line 478 "HomeDescriptor.puma"
+/* line 593 "HomeDescriptor.puma" */
   {
-# line 480 "HomeDescriptor.puma"
+/* line 595 "HomeDescriptor.puma" */
    if (! ((VDIsSubSet (vard2, vard1)))) goto yyL1;
   }
    return;
 yyL1:;
 
-# line 483 "HomeDescriptor.puma"
+/* line 598 "HomeDescriptor.puma" */
   {
-# line 485 "HomeDescriptor.puma"
+/* line 600 "HomeDescriptor.puma" */
    if (! ((VDIsSubSet (vard1, vard2)))) goto yyL2;
   {
-# line 487 "HomeDescriptor.puma"
+/* line 602 "HomeDescriptor.puma" */
  *vard1 = *vard2; 
   }
   }
    return;
 yyL2:;
 
-# line 498 "HomeDescriptor.puma"
+/* line 613 "HomeDescriptor.puma" */
  {
   int top_dim;
   {
-# line 500 "HomeDescriptor.puma"
-
-# line 502 "HomeDescriptor.puma"
+/* line 617 "HomeDescriptor.puma" */
 
 #ifdef DEBUG
      printf ("Union of two homes\n");
@@ -734,14 +884,14 @@ yyL2:;
      PrintVarDescriptor (vard2);
 #endif
    
-# line 510 "HomeDescriptor.puma"
+/* line 625 "HomeDescriptor.puma" */
    if (! ((SameDescriptorTopology (vard1, vard2)))) goto yyL3;
   {
-# line 512 "HomeDescriptor.puma"
+/* line 627 "HomeDescriptor.puma" */
    MakeTemplateDescriptor (vard1);
-# line 513 "HomeDescriptor.puma"
+/* line 628 "HomeDescriptor.puma" */
    MakeTemplateDescriptor (vard2);
-# line 515 "HomeDescriptor.puma"
+/* line 630 "HomeDescriptor.puma" */
  for (top_dim = 1; top_dim <= VarRank (vard1->topology_obj); top_dim ++)
 
         UnionTopDims (vard1, vard2, top_dim);
@@ -760,9 +910,9 @@ yyL2:;
  }
 yyL3:;
 
-# line 529 "HomeDescriptor.puma"
+/* line 644 "HomeDescriptor.puma" */
   {
-# line 531 "HomeDescriptor.puma"
+/* line 646 "HomeDescriptor.puma" */
    MakeReplicatedDescriptor (vard1);
   }
    return;
@@ -780,16 +930,12 @@ static void UnionTopDims
  register int top_dim;
 # endif
 {
-# line 543 "HomeDescriptor.puma"
+/* line 658 "HomeDescriptor.puma" */
  {
   int idim1;
   int idim2;
   {
-# line 545 "HomeDescriptor.puma"
-
-# line 546 "HomeDescriptor.puma"
-
-# line 548 "HomeDescriptor.puma"
+/* line 663 "HomeDescriptor.puma" */
  idim1 = vard1->on_index_dim [top_dim-1];
      idim2 = vard2->on_index_dim [top_dim-1];
 
@@ -811,7 +957,7 @@ static void UnionTopDims
 
        else
 
-        { bool found;
+        { rbool found;
           int  diff;
 
           
@@ -835,110 +981,108 @@ static void UnionTopDims
 
 void MakeSerialDescriptor
 # if defined __STDC__ | defined __cplusplus
-(pvar home, register bool * yyP1)
+(pvar home, register rbool * yyP1)
 # else
 (home, yyP1)
  pvar home;
- register bool * yyP1;
+ register rbool * yyP1;
 # endif
 {
-# line 594 "HomeDescriptor.puma"
+/* line 709 "HomeDescriptor.puma" */
   {
-# line 596 "HomeDescriptor.puma"
+/* line 711 "HomeDescriptor.puma" */
    if (! ((IsHostDescriptor (home)))) goto yyL1;
   }
-   * yyP1 = true;
+   * yyP1 = rtrue;
    return;
 yyL1:;
 
-# line 599 "HomeDescriptor.puma"
+/* line 714 "HomeDescriptor.puma" */
   {
-# line 601 "HomeDescriptor.puma"
+/* line 716 "HomeDescriptor.puma" */
    if (! ((IsReplicatedDescriptor (home)))) goto yyL2;
   {
-# line 602 "HomeDescriptor.puma"
+/* line 717 "HomeDescriptor.puma" */
    MakeHostDescriptor (home);
   }
   }
-   * yyP1 = true;
+   * yyP1 = rtrue;
    return;
 yyL2:;
 
-# line 605 "HomeDescriptor.puma"
+/* line 720 "HomeDescriptor.puma" */
   {
-# line 607 "HomeDescriptor.puma"
+/* line 722 "HomeDescriptor.puma" */
    if (! ((IsNoDescriptor (home)))) goto yyL3;
   {
-# line 608 "HomeDescriptor.puma"
+/* line 723 "HomeDescriptor.puma" */
    MakeHostDescriptor (home);
   }
   }
-   * yyP1 = true;
+   * yyP1 = rtrue;
    return;
 yyL3:;
 
-# line 611 "HomeDescriptor.puma"
+/* line 726 "HomeDescriptor.puma" */
   {
-# line 613 "HomeDescriptor.puma"
+/* line 728 "HomeDescriptor.puma" */
    if (! ((home -> topology_obj == GetDefaultTopology (0)))) goto yyL4;
   {
-# line 614 "HomeDescriptor.puma"
+/* line 729 "HomeDescriptor.puma" */
    MakeHostDescriptor (home);
   }
   }
-   * yyP1 = true;
+   * yyP1 = rtrue;
    return;
 yyL4:;
 
-# line 617 "HomeDescriptor.puma"
+/* line 732 "HomeDescriptor.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 619 "HomeDescriptor.puma"
-
-# line 621 "HomeDescriptor.puma"
+/* line 736 "HomeDescriptor.puma" */
  int i;
 
-     okay = true;
+     okay = rtrue;
      for (i=0; i<home->topology_rank; i++)
 
         { tTree val = home->on_val[i];
 
           if (val == NoTree)
-             okay = false;                  
+             okay = rfalse;                  
 
            else if (TreeRank (home->on_val[i]) > 0)
-             okay = false;
+             okay = rfalse;
         }
    
-# line 635 "HomeDescriptor.puma"
+/* line 750 "HomeDescriptor.puma" */
    if (! ((okay))) goto yyL5;
   }
-   * yyP1 = true;
+   * yyP1 = rtrue;
    return;
  }
 yyL5:;
 
-# line 638 "HomeDescriptor.puma"
+/* line 753 "HomeDescriptor.puma" */
   {
-# line 640 "HomeDescriptor.puma"
+/* line 755 "HomeDescriptor.puma" */
    printf ("could not serialize : \n");
-# line 641 "HomeDescriptor.puma"
+/* line 756 "HomeDescriptor.puma" */
    PrintVarDescriptor (home);
-# line 642 "HomeDescriptor.puma"
+/* line 757 "HomeDescriptor.puma" */
    MakeTemplateDescriptor (home);
-# line 643 "HomeDescriptor.puma"
+/* line 758 "HomeDescriptor.puma" */
    printf ("template : \n");
-# line 644 "HomeDescriptor.puma"
+/* line 759 "HomeDescriptor.puma" */
    PrintVarDescriptor (home);
   }
-   * yyP1 = false;
+   * yyP1 = rfalse;
    return;
 
 ;
 }
 
-bool IsSerialDescriptor
+rbool IsSerialDescriptor
 # if defined __STDC__ | defined __cplusplus
 (pvar home)
 # else
@@ -946,88 +1090,86 @@ bool IsSerialDescriptor
  pvar home;
 # endif
 {
-# line 661 "HomeDescriptor.puma"
+/* line 776 "HomeDescriptor.puma" */
   {
-# line 663 "HomeDescriptor.puma"
+/* line 778 "HomeDescriptor.puma" */
    if (! ((GetCurrentModel () == HPF_SERIAL))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 666 "HomeDescriptor.puma"
+/* line 781 "HomeDescriptor.puma" */
   {
-# line 668 "HomeDescriptor.puma"
+/* line 783 "HomeDescriptor.puma" */
    if (! ((IsHostDescriptor (home)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 671 "HomeDescriptor.puma"
+/* line 786 "HomeDescriptor.puma" */
   {
-# line 673 "HomeDescriptor.puma"
+/* line 788 "HomeDescriptor.puma" */
    if (! ((IsNoDescriptor (home)))) goto yyL3;
   {
-# line 674 "HomeDescriptor.puma"
-   return false;
+/* line 789 "HomeDescriptor.puma" */
+   return rfalse;
   }
   }
 yyL3:;
 
-# line 677 "HomeDescriptor.puma"
+/* line 792 "HomeDescriptor.puma" */
   {
-# line 679 "HomeDescriptor.puma"
+/* line 794 "HomeDescriptor.puma" */
    if (! ((home -> topology_obj == NoObject))) goto yyL4;
   {
-# line 680 "HomeDescriptor.puma"
+/* line 795 "HomeDescriptor.puma" */
    if (! ((home -> template_inherited == 1))) goto yyL4;
   {
-# line 682 "HomeDescriptor.puma"
-   return false;
+/* line 797 "HomeDescriptor.puma" */
+   return rfalse;
   }
   }
   }
 yyL4:;
 
-# line 685 "HomeDescriptor.puma"
+/* line 800 "HomeDescriptor.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 687 "HomeDescriptor.puma"
+/* line 802 "HomeDescriptor.puma" */
    if (! ((home -> topology_obj != GetDefaultTopology (0)))) goto yyL5;
   {
-# line 689 "HomeDescriptor.puma"
-
-# line 691 "HomeDescriptor.puma"
+/* line 806 "HomeDescriptor.puma" */
  int i;
 
-     okay = true;
+     okay = rtrue;
 
      for (i=0; i<home->topology_rank; i++)
 
         { tTree val = home->on_val[i];
 
           if (val == NoTree)
-             okay = false;                  
+             okay = rfalse;                  
 
            else if (TreeRank (home->on_val[i]) > 0)
-             okay = false;
+             okay = rfalse;
         }
    
-# line 706 "HomeDescriptor.puma"
+/* line 821 "HomeDescriptor.puma" */
    if (! ((okay))) goto yyL5;
   }
   }
-   return true;
+   return rtrue;
  }
 yyL5:;
 
-  return false;
+  return rfalse;
 }
 
-void BeginHomeDescriptor ()
+void BeginHomeDescriptor ARGS ((void))
 {
 }
 
-void CloseHomeDescriptor ()
+void CloseHomeDescriptor ARGS ((void))
 {
 }
