@@ -1,6 +1,13 @@
 # include "InterProc.h"
-# include "yyInterProc.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -9,45 +16,49 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 21 "InterProc.puma"
+/* line 21 "InterProc.puma" */
 
 
 # undef  DEBUG
 
 # define MODULE "InterProc"
+
+
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 # include "protocol.h"
 
 # include "Traverse.h"
+# include "StrUnparse.h"
+# include <string.h>
 
 # include "Common.h"
 # include "Types.h"           /* ArrayFormals, ... */
 
+# include "Rank.h"
 # include "Calling.h"
 # include "CallGraphFns.h"
 # include "Nesting.h"
+# include "DefTable.h"
 # include "Objects.h"
+# include "RegShadow.h"
 
 typedef void CallFunction ();
 
@@ -56,16 +67,30 @@ static int overlap_update;
 static int  ip_update;         /* used globally for terminating  */
 static int  ip_phases;         /* counts phases                  */
 
-static char CalledRoutine  [MAXID_LENGTH];
-static char CallingRoutine [MAXID_LENGTH];
+static char CalledRoutine  [MAX_ID_LENGTH];
+static char CallingRoutine [MAX_ID_LENGTH];
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* InterProc_Exit) () = yyExit;
+# include "yyInterProc.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* InterProc_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -74,23 +99,25 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module InterProc, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module InterProc, routine %s failed\n",
+  yyFunction);
  InterProc_Exit ();
 }
 
-void Interprocedural ARGS((tTree t));
-static void IP_Analysis ARGS((tTree t));
-static void Dummy ARGS((tDefinitions actual_obj, tDefinitions formal_obj));
-static void BottomUp ARGS((tCallGraph t, CallFunction f));
-static void TopDown ARGS((tCallGraph t, CallFunction f));
-static void UnMark ARGS((tCallGraph c));
-static void ApplyToCalls ARGS((tDefinitions obj, CallFunction f));
-static void DoFunction ARGS((tTree t));
-static void SetSameOverlap ARGS((tDefinitions actual_obj, tDefinitions formal_obj));
-static tDefinitions GetCommonReference ARGS((tDefinitions common_obj));
-static void ProtocolUpdate ARGS((tDefinitions actual_obj, tDefinitions formal_obj, int kind));
-static void SetSameOverlapFormals ARGS((tTree actuallist, tTree dummylist));
-static void SetSameOverlapSize ARGS((tTree actual, tTree dummy));
+void Interprocedural ARGS ((tTree t));
+static void IP_Analysis ARGS ((tTree t));
+static void Dummy ARGS ((tDefinitions actual_obj, tDefinitions formal_obj));
+static void BottomUp ARGS ((tCallGraph t, CallFunction f));
+static void TopDown ARGS ((tCallGraph t, CallFunction f));
+static void UnMark ARGS ((tCallGraph c));
+static void ApplyToCalls ARGS ((tDefinitions obj, CallFunction f));
+static void DoFunction ARGS ((tTree t));
+static void SetSameOverlap ARGS ((tDefinitions actual_obj, tDefinitions formal_obj));
+static tDefinitions GetCommonReference ARGS ((tDefinitions common_obj));
+static void ProtocolUpdate ARGS ((tDefinitions actual_obj, tDefinitions formal_obj, int kind));
+static void CompareShadow ARGS ((tDefinitions actual, tDefinitions dummy));
+static void SetSameOverlapFormals ARGS ((tTree actuallist, tTree dummylist));
+static void SetSameOverlapSize ARGS ((tTree actual, tTree dummy));
 
 void Interprocedural
 # if defined __STDC__ | defined __cplusplus
@@ -100,18 +127,16 @@ void Interprocedural
  register tTree t;
 # endif
 {
-# line 60 "InterProc.puma"
+/* line 67 "InterProc.puma" */
 
 char msg[100];
 
   if (t->Kind == kCOMP_UNIT) {
-# line 70 "InterProc.puma"
+/* line 77 "InterProc.puma" */
   {
-# line 72 "InterProc.puma"
-   open_protocol ("adaptor.ipa");
-# line 77 "InterProc.puma"
+/* line 82 "InterProc.puma" */
    FullTraverseAST (t->COMP_UNIT.COMP_ELEMENTS, IP_Analysis);
-# line 79 "InterProc.puma"
+/* line 84 "InterProc.puma" */
  if (protocol_errors () == 0)
 
        {  BeginCalling ();
@@ -144,11 +169,11 @@ char msg[100];
  
           } while (ip_update != 0);
 
+         PrintShadowInfo ();
+
        }
 
     
-# line 115 "InterProc.puma"
-   close_protocol ();
   }
    return;
 
@@ -165,27 +190,27 @@ static void IP_Analysis
 # endif
 {
   if (t->Kind == kCOMMON_DECL) {
-# line 130 "InterProc.puma"
+/* line 135 "InterProc.puma" */
   {
-# line 132 "InterProc.puma"
+/* line 137 "InterProc.puma" */
    CheckCommonDescriptors (t, Dummy);
   }
    return;
 
   }
   if (t->Kind == kCALL_STMT) {
-# line 135 "InterProc.puma"
+/* line 140 "InterProc.puma" */
   {
-# line 137 "InterProc.puma"
+/* line 142 "InterProc.puma" */
    CheckActualDescriptors (t, Dummy);
   }
    return;
 
   }
   if (t->Kind == kFUNC_CALL_EXP) {
-# line 140 "InterProc.puma"
+/* line 145 "InterProc.puma" */
   {
-# line 142 "InterProc.puma"
+/* line 147 "InterProc.puma" */
    CheckActualDescriptors (t, Dummy);
   }
    return;
@@ -205,16 +230,16 @@ static void Dummy
 {
   if (formal_obj->Kind == kVarObject) {
   if (formal_obj->VarObject.Kind->Kind == kVarCommon) {
-# line 147 "InterProc.puma"
+/* line 152 "InterProc.puma" */
   {
-# line 151 "InterProc.puma"
+/* line 156 "InterProc.puma" */
    formal_obj->VarObject.Kind->VarCommon.ref_obj = actual_obj;
   }
    return;
 
   }
   }
-# line 154 "InterProc.puma"
+/* line 159 "InterProc.puma" */
    return;
 
 ;
@@ -229,77 +254,78 @@ static void BottomUp
  CallFunction f;
 # endif
 {
-# line 173 "InterProc.puma"
+ yyRecursion:
+/* line 178 "InterProc.puma" */
   {
-# line 174 "InterProc.puma"
+/* line 179 "InterProc.puma" */
    if (! ((t == NoCallGraph))) goto yyL1;
   }
    return;
 yyL1:;
 
-  if (t->Kind == kCallGraph) {
-# line 177 "InterProc.puma"
+  if (t->Kind == kCallingGraph) {
+/* line 182 "InterProc.puma" */
   {
-# line 179 "InterProc.puma"
+/* line 184 "InterProc.puma" */
 
 #ifdef DEBUG
     printf ("Bottom Up Visting of all Nodes\n");
 #endif 
     
-# line 185 "InterProc.puma"
-   UnMark (t->CallGraph.UserNodes);
-# line 186 "InterProc.puma"
-   BottomUp (t->CallGraph.UserNodes, f);
+/* line 190 "InterProc.puma" */
+   UnMark (t->CallingGraph.UserNodes);
+/* line 191 "InterProc.puma" */
+   t = t->CallingGraph.UserNodes;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kCallNodeList) {
-# line 189 "InterProc.puma"
+/* line 194 "InterProc.puma" */
   {
-# line 190 "InterProc.puma"
+/* line 195 "InterProc.puma" */
    BottomUp (t->CallNodeList.Elem, f);
-# line 191 "InterProc.puma"
-   BottomUp (t->CallNodeList.Next, f);
+/* line 196 "InterProc.puma" */
+   t = t->CallNodeList.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kCallNode) {
-# line 194 "InterProc.puma"
+/* line 199 "InterProc.puma" */
   {
-# line 195 "InterProc.puma"
+/* line 200 "InterProc.puma" */
    if (! ((t->CallNode.unitkind == 2))) goto yyL4;
   }
    return;
 yyL4:;
 
-# line 198 "InterProc.puma"
+/* line 203 "InterProc.puma" */
   {
-# line 200 "InterProc.puma"
+/* line 205 "InterProc.puma" */
    if (! ((t->CallNode.unitkind == 1))) goto yyL5;
   {
-# line 202 "InterProc.puma"
+/* line 207 "InterProc.puma" */
    ApplyToCalls (t->CallNode.val, f);
-# line 204 "InterProc.puma"
+/* line 209 "InterProc.puma" */
    t->CallNode.unitkind = 2;
   }
   }
    return;
 yyL5:;
 
-# line 207 "InterProc.puma"
+/* line 212 "InterProc.puma" */
   {
-# line 209 "InterProc.puma"
+/* line 214 "InterProc.puma" */
    if (! ((UnitKind (t->CallNode.val) == 0))) goto yyL6;
   {
-# line 213 "InterProc.puma"
+/* line 218 "InterProc.puma" */
    t->CallNode.unitkind = 1;
-# line 215 "InterProc.puma"
+/* line 220 "InterProc.puma" */
    BottomUp (t->CallNode.calling, f);
-# line 217 "InterProc.puma"
+/* line 222 "InterProc.puma" */
    ApplyToCalls (t->CallNode.val, f);
-# line 219 "InterProc.puma"
+/* line 224 "InterProc.puma" */
    t->CallNode.unitkind = 2;
   }
   }
@@ -308,14 +334,14 @@ yyL6:;
 
   }
   if (t->Kind == kCallEdgeList) {
-# line 222 "InterProc.puma"
+/* line 227 "InterProc.puma" */
   {
-# line 223 "InterProc.puma"
+/* line 228 "InterProc.puma" */
    BottomUp (t->CallEdgeList.Node, f);
-# line 224 "InterProc.puma"
-   BottomUp (t->CallEdgeList.Next, f);
+/* line 229 "InterProc.puma" */
+   t = t->CallEdgeList.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
 ;
@@ -330,77 +356,78 @@ static void TopDown
  CallFunction f;
 # endif
 {
-# line 243 "InterProc.puma"
+ yyRecursion:
+/* line 248 "InterProc.puma" */
   {
-# line 244 "InterProc.puma"
+/* line 249 "InterProc.puma" */
    if (! ((t == NoCallGraph))) goto yyL1;
   }
    return;
 yyL1:;
 
-  if (t->Kind == kCallGraph) {
-# line 247 "InterProc.puma"
+  if (t->Kind == kCallingGraph) {
+/* line 252 "InterProc.puma" */
   {
-# line 249 "InterProc.puma"
+/* line 254 "InterProc.puma" */
 
 #ifdef DEBUG
     printf ("Top Down Visting of all Nodes\n");
 #endif 
     
-# line 255 "InterProc.puma"
-   UnMark (t->CallGraph.UserNodes);
-# line 256 "InterProc.puma"
-   TopDown (t->CallGraph.UserNodes, f);
+/* line 260 "InterProc.puma" */
+   UnMark (t->CallingGraph.UserNodes);
+/* line 261 "InterProc.puma" */
+   t = t->CallingGraph.UserNodes;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kCallNodeList) {
-# line 259 "InterProc.puma"
+/* line 264 "InterProc.puma" */
   {
-# line 260 "InterProc.puma"
+/* line 265 "InterProc.puma" */
    TopDown (t->CallNodeList.Elem, f);
-# line 261 "InterProc.puma"
-   TopDown (t->CallNodeList.Next, f);
+/* line 266 "InterProc.puma" */
+   t = t->CallNodeList.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kCallNode) {
-# line 264 "InterProc.puma"
+/* line 269 "InterProc.puma" */
   {
-# line 265 "InterProc.puma"
+/* line 270 "InterProc.puma" */
    if (! ((t->CallNode.unitkind == 2))) goto yyL4;
   }
    return;
 yyL4:;
 
-# line 268 "InterProc.puma"
+/* line 273 "InterProc.puma" */
   {
-# line 270 "InterProc.puma"
+/* line 275 "InterProc.puma" */
    if (! ((t->CallNode.unitkind == 1))) goto yyL5;
   {
-# line 272 "InterProc.puma"
+/* line 277 "InterProc.puma" */
    ApplyToCalls (t->CallNode.val, f);
-# line 274 "InterProc.puma"
+/* line 279 "InterProc.puma" */
    t->CallNode.unitkind = 2;
   }
   }
    return;
 yyL5:;
 
-# line 277 "InterProc.puma"
+/* line 282 "InterProc.puma" */
   {
-# line 279 "InterProc.puma"
+/* line 284 "InterProc.puma" */
    if (! ((UnitKind (t->CallNode.val) == 0))) goto yyL6;
   {
-# line 283 "InterProc.puma"
+/* line 288 "InterProc.puma" */
    t->CallNode.unitkind = 1;
-# line 285 "InterProc.puma"
+/* line 290 "InterProc.puma" */
    TopDown (t->CallNode.called_by, f);
-# line 287 "InterProc.puma"
+/* line 292 "InterProc.puma" */
    ApplyToCalls (t->CallNode.val, f);
-# line 289 "InterProc.puma"
+/* line 294 "InterProc.puma" */
    t->CallNode.unitkind = 2;
   }
   }
@@ -409,14 +436,14 @@ yyL6:;
 
   }
   if (t->Kind == kCallEdgeList) {
-# line 292 "InterProc.puma"
+/* line 297 "InterProc.puma" */
   {
-# line 294 "InterProc.puma"
+/* line 299 "InterProc.puma" */
    TopDown (t->CallEdgeList.Node, f);
-# line 295 "InterProc.puma"
-   TopDown (t->CallEdgeList.Next, f);
+/* line 300 "InterProc.puma" */
+   t = t->CallEdgeList.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
 ;
@@ -430,42 +457,43 @@ static void UnMark
  register tCallGraph c;
 # endif
 {
-# line 306 "InterProc.puma"
+ yyRecursion:
+/* line 311 "InterProc.puma" */
   {
-# line 307 "InterProc.puma"
+/* line 312 "InterProc.puma" */
    if (! ((c == NoCallGraph))) goto yyL1;
   }
    return;
 yyL1:;
 
-  if (c->Kind == kCallGraph) {
-# line 310 "InterProc.puma"
+  if (c->Kind == kCallingGraph) {
+/* line 315 "InterProc.puma" */
   {
-# line 311 "InterProc.puma"
-   UnMark (c->CallGraph.UserNodes);
-# line 312 "InterProc.puma"
-   UnMark (c->CallGraph.IntrinsicNodes);
-# line 313 "InterProc.puma"
-   UnMark (c->CallGraph.ExternalNodes);
+/* line 316 "InterProc.puma" */
+   UnMark (c->CallingGraph.UserNodes);
+/* line 317 "InterProc.puma" */
+   UnMark (c->CallingGraph.IntrinsicNodes);
+/* line 318 "InterProc.puma" */
+   c = c->CallingGraph.ExternalNodes;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (c->Kind == kCallNodeList) {
-# line 316 "InterProc.puma"
+/* line 321 "InterProc.puma" */
   {
-# line 317 "InterProc.puma"
+/* line 322 "InterProc.puma" */
    UnMark (c->CallNodeList.Elem);
-# line 318 "InterProc.puma"
-   UnMark (c->CallNodeList.Next);
+/* line 323 "InterProc.puma" */
+   c = c->CallNodeList.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (c->Kind == kCallNode) {
-# line 321 "InterProc.puma"
+/* line 326 "InterProc.puma" */
   {
-# line 322 "InterProc.puma"
+/* line 327 "InterProc.puma" */
    c->CallNode.unitkind = 0;
   }
    return;
@@ -485,18 +513,18 @@ static void ApplyToCalls
 {
   if (obj->Kind == kProcObject) {
   if (obj->ProcObject.decl->Kind == kPROGRAM_DECL) {
-# line 335 "InterProc.puma"
+/* line 340 "InterProc.puma" */
   {
-# line 337 "InterProc.puma"
+/* line 342 "InterProc.puma" */
    if (! ((obj->ProcObject.decl->PROGRAM_DECL.PROGRAM_BODY != NoTree))) goto yyL1;
   {
-# line 339 "InterProc.puma"
+/* line 344 "InterProc.puma" */
    GetString (obj->ProcObject.Ident, CallingRoutine);
-# line 340 "InterProc.puma"
+/* line 345 "InterProc.puma" */
    NestOpenUnitObj (obj);
-# line 341 "InterProc.puma"
+/* line 346 "InterProc.puma" */
    FullTraverseAST (obj->ProcObject.decl->PROGRAM_DECL.PROGRAM_BODY, f);
-# line 342 "InterProc.puma"
+/* line 347 "InterProc.puma" */
    NestCloseUnitObj (obj);
   }
   }
@@ -505,18 +533,18 @@ yyL1:;
 
   }
   if (obj->ProcObject.decl->Kind == kPROC_DECL) {
-# line 345 "InterProc.puma"
+/* line 350 "InterProc.puma" */
   {
-# line 347 "InterProc.puma"
+/* line 352 "InterProc.puma" */
    if (! ((obj->ProcObject.decl->PROC_DECL.PROC_BODY != NoTree))) goto yyL2;
   {
-# line 349 "InterProc.puma"
+/* line 354 "InterProc.puma" */
    GetString (obj->ProcObject.Ident, CallingRoutine);
-# line 350 "InterProc.puma"
+/* line 355 "InterProc.puma" */
    NestOpenUnitObj (obj);
-# line 351 "InterProc.puma"
+/* line 356 "InterProc.puma" */
    FullTraverseAST (obj->ProcObject.decl->PROC_DECL.PROC_BODY, f);
-# line 352 "InterProc.puma"
+/* line 357 "InterProc.puma" */
    NestCloseUnitObj (obj);
   }
   }
@@ -526,15 +554,15 @@ yyL2:;
   }
   }
   if (obj->Kind == kFuncObject) {
-# line 355 "InterProc.puma"
+/* line 360 "InterProc.puma" */
   {
-# line 357 "InterProc.puma"
+/* line 362 "InterProc.puma" */
    GetString (obj->FuncObject.Ident, CallingRoutine);
-# line 358 "InterProc.puma"
+/* line 363 "InterProc.puma" */
    NestOpenUnitObj (obj);
-# line 359 "InterProc.puma"
+/* line 364 "InterProc.puma" */
    FullTraverseAST (obj->FuncObject.decl, f);
-# line 360 "InterProc.puma"
+/* line 365 "InterProc.puma" */
    NestCloseUnitObj (obj);
   }
    return;
@@ -552,31 +580,31 @@ static void DoFunction
 # endif
 {
   if (t->Kind == kCOMMON_DECL) {
-# line 378 "InterProc.puma"
+/* line 383 "InterProc.puma" */
   {
-# line 380 "InterProc.puma"
+/* line 385 "InterProc.puma" */
    CheckCommonDescriptors (t, SetSameOverlap);
   }
    return;
 
   }
   if (t->Kind == kCALL_STMT) {
-# line 383 "InterProc.puma"
+/* line 388 "InterProc.puma" */
   {
-# line 387 "InterProc.puma"
+/* line 392 "InterProc.puma" */
    GetString (t->CALL_STMT.CALL_ID->PROC_OBJ.Ident, CalledRoutine);
-# line 392 "InterProc.puma"
+/* line 397 "InterProc.puma" */
    CheckActualDescriptors (t, SetSameOverlap);
   }
    return;
 
   }
   if (t->Kind == kFUNC_CALL_EXP) {
-# line 395 "InterProc.puma"
+/* line 400 "InterProc.puma" */
   {
-# line 399 "InterProc.puma"
+/* line 404 "InterProc.puma" */
    GetString (t->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Ident, CalledRoutine);
-# line 400 "InterProc.puma"
+/* line 405 "InterProc.puma" */
    CheckActualDescriptors (t, SetSameOverlap);
   }
    return;
@@ -594,12 +622,12 @@ static void SetSameOverlap
  register tDefinitions formal_obj;
 # endif
 {
-# line 414 "InterProc.puma"
+/* line 419 "InterProc.puma" */
   {
-# line 416 "InterProc.puma"
+/* line 421 "InterProc.puma" */
    if (! ((VarRank (actual_obj) != VarRank (formal_obj)))) goto yyL1;
   {
-# line 420 "InterProc.puma"
+/* line 425 "InterProc.puma" */
  if (semantic_check == 0)
        warning_protocol ("no overlap update: rank of dummy/actual mismatch");
      else
@@ -612,29 +640,27 @@ static void SetSameOverlap
    return;
 yyL1:;
 
-# line 429 "InterProc.puma"
+/* line 434 "InterProc.puma" */
   {
-# line 431 "InterProc.puma"
+/* line 436 "InterProc.puma" */
    if (! ((VarRank (formal_obj) == 0))) goto yyL2;
   }
    return;
 yyL2:;
 
-# line 434 "InterProc.puma"
+/* line 439 "InterProc.puma" */
  {
   tDefinitions ref_actual_obj;
   {
-# line 436 "InterProc.puma"
+/* line 441 "InterProc.puma" */
    overlap_update = 0;
-# line 438 "InterProc.puma"
-   SetSameOverlapFormals (ArrayFormals (actual_obj), ArrayFormals (formal_obj));
-# line 441 "InterProc.puma"
+/* line 443 "InterProc.puma" */
+   CompareShadow (actual_obj, formal_obj);
+/* line 445 "InterProc.puma" */
    ProtocolUpdate (actual_obj, formal_obj, overlap_update);
-# line 443 "InterProc.puma"
+/* line 447 "InterProc.puma" */
  if (overlap_update) ip_update++; 
-# line 455 "InterProc.puma"
-
-# line 457 "InterProc.puma"
+/* line 461 "InterProc.puma" */
  if ( IsVarCommon(actual_obj) && IsVarDummy(formal_obj))
     
      {   ref_actual_obj = GetCommonReference (actual_obj);
@@ -661,16 +687,16 @@ static tDefinitions GetCommonReference
 {
   if (common_obj->Kind == kVarObject) {
   if (common_obj->VarObject.Kind->Kind == kVarCommon) {
-# line 470 "InterProc.puma"
+/* line 474 "InterProc.puma" */
    return common_obj->VarObject.Kind->VarCommon.ref_obj;
 
   }
   }
-# line 474 "InterProc.puma"
+/* line 478 "InterProc.puma" */
   {
-# line 475 "InterProc.puma"
+/* line 479 "InterProc.puma" */
    obj_error_protocol ("common object expected ", common_obj);
-# line 476 "InterProc.puma"
+/* line 480 "InterProc.puma" */
    failure_protocol (MODULE, "GetCommonReference", common_obj->Object.decl);
   }
    return NoObject;
@@ -687,20 +713,20 @@ static void ProtocolUpdate
  register int kind;
 # endif
 {
-# line 482 "InterProc.puma"
+/* line 486 "InterProc.puma" */
 
 char msg[100];
 
-# line 486 "InterProc.puma"
+/* line 490 "InterProc.puma" */
   {
-# line 489 "InterProc.puma"
+/* line 493 "InterProc.puma" */
    goto yyL1;
   }
 yyL1:;
 
-# line 492 "InterProc.puma"
+/* line 496 "InterProc.puma" */
   {
-# line 494 "InterProc.puma"
+/* line 498 "InterProc.puma" */
  if (kind == 1)
         print_protocol ("inherited overlap from formal");
       else if (kind == 2)
@@ -725,6 +751,72 @@ yyL1:;
 ;
 }
 
+static void CompareShadow
+# if defined __STDC__ | defined __cplusplus
+(register tDefinitions actual, register tDefinitions dummy)
+# else
+(actual, dummy)
+ register tDefinitions actual;
+ register tDefinitions dummy;
+# endif
+{
+ yyRecursion:
+  if (actual->Kind == kVarObject) {
+  if (dummy->Kind == kVarObject) {
+/* line 526 "InterProc.puma" */
+  {
+/* line 528 "InterProc.puma" */
+   if (! ((IsZeroShadow (actual->VarObject.shadow)))) goto yyL1;
+  {
+/* line 529 "InterProc.puma" */
+   if (! ((IsZeroShadow (dummy->VarObject.shadow)))) goto yyL1;
+  }
+  }
+   return;
+yyL1:;
+
+/* line 534 "InterProc.puma" */
+  {
+/* line 536 "InterProc.puma" */
+   if (! ((actual->VarObject.shadow == NoTree))) goto yyL2;
+  {
+/* line 538 "InterProc.puma" */
+   SetDefaultShadow (actual);
+/* line 539 "InterProc.puma" */
+   goto yyRecursion;
+  }
+  }
+yyL2:;
+
+/* line 542 "InterProc.puma" */
+  {
+/* line 544 "InterProc.puma" */
+   if (! ((dummy->VarObject.shadow == NoTree))) goto yyL3;
+  {
+/* line 546 "InterProc.puma" */
+   SetDefaultShadow (dummy);
+/* line 547 "InterProc.puma" */
+   goto yyRecursion;
+  }
+  }
+yyL3:;
+
+/* line 550 "InterProc.puma" */
+  {
+/* line 554 "InterProc.puma" */
+   FixShadowVarObject (actual);
+/* line 555 "InterProc.puma" */
+   FixShadowVarObject (dummy);
+/* line 557 "InterProc.puma" */
+   SetSameOverlapFormals (actual->VarObject.shadow, dummy->VarObject.shadow);
+  }
+   return;
+
+  }
+  }
+;
+}
+
 static void SetSameOverlapFormals
 # if defined __STDC__ | defined __cplusplus
 (register tTree actuallist, register tTree dummylist)
@@ -734,29 +826,31 @@ static void SetSameOverlapFormals
  register tTree dummylist;
 # endif
 {
-  if (actuallist->Kind == kSHAPE_LIST) {
-  if (dummylist->Kind == kSHAPE_LIST) {
-# line 526 "InterProc.puma"
+ yyRecursion:
+  if (actuallist->Kind == kSHADOW_LIST) {
+  if (dummylist->Kind == kSHADOW_LIST) {
+/* line 572 "InterProc.puma" */
   {
-# line 528 "InterProc.puma"
-   SetSameOverlapSize (actuallist->SHAPE_LIST.Elem, dummylist->SHAPE_LIST.Elem);
-# line 529 "InterProc.puma"
-   SetSameOverlapFormals (actuallist->SHAPE_LIST.Next, dummylist->SHAPE_LIST.Next);
+/* line 574 "InterProc.puma" */
+   SetSameOverlapSize (actuallist->SHADOW_LIST.Elem, dummylist->SHADOW_LIST.Elem);
+/* line 575 "InterProc.puma" */
+   actuallist = actuallist->SHADOW_LIST.Next;
+   dummylist = dummylist->SHADOW_LIST.Next;
+   goto yyRecursion;
   }
+
+  }
+  }
+  if (actuallist->Kind == kSHADOW_EMPTY) {
+  if (dummylist->Kind == kSHADOW_EMPTY) {
+/* line 578 "InterProc.puma" */
    return;
 
   }
   }
-  if (actuallist->Kind == kSHAPE_EMPTY) {
-  if (dummylist->Kind == kSHAPE_EMPTY) {
-# line 532 "InterProc.puma"
-   return;
-
-  }
-  }
-# line 537 "InterProc.puma"
+/* line 583 "InterProc.puma" */
   {
-# line 538 "InterProc.puma"
+/* line 585 "InterProc.puma" */
    failure2_protocol (MODULE, "SetSameOverlapFormals", actuallist, dummylist);
   }
    return;
@@ -773,35 +867,24 @@ static void SetSameOverlapSize
  register tTree dummy;
 # endif
 {
-  if (Tree_IsType (actual, kSHAPE_SPEC)) {
-  if (Tree_IsType (dummy, kSHAPE_SPEC)) {
-# line 553 "InterProc.puma"
+  if (actual->Kind == kSHADOW_SPEC) {
+  if (dummy->Kind == kSHADOW_SPEC) {
+/* line 602 "InterProc.puma" */
   {
-# line 556 "InterProc.puma"
-   SetSameOverlapSize (actual->SHAPE_SPEC.Overlap, dummy->SHAPE_SPEC.Overlap);
-  }
-   return;
-
-  }
-  }
-  if (actual->Kind == kOVERLAP_SPEC) {
-  if (dummy->Kind == kOVERLAP_SPEC) {
-# line 561 "InterProc.puma"
-  {
-# line 563 "InterProc.puma"
- if (actual->OVERLAP_SPEC.left_size < dummy->OVERLAP_SPEC.left_size)  { actual->OVERLAP_SPEC.left_size = dummy->OVERLAP_SPEC.left_size;  overlap_update = overlap_update | 1; }
-    if (dummy->OVERLAP_SPEC.left_size < actual->OVERLAP_SPEC.left_size)  { dummy->OVERLAP_SPEC.left_size = actual->OVERLAP_SPEC.left_size;  overlap_update = overlap_update | 2; }
-    if (actual->OVERLAP_SPEC.right_size < dummy->OVERLAP_SPEC.right_size)  { actual->OVERLAP_SPEC.right_size = dummy->OVERLAP_SPEC.right_size;  overlap_update = overlap_update | 1; }
-    if (dummy->OVERLAP_SPEC.right_size < actual->OVERLAP_SPEC.right_size)  { dummy->OVERLAP_SPEC.right_size = actual->OVERLAP_SPEC.right_size;  overlap_update = overlap_update | 2; }
+/* line 605 "InterProc.puma" */
+ if (actual->SHADOW_SPEC.left_size < dummy->SHADOW_SPEC.left_size)  { actual->SHADOW_SPEC.left_size = dummy->SHADOW_SPEC.left_size;  overlap_update = overlap_update | 1; }
+    if (dummy->SHADOW_SPEC.left_size < actual->SHADOW_SPEC.left_size)  { dummy->SHADOW_SPEC.left_size = actual->SHADOW_SPEC.left_size;  overlap_update = overlap_update | 2; }
+    if (actual->SHADOW_SPEC.right_size < dummy->SHADOW_SPEC.right_size)  { actual->SHADOW_SPEC.right_size = dummy->SHADOW_SPEC.right_size;  overlap_update = overlap_update | 1; }
+    if (dummy->SHADOW_SPEC.right_size < actual->SHADOW_SPEC.right_size)  { dummy->SHADOW_SPEC.right_size = actual->SHADOW_SPEC.right_size;  overlap_update = overlap_update | 2; }
   
   }
    return;
 
   }
   }
-# line 570 "InterProc.puma"
+/* line 612 "InterProc.puma" */
   {
-# line 571 "InterProc.puma"
+/* line 614 "InterProc.puma" */
    failure2_protocol (MODULE, "SetSameOverlapSize", actual, dummy);
   }
    return;
@@ -809,10 +892,10 @@ static void SetSameOverlapSize
 ;
 }
 
-void BeginInterProc ()
+void BeginInterProc ARGS ((void))
 {
 }
 
-void CloseInterProc ()
+void CloseInterProc ARGS ((void))
 {
 }
