@@ -1,6 +1,13 @@
 # include "MakeHome.h"
-# include "yyMakeHome.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,32 +15,29 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 48 "MakeHome.puma"
+/* line 48 "MakeHome.puma" */
 
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
@@ -42,6 +46,8 @@
 # include "FindHome.h"
 # include "Temporary.h"
 
+# include "Unparse.h"             /* FileUnparse for DEBUG mode */
+
 # include "Nesting.h"               /* NestOpen */
 
 # include "Objects.h"
@@ -56,15 +62,30 @@
 # include "ArrayData.h"
 
 # define MODULE "MakeHome"
+
 # undef DEBUG
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* MakeHome_Exit) () = yyExit;
+# include "yyMakeHome.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* MakeHome_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -73,28 +94,29 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module MakeHome, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module MakeHome, routine %s failed\n",
+  yyFunction);
  MakeHome_Exit ();
 }
 
-void MakeHome ARGS((tTree t));
-static void DetermineHome ARGS((tTree t));
-tTree MakeNewHome ARGS((tTree t));
-static bool StopDetermineHome ARGS((tTree t));
-tTree DoDetermineHome ARGS((tTree t));
-static tTree PropagateHome ARGS((tTree t, pvar outer_home));
-static void InfoDoLoopHome ARGS((tTree loop, pvar iter_home));
-static tTree SetNewHome ARGS((tTree stmt, pvar outer_home, pvar inner_home));
-static tTree BuildNewVarHome ARGS((tTree stmt, pvar home));
-static int GetBodyLine ARGS((tTree stmt));
-static void MakeHomeTemplate ARGS((pvar home, pvar home1, tTree * yyP2, tTree * yyP1));
-static void SetIterationsHome ARGS((tTree loop, pvar iter_home));
-static void SetSelection ARGS((tTree loop, pvar iter_home));
-static void FindNewHomeDescriptor ARGS((tTree stmt, pvar cur_home, pvar new_home));
-static void FindSerialHomeDescriptor ARGS((tTree stmt, pvar cur_home, pvar new_home));
-static void DetermineFinalHome ARGS((pvar cur_home, pvar min_home, pvar new_home));
-static void InheritTemplate ARGS((pvar home, pvar sub_home));
-static void CheckCallHome ARGS((tTree t, pvar final_home));
+void MakeHome ARGS ((tTree t));
+static void DetermineHome ARGS ((tTree t));
+tTree MakeNewHome ARGS ((tTree t));
+static rbool StopDetermineHome ARGS ((tTree t));
+tTree DoDetermineHome ARGS ((tTree t));
+static tTree PropagateHome ARGS ((tTree t, pvar outer_home));
+static void InfoDoLoopHome ARGS ((tTree loop, pvar iter_home));
+static tTree SetNewHome ARGS ((tTree stmt, pvar outer_home, pvar inner_home));
+static tTree BuildNewVarHome ARGS ((tTree stmt, pvar home));
+static int GetBodyLine ARGS ((tTree stmt));
+static void MakeHomeTemplate ARGS ((pvar home, pvar home1, tTree * yyP2, tTree * yyP1));
+static void SetIterationsHome ARGS ((tTree loop, pvar iter_home));
+static void SetSelection ARGS ((tTree loop, pvar iter_home));
+static void FindNewHomeDescriptor ARGS ((tTree stmt, pvar cur_home, pvar new_home));
+static void FindSerialHomeDescriptor ARGS ((tTree stmt, pvar cur_home, pvar new_home));
+static void DetermineFinalHome ARGS ((pvar cur_home, pvar min_home, pvar new_home));
+static void InheritTemplate ARGS ((pvar home, pvar sub_home));
+static void CheckCallHome ARGS ((tTree t, pvar final_home));
 
 void MakeHome
 # if defined __STDC__ | defined __cplusplus
@@ -104,121 +126,118 @@ void MakeHome
  register tTree t;
 # endif
 {
+ yyRecursion:
 
   switch (t->Kind) {
   case kCOMP_UNIT:
-# line 91 "MakeHome.puma"
+/* line 94 "MakeHome.puma" */
   {
-# line 93 "MakeHome.puma"
-   open_protocol ("adaptor.home");
-# line 94 "MakeHome.puma"
-   MakeHome (t->COMP_UNIT.COMP_ELEMENTS);
-# line 95 "MakeHome.puma"
-   close_protocol ();
+/* line 96 "MakeHome.puma" */
+   t = t->COMP_UNIT.COMP_ELEMENTS;
+   goto yyRecursion;
   }
-   return;
 
   case kUNIT_LIST:
-# line 100 "MakeHome.puma"
+/* line 101 "MakeHome.puma" */
   {
-# line 102 "MakeHome.puma"
+/* line 103 "MakeHome.puma" */
    MakeHome (t->UNIT_LIST.Elem);
-# line 103 "MakeHome.puma"
-   MakeHome (t->UNIT_LIST.Next);
+/* line 104 "MakeHome.puma" */
+   t = t->UNIT_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kUNIT_EMPTY:
-# line 106 "MakeHome.puma"
+/* line 107 "MakeHome.puma" */
    return;
 
   case kPROGRAM_DECL:
-# line 109 "MakeHome.puma"
+/* line 110 "MakeHome.puma" */
   {
-# line 111 "MakeHome.puma"
+/* line 112 "MakeHome.puma" */
    NestOpenUnit (t);
-# line 112 "MakeHome.puma"
+/* line 113 "MakeHome.puma" */
    MakeHome (t->PROGRAM_DECL.PROGRAM_BODY);
-# line 113 "MakeHome.puma"
+/* line 114 "MakeHome.puma" */
    NestCloseUnit (t);
   }
    return;
 
   case kPROC_DECL:
-# line 116 "MakeHome.puma"
+/* line 117 "MakeHome.puma" */
   {
-# line 118 "MakeHome.puma"
+/* line 119 "MakeHome.puma" */
    NestOpenUnit (t);
-# line 119 "MakeHome.puma"
+/* line 120 "MakeHome.puma" */
    MakeHome (t->PROC_DECL.PROC_BODY);
-# line 120 "MakeHome.puma"
+/* line 121 "MakeHome.puma" */
    NestCloseUnit (t);
   }
    return;
 
   case kFUNC_DECL:
-# line 123 "MakeHome.puma"
+/* line 124 "MakeHome.puma" */
   {
-# line 125 "MakeHome.puma"
+/* line 126 "MakeHome.puma" */
    NestOpenUnit (t);
-# line 126 "MakeHome.puma"
+/* line 127 "MakeHome.puma" */
    MakeHome (t->FUNC_DECL.FUNC_BODY);
-# line 127 "MakeHome.puma"
+/* line 128 "MakeHome.puma" */
    NestCloseUnit (t);
   }
    return;
 
   case kBLOCK_DATA_DECL:
-# line 130 "MakeHome.puma"
+/* line 131 "MakeHome.puma" */
   {
-# line 132 "MakeHome.puma"
+/* line 133 "MakeHome.puma" */
    NestOpenUnit (t);
-# line 133 "MakeHome.puma"
+/* line 134 "MakeHome.puma" */
    MakeHome (t->BLOCK_DATA_DECL.DATA_BODY);
-# line 134 "MakeHome.puma"
+/* line 135 "MakeHome.puma" */
    NestCloseUnit (t);
   }
    return;
 
   case kMODULE_DECL:
-# line 137 "MakeHome.puma"
+/* line 138 "MakeHome.puma" */
   {
-# line 139 "MakeHome.puma"
+/* line 140 "MakeHome.puma" */
    NestOpenUnit (t);
-# line 140 "MakeHome.puma"
+/* line 141 "MakeHome.puma" */
    MakeHome (t->MODULE_DECL.MODULE_BODY);
-# line 141 "MakeHome.puma"
+/* line 142 "MakeHome.puma" */
    NestCloseUnit (t);
   }
    return;
 
   case kBODY_NODE:
-# line 144 "MakeHome.puma"
+/* line 145 "MakeHome.puma" */
   {
-# line 146 "MakeHome.puma"
+/* line 147 "MakeHome.puma" */
    if (! ((IsPureUnit (GetCurrentUnit ())))) goto yyL9;
   }
    return;
 yyL9:;
 
-# line 151 "MakeHome.puma"
+/* line 152 "MakeHome.puma" */
   {
-# line 153 "MakeHome.puma"
+/* line 154 "MakeHome.puma" */
    TemporaryInit (t);
-# line 155 "MakeHome.puma"
+/* line 156 "MakeHome.puma" */
    print_protocol ("Step 1 : check home \n");
-# line 157 "MakeHome.puma"
+/* line 158 "MakeHome.puma" */
    CheckHome (t);
-# line 159 "MakeHome.puma"
+/* line 160 "MakeHome.puma" */
    print_protocol ("Step 2 : find home \n");
-# line 161 "MakeHome.puma"
+/* line 162 "MakeHome.puma" */
    DetermineHome (t);
-# line 163 "MakeHome.puma"
+/* line 164 "MakeHome.puma" */
    TemporaryDone (t);
-# line 165 "MakeHome.puma"
-   MakeHome (t->BODY_NODE.INTERNALS);
+/* line 166 "MakeHome.puma" */
+   t = t->BODY_NODE.INTERNALS;
+   goto yyRecursion;
   }
-   return;
 
   }
 
@@ -234,17 +253,17 @@ static void DetermineHome
 # endif
 {
   if (t->Kind == kBODY_NODE) {
-# line 201 "MakeHome.puma"
+/* line 202 "MakeHome.puma" */
   {
-# line 203 "MakeHome.puma"
+/* line 204 "MakeHome.puma" */
  t->BODY_NODE.STATS = MakeNewHome (t->BODY_NODE.STATS); 
   }
    return;
 
   }
-# line 206 "MakeHome.puma"
+/* line 207 "MakeHome.puma" */
   {
-# line 207 "MakeHome.puma"
+/* line 208 "MakeHome.puma" */
    failure_protocol (MODULE, "DetermineHome", t);
   }
    return;
@@ -260,12 +279,12 @@ tTree MakeNewHome
  register tTree t;
 # endif
 {
-# line 212 "MakeHome.puma"
+/* line 213 "MakeHome.puma" */
    return ReplaceAST (t, StopDetermineHome, DoDetermineHome);
 
 }
 
-static bool StopDetermineHome
+static rbool StopDetermineHome
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -275,32 +294,32 @@ static bool StopDetermineHome
 {
   if (t->Kind == kACF_DO) {
   if (t->ACF_DO.DO_DEP_INFO->Kind == kINDEP_INFO) {
-# line 230 "MakeHome.puma"
-   return true;
+/* line 231 "MakeHome.puma" */
+   return rtrue;
 
   }
   }
   if (t->Kind == kACF_FORALL) {
   if (t->ACF_FORALL.FORALL_DEP_INFO->Kind == kINDEP_INFO) {
-# line 233 "MakeHome.puma"
-   return true;
+/* line 234 "MakeHome.puma" */
+   return rtrue;
 
   }
   }
   if (t->Kind == kACF_HOME) {
-# line 236 "MakeHome.puma"
-   return true;
+/* line 237 "MakeHome.puma" */
+   return rtrue;
 
   }
   if (t->Kind == kACF_BASIC) {
-# line 239 "MakeHome.puma"
-   return true;
+/* line 240 "MakeHome.puma" */
+   return rtrue;
 
   }
-# line 242 "MakeHome.puma"
+/* line 243 "MakeHome.puma" */
   {
-# line 243 "MakeHome.puma"
-   return false;
+/* line 244 "MakeHome.puma" */
+   return rfalse;
   }
 
 }
@@ -313,89 +332,72 @@ tTree DoDetermineHome
  register tTree t;
 # endif
 {
+ yyRecursion:
   if (t->Kind == kACF_DO) {
   if (t->ACF_DO.DO_DEP_INFO->Kind == kINDEP_INFO) {
-# line 263 "MakeHome.puma"
+/* line 264 "MakeHome.puma" */
  {
   var_descriptor vard;
   tTree new;
   {
-# line 265 "MakeHome.puma"
-
-# line 266 "MakeHome.puma"
-
-# line 268 "MakeHome.puma"
+/* line 269 "MakeHome.puma" */
    MakeReplicatedDescriptor (& vard);
-# line 270 "MakeHome.puma"
+/* line 271 "MakeHome.puma" */
    new = PropagateHome (t, & vard);
-# line 272 "MakeHome.puma"
+/* line 273 "MakeHome.puma" */
    tree_protocol ("home of independent loop identified : \n", new);
   }
-  {
    return new;
-  }
  }
 
   }
   }
   if (t->Kind == kACF_FORALL) {
-# line 277 "MakeHome.puma"
+/* line 278 "MakeHome.puma" */
   {
-# line 279 "MakeHome.puma"
+/* line 280 "MakeHome.puma" */
  t->Kind = kACF_DO; 
   }
-   return DoDetermineHome (t);
+   goto yyRecursion;
 
   }
   if (t->Kind == kACF_HOME) {
-# line 284 "MakeHome.puma"
+/* line 285 "MakeHome.puma" */
  {
   var_descriptor vard;
   tTree new;
   {
-# line 286 "MakeHome.puma"
-
-# line 287 "MakeHome.puma"
-
-# line 289 "MakeHome.puma"
+/* line 290 "MakeHome.puma" */
    MakeReplicatedDescriptor (& vard);
-# line 291 "MakeHome.puma"
+/* line 292 "MakeHome.puma" */
    new = PropagateHome (t, & vard);
-# line 293 "MakeHome.puma"
+/* line 294 "MakeHome.puma" */
    tree_protocol ("homes of home stmt/block identified : \n", new);
   }
-  {
    return new;
-  }
  }
 
   }
   if (t->Kind == kACF_BASIC) {
-# line 298 "MakeHome.puma"
+/* line 299 "MakeHome.puma" */
  {
   var_descriptor vard;
   tTree new;
   {
-# line 300 "MakeHome.puma"
-
-# line 301 "MakeHome.puma"
-
-# line 303 "MakeHome.puma"
+/* line 304 "MakeHome.puma" */
    MakeReplicatedDescriptor (& vard);
-# line 305 "MakeHome.puma"
+/* line 306 "MakeHome.puma" */
    new = PropagateHome (t, & vard);
-# line 307 "MakeHome.puma"
+/* line 308 "MakeHome.puma" */
  if (new != t)
         tree_protocol ("home of basic statement found : \n", new);
    
   }
-  {
    return new;
-  }
  }
 
   }
-# line 314 "MakeHome.puma"
+/* line 315 "MakeHome.puma" */
    return t;
 
 }
@@ -409,80 +411,73 @@ static tTree PropagateHome
  pvar outer_home;
 # endif
 {
+ yyRecursion:
 
   switch (t->Kind) {
   case kACF_LIST:
-# line 332 "MakeHome.puma"
+/* line 333 "MakeHome.puma" */
  {
   tTree newacf;
   {
-# line 334 "MakeHome.puma"
-
-# line 336 "MakeHome.puma"
+/* line 337 "MakeHome.puma" */
    set_protocol_stmt (t->ACF_LIST.Elem);
-# line 338 "MakeHome.puma"
+/* line 339 "MakeHome.puma" */
  newacf = PropagateHome (t->ACF_LIST.Elem, outer_home);         
      t->ACF_LIST.Next   = PropagateHome (t->ACF_LIST.Next, outer_home);         
    
-# line 342 "MakeHome.puma"
+/* line 343 "MakeHome.puma" */
    newacf = ReplaceACF (t, newacf, t->ACF_LIST.Next);
   }
-  {
    return newacf;
-  }
  }
 
   case kACF_EMPTY:
-# line 347 "MakeHome.puma"
+/* line 348 "MakeHome.puma" */
    return t;
 
   case kACF_FORALL:
   if (t->ACF_FORALL.FORALL_DEP_INFO->Kind == kINDEP_INFO) {
-# line 352 "MakeHome.puma"
+/* line 353 "MakeHome.puma" */
   {
-# line 354 "MakeHome.puma"
+/* line 355 "MakeHome.puma" */
  t->Kind = kACF_DO; 
   }
-   return PropagateHome (t, outer_home);
+   goto yyRecursion;
 
   }
   break;
   case kACF_DO:
   if (t->ACF_DO.DO_DEP_INFO->Kind == kINDEP_INFO) {
-# line 359 "MakeHome.puma"
+/* line 360 "MakeHome.puma" */
  {
   var_descriptor iter_home;
   tTree yyV1;
   tTree yyV2;
-  bool okay;
+  rbool okay;
   {
-# line 361 "MakeHome.puma"
+/* line 362 "MakeHome.puma" */
    IncParNesting (t);
-# line 363 "MakeHome.puma"
-
-# line 365 "MakeHome.puma"
+/* line 366 "MakeHome.puma" */
    FindNewHomeDescriptor (t->ACF_DO.DO_BODY, outer_home, & iter_home);
-# line 367 "MakeHome.puma"
+/* line 368 "MakeHome.puma" */
    tree_protocol ("home of loop ", PrintableDescriptorVar (outer_home));
-# line 368 "MakeHome.puma"
+/* line 369 "MakeHome.puma" */
    tree_protocol ("home of loop body = ", PrintableDescriptorVar (& iter_home));
-# line 370 "MakeHome.puma"
+/* line 371 "MakeHome.puma" */
    InfoDoLoopHome (t, & iter_home);
-# line 372 "MakeHome.puma"
+/* line 373 "MakeHome.puma" */
    MakeHomeTemplate (outer_home, & iter_home, & yyV1, & yyV2);
-# line 374 "MakeHome.puma"
+/* line 375 "MakeHome.puma" */
    SetIterationsHome (t, & iter_home);
-# line 375 "MakeHome.puma"
+/* line 376 "MakeHome.puma" */
    SetSelection (t, & iter_home);
-# line 377 "MakeHome.puma"
+/* line 378 "MakeHome.puma" */
  t->ACF_DO.DO_BODY = PropagateHome (t->ACF_DO.DO_BODY, &iter_home); 
      t->ACF_DO.DO_BODY = MakeStmtList (SetNewHome (t->ACF_DO.DO_BODY, outer_home, &iter_home));
    
-# line 381 "MakeHome.puma"
-
-# line 383 "MakeHome.puma"
+/* line 384 "MakeHome.puma" */
    ExpandDescriptor (& iter_home, t->ACF_DO.DO_ID, & okay);
-# line 385 "MakeHome.puma"
+/* line 386 "MakeHome.puma" */
  if (!okay)
 
         { set_protocol_stmt(t);
@@ -490,19 +485,17 @@ static tTree PropagateHome
         }
 
    
-# line 393 "MakeHome.puma"
+/* line 394 "MakeHome.puma" */
    DecParNesting (t);
   }
-  {
    return ComposeNewACF (yyV1, SetNewHome (t, outer_home, & iter_home), yyV2);
-  }
  }
 
   }
   if (t->ACF_DO.DO_DEP_INFO->Kind == kSERIAL_INFO) {
-# line 564 "MakeHome.puma"
+/* line 565 "MakeHome.puma" */
   {
-# line 566 "MakeHome.puma"
+/* line 567 "MakeHome.puma" */
  t->ACF_DO.DO_BODY = PropagateHome (t->ACF_DO.DO_BODY, outer_home); 
   }
    return t;
@@ -510,215 +503,179 @@ static tTree PropagateHome
   }
   break;
   case kACF_NEW:
-# line 399 "MakeHome.puma"
+/* line 400 "MakeHome.puma" */
   {
-# line 401 "MakeHome.puma"
+/* line 402 "MakeHome.puma" */
    IncParNesting (t);
-# line 402 "MakeHome.puma"
+/* line 403 "MakeHome.puma" */
  t->ACF_NEW.NEW_BODY = PropagateHome (t->ACF_NEW.NEW_BODY, outer_home); 
-# line 403 "MakeHome.puma"
+/* line 404 "MakeHome.puma" */
    DecParNesting (t);
   }
    return t;
 
   case kACF_RESIDENT:
-# line 408 "MakeHome.puma"
+/* line 409 "MakeHome.puma" */
   {
-# line 410 "MakeHome.puma"
+/* line 411 "MakeHome.puma" */
    IncParNesting (t);
-# line 411 "MakeHome.puma"
+/* line 412 "MakeHome.puma" */
  t->ACF_RESIDENT.RESIDENT_BODY = PropagateHome (t->ACF_RESIDENT.RESIDENT_BODY, outer_home); 
-# line 412 "MakeHome.puma"
+/* line 413 "MakeHome.puma" */
    DecParNesting (t);
   }
    return t;
 
   case kACF_TASK_REGION:
-# line 417 "MakeHome.puma"
+/* line 418 "MakeHome.puma" */
  {
   var_descriptor new_home;
   {
-# line 419 "MakeHome.puma"
+/* line 420 "MakeHome.puma" */
    IncParNesting (t);
-# line 421 "MakeHome.puma"
-
-# line 423 "MakeHome.puma"
+/* line 424 "MakeHome.puma" */
    FindNewHomeDescriptor (t, outer_home, & new_home);
-# line 425 "MakeHome.puma"
+/* line 426 "MakeHome.puma" */
  t->ACF_TASK_REGION.TASK_BODY = PropagateHome (t->ACF_TASK_REGION.TASK_BODY, &new_home); 
-# line 427 "MakeHome.puma"
+/* line 428 "MakeHome.puma" */
    DecParNesting (t);
   }
-  {
    return SetNewHome (t, outer_home, & new_home);
-  }
  }
 
   case kACF_REDUCTION:
-# line 432 "MakeHome.puma"
+/* line 433 "MakeHome.puma" */
  {
   var_descriptor new_home;
   {
-# line 434 "MakeHome.puma"
+/* line 435 "MakeHome.puma" */
    IncParNesting (t);
-# line 436 "MakeHome.puma"
-
-# line 438 "MakeHome.puma"
+/* line 439 "MakeHome.puma" */
    FindNewHomeDescriptor (t, outer_home, & new_home);
-# line 440 "MakeHome.puma"
+/* line 441 "MakeHome.puma" */
  t->ACF_REDUCTION.REDUCTION_BODY = PropagateHome (t->ACF_REDUCTION.REDUCTION_BODY, &new_home); 
-# line 442 "MakeHome.puma"
+/* line 443 "MakeHome.puma" */
    DecParNesting (t);
   }
-  {
    return SetNewHome (t, outer_home, & new_home);
-  }
  }
 
   case kACF_HOME:
   if (t->ACF_HOME.HOME_VAR->Kind == kON_VAR_CLAUSE) {
-# line 447 "MakeHome.puma"
+/* line 448 "MakeHome.puma" */
  {
   var_descriptor on_home;
   {
-# line 449 "MakeHome.puma"
-
-# line 451 "MakeHome.puma"
+/* line 452 "MakeHome.puma" */
    IncParNesting (t);
-# line 462 "MakeHome.puma"
+/* line 463 "MakeHome.puma" */
    SetVarDescriptor (t->ACF_HOME.HOME_VAR->ON_VAR_CLAUSE.ON_VAR, & on_home);
-# line 464 "MakeHome.puma"
+/* line 465 "MakeHome.puma" */
  t->ACF_HOME.HOME_BODY = PropagateHome (t->ACF_HOME.HOME_BODY, &on_home); 
-# line 466 "MakeHome.puma"
+/* line 467 "MakeHome.puma" */
    DecParNesting (t);
   }
-  {
    return t;
-  }
  }
 
   }
   if (t->ACF_HOME.HOME_VAR->Kind == kON_ALL_CLAUSE) {
-# line 471 "MakeHome.puma"
+/* line 472 "MakeHome.puma" */
  {
   var_descriptor on_home;
   {
-# line 473 "MakeHome.puma"
-
-# line 475 "MakeHome.puma"
+/* line 476 "MakeHome.puma" */
    IncParNesting (t);
-# line 477 "MakeHome.puma"
+/* line 478 "MakeHome.puma" */
    MakeReplicatedDescriptor (& on_home);
-# line 479 "MakeHome.puma"
+/* line 480 "MakeHome.puma" */
  t->ACF_HOME.HOME_BODY = PropagateHome (t->ACF_HOME.HOME_BODY, &on_home); 
-# line 481 "MakeHome.puma"
+/* line 482 "MakeHome.puma" */
    DecParNesting (t);
   }
-  {
    return t;
-  }
  }
 
   }
   if (t->ACF_HOME.HOME_VAR->Kind == kON_PROC_CLAUSE) {
-# line 486 "MakeHome.puma"
+/* line 487 "MakeHome.puma" */
  {
   var_descriptor on_home;
   {
-# line 488 "MakeHome.puma"
-
-# line 490 "MakeHome.puma"
+/* line 491 "MakeHome.puma" */
    IncParNesting (t);
-# line 492 "MakeHome.puma"
+/* line 493 "MakeHome.puma" */
    SetProcDescriptor (t->ACF_HOME.HOME_VAR->ON_PROC_CLAUSE.ON_PROC, & on_home);
-# line 494 "MakeHome.puma"
+/* line 495 "MakeHome.puma" */
  t->ACF_HOME.HOME_BODY = PropagateHome (t->ACF_HOME.HOME_BODY, &on_home); 
-# line 496 "MakeHome.puma"
+/* line 497 "MakeHome.puma" */
    DecParNesting (t);
   }
-  {
    return t;
-  }
  }
 
   }
   break;
   case kACF_BASIC:
   if (t->ACF_BASIC.BASIC_STMT->Kind == kIO_STMT) {
-# line 501 "MakeHome.puma"
+/* line 502 "MakeHome.puma" */
  {
   var_descriptor new_home;
   {
-# line 503 "MakeHome.puma"
-
-# line 505 "MakeHome.puma"
+/* line 506 "MakeHome.puma" */
    FindSerialHomeDescriptor (t, outer_home, & new_home);
   }
-  {
    return SetNewHome (t, outer_home, & new_home);
-  }
  }
 
   }
   if (t->ACF_BASIC.BASIC_STMT->Kind == kREDUCE_STMT) {
-# line 514 "MakeHome.puma"
+/* line 515 "MakeHome.puma" */
  {
   var_descriptor new_home;
   {
-# line 516 "MakeHome.puma"
-
-# line 520 "MakeHome.puma"
+/* line 521 "MakeHome.puma" */
    FindNewHomeDescriptor (t, outer_home, & new_home);
   }
-  {
    return SetNewHome (t, outer_home, & new_home);
-  }
  }
 
   }
   if (t->ACF_BASIC.BASIC_STMT->Kind == kCALL_STMT) {
-# line 525 "MakeHome.puma"
+/* line 526 "MakeHome.puma" */
  {
   var_descriptor new_home;
   {
-# line 527 "MakeHome.puma"
+/* line 528 "MakeHome.puma" */
    if (! ((IsSerialCall (t->ACF_BASIC.BASIC_STMT)))) goto yyL14;
   {
-# line 529 "MakeHome.puma"
-
-# line 531 "MakeHome.puma"
+/* line 532 "MakeHome.puma" */
    FindSerialHomeDescriptor (t, outer_home, & new_home);
   }
   }
-  {
    return SetNewHome (t, outer_home, & new_home);
-  }
  }
 yyL14:;
 
   }
-# line 536 "MakeHome.puma"
+/* line 537 "MakeHome.puma" */
  {
   var_descriptor new_home;
   {
-# line 538 "MakeHome.puma"
+/* line 539 "MakeHome.puma" */
    SetActualData (t);
-# line 540 "MakeHome.puma"
-
-# line 542 "MakeHome.puma"
+/* line 543 "MakeHome.puma" */
    FindNewHomeDescriptor (t, outer_home, & new_home);
-# line 544 "MakeHome.puma"
+/* line 545 "MakeHome.puma" */
    CheckCallHome (t, & new_home);
   }
-  {
    return SetNewHome (t, outer_home, & new_home);
-  }
  }
 
   case kACF_IF:
-# line 555 "MakeHome.puma"
+/* line 556 "MakeHome.puma" */
   {
-# line 557 "MakeHome.puma"
+/* line 558 "MakeHome.puma" */
  t->ACF_IF.THEN_PART = PropagateHome (t->ACF_IF.THEN_PART, outer_home);
      t->ACF_IF.ELSE_PART = PropagateHome (t->ACF_IF.ELSE_PART, outer_home);
    
@@ -726,22 +683,22 @@ yyL14:;
    return t;
 
   case kACF_WHILE:
-# line 571 "MakeHome.puma"
+/* line 572 "MakeHome.puma" */
   {
-# line 573 "MakeHome.puma"
+/* line 574 "MakeHome.puma" */
  t->ACF_WHILE.WHILE_BODY = PropagateHome (t->ACF_WHILE.WHILE_BODY, outer_home); 
   }
    return t;
 
   case kACF_DUMMY:
-# line 578 "MakeHome.puma"
+/* line 579 "MakeHome.puma" */
    return t;
 
   }
 
-# line 583 "MakeHome.puma"
+/* line 584 "MakeHome.puma" */
   {
-# line 585 "MakeHome.puma"
+/* line 586 "MakeHome.puma" */
    error_protocol ("illegal statement to propagate home");
   }
    return t;
@@ -757,19 +714,19 @@ static void InfoDoLoopHome
  pvar iter_home;
 # endif
 {
-# line 597 "MakeHome.puma"
+/* line 598 "MakeHome.puma" */
   {
-# line 599 "MakeHome.puma"
+/* line 600 "MakeHome.puma" */
    if (! ((! IsParallelHomeDescriptor (iter_home)))) goto yyL1;
   {
-# line 601 "MakeHome.puma"
-   if (! ((GetCurrentModel () == HPF_GLOBAL))) goto yyL1;
+/* line 602 "MakeHome.puma" */
+   if (! ((IsNonLocalModel (GetCurrentModel ())))) goto yyL1;
   {
-# line 603 "MakeHome.puma"
+/* line 604 "MakeHome.puma" */
    set_protocol_stmt (loop);
-# line 605 "MakeHome.puma"
+/* line 606 "MakeHome.puma" */
    warning_protocol ("no parallel home descriptor for iterations found");
-# line 606 "MakeHome.puma"
+/* line 607 "MakeHome.puma" */
    tree_protocol ("proposed home is : ", PrintableDescriptorVar (iter_home));
   }
   }
@@ -790,48 +747,61 @@ static tTree SetNewHome
  pvar inner_home;
 # endif
 {
-# line 622 "MakeHome.puma"
+/* line 623 "MakeHome.puma" */
   {
-# line 632 "MakeHome.puma"
+/* line 625 "MakeHome.puma" */
+
+
+#ifdef DEBUG
+     tree_protocol ("set new home : \n", stmt);
+     tree_protocol ("outer home: ", PrintableDescriptorVar (outer_home));
+     tree_protocol ("inner home: ", PrintableDescriptorVar (inner_home));
+#endif
+   
+/* line 636 "MakeHome.puma" */
    if (! ((VDIsSame (outer_home, inner_home)))) goto yyL1;
   }
    return stmt;
 yyL1:;
 
-# line 637 "MakeHome.puma"
+/* line 641 "MakeHome.puma" */
   {
-# line 641 "MakeHome.puma"
-   if (! ((IsNoDescriptor (inner_home)))) goto yyL2;
+/* line 645 "MakeHome.puma" */
+   if (! ((IsFictiveParDescriptor (inner_home)))) goto yyL2;
   }
    return stmt;
 yyL2:;
 
-# line 646 "MakeHome.puma"
+/* line 650 "MakeHome.puma" */
+  {
+/* line 654 "MakeHome.puma" */
+   if (! ((IsNoDescriptor (inner_home)))) goto yyL3;
+  }
+   return stmt;
+yyL3:;
+
+/* line 659 "MakeHome.puma" */
  {
   tTree home_stmt;
   {
-# line 648 "MakeHome.puma"
-
-# line 650 "MakeHome.puma"
-   if (! ((IsReplicatedDescriptor (outer_home)))) goto yyL3;
+/* line 663 "MakeHome.puma" */
+   if (! ((IsReplicatedDescriptor (outer_home)))) goto yyL4;
   {
-# line 651 "MakeHome.puma"
-   if (! ((IsHostDescriptor (inner_home)))) goto yyL3;
+/* line 664 "MakeHome.puma" */
+   if (! ((IsHostDescriptor (inner_home)))) goto yyL4;
   {
-# line 653 "MakeHome.puma"
-   home_stmt = mACF_HOME (mON_HOST_CLAUSE (), false, MakeStmtList (stmt));
-# line 656 "MakeHome.puma"
+/* line 666 "MakeHome.puma" */
+   home_stmt = mACF_HOME (mON_HOST_CLAUSE (), rfalse, MakeStmtList (stmt));
+/* line 669 "MakeHome.puma" */
    LineACFNode (home_stmt, GetBodyLine (stmt));
   }
   }
   }
-  {
    return home_stmt;
-  }
  }
-yyL3:;
+yyL4:;
 
-# line 661 "MakeHome.puma"
+/* line 674 "MakeHome.puma" */
    return BuildNewVarHome (stmt, inner_home);
 
 }
@@ -849,9 +819,9 @@ static tTree BuildNewVarHome
   if (stmt->ACF_LIST.Elem->Kind == kACF_HOME) {
   if (stmt->ACF_LIST.Elem->ACF_HOME.HOME_VAR->Kind == kON_VAR_CLAUSE) {
   if (stmt->ACF_LIST.Next->Kind == kACF_EMPTY) {
-# line 676 "MakeHome.puma"
+/* line 689 "MakeHome.puma" */
   {
-# line 678 "MakeHome.puma"
+/* line 691 "MakeHome.puma" */
  stmt->ACF_LIST.Elem->ACF_HOME.HOME_VAR->ON_VAR_CLAUSE.ON_VAR = MakeDescriptorVar (home); 
   }
    return stmt;
@@ -862,29 +832,25 @@ static tTree BuildNewVarHome
   }
   if (stmt->Kind == kACF_HOME) {
   if (stmt->ACF_HOME.HOME_VAR->Kind == kON_VAR_CLAUSE) {
-# line 683 "MakeHome.puma"
+/* line 696 "MakeHome.puma" */
   {
-# line 685 "MakeHome.puma"
+/* line 698 "MakeHome.puma" */
  stmt->ACF_HOME.HOME_VAR->ON_VAR_CLAUSE.ON_VAR = MakeDescriptorVar (home); 
   }
    return stmt;
 
   }
   }
-# line 690 "MakeHome.puma"
+/* line 703 "MakeHome.puma" */
  {
   tTree home_stmt;
   {
-# line 692 "MakeHome.puma"
-
-# line 694 "MakeHome.puma"
-   home_stmt = mACF_HOME (mON_VAR_CLAUSE (MakeDescriptorVar (home)), false, MakeStmtList (stmt));
-# line 698 "MakeHome.puma"
+/* line 707 "MakeHome.puma" */
+   home_stmt = mACF_HOME (mON_VAR_CLAUSE (MakeDescriptorVar (home)), rfalse, MakeStmtList (stmt));
+/* line 711 "MakeHome.puma" */
    LineACFNode (home_stmt, GetBodyLine (stmt));
   }
-  {
    return home_stmt;
-  }
  }
 
 }
@@ -897,32 +863,34 @@ static int GetBodyLine
  register tTree stmt;
 # endif
 {
-# line 705 "MakeHome.puma"
+ yyRecursion:
+/* line 718 "MakeHome.puma" */
   {
-# line 706 "MakeHome.puma"
+/* line 719 "MakeHome.puma" */
    if (! ((stmt == NoTree))) goto yyL1;
   }
    return 0;
 yyL1:;
 
   if (stmt->Kind == kACF_LIST) {
-# line 710 "MakeHome.puma"
-   return GetBodyLine (stmt->ACF_LIST.Elem);
+/* line 723 "MakeHome.puma" */
+   stmt = stmt->ACF_LIST.Elem;
+   goto yyRecursion;
 
   }
   if (Tree_IsType (stmt, kACF_NODE)) {
-# line 715 "MakeHome.puma"
+/* line 728 "MakeHome.puma" */
    return stmt->ACF_NODE.Line;
 
   }
   if (stmt->Kind == kACF_EMPTY) {
-# line 720 "MakeHome.puma"
+/* line 733 "MakeHome.puma" */
    return 0;
 
   }
-# line 724 "MakeHome.puma"
+/* line 737 "MakeHome.puma" */
   {
-# line 725 "MakeHome.puma"
+/* line 738 "MakeHome.puma" */
    failure_protocol (MODULE, "GetBodyLine", stmt);
   }
    return 0;
@@ -940,34 +908,28 @@ static void MakeHomeTemplate
  register tTree * yyP1;
 # endif
 {
-# line 745 "MakeHome.puma"
+/* line 758 "MakeHome.puma" */
  {
   tTree tmp_var;
   tTree allocate;
   tTree deallocate;
   {
-# line 747 "MakeHome.puma"
+/* line 760 "MakeHome.puma" */
    if (! ((IsReplicatedDescriptor (home)))) goto yyL1;
   {
-# line 748 "MakeHome.puma"
+/* line 761 "MakeHome.puma" */
    if (! ((! IsReplicatedDescriptor (home1)))) goto yyL1;
   {
-# line 749 "MakeHome.puma"
+/* line 762 "MakeHome.puma" */
    if (! ((! IsNoDescriptor (home1)))) goto yyL1;
   {
-# line 750 "MakeHome.puma"
+/* line 763 "MakeHome.puma" */
    if (! ((IsTemplateDescriptor (home1)))) goto yyL1;
   {
-# line 752 "MakeHome.puma"
+/* line 765 "MakeHome.puma" */
    if (! ((home1 -> var_tree == NoTree))) goto yyL1;
   {
-# line 756 "MakeHome.puma"
-
-# line 757 "MakeHome.puma"
-
-# line 758 "MakeHome.puma"
-
-# line 760 "MakeHome.puma"
+/* line 773 "MakeHome.puma" */
  
 
      home1->type_kind = kDUMMY_TYPE;
@@ -997,7 +959,7 @@ static void MakeHomeTemplate
  }
 yyL1:;
 
-# line 779 "MakeHome.puma"
+/* line 792 "MakeHome.puma" */
    * yyP2 = NoTree;
    * yyP1 = NoTree;
    return;
@@ -1015,12 +977,12 @@ static void SetIterationsHome
 # endif
 {
   if (loop->Kind == kACF_DO) {
-# line 797 "MakeHome.puma"
+/* line 810 "MakeHome.puma" */
   {
-# line 799 "MakeHome.puma"
+/* line 812 "MakeHome.puma" */
    if (! ((loop->ACF_DO.DO_HOME_INFO == NoTree))) goto yyL1;
   {
-# line 801 "MakeHome.puma"
+/* line 814 "MakeHome.puma" */
    failure_protocol (MODULE, "SetIterationsHome", NoTree);
   }
   }
@@ -1028,24 +990,22 @@ static void SetIterationsHome
 yyL1:;
 
   if (loop->ACF_DO.DO_HOME_INFO->Kind == kCOMM_INFO) {
-# line 804 "MakeHome.puma"
+/* line 817 "MakeHome.puma" */
   {
-# line 806 "MakeHome.puma"
+/* line 819 "MakeHome.puma" */
  loop->ACF_DO.DO_HOME_INFO->COMM_INFO.home_var = MakeDescriptorVar (iter_home); 
   }
    return;
 
   }
   if (loop->ACF_DO.DO_HOME_INFO->Kind == kNO_HOME_INFO) {
-# line 809 "MakeHome.puma"
+/* line 822 "MakeHome.puma" */
  {
   tTree hvar;
   {
-# line 811 "MakeHome.puma"
-
-# line 813 "MakeHome.puma"
+/* line 826 "MakeHome.puma" */
  hvar = MakeDescriptorVar (iter_home); 
-    loop->ACF_DO.DO_HOME_INFO = mCOMM_INFO (hvar, false);         
+    loop->ACF_DO.DO_HOME_INFO = mCOMM_INFO (hvar, rfalse);         
   
   }
    return;
@@ -1053,9 +1013,9 @@ yyL1:;
 
   }
   }
-# line 818 "MakeHome.puma"
+/* line 831 "MakeHome.puma" */
   {
-# line 820 "MakeHome.puma"
+/* line 833 "MakeHome.puma" */
    failure_protocol (MODULE, "SetIterationsHome", loop);
   }
    return;
@@ -1074,15 +1034,13 @@ static void SetSelection
 {
   if (loop->Kind == kACF_DO) {
   if (loop->ACF_DO.DO_DEP_INFO->Kind == kINDEP_INFO) {
-# line 831 "MakeHome.puma"
+/* line 844 "MakeHome.puma" */
  {
   int dim;
   {
-# line 833 "MakeHome.puma"
-
-# line 835 "MakeHome.puma"
+/* line 848 "MakeHome.puma" */
    dim = FindDescriptorDim (iter_home, loop->ACF_DO.DO_ID);
-# line 837 "MakeHome.puma"
+/* line 850 "MakeHome.puma" */
  if (dim > 0)
         loop->ACF_DO.DO_DEP_INFO->INDEP_INFO.selection = iter_home->selections[dim-1];
    
@@ -1105,15 +1063,13 @@ static void FindNewHomeDescriptor
  pvar new_home;
 # endif
 {
-# line 873 "MakeHome.puma"
+/* line 886 "MakeHome.puma" */
  {
   var_descriptor min_home;
   {
-# line 875 "MakeHome.puma"
-
-# line 877 "MakeHome.puma"
+/* line 890 "MakeHome.puma" */
    FindHome (stmt, & min_home, new_home);
-# line 879 "MakeHome.puma"
+/* line 892 "MakeHome.puma" */
  if (!VDIsSubSet (&min_home, cur_home))
 
        { tree_protocol ("illegal home constellation :\n", stmt);
@@ -1177,44 +1133,42 @@ static void FindSerialHomeDescriptor
  pvar new_home;
 # endif
 {
-# line 941 "MakeHome.puma"
+/* line 954 "MakeHome.puma" */
   {
-# line 945 "MakeHome.puma"
-   if (! ((GetCurrentModel () != HPF_GLOBAL))) goto yyL1;
+/* line 958 "MakeHome.puma" */
+   if (! ((! IsNonLocalModel (GetCurrentModel ())))) goto yyL1;
   {
-# line 947 "MakeHome.puma"
+/* line 960 "MakeHome.puma" */
  *new_home = *cur_home; 
   }
   }
    return;
 yyL1:;
 
-# line 950 "MakeHome.puma"
+/* line 963 "MakeHome.puma" */
  {
-  bool done;
+  rbool done;
   {
-# line 954 "MakeHome.puma"
-
-# line 958 "MakeHome.puma"
+/* line 971 "MakeHome.puma" */
    FindNewHomeDescriptor (stmt, cur_home, new_home);
-# line 959 "MakeHome.puma"
+/* line 972 "MakeHome.puma" */
    MakeSerialDescriptor (new_home, & done);
-# line 961 "MakeHome.puma"
+/* line 974 "MakeHome.puma" */
    if (! ((done))) goto yyL2;
   }
    return;
  }
 yyL2:;
 
-# line 964 "MakeHome.puma"
+/* line 977 "MakeHome.puma" */
   {
-# line 966 "MakeHome.puma"
+/* line 979 "MakeHome.puma" */
    set_protocol_stmt (stmt);
-# line 967 "MakeHome.puma"
+/* line 980 "MakeHome.puma" */
    error_protocol ("illegal home for SERIAL or IO routine");
-# line 968 "MakeHome.puma"
+/* line 981 "MakeHome.puma" */
    tree_protocol ("home for stmt is: ", PrintableDescriptorVar (cur_home));
-# line 969 "MakeHome.puma"
+/* line 982 "MakeHome.puma" */
  *new_home = *cur_home; 
   }
    return;
@@ -1232,71 +1186,71 @@ static void DetermineFinalHome
  pvar new_home;
 # endif
 {
-# line 998 "MakeHome.puma"
+/* line 1011 "MakeHome.puma" */
   {
-# line 1000 "MakeHome.puma"
+/* line 1013 "MakeHome.puma" */
    if (! ((! VDIsSubSet (min_home, new_home)))) goto yyL1;
   {
-# line 1001 "MakeHome.puma"
+/* line 1014 "MakeHome.puma" */
  *new_home = *min_home; 
   }
   }
    return;
 yyL1:;
 
-# line 1004 "MakeHome.puma"
+/* line 1017 "MakeHome.puma" */
   {
-# line 1006 "MakeHome.puma"
+/* line 1019 "MakeHome.puma" */
    if (! ((IsNoDescriptor (new_home)))) goto yyL2;
   {
-# line 1007 "MakeHome.puma"
+/* line 1020 "MakeHome.puma" */
  *new_home = *cur_home; 
   }
   }
    return;
 yyL2:;
 
-# line 1010 "MakeHome.puma"
+/* line 1023 "MakeHome.puma" */
   {
-# line 1015 "MakeHome.puma"
+/* line 1028 "MakeHome.puma" */
    if (! ((IsFullLoopDescriptor (new_home)))) goto yyL3;
   {
-# line 1016 "MakeHome.puma"
+/* line 1029 "MakeHome.puma" */
  *new_home = *cur_home; 
   }
   }
    return;
 yyL3:;
 
-# line 1019 "MakeHome.puma"
+/* line 1032 "MakeHome.puma" */
   {
-# line 1021 "MakeHome.puma"
+/* line 1034 "MakeHome.puma" */
    if (! ((VDIsSubSet (new_home, cur_home)))) goto yyL4;
   {
-# line 1022 "MakeHome.puma"
+/* line 1035 "MakeHome.puma" */
    InheritTemplate (cur_home, new_home);
   }
   }
    return;
 yyL4:;
 
-# line 1025 "MakeHome.puma"
+/* line 1038 "MakeHome.puma" */
   {
-# line 1027 "MakeHome.puma"
+/* line 1040 "MakeHome.puma" */
    if (! ((IsNoDescriptor (min_home)))) goto yyL5;
   {
-# line 1028 "MakeHome.puma"
+/* line 1041 "MakeHome.puma" */
    IntersectDescriptor (new_home, cur_home);
-# line 1032 "MakeHome.puma"
+/* line 1045 "MakeHome.puma" */
    if (! ((! IsNoDescriptor (new_home)))) goto yyL5;
   }
   }
    return;
 yyL5:;
 
-# line 1035 "MakeHome.puma"
+/* line 1048 "MakeHome.puma" */
   {
-# line 1037 "MakeHome.puma"
+/* line 1050 "MakeHome.puma" */
  *new_home = *cur_home; 
   }
    return;
@@ -1313,9 +1267,9 @@ static void InheritTemplate
  pvar sub_home;
 # endif
 {
-# line 1051 "MakeHome.puma"
+/* line 1064 "MakeHome.puma" */
   {
-# line 1053 "MakeHome.puma"
+/* line 1066 "MakeHome.puma" */
  if (sub_home->template_obj == NoObject)
 
       { sub_home->template_obj = home->template_obj;
@@ -1337,28 +1291,29 @@ static void CheckCallHome
  pvar final_home;
 # endif
 {
+ yyRecursion:
   if (t->Kind == kACF_BASIC) {
-# line 1073 "MakeHome.puma"
+/* line 1086 "MakeHome.puma" */
   {
-# line 1075 "MakeHome.puma"
-   CheckCallHome (t->ACF_BASIC.BASIC_STMT, final_home);
+/* line 1088 "MakeHome.puma" */
+   t = t->ACF_BASIC.BASIC_STMT;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kCALL_STMT) {
-# line 1078 "MakeHome.puma"
+/* line 1091 "MakeHome.puma" */
   {
-# line 1080 "MakeHome.puma"
+/* line 1093 "MakeHome.puma" */
    if (! ((! IsPureCall (t)))) goto yyL2;
   {
-# line 1081 "MakeHome.puma"
+/* line 1094 "MakeHome.puma" */
    if (! ((! IsSerialCall (t)))) goto yyL2;
   {
-# line 1082 "MakeHome.puma"
+/* line 1095 "MakeHome.puma" */
    if (! ((! IsReplicatedDescriptor (final_home)))) goto yyL2;
   {
-# line 1084 "MakeHome.puma"
+/* line 1097 "MakeHome.puma" */
    serious_warning_protocol ("PURE/RESIDENT assumed for subroutine CALL");
   }
   }
@@ -1371,10 +1326,10 @@ yyL2:;
 ;
 }
 
-void BeginMakeHome ()
+void BeginMakeHome ARGS ((void))
 {
 }
 
-void CloseMakeHome ()
+void CloseMakeHome ARGS ((void))
 {
 }
