@@ -1,6 +1,13 @@
 # include "MakeLoops.h"
-# include "yyMakeLoops.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,31 +15,28 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 38 "MakeLoops.puma"
+/* line 39 "MakeLoops.puma" */
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
@@ -55,11 +59,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* MakeLoops_Exit) () = yyExit;
+# include "yyMakeLoops.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* MakeLoops_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -68,34 +86,36 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module MakeLoops, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module MakeLoops, routine %s failed\n",
+  yyFunction);
  MakeLoops_Exit ();
 }
 
-static void FreeTemporaries0 ARGS(());
-tTree SetActualShape ARGS((tTree t, shape s));
-static tTree SetSpreadActualShape ARGS((tTree t, shape s));
-static tTree SetTransposeShape ARGS((tTree t, shape s));
-static void SetActualIndexShape ARGS((tTree ind, shape s, int n));
-tTree MakeOuterLoops ARGS((int line, shape s, tTree body, int kind));
-static tTree MakeListBody ARGS((tTree t));
-tTree MakeOuterImpliedLoops ARGS((shape s, tTree body));
-tTree MakeOuterImpliedLoopsV ARGS((shape s, tTree body));
+static void FreeTemporaries0 ARGS ((void));
+tTree SetActualShape ARGS ((tTree t, shape s));
+static tTree SetSpreadActualShape ARGS ((tTree t, shape s));
+static tTree SetTransposeShape ARGS ((tTree t, shape s));
+static void SetActualIndexShape ARGS ((tTree ind, shape s, int n));
+tTree MakeOuterLoops ARGS ((int line, shape s, tTree body, int kind));
+tTree MakeOuterLoops1 ARGS ((int nest, int line, shape s, tTree body, int kind));
+static tTree MakeListBody ARGS ((tTree t));
+tTree MakeOuterImpliedLoops ARGS ((shape s, tTree body));
+tTree MakeOuterImpliedLoopsV ARGS ((shape s, tTree body));
+static tTree MakeLoopSlice ARGS ((tTree lb, tTree ub, tTree str));
+static void CheckBound ARGS ((tTree exp));
 
 static void FreeTemporaries0
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 66 "MakeLoops.puma"
+/* line 67 "MakeLoops.puma" */
  {
   tTree help;
   {
-# line 67 "MakeLoops.puma"
-
-# line 69 "MakeLoops.puma"
+/* line 70 "MakeLoops.puma" */
  FreeTemporaries (&help, &help); 
   }
    return;
@@ -113,16 +133,16 @@ tTree SetActualShape
  shape s;
 # endif
 {
-# line 85 "MakeLoops.puma"
+/* line 86 "MakeLoops.puma" */
 
 tTree newexp;
 
 
   switch (t->Kind) {
   case kACF_WHERE:
-# line 89 "MakeLoops.puma"
+/* line 90 "MakeLoops.puma" */
   {
-# line 91 "MakeLoops.puma"
+/* line 92 "MakeLoops.puma" */
  t->ACF_WHERE.WHERE_EXP   = SetActualShape (t->ACF_WHERE.WHERE_EXP  , s);
      t->ACF_WHERE.TRUE_PART = SetActualShape (t->ACF_WHERE.TRUE_PART, s);
      t->ACF_WHERE.FALSE_PART = SetActualShape (t->ACF_WHERE.FALSE_PART, s); 
@@ -134,63 +154,63 @@ tTree newexp;
    return t;
 
   case kACF_LIST:
-# line 102 "MakeLoops.puma"
+/* line 103 "MakeLoops.puma" */
   {
-# line 104 "MakeLoops.puma"
+/* line 105 "MakeLoops.puma" */
  t->ACF_LIST.Elem = SetActualShape (t->ACF_LIST.Elem, s);
      t->ACF_LIST.Next = SetActualShape (t->ACF_LIST.Next, s); 
   }
    return t;
 
   case kACF_EMPTY:
-# line 109 "MakeLoops.puma"
+/* line 110 "MakeLoops.puma" */
    return t;
 
   case kACF_BASIC:
-# line 113 "MakeLoops.puma"
+/* line 114 "MakeLoops.puma" */
   {
-# line 114 "MakeLoops.puma"
+/* line 115 "MakeLoops.puma" */
  t->ACF_BASIC.BASIC_STMT = SetActualShape (t->ACF_BASIC.BASIC_STMT, s); 
   }
    return t;
 
   case kASSIGN_STMT:
-# line 118 "MakeLoops.puma"
+/* line 119 "MakeLoops.puma" */
   {
-# line 119 "MakeLoops.puma"
+/* line 120 "MakeLoops.puma" */
  t->ASSIGN_STMT.ASSIGN_VAR = SetActualShape (t->ASSIGN_STMT.ASSIGN_VAR, s);
      t->ASSIGN_STMT.ASSIGN_EXP = SetActualShape (t->ASSIGN_STMT.ASSIGN_EXP, s); 
   }
    return t;
 
   case kOP_EXP:
-# line 124 "MakeLoops.puma"
+/* line 125 "MakeLoops.puma" */
   {
-# line 125 "MakeLoops.puma"
+/* line 126 "MakeLoops.puma" */
  t->OP_EXP.OPND1 = SetActualShape (t->OP_EXP.OPND1, s);
      t->OP_EXP.OPND2 = SetActualShape (t->OP_EXP.OPND2, s); 
   }
    return t;
 
   case kOP1_EXP:
-# line 130 "MakeLoops.puma"
+/* line 131 "MakeLoops.puma" */
   {
-# line 131 "MakeLoops.puma"
+/* line 132 "MakeLoops.puma" */
  t->OP1_EXP.OPND = SetActualShape (t->OP1_EXP.OPND, s); 
   }
    return t;
 
   case kCONST_EXP:
-# line 135 "MakeLoops.puma"
+/* line 136 "MakeLoops.puma" */
    return t;
 
   case kARRAY_EXP:
   if (t->ARRAY_EXP.ELEMENTS->Kind == kBTE_LIST) {
   if (t->ARRAY_EXP.ELEMENTS->BTE_LIST.Elem->Kind == kSLICE_EXP) {
   if (t->ARRAY_EXP.ELEMENTS->BTE_LIST.Next->Kind == kBTE_EMPTY) {
-# line 139 "MakeLoops.puma"
+/* line 140 "MakeLoops.puma" */
   {
-# line 140 "MakeLoops.puma"
+/* line 141 "MakeLoops.puma" */
  if (s->rank != 1)
         { printf ("Illegal formal shape for current array expression\n");
           WriteTree (stdout, t);
@@ -210,9 +230,9 @@ tTree newexp;
   if (t->ARRAY_EXP.ELEMENTS->BTE_LIST.Elem->DO_EXP.BODY->Kind == kBTE_LIST) {
   if (t->ARRAY_EXP.ELEMENTS->BTE_LIST.Elem->DO_EXP.BODY->BTE_LIST.Next->Kind == kBTE_EMPTY) {
   if (t->ARRAY_EXP.ELEMENTS->BTE_LIST.Next->Kind == kBTE_EMPTY) {
-# line 153 "MakeLoops.puma"
+/* line 154 "MakeLoops.puma" */
   {
-# line 156 "MakeLoops.puma"
+/* line 157 "MakeLoops.puma" */
  newexp = t->ARRAY_EXP.ELEMENTS->BTE_LIST.Elem->DO_EXP.BODY->BTE_LIST.Elem;
  
      newexp = FindShapeExp (t->ARRAY_EXP.ELEMENTS->BTE_LIST.Elem->DO_EXP.RANGE, s->bounds[0][0], s->bounds[0][1],
@@ -231,19 +251,19 @@ tTree newexp;
   }
   }
   }
-# line 169 "MakeLoops.puma"
+/* line 170 "MakeLoops.puma" */
   {
-# line 170 "MakeLoops.puma"
+/* line 171 "MakeLoops.puma" */
    error_protocol ("cannot deal with this array expression");
-# line 171 "MakeLoops.puma"
+/* line 172 "MakeLoops.puma" */
    tree_protocol ("array expression is : ", t);
   }
    return t;
 
   case kFUNC_CALL_EXP:
-# line 175 "MakeLoops.puma"
+/* line 176 "MakeLoops.puma" */
   {
-# line 176 "MakeLoops.puma"
+/* line 177 "MakeLoops.puma" */
  
      newexp = t;
      if (IsIntrCall (t))
@@ -270,9 +290,9 @@ tTree newexp;
    return newexp;
 
   case kBTP_LIST:
-# line 201 "MakeLoops.puma"
+/* line 202 "MakeLoops.puma" */
   {
-# line 203 "MakeLoops.puma"
+/* line 204 "MakeLoops.puma" */
  t->BTP_LIST.Elem = SetActualShape (t->BTP_LIST.Elem, s); 
      t->BTP_LIST.Next = SetActualShape (t->BTP_LIST.Next, s);
    
@@ -280,96 +300,100 @@ tTree newexp;
    return t;
 
   case kBTP_EMPTY:
-# line 210 "MakeLoops.puma"
+/* line 211 "MakeLoops.puma" */
    return t;
 
   case kNO_PARAM:
-# line 215 "MakeLoops.puma"
+/* line 216 "MakeLoops.puma" */
    return t;
 
   case kVAR_PARAM:
-# line 220 "MakeLoops.puma"
+/* line 221 "MakeLoops.puma" */
   {
-# line 222 "MakeLoops.puma"
+/* line 223 "MakeLoops.puma" */
  t->VAR_PARAM.V = SetActualShape (t->VAR_PARAM.V, s); 
   }
    return t;
 
   case kADDR:
-# line 226 "MakeLoops.puma"
+/* line 227 "MakeLoops.puma" */
   {
-# line 228 "MakeLoops.puma"
+/* line 229 "MakeLoops.puma" */
  t->ADDR.E = SetActualShape (t->ADDR.E, s); 
   }
    return t;
 
+  case kTYPE_EXP:
+/* line 233 "MakeLoops.puma" */
+   return t;
+
   case kVAR_EXP:
-# line 232 "MakeLoops.puma"
+/* line 238 "MakeLoops.puma" */
   {
-# line 233 "MakeLoops.puma"
+/* line 239 "MakeLoops.puma" */
  t->VAR_EXP.V = SetActualShape (t->VAR_EXP.V, s); 
   }
    return t;
 
   case kUSED_VAR:
-# line 237 "MakeLoops.puma"
+/* line 243 "MakeLoops.puma" */
   {
-# line 239 "MakeLoops.puma"
-   if (! ((TreeRank (t) == 0))) goto yyL19;
+/* line 245 "MakeLoops.puma" */
+   if (! ((TreeRank (t) == 0))) goto yyL20;
   }
    return t;
-yyL19:;
+yyL20:;
 
-# line 243 "MakeLoops.puma"
+/* line 249 "MakeLoops.puma" */
   {
-# line 245 "MakeLoops.puma"
+/* line 251 "MakeLoops.puma" */
    error_protocol ("array variable not in full shape !! ");
-# line 246 "MakeLoops.puma"
+/* line 252 "MakeLoops.puma" */
    tree_protocol ("variable is ", t);
   }
    return t;
 
   case kLOOP_VAR:
-# line 250 "MakeLoops.puma"
+/* line 256 "MakeLoops.puma" */
    return t;
 
   case kSELECTED_VAR:
-# line 254 "MakeLoops.puma"
+/* line 260 "MakeLoops.puma" */
   {
-# line 255 "MakeLoops.puma"
-   if (! (SetActualShape (t->SELECTED_VAR.SELEC_VAR, s))) goto yyL22;
-  }
-   return t;
-yyL22:;
-
-  break;
-  case kINDEXED_VAR:
-  if (t->INDEXED_VAR.IND_VAR->Kind == kSELECTED_VAR) {
-# line 259 "MakeLoops.puma"
-  {
-# line 261 "MakeLoops.puma"
-   if (! ((TreeRank (t->INDEXED_VAR.IND_VAR->SELECTED_VAR.SELEC_VAR) > 0))) goto yyL23;
-  {
-# line 262 "MakeLoops.puma"
-   if (! (SetActualShape (t->INDEXED_VAR.IND_VAR->SELECTED_VAR.SELEC_VAR, s))) goto yyL23;
-  }
+/* line 261 "MakeLoops.puma" */
+   if (! (SetActualShape (t->SELECTED_VAR.SELEC_VAR, s))) goto yyL23;
   }
    return t;
 yyL23:;
 
-  }
-# line 266 "MakeLoops.puma"
+  break;
+  case kINDEXED_VAR:
+  if (t->INDEXED_VAR.IND_VAR->Kind == kSELECTED_VAR) {
+/* line 265 "MakeLoops.puma" */
   {
-# line 268 "MakeLoops.puma"
+/* line 267 "MakeLoops.puma" */
+   if (! ((TreeRank (t->INDEXED_VAR.IND_VAR->SELECTED_VAR.SELEC_VAR) > 0))) goto yyL24;
+  {
+/* line 268 "MakeLoops.puma" */
+   if (! (SetActualShape (t->INDEXED_VAR.IND_VAR->SELECTED_VAR.SELEC_VAR, s))) goto yyL24;
+  }
+  }
+   return t;
+yyL24:;
+
+  }
+/* line 272 "MakeLoops.puma" */
+  {
+/* line 274 "MakeLoops.puma" */
    SetActualIndexShape (t->INDEXED_VAR.IND_EXPS, s, 0);
   }
    return t;
 
   }
 
-# line 272 "MakeLoops.puma"
+/* line 278 "MakeLoops.puma" */
   {
-# line 273 "MakeLoops.puma"
+/* line 279 "MakeLoops.puma" */
    failure_protocol (MODULE, "SetActualShape", t);
   }
    return NoTree;
@@ -385,10 +409,10 @@ static tTree SetSpreadActualShape
  shape s;
 # endif
 {
-# line 279 "MakeLoops.puma"
+/* line 285 "MakeLoops.puma" */
 
 int i, k, dimval;
-bool found;
+rbool found;
 tTree newexp;
 struct_shape h_shp;
 
@@ -399,9 +423,9 @@ struct_shape h_shp;
   if (t->BTP_LIST.Next->BTP_LIST.Elem->VAR_PARAM.V->Kind == kADDR) {
   if (t->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_LIST) {
   if (t->BTP_LIST.Next->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_EMPTY) {
-# line 286 "MakeLoops.puma"
+/* line 292 "MakeLoops.puma" */
   {
-# line 291 "MakeLoops.puma"
+/* line 297 "MakeLoops.puma" */
  
 
     newexp = VarParamToExp (t->BTP_LIST.Elem);
@@ -436,9 +460,9 @@ struct_shape h_shp;
   }
   }
   }
-# line 319 "MakeLoops.puma"
+/* line 325 "MakeLoops.puma" */
   {
-# line 320 "MakeLoops.puma"
+/* line 326 "MakeLoops.puma" */
    error_protocol ("illegal SPREAD for SetSpreadActualShape");
   }
    return t;
@@ -459,13 +483,11 @@ static tTree SetTransposeShape
   if (t->BTP_LIST.Next->Kind == kBTP_LIST) {
   if (t->BTP_LIST.Next->BTP_LIST.Elem->Kind == kNO_PARAM) {
   if (t->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_EMPTY) {
-# line 332 "MakeLoops.puma"
+/* line 338 "MakeLoops.puma" */
  {
   struct_shape tshape;
   {
-# line 335 "MakeLoops.puma"
-
-# line 337 "MakeLoops.puma"
+/* line 343 "MakeLoops.puma" */
  tshape = *s;
     tshape.bounds[0][0] = s->bounds[1][0];
     tshape.bounds[0][1] = s->bounds[1][1];
@@ -478,9 +500,7 @@ static tTree SetTransposeShape
     
   
   }
-  {
    return SetActualShape (VarParamToExp (t->BTP_LIST.Elem), & tshape);
-  }
  }
 
   }
@@ -488,9 +508,9 @@ static tTree SetTransposeShape
   }
   }
   }
-# line 352 "MakeLoops.puma"
+/* line 358 "MakeLoops.puma" */
   {
-# line 354 "MakeLoops.puma"
+/* line 360 "MakeLoops.puma" */
    error_protocol ("illegal TRANSPOSE for SetTransposeShape");
   }
    return t;
@@ -507,20 +527,19 @@ static void SetActualIndexShape
  register int n;
 # endif
 {
-# line 361 "MakeLoops.puma"
+/* line 367 "MakeLoops.puma" */
 
 int rank;
 struct_shape h_shp;
 
+ yyRecursion:
   if (ind->Kind == kBTE_LIST) {
   if (ind->BTE_LIST.Elem->Kind == kSLICE_EXP) {
-# line 366 "MakeLoops.puma"
+/* line 372 "MakeLoops.puma" */
  {
   tTree nexp;
   {
-# line 368 "MakeLoops.puma"
-
-# line 370 "MakeLoops.puma"
+/* line 376 "MakeLoops.puma" */
  nexp = mVAR_EXP (GetLoopTemporary (s->perm[n]));
 
      ind->BTE_LIST.Elem = FindShapeExp (ind->BTE_LIST.Elem, s->bounds[n][0], s->bounds[n][1], 
@@ -533,33 +552,33 @@ struct_shape h_shp;
  }
 
   }
-# line 379 "MakeLoops.puma"
+/* line 385 "MakeLoops.puma" */
   {
-# line 381 "MakeLoops.puma"
+/* line 387 "MakeLoops.puma" */
    if (! ((TreeRank (ind->BTE_LIST.Elem) == 0))) goto yyL2;
   {
-# line 383 "MakeLoops.puma"
-   SetActualIndexShape (ind->BTE_LIST.Next, s, n);
+/* line 389 "MakeLoops.puma" */
+   ind = ind->BTE_LIST.Next;
+   goto yyRecursion;
   }
   }
-   return;
 yyL2:;
 
-# line 386 "MakeLoops.puma"
+/* line 392 "MakeLoops.puma" */
   {
-# line 388 "MakeLoops.puma"
+/* line 394 "MakeLoops.puma" */
    if (! ((TreeRank (ind->BTE_LIST.Elem) != 1))) goto yyL3;
   {
-# line 390 "MakeLoops.puma"
+/* line 396 "MakeLoops.puma" */
    error_protocol ("wrong indirect addressing in SetActualIndexShape");
   }
   }
    return;
 yyL3:;
 
-# line 393 "MakeLoops.puma"
+/* line 399 "MakeLoops.puma" */
   {
-# line 397 "MakeLoops.puma"
+/* line 403 "MakeLoops.puma" */
  
 
     h_shp.rank = 1;
@@ -571,20 +590,21 @@ yyL3:;
     ind->BTE_LIST.Elem = SetActualShape (ind->BTE_LIST.Elem, &h_shp);
 
   
-# line 409 "MakeLoops.puma"
-   SetActualIndexShape (ind->BTE_LIST.Next, s, n + 1);
+/* line 415 "MakeLoops.puma" */
+   ind = ind->BTE_LIST.Next;
+   n = n + 1;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (ind->Kind == kBTE_EMPTY) {
-# line 412 "MakeLoops.puma"
+/* line 418 "MakeLoops.puma" */
    return;
 
   }
-# line 415 "MakeLoops.puma"
+/* line 421 "MakeLoops.puma" */
   {
-# line 417 "MakeLoops.puma"
+/* line 423 "MakeLoops.puma" */
    failure_protocol (MODULE, "SetActualIndexShape", ind);
   }
    return;
@@ -603,36 +623,49 @@ tTree MakeOuterLoops
  register int kind;
 # endif
 {
-# line 432 "MakeLoops.puma"
+/* line 438 "MakeLoops.puma" */
+   return MakeOuterLoops1 (0, line, s, body, kind);
+
+}
+
+tTree MakeOuterLoops1
+# if defined __STDC__ | defined __cplusplus
+(register int nest, register int line, shape s, register tTree body, register int kind)
+# else
+(nest, line, s, body, kind)
+ register int nest;
+ register int line;
+ shape s;
+ register tTree body;
+ register int kind;
+# endif
+{
+/* line 446 "MakeLoops.puma" */
 
 tTree new, var, range, info;
 int i;
 
-# line 443 "MakeLoops.puma"
+/* line 457 "MakeLoops.puma" */
   {
-# line 445 "MakeLoops.puma"
+/* line 459 "MakeLoops.puma" */
  new = body;
     for (i=0; i<s->rank; i++)
       { 
         if (s->bounds[i][0] != s->bounds[i][1])
           { 
             new = MakeListBody (new);
-            var = GetLoopTemporary (s->perm[i]);
+            var = GetLoopTemporary (nest+s->perm[i]);
             
-            if (s->bounds[i][2] != NoTree)
-                range = s->bounds[i][2];
-              else
-                range = mDUMMY_EXP();
-            
-            range = mSLICE_EXP (s->bounds[i][0], 
-                                s->bounds[i][1], range);
+            range = MakeLoopSlice (s->bounds[i][0], 
+                                   s->bounds[i][1],
+                                   s->bounds[i][2]);
             if (kind == GEN_DO_INDEPENDENT)
-               { info = mINDEP_INFO (false, 0, NoTree, NoTree);
+               { info = mINDEP_INFO (rfalse, 0, NoTree, NoTree);
                  new = mACF_DO (var, range, new, info, mNO_HOME_INFO ());
                  LineACFNode (new, line);
                }
              else if (kind == GEN_FORALL)
-               { info = mINDEP_INFO (false, 0, NoTree, NoTree);
+               { info = mINDEP_INFO (rfalse, 0, NoTree, NoTree);
                  new = mACF_FORALL (var, range, new, info, mNO_HOME_INFO());
                  LineACFNode (new, line);
                }
@@ -658,21 +691,22 @@ static tTree MakeListBody
 # endif
 {
   if (t->Kind == kACF_LIST) {
-# line 483 "MakeLoops.puma"
+/* line 493 "MakeLoops.puma" */
    return t;
 
   }
   if (t->Kind == kACF_EMPTY) {
-# line 487 "MakeLoops.puma"
+/* line 497 "MakeLoops.puma" */
    return t;
 
   }
   if (Tree_IsType (t, kACF_NODE)) {
-# line 491 "MakeLoops.puma"
+/* line 501 "MakeLoops.puma" */
    return mACF_LIST (t, mACF_EMPTY ());
 
   }
  yyAbort ("MakeListBody");
+ { tTree yyDummy; return yyDummy; }
 }
 
 tTree MakeOuterImpliedLoops
@@ -684,14 +718,14 @@ tTree MakeOuterImpliedLoops
  register tTree body;
 # endif
 {
-# line 503 "MakeLoops.puma"
+/* line 513 "MakeLoops.puma" */
 
 tTree new, var, range;
 int i;
 
-# line 508 "MakeLoops.puma"
+/* line 518 "MakeLoops.puma" */
   {
-# line 509 "MakeLoops.puma"
+/* line 519 "MakeLoops.puma" */
  new = body;
     for (i=0; i<s->rank; i++)
       { 
@@ -700,13 +734,9 @@ int i;
             new = mBTE_LIST (new, mBTE_EMPTY());
             var = GetLoopTemporary (i+1);
             
-            if (s->bounds[i][2] != NoTree)
-                range = s->bounds[i][2];
-              else
-                range = mDUMMY_EXP();
-            
-            range = mSLICE_EXP (s->bounds[i][0], 
-                                s->bounds[i][1], range);
+            range = MakeLoopSlice (s->bounds[i][0], 
+                                   s->bounds[i][1],
+                                   s->bounds[i][2]);
             new = mDO_EXP (var, range, new);
           }
       }
@@ -725,14 +755,14 @@ tTree MakeOuterImpliedLoopsV
  register tTree body;
 # endif
 {
-# line 539 "MakeLoops.puma"
+/* line 545 "MakeLoops.puma" */
 
 tTree new, var, range;
 int i;
 
-# line 544 "MakeLoops.puma"
+/* line 550 "MakeLoops.puma" */
   {
-# line 545 "MakeLoops.puma"
+/* line 551 "MakeLoops.puma" */
  new = body;
     for (i=0; i<s->rank; i++)
       { 
@@ -757,10 +787,61 @@ int i;
 
 }
 
-void BeginMakeLoops ()
+static tTree MakeLoopSlice
+# if defined __STDC__ | defined __cplusplus
+(register tTree lb, register tTree ub, register tTree str)
+# else
+(lb, ub, str)
+ register tTree lb;
+ register tTree ub;
+ register tTree str;
+# endif
+{
+ yyRecursion:
+/* line 584 "MakeLoops.puma" */
+  {
+/* line 586 "MakeLoops.puma" */
+   if (! ((str == NoTree))) goto yyL1;
+  }
+   str = mDUMMY_EXP ();
+   goto yyRecursion;
+yyL1:;
+
+/* line 591 "MakeLoops.puma" */
+  {
+/* line 593 "MakeLoops.puma" */
+   CheckBound (lb);
+/* line 594 "MakeLoops.puma" */
+   CheckBound (ub);
+  }
+   return mSLICE_EXP (lb, ub, str);
+
+}
+
+static void CheckBound
+# if defined __STDC__ | defined __cplusplus
+(register tTree exp)
+# else
+(exp)
+ register tTree exp;
+# endif
+{
+  if (exp->Kind == kBOUND_EXP) {
+/* line 603 "MakeLoops.puma" */
+  {
+/* line 605 "MakeLoops.puma" */
+ exp->BOUND_EXP.VAR = CopyTree (exp->BOUND_EXP.VAR); 
+  }
+   return;
+
+  }
+;
+}
+
+void BeginMakeLoops ARGS ((void))
 {
 }
 
-void CloseMakeLoops ()
+void CloseMakeLoops ARGS ((void))
 {
 }
