@@ -1,6 +1,13 @@
 # include "MakeMovement.h"
-# include "yyMakeMovement.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,32 +15,29 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 26 "MakeMovement.puma"
+/* line 26 "MakeMovement.puma" */
 
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
@@ -53,11 +57,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* MakeMovement_Exit) () = yyExit;
+# include "yyMakeMovement.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* MakeMovement_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -66,22 +84,23 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module MakeMovement, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module MakeMovement, routine %s failed\n",
+  yyFunction);
  MakeMovement_Exit ();
 }
 
-void MakeMovement ARGS((tTree t));
-static bool StopMakeMoveStmts ARGS((tTree t));
-static tTree DoMakeMoveStmts ARGS((tTree t));
-static void IsOnAssignment ARGS((tTree var, tTree exp, bool * yyP2, tTree * yyP1));
-int TranslateMovement ARGS((tTree stmt));
-static void TransMoveStmt ARGS((tTree stmt, int * yyP4, tTree * yyP3));
-static tTree SetMoveMask ARGS((tTree stmt, tTree mask));
-static tTree AddMask ARGS((tTree mask1, tTree mask2));
-static void TranslateAssignment ARGS((tTree assign, int * yyP6, tTree * yyP5));
-static void TranslateReduce ARGS((tTree reduce, int * yyP8, tTree * yyP7));
-static tTree TranslateIndexes ARGS((tTree index_list));
-static void MakeLocalInfo ARGS((tTree t));
+void MakeMovement ARGS ((tTree t));
+static rbool StopMakeMoveStmts ARGS ((tTree t));
+static tTree DoMakeMoveStmts ARGS ((tTree t));
+static void IsOnAssignment ARGS ((tTree var, tTree exp, rbool * yyP2, tTree * yyP1));
+int TranslateMovement ARGS ((tTree stmt));
+static void TransMoveStmt ARGS ((tTree stmt, int * yyP4, tTree * yyP3));
+static tTree SetMoveMask ARGS ((tTree stmt, tTree mask));
+static tTree AddMask ARGS ((tTree mask1, tTree mask2));
+static void TranslateAssignment ARGS ((tTree assign, int * yyP6, tTree * yyP5));
+static void TranslateReduce ARGS ((tTree reduce, int * yyP8, tTree * yyP7));
+static tTree TranslateIndexes ARGS ((tTree index_list));
+static void MakeLocalInfo ARGS ((tTree t));
 
 void MakeMovement
 # if defined __STDC__ | defined __cplusplus
@@ -92,17 +111,17 @@ void MakeMovement
 # endif
 {
   if (t->Kind == kBODY_NODE) {
-# line 58 "MakeMovement.puma"
+/* line 58 "MakeMovement.puma" */
   {
-# line 60 "MakeMovement.puma"
+/* line 60 "MakeMovement.puma" */
  t->BODY_NODE.STATS = ReplaceAST (t->BODY_NODE.STATS, StopMakeMoveStmts, DoMakeMoveStmts); 
   }
    return;
 
   }
-# line 63 "MakeMovement.puma"
+/* line 63 "MakeMovement.puma" */
   {
-# line 64 "MakeMovement.puma"
+/* line 64 "MakeMovement.puma" */
    failure_protocol (MODULE, "MakeMovements", t);
   }
    return;
@@ -110,7 +129,7 @@ void MakeMovement
 ;
 }
 
-static bool StopMakeMoveStmts
+static rbool StopMakeMoveStmts
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -119,21 +138,21 @@ static bool StopMakeMoveStmts
 # endif
 {
   if (t->Kind == kACF_FORALL) {
-# line 80 "MakeMovement.puma"
-   return true;
+/* line 80 "MakeMovement.puma" */
+   return rtrue;
 
   }
   if (t->Kind == kACF_WHERE) {
-# line 83 "MakeMovement.puma"
-   return true;
+/* line 83 "MakeMovement.puma" */
+   return rtrue;
 
   }
   if (t->Kind == kACF_HOME) {
-# line 86 "MakeMovement.puma"
-   return true;
+/* line 86 "MakeMovement.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
 static tTree DoMakeMoveStmts
@@ -145,18 +164,14 @@ static tTree DoMakeMoveStmts
 # endif
 {
   if (t->Kind == kACF_FORALL) {
-# line 91 "MakeMovement.puma"
+/* line 91 "MakeMovement.puma" */
  {
   int code;
   tTree new;
   {
-# line 93 "MakeMovement.puma"
-
-# line 94 "MakeMovement.puma"
-
-# line 96 "MakeMovement.puma"
+/* line 96 "MakeMovement.puma" */
    code = TranslateMovement (t);
-# line 98 "MakeMovement.puma"
+/* line 98 "MakeMovement.puma" */
  if (code == 0)
 
         { new = DoDetermineHome (t);
@@ -177,93 +192,83 @@ static tTree DoMakeMoveStmts
 
    
   }
-  {
    return new;
-  }
  }
 
   }
   if (t->Kind == kACF_BASIC) {
   if (t->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
-# line 123 "MakeMovement.puma"
+/* line 123 "MakeMovement.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   tTree yyV2;
   tTree new;
   {
-# line 125 "MakeMovement.puma"
+/* line 125 "MakeMovement.puma" */
    IsOnAssignment (t->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR, t->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_EXP, & yyV1, & yyV2);
-# line 127 "MakeMovement.puma"
+/* line 127 "MakeMovement.puma" */
    if (! ((yyV1))) goto yyL2;
   {
-# line 129 "MakeMovement.puma"
-
-# line 131 "MakeMovement.puma"
+/* line 131 "MakeMovement.puma" */
    stmt_protocol ("statement will be executed by owner");
-# line 133 "MakeMovement.puma"
-   new = mACF_HOME (mON_VAR_CLAUSE (yyV2), false, mACF_LIST (t, mACF_EMPTY ()));
-# line 137 "MakeMovement.puma"
+/* line 133 "MakeMovement.puma" */
+   new = mACF_HOME (mON_VAR_CLAUSE (yyV2), rfalse, mACF_LIST (t, mACF_EMPTY ()));
+/* line 137 "MakeMovement.puma" */
    tree_protocol ("new home stmt : \n", new);
   }
   }
-  {
    return new;
-  }
  }
 yyL2:;
 
-# line 142 "MakeMovement.puma"
+/* line 142 "MakeMovement.puma" */
  {
   int code;
   {
-# line 144 "MakeMovement.puma"
-
-# line 146 "MakeMovement.puma"
+/* line 146 "MakeMovement.puma" */
    code = TranslateMovement (t);
   }
-  {
    return t;
-  }
  }
 
   }
   }
-# line 151 "MakeMovement.puma"
+/* line 151 "MakeMovement.puma" */
    return t;
 
 }
 
 static void IsOnAssignment
 # if defined __STDC__ | defined __cplusplus
-(register tTree var, register tTree exp, register bool * yyP2, register tTree * yyP1)
+(register tTree var, register tTree exp, register rbool * yyP2, register tTree * yyP1)
 # else
 (var, exp, yyP2, yyP1)
  register tTree var;
  register tTree exp;
- register bool * yyP2;
+ register rbool * yyP2;
  register tTree * yyP1;
 # endif
 {
   if (var->Kind == kUSED_VAR) {
   if (exp->Kind == kVAR_EXP) {
-# line 168 "MakeMovement.puma"
+/* line 168 "MakeMovement.puma" */
   {
-# line 170 "MakeMovement.puma"
+/* line 170 "MakeMovement.puma" */
    if (! ((TreeRank (var) == 0))) goto yyL1;
   {
-# line 171 "MakeMovement.puma"
+/* line 171 "MakeMovement.puma" */
    if (! ((TreeRank (exp->VAR_EXP.V) == 0))) goto yyL1;
   {
-# line 173 "MakeMovement.puma"
+/* line 173 "MakeMovement.puma" */
    if (! ((CountMovements (var, exp->VAR_EXP.V) != 0))) goto yyL1;
   {
-# line 174 "MakeMovement.puma"
+/* line 174 "MakeMovement.puma" */
    if (! ((CountMovements (exp->VAR_EXP.V, var) == 0))) goto yyL1;
   }
   }
   }
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    * yyP1 = exp->VAR_EXP.V;
    return;
 yyL1:;
@@ -272,32 +277,32 @@ yyL1:;
   }
   if (exp->Kind == kVAR_EXP) {
   if (exp->VAR_EXP.V->Kind == kUSED_VAR) {
-# line 179 "MakeMovement.puma"
+/* line 179 "MakeMovement.puma" */
   {
-# line 181 "MakeMovement.puma"
+/* line 181 "MakeMovement.puma" */
    if (! ((TreeRank (var) == 0))) goto yyL2;
   {
-# line 182 "MakeMovement.puma"
+/* line 182 "MakeMovement.puma" */
    if (! ((TreeRank (exp->VAR_EXP.V) == 0))) goto yyL2;
   {
-# line 184 "MakeMovement.puma"
+/* line 184 "MakeMovement.puma" */
    if (! ((CountMovements (var, exp->VAR_EXP.V) == 0))) goto yyL2;
   {
-# line 185 "MakeMovement.puma"
+/* line 185 "MakeMovement.puma" */
    if (! ((CountMovements (exp->VAR_EXP.V, var) != 0))) goto yyL2;
   }
   }
   }
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    * yyP1 = var;
    return;
 yyL2:;
 
   }
   }
-# line 188 "MakeMovement.puma"
-   * yyP2 = false;
+/* line 188 "MakeMovement.puma" */
+   * yyP2 = rfalse;
    * yyP1 = NoTree;
    return;
 
@@ -312,25 +317,23 @@ int TranslateMovement
  register tTree stmt;
 # endif
 {
-# line 235 "MakeMovement.puma"
+/* line 235 "MakeMovement.puma" */
  {
   int yyV1;
   tTree yyV2;
   {
-# line 237 "MakeMovement.puma"
+/* line 237 "MakeMovement.puma" */
    stmt_protocol ("movement will be classified");
-# line 239 "MakeMovement.puma"
+/* line 239 "MakeMovement.puma" */
    TransMoveStmt (stmt, & yyV1, & yyV2);
-# line 241 "MakeMovement.puma"
+/* line 241 "MakeMovement.puma" */
  if (yyV1 == 0)
        tree_protocol ("is local stmt:\n", stmt);
      else
        tree_protocol ("is movement stmt :\n", stmt);
   
   }
-  {
    return yyV1;
-  }
  }
 
 }
@@ -346,12 +349,12 @@ static void TransMoveStmt
 # endif
 {
   if (stmt->Kind == kACF_FORALL) {
-# line 263 "MakeMovement.puma"
+/* line 263 "MakeMovement.puma" */
  {
   int yyV1;
   tTree yyV2;
   {
-# line 265 "MakeMovement.puma"
+/* line 265 "MakeMovement.puma" */
    TransMoveStmt (stmt->ACF_FORALL.FORALL_BODY, & yyV1, & yyV2);
   }
    * yyP4 = yyV1;
@@ -361,14 +364,14 @@ static void TransMoveStmt
 
   }
   if (stmt->Kind == kACF_IF) {
-# line 268 "MakeMovement.puma"
+/* line 268 "MakeMovement.puma" */
  {
   int yyV1;
   tTree yyV2;
   {
-# line 270 "MakeMovement.puma"
+/* line 270 "MakeMovement.puma" */
    TransMoveStmt (stmt->ACF_IF.THEN_PART, & yyV1, & yyV2);
-# line 272 "MakeMovement.puma"
+/* line 272 "MakeMovement.puma" */
  if (yyV1 == 0)
        { 
          stmt->ACF_IF.THEN_PART = yyV2; 
@@ -386,14 +389,14 @@ static void TransMoveStmt
   }
   if (stmt->Kind == kACF_LIST) {
   if (stmt->ACF_LIST.Next->Kind == kACF_EMPTY) {
-# line 282 "MakeMovement.puma"
+/* line 282 "MakeMovement.puma" */
  {
   int yyV1;
   tTree yyV2;
   {
-# line 284 "MakeMovement.puma"
+/* line 284 "MakeMovement.puma" */
    TransMoveStmt (stmt->ACF_LIST.Elem, & yyV1, & yyV2);
-# line 285 "MakeMovement.puma"
+/* line 285 "MakeMovement.puma" */
  stmt->ACF_LIST.Elem = yyV2; 
   }
    * yyP4 = yyV1;
@@ -402,18 +405,18 @@ static void TransMoveStmt
  }
 
   }
-# line 288 "MakeMovement.puma"
+/* line 288 "MakeMovement.puma" */
  {
   int yyV1;
   tTree yyV2;
   int yyV3;
   tTree yyV4;
   {
-# line 292 "MakeMovement.puma"
+/* line 292 "MakeMovement.puma" */
    TransMoveStmt (stmt->ACF_LIST.Elem, & yyV1, & yyV2);
-# line 293 "MakeMovement.puma"
+/* line 293 "MakeMovement.puma" */
    TransMoveStmt (stmt->ACF_LIST.Next, & yyV3, & yyV4);
-# line 295 "MakeMovement.puma"
+/* line 295 "MakeMovement.puma" */
  if (yyV1 || yyV3)
        error_protocol ("more than one movement in FORALL nest");
   
@@ -426,14 +429,14 @@ static void TransMoveStmt
   }
   if (stmt->Kind == kACF_BASIC) {
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
-# line 300 "MakeMovement.puma"
+/* line 300 "MakeMovement.puma" */
  {
   int yyV1;
   tTree yyV2;
   {
-# line 302 "MakeMovement.puma"
+/* line 302 "MakeMovement.puma" */
    TranslateAssignment (stmt->ACF_BASIC.BASIC_STMT, & yyV1, & yyV2);
-# line 303 "MakeMovement.puma"
+/* line 303 "MakeMovement.puma" */
  stmt->ACF_BASIC.BASIC_STMT = yyV2; 
   }
    * yyP4 = yyV1;
@@ -443,14 +446,14 @@ static void TransMoveStmt
 
   }
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kREDUCE_STMT) {
-# line 306 "MakeMovement.puma"
+/* line 306 "MakeMovement.puma" */
  {
   int yyV1;
   tTree yyV2;
   {
-# line 308 "MakeMovement.puma"
+/* line 308 "MakeMovement.puma" */
    TranslateReduce (stmt->ACF_BASIC.BASIC_STMT, & yyV1, & yyV2);
-# line 309 "MakeMovement.puma"
+/* line 309 "MakeMovement.puma" */
  stmt->ACF_BASIC.BASIC_STMT = yyV2; 
   }
    * yyP4 = yyV1;
@@ -460,9 +463,9 @@ static void TransMoveStmt
 
   }
   }
-# line 312 "MakeMovement.puma"
+/* line 312 "MakeMovement.puma" */
   {
-# line 313 "MakeMovement.puma"
+/* line 313 "MakeMovement.puma" */
    failure_protocol (MODULE, "TransMoveStmt", stmt);
   }
    * yyP4 = 0;
@@ -481,36 +484,49 @@ static tTree SetMoveMask
  register tTree mask;
 # endif
 {
+ yyRecursion:
   if (stmt->Kind == kACF_LIST) {
   if (stmt->ACF_LIST.Next->Kind == kACF_EMPTY) {
-# line 327 "MakeMovement.puma"
-   return SetMoveMask (stmt->ACF_LIST.Elem, mask);
+/* line 327 "MakeMovement.puma" */
+   stmt = stmt->ACF_LIST.Elem;
+   goto yyRecursion;
 
   }
   }
   if (stmt->Kind == kACF_BASIC) {
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kSCATTER_STMT) {
-# line 332 "MakeMovement.puma"
+/* line 332 "MakeMovement.puma" */
   {
-# line 334 "MakeMovement.puma"
+/* line 334 "MakeMovement.puma" */
  stmt->ACF_BASIC.BASIC_STMT->SCATTER_STMT.MASK = AddMask (stmt->ACF_BASIC.BASIC_STMT->SCATTER_STMT.MASK, mask); 
   }
    return stmt;
 
   }
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kMOVE_STMT) {
-# line 339 "MakeMovement.puma"
+/* line 339 "MakeMovement.puma" */
   {
-# line 341 "MakeMovement.puma"
+/* line 341 "MakeMovement.puma" */
  stmt->ACF_BASIC.BASIC_STMT->MOVE_STMT.MASK = AddMask (stmt->ACF_BASIC.BASIC_STMT->MOVE_STMT.MASK, mask); 
   }
    return stmt;
 
   }
   }
-# line 346 "MakeMovement.puma"
+  if (stmt->Kind == kACF_FORALL) {
+/* line 346 "MakeMovement.puma" */
   {
-# line 348 "MakeMovement.puma"
+/* line 348 "MakeMovement.puma" */
+   set_protocol_stmt (stmt);
+/* line 349 "MakeMovement.puma" */
+   serious_warning_protocol ("mask not applied to communication stmt");
+  }
+   return stmt;
+
+  }
+/* line 354 "MakeMovement.puma" */
+  {
+/* line 356 "MakeMovement.puma" */
    failure_protocol (MODULE, "SetMoveMask", stmt);
   }
    return NoTree;
@@ -527,13 +543,13 @@ static tTree AddMask
 # endif
 {
   if (mask1->Kind == kNO_PARAM) {
-# line 361 "MakeMovement.puma"
+/* line 369 "MakeMovement.puma" */
    return ExpToVarParam (mask2);
 
   }
-# line 366 "MakeMovement.puma"
+/* line 374 "MakeMovement.puma" */
   {
-# line 367 "MakeMovement.puma"
+/* line 375 "MakeMovement.puma" */
    error_protocol ("too many masks for data movement");
   }
    return mask1;
@@ -551,12 +567,12 @@ static void TranslateAssignment
 # endif
 {
   if (assign->Kind == kASSIGN_STMT) {
-# line 389 "MakeMovement.puma"
+/* line 397 "MakeMovement.puma" */
   {
-# line 391 "MakeMovement.puma"
+/* line 399 "MakeMovement.puma" */
    if (! ((CountMovements (assign->ASSIGN_STMT.ASSIGN_VAR, assign->ASSIGN_STMT.ASSIGN_EXP) == 0))) goto yyL1;
   {
-# line 392 "MakeMovement.puma"
+/* line 400 "MakeMovement.puma" */
    if (! ((IsDescriptorVar (assign->ASSIGN_STMT.ASSIGN_VAR)))) goto yyL1;
   }
   }
@@ -566,19 +582,19 @@ static void TranslateAssignment
 yyL1:;
 
   if (assign->ASSIGN_STMT.ASSIGN_EXP->Kind == kFUNC_CALL_EXP) {
-# line 395 "MakeMovement.puma"
+/* line 403 "MakeMovement.puma" */
    * yyP6 = 5;
    * yyP5 = assign;
    return;
 
   }
   if (assign->ASSIGN_STMT.ASSIGN_EXP->Kind == kVAR_EXP) {
-# line 398 "MakeMovement.puma"
+/* line 406 "MakeMovement.puma" */
   {
-# line 401 "MakeMovement.puma"
+/* line 409 "MakeMovement.puma" */
    if (! ((IsDescriptorVar (assign->ASSIGN_STMT.ASSIGN_VAR)))) goto yyL3;
   {
-# line 402 "MakeMovement.puma"
+/* line 410 "MakeMovement.puma" */
    if (! ((IsDescriptorVar (assign->ASSIGN_STMT.ASSIGN_EXP->VAR_EXP.V)))) goto yyL3;
   }
   }
@@ -588,17 +604,15 @@ yyL1:;
 yyL3:;
 
   if (assign->ASSIGN_STMT.ASSIGN_EXP->VAR_EXP.V->Kind == kINDEXED_VAR) {
-# line 405 "MakeMovement.puma"
+/* line 413 "MakeMovement.puma" */
  {
   tTree new;
   {
-# line 407 "MakeMovement.puma"
+/* line 415 "MakeMovement.puma" */
    if (! ((IsDescriptorVar (assign->ASSIGN_STMT.ASSIGN_VAR)))) goto yyL4;
   {
-# line 411 "MakeMovement.puma"
-
-# line 413 "MakeMovement.puma"
-   if (! (new = mSCATTER_STMT (MakeIdent ("GET_SCATTER", 11), assign->ASSIGN_STMT.ASSIGN_VAR, MakeFullShape (assign->ASSIGN_STMT.ASSIGN_EXP->VAR_EXP.V->INDEXED_VAR.IND_VAR), TranslateIndexes (assign->ASSIGN_STMT.ASSIGN_EXP->VAR_EXP.V->INDEXED_VAR.IND_EXPS), mNO_PARAM (kDUMMY_TYPE)))) goto yyL4;
+/* line 421 "MakeMovement.puma" */
+   new = mSCATTER_STMT (MakeIdent ("GET_SCATTER", 11), assign->ASSIGN_STMT.ASSIGN_VAR, MakeFullShape (assign->ASSIGN_STMT.ASSIGN_EXP->VAR_EXP.V->INDEXED_VAR.IND_VAR), TranslateIndexes (assign->ASSIGN_STMT.ASSIGN_EXP->VAR_EXP.V->INDEXED_VAR.IND_EXPS), mNO_PARAM (kDUMMY_TYPE));
   }
   }
    * yyP6 = 3;
@@ -611,16 +625,14 @@ yyL4:;
   }
   if (assign->ASSIGN_STMT.ASSIGN_VAR->Kind == kINDEXED_VAR) {
   if (assign->ASSIGN_STMT.ASSIGN_EXP->Kind == kVAR_EXP) {
-# line 418 "MakeMovement.puma"
+/* line 426 "MakeMovement.puma" */
  {
   tTree new;
   {
-# line 420 "MakeMovement.puma"
+/* line 428 "MakeMovement.puma" */
    if (! ((IsDescriptorVar (assign->ASSIGN_STMT.ASSIGN_EXP->VAR_EXP.V)))) goto yyL5;
   {
-# line 424 "MakeMovement.puma"
-
-# line 426 "MakeMovement.puma"
+/* line 434 "MakeMovement.puma" */
    new = mSCATTER_STMT (MakeIdent ("COPY_SCATTER", 12), assign->ASSIGN_STMT.ASSIGN_EXP->VAR_EXP.V, MakeFullShape (assign->ASSIGN_STMT.ASSIGN_VAR->INDEXED_VAR.IND_VAR), TranslateIndexes (assign->ASSIGN_STMT.ASSIGN_VAR->INDEXED_VAR.IND_EXPS), mNO_PARAM (kDUMMY_TYPE));
   }
   }
@@ -633,9 +645,9 @@ yyL5:;
   }
   }
   }
-# line 431 "MakeMovement.puma"
+/* line 439 "MakeMovement.puma" */
   {
-# line 433 "MakeMovement.puma"
+/* line 441 "MakeMovement.puma" */
    error_protocol ("could not translate/classify assignment");
   }
    * yyP6 = 0;
@@ -655,7 +667,7 @@ static void TranslateReduce
  register tTree * yyP7;
 # endif
 {
-# line 448 "MakeMovement.puma"
+/* line 456 "MakeMovement.puma" */
 
 char fn_name      [20];
 char scatter_name [30];
@@ -667,20 +679,18 @@ char scatter_name [30];
   if (reduce->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->Kind == kBTP_LIST) {
   if (reduce->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->BTP_LIST.Elem->Kind == kVAR_PARAM) {
   if (reduce->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_EMPTY) {
-# line 453 "MakeMovement.puma"
+/* line 461 "MakeMovement.puma" */
  {
   tTree new;
   {
-# line 457 "MakeMovement.puma"
+/* line 465 "MakeMovement.puma" */
    if (! ((IsDescriptorVar (reduce->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->BTP_LIST.Elem->VAR_PARAM.V)))) goto yyL1;
   {
-# line 461 "MakeMovement.puma"
-
-# line 463 "MakeMovement.puma"
+/* line 471 "MakeMovement.puma" */
    GetString (reduce->REDUCE_STMT.RED_FUNC->PROC_OBJ.Ident, fn_name);
-# line 465 "MakeMovement.puma"
+/* line 473 "MakeMovement.puma" */
    sprintf (scatter_name, "%s_SCATTER", fn_name);
-# line 467 "MakeMovement.puma"
+/* line 475 "MakeMovement.puma" */
    new = mSCATTER_STMT (IsIdent (scatter_name), reduce->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->BTP_LIST.Elem->VAR_PARAM.V, MakeFullShape (reduce->REDUCE_STMT.RED_PARAMS->BTP_LIST.Elem->VAR_PARAM.V->INDEXED_VAR.IND_VAR), TranslateIndexes (reduce->REDUCE_STMT.RED_PARAMS->BTP_LIST.Elem->VAR_PARAM.V->INDEXED_VAR.IND_EXPS), mNO_PARAM (kDUMMY_TYPE));
   }
   }
@@ -697,9 +707,9 @@ yyL1:;
   }
   }
   }
-# line 472 "MakeMovement.puma"
+/* line 480 "MakeMovement.puma" */
   {
-# line 474 "MakeMovement.puma"
+/* line 482 "MakeMovement.puma" */
    error_protocol ("could not translate/classify reduce");
   }
    * yyP8 = 0;
@@ -719,24 +729,24 @@ static tTree TranslateIndexes
 {
   if (index_list->Kind == kBTE_LIST) {
   if (index_list->BTE_LIST.Elem->Kind == kVAR_EXP) {
-# line 485 "MakeMovement.puma"
+/* line 493 "MakeMovement.puma" */
    return mBTP_LIST (mVAR_PARAM (index_list->BTE_LIST.Elem->VAR_EXP.V), TranslateIndexes (index_list->BTE_LIST.Next));
 
   }
   if (index_list->BTE_LIST.Elem->Kind == kCONST_EXP) {
-# line 490 "MakeMovement.puma"
+/* line 498 "MakeMovement.puma" */
    return mBTP_LIST (ExpToVarParam (index_list->BTE_LIST.Elem), TranslateIndexes (index_list->BTE_LIST.Next));
 
   }
   }
   if (index_list->Kind == kBTE_EMPTY) {
-# line 495 "MakeMovement.puma"
+/* line 503 "MakeMovement.puma" */
    return mBTP_EMPTY ();
 
   }
-# line 500 "MakeMovement.puma"
+/* line 508 "MakeMovement.puma" */
   {
-# line 501 "MakeMovement.puma"
+/* line 509 "MakeMovement.puma" */
    failure_protocol (MODULE, "TranslateIndexes", index_list);
   }
    return NoTree;
@@ -751,63 +761,64 @@ static void MakeLocalInfo
  register tTree t;
 # endif
 {
+ yyRecursion:
   if (t->Kind == kACF_DO) {
   if (t->ACF_DO.DO_HOME_INFO->Kind == kCOMM_INFO) {
-# line 516 "MakeMovement.puma"
+/* line 524 "MakeMovement.puma" */
   {
-# line 518 "MakeMovement.puma"
+/* line 526 "MakeMovement.puma" */
    MakeLocalInfo (t->ACF_DO.DO_BODY);
-# line 520 "MakeMovement.puma"
- t->ACF_DO.DO_HOME_INFO->COMM_INFO.is_local = true; 
+/* line 528 "MakeMovement.puma" */
+ t->ACF_DO.DO_HOME_INFO->COMM_INFO.is_local = rtrue; 
   }
    return;
 
   }
   }
   if (t->Kind == kACF_IF) {
-# line 523 "MakeMovement.puma"
+/* line 531 "MakeMovement.puma" */
   {
-# line 525 "MakeMovement.puma"
+/* line 533 "MakeMovement.puma" */
    MakeLocalInfo (t->ACF_IF.THEN_PART);
-# line 526 "MakeMovement.puma"
-   MakeLocalInfo (t->ACF_IF.ELSE_PART);
+/* line 534 "MakeMovement.puma" */
+   t = t->ACF_IF.ELSE_PART;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kACF_LIST) {
-# line 529 "MakeMovement.puma"
+/* line 537 "MakeMovement.puma" */
   {
-# line 531 "MakeMovement.puma"
+/* line 539 "MakeMovement.puma" */
    MakeLocalInfo (t->ACF_LIST.Elem);
-# line 532 "MakeMovement.puma"
-   MakeLocalInfo (t->ACF_LIST.Next);
+/* line 540 "MakeMovement.puma" */
+   t = t->ACF_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kACF_EMPTY) {
-# line 535 "MakeMovement.puma"
+/* line 543 "MakeMovement.puma" */
    return;
 
   }
   if (t->Kind == kACF_BASIC) {
-# line 538 "MakeMovement.puma"
+/* line 546 "MakeMovement.puma" */
    return;
 
   }
   if (t->Kind == kACF_HOME) {
-# line 541 "MakeMovement.puma"
+/* line 549 "MakeMovement.puma" */
   {
-# line 543 "MakeMovement.puma"
-   MakeLocalInfo (t->ACF_HOME.HOME_BODY);
+/* line 551 "MakeMovement.puma" */
+   t = t->ACF_HOME.HOME_BODY;
+   goto yyRecursion;
   }
-   return;
 
   }
-# line 546 "MakeMovement.puma"
+/* line 554 "MakeMovement.puma" */
   {
-# line 548 "MakeMovement.puma"
+/* line 556 "MakeMovement.puma" */
    failure_protocol (MODULE, "MakeLocalInfo", t);
   }
    return;
@@ -815,10 +826,10 @@ static void MakeLocalInfo
 ;
 }
 
-void BeginMakeMovement ()
+void BeginMakeMovement ARGS ((void))
 {
 }
 
-void CloseMakeMovement ()
+void CloseMakeMovement ARGS ((void))
 {
 }
