@@ -1,6 +1,13 @@
 # include "MakeOvUpdates.h"
-# include "yyMakeOvUpdates.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,49 +15,64 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 32 "MakeOvUpdates.puma"
+/* line 32 "MakeOvUpdates.puma" */
 
 
 # define MODULE "MakeOvUpdates"
 
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 # include "protocol.h"
 
+# include "Objects.h"
+# include "Rank.h"
 # include "Types.h"
-# include "Transform.h"             /* ReplaceACF       */
-# include "Expressions.h"           /* MakeConstant     */
+# include "Transform.h"             /* ReplaceACF            */
+# include "Expressions.h"           /* MakeConstant          */
 
-# include "MoveControl.h"           /* GetShifting, ... */
+# include "MoveControl.h"           /* GetShifting, ...      */
 # include "OverlapUpdate.h"         /* SetOverlapUpdate, ... */
+# include "RegShadow.h"             /* HasEnoughShadow       */
 
 
 
-static void yyExit () { Exit (1); }
 
-void (* MakeOvUpdates_Exit) () = yyExit;
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
+# include "yyMakeOvUpdates.h"
+
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* MakeOvUpdates_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -59,25 +81,24 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module MakeOvUpdates, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module MakeOvUpdates, routine %s failed\n",
+  yyFunction);
  MakeOvUpdates_Exit ();
 }
 
-void MakeOvUpdates ARGS((tTree t));
-static tTree OverlapUpdateACF ARGS((tTree t));
-static void OverlapUpdateParallel ARGS((tTree stmt));
-static void OverlapUpdateMask ARGS((tTree stmt, tTree mask));
-static void OverlapUpdateWHERE ARGS((tTree stmt, tTree mask, tTree * yyP1));
-static tTree ReplaceCShifts ARGS((tTree exp));
-static tTree TranslateCSHIFT ARGS((tTree exp));
-static void OverlapUpdates ARGS((tTree var, tTree exp));
-static void OverlapIndexUpdates ARGS((tTree owner, tTree var));
-static void OverlapVarUpdates ARGS((tTree left_var, tTree right_var));
-static void ApplyShiftToVar ARGS((tTree var, pshift pos));
-static void ApplyShiftToIndexes ARGS((tTree indexes, int n, pshift pos));
-static bool ValidOverlapUpdate ARGS((tTree var, pshift pos));
-static bool IsEnoughVarOverlap ARGS((tTree indexes, int dim, pshift pos));
-static bool SufficientOverlapIndex ARGS((tTree index, int pos));
+void MakeOvUpdates ARGS ((tTree t));
+static tTree OverlapUpdateACF ARGS ((tTree t));
+static void OverlapUpdateParallel ARGS ((tTree stmt));
+static void OverlapUpdateMask ARGS ((tTree stmt, tTree mask));
+static void OverlapUpdateWHERE ARGS ((tTree stmt, tTree mask, tTree * yyP1));
+static tTree ReplaceCShifts ARGS ((tTree exp));
+static tTree TranslateCSHIFT ARGS ((tTree exp));
+static void OverlapUpdates ARGS ((tTree var, tTree exp));
+static void OverlapIndexUpdates ARGS ((tTree owner, tTree var));
+static void OverlapVarUpdates ARGS ((tTree left_var, tTree right_var));
+static void ApplyShiftToVar ARGS ((tTree var, pshift pos));
+static void ApplyShiftToIndexes ARGS ((tTree indexes, int n, pshift pos));
+static rbool ValidOverlapUpdate ARGS ((tTree var, pshift pos));
 
 void MakeOvUpdates
 # if defined __STDC__ | defined __cplusplus
@@ -88,17 +109,17 @@ void MakeOvUpdates
 # endif
 {
   if (t->Kind == kBODY_NODE) {
-# line 57 "MakeOvUpdates.puma"
+/* line 61 "MakeOvUpdates.puma" */
   {
-# line 59 "MakeOvUpdates.puma"
+/* line 63 "MakeOvUpdates.puma" */
  t->BODY_NODE.STATS = OverlapUpdateACF (t->BODY_NODE.STATS); 
   }
    return;
 
   }
-# line 62 "MakeOvUpdates.puma"
+/* line 66 "MakeOvUpdates.puma" */
   {
-# line 63 "MakeOvUpdates.puma"
+/* line 67 "MakeOvUpdates.puma" */
    failure_protocol (MODULE, "MakeOvUpdates", t);
   }
    return;
@@ -117,62 +138,54 @@ static tTree OverlapUpdateACF
 
   switch (t->Kind) {
   case kACF_LIST:
-# line 78 "MakeOvUpdates.puma"
+/* line 82 "MakeOvUpdates.puma" */
  {
   tTree newacf;
   {
-# line 80 "MakeOvUpdates.puma"
-
-# line 82 "MakeOvUpdates.puma"
+/* line 86 "MakeOvUpdates.puma" */
    set_protocol_stmt (t->ACF_LIST.Elem);
-# line 84 "MakeOvUpdates.puma"
+/* line 88 "MakeOvUpdates.puma" */
  newacf = OverlapUpdateACF (t->ACF_LIST.Elem);            
      t->ACF_LIST.Next   = OverlapUpdateACF (t->ACF_LIST.Next);            
    
-# line 88 "MakeOvUpdates.puma"
+/* line 92 "MakeOvUpdates.puma" */
    newacf = ReplaceACF (t, newacf, t->ACF_LIST.Next);
   }
-  {
    return newacf;
-  }
  }
 
   case kACF_EMPTY:
-# line 93 "MakeOvUpdates.puma"
+/* line 97 "MakeOvUpdates.puma" */
    return t;
 
   case kACF_BASIC:
   if (t->ACF_BASIC.BASIC_STMT->Kind == kALLOCATE_STMT) {
-# line 97 "MakeOvUpdates.puma"
+/* line 101 "MakeOvUpdates.puma" */
    return t;
 
   }
   if (t->ACF_BASIC.BASIC_STMT->Kind == kDEALLOCATE_STMT) {
-# line 101 "MakeOvUpdates.puma"
+/* line 105 "MakeOvUpdates.puma" */
    return t;
 
   }
   if (t->ACF_BASIC.BASIC_STMT->Kind == kCREATE_DSP_STMT) {
-# line 105 "MakeOvUpdates.puma"
+/* line 109 "MakeOvUpdates.puma" */
    return t;
 
   }
   if (t->ACF_BASIC.BASIC_STMT->Kind == kFREE_DSP_STMT) {
-# line 109 "MakeOvUpdates.puma"
+/* line 113 "MakeOvUpdates.puma" */
    return t;
 
   }
   if (t->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
-# line 113 "MakeOvUpdates.puma"
+/* line 117 "MakeOvUpdates.puma" */
  {
   tTree OvUpdates;
   tTree NewStmt;
   {
-# line 115 "MakeOvUpdates.puma"
-
-# line 116 "MakeOvUpdates.puma"
-
-# line 118 "MakeOvUpdates.puma"
+/* line 122 "MakeOvUpdates.puma" */
  InitOverlapUpdates ();               
 
      
@@ -202,19 +215,17 @@ static tTree OverlapUpdateACF
 
    
   }
-  {
    return NewStmt;
-  }
  }
 
   }
-# line 150 "MakeOvUpdates.puma"
+/* line 154 "MakeOvUpdates.puma" */
    return t;
 
   case kACF_IF:
-# line 154 "MakeOvUpdates.puma"
+/* line 158 "MakeOvUpdates.puma" */
   {
-# line 156 "MakeOvUpdates.puma"
+/* line 160 "MakeOvUpdates.puma" */
  t->ACF_IF.THEN_PART = OverlapUpdateACF (t->ACF_IF.THEN_PART);
      t->ACF_IF.ELSE_PART = OverlapUpdateACF (t->ACF_IF.ELSE_PART);
    
@@ -222,38 +233,34 @@ static tTree OverlapUpdateACF
    return t;
 
   case kACF_DUMMY:
-# line 162 "MakeOvUpdates.puma"
+/* line 166 "MakeOvUpdates.puma" */
    return t;
 
   case kACF_WHILE:
-# line 166 "MakeOvUpdates.puma"
+/* line 170 "MakeOvUpdates.puma" */
   {
-# line 168 "MakeOvUpdates.puma"
+/* line 172 "MakeOvUpdates.puma" */
  t->ACF_WHILE.WHILE_BODY = OverlapUpdateACF (t->ACF_WHILE.WHILE_BODY); 
   }
    return t;
 
   case kACF_DO:
   if (t->ACF_DO.DO_DEP_INFO->Kind == kSERIAL_INFO) {
-# line 172 "MakeOvUpdates.puma"
+/* line 176 "MakeOvUpdates.puma" */
   {
-# line 174 "MakeOvUpdates.puma"
+/* line 178 "MakeOvUpdates.puma" */
  t->ACF_DO.DO_BODY = OverlapUpdateACF (t->ACF_DO.DO_BODY); 
   }
    return t;
 
   }
   if (t->ACF_DO.DO_DEP_INFO->Kind == kINDEP_INFO) {
-# line 246 "MakeOvUpdates.puma"
+/* line 250 "MakeOvUpdates.puma" */
  {
   tTree OvUpdates;
   tTree NewStmt;
   {
-# line 248 "MakeOvUpdates.puma"
-
-# line 249 "MakeOvUpdates.puma"
-
-# line 251 "MakeOvUpdates.puma"
+/* line 255 "MakeOvUpdates.puma" */
  InitOverlapUpdates ();               
  
      
@@ -281,9 +288,7 @@ static tTree OverlapUpdateACF
  
    
   }
-  {
    return NewStmt;
-  }
  }
 
   }
@@ -291,16 +296,12 @@ static tTree OverlapUpdateACF
   }
 
   if (t->Kind == kACF_WHERE) {
-# line 179 "MakeOvUpdates.puma"
+/* line 183 "MakeOvUpdates.puma" */
  {
   tTree OvUpdates;
   tTree NewStmt;
   {
-# line 181 "MakeOvUpdates.puma"
-
-# line 182 "MakeOvUpdates.puma"
-
-# line 186 "MakeOvUpdates.puma"
+/* line 190 "MakeOvUpdates.puma" */
  NewStmt = t;
 
      InitOverlapUpdates ();
@@ -323,23 +324,17 @@ static tTree OverlapUpdateACF
 
    
   }
-  {
    return NewStmt;
-  }
  }
 
   }
   if (t->Kind == kACF_FORALL) {
-# line 211 "MakeOvUpdates.puma"
+/* line 215 "MakeOvUpdates.puma" */
  {
   tTree OvUpdates;
   tTree NewStmt;
   {
-# line 213 "MakeOvUpdates.puma"
-
-# line 214 "MakeOvUpdates.puma"
-
-# line 216 "MakeOvUpdates.puma"
+/* line 220 "MakeOvUpdates.puma" */
  InitOverlapUpdates ();               
 
      
@@ -367,20 +362,18 @@ static tTree OverlapUpdateACF
 
    
   }
-  {
    return NewStmt;
-  }
  }
 
   }
   if (Tree_IsType (t, kACF_NODE)) {
-# line 281 "MakeOvUpdates.puma"
+/* line 285 "MakeOvUpdates.puma" */
    return t;
 
   }
-# line 285 "MakeOvUpdates.puma"
+/* line 289 "MakeOvUpdates.puma" */
   {
-# line 286 "MakeOvUpdates.puma"
+/* line 290 "MakeOvUpdates.puma" */
    failure_protocol (MODULE, "OverlapUpdateACF", t);
   }
    return NoTree;
@@ -395,144 +388,145 @@ static void OverlapUpdateParallel
  register tTree stmt;
 # endif
 {
+ yyRecursion:
 
   switch (stmt->Kind) {
   case kACF_EMPTY:
-# line 298 "MakeOvUpdates.puma"
+/* line 302 "MakeOvUpdates.puma" */
    return;
 
   case kACF_LIST:
-# line 301 "MakeOvUpdates.puma"
+/* line 305 "MakeOvUpdates.puma" */
   {
-# line 303 "MakeOvUpdates.puma"
+/* line 307 "MakeOvUpdates.puma" */
    OverlapUpdateParallel (stmt->ACF_LIST.Elem);
-# line 304 "MakeOvUpdates.puma"
-   OverlapUpdateParallel (stmt->ACF_LIST.Next);
+/* line 308 "MakeOvUpdates.puma" */
+   stmt = stmt->ACF_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kACF_IF:
-# line 307 "MakeOvUpdates.puma"
+/* line 311 "MakeOvUpdates.puma" */
   {
-# line 309 "MakeOvUpdates.puma"
+/* line 313 "MakeOvUpdates.puma" */
    OverlapUpdateParallel (stmt->ACF_IF.THEN_PART);
-# line 310 "MakeOvUpdates.puma"
+/* line 314 "MakeOvUpdates.puma" */
    OverlapUpdateParallel (stmt->ACF_IF.ELSE_PART);
-# line 311 "MakeOvUpdates.puma"
+/* line 315 "MakeOvUpdates.puma" */
    OverlapUpdateMask (stmt->ACF_IF.THEN_PART, stmt->ACF_IF.IF_EXP);
-# line 312 "MakeOvUpdates.puma"
+/* line 316 "MakeOvUpdates.puma" */
    OverlapUpdateMask (stmt->ACF_IF.ELSE_PART, stmt->ACF_IF.IF_EXP);
   }
    return;
 
   case kACF_WHERE:
-# line 315 "MakeOvUpdates.puma"
+/* line 319 "MakeOvUpdates.puma" */
   {
-# line 317 "MakeOvUpdates.puma"
+/* line 321 "MakeOvUpdates.puma" */
    OverlapUpdateParallel (stmt->ACF_WHERE.TRUE_PART);
-# line 318 "MakeOvUpdates.puma"
+/* line 322 "MakeOvUpdates.puma" */
    OverlapUpdateParallel (stmt->ACF_WHERE.FALSE_PART);
-# line 319 "MakeOvUpdates.puma"
+/* line 323 "MakeOvUpdates.puma" */
    OverlapUpdateMask (stmt->ACF_WHERE.TRUE_PART, stmt->ACF_WHERE.WHERE_EXP);
-# line 320 "MakeOvUpdates.puma"
+/* line 324 "MakeOvUpdates.puma" */
    OverlapUpdateMask (stmt->ACF_WHERE.FALSE_PART, stmt->ACF_WHERE.WHERE_EXP);
   }
    return;
 
   case kACF_BASIC:
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
-# line 323 "MakeOvUpdates.puma"
+/* line 327 "MakeOvUpdates.puma" */
   {
-# line 325 "MakeOvUpdates.puma"
+/* line 329 "MakeOvUpdates.puma" */
   
-# line 327 "MakeOvUpdates.puma"
+/* line 331 "MakeOvUpdates.puma" */
    OverlapUpdates (stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR, stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_EXP);
   }
    return;
 
   }
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kREDUCE_STMT) {
-# line 330 "MakeOvUpdates.puma"
+/* line 334 "MakeOvUpdates.puma" */
    return;
 
   }
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kCALL_STMT) {
-# line 333 "MakeOvUpdates.puma"
+/* line 337 "MakeOvUpdates.puma" */
    return;
 
   }
-# line 338 "MakeOvUpdates.puma"
+/* line 342 "MakeOvUpdates.puma" */
    return;
 
   case kACF_FORALL:
-# line 341 "MakeOvUpdates.puma"
+/* line 345 "MakeOvUpdates.puma" */
   {
-# line 343 "MakeOvUpdates.puma"
+/* line 347 "MakeOvUpdates.puma" */
    OverlapUpdateParallel (stmt->ACF_FORALL.FORALL_BODY);
-# line 344 "MakeOvUpdates.puma"
+/* line 348 "MakeOvUpdates.puma" */
    VectorizeOverlapUpdates (stmt->ACF_FORALL.FORALL_ID, stmt->ACF_FORALL.FORALL_RANGE);
   }
    return;
 
   case kACF_DO:
-# line 347 "MakeOvUpdates.puma"
+/* line 351 "MakeOvUpdates.puma" */
   {
-# line 349 "MakeOvUpdates.puma"
+/* line 353 "MakeOvUpdates.puma" */
    OverlapUpdateParallel (stmt->ACF_DO.DO_BODY);
-# line 350 "MakeOvUpdates.puma"
+/* line 354 "MakeOvUpdates.puma" */
    VectorizeOverlapUpdates (stmt->ACF_DO.DO_ID, stmt->ACF_DO.DO_RANGE);
   }
    return;
 
   case kACF_HOME:
-# line 353 "MakeOvUpdates.puma"
+/* line 357 "MakeOvUpdates.puma" */
   {
-# line 355 "MakeOvUpdates.puma"
-   OverlapUpdateParallel (stmt->ACF_HOME.HOME_BODY);
+/* line 359 "MakeOvUpdates.puma" */
+   stmt = stmt->ACF_HOME.HOME_BODY;
+   goto yyRecursion;
   }
-   return;
 
   case kACF_NEW:
-# line 358 "MakeOvUpdates.puma"
+/* line 362 "MakeOvUpdates.puma" */
   {
-# line 360 "MakeOvUpdates.puma"
-   OverlapUpdateParallel (stmt->ACF_NEW.NEW_BODY);
+/* line 364 "MakeOvUpdates.puma" */
+   stmt = stmt->ACF_NEW.NEW_BODY;
+   goto yyRecursion;
   }
-   return;
 
   case kACF_REDUCTION:
-# line 363 "MakeOvUpdates.puma"
+/* line 367 "MakeOvUpdates.puma" */
   {
-# line 365 "MakeOvUpdates.puma"
-   OverlapUpdateParallel (stmt->ACF_REDUCTION.REDUCTION_BODY);
+/* line 369 "MakeOvUpdates.puma" */
+   stmt = stmt->ACF_REDUCTION.REDUCTION_BODY;
+   goto yyRecursion;
   }
-   return;
 
   case kACF_RESIDENT:
-# line 368 "MakeOvUpdates.puma"
+/* line 372 "MakeOvUpdates.puma" */
   {
-# line 370 "MakeOvUpdates.puma"
-   OverlapUpdateParallel (stmt->ACF_RESIDENT.RESIDENT_BODY);
+/* line 374 "MakeOvUpdates.puma" */
+   stmt = stmt->ACF_RESIDENT.RESIDENT_BODY;
+   goto yyRecursion;
   }
-   return;
 
   case kACF_WHILE:
-# line 373 "MakeOvUpdates.puma"
+/* line 377 "MakeOvUpdates.puma" */
   {
-# line 375 "MakeOvUpdates.puma"
-   OverlapUpdateParallel (stmt->ACF_WHILE.WHILE_BODY);
+/* line 379 "MakeOvUpdates.puma" */
+   stmt = stmt->ACF_WHILE.WHILE_BODY;
+   goto yyRecursion;
   }
-   return;
 
   case kACF_DUMMY:
-# line 378 "MakeOvUpdates.puma"
+/* line 382 "MakeOvUpdates.puma" */
    return;
 
   }
 
-# line 381 "MakeOvUpdates.puma"
+/* line 385 "MakeOvUpdates.puma" */
   {
-# line 382 "MakeOvUpdates.puma"
+/* line 386 "MakeOvUpdates.puma" */
    failure_protocol (MODULE, "OverlapUpdateParallel", stmt);
   }
    return;
@@ -549,49 +543,50 @@ static void OverlapUpdateMask
  register tTree mask;
 # endif
 {
+ yyRecursion:
   if (stmt->Kind == kACF_EMPTY) {
-# line 397 "MakeOvUpdates.puma"
+/* line 401 "MakeOvUpdates.puma" */
    return;
 
   }
   if (stmt->Kind == kACF_LIST) {
-# line 400 "MakeOvUpdates.puma"
+/* line 404 "MakeOvUpdates.puma" */
   {
-# line 402 "MakeOvUpdates.puma"
+/* line 406 "MakeOvUpdates.puma" */
    OverlapUpdateMask (stmt->ACF_LIST.Elem, mask);
-# line 403 "MakeOvUpdates.puma"
-   OverlapUpdateMask (stmt->ACF_LIST.Next, mask);
+/* line 407 "MakeOvUpdates.puma" */
+   stmt = stmt->ACF_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (stmt->Kind == kACF_WHERE) {
-# line 406 "MakeOvUpdates.puma"
+/* line 410 "MakeOvUpdates.puma" */
   {
-# line 408 "MakeOvUpdates.puma"
+/* line 412 "MakeOvUpdates.puma" */
    OverlapUpdateMask (stmt->ACF_WHERE.TRUE_PART, mask);
-# line 409 "MakeOvUpdates.puma"
-   OverlapUpdateMask (stmt->ACF_WHERE.FALSE_PART, mask);
+/* line 413 "MakeOvUpdates.puma" */
+   stmt = stmt->ACF_WHERE.FALSE_PART;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (stmt->Kind == kACF_IF) {
-# line 412 "MakeOvUpdates.puma"
+/* line 416 "MakeOvUpdates.puma" */
   {
-# line 414 "MakeOvUpdates.puma"
+/* line 418 "MakeOvUpdates.puma" */
    OverlapUpdateMask (stmt->ACF_IF.THEN_PART, mask);
-# line 415 "MakeOvUpdates.puma"
-   OverlapUpdateMask (stmt->ACF_IF.ELSE_PART, mask);
+/* line 419 "MakeOvUpdates.puma" */
+   stmt = stmt->ACF_IF.ELSE_PART;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (stmt->Kind == kACF_BASIC) {
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
-# line 418 "MakeOvUpdates.puma"
+/* line 422 "MakeOvUpdates.puma" */
   {
-# line 420 "MakeOvUpdates.puma"
+/* line 424 "MakeOvUpdates.puma" */
    OverlapUpdates (stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR, mask);
   }
    return;
@@ -611,12 +606,12 @@ static void OverlapUpdateWHERE
  register tTree * yyP1;
 # endif
 {
-# line 442 "MakeOvUpdates.puma"
+/* line 446 "MakeOvUpdates.puma" */
   {
-# line 444 "MakeOvUpdates.puma"
+/* line 448 "MakeOvUpdates.puma" */
    if (! ((stmt == NoTree))) goto yyL1;
   {
-# line 445 "MakeOvUpdates.puma"
+/* line 449 "MakeOvUpdates.puma" */
    failure_protocol (MODULE, "OverlapUpdateWHERE", stmt);
   }
   }
@@ -625,20 +620,20 @@ static void OverlapUpdateWHERE
 yyL1:;
 
   if (stmt->Kind == kACF_EMPTY) {
-# line 448 "MakeOvUpdates.puma"
+/* line 452 "MakeOvUpdates.puma" */
    * yyP1 = mask;
    return;
 
   }
   if (stmt->Kind == kACF_LIST) {
-# line 451 "MakeOvUpdates.puma"
+/* line 455 "MakeOvUpdates.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 453 "MakeOvUpdates.puma"
+/* line 457 "MakeOvUpdates.puma" */
    OverlapUpdateWHERE (stmt->ACF_LIST.Elem, mask, & yyV1);
-# line 454 "MakeOvUpdates.puma"
+/* line 458 "MakeOvUpdates.puma" */
    OverlapUpdateWHERE (stmt->ACF_LIST.Next, yyV1, & yyV2);
   }
    * yyP1 = yyV2;
@@ -647,26 +642,24 @@ yyL1:;
 
   }
   if (stmt->Kind == kACF_WHERE) {
-# line 457 "MakeOvUpdates.puma"
+/* line 461 "MakeOvUpdates.puma" */
    * yyP1 = mask;
    return;
 
   }
   if (stmt->Kind == kACF_BASIC) {
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
-# line 462 "MakeOvUpdates.puma"
+/* line 466 "MakeOvUpdates.puma" */
  {
   tTree new_mask;
   {
-# line 466 "MakeOvUpdates.puma"
+/* line 470 "MakeOvUpdates.puma" */
  stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_EXP = ReplaceCShifts (stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_EXP); 
-# line 470 "MakeOvUpdates.puma"
+/* line 474 "MakeOvUpdates.puma" */
    OverlapUpdates (stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR, stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_EXP);
-# line 472 "MakeOvUpdates.puma"
-
-# line 476 "MakeOvUpdates.puma"
+/* line 480 "MakeOvUpdates.puma" */
  new_mask = ReplaceCShifts (mask); 
-# line 480 "MakeOvUpdates.puma"
+/* line 484 "MakeOvUpdates.puma" */
    OverlapUpdates (stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR, new_mask);
   }
    * yyP1 = new_mask;
@@ -675,9 +668,9 @@ yyL1:;
 
   }
   }
-# line 483 "MakeOvUpdates.puma"
+/* line 487 "MakeOvUpdates.puma" */
   {
-# line 484 "MakeOvUpdates.puma"
+/* line 488 "MakeOvUpdates.puma" */
    failure_protocol (MODULE, "OverlapUpdateWHERE", stmt);
   }
    * yyP1 = NoTree;
@@ -697,37 +690,37 @@ static tTree ReplaceCShifts
 
   switch (exp->Kind) {
   case kDUMMY_EXP:
-# line 495 "MakeOvUpdates.puma"
+/* line 499 "MakeOvUpdates.puma" */
    return exp;
 
   case kCONST_EXP:
-# line 499 "MakeOvUpdates.puma"
+/* line 503 "MakeOvUpdates.puma" */
    return exp;
 
   case kBOUND_EXP:
-# line 503 "MakeOvUpdates.puma"
+/* line 507 "MakeOvUpdates.puma" */
    return exp;
 
   case kRANK_EXP:
-# line 507 "MakeOvUpdates.puma"
+/* line 511 "MakeOvUpdates.puma" */
    return exp;
 
   case kARRAY_EXP:
-# line 511 "MakeOvUpdates.puma"
+/* line 515 "MakeOvUpdates.puma" */
    return exp;
 
   case kTYPE_EXP:
-# line 515 "MakeOvUpdates.puma"
+/* line 519 "MakeOvUpdates.puma" */
    return exp;
 
   case kSLICE_EXP:
-# line 519 "MakeOvUpdates.puma"
+/* line 523 "MakeOvUpdates.puma" */
    return exp;
 
   case kOP_EXP:
-# line 523 "MakeOvUpdates.puma"
+/* line 527 "MakeOvUpdates.puma" */
   {
-# line 524 "MakeOvUpdates.puma"
+/* line 528 "MakeOvUpdates.puma" */
  exp->OP_EXP.OPND1 = ReplaceCShifts (exp->OP_EXP.OPND1);
      exp->OP_EXP.OPND2 = ReplaceCShifts (exp->OP_EXP.OPND2);
    
@@ -735,57 +728,53 @@ static tTree ReplaceCShifts
    return exp;
 
   case kOP1_EXP:
-# line 530 "MakeOvUpdates.puma"
+/* line 534 "MakeOvUpdates.puma" */
   {
-# line 531 "MakeOvUpdates.puma"
+/* line 535 "MakeOvUpdates.puma" */
  exp->OP1_EXP.OPND = ReplaceCShifts (exp->OP1_EXP.OPND); 
   }
    return exp;
 
   case kPERM_EXP:
-# line 535 "MakeOvUpdates.puma"
+/* line 539 "MakeOvUpdates.puma" */
    return exp;
 
   case kVAR_EXP:
-# line 539 "MakeOvUpdates.puma"
+/* line 543 "MakeOvUpdates.puma" */
    return exp;
 
   case kFUNC_CALL_EXP:
-# line 543 "MakeOvUpdates.puma"
+/* line 547 "MakeOvUpdates.puma" */
   {
-# line 544 "MakeOvUpdates.puma"
+/* line 548 "MakeOvUpdates.puma" */
    if (! ((IsIntrCall (exp)))) goto yyL12;
   {
-# line 545 "MakeOvUpdates.puma"
+/* line 549 "MakeOvUpdates.puma" */
    if (! ((exp->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Ident == MakeIdent ("CSHIFT", 6)))) goto yyL12;
   }
   }
    return TranslateCSHIFT (exp);
 yyL12:;
 
-# line 549 "MakeOvUpdates.puma"
+/* line 553 "MakeOvUpdates.puma" */
   {
-# line 550 "MakeOvUpdates.puma"
-   if (! ((IsIntrCall (exp)))) goto yyL13;
+/* line 555 "MakeOvUpdates.puma" */
+   if (! ((IsElementalCall (exp)))) goto yyL13;
   {
-# line 551 "MakeOvUpdates.puma"
-   if (! ((IntrFuncElemental (exp->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Ident)))) goto yyL13;
-  {
-# line 552 "MakeOvUpdates.puma"
+/* line 556 "MakeOvUpdates.puma" */
  exp->FUNC_CALL_EXP.FUNC_PARAMS = ReplaceCShifts (exp->FUNC_CALL_EXP.FUNC_PARAMS); 
-  }
   }
   }
    return exp;
 yyL13:;
 
-# line 584 "MakeOvUpdates.puma"
+/* line 588 "MakeOvUpdates.puma" */
    return exp;
 
   case kBTP_LIST:
-# line 556 "MakeOvUpdates.puma"
+/* line 560 "MakeOvUpdates.puma" */
   {
-# line 558 "MakeOvUpdates.puma"
+/* line 562 "MakeOvUpdates.puma" */
  exp->BTP_LIST.Elem = ReplaceCShifts (exp->BTP_LIST.Elem); 
      exp->BTP_LIST.Next = ReplaceCShifts (exp->BTP_LIST.Next); 
    
@@ -793,31 +782,31 @@ yyL13:;
    return exp;
 
   case kBTP_EMPTY:
-# line 564 "MakeOvUpdates.puma"
+/* line 568 "MakeOvUpdates.puma" */
    return exp;
 
   case kNO_PARAM:
-# line 569 "MakeOvUpdates.puma"
+/* line 573 "MakeOvUpdates.puma" */
    return exp;
 
   case kVAR_PARAM:
   if (exp->VAR_PARAM.V->Kind == kADDR) {
-# line 574 "MakeOvUpdates.puma"
+/* line 578 "MakeOvUpdates.puma" */
   {
-# line 576 "MakeOvUpdates.puma"
+/* line 580 "MakeOvUpdates.puma" */
  exp->VAR_PARAM.V->ADDR.E = ReplaceCShifts (exp->VAR_PARAM.V->ADDR.E); 
   }
    return exp;
 
   }
-# line 580 "MakeOvUpdates.puma"
+/* line 584 "MakeOvUpdates.puma" */
    return exp;
 
   }
 
-# line 588 "MakeOvUpdates.puma"
+/* line 592 "MakeOvUpdates.puma" */
   {
-# line 589 "MakeOvUpdates.puma"
+/* line 593 "MakeOvUpdates.puma" */
    failure_protocol (MODULE, "ReplaceCShifts", exp);
   }
    return exp;
@@ -833,50 +822,43 @@ static tTree TranslateCSHIFT
 # endif
 {
   if (exp->Kind == kFUNC_CALL_EXP) {
-# line 604 "MakeOvUpdates.puma"
+/* line 608 "MakeOvUpdates.puma" */
  {
   shift_vector pos;
-  bool found;
+  rbool found;
   tTree var;
   {
-# line 606 "MakeOvUpdates.puma"
-
-# line 607 "MakeOvUpdates.puma"
-
-# line 608 "MakeOvUpdates.puma"
-
-# line 610 "MakeOvUpdates.puma"
+/* line 614 "MakeOvUpdates.puma" */
    GetCircularShifting (exp, & found, & var, & pos);
-# line 611 "MakeOvUpdates.puma"
+/* line 615 "MakeOvUpdates.puma" */
    if (! ((found))) goto yyL1;
   {
-# line 613 "MakeOvUpdates.puma"
+/* line 617 "MakeOvUpdates.puma" */
    stmt_protocol ("replace CSHIFT");
-# line 615 "MakeOvUpdates.puma"
+/* line 619 "MakeOvUpdates.puma" */
    if (! ((ValidOverlapUpdate (var, & pos)))) goto yyL1;
   {
-# line 619 "MakeOvUpdates.puma"
+/* line 623 "MakeOvUpdates.puma" */
    SetShiftOverlapUpdate (var, & pos);
-# line 621 "MakeOvUpdates.puma"
+/* line 625 "MakeOvUpdates.puma" */
    tree_protocol ("this is updated var: ", var);
-# line 623 "MakeOvUpdates.puma"
+/* line 627 "MakeOvUpdates.puma" */
    ApplyShiftToVar (var, & pos);
-# line 625 "MakeOvUpdates.puma"
+/* line 629 "MakeOvUpdates.puma" */
    tree_protocol ("this is shifted var: ", var);
   }
   }
   }
-  {
    return mVAR_EXP (var);
-  }
  }
 yyL1:;
 
-# line 630 "MakeOvUpdates.puma"
+/* line 634 "MakeOvUpdates.puma" */
    return exp;
 
   }
  yyAbort ("TranslateCSHIFT");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static void OverlapUpdates
@@ -888,138 +870,135 @@ static void OverlapUpdates
  register tTree exp;
 # endif
 {
+ yyRecursion:
 
   switch (exp->Kind) {
   case kDUMMY_EXP:
-# line 642 "MakeOvUpdates.puma"
+/* line 646 "MakeOvUpdates.puma" */
    return;
 
   case kCONST_EXP:
-# line 645 "MakeOvUpdates.puma"
+/* line 649 "MakeOvUpdates.puma" */
    return;
 
   case kARRAY_EXP:
-# line 648 "MakeOvUpdates.puma"
+/* line 652 "MakeOvUpdates.puma" */
    return;
 
   case kBOUND_EXP:
-# line 651 "MakeOvUpdates.puma"
+/* line 655 "MakeOvUpdates.puma" */
    return;
 
   case kTYPE_EXP:
-# line 654 "MakeOvUpdates.puma"
+/* line 658 "MakeOvUpdates.puma" */
    return;
 
   case kSLICE_EXP:
-# line 657 "MakeOvUpdates.puma"
+/* line 661 "MakeOvUpdates.puma" */
    return;
 
   case kOP_EXP:
-# line 660 "MakeOvUpdates.puma"
+/* line 664 "MakeOvUpdates.puma" */
   {
-# line 661 "MakeOvUpdates.puma"
+/* line 665 "MakeOvUpdates.puma" */
    OverlapUpdates (var, exp->OP_EXP.OPND1);
-# line 662 "MakeOvUpdates.puma"
-   OverlapUpdates (var, exp->OP_EXP.OPND2);
+/* line 666 "MakeOvUpdates.puma" */
+   exp = exp->OP_EXP.OPND2;
+   goto yyRecursion;
   }
-   return;
 
   case kOP1_EXP:
-# line 665 "MakeOvUpdates.puma"
+/* line 669 "MakeOvUpdates.puma" */
   {
-# line 666 "MakeOvUpdates.puma"
-   OverlapUpdates (var, exp->OP1_EXP.OPND);
+/* line 670 "MakeOvUpdates.puma" */
+   exp = exp->OP1_EXP.OPND;
+   goto yyRecursion;
   }
-   return;
 
   case kPERM_EXP:
-# line 669 "MakeOvUpdates.puma"
+/* line 673 "MakeOvUpdates.puma" */
    return;
 
   case kVAR_EXP:
-# line 672 "MakeOvUpdates.puma"
+/* line 676 "MakeOvUpdates.puma" */
   {
-# line 673 "MakeOvUpdates.puma"
+/* line 677 "MakeOvUpdates.puma" */
    OverlapVarUpdates (var, exp->VAR_EXP.V);
-# line 674 "MakeOvUpdates.puma"
+/* line 678 "MakeOvUpdates.puma" */
    OverlapIndexUpdates (var, exp->VAR_EXP.V);
   }
    return;
 
   case kFUNC_CALL_EXP:
-# line 677 "MakeOvUpdates.puma"
+/* line 681 "MakeOvUpdates.puma" */
   {
-# line 678 "MakeOvUpdates.puma"
-   if (! ((IsIntrCall (exp)))) goto yyL11;
+/* line 683 "MakeOvUpdates.puma" */
+   if (! ((IsElementalCall (exp)))) goto yyL11;
   {
-# line 679 "MakeOvUpdates.puma"
-   if (! ((IntrFuncElemental (exp->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Ident)))) goto yyL11;
-  {
-# line 680 "MakeOvUpdates.puma"
-   OverlapUpdates (var, exp->FUNC_CALL_EXP.FUNC_PARAMS);
+/* line 685 "MakeOvUpdates.puma" */
+   exp = exp->FUNC_CALL_EXP.FUNC_PARAMS;
+   goto yyRecursion;
   }
   }
-  }
-   return;
 yyL11:;
 
-# line 683 "MakeOvUpdates.puma"
+/* line 688 "MakeOvUpdates.puma" */
    return;
 
   case kBTP_LIST:
-# line 686 "MakeOvUpdates.puma"
+/* line 691 "MakeOvUpdates.puma" */
   {
-# line 687 "MakeOvUpdates.puma"
+/* line 692 "MakeOvUpdates.puma" */
    OverlapUpdates (var, exp->BTP_LIST.Elem);
-# line 688 "MakeOvUpdates.puma"
-   OverlapUpdates (var, exp->BTP_LIST.Next);
+/* line 693 "MakeOvUpdates.puma" */
+   exp = exp->BTP_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kBTP_EMPTY:
-# line 691 "MakeOvUpdates.puma"
+/* line 696 "MakeOvUpdates.puma" */
    return;
 
   case kBTE_LIST:
-# line 694 "MakeOvUpdates.puma"
+/* line 699 "MakeOvUpdates.puma" */
   {
-# line 695 "MakeOvUpdates.puma"
+/* line 700 "MakeOvUpdates.puma" */
    OverlapUpdates (var, exp->BTE_LIST.Elem);
-# line 696 "MakeOvUpdates.puma"
-   OverlapUpdates (var, exp->BTE_LIST.Next);
+/* line 701 "MakeOvUpdates.puma" */
+   exp = exp->BTE_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kBTE_EMPTY:
-# line 699 "MakeOvUpdates.puma"
+/* line 704 "MakeOvUpdates.puma" */
    return;
 
   case kNO_PARAM:
-# line 702 "MakeOvUpdates.puma"
+/* line 707 "MakeOvUpdates.puma" */
    return;
 
   case kVAR_PARAM:
   if (exp->VAR_PARAM.V->Kind == kADDR) {
-# line 705 "MakeOvUpdates.puma"
+/* line 710 "MakeOvUpdates.puma" */
   {
-# line 706 "MakeOvUpdates.puma"
-   OverlapUpdates (var, exp->VAR_PARAM.V->ADDR.E);
+/* line 711 "MakeOvUpdates.puma" */
+   exp = exp->VAR_PARAM.V->ADDR.E;
+   goto yyRecursion;
   }
-   return;
 
   }
-# line 709 "MakeOvUpdates.puma"
+/* line 714 "MakeOvUpdates.puma" */
   {
-# line 710 "MakeOvUpdates.puma"
+/* line 715 "MakeOvUpdates.puma" */
    OverlapVarUpdates (var, exp->VAR_PARAM.V);
   }
    return;
 
   }
 
-# line 713 "MakeOvUpdates.puma"
+/* line 718 "MakeOvUpdates.puma" */
   {
-# line 714 "MakeOvUpdates.puma"
+/* line 719 "MakeOvUpdates.puma" */
    failure_protocol (MODULE, "OverlapUpdates", exp);
   }
    return;
@@ -1036,32 +1015,33 @@ static void OverlapIndexUpdates
  register tTree var;
 # endif
 {
+ yyRecursion:
   if (var->Kind == kINDEXED_VAR) {
-# line 719 "MakeOvUpdates.puma"
+/* line 724 "MakeOvUpdates.puma" */
   {
-# line 721 "MakeOvUpdates.puma"
+/* line 726 "MakeOvUpdates.puma" */
    OverlapIndexUpdates (owner, var->INDEXED_VAR.IND_VAR);
-# line 722 "MakeOvUpdates.puma"
+/* line 727 "MakeOvUpdates.puma" */
    OverlapUpdates (owner, var->INDEXED_VAR.IND_EXPS);
   }
    return;
 
   }
   if (var->Kind == kSELECTED_VAR) {
-# line 725 "MakeOvUpdates.puma"
+/* line 730 "MakeOvUpdates.puma" */
   {
-# line 727 "MakeOvUpdates.puma"
-   OverlapIndexUpdates (owner, var->SELECTED_VAR.SELEC_VAR);
+/* line 732 "MakeOvUpdates.puma" */
+   var = var->SELECTED_VAR.SELEC_VAR;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (var->Kind == kSUBSTRING_VAR) {
-# line 730 "MakeOvUpdates.puma"
+/* line 735 "MakeOvUpdates.puma" */
   {
-# line 732 "MakeOvUpdates.puma"
+/* line 737 "MakeOvUpdates.puma" */
    OverlapIndexUpdates (owner, var->SUBSTRING_VAR.IND_VAR);
-# line 733 "MakeOvUpdates.puma"
+/* line 738 "MakeOvUpdates.puma" */
    OverlapUpdates (owner, var->SUBSTRING_VAR.IND_EXP);
   }
    return;
@@ -1080,24 +1060,20 @@ static void OverlapVarUpdates
 # endif
 {
   if (right_var->Kind == kINDEXED_VAR) {
-# line 738 "MakeOvUpdates.puma"
+/* line 743 "MakeOvUpdates.puma" */
  {
   shift_vector pos;
-  bool found;
+  rbool found;
   {
-# line 740 "MakeOvUpdates.puma"
-
-# line 741 "MakeOvUpdates.puma"
-
-# line 743 "MakeOvUpdates.puma"
+/* line 748 "MakeOvUpdates.puma" */
    GetShifting (left_var, right_var, & found, & pos);
-# line 745 "MakeOvUpdates.puma"
+/* line 750 "MakeOvUpdates.puma" */
    if (! ((found))) goto yyL1;
   {
-# line 747 "MakeOvUpdates.puma"
+/* line 752 "MakeOvUpdates.puma" */
    if (! ((ValidOverlapUpdate (right_var, & pos)))) goto yyL1;
   {
-# line 749 "MakeOvUpdates.puma"
+/* line 754 "MakeOvUpdates.puma" */
    SetOverlapUpdate (right_var, & pos);
   }
   }
@@ -1107,7 +1083,7 @@ static void OverlapVarUpdates
 yyL1:;
 
   }
-# line 752 "MakeOvUpdates.puma"
+/* line 757 "MakeOvUpdates.puma" */
    return;
 
 ;
@@ -1123,17 +1099,17 @@ static void ApplyShiftToVar
 # endif
 {
   if (var->Kind == kINDEXED_VAR) {
-# line 765 "MakeOvUpdates.puma"
+/* line 770 "MakeOvUpdates.puma" */
   {
-# line 767 "MakeOvUpdates.puma"
+/* line 772 "MakeOvUpdates.puma" */
    ApplyShiftToIndexes (var->INDEXED_VAR.IND_EXPS, 0, pos);
   }
    return;
 
   }
-# line 770 "MakeOvUpdates.puma"
+/* line 775 "MakeOvUpdates.puma" */
   {
-# line 771 "MakeOvUpdates.puma"
+/* line 776 "MakeOvUpdates.puma" */
    failure_protocol (MODULE, "ApplyShiftToVar", var);
   }
    return;
@@ -1151,10 +1127,11 @@ static void ApplyShiftToIndexes
  pshift pos;
 # endif
 {
+ yyRecursion:
   if (indexes->Kind == kBTE_EMPTY) {
-# line 776 "MakeOvUpdates.puma"
+/* line 781 "MakeOvUpdates.puma" */
   {
-# line 777 "MakeOvUpdates.puma"
+/* line 782 "MakeOvUpdates.puma" */
    if (! ((pos -> shift_rank == n))) goto yyL1;
   }
    return;
@@ -1163,31 +1140,33 @@ yyL1:;
   }
   if (indexes->Kind == kBTE_LIST) {
   if (indexes->BTE_LIST.Elem->Kind == kSLICE_EXP) {
-# line 780 "MakeOvUpdates.puma"
+/* line 785 "MakeOvUpdates.puma" */
   {
-# line 782 "MakeOvUpdates.puma"
- indexes->BTE_LIST.Elem->SLICE_EXP.START = AddConstant (indexes->BTE_LIST.Elem->SLICE_EXP.START, -pos->shift_pos[n]);
+/* line 787 "MakeOvUpdates.puma" */
+ indexes->BTE_LIST.Elem->SLICE_EXP.FIRST = AddConstant (indexes->BTE_LIST.Elem->SLICE_EXP.FIRST, -pos->shift_pos[n]);
     indexes->BTE_LIST.Elem->SLICE_EXP.STOP  = AddConstant (indexes->BTE_LIST.Elem->SLICE_EXP.STOP , -pos->shift_pos[n]);
   
-# line 786 "MakeOvUpdates.puma"
-   ApplyShiftToIndexes (indexes->BTE_LIST.Next, n + 1, pos);
+/* line 791 "MakeOvUpdates.puma" */
+   indexes = indexes->BTE_LIST.Next;
+   n = n + 1;
+   goto yyRecursion;
   }
-   return;
 
   }
-# line 789 "MakeOvUpdates.puma"
+/* line 794 "MakeOvUpdates.puma" */
   {
-# line 791 "MakeOvUpdates.puma"
+/* line 796 "MakeOvUpdates.puma" */
  indexes->BTE_LIST.Elem = AddConstant (indexes->BTE_LIST.Elem, pos->shift_pos[n]); 
-# line 793 "MakeOvUpdates.puma"
-   ApplyShiftToIndexes (indexes->BTE_LIST.Next, n + 1, pos);
+/* line 798 "MakeOvUpdates.puma" */
+   indexes = indexes->BTE_LIST.Next;
+   n = n + 1;
+   goto yyRecursion;
   }
-   return;
 
   }
-# line 796 "MakeOvUpdates.puma"
+/* line 801 "MakeOvUpdates.puma" */
   {
-# line 797 "MakeOvUpdates.puma"
+/* line 802 "MakeOvUpdates.puma" */
    failure_protocol (MODULE, "ApplyShiftToIndexes", indexes);
   }
    return;
@@ -1195,7 +1174,7 @@ yyL1:;
 ;
 }
 
-static bool ValidOverlapUpdate
+static rbool ValidOverlapUpdate
 # if defined __STDC__ | defined __cplusplus
 (register tTree var, pshift pos)
 # else
@@ -1205,114 +1184,46 @@ static bool ValidOverlapUpdate
 # endif
 {
   if (var->Kind == kINDEXED_VAR) {
-# line 810 "MakeOvUpdates.puma"
+/* line 815 "MakeOvUpdates.puma" */
   {
-# line 811 "MakeOvUpdates.puma"
+/* line 816 "MakeOvUpdates.puma" */
    if (! ((ValidOverlapUpdate (var->INDEXED_VAR.IND_VAR, pos)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   if (var->Kind == kUSED_VAR) {
-# line 814 "MakeOvUpdates.puma"
+/* line 819 "MakeOvUpdates.puma" */
   {
-# line 815 "MakeOvUpdates.puma"
+/* line 820 "MakeOvUpdates.puma" */
    if (! ((ValidOverlapUpdate (var->USED_VAR.VARNAME, pos)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   if (var->Kind == kVAR_OBJ) {
-# line 818 "MakeOvUpdates.puma"
+/* line 823 "MakeOvUpdates.puma" */
   {
-# line 820 "MakeOvUpdates.puma"
+/* line 825 "MakeOvUpdates.puma" */
    if (! ((VarRank (var->VAR_OBJ.Object) == pos -> shift_rank))) goto yyL3;
   {
-# line 822 "MakeOvUpdates.puma"
-   if (! ((IsEnoughVarOverlap (ArrayFormals (var->VAR_OBJ.Object), 0, pos)))) goto yyL3;
+/* line 827 "MakeOvUpdates.puma" */
+   if (! ((HasEnoughShadow (var->VAR_OBJ.Object, pos)))) goto yyL3;
   }
   }
-   return true;
+   return rtrue;
 yyL3:;
 
   }
-  return false;
+  return rfalse;
 }
 
-static bool IsEnoughVarOverlap
-# if defined __STDC__ | defined __cplusplus
-(register tTree indexes, register int dim, pshift pos)
-# else
-(indexes, dim, pos)
- register tTree indexes;
- register int dim;
- pshift pos;
-# endif
-{
-  if (indexes->Kind == kSHAPE_EMPTY) {
-# line 835 "MakeOvUpdates.puma"
-   return true;
-
-  }
-  if (indexes->Kind == kSHAPE_LIST) {
-# line 840 "MakeOvUpdates.puma"
-  {
-# line 841 "MakeOvUpdates.puma"
-   if (! ((SufficientOverlapIndex (indexes->SHAPE_LIST.Elem, pos -> shift_pos [dim])))) goto yyL2;
-  {
-# line 842 "MakeOvUpdates.puma"
-   if (! ((IsEnoughVarOverlap (indexes->SHAPE_LIST.Next, dim + 1, pos)))) goto yyL2;
-  }
-  }
-   return true;
-yyL2:;
-
-  }
-  return false;
-}
-
-static bool SufficientOverlapIndex
-# if defined __STDC__ | defined __cplusplus
-(register tTree index, register int pos)
-# else
-(index, pos)
- register tTree index;
- register int pos;
-# endif
-{
-  if (index->Kind == kOVERLAP_SPEC) {
-# line 855 "MakeOvUpdates.puma"
-  {
-# line 857 "MakeOvUpdates.puma"
-   if (! ((index->OVERLAP_SPEC.right_size >= - pos))) goto yyL1;
-  {
-# line 858 "MakeOvUpdates.puma"
-   if (! ((index->OVERLAP_SPEC.left_size >= pos))) goto yyL1;
-  }
-  }
-   return true;
-yyL1:;
-
-  }
-  if (Tree_IsType (index, kSHAPE_SPEC)) {
-# line 861 "MakeOvUpdates.puma"
-  {
-# line 863 "MakeOvUpdates.puma"
-   if (! ((SufficientOverlapIndex (index->SHAPE_SPEC.Overlap, pos)))) goto yyL2;
-  }
-   return true;
-yyL2:;
-
-  }
-  return false;
-}
-
-void BeginMakeOvUpdates ()
+void BeginMakeOvUpdates ARGS ((void))
 {
 }
 
-void CloseMakeOvUpdates ()
+void CloseMakeOvUpdates ARGS ((void))
 {
 }
