@@ -1,6 +1,13 @@
 # include "MakeRagged.h"
-# include "yyMakeRagged.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,31 +15,28 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 27 "MakeRagged.puma"
+/* line 27 "MakeRagged.puma" */
 
 
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 # include "protocol.h"
 
 # include "DefTable.h"
@@ -42,7 +46,7 @@
 # include "Nesting.h"
 # include "Transform.h"
 # include "MapDefs.h"     /* SetDistribution */
-# include "SemMap.h"      /* EvalObjDistribution */
+# include "SemMap.h"      /* EvalObjSpecMapping  */
 # include "BuildTree.h"   /* ConcatDecls         */
 # include "Rank.h"
 
@@ -50,11 +54,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* MakeRagged_Exit) () = yyExit;
+# include "yyMakeRagged.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* MakeRagged_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -63,36 +81,37 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module MakeRagged, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module MakeRagged, routine %s failed\n",
+  yyFunction);
  MakeRagged_Exit ();
 }
 
-tTree MakeRaggedDecls ARGS((tTree decl, tDefinitions obj));
-static tTree TranslateTreeObject ARGS((tDefinitions tree_obj, tDefinitions ragged_obj, int level, tIdent name, tTree type));
-static tTree TranslateTypeComponents ARGS((tDefinitions tree_obj, tDefinitions ragged_obj, int level, tIdent name, tDefinitions entries));
-static tDefinitions GetTypeEntries ARGS((tDefinitions t));
-static tIdent AddId ARGS((tIdent name, tIdent add));
-static void AddRaggedObject ARGS((tDefinitions ragged_obj, int level, tDefinitions obj));
-static tDefinitions AddRaggedComponent ARGS((tDefinitions levels, int depth, int level, tDefinitions obj));
-static int GetDistributionLevel ARGS((tDefinitions obj));
-static int GetFirstDistDim ARGS((tDefinitions dim));
-static tTree MakeDistributeDecl ARGS((tIdent name, int level, int dist_level));
-bool IsTreeAccess ARGS((tTree t));
-tTree MakeRaggedAccess ARGS((tTree var));
-static bool IsRaggedVarAccess ARGS((tTree var));
-static tTree MakeRaggedVarAccess ARGS((tTree var));
-tTree MakeRaggedTreeAccess ARGS((tTree var));
-static tTree AppendIndexes ARGS((tTree var, tTree indexes));
-static void AppendId ARGS((tTree var, tIdent id));
-tTree MakeRaggedAllocates ARGS((tTree stmt));
-static tTree TranslateAllocParams ARGS((tTree params));
-tTree MakeRaggedDeallocates ARGS((tTree stmt));
-static tTree CheckEmptyDeallocate ARGS((tTree stmt));
-static tTree TranslateDeallocParams ARGS((tTree params));
-static void GetRaggedObject ARGS((tTree t, tDefinitions * yyP1));
-static tTree MakeAllAllocates ARGS((tDefinitions def, tTree tail));
-static tTree MakeAllDeallocates ARGS((tDefinitions def, tTree tail));
-static tTree MakeDeferredShape ARGS((int rank));
+tTree MakeRaggedDecls ARGS ((tTree decl, tDefinitions obj));
+static tTree TranslateTreeObject ARGS ((tDefinitions tree_obj, tDefinitions ragged_obj, int level, tIdent name, tTree type));
+static tTree TranslateTypeComponents ARGS ((tDefinitions tree_obj, tDefinitions ragged_obj, int level, tIdent name, tDefinitions entries));
+static tDefinitions GetTypeEntries ARGS ((tDefinitions t));
+static tIdent AddId ARGS ((tIdent name, tIdent add));
+static void AddRaggedObject ARGS ((tDefinitions ragged_obj, int level, tDefinitions obj));
+static tDefinitions AddRaggedComponent ARGS ((tDefinitions levels, int depth, int level, tDefinitions obj));
+static int GetDistributionLevel ARGS ((tDefinitions obj));
+static int GetFirstDistDim ARGS ((tDefinitions dim));
+static tTree MakeDistributeDecl ARGS ((tIdent name, int level, int dist_level));
+rbool IsTreeAccess ARGS ((tTree t));
+tTree MakeRaggedAccess ARGS ((tTree var));
+static rbool IsRaggedVarAccess ARGS ((tTree var));
+static tTree MakeRaggedVarAccess ARGS ((tTree var));
+tTree MakeRaggedTreeAccess ARGS ((tTree var));
+static tTree AppendIndexes ARGS ((tTree var, tTree indexes));
+static void AppendId ARGS ((tTree var, tIdent id));
+tTree MakeRaggedAllocates ARGS ((tTree stmt));
+static tTree TranslateAllocParams ARGS ((tTree params));
+tTree MakeRaggedDeallocates ARGS ((tTree stmt));
+static tTree CheckEmptyDeallocate ARGS ((tTree stmt));
+static tTree TranslateDeallocParams ARGS ((tTree params));
+static void GetRaggedObject ARGS ((tTree t, tDefinitions * yyP1));
+static tTree MakeAllAllocates ARGS ((tDefinitions def, tTree tail));
+static tTree MakeAllDeallocates ARGS ((tDefinitions def, tTree tail));
+static tTree MakeDeferredShape ARGS ((int rank));
 
 tTree MakeRaggedDecls
 # if defined __STDC__ | defined __cplusplus
@@ -103,9 +122,9 @@ tTree MakeRaggedDecls
  register tDefinitions obj;
 # endif
 {
-# line 56 "MakeRagged.puma"
+/* line 56 "MakeRagged.puma" */
   {
-# line 58 "MakeRagged.puma"
+/* line 58 "MakeRagged.puma" */
    if (! ((! IsTreeObject (obj)))) goto yyL1;
   }
    return decl;
@@ -113,19 +132,13 @@ yyL1:;
 
   if (decl->Kind == kVAR_DECL) {
   if (obj->Kind == kVarObject) {
-# line 63 "MakeRagged.puma"
+/* line 63 "MakeRagged.puma" */
  {
   tDefinitions ragged_obj;
   tTree ragged_decl;
   tTree comp_decls;
   {
-# line 66 "MakeRagged.puma"
-
-# line 67 "MakeRagged.puma"
-
-# line 68 "MakeRagged.puma"
-
-# line 70 "MakeRagged.puma"
+/* line 70 "MakeRagged.puma" */
  int i;
 
      ragged_obj = mRaggedObject (obj->VarObject.Ident, obj->VarObject.decl, obj->VarObject.private, obj->VarObject.in, 0, mSCOPE_EMPTY (), obj->VarObject.Dist);
@@ -142,21 +155,19 @@ yyL1:;
 
    
   }
-  {
    return mDECL_LIST (ragged_decl, comp_decls);
-  }
  }
 
   }
   }
   if (decl->Kind == kTREE_DECL) {
-# line 89 "MakeRagged.puma"
+/* line 89 "MakeRagged.puma" */
    return NoTree;
 
   }
-# line 94 "MakeRagged.puma"
+/* line 94 "MakeRagged.puma" */
   {
-# line 96 "MakeRagged.puma"
+/* line 96 "MakeRagged.puma" */
    failure_protocol (MODULE, "MakeRaggedDecls", decl);
   }
    return NoTree;
@@ -175,35 +186,22 @@ static tTree TranslateTreeObject
  register tTree type;
 # endif
 {
-  if (type->Kind == kPOINTER_TYPE) {
-# line 110 "MakeRagged.puma"
-   return TranslateTreeObject (tree_obj, ragged_obj, level, name, type->POINTER_TYPE.PTR_COMP);
-
-  }
   if (type->Kind == kARRAY_TYPE) {
   if (type->ARRAY_TYPE.ARRAY_COMP_TYPE->Kind == kTYPE_ID) {
-# line 115 "MakeRagged.puma"
+/* line 110 "MakeRagged.puma" */
    return TranslateTypeComponents (tree_obj, ragged_obj, level + 1, name, GetTypeEntries (type->ARRAY_TYPE.ARRAY_COMP_TYPE->TYPE_ID.ID->TYPE_OBJ.Object));
 
   }
   }
   if (ragged_obj->Kind == kRaggedObject) {
-# line 121 "MakeRagged.puma"
+/* line 116 "MakeRagged.puma" */
  {
   tTree new_decl;
   tTree new_dist;
   tTree new_type;
   tDefinitions new_obj;
   {
-# line 123 "MakeRagged.puma"
-
-# line 124 "MakeRagged.puma"
-
-# line 126 "MakeRagged.puma"
-
-# line 127 "MakeRagged.puma"
-
-# line 129 "MakeRagged.puma"
+/* line 124 "MakeRagged.puma" */
  int i;
 
      if (level > ragged_obj->RaggedObject.depth) ragged_obj->RaggedObject.depth = level;
@@ -215,7 +213,7 @@ static tTree TranslateTreeObject
 
      new_obj = MakeNewObject (new_decl, ragged_obj);
 
-     new_obj->VarObject.Kind->VarLocal.dynamic = arr_assumed_shape;
+     new_obj->VarObject.arr_kind = arr_assumed_shape;
 
      new_dist = MakeDistributeDecl (name, level, GetDistributionLevel (tree_obj));
 
@@ -224,17 +222,16 @@ static tTree TranslateTreeObject
      AddRaggedObject (ragged_obj, level, new_obj);
 
      SetDistributions (new_dist, GetCurrentUnitObject());
-     EvalObjDistribution (new_obj);
+     EvalObjSpecMapping (new_obj);
 
    
   }
-  {
    return mDECL_LIST (new_decl, mDECL_LIST (new_dist, NoTree));
-  }
  }
 
   }
  yyAbort ("TranslateTreeObject");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static tTree TranslateTypeComponents
@@ -250,21 +247,22 @@ static tTree TranslateTypeComponents
 # endif
 {
   if (entries->Kind == kENTRY_LIST) {
-# line 161 "MakeRagged.puma"
+/* line 156 "MakeRagged.puma" */
    return ConcatDecls (TranslateTypeComponents (tree_obj, ragged_obj, level, name, entries->ENTRY_LIST.Elem), TranslateTypeComponents (tree_obj, ragged_obj, level, name, entries->ENTRY_LIST.Next));
 
   }
   if (entries->Kind == kENTRY_EMPTY) {
-# line 168 "MakeRagged.puma"
+/* line 163 "MakeRagged.puma" */
    return NoTree;
 
   }
   if (entries->Kind == kVarObject) {
-# line 173 "MakeRagged.puma"
+/* line 168 "MakeRagged.puma" */
    return TranslateTreeObject (tree_obj, ragged_obj, level, AddId (name, entries->VarObject.Ident), GetObjectType (entries));
 
   }
  yyAbort ("TranslateTypeComponents");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static tDefinitions GetTypeEntries
@@ -276,11 +274,12 @@ static tDefinitions GetTypeEntries
 # endif
 {
   if (t->Kind == kTypeObject) {
-# line 185 "MakeRagged.puma"
+/* line 180 "MakeRagged.puma" */
    return t->TypeObject.Components;
 
   }
  yyAbort ("GetTypeEntries");
+ { tDefinitions yyDummy; return yyDummy; }
 }
 
 static tIdent AddId
@@ -292,17 +291,17 @@ static tIdent AddId
  register tIdent add;
 # endif
 {
-# line 192 "MakeRagged.puma"
+/* line 187 "MakeRagged.puma" */
 
-char str1[MAXID_LENGTH], str2[MAXID_LENGTH], str3[MAXID_LENGTH]; 
+char str1[MAX_ID_LENGTH], str2[MAX_ID_LENGTH], str3[MAX_ID_LENGTH]; 
 
-# line 196 "MakeRagged.puma"
+/* line 191 "MakeRagged.puma" */
   {
-# line 198 "MakeRagged.puma"
+/* line 193 "MakeRagged.puma" */
    GetString (name, str1);
-# line 199 "MakeRagged.puma"
+/* line 194 "MakeRagged.puma" */
    GetString (add, str2);
-# line 200 "MakeRagged.puma"
+/* line 195 "MakeRagged.puma" */
    sprintf (str3, "%s_%s", str1, str2);
   }
    return IsIdent (str3);
@@ -320,9 +319,9 @@ static void AddRaggedObject
 # endif
 {
   if (ragged_obj->Kind == kRaggedObject) {
-# line 211 "MakeRagged.puma"
+/* line 206 "MakeRagged.puma" */
   {
-# line 213 "MakeRagged.puma"
+/* line 208 "MakeRagged.puma" */
  if (level > ragged_obj->RaggedObject.depth) ragged_obj->RaggedObject.depth = level;
 
      printf ("add ragged component at level  %d\n", level);
@@ -347,34 +346,34 @@ static tDefinitions AddRaggedComponent
  register tDefinitions obj;
 # endif
 {
-# line 224 "MakeRagged.puma"
+/* line 219 "MakeRagged.puma" */
   {
-# line 226 "MakeRagged.puma"
+/* line 221 "MakeRagged.puma" */
    printf ("add ragged component at depth %d, level %d\n", depth, level);
-# line 227 "MakeRagged.puma"
+/* line 222 "MakeRagged.puma" */
    goto yyL1;
   }
 yyL1:;
 
   if (levels->Kind == kSCOPE_LIST) {
-# line 230 "MakeRagged.puma"
+/* line 225 "MakeRagged.puma" */
   {
-# line 232 "MakeRagged.puma"
+/* line 227 "MakeRagged.puma" */
    if (! ((depth == level))) goto yyL2;
   {
-# line 234 "MakeRagged.puma"
+/* line 229 "MakeRagged.puma" */
  levels->SCOPE_LIST.Table = mENTRY_LIST (obj, levels->SCOPE_LIST.Table); 
   }
   }
    return levels;
 yyL2:;
 
-# line 239 "MakeRagged.puma"
+/* line 234 "MakeRagged.puma" */
   {
-# line 241 "MakeRagged.puma"
+/* line 236 "MakeRagged.puma" */
    if (! ((depth < level))) goto yyL3;
   {
-# line 243 "MakeRagged.puma"
+/* line 238 "MakeRagged.puma" */
  levels->SCOPE_LIST.Hidden = AddRaggedComponent (levels->SCOPE_LIST.Hidden, depth+1, level, obj); 
   }
   }
@@ -383,24 +382,24 @@ yyL3:;
 
   }
   if (levels->Kind == kSCOPE_EMPTY) {
-# line 248 "MakeRagged.puma"
+/* line 243 "MakeRagged.puma" */
   {
-# line 250 "MakeRagged.puma"
+/* line 245 "MakeRagged.puma" */
    if (! ((depth == level))) goto yyL4;
   }
    return mSCOPE_LIST (mENTRY_LIST (obj, mENTRY_EMPTY ()), mSCOPE_EMPTY ());
 yyL4:;
 
-# line 255 "MakeRagged.puma"
+/* line 250 "MakeRagged.puma" */
   {
-# line 257 "MakeRagged.puma"
+/* line 252 "MakeRagged.puma" */
    if (! ((depth < level))) goto yyL5;
   }
    return mSCOPE_LIST (mSCOPE_EMPTY (), AddRaggedComponent (levels, depth + 1, level, obj));
 yyL5:;
 
   }
-# line 263 "MakeRagged.puma"
+/* line 258 "MakeRagged.puma" */
    return levels;
 
 }
@@ -413,17 +412,19 @@ static int GetDistributionLevel
  register tDefinitions obj;
 # endif
 {
+ yyRecursion:
   if (obj->Kind == kVarObject) {
-# line 281 "MakeRagged.puma"
-   return GetDistributionLevel (obj->VarObject.Dist);
+/* line 276 "MakeRagged.puma" */
+   obj = obj->VarObject.Dist->Mapping.spec;
+   goto yyRecursion;
 
   }
-  if (obj->Kind == kNodeDistribution) {
-# line 286 "MakeRagged.puma"
-   return GetFirstDistDim (obj->NodeDistribution.ArrayList);
+  if (obj->Kind == kMapDist) {
+/* line 281 "MakeRagged.puma" */
+   return GetFirstDistDim (obj->MapDist.Distribution->Distribution.ArrayList);
 
   }
-# line 291 "MakeRagged.puma"
+/* line 286 "MakeRagged.puma" */
    return 0;
 
 }
@@ -438,20 +439,21 @@ static int GetFirstDistDim
 {
   if (dim->Kind == kDIM_LIST) {
   if (dim->DIM_LIST.Elem->Kind == kDistributedDimension) {
-# line 297 "MakeRagged.puma"
+/* line 292 "MakeRagged.puma" */
    return 1;
 
   }
-# line 302 "MakeRagged.puma"
+/* line 297 "MakeRagged.puma" */
    return GetFirstDistDim (dim->DIM_LIST.Next) + 1;
 
   }
   if (dim->Kind == kDIM_EMPTY) {
-# line 307 "MakeRagged.puma"
+/* line 302 "MakeRagged.puma" */
    return 1;
 
   }
  yyAbort ("GetFirstDistDim");
+ { int yyDummy; return yyDummy; }
 }
 
 static tTree MakeDistributeDecl
@@ -464,13 +466,11 @@ static tTree MakeDistributeDecl
  register int dist_level;
 # endif
 {
-# line 320 "MakeRagged.puma"
+/* line 315 "MakeRagged.puma" */
  {
   tTree decl;
   {
-# line 322 "MakeRagged.puma"
-
-# line 324 "MakeRagged.puma"
+/* line 319 "MakeRagged.puma" */
  
 
      int i;
@@ -489,14 +489,12 @@ static tTree MakeDistributeDecl
                               mDEFAULT_PROCESSOR_ARRAY());
    
   }
-  {
    return decl;
-  }
  }
 
 }
 
-bool IsTreeAccess
+rbool IsTreeAccess
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -505,36 +503,36 @@ bool IsTreeAccess
 # endif
 {
   if (t->Kind == kINDEXED_VAR) {
-# line 347 "MakeRagged.puma"
+/* line 342 "MakeRagged.puma" */
   {
-# line 349 "MakeRagged.puma"
+/* line 344 "MakeRagged.puma" */
    if (! ((IsTreeAccess (t->INDEXED_VAR.IND_VAR)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   if (t->Kind == kSELECTED_VAR) {
-# line 352 "MakeRagged.puma"
+/* line 347 "MakeRagged.puma" */
   {
-# line 354 "MakeRagged.puma"
+/* line 349 "MakeRagged.puma" */
    if (! ((IsTreeAccess (t->SELECTED_VAR.SELEC_VAR)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   if (t->Kind == kUSED_VAR) {
-# line 357 "MakeRagged.puma"
+/* line 352 "MakeRagged.puma" */
   {
-# line 359 "MakeRagged.puma"
+/* line 354 "MakeRagged.puma" */
    if (! ((IsTreeObject (t->USED_VAR.VARNAME->VAR_OBJ.Object)))) goto yyL3;
   }
-   return true;
+   return rtrue;
 yyL3:;
 
   }
-  return false;
+  return rfalse;
 }
 
 tTree MakeRaggedAccess
@@ -545,20 +543,20 @@ tTree MakeRaggedAccess
  register tTree var;
 # endif
 {
-# line 370 "MakeRagged.puma"
+/* line 365 "MakeRagged.puma" */
   {
-# line 372 "MakeRagged.puma"
+/* line 367 "MakeRagged.puma" */
    if (! ((IsRaggedVarAccess (var)))) goto yyL1;
   }
    return MakeRaggedVarAccess (var);
 yyL1:;
 
-# line 377 "MakeRagged.puma"
+/* line 372 "MakeRagged.puma" */
    return MakeRaggedTreeAccess (var);
 
 }
 
-static bool IsRaggedVarAccess
+static rbool IsRaggedVarAccess
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -567,16 +565,16 @@ static bool IsRaggedVarAccess
 # endif
 {
   if (var->Kind == kSELECTED_VAR) {
-# line 384 "MakeRagged.puma"
+/* line 379 "MakeRagged.puma" */
   {
-# line 386 "MakeRagged.puma"
+/* line 381 "MakeRagged.puma" */
    if (! ((VarRank (var->SELECTED_VAR.SELECTOR->REC_COMP.Object) == 0))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
-  return false;
+  return rfalse;
 }
 
 static tTree MakeRaggedVarAccess
@@ -588,36 +586,32 @@ static tTree MakeRaggedVarAccess
 # endif
 {
   if (var->Kind == kINDEXED_VAR) {
-# line 391 "MakeRagged.puma"
+/* line 386 "MakeRagged.puma" */
    return AppendIndexes (MakeRaggedVarAccess (var->INDEXED_VAR.IND_VAR), var->INDEXED_VAR.IND_EXPS);
 
   }
   if (var->Kind == kSELECTED_VAR) {
-# line 396 "MakeRagged.puma"
+/* line 391 "MakeRagged.puma" */
  {
   tTree new;
   {
-# line 398 "MakeRagged.puma"
-
-# line 400 "MakeRagged.puma"
+/* line 395 "MakeRagged.puma" */
    new = MakeRaggedVarAccess (var->SELECTED_VAR.SELEC_VAR);
-# line 402 "MakeRagged.puma"
+/* line 397 "MakeRagged.puma" */
    AppendId (new, var->SELECTED_VAR.SELECTOR->REC_COMP.Ident);
   }
-  {
    return new;
-  }
  }
 
   }
   if (var->Kind == kUSED_VAR) {
-# line 407 "MakeRagged.puma"
+/* line 402 "MakeRagged.puma" */
    return var;
 
   }
-# line 412 "MakeRagged.puma"
+/* line 407 "MakeRagged.puma" */
   {
-# line 413 "MakeRagged.puma"
+/* line 408 "MakeRagged.puma" */
    failure_protocol (MODULE, "MakeRaggedVarAccess", var);
   }
    return NoTree;
@@ -632,28 +626,30 @@ tTree MakeRaggedTreeAccess
  register tTree var;
 # endif
 {
+ yyRecursion:
   if (var->Kind == kINDEXED_VAR) {
-# line 425 "MakeRagged.puma"
+/* line 420 "MakeRagged.puma" */
    return AppendIndexes (MakeRaggedTreeAccess (var->INDEXED_VAR.IND_VAR), var->INDEXED_VAR.IND_EXPS);
 
   }
   if (var->Kind == kSELECTED_VAR) {
-# line 430 "MakeRagged.puma"
-   return MakeRaggedTreeAccess (var->SELECTED_VAR.SELEC_VAR);
+/* line 425 "MakeRagged.puma" */
+   var = var->SELECTED_VAR.SELEC_VAR;
+   goto yyRecursion;
 
   }
   if (var->Kind == kUSED_VAR) {
-# line 435 "MakeRagged.puma"
+/* line 430 "MakeRagged.puma" */
   {
-# line 437 "MakeRagged.puma"
+/* line 432 "MakeRagged.puma" */
  var->USED_VAR.VARNAME->VAR_OBJ.Object = GetGlobalObject (var->USED_VAR.VARNAME->VAR_OBJ.Ident); 
   }
    return var;
 
   }
-# line 442 "MakeRagged.puma"
+/* line 437 "MakeRagged.puma" */
   {
-# line 443 "MakeRagged.puma"
+/* line 438 "MakeRagged.puma" */
    failure_protocol (MODULE, "MakeRaggedVarAccess", var);
   }
    return NoTree;
@@ -670,14 +666,14 @@ static tTree AppendIndexes
 # endif
 {
   if (var->Kind == kUSED_VAR) {
-# line 449 "MakeRagged.puma"
+/* line 444 "MakeRagged.puma" */
    return mINDEXED_VAR (var, indexes);
 
   }
   if (var->Kind == kINDEXED_VAR) {
-# line 454 "MakeRagged.puma"
+/* line 449 "MakeRagged.puma" */
   {
-# line 456 "MakeRagged.puma"
+/* line 451 "MakeRagged.puma" */
  var->INDEXED_VAR.IND_EXPS = AppendIndexes (var->INDEXED_VAR.IND_EXPS, indexes); 
   }
    return var;
@@ -685,25 +681,25 @@ static tTree AppendIndexes
   }
   if (var->Kind == kBTE_LIST) {
   if (var->BTE_LIST.Next->Kind == kBTE_EMPTY) {
-# line 461 "MakeRagged.puma"
+/* line 456 "MakeRagged.puma" */
   {
-# line 463 "MakeRagged.puma"
+/* line 458 "MakeRagged.puma" */
  var->BTE_LIST.Next = indexes; 
   }
    return var;
 
   }
-# line 468 "MakeRagged.puma"
+/* line 463 "MakeRagged.puma" */
   {
-# line 470 "MakeRagged.puma"
+/* line 465 "MakeRagged.puma" */
  var->BTE_LIST.Next = AppendIndexes (var->BTE_LIST.Next, indexes); 
   }
    return var;
 
   }
-# line 475 "MakeRagged.puma"
+/* line 470 "MakeRagged.puma" */
   {
-# line 477 "MakeRagged.puma"
+/* line 472 "MakeRagged.puma" */
    failure_protocol (MODULE, "AppendIndexes", var);
   }
    return NoTree;
@@ -719,10 +715,11 @@ static void AppendId
  register tIdent id;
 # endif
 {
+ yyRecursion:
   if (var->Kind == kUSED_VAR) {
-# line 484 "MakeRagged.puma"
+/* line 479 "MakeRagged.puma" */
   {
-# line 486 "MakeRagged.puma"
+/* line 481 "MakeRagged.puma" */
  var->USED_VAR.VARNAME->VAR_OBJ.Ident = AddId (var->USED_VAR.VARNAME->VAR_OBJ.Ident, id);
      var->USED_VAR.VARNAME->VAR_OBJ.Object  = GetGlobalObject (var->USED_VAR.VARNAME->VAR_OBJ.Ident);
    
@@ -731,17 +728,17 @@ static void AppendId
 
   }
   if (var->Kind == kINDEXED_VAR) {
-# line 491 "MakeRagged.puma"
+/* line 486 "MakeRagged.puma" */
   {
-# line 493 "MakeRagged.puma"
-   AppendId (var->INDEXED_VAR.IND_VAR, id);
+/* line 488 "MakeRagged.puma" */
+   var = var->INDEXED_VAR.IND_VAR;
+   goto yyRecursion;
   }
-   return;
 
   }
-# line 496 "MakeRagged.puma"
+/* line 491 "MakeRagged.puma" */
   {
-# line 498 "MakeRagged.puma"
+/* line 493 "MakeRagged.puma" */
    failure_protocol (MODULE, "AppendId", var);
   }
    return;
@@ -759,18 +756,18 @@ tTree MakeRaggedAllocates
 {
   if (stmt->Kind == kACF_BASIC) {
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kALLOCATE_STMT) {
-# line 509 "MakeRagged.puma"
+/* line 504 "MakeRagged.puma" */
   {
-# line 511 "MakeRagged.puma"
+/* line 506 "MakeRagged.puma" */
  stmt->ACF_BASIC.BASIC_STMT->ALLOCATE_STMT.PARAMS = TranslateAllocParams (stmt->ACF_BASIC.BASIC_STMT->ALLOCATE_STMT.PARAMS); 
   }
    return stmt;
 
   }
   }
-# line 516 "MakeRagged.puma"
+/* line 511 "MakeRagged.puma" */
   {
-# line 518 "MakeRagged.puma"
+/* line 513 "MakeRagged.puma" */
    failure_protocol (MODULE, "MakeRaggedAllocates", stmt);
   }
    return NoTree;
@@ -788,32 +785,30 @@ static tTree TranslateAllocParams
   if (params->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Elem->Kind == kVAR_PARAM) {
   if (params->BTP_LIST.Elem->VAR_PARAM.V->Kind == kUSED_VAR) {
-# line 531 "MakeRagged.puma"
+/* line 526 "MakeRagged.puma" */
  {
   tDefinitions yyV1;
   {
-# line 533 "MakeRagged.puma"
+/* line 528 "MakeRagged.puma" */
    if (! ((IsTreeAccess (params->BTP_LIST.Elem->VAR_PARAM.V)))) goto yyL1;
   {
-# line 535 "MakeRagged.puma"
+/* line 530 "MakeRagged.puma" */
    GetRaggedObject (params->BTP_LIST.Elem->VAR_PARAM.V, & yyV1);
-# line 537 "MakeRagged.puma"
+/* line 532 "MakeRagged.puma" */
  params->BTP_LIST.Next = MakeAllAllocates (yyV1, TranslateAllocParams (params->BTP_LIST.Next)); 
   }
   }
-  {
    return params;
-  }
  }
 yyL1:;
 
   }
-# line 542 "MakeRagged.puma"
+/* line 537 "MakeRagged.puma" */
   {
-# line 544 "MakeRagged.puma"
+/* line 539 "MakeRagged.puma" */
    if (! ((IsTreeAccess (params->BTP_LIST.Elem->VAR_PARAM.V)))) goto yyL2;
   {
-# line 546 "MakeRagged.puma"
+/* line 541 "MakeRagged.puma" */
  params->BTP_LIST.Elem->VAR_PARAM.V    = MakeRaggedTreeAccess (params->BTP_LIST.Elem->VAR_PARAM.V); 
      params->BTP_LIST.Next = TranslateAllocParams (params->BTP_LIST.Next); 
   }
@@ -821,9 +816,9 @@ yyL1:;
    return params;
 yyL2:;
 
-# line 552 "MakeRagged.puma"
+/* line 547 "MakeRagged.puma" */
   {
-# line 554 "MakeRagged.puma"
+/* line 549 "MakeRagged.puma" */
  params->BTP_LIST.Next = TranslateAllocParams (params->BTP_LIST.Next); 
   }
    return params;
@@ -831,11 +826,12 @@ yyL2:;
   }
   }
   if (params->Kind == kBTP_EMPTY) {
-# line 559 "MakeRagged.puma"
+/* line 554 "MakeRagged.puma" */
    return params;
 
   }
  yyAbort ("TranslateAllocParams");
+ { tTree yyDummy; return yyDummy; }
 }
 
 tTree MakeRaggedDeallocates
@@ -848,18 +844,18 @@ tTree MakeRaggedDeallocates
 {
   if (stmt->Kind == kACF_BASIC) {
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kDEALLOCATE_STMT) {
-# line 572 "MakeRagged.puma"
+/* line 567 "MakeRagged.puma" */
   {
-# line 574 "MakeRagged.puma"
+/* line 569 "MakeRagged.puma" */
  stmt->ACF_BASIC.BASIC_STMT->DEALLOCATE_STMT.PARAMS = TranslateDeallocParams (stmt->ACF_BASIC.BASIC_STMT->DEALLOCATE_STMT.PARAMS); 
   }
    return CheckEmptyDeallocate (stmt);
 
   }
   }
-# line 579 "MakeRagged.puma"
+/* line 574 "MakeRagged.puma" */
   {
-# line 581 "MakeRagged.puma"
+/* line 576 "MakeRagged.puma" */
    failure_protocol (MODULE, "MakeRaggedAllocates", stmt);
   }
    return NoTree;
@@ -877,13 +873,13 @@ static tTree CheckEmptyDeallocate
   if (stmt->Kind == kACF_BASIC) {
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kDEALLOCATE_STMT) {
   if (stmt->ACF_BASIC.BASIC_STMT->DEALLOCATE_STMT.PARAMS->Kind == kBTP_EMPTY) {
-# line 588 "MakeRagged.puma"
+/* line 583 "MakeRagged.puma" */
    return NoTree;
 
   }
   }
   }
-# line 593 "MakeRagged.puma"
+/* line 588 "MakeRagged.puma" */
    return stmt;
 
 }
@@ -896,40 +892,40 @@ static tTree TranslateDeallocParams
  register tTree params;
 # endif
 {
+ yyRecursion:
   if (params->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Elem->Kind == kVAR_PARAM) {
   if (params->BTP_LIST.Elem->VAR_PARAM.V->Kind == kUSED_VAR) {
-# line 600 "MakeRagged.puma"
+/* line 595 "MakeRagged.puma" */
  {
   tDefinitions yyV1;
   {
-# line 602 "MakeRagged.puma"
+/* line 597 "MakeRagged.puma" */
    if (! ((IsTreeAccess (params->BTP_LIST.Elem->VAR_PARAM.V)))) goto yyL1;
   {
-# line 604 "MakeRagged.puma"
+/* line 599 "MakeRagged.puma" */
  params->BTP_LIST.Next = TranslateDeallocParams (params->BTP_LIST.Next); 
-# line 606 "MakeRagged.puma"
+/* line 601 "MakeRagged.puma" */
    GetRaggedObject (params->BTP_LIST.Elem->VAR_PARAM.V, & yyV1);
   }
   }
-  {
    return MakeAllDeallocates (yyV1, params);
-  }
  }
 yyL1:;
 
   }
-# line 611 "MakeRagged.puma"
+/* line 606 "MakeRagged.puma" */
   {
-# line 613 "MakeRagged.puma"
+/* line 608 "MakeRagged.puma" */
    if (! ((IsTreeAccess (params->BTP_LIST.Elem->VAR_PARAM.V)))) goto yyL2;
   }
-   return TranslateDeallocParams (params->BTP_LIST.Next);
+   params = params->BTP_LIST.Next;
+   goto yyRecursion;
 yyL2:;
 
-# line 619 "MakeRagged.puma"
+/* line 614 "MakeRagged.puma" */
   {
-# line 621 "MakeRagged.puma"
+/* line 616 "MakeRagged.puma" */
  params->BTP_LIST.Next = TranslateDeallocParams (params->BTP_LIST.Next); 
   }
    return params;
@@ -937,11 +933,12 @@ yyL2:;
   }
   }
   if (params->Kind == kBTP_EMPTY) {
-# line 626 "MakeRagged.puma"
+/* line 621 "MakeRagged.puma" */
    return params;
 
   }
  yyAbort ("TranslateDeallocParams");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static void GetRaggedObject
@@ -954,9 +951,9 @@ static void GetRaggedObject
 # endif
 {
   if (t->Kind == kUSED_VAR) {
-# line 633 "MakeRagged.puma"
+/* line 628 "MakeRagged.puma" */
   {
-# line 635 "MakeRagged.puma"
+/* line 630 "MakeRagged.puma" */
  t->USED_VAR.VARNAME->VAR_OBJ.Object = GetGlobalObject (t->USED_VAR.VARNAME->VAR_OBJ.Ident); 
   }
    * yyP1 = t->USED_VAR.VARNAME->VAR_OBJ.Object;
@@ -975,45 +972,45 @@ static tTree MakeAllAllocates
  register tTree tail;
 # endif
 {
+ yyRecursion:
   if (def->Kind == kRaggedObject) {
-# line 641 "MakeRagged.puma"
-   return MakeAllAllocates (def->RaggedObject.Levels, tail);
+/* line 636 "MakeRagged.puma" */
+   def = def->RaggedObject.Levels;
+   goto yyRecursion;
 
   }
   if (def->Kind == kSCOPE_LIST) {
-# line 646 "MakeRagged.puma"
+/* line 641 "MakeRagged.puma" */
   {
-# line 648 "MakeRagged.puma"
+/* line 643 "MakeRagged.puma" */
    printf ("scope list\n");
   }
    return MakeAllAllocates (def->SCOPE_LIST.Table, MakeAllAllocates (def->SCOPE_LIST.Hidden, tail));
 
   }
   if (def->Kind == kSCOPE_EMPTY) {
-# line 654 "MakeRagged.puma"
+/* line 649 "MakeRagged.puma" */
    return tail;
 
   }
   if (def->Kind == kENTRY_LIST) {
-# line 659 "MakeRagged.puma"
+/* line 654 "MakeRagged.puma" */
    return MakeAllAllocates (def->ENTRY_LIST.Elem, MakeAllAllocates (def->ENTRY_LIST.Next, tail));
 
   }
   if (def->Kind == kENTRY_EMPTY) {
-# line 665 "MakeRagged.puma"
+/* line 660 "MakeRagged.puma" */
    return tail;
 
   }
   if (def->Kind == kVarObject) {
-# line 670 "MakeRagged.puma"
+/* line 665 "MakeRagged.puma" */
  {
   tTree stmt;
   {
-# line 672 "MakeRagged.puma"
+/* line 667 "MakeRagged.puma" */
    printf ("make allocate for object of ragged array\n");
-# line 674 "MakeRagged.puma"
-
-# line 676 "MakeRagged.puma"
+/* line 671 "MakeRagged.puma" */
  stmt = mVAR_OBJ (0, def->VarObject.Ident);
      stmt->VAR_OBJ.Object = def;
 
@@ -1021,13 +1018,12 @@ static tTree MakeAllAllocates
      stmt = mBTP_LIST (stmt, tail);
    
   }
-  {
    return stmt;
-  }
  }
 
   }
  yyAbort ("MakeAllAllocates");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static tTree MakeAllDeallocates
@@ -1039,41 +1035,41 @@ static tTree MakeAllDeallocates
  register tTree tail;
 # endif
 {
+ yyRecursion:
   if (def->Kind == kRaggedObject) {
-# line 688 "MakeRagged.puma"
-   return MakeAllDeallocates (def->RaggedObject.Levels, tail);
+/* line 683 "MakeRagged.puma" */
+   def = def->RaggedObject.Levels;
+   goto yyRecursion;
 
   }
   if (def->Kind == kSCOPE_LIST) {
-# line 693 "MakeRagged.puma"
+/* line 688 "MakeRagged.puma" */
    return MakeAllDeallocates (def->SCOPE_LIST.Hidden, MakeAllDeallocates (def->SCOPE_LIST.Table, tail));
 
   }
   if (def->Kind == kSCOPE_EMPTY) {
-# line 699 "MakeRagged.puma"
+/* line 694 "MakeRagged.puma" */
    return tail;
 
   }
   if (def->Kind == kENTRY_LIST) {
-# line 704 "MakeRagged.puma"
+/* line 699 "MakeRagged.puma" */
    return MakeAllDeallocates (def->ENTRY_LIST.Next, MakeAllDeallocates (def->ENTRY_LIST.Elem, tail));
 
   }
   if (def->Kind == kENTRY_EMPTY) {
-# line 710 "MakeRagged.puma"
+/* line 705 "MakeRagged.puma" */
    return tail;
 
   }
   if (def->Kind == kVarObject) {
-# line 715 "MakeRagged.puma"
+/* line 710 "MakeRagged.puma" */
  {
   tTree params;
   {
-# line 717 "MakeRagged.puma"
+/* line 712 "MakeRagged.puma" */
    printf ("make deallocate for object of ragged array\n");
-# line 719 "MakeRagged.puma"
-
-# line 721 "MakeRagged.puma"
+/* line 716 "MakeRagged.puma" */
  params = mVAR_OBJ (0, def->VarObject.Ident);
      params->VAR_OBJ.Object = def;
 
@@ -1081,13 +1077,12 @@ static tTree MakeAllDeallocates
      params = mBTP_LIST (params, tail);
    
   }
-  {
    return params;
-  }
  }
 
   }
  yyAbort ("MakeAllDeallocates");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static tTree MakeDeferredShape
@@ -1098,36 +1093,30 @@ static tTree MakeDeferredShape
  register int rank;
 # endif
 {
-# line 733 "MakeRagged.puma"
+/* line 728 "MakeRagged.puma" */
   {
-# line 735 "MakeRagged.puma"
+/* line 730 "MakeRagged.puma" */
    if (! ((rank <= 0))) goto yyL1;
   }
    return mSHAPE_EMPTY ();
 yyL1:;
 
-# line 740 "MakeRagged.puma"
+/* line 735 "MakeRagged.puma" */
  {
   tTree shp;
   {
-# line 742 "MakeRagged.puma"
-
-# line 744 "MakeRagged.puma"
- shp = mDEFERRED_SHAPE ();
-     shp->DEFERRED_SHAPE.Overlap = mOVERLAP_SPEC (0, 0, 0, 0);
-   
+/* line 739 "MakeRagged.puma" */
+ shp = mDEFERRED_SHAPE (); 
   }
-  {
    return mSHAPE_LIST (shp, MakeDeferredShape (rank - 1));
-  }
  }
 
 }
 
-void BeginMakeRagged ()
+void BeginMakeRagged ARGS ((void))
 {
 }
 
-void CloseMakeRagged ()
+void CloseMakeRagged ARGS ((void))
 {
 }
