@@ -1,6 +1,13 @@
 # include "MakeRemapping.h"
-# include "yyMakeRemapping.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,32 +15,29 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 21 "MakeRemapping.puma"
+/* line 21 "MakeRemapping.puma" */
 
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
  
 # include "protocol.h"
 
@@ -41,12 +45,18 @@
 # include "Transform.h"    /* ComposeACF, ...                      */
 
 # include "Rank.h"
+# include "Objects.h"          /* IsLocalCall            */
 # include "Distributions.h"
 # include "MoveControl.h"
+# include "Nesting.h"          /* GetCurrentModel        */
 # include "Intrinsics.h"
 # include "SpecialMap.h"
 # include "VarDescriptor.h" 
+# include "FindHome.h"         /* FindSerialHome         */
+# include "ExpDescriptor.h"    /* GetDummyVarDescriptor  */
 # include "Descriptor.h"       /* PrintableDescriptorVar */
+# include "Expansion.h"        /* NoExpansion            */
+# include "TempAssigns.h"      /* MakeTempVarStmt        */
 
 # define MODULE "MakeRemapping"
  
@@ -54,11 +64,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* MakeRemapping_Exit) () = yyExit;
+# include "yyMakeRemapping.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* MakeRemapping_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -67,30 +91,31 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module MakeRemapping, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module MakeRemapping, routine %s failed\n",
+  yyFunction);
  MakeRemapping_Exit ();
 }
 
-void MakeRemapping ARGS((tTree t));
-static bool StopMakeRemapArgs ARGS((tTree t));
-static tTree DoMakeRemapArgs ARGS((tTree t));
-static bool CanHaveRemapping ARGS((tTree stmt));
-static void RemappingArguments ARGS((tTree basic_stmt, tTree * yyP2, tTree * yyP1));
-static void IntrArgRemapping ARGS((tIdent name, tTree var, tTree actuals, tTree * yyP4, tTree * yyP3));
-static void AlignParamList ARGS((tTree target, tTree params, tTree * yyP6, tTree * yyP5));
-static void AlignParam ARGS((tTree target, tTree param, tTree * yyP8, tTree * yyP7));
-static void ParamListRemapping ARGS((tTree actuals, tTree call, tTree * yyP10, tTree * yyP9));
-static void ParamRemapping ARGS((tTree actual, tTree call, tTree * yyP12, tTree * yyP11));
-static void IntrResRemapping ARGS((tIdent name, tTree assign, tTree * yyP14, tTree * yyP13));
-static void NewResultTmp ARGS((tTree assign, tTree * yyP16, tTree * yyP15));
-static void MakeSerialArguments ARGS((tTree actuals, tTree * yyP18, tTree * yyP17));
-static void MakeSerialArgument ARGS((tTree actual, tTree * yyP20, tTree * yyP19));
-static void MakeSerialVar ARGS((tTree var, tTree * yyP23, tTree * yyP22, tTree * yyP21));
-static bool AcceptedSerialVar ARGS((tTree var));
-static bool MightBeRedistributed ARGS((tTree p, tTree actual, tDefinitions dummy));
-static void FindParamDescriptor ARGS((tTree actual, bool * yyP25, var_descriptor * yyP24));
-static void MakeParamTemporary ARGS((tTree actual, pvar vard, tTree * yyP27, tTree * yyP26));
-static void MakeNewAssignment ARGS((tTree exp, pvar vard, tTree * yyP30, tTree * yyP29, tTree * yyP28));
+void MakeRemapping ARGS ((tTree t));
+static rbool StopMakeRemapArgs ARGS ((tTree t));
+static tTree DoMakeRemapArgs ARGS ((tTree t));
+static rbool CanHaveRemapping ARGS ((tTree stmt));
+static void RemappingArguments ARGS ((tTree basic_stmt, tTree * yyP2, tTree * yyP1));
+static void IntrArgRemapping ARGS ((tIdent name, tTree var, tTree actuals, tTree * yyP4, tTree * yyP3));
+static void AlignParamList ARGS ((tTree target, tTree params, tTree * yyP6, tTree * yyP5));
+static void AlignParam ARGS ((tTree target, tTree param, tTree * yyP8, tTree * yyP7));
+static void ParamListRemapping ARGS ((tTree actuals, tTree call, tTree * yyP10, tTree * yyP9));
+static void ParamRemapping ARGS ((tTree actual, tTree call, tTree * yyP12, tTree * yyP11));
+static void IntrResRemapping ARGS ((tIdent name, tTree assign, tTree * yyP14, tTree * yyP13));
+static void NewResultTmp ARGS ((tTree assign, tTree * yyP16, tTree * yyP15));
+static void MakeSerialArguments ARGS ((tTree actuals, tTree * yyP18, tTree * yyP17));
+static void MakeSerialArgument ARGS ((tTree actual, tTree * yyP20, tTree * yyP19));
+static void MakeSerialVar ARGS ((tTree var, tTree * yyP23, tTree * yyP22, tTree * yyP21));
+static rbool AcceptedSerialVar ARGS ((tTree var));
+static rbool MightBeRedistributed ARGS ((tTree p, tTree actual, tDefinitions dummy));
+static void FindParamDescriptor ARGS ((tTree actual, rbool * yyP25, var_descriptor * yyP24));
+static void MakeParamTemporary ARGS ((tTree actual, pvar vard, tTree * yyP27, tTree * yyP26));
+static void MakeNewAssignment ARGS ((tTree exp, pvar vard, tTree * yyP30, tTree * yyP29, tTree * yyP28));
 
 void MakeRemapping
 # if defined __STDC__ | defined __cplusplus
@@ -101,17 +126,17 @@ void MakeRemapping
 # endif
 {
   if (t->Kind == kBODY_NODE) {
-# line 54 "MakeRemapping.puma"
+/* line 60 "MakeRemapping.puma" */
   {
-# line 56 "MakeRemapping.puma"
+/* line 62 "MakeRemapping.puma" */
  t->BODY_NODE.STATS = ReplaceAST (t->BODY_NODE.STATS, StopMakeRemapArgs, DoMakeRemapArgs); 
   }
    return;
 
   }
-# line 59 "MakeRemapping.puma"
+/* line 65 "MakeRemapping.puma" */
   {
-# line 60 "MakeRemapping.puma"
+/* line 66 "MakeRemapping.puma" */
    failure_protocol (MODULE, "MakeRemapping", t);
   }
    return;
@@ -119,7 +144,7 @@ void MakeRemapping
 ;
 }
 
-static bool StopMakeRemapArgs
+static rbool StopMakeRemapArgs
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -127,10 +152,10 @@ static bool StopMakeRemapArgs
  register tTree t;
 # endif
 {
-# line 77 "MakeRemapping.puma"
+/* line 83 "MakeRemapping.puma" */
   {
-# line 78 "MakeRemapping.puma"
-   return false;
+/* line 84 "MakeRemapping.puma" */
+   return rfalse;
   }
 
 }
@@ -144,17 +169,17 @@ static tTree DoMakeRemapArgs
 # endif
 {
   if (t->Kind == kACF_BASIC) {
-# line 89 "MakeRemapping.puma"
+/* line 95 "MakeRemapping.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 91 "MakeRemapping.puma"
+/* line 97 "MakeRemapping.puma" */
    if (! ((CanHaveRemapping (t->ACF_BASIC.BASIC_STMT)))) goto yyL1;
   {
-# line 93 "MakeRemapping.puma"
+/* line 99 "MakeRemapping.puma" */
    RemappingArguments (t->ACF_BASIC.BASIC_STMT, & yyV1, & yyV2);
-# line 95 "MakeRemapping.puma"
+/* line 101 "MakeRemapping.puma" */
  if (yyV1 != NoTree)
 
       { set_protocol_stmt (t);
@@ -165,19 +190,17 @@ static tTree DoMakeRemapArgs
   
   }
   }
-  {
    return ComposeNewACF (yyV1, t, yyV2);
-  }
  }
 yyL1:;
 
   }
-# line 107 "MakeRemapping.puma"
+/* line 113 "MakeRemapping.puma" */
    return t;
 
 }
 
-static bool CanHaveRemapping
+static rbool CanHaveRemapping
 # if defined __STDC__ | defined __cplusplus
 (register tTree stmt)
 # else
@@ -186,18 +209,18 @@ static bool CanHaveRemapping
 # endif
 {
   if (stmt->Kind == kCALL_STMT) {
-# line 119 "MakeRemapping.puma"
-   return true;
+/* line 125 "MakeRemapping.puma" */
+   return rtrue;
 
   }
   if (stmt->Kind == kASSIGN_STMT) {
   if (stmt->ASSIGN_STMT.ASSIGN_EXP->Kind == kFUNC_CALL_EXP) {
-# line 122 "MakeRemapping.puma"
-   return true;
+/* line 128 "MakeRemapping.puma" */
+   return rtrue;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
 static void RemappingArguments
@@ -212,23 +235,23 @@ static void RemappingArguments
 {
   if (basic_stmt->Kind == kASSIGN_STMT) {
   if (basic_stmt->ASSIGN_STMT.ASSIGN_EXP->Kind == kFUNC_CALL_EXP) {
-# line 132 "MakeRemapping.puma"
+/* line 138 "MakeRemapping.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   tTree yyV4;
   {
-# line 136 "MakeRemapping.puma"
+/* line 142 "MakeRemapping.puma" */
    if (! ((IsIntrCall (basic_stmt->ASSIGN_STMT.ASSIGN_EXP)))) goto yyL1;
   {
-# line 138 "MakeRemapping.puma"
+/* line 144 "MakeRemapping.puma" */
    IntrArgRemapping (basic_stmt->ASSIGN_STMT.ASSIGN_EXP->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Ident, basic_stmt->ASSIGN_STMT.ASSIGN_VAR, basic_stmt->ASSIGN_STMT.ASSIGN_EXP->FUNC_CALL_EXP.FUNC_PARAMS, & yyV1, & yyV2);
-# line 139 "MakeRemapping.puma"
+/* line 145 "MakeRemapping.puma" */
    IntrResRemapping (basic_stmt->ASSIGN_STMT.ASSIGN_EXP->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Ident, basic_stmt, & yyV3, & yyV4);
-# line 141 "MakeRemapping.puma"
+/* line 147 "MakeRemapping.puma" */
    yyV1 = CombineACF (yyV1, yyV3);
-# line 142 "MakeRemapping.puma"
+/* line 148 "MakeRemapping.puma" */
    yyV2 = CombineACF (yyV4, yyV2);
   }
   }
@@ -238,26 +261,22 @@ static void RemappingArguments
  }
 yyL1:;
 
-# line 145 "MakeRemapping.puma"
+/* line 151 "MakeRemapping.puma" */
  {
-  bool found;
+  rbool found;
   var_descriptor home;
   {
-# line 149 "MakeRemapping.puma"
+/* line 155 "MakeRemapping.puma" */
    if (! ((IsSerialCall (basic_stmt->ASSIGN_STMT.ASSIGN_EXP)))) goto yyL2;
   {
-# line 151 "MakeRemapping.puma"
-
-# line 152 "MakeRemapping.puma"
-
-# line 154 "MakeRemapping.puma"
+/* line 160 "MakeRemapping.puma" */
    FindSerialHome (basic_stmt, & home, & found);
-# line 155 "MakeRemapping.puma"
+/* line 161 "MakeRemapping.puma" */
    if (! ((found))) goto yyL2;
   {
-# line 157 "MakeRemapping.puma"
+/* line 163 "MakeRemapping.puma" */
    stmt_protocol ("serial function call on dedicated home (no remapping)");
-# line 158 "MakeRemapping.puma"
+/* line 164 "MakeRemapping.puma" */
    tree_protocol ("home of serial call : ", PrintableDescriptorVar (& home));
   }
   }
@@ -268,7 +287,7 @@ yyL1:;
  }
 yyL2:;
 
-# line 161 "MakeRemapping.puma"
+/* line 167 "MakeRemapping.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -276,20 +295,20 @@ yyL2:;
   tTree yyV4;
   tTree yyV5;
   {
-# line 165 "MakeRemapping.puma"
+/* line 171 "MakeRemapping.puma" */
    if (! ((IsSerialCall (basic_stmt->ASSIGN_STMT.ASSIGN_EXP)))) goto yyL3;
   {
-# line 167 "MakeRemapping.puma"
+/* line 173 "MakeRemapping.puma" */
    stmt_protocol ("serial call will be executed on HOST");
-# line 169 "MakeRemapping.puma"
+/* line 175 "MakeRemapping.puma" */
    MakeSerialArguments (basic_stmt->ASSIGN_STMT.ASSIGN_EXP->FUNC_CALL_EXP.FUNC_PARAMS, & yyV1, & yyV2);
-# line 170 "MakeRemapping.puma"
+/* line 176 "MakeRemapping.puma" */
    MakeSerialVar (basic_stmt->ASSIGN_STMT.ASSIGN_VAR, & yyV3, & yyV4, & yyV5);
-# line 172 "MakeRemapping.puma"
+/* line 178 "MakeRemapping.puma" */
  basic_stmt->ASSIGN_STMT.ASSIGN_VAR = yyV3; 
-# line 174 "MakeRemapping.puma"
+/* line 180 "MakeRemapping.puma" */
    yyV1 = CombineACF (yyV4, yyV1);
-# line 175 "MakeRemapping.puma"
+/* line 181 "MakeRemapping.puma" */
    yyV2 = CombineACF (yyV2, yyV5);
   }
   }
@@ -302,26 +321,22 @@ yyL3:;
   }
   }
   if (basic_stmt->Kind == kCALL_STMT) {
-# line 178 "MakeRemapping.puma"
+/* line 184 "MakeRemapping.puma" */
  {
-  bool found;
+  rbool found;
   var_descriptor home;
   {
-# line 180 "MakeRemapping.puma"
+/* line 186 "MakeRemapping.puma" */
    if (! ((IsSerialCall (basic_stmt)))) goto yyL4;
   {
-# line 182 "MakeRemapping.puma"
-
-# line 183 "MakeRemapping.puma"
-
-# line 185 "MakeRemapping.puma"
+/* line 191 "MakeRemapping.puma" */
    FindSerialHome (basic_stmt, & home, & found);
-# line 186 "MakeRemapping.puma"
+/* line 192 "MakeRemapping.puma" */
    if (! ((found))) goto yyL4;
   {
-# line 188 "MakeRemapping.puma"
+/* line 194 "MakeRemapping.puma" */
    stmt_protocol ("serial subroutine call on dedicated home (no remapping)");
-# line 189 "MakeRemapping.puma"
+/* line 195 "MakeRemapping.puma" */
    tree_protocol ("home of serial call : ", PrintableDescriptorVar (& home));
   }
   }
@@ -332,17 +347,17 @@ yyL3:;
  }
 yyL4:;
 
-# line 192 "MakeRemapping.puma"
+/* line 198 "MakeRemapping.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 194 "MakeRemapping.puma"
+/* line 200 "MakeRemapping.puma" */
    if (! ((IsSerialCall (basic_stmt)))) goto yyL5;
   {
-# line 196 "MakeRemapping.puma"
+/* line 202 "MakeRemapping.puma" */
    stmt_protocol ("serial subroutine call will be executed on HOST");
-# line 198 "MakeRemapping.puma"
+/* line 204 "MakeRemapping.puma" */
    MakeSerialArguments (basic_stmt->CALL_STMT.CALL_PARAMS, & yyV1, & yyV2);
   }
   }
@@ -352,12 +367,12 @@ yyL4:;
  }
 yyL5:;
 
-# line 201 "MakeRemapping.puma"
+/* line 207 "MakeRemapping.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 203 "MakeRemapping.puma"
+/* line 209 "MakeRemapping.puma" */
    ParamListRemapping (basic_stmt->CALL_STMT.CALL_PARAMS, basic_stmt, & yyV1, & yyV2);
   }
    * yyP2 = yyV1;
@@ -366,7 +381,7 @@ yyL5:;
  }
 
   }
-# line 206 "MakeRemapping.puma"
+/* line 212 "MakeRemapping.puma" */
    * yyP2 = NoTree;
    * yyP1 = NoTree;
    return;
@@ -389,15 +404,15 @@ static void IntrArgRemapping
   if (actuals->Kind == kBTP_LIST) {
   if (actuals->BTP_LIST.Elem->Kind == kVAR_PARAM) {
   if (actuals->BTP_LIST.Next->Kind == kBTP_LIST) {
-# line 227 "MakeRemapping.puma"
+/* line 233 "MakeRemapping.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 230 "MakeRemapping.puma"
+/* line 236 "MakeRemapping.puma" */
    if (! ((IntrFuncScatter (name)))) goto yyL1;
   {
-# line 234 "MakeRemapping.puma"
+/* line 240 "MakeRemapping.puma" */
    AlignParamList (actuals->BTP_LIST.Elem->VAR_PARAM.V, actuals->BTP_LIST.Next->BTP_LIST.Next, & yyV1, & yyV2);
   }
   }
@@ -410,7 +425,7 @@ yyL1:;
   }
   }
   }
-# line 237 "MakeRemapping.puma"
+/* line 243 "MakeRemapping.puma" */
    * yyP4 = NoTree;
    * yyP3 = NoTree;
    return;
@@ -430,20 +445,20 @@ static void AlignParamList
 # endif
 {
   if (params->Kind == kBTP_LIST) {
-# line 249 "MakeRemapping.puma"
+/* line 255 "MakeRemapping.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   tTree yyV4;
   {
-# line 251 "MakeRemapping.puma"
+/* line 257 "MakeRemapping.puma" */
    AlignParam (target, params->BTP_LIST.Elem, & yyV1, & yyV2);
-# line 252 "MakeRemapping.puma"
+/* line 258 "MakeRemapping.puma" */
    AlignParamList (target, params->BTP_LIST.Next, & yyV3, & yyV4);
-# line 254 "MakeRemapping.puma"
+/* line 260 "MakeRemapping.puma" */
    yyV1 = CombineACF (yyV1, yyV3);
-# line 255 "MakeRemapping.puma"
+/* line 261 "MakeRemapping.puma" */
    yyV2 = CombineACF (yyV4, yyV2);
   }
    * yyP6 = yyV1;
@@ -453,15 +468,15 @@ static void AlignParamList
 
   }
   if (params->Kind == kBTP_EMPTY) {
-# line 258 "MakeRemapping.puma"
+/* line 264 "MakeRemapping.puma" */
    * yyP6 = NoTree;
    * yyP5 = NoTree;
    return;
 
   }
-# line 261 "MakeRemapping.puma"
+/* line 267 "MakeRemapping.puma" */
   {
-# line 263 "MakeRemapping.puma"
+/* line 269 "MakeRemapping.puma" */
    failure_protocol (MODULE, "AlignParamList", params);
   }
    * yyP6 = NoTree;
@@ -483,12 +498,12 @@ static void AlignParam
 # endif
 {
   if (param->Kind == kVAR_PARAM) {
-# line 268 "MakeRemapping.puma"
+/* line 274 "MakeRemapping.puma" */
   {
-# line 272 "MakeRemapping.puma"
+/* line 278 "MakeRemapping.puma" */
    if (! ((TreeRank (param->VAR_PARAM.V) == 0))) goto yyL1;
   {
-# line 273 "MakeRemapping.puma"
+/* line 279 "MakeRemapping.puma" */
    if (! ((TreeReadDistribution (param) == 0))) goto yyL1;
   }
   }
@@ -497,29 +512,23 @@ static void AlignParam
    return;
 yyL1:;
 
-# line 276 "MakeRemapping.puma"
+/* line 282 "MakeRemapping.puma" */
  {
   var_descriptor vd1;
   var_descriptor vd2;
-  bool found;
+  rbool found;
   {
-# line 278 "MakeRemapping.puma"
-
-# line 279 "MakeRemapping.puma"
-
-# line 281 "MakeRemapping.puma"
-
-# line 283 "MakeRemapping.puma"
+/* line 289 "MakeRemapping.puma" */
    GetVarDescriptor (target, & found, & vd1);
-# line 284 "MakeRemapping.puma"
+/* line 290 "MakeRemapping.puma" */
    if (! ((found))) goto yyL2;
   {
-# line 285 "MakeRemapping.puma"
+/* line 291 "MakeRemapping.puma" */
    GetVarDescriptor (param->VAR_PARAM.V, & found, & vd2);
-# line 286 "MakeRemapping.puma"
+/* line 292 "MakeRemapping.puma" */
    if (! ((found))) goto yyL2;
   {
-# line 288 "MakeRemapping.puma"
+/* line 294 "MakeRemapping.puma" */
  if (!VDIsAligned (&vd1, &vd2))
        { error_protocol ("SCATTER requires aligned parameters");
          tree_protocol ("not aligned argument is : ", param->VAR_PARAM.V);
@@ -537,17 +546,17 @@ yyL2:;
 
   }
   if (param->Kind == kNO_PARAM) {
-# line 296 "MakeRemapping.puma"
+/* line 302 "MakeRemapping.puma" */
    * yyP8 = NoTree;
    * yyP7 = NoTree;
    return;
 
   }
-# line 301 "MakeRemapping.puma"
+/* line 307 "MakeRemapping.puma" */
   {
-# line 303 "MakeRemapping.puma"
+/* line 309 "MakeRemapping.puma" */
    error_protocol ("could not verify alignment in SCATTER");
-# line 304 "MakeRemapping.puma"
+/* line 310 "MakeRemapping.puma" */
    tree_protocol ("not verified argument is : ", param);
   }
    * yyP8 = NoTree;
@@ -569,20 +578,20 @@ static void ParamListRemapping
 # endif
 {
   if (actuals->Kind == kBTP_LIST) {
-# line 314 "MakeRemapping.puma"
+/* line 320 "MakeRemapping.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   tTree yyV4;
   {
-# line 316 "MakeRemapping.puma"
+/* line 322 "MakeRemapping.puma" */
    ParamRemapping (actuals->BTP_LIST.Elem, call, & yyV1, & yyV2);
-# line 317 "MakeRemapping.puma"
+/* line 323 "MakeRemapping.puma" */
    ParamListRemapping (actuals->BTP_LIST.Next, call, & yyV3, & yyV4);
-# line 319 "MakeRemapping.puma"
+/* line 325 "MakeRemapping.puma" */
    yyV1 = CombineACF (yyV1, yyV3);
-# line 320 "MakeRemapping.puma"
+/* line 326 "MakeRemapping.puma" */
    yyV2 = CombineACF (yyV4, yyV2);
   }
    * yyP10 = yyV1;
@@ -592,15 +601,15 @@ static void ParamListRemapping
 
   }
   if (actuals->Kind == kBTP_EMPTY) {
-# line 323 "MakeRemapping.puma"
+/* line 329 "MakeRemapping.puma" */
    * yyP10 = NoTree;
    * yyP9 = NoTree;
    return;
 
   }
-# line 326 "MakeRemapping.puma"
+/* line 332 "MakeRemapping.puma" */
   {
-# line 328 "MakeRemapping.puma"
+/* line 334 "MakeRemapping.puma" */
    failure_protocol (MODULE, "ParamListRempapping", actuals);
   }
    * yyP10 = NoTree;
@@ -622,35 +631,31 @@ static void ParamRemapping
 # endif
 {
   if (actual->Kind == kVAR_PARAM) {
-# line 333 "MakeRemapping.puma"
+/* line 339 "MakeRemapping.puma" */
  {
-  bool found;
+  rbool found;
   var_descriptor vard;
   tTree yyV1;
   tTree yyV2;
   {
-# line 335 "MakeRemapping.puma"
+/* line 341 "MakeRemapping.puma" */
    if (! ((actual->VAR_PARAM.formal != NoObject))) goto yyL1;
   {
-# line 336 "MakeRemapping.puma"
+/* line 342 "MakeRemapping.puma" */
    if (! ((call != NoTree))) goto yyL1;
   {
-# line 338 "MakeRemapping.puma"
-   if (! ((GetCurrentModel () == HPF_GLOBAL))) goto yyL1;
+/* line 344 "MakeRemapping.puma" */
+   if (! ((IsNonLocalModel (GetCurrentModel ())))) goto yyL1;
   {
-# line 340 "MakeRemapping.puma"
+/* line 346 "MakeRemapping.puma" */
    if (! ((IsLocalCall (call)))) goto yyL1;
   {
-# line 342 "MakeRemapping.puma"
+/* line 348 "MakeRemapping.puma" */
    if (! ((MightBeRedistributed (call, actual, actual->VAR_PARAM.formal)))) goto yyL1;
   {
-# line 344 "MakeRemapping.puma"
-
-# line 345 "MakeRemapping.puma"
-
-# line 347 "MakeRemapping.puma"
+/* line 353 "MakeRemapping.puma" */
    GetDummyVarDescriptor (actual, actual->VAR_PARAM.formal, call, & found, & vard);
-# line 349 "MakeRemapping.puma"
+/* line 355 "MakeRemapping.puma" */
  if (!found)
 
        { error_protocol ("could not create dummy descriptor");
@@ -667,10 +672,10 @@ static void ParamRemapping
        }
 
    
-# line 366 "MakeRemapping.puma"
+/* line 372 "MakeRemapping.puma" */
    if (! ((found))) goto yyL1;
   {
-# line 368 "MakeRemapping.puma"
+/* line 374 "MakeRemapping.puma" */
    MakeParamTemporary (actual, & vard, & yyV1, & yyV2);
   }
   }
@@ -685,15 +690,15 @@ static void ParamRemapping
  }
 yyL1:;
 
-# line 371 "MakeRemapping.puma"
+/* line 377 "MakeRemapping.puma" */
   {
-# line 373 "MakeRemapping.puma"
+/* line 379 "MakeRemapping.puma" */
    if (! ((actual->VAR_PARAM.formal != NoObject))) goto yyL2;
   {
-# line 374 "MakeRemapping.puma"
+/* line 380 "MakeRemapping.puma" */
    if (! ((call != NoTree))) goto yyL2;
   {
-# line 378 "MakeRemapping.puma"
+/* line 384 "MakeRemapping.puma" */
  
 
      if (MightBeRedistributed (call, actual, actual->VAR_PARAM.formal))
@@ -714,7 +719,7 @@ yyL1:;
 yyL2:;
 
   }
-# line 392 "MakeRemapping.puma"
+/* line 398 "MakeRemapping.puma" */
    * yyP12 = NoTree;
    * yyP11 = NoTree;
    return;
@@ -733,15 +738,15 @@ static void IntrResRemapping
  register tTree * yyP13;
 # endif
 {
-# line 404 "MakeRemapping.puma"
+/* line 410 "MakeRemapping.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 406 "MakeRemapping.puma"
+/* line 412 "MakeRemapping.puma" */
    if (! ((IntrFuncRed (name) || IntrFuncLocRed (name)))) goto yyL1;
   {
-# line 408 "MakeRemapping.puma"
+/* line 414 "MakeRemapping.puma" */
    NewResultTmp (assign, & yyV1, & yyV2);
   }
   }
@@ -751,7 +756,7 @@ static void IntrResRemapping
  }
 yyL1:;
 
-# line 411 "MakeRemapping.puma"
+/* line 417 "MakeRemapping.puma" */
    * yyP14 = NoTree;
    * yyP13 = NoTree;
    return;
@@ -771,29 +776,23 @@ static void NewResultTmp
 {
   if (assign->Kind == kASSIGN_STMT) {
   if (assign->ASSIGN_STMT.ASSIGN_EXP->Kind == kFUNC_CALL_EXP) {
-# line 422 "MakeRemapping.puma"
+/* line 428 "MakeRemapping.puma" */
  {
-  bool found;
+  rbool found;
   var_descriptor vard1;
   var_descriptor vard2;
   {
-# line 426 "MakeRemapping.puma"
-
-# line 427 "MakeRemapping.puma"
-
-# line 428 "MakeRemapping.puma"
-
-# line 430 "MakeRemapping.puma"
+/* line 436 "MakeRemapping.puma" */
    GetExpDescriptor (assign->ASSIGN_STMT.ASSIGN_EXP, & found, & vard2);
-# line 431 "MakeRemapping.puma"
+/* line 437 "MakeRemapping.puma" */
    if (! ((found))) goto yyL1;
   {
-# line 433 "MakeRemapping.puma"
+/* line 439 "MakeRemapping.puma" */
    GetVarDescriptor (assign->ASSIGN_STMT.ASSIGN_VAR, & found, & vard1);
-# line 434 "MakeRemapping.puma"
+/* line 440 "MakeRemapping.puma" */
    if (! ((found))) goto yyL1;
   {
-# line 436 "MakeRemapping.puma"
+/* line 442 "MakeRemapping.puma" */
 
 #ifdef DEBUG
     if (!VDIsLocalAssignment (&vard1, &vard2))
@@ -810,10 +809,10 @@ static void NewResultTmp
        }
 #endif
    
-# line 453 "MakeRemapping.puma"
+/* line 459 "MakeRemapping.puma" */
    if (! ((VDIsLocalAssignment (& vard1, & vard2)))) goto yyL1;
   {
-# line 454 "MakeRemapping.puma"
+/* line 460 "MakeRemapping.puma" */
    if (! ((VDIsLocalAssignment (& vard2, & vard1)))) goto yyL1;
   }
   }
@@ -825,26 +824,22 @@ static void NewResultTmp
  }
 yyL1:;
 
-# line 457 "MakeRemapping.puma"
+/* line 463 "MakeRemapping.puma" */
  {
-  bool found;
+  rbool found;
   var_descriptor vard;
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 459 "MakeRemapping.puma"
-
-# line 460 "MakeRemapping.puma"
-
-# line 462 "MakeRemapping.puma"
+/* line 468 "MakeRemapping.puma" */
    GetExpDescriptor (assign->ASSIGN_STMT.ASSIGN_EXP, & found, & vard);
-# line 463 "MakeRemapping.puma"
+/* line 469 "MakeRemapping.puma" */
    if (! ((found))) goto yyL2;
   {
-# line 465 "MakeRemapping.puma"
+/* line 471 "MakeRemapping.puma" */
    MakeNewAssignment (assign->ASSIGN_STMT.ASSIGN_EXP, & vard, & yyV1, & yyV2, & yyV3);
-# line 466 "MakeRemapping.puma"
+/* line 472 "MakeRemapping.puma" */
  assign->ASSIGN_STMT.ASSIGN_EXP = yyV1; 
   }
   }
@@ -854,11 +849,11 @@ yyL1:;
  }
 yyL2:;
 
-# line 469 "MakeRemapping.puma"
+/* line 475 "MakeRemapping.puma" */
   {
-# line 471 "MakeRemapping.puma"
+/* line 477 "MakeRemapping.puma" */
    error_protocol ("no descriptor for required result remapping found");
-# line 472 "MakeRemapping.puma"
+/* line 478 "MakeRemapping.puma" */
    tree_protocol ("call is : ", assign->ASSIGN_STMT.ASSIGN_EXP);
   }
    * yyP16 = NoTree;
@@ -881,20 +876,20 @@ static void MakeSerialArguments
 # endif
 {
   if (actuals->Kind == kBTP_LIST) {
-# line 483 "MakeRemapping.puma"
+/* line 489 "MakeRemapping.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   tTree yyV4;
   {
-# line 485 "MakeRemapping.puma"
+/* line 491 "MakeRemapping.puma" */
    MakeSerialArgument (actuals->BTP_LIST.Elem, & yyV1, & yyV2);
-# line 486 "MakeRemapping.puma"
+/* line 492 "MakeRemapping.puma" */
    MakeSerialArguments (actuals->BTP_LIST.Next, & yyV3, & yyV4);
-# line 488 "MakeRemapping.puma"
+/* line 494 "MakeRemapping.puma" */
    yyV1 = CombineACF (yyV1, yyV3);
-# line 489 "MakeRemapping.puma"
+/* line 495 "MakeRemapping.puma" */
    yyV2 = CombineACF (yyV4, yyV2);
   }
    * yyP18 = yyV1;
@@ -904,7 +899,7 @@ static void MakeSerialArguments
 
   }
   if (actuals->Kind == kBTP_EMPTY) {
-# line 492 "MakeRemapping.puma"
+/* line 498 "MakeRemapping.puma" */
    * yyP18 = NoTree;
    * yyP17 = NoTree;
    return;
@@ -924,9 +919,9 @@ static void MakeSerialArgument
 # endif
 {
   if (actual->Kind == kVAR_PARAM) {
-# line 497 "MakeRemapping.puma"
+/* line 503 "MakeRemapping.puma" */
   {
-# line 499 "MakeRemapping.puma"
+/* line 505 "MakeRemapping.puma" */
    if (! ((AcceptedSerialVar (actual->VAR_PARAM.V)))) goto yyL1;
   }
    * yyP20 = NoTree;
@@ -936,41 +931,41 @@ yyL1:;
 
   }
   if (actual->Kind == kNO_PARAM) {
-# line 502 "MakeRemapping.puma"
+/* line 508 "MakeRemapping.puma" */
    * yyP20 = NoTree;
    * yyP19 = NoTree;
    return;
 
   }
   if (actual->Kind == kFUNC_PARAM) {
-# line 505 "MakeRemapping.puma"
+/* line 511 "MakeRemapping.puma" */
    * yyP20 = NoTree;
    * yyP19 = NoTree;
    return;
 
   }
   if (actual->Kind == kPROC_PARAM) {
-# line 508 "MakeRemapping.puma"
+/* line 514 "MakeRemapping.puma" */
    * yyP20 = NoTree;
    * yyP19 = NoTree;
    return;
 
   }
-# line 511 "MakeRemapping.puma"
+/* line 517 "MakeRemapping.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   var_descriptor yyV2;
   tTree yyV3;
   tTree yyV4;
   {
-# line 513 "MakeRemapping.puma"
+/* line 519 "MakeRemapping.puma" */
    FindParamDescriptor (actual, & yyV1, & yyV2);
-# line 514 "MakeRemapping.puma"
+/* line 520 "MakeRemapping.puma" */
    if (! ((yyV1))) goto yyL5;
   {
-# line 516 "MakeRemapping.puma"
+/* line 522 "MakeRemapping.puma" */
  yyV2.topology_rank = -1;    
-# line 518 "MakeRemapping.puma"
+/* line 524 "MakeRemapping.puma" */
    MakeParamTemporary (actual, & yyV2, & yyV3, & yyV4);
   }
   }
@@ -980,11 +975,11 @@ yyL1:;
  }
 yyL5:;
 
-# line 521 "MakeRemapping.puma"
+/* line 527 "MakeRemapping.puma" */
   {
-# line 523 "MakeRemapping.puma"
+/* line 529 "MakeRemapping.puma" */
    error_protocol ("could not serialize actual argument");
-# line 524 "MakeRemapping.puma"
+/* line 530 "MakeRemapping.puma" */
    tree_protocol ("actual argument : ", actual);
   }
    * yyP20 = NoTree;
@@ -1005,9 +1000,9 @@ static void MakeSerialVar
  register tTree * yyP21;
 # endif
 {
-# line 529 "MakeRemapping.puma"
+/* line 535 "MakeRemapping.puma" */
   {
-# line 531 "MakeRemapping.puma"
+/* line 537 "MakeRemapping.puma" */
    if (! ((AcceptedSerialVar (var)))) goto yyL1;
   }
    * yyP23 = var;
@@ -1016,29 +1011,21 @@ static void MakeSerialVar
    return;
 yyL1:;
 
-# line 534 "MakeRemapping.puma"
+/* line 540 "MakeRemapping.puma" */
  {
   var_descriptor vard;
   tTree pre_stmts;
   tTree post_stmts;
   tTree new_var;
   {
-# line 536 "MakeRemapping.puma"
-
-# line 538 "MakeRemapping.puma"
+/* line 544 "MakeRemapping.puma" */
    SetVarDescriptor (var, & vard);
-# line 540 "MakeRemapping.puma"
+/* line 546 "MakeRemapping.puma" */
  vard.topology_rank = -1;    
-# line 542 "MakeRemapping.puma"
-
-# line 543 "MakeRemapping.puma"
-
-# line 545 "MakeRemapping.puma"
-
-# line 547 "MakeRemapping.puma"
-   NoExpansion (vard);
-# line 548 "MakeRemapping.puma"
-   MakeTempVarStmt (var, IntentOut, vard, & new_var, & pre_stmts, & post_stmts);
+/* line 553 "MakeRemapping.puma" */
+   NoExpansion (& vard);
+/* line 554 "MakeRemapping.puma" */
+   MakeTempVarStmt (var, IntentOut, & vard, & new_var, & pre_stmts, & post_stmts);
   }
    * yyP23 = new_var;
    * yyP22 = pre_stmts;
@@ -1049,7 +1036,7 @@ yyL1:;
 ;
 }
 
-static bool AcceptedSerialVar
+static rbool AcceptedSerialVar
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -1057,42 +1044,40 @@ static bool AcceptedSerialVar
  register tTree var;
 # endif
 {
-# line 559 "MakeRemapping.puma"
+/* line 565 "MakeRemapping.puma" */
  {
   int dist;
   {
-# line 561 "MakeRemapping.puma"
+/* line 567 "MakeRemapping.puma" */
    if (! ((TreeRank (var) > 0))) goto yyL1;
   {
-# line 563 "MakeRemapping.puma"
-
-# line 565 "MakeRemapping.puma"
+/* line 571 "MakeRemapping.puma" */
    dist = TreeWriteDistribution (var);
-# line 567 "MakeRemapping.puma"
+/* line 573 "MakeRemapping.puma" */
    if (! (((dist == 0) || (dist == - 1) || (dist == 2)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-# line 570 "MakeRemapping.puma"
+/* line 576 "MakeRemapping.puma" */
   {
-# line 572 "MakeRemapping.puma"
+/* line 578 "MakeRemapping.puma" */
    if (! ((TreeRank (var) > 0))) goto yyL2;
   {
-# line 573 "MakeRemapping.puma"
-   return false;
+/* line 579 "MakeRemapping.puma" */
+   return rfalse;
   }
   }
 yyL2:;
 
-# line 576 "MakeRemapping.puma"
-   return true;
+/* line 582 "MakeRemapping.puma" */
+   return rtrue;
 
 }
 
-static bool MightBeRedistributed
+static rbool MightBeRedistributed
 # if defined __STDC__ | defined __cplusplus
 (register tTree p, register tTree actual, register tDefinitions dummy)
 # else
@@ -1102,21 +1087,17 @@ static bool MightBeRedistributed
  register tDefinitions dummy;
 # endif
 {
-# line 589 "MakeRemapping.puma"
+/* line 595 "MakeRemapping.puma" */
  {
-  bool might_be;
+  rbool might_be;
   var_descriptor dummy_vard;
-  bool yyV1;
+  rbool yyV1;
   var_descriptor yyV2;
   {
-# line 591 "MakeRemapping.puma"
-
-# line 592 "MakeRemapping.puma"
-
-# line 594 "MakeRemapping.puma"
+/* line 600 "MakeRemapping.puma" */
    FindParamDescriptor (actual, & yyV1, & yyV2);
-# line 596 "MakeRemapping.puma"
- might_be = true;
+/* line 602 "MakeRemapping.puma" */
+ might_be = rtrue;
 
      if (yyV1)
  
@@ -1128,38 +1109,34 @@ static bool MightBeRedistributed
 
        }
    
-# line 609 "MakeRemapping.puma"
+/* line 615 "MakeRemapping.puma" */
    if (! ((might_be))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
 static void FindParamDescriptor
 # if defined __STDC__ | defined __cplusplus
-(register tTree actual, register bool * yyP25, var_descriptor * yyP24)
+(register tTree actual, register rbool * yyP25, var_descriptor * yyP24)
 # else
 (actual, yyP25, yyP24)
  register tTree actual;
- register bool * yyP25;
+ register rbool * yyP25;
  var_descriptor * yyP24;
 # endif
 {
-# line 620 "MakeRemapping.puma"
+/* line 626 "MakeRemapping.puma" */
  {
-  bool found;
+  rbool found;
   var_descriptor vard;
   {
-# line 622 "MakeRemapping.puma"
-
-# line 623 "MakeRemapping.puma"
-
-# line 625 "MakeRemapping.puma"
+/* line 631 "MakeRemapping.puma" */
    GetParamDescriptor (actual, & found, & vard);
-# line 626 "MakeRemapping.puma"
+/* line 632 "MakeRemapping.puma" */
    if (! ((found))) goto yyL1;
   }
    * yyP25 = found;
@@ -1168,14 +1145,12 @@ static void FindParamDescriptor
  }
 yyL1:;
 
-# line 629 "MakeRemapping.puma"
+/* line 635 "MakeRemapping.puma" */
  {
   var_descriptor vard;
   {
-# line 631 "MakeRemapping.puma"
-
   }
-   * yyP25 = false;
+   * yyP25 = rfalse;
    * yyP24 = vard;
    return;
  }
@@ -1196,26 +1171,18 @@ static void MakeParamTemporary
 {
   if (actual->Kind == kVAR_PARAM) {
   if (actual->VAR_PARAM.V->Kind == kADDR) {
-# line 642 "MakeRemapping.puma"
+/* line 648 "MakeRemapping.puma" */
  {
   tTree pre_stmts;
   tTree post_stmts;
   tTree assign;
   tTree tmp_var;
   {
-# line 647 "MakeRemapping.puma"
-
-# line 648 "MakeRemapping.puma"
-
-# line 649 "MakeRemapping.puma"
-
-# line 650 "MakeRemapping.puma"
-
-# line 652 "MakeRemapping.puma"
+/* line 658 "MakeRemapping.puma" */
    NoExpansion (vard);
-# line 653 "MakeRemapping.puma"
+/* line 659 "MakeRemapping.puma" */
    MakeTemporaryStmt (actual->VAR_PARAM.V->ADDR.E, vard, & pre_stmts, & tmp_var, & assign, & post_stmts);
-# line 655 "MakeRemapping.puma"
+/* line 661 "MakeRemapping.puma" */
  actual->VAR_PARAM.V = tmp_var; 
   }
    * yyP27 = CombineACF (pre_stmts, assign);
@@ -1224,23 +1191,17 @@ static void MakeParamTemporary
  }
 
   }
-# line 658 "MakeRemapping.puma"
+/* line 664 "MakeRemapping.puma" */
  {
   tTree pre_stmts;
   tTree post_stmts;
   tTree new_var;
   {
-# line 660 "MakeRemapping.puma"
-
-# line 661 "MakeRemapping.puma"
-
-# line 663 "MakeRemapping.puma"
-
-# line 665 "MakeRemapping.puma"
+/* line 671 "MakeRemapping.puma" */
    NoExpansion (vard);
-# line 666 "MakeRemapping.puma"
+/* line 672 "MakeRemapping.puma" */
    MakeTempVarStmt (actual->VAR_PARAM.V, actual->VAR_PARAM.intent, vard, & new_var, & pre_stmts, & post_stmts);
-# line 668 "MakeRemapping.puma"
+/* line 674 "MakeRemapping.puma" */
  actual->VAR_PARAM.V = new_var; 
   }
    * yyP27 = pre_stmts;
@@ -1249,9 +1210,9 @@ static void MakeParamTemporary
  }
 
   }
-# line 671 "MakeRemapping.puma"
+/* line 677 "MakeRemapping.puma" */
   {
-# line 673 "MakeRemapping.puma"
+/* line 679 "MakeRemapping.puma" */
    failure_protocol (MODULE, "MakeParamTemporary", actual);
   }
    * yyP27 = NoTree;
@@ -1273,24 +1234,16 @@ static void MakeNewAssignment
  register tTree * yyP28;
 # endif
 {
-# line 684 "MakeRemapping.puma"
+/* line 690 "MakeRemapping.puma" */
  {
   tTree pre_stmt;
   tTree post_stmt;
   tTree tmp_var;
   tTree assign;
   {
-# line 686 "MakeRemapping.puma"
-
-# line 687 "MakeRemapping.puma"
-
-# line 688 "MakeRemapping.puma"
-
-# line 690 "MakeRemapping.puma"
-
-# line 692 "MakeRemapping.puma"
+/* line 698 "MakeRemapping.puma" */
    NoExpansion (vard);
-# line 694 "MakeRemapping.puma"
+/* line 700 "MakeRemapping.puma" */
    MakeTemporaryStmt (exp, vard, & pre_stmt, & tmp_var, & assign, & post_stmt);
   }
    * yyP30 = mVAR_EXP (tmp_var);
@@ -1302,10 +1255,10 @@ static void MakeNewAssignment
 ;
 }
 
-void BeginMakeRemapping ()
+void BeginMakeRemapping ARGS ((void))
 {
 }
 
-void CloseMakeRemapping ()
+void CloseMakeRemapping ARGS ((void))
 {
 }
