@@ -1,6 +1,13 @@
 # include "MakeSerial.h"
-# include "yyMakeSerial.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,32 +15,29 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 42 "MakeSerial.puma"
+/* line 42 "MakeSerial.puma" */
 
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
@@ -50,15 +54,34 @@
 # include "Nesting.h"          /* GetOuterLoop */
 # include "Accepted.h"
 
+# include "NewStatements.h"
+# include "TempAssigns.h"
+# include "Rank.h"
+# include "Expansion.h"
+
 # define MODULE "MakeSerial"
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* MakeSerial_Exit) () = yyExit;
+# include "yyMakeSerial.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* MakeSerial_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -67,28 +90,29 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module MakeSerial, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module MakeSerial, routine %s failed\n",
+  yyFunction);
  MakeSerial_Exit ();
 }
 
-void MakeSerial ARGS((tTree t));
-static bool StopMakeSerial ARGS((tTree t));
-static tTree DoMakeSerial ARGS((tTree t));
-static bool NeedsSerialization ARGS((tTree stmt));
-static bool UpdatesExpression ARGS((tTree stmts, tTree exp));
-static void NewWhereMask ARGS((tTree t, tTree * yyP2, tTree * yyP1));
-static bool NewWhereMaskRequired ARGS((tTree t));
-static void MakeNewWhereMask ARGS((tTree t, tTree * yyP4, tTree * yyP3));
-static void NewIfMask ARGS((tTree t, tTree * yyP6, tTree * yyP5));
-static bool NewIfMaskRequired ARGS((tTree t));
-static void MakeNewIfMask ARGS((tTree t, tTree * yyP8, tTree * yyP7));
-static void NewForallBounds ARGS((tTree t, tTree * yyP10, tTree * yyP9));
-static bool NewForallBoundsRequired ARGS((tTree t));
-static tTree SplitWhere ARGS((tTree where_stmt));
-static tTree SplitIf ARGS((tTree if_stmt));
-static void SplitAssign ARGS((tTree s, tTree * yyP12, tTree * yyP11));
-static void MyMakeTempAssignment ARGS((tTree exp, pvar vard, tTree * yyP15, tTree * yyP14, tTree * yyP13));
-static bool HasOuterIndependentLoop ARGS(());
+void MakeSerial ARGS ((tTree t));
+static rbool StopMakeSerial ARGS ((tTree t));
+static tTree DoMakeSerial ARGS ((tTree t));
+static rbool NeedsSerialization ARGS ((tTree stmt));
+static rbool UpdatesExpression ARGS ((tTree stmts, tTree exp));
+static void NewWhereMask ARGS ((tTree t, tTree * yyP2, tTree * yyP1));
+static rbool NewWhereMaskRequired ARGS ((tTree t));
+static void MakeNewWhereMask ARGS ((tTree t, tTree * yyP4, tTree * yyP3));
+static void NewIfMask ARGS ((tTree t, tTree * yyP6, tTree * yyP5));
+static rbool NewIfMaskRequired ARGS ((tTree t));
+static void MakeNewIfMask ARGS ((tTree t, tTree * yyP8, tTree * yyP7));
+static void NewForallBounds ARGS ((tTree t, tTree * yyP10, tTree * yyP9));
+static rbool NewForallBoundsRequired ARGS ((tTree t));
+static tTree SplitWhere ARGS ((tTree where_stmt));
+static tTree SplitIf ARGS ((tTree if_stmt));
+static void SplitAssign ARGS ((tTree s, tTree * yyP12, tTree * yyP11));
+static void MyMakeTempAssignment ARGS ((tTree exp, pvar vard, tTree * yyP15, tTree * yyP14, tTree * yyP13));
+static rbool HasOuterIndependentLoop ARGS ((void));
 
 void MakeSerial
 # if defined __STDC__ | defined __cplusplus
@@ -99,17 +123,17 @@ void MakeSerial
 # endif
 {
   if (t->Kind == kBODY_NODE) {
-# line 75 "MakeSerial.puma"
+/* line 80 "MakeSerial.puma" */
   {
-# line 77 "MakeSerial.puma"
+/* line 82 "MakeSerial.puma" */
  t->BODY_NODE.STATS = ReplaceAST (t->BODY_NODE.STATS, StopMakeSerial, DoMakeSerial); 
   }
    return;
 
   }
-# line 80 "MakeSerial.puma"
+/* line 85 "MakeSerial.puma" */
   {
-# line 81 "MakeSerial.puma"
+/* line 86 "MakeSerial.puma" */
    failure_protocol (MODULE, "MakeSerial", t);
   }
    return;
@@ -117,7 +141,7 @@ void MakeSerial
 ;
 }
 
-static bool StopMakeSerial
+static rbool StopMakeSerial
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -125,10 +149,10 @@ static bool StopMakeSerial
  register tTree t;
 # endif
 {
-# line 98 "MakeSerial.puma"
+/* line 103 "MakeSerial.puma" */
   {
-# line 99 "MakeSerial.puma"
-   return false;
+/* line 104 "MakeSerial.puma" */
+   return rfalse;
   }
 
 }
@@ -143,125 +167,109 @@ static tTree DoMakeSerial
 {
   if (t->Kind == kACF_BASIC) {
   if (t->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
-# line 125 "MakeSerial.puma"
+/* line 130 "MakeSerial.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree new;
   {
-# line 127 "MakeSerial.puma"
+/* line 132 "MakeSerial.puma" */
    if (! ((NeedsSerialization (t)))) goto yyL1;
   {
-# line 129 "MakeSerial.puma"
+/* line 134 "MakeSerial.puma" */
    stmt_protocol ("array assignment will be serialized");
-# line 131 "MakeSerial.puma"
+/* line 136 "MakeSerial.puma" */
    SplitAssign (t->ACF_BASIC.BASIC_STMT, & yyV1, & yyV2);
-# line 133 "MakeSerial.puma"
-
-# line 135 "MakeSerial.puma"
+/* line 140 "MakeSerial.puma" */
    new = ComposeNewACF (yyV1, t, yyV2);
-# line 137 "MakeSerial.puma"
+/* line 142 "MakeSerial.puma" */
    tree_protocol ("after serialization : \n", new);
   }
   }
-  {
    return new;
-  }
  }
 yyL1:;
 
-# line 142 "MakeSerial.puma"
+/* line 147 "MakeSerial.puma" */
    return t;
 
   }
   }
   if (t->Kind == kACF_WHERE) {
-# line 157 "MakeSerial.puma"
+/* line 162 "MakeSerial.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree new;
   {
-# line 161 "MakeSerial.puma"
+/* line 166 "MakeSerial.puma" */
    NewWhereMask (t, & yyV1, & yyV2);
-# line 163 "MakeSerial.puma"
-
-# line 165 "MakeSerial.puma"
+/* line 170 "MakeSerial.puma" */
    new = CombineACF (yyV1, CombineACF (SplitWhere (t), yyV2));
-# line 167 "MakeSerial.puma"
+/* line 172 "MakeSerial.puma" */
    tree_protocol ("serialized WHERE statement: \n", new);
   }
-  {
    return new;
-  }
  }
 
   }
   if (t->Kind == kACF_IF) {
-# line 182 "MakeSerial.puma"
+/* line 187 "MakeSerial.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree new;
   {
-# line 184 "MakeSerial.puma"
+/* line 189 "MakeSerial.puma" */
    if (! ((OuterLoops () > 0))) goto yyL4;
   {
-# line 185 "MakeSerial.puma"
+/* line 191 "MakeSerial.puma" */
    if (! ((IsForallLoop (GetOuterLoop (OuterLoops ()))))) goto yyL4;
   {
-# line 189 "MakeSerial.puma"
+/* line 195 "MakeSerial.puma" */
    NewIfMask (t, & yyV1, & yyV2);
-# line 191 "MakeSerial.puma"
-
-# line 193 "MakeSerial.puma"
+/* line 199 "MakeSerial.puma" */
    new = CombineACF (yyV1, CombineACF (SplitIf (t), yyV2));
-# line 195 "MakeSerial.puma"
+/* line 201 "MakeSerial.puma" */
    tree_protocol ("serialized FORALL mask statement: \n", new);
   }
   }
   }
-  {
    return new;
-  }
  }
 yyL4:;
 
   }
   if (t->Kind == kACF_FORALL) {
-# line 223 "MakeSerial.puma"
+/* line 229 "MakeSerial.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree new;
   {
-# line 227 "MakeSerial.puma"
+/* line 233 "MakeSerial.puma" */
    NewForallBounds (t, & yyV1, & yyV2);
-# line 229 "MakeSerial.puma"
-
-# line 233 "MakeSerial.puma"
-   t->ACF_FORALL.FORALL_DEP_INFO = mINDEP_INFO (false, 0, NoTree, NoTree);
-# line 237 "MakeSerial.puma"
+/* line 239 "MakeSerial.puma" */
+   t->ACF_FORALL.FORALL_DEP_INFO = mINDEP_INFO (rfalse, 0, NoTree, NoTree);
+/* line 243 "MakeSerial.puma" */
    new = t->ACF_FORALL.FORALL_BODY;
-# line 238 "MakeSerial.puma"
+/* line 244 "MakeSerial.puma" */
    ExpandNewStatements (t->ACF_FORALL.FORALL_BODY, t, t->ACF_FORALL.Line);
-# line 239 "MakeSerial.puma"
+/* line 245 "MakeSerial.puma" */
    new = CombineACF (yyV1, CombineACF (new, yyV2));
-# line 241 "MakeSerial.puma"
+/* line 247 "MakeSerial.puma" */
    tree_protocol ("splitted forall : \n", new);
   }
-  {
    return new;
-  }
  }
 
   }
-# line 248 "MakeSerial.puma"
+/* line 254 "MakeSerial.puma" */
    return t;
 
 }
 
-static bool NeedsSerialization
+static rbool NeedsSerialization
 # if defined __STDC__ | defined __cplusplus
 (register tTree stmt)
 # else
@@ -271,20 +279,20 @@ static bool NeedsSerialization
 {
   if (stmt->Kind == kACF_BASIC) {
   if (stmt->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
-# line 267 "MakeSerial.puma"
+/* line 273 "MakeSerial.puma" */
   {
-# line 269 "MakeSerial.puma"
+/* line 275 "MakeSerial.puma" */
    if (! ((ForallVarExpDependences (stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR, stmt->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_EXP)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
-static bool UpdatesExpression
+static rbool UpdatesExpression
 # if defined __STDC__ | defined __cplusplus
 (register tTree stmts, register tTree exp)
 # else
@@ -293,51 +301,51 @@ static bool UpdatesExpression
  register tTree exp;
 # endif
 {
+ yyRecursion:
   if (stmts->Kind == kACF_LIST) {
-# line 282 "MakeSerial.puma"
+/* line 288 "MakeSerial.puma" */
   {
-# line 283 "MakeSerial.puma"
+/* line 289 "MakeSerial.puma" */
    if (! (UpdatesExpression (stmts->ACF_LIST.Elem, exp))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 286 "MakeSerial.puma"
+/* line 292 "MakeSerial.puma" */
   {
-# line 287 "MakeSerial.puma"
-   if (! (UpdatesExpression (stmts->ACF_LIST.Next, exp))) goto yyL2;
+/* line 293 "MakeSerial.puma" */
+   stmts = stmts->ACF_LIST.Next;
+   goto yyRecursion;
   }
-   return true;
-yyL2:;
 
   }
   if (stmts->Kind == kACF_BASIC) {
   if (stmts->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
-# line 290 "MakeSerial.puma"
+/* line 296 "MakeSerial.puma" */
   {
-# line 292 "MakeSerial.puma"
+/* line 298 "MakeSerial.puma" */
    if (! ((ForallVarExpDependences (stmts->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR, exp)))) goto yyL3;
   }
-   return true;
+   return rtrue;
 yyL3:;
 
   }
   }
   if (stmts->Kind == kACF_WHERE) {
-# line 295 "MakeSerial.puma"
+/* line 301 "MakeSerial.puma" */
   {
-# line 297 "MakeSerial.puma"
+/* line 303 "MakeSerial.puma" */
    if (! (UpdatesExpression (stmts->ACF_WHERE.TRUE_PART, exp))) goto yyL4;
   {
-# line 298 "MakeSerial.puma"
-   if (! (UpdatesExpression (stmts->ACF_WHERE.FALSE_PART, exp))) goto yyL4;
+/* line 304 "MakeSerial.puma" */
+   stmts = stmts->ACF_WHERE.FALSE_PART;
+   goto yyRecursion;
   }
   }
-   return true;
 yyL4:;
 
   }
-  return false;
+  return rfalse;
 }
 
 static void NewWhereMask
@@ -351,15 +359,15 @@ static void NewWhereMask
 # endif
 {
   if (t->Kind == kACF_WHERE) {
-# line 314 "MakeSerial.puma"
+/* line 320 "MakeSerial.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 316 "MakeSerial.puma"
+/* line 322 "MakeSerial.puma" */
    if (! ((NewWhereMaskRequired (t)))) goto yyL1;
   {
-# line 317 "MakeSerial.puma"
+/* line 323 "MakeSerial.puma" */
    MakeNewWhereMask (t, & yyV1, & yyV2);
   }
   }
@@ -369,7 +377,7 @@ static void NewWhereMask
  }
 yyL1:;
 
-# line 322 "MakeSerial.puma"
+/* line 328 "MakeSerial.puma" */
    * yyP2 = NoTree;
    * yyP1 = NoTree;
    return;
@@ -378,7 +386,7 @@ yyL1:;
 ;
 }
 
-static bool NewWhereMaskRequired
+static rbool NewWhereMaskRequired
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -387,40 +395,40 @@ static bool NewWhereMaskRequired
 # endif
 {
   if (t->Kind == kACF_WHERE) {
-# line 336 "MakeSerial.puma"
+/* line 342 "MakeSerial.puma" */
   {
-# line 338 "MakeSerial.puma"
+/* line 344 "MakeSerial.puma" */
    if (! ((UpdatesExpression (t->ACF_WHERE.TRUE_PART, t->ACF_WHERE.WHERE_EXP)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 341 "MakeSerial.puma"
+/* line 347 "MakeSerial.puma" */
   {
-# line 343 "MakeSerial.puma"
+/* line 349 "MakeSerial.puma" */
    if (! ((UpdatesExpression (t->ACF_WHERE.FALSE_PART, t->ACF_WHERE.WHERE_EXP)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   if (t->ACF_WHERE.WHERE_EXP->Kind == kVAR_EXP) {
-# line 346 "MakeSerial.puma"
+/* line 352 "MakeSerial.puma" */
   {
-# line 348 "MakeSerial.puma"
+/* line 354 "MakeSerial.puma" */
    if (! ((IsDescriptorVar (t->ACF_WHERE.WHERE_EXP->VAR_EXP.V)))) goto yyL3;
   {
-# line 349 "MakeSerial.puma"
-   return false;
+/* line 355 "MakeSerial.puma" */
+   return rfalse;
   }
   }
 yyL3:;
 
   }
-# line 352 "MakeSerial.puma"
-   return true;
+/* line 358 "MakeSerial.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
 static void MakeNewWhereMask
@@ -434,28 +442,24 @@ static void MakeNewWhereMask
 # endif
 {
   if (t->Kind == kACF_WHERE) {
-# line 363 "MakeSerial.puma"
+/* line 369 "MakeSerial.puma" */
  {
   var_descriptor vard;
-  bool found;
+  rbool found;
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 365 "MakeSerial.puma"
-
-# line 366 "MakeSerial.puma"
-
-# line 368 "MakeSerial.puma"
+/* line 374 "MakeSerial.puma" */
    GetExpDescriptor (t->ACF_WHERE.WHERE_EXP, & found, & vard);
-# line 369 "MakeSerial.puma"
+/* line 375 "MakeSerial.puma" */
    if (! ((found))) goto yyL1;
   {
-# line 371 "MakeSerial.puma"
+/* line 377 "MakeSerial.puma" */
    SetExpType (t->ACF_WHERE.WHERE_EXP, & vard);
-# line 373 "MakeSerial.puma"
+/* line 379 "MakeSerial.puma" */
    MyMakeTempAssignment (t->ACF_WHERE.WHERE_EXP, & vard, & yyV1, & yyV2, & yyV3);
-# line 375 "MakeSerial.puma"
+/* line 381 "MakeSerial.puma" */
  t->ACF_WHERE.WHERE_EXP = yyV1; 
   }
   }
@@ -466,9 +470,9 @@ static void MakeNewWhereMask
 yyL1:;
 
   }
-# line 378 "MakeSerial.puma"
+/* line 384 "MakeSerial.puma" */
   {
-# line 380 "MakeSerial.puma"
+/* line 386 "MakeSerial.puma" */
    error_protocol ("no var descriptor for where statement");
   }
    * yyP4 = NoTree;
@@ -489,15 +493,15 @@ static void NewIfMask
 # endif
 {
   if (t->Kind == kACF_IF) {
-# line 396 "MakeSerial.puma"
+/* line 402 "MakeSerial.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 398 "MakeSerial.puma"
+/* line 404 "MakeSerial.puma" */
    if (! ((NewIfMaskRequired (t)))) goto yyL1;
   {
-# line 399 "MakeSerial.puma"
+/* line 405 "MakeSerial.puma" */
    MakeNewIfMask (t, & yyV1, & yyV2);
   }
   }
@@ -507,7 +511,7 @@ static void NewIfMask
  }
 yyL1:;
 
-# line 404 "MakeSerial.puma"
+/* line 410 "MakeSerial.puma" */
    * yyP6 = NoTree;
    * yyP5 = NoTree;
    return;
@@ -516,7 +520,7 @@ yyL1:;
 ;
 }
 
-static bool NewIfMaskRequired
+static rbool NewIfMaskRequired
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -525,40 +529,40 @@ static bool NewIfMaskRequired
 # endif
 {
   if (t->Kind == kACF_IF) {
-# line 415 "MakeSerial.puma"
+/* line 421 "MakeSerial.puma" */
   {
-# line 417 "MakeSerial.puma"
+/* line 423 "MakeSerial.puma" */
    if (! ((UpdatesExpression (t->ACF_IF.THEN_PART, t->ACF_IF.IF_EXP)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 420 "MakeSerial.puma"
+/* line 426 "MakeSerial.puma" */
   {
-# line 422 "MakeSerial.puma"
+/* line 428 "MakeSerial.puma" */
    if (! ((UpdatesExpression (t->ACF_IF.ELSE_PART, t->ACF_IF.IF_EXP)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   if (t->ACF_IF.IF_EXP->Kind == kVAR_EXP) {
-# line 425 "MakeSerial.puma"
+/* line 431 "MakeSerial.puma" */
   {
-# line 427 "MakeSerial.puma"
+/* line 433 "MakeSerial.puma" */
    if (! ((IsDescriptorVar (t->ACF_IF.IF_EXP->VAR_EXP.V)))) goto yyL3;
   {
-# line 428 "MakeSerial.puma"
-   return false;
+/* line 434 "MakeSerial.puma" */
+   return rfalse;
   }
   }
 yyL3:;
 
   }
-# line 431 "MakeSerial.puma"
-   return true;
+/* line 437 "MakeSerial.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
 static void MakeNewIfMask
@@ -572,36 +576,24 @@ static void MakeNewIfMask
 # endif
 {
   if (t->Kind == kACF_IF) {
-# line 442 "MakeSerial.puma"
+/* line 448 "MakeSerial.puma" */
  {
   var_descriptor vard;
-  bool found;
+  rbool found;
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 444 "MakeSerial.puma"
-
-# line 445 "MakeSerial.puma"
-
-# line 447 "MakeSerial.puma"
+/* line 453 "MakeSerial.puma" */
    GetExpDescriptor (t->ACF_IF.IF_EXP, & found, & vard);
-# line 448 "MakeSerial.puma"
+/* line 454 "MakeSerial.puma" */
    if (! ((found))) goto yyL1;
   {
-# line 450 "MakeSerial.puma"
-   printf ("exp descriptor for if mask : ");
-# line 451 "MakeSerial.puma"
-   FileUnparse (stdout, t->ACF_IF.IF_EXP);
-# line 451 "MakeSerial.puma"
-   printf ("\n");
-# line 452 "MakeSerial.puma"
-   PrintVarDescriptor (& vard);
-# line 454 "MakeSerial.puma"
+/* line 462 "MakeSerial.puma" */
    SetExpType (t->ACF_IF.IF_EXP, & vard);
-# line 456 "MakeSerial.puma"
+/* line 464 "MakeSerial.puma" */
    MyMakeTempAssignment (t->ACF_IF.IF_EXP, & vard, & yyV1, & yyV2, & yyV3);
-# line 458 "MakeSerial.puma"
+/* line 466 "MakeSerial.puma" */
  t->ACF_IF.IF_EXP = yyV1; 
   }
   }
@@ -612,9 +604,9 @@ static void MakeNewIfMask
 yyL1:;
 
   }
-# line 461 "MakeSerial.puma"
+/* line 469 "MakeSerial.puma" */
   {
-# line 463 "MakeSerial.puma"
+/* line 471 "MakeSerial.puma" */
    error_protocol ("no var descriptor for masked forall statement");
   }
    * yyP8 = NoTree;
@@ -635,12 +627,12 @@ static void NewForallBounds
 # endif
 {
   if (t->Kind == kACF_FORALL) {
-# line 478 "MakeSerial.puma"
+/* line 486 "MakeSerial.puma" */
   {
-# line 482 "MakeSerial.puma"
+/* line 490 "MakeSerial.puma" */
    if (! ((NewForallBoundsRequired (t)))) goto yyL1;
   {
-# line 484 "MakeSerial.puma"
+/* line 492 "MakeSerial.puma" */
    error_protocol ("cannot create new bounds for FORALL loops");
   }
   }
@@ -649,7 +641,7 @@ static void NewForallBounds
    return;
 yyL1:;
 
-# line 487 "MakeSerial.puma"
+/* line 495 "MakeSerial.puma" */
    * yyP10 = NoTree;
    * yyP9 = NoTree;
    return;
@@ -658,7 +650,7 @@ yyL1:;
 ;
 }
 
-static bool NewForallBoundsRequired
+static rbool NewForallBoundsRequired
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -667,32 +659,32 @@ static bool NewForallBoundsRequired
 # endif
 {
   if (t->Kind == kACF_FORALL) {
-# line 498 "MakeSerial.puma"
+/* line 506 "MakeSerial.puma" */
   {
-# line 500 "MakeSerial.puma"
-   if (! ((UpdatesExpression (t->ACF_FORALL.FORALL_BODY, t->ACF_FORALL.FORALL_RANGE->SLICE_EXP.START)))) goto yyL1;
+/* line 508 "MakeSerial.puma" */
+   if (! ((UpdatesExpression (t->ACF_FORALL.FORALL_BODY, t->ACF_FORALL.FORALL_RANGE->SLICE_EXP.FIRST)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 503 "MakeSerial.puma"
+/* line 511 "MakeSerial.puma" */
   {
-# line 505 "MakeSerial.puma"
+/* line 513 "MakeSerial.puma" */
    if (! ((UpdatesExpression (t->ACF_FORALL.FORALL_BODY, t->ACF_FORALL.FORALL_RANGE->SLICE_EXP.STOP)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 508 "MakeSerial.puma"
+/* line 516 "MakeSerial.puma" */
   {
-# line 510 "MakeSerial.puma"
+/* line 518 "MakeSerial.puma" */
    if (! ((UpdatesExpression (t->ACF_FORALL.FORALL_BODY, t->ACF_FORALL.FORALL_RANGE->SLICE_EXP.INC)))) goto yyL3;
   }
-   return true;
+   return rtrue;
 yyL3:;
 
   }
-  return false;
+  return rfalse;
 }
 
 static tTree SplitWhere
@@ -704,19 +696,19 @@ static tTree SplitWhere
 # endif
 {
   if (where_stmt->Kind == kACF_WHERE) {
-# line 530 "MakeSerial.puma"
+/* line 538 "MakeSerial.puma" */
   {
-# line 532 "MakeSerial.puma"
-   MaskNewStatements (where_stmt->ACF_WHERE.TRUE_PART, where_stmt->ACF_WHERE.WHERE_EXP, true, where_stmt->ACF_WHERE.Line);
-# line 533 "MakeSerial.puma"
-   MaskNewStatements (where_stmt->ACF_WHERE.FALSE_PART, where_stmt->ACF_WHERE.WHERE_EXP, false, where_stmt->ACF_WHERE.Line);
+/* line 540 "MakeSerial.puma" */
+   MaskNewStatements (where_stmt->ACF_WHERE.TRUE_PART, where_stmt->ACF_WHERE.WHERE_EXP, rtrue, where_stmt->ACF_WHERE.Line);
+/* line 541 "MakeSerial.puma" */
+   MaskNewStatements (where_stmt->ACF_WHERE.FALSE_PART, where_stmt->ACF_WHERE.WHERE_EXP, rfalse, where_stmt->ACF_WHERE.Line);
   }
    return CombineACF (where_stmt->ACF_WHERE.TRUE_PART, where_stmt->ACF_WHERE.FALSE_PART);
 
   }
-# line 538 "MakeSerial.puma"
+/* line 546 "MakeSerial.puma" */
   {
-# line 539 "MakeSerial.puma"
+/* line 547 "MakeSerial.puma" */
    failure_protocol (MODULE, "SplitWhere", where_stmt);
   }
    return NoTree;
@@ -732,19 +724,19 @@ static tTree SplitIf
 # endif
 {
   if (if_stmt->Kind == kACF_IF) {
-# line 560 "MakeSerial.puma"
+/* line 568 "MakeSerial.puma" */
   {
-# line 562 "MakeSerial.puma"
-   MaskNewStatements (if_stmt->ACF_IF.THEN_PART, if_stmt->ACF_IF.IF_EXP, true, if_stmt->ACF_IF.Line);
-# line 563 "MakeSerial.puma"
-   MaskNewStatements (if_stmt->ACF_IF.ELSE_PART, if_stmt->ACF_IF.IF_EXP, false, if_stmt->ACF_IF.Line);
+/* line 570 "MakeSerial.puma" */
+   MaskNewStatements (if_stmt->ACF_IF.THEN_PART, if_stmt->ACF_IF.IF_EXP, rtrue, if_stmt->ACF_IF.Line);
+/* line 571 "MakeSerial.puma" */
+   MaskNewStatements (if_stmt->ACF_IF.ELSE_PART, if_stmt->ACF_IF.IF_EXP, rfalse, if_stmt->ACF_IF.Line);
   }
    return CombineACF (if_stmt->ACF_IF.THEN_PART, if_stmt->ACF_IF.ELSE_PART);
 
   }
-# line 568 "MakeSerial.puma"
+/* line 576 "MakeSerial.puma" */
   {
-# line 569 "MakeSerial.puma"
+/* line 577 "MakeSerial.puma" */
    failure_protocol (MODULE, "SplitIf", if_stmt);
   }
    return NoTree;
@@ -762,29 +754,25 @@ static void SplitAssign
 # endif
 {
   if (s->Kind == kASSIGN_STMT) {
-# line 591 "MakeSerial.puma"
+/* line 599 "MakeSerial.puma" */
  {
   var_descriptor vard;
-  bool found;
+  rbool found;
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 593 "MakeSerial.puma"
-
-# line 594 "MakeSerial.puma"
-
-# line 596 "MakeSerial.puma"
+/* line 604 "MakeSerial.puma" */
    if (! ((TreeRank (s->ASSIGN_STMT.ASSIGN_EXP) < TreeRank (s->ASSIGN_STMT.ASSIGN_VAR)))) goto yyL1;
   {
-# line 598 "MakeSerial.puma"
+/* line 606 "MakeSerial.puma" */
    GetExpDescriptor (s->ASSIGN_STMT.ASSIGN_EXP, & found, & vard);
-# line 599 "MakeSerial.puma"
+/* line 607 "MakeSerial.puma" */
    if (! ((found))) goto yyL1;
   {
-# line 601 "MakeSerial.puma"
+/* line 609 "MakeSerial.puma" */
    MyMakeTempAssignment (s->ASSIGN_STMT.ASSIGN_EXP, & vard, & yyV1, & yyV2, & yyV3);
-# line 603 "MakeSerial.puma"
+/* line 611 "MakeSerial.puma" */
  s->ASSIGN_STMT.ASSIGN_EXP = yyV1; 
   }
   }
@@ -795,28 +783,24 @@ static void SplitAssign
  }
 yyL1:;
 
-# line 606 "MakeSerial.puma"
+/* line 614 "MakeSerial.puma" */
  {
   var_descriptor vard;
-  bool found;
+  rbool found;
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 608 "MakeSerial.puma"
-
-# line 609 "MakeSerial.puma"
-
-# line 611 "MakeSerial.puma"
+/* line 619 "MakeSerial.puma" */
    GetVarDescriptor (s->ASSIGN_STMT.ASSIGN_VAR, & found, & vard);
-# line 612 "MakeSerial.puma"
+/* line 620 "MakeSerial.puma" */
    if (! ((found))) goto yyL2;
   {
-# line 614 "MakeSerial.puma"
+/* line 622 "MakeSerial.puma" */
    SetExpType (s->ASSIGN_STMT.ASSIGN_EXP, & vard);
-# line 616 "MakeSerial.puma"
+/* line 624 "MakeSerial.puma" */
    MyMakeTempAssignment (s->ASSIGN_STMT.ASSIGN_EXP, & vard, & yyV1, & yyV2, & yyV3);
-# line 618 "MakeSerial.puma"
+/* line 626 "MakeSerial.puma" */
  s->ASSIGN_STMT.ASSIGN_EXP = yyV1; 
   }
   }
@@ -827,9 +811,9 @@ yyL1:;
 yyL2:;
 
   }
-# line 621 "MakeSerial.puma"
+/* line 629 "MakeSerial.puma" */
   {
-# line 623 "MakeSerial.puma"
+/* line 631 "MakeSerial.puma" */
    error_protocol ("no home descriptor for this assignment");
   }
    * yyP12 = NoTree;
@@ -851,28 +835,22 @@ static void MyMakeTempAssignment
  register tTree * yyP13;
 # endif
 {
-# line 636 "MakeSerial.puma"
+/* line 644 "MakeSerial.puma" */
  {
   tTree pre_stmt;
   tTree post_stmt;
   tTree new_exp;
   {
-# line 638 "MakeSerial.puma"
-
-# line 639 "MakeSerial.puma"
-
-# line 640 "MakeSerial.puma"
-
-# line 642 "MakeSerial.puma"
+/* line 650 "MakeSerial.puma" */
    ForallExpansion (vard);
-# line 646 "MakeSerial.puma"
+/* line 654 "MakeSerial.puma" */
  if (HasOuterIndependentLoop ())
 
          { vard->topology_rank = 0;
            vard->topology_obj = GetDefaultTopology (0);
          }
     
-# line 653 "MakeSerial.puma"
+/* line 661 "MakeSerial.puma" */
    MakeTempAssignment (exp, vard, & new_exp, & pre_stmt, & post_stmt);
   }
    * yyP15 = new_exp;
@@ -884,40 +862,38 @@ static void MyMakeTempAssignment
 ;
 }
 
-static bool HasOuterIndependentLoop
+static rbool HasOuterIndependentLoop
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 664 "MakeSerial.puma"
+/* line 672 "MakeSerial.puma" */
  {
-  bool parloop;
+  rbool parloop;
   {
-# line 665 "MakeSerial.puma"
-
-# line 667 "MakeSerial.puma"
+/* line 675 "MakeSerial.puma" */
  int i;
-      parloop = false;
+      parloop = rfalse;
  
       for (i=1; i<= OuterLoops(); i++)
         parloop = parloop || IsIndepDoLoop (GetOuterLoop(i));
     
-# line 673 "MakeSerial.puma"
+/* line 681 "MakeSerial.puma" */
    if (! ((parloop))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-void BeginMakeSerial ()
+void BeginMakeSerial ARGS ((void))
 {
 }
 
-void CloseMakeSerial ()
+void CloseMakeSerial ARGS ((void))
 {
 }
