@@ -1,6 +1,13 @@
 # include "MoveDescriptor.h"
-# include "yyMoveDescriptor.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,35 +15,33 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 62 "MoveDescriptor.puma"
+/* line 52 "MoveDescriptor.puma" */
 
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
+# include "TreeOps.h"        /* IsSelector         */
 # include "Rank.h"
 # include "AlignCheck.h"     /* GetSimpleAlignment */
 
@@ -60,18 +65,26 @@
 *                                                                   *
 ********************************************************************/
 
-static bool ActShapeIsSlice (vd, ind)
+static rbool ActShapeIsSlice (vd, ind)
 
 pvar vd;
 int  ind;
 
-/* returns true if actual index at position ind (1..act_rank) is slice */
+/* returns rtrue if actual index at position ind (1..act_rank) is slice */
 
-{ if (vd->actual_shape[ind-1][0] != vd->actual_shape[ind-1][1])
-     return (true);
-  if (TreeRank (vd->actual_shape[ind-1][0]) > 0)
-     return (true);
-  return (false);
+{ tTree lb;
+
+  lb = vd->actual_shape[ind-1][0];
+
+  if (lb == NoTree) return (rtrue);       /* full shape             */
+
+  if (IsSelector (lb)) return (rfalse);   /* selection of component */
+
+  if (lb != vd->actual_shape[ind-1][1])
+     return (rtrue);
+  if (TreeRank (lb) > 0)
+     return (rtrue);
+  return (rfalse);
 
 } /* ActShapeIsSlice */
 
@@ -82,6 +95,12 @@ int  ind;
 
 */
 
+/*********************************************************************
+*                                                                    *
+*                                                                    *
+*                                                                    *
+*********************************************************************/
+
 static int IndexRankPos (vd, ind)
 
 pvar vd;
@@ -117,11 +136,25 @@ int  ind;
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* MoveDescriptor_Exit) () = yyExit;
+# include "yyMoveDescriptor.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* MoveDescriptor_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -130,25 +163,25 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module MoveDescriptor, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module MoveDescriptor, routine %s failed\n",
+  yyFunction);
  MoveDescriptor_Exit ();
 }
 
-void PrintShifting ARGS((pshift s));
-bool IsZeroShifting ARGS((pshift shift));
-bool VDIsShifting ARGS((pvar d1, pvar d2, pshift sv));
-bool VDIsSame ARGS((pvar d1, pvar d2));
-bool VDIsSubSet ARGS((pvar vard1, pvar vard2));
-void VDIsTopDimAligned ARGS((pvar vard1, pvar vard2, int top_dim, bool * yyP2, int * yyP1));
-static void IndexDifference ARGS((pvar vard1, int idim1, pvar vard2, int idim2, bool * yyP4, int * yyP3));
-bool VDIsSubTopDim ARGS((pvar vard1, pvar vard2, int top_dim));
-static bool VDHasFullIndexDim ARGS((pvar vard, int index_dim));
-static bool IsTheFormalRange ARGS((tTree actual, tTree formal));
-bool VDIsAligned ARGS((pvar vard1, pvar vard2));
-static bool VDFullAlignedDim ARGS((pvar vard1, pvar vard2, int top_dim));
-static bool SameDistribution ARGS((pvar vard1, pvar vard2, int top_dim));
-static bool SameDistributionKind ARGS((int kind1, int kind2));
-static bool SameDistributionSize ARGS((tTree size1, tTree size2));
+void PrintShifting ARGS ((pshift s));
+rbool VDIsShifting ARGS ((pvar d1, pvar d2, pshift sv));
+rbool VDIsSame ARGS ((pvar d1, pvar d2));
+rbool VDIsSubSet ARGS ((pvar vard1, pvar vard2));
+void VDIsTopDimAligned ARGS ((pvar vard1, pvar vard2, int top_dim, rbool * yyP2, int * yyP1));
+static void IndexDifference ARGS ((pvar vard1, int idim1, pvar vard2, int idim2, rbool * yyP4, int * yyP3));
+rbool VDIsSubTopDim ARGS ((pvar vard1, pvar vard2, int top_dim));
+static rbool VDHasFullIndexDim ARGS ((pvar vard, int index_dim));
+static rbool IsTheFormalRange ARGS ((tTree actual, tTree formal));
+rbool VDIsAligned ARGS ((pvar vard1, pvar vard2));
+static rbool VDFullAlignedDim ARGS ((pvar vard1, pvar vard2, int top_dim));
+static rbool SameDistribution ARGS ((pvar vard1, pvar vard2, int top_dim));
+static rbool SameDistributionKind ARGS ((int kind1, int kind2));
+static rbool SameDistributionSize ARGS ((tTree size1, tTree size2));
 
 void PrintShifting
 # if defined __STDC__ | defined __cplusplus
@@ -158,13 +191,11 @@ void PrintShifting
  pshift s;
 # endif
 {
-# line 158 "MoveDescriptor.puma"
+/* line 163 "MoveDescriptor.puma" */
  {
   int i;
   {
-# line 160 "MoveDescriptor.puma"
-
-# line 161 "MoveDescriptor.puma"
+/* line 166 "MoveDescriptor.puma" */
  printf ("Shifting, rank = %d, pos = [", s->shift_rank);
      for (i=0; i<s->shift_rank; i++)
          printf (" %d", s->shift_pos[i]);
@@ -177,43 +208,7 @@ void PrintShifting
 ;
 }
 
-bool IsZeroShifting
-# if defined __STDC__ | defined __cplusplus
-(pshift shift)
-# else
-(shift)
- pshift shift;
-# endif
-{
-# line 176 "MoveDescriptor.puma"
- {
-  bool okay;
-  int i;
-  int n;
-  {
-# line 178 "MoveDescriptor.puma"
-
-# line 180 "MoveDescriptor.puma"
-
-# line 181 "MoveDescriptor.puma"
-
-# line 183 "MoveDescriptor.puma"
- okay = true;
-    n = shift->shift_rank;
-    for (i=0; i<n; i++)
-       okay = okay && (shift->shift_pos[i] == 0);
-  
-# line 189 "MoveDescriptor.puma"
-   if (! ((okay))) goto yyL1;
-  }
-   return true;
- }
-yyL1:;
-
-  return false;
-}
-
-bool VDIsShifting
+rbool VDIsShifting
 # if defined __STDC__ | defined __cplusplus
 (pvar d1, pvar d2, pshift sv)
 # else
@@ -223,27 +218,21 @@ bool VDIsShifting
  pshift sv;
 # endif
 {
-# line 210 "MoveDescriptor.puma"
+/* line 191 "MoveDescriptor.puma" */
  {
   int top_rank;
   int top_dim;
-  bool found;
+  rbool found;
   {
-# line 220 "MoveDescriptor.puma"
+/* line 201 "MoveDescriptor.puma" */
    if (! ((SameDescriptorTopology (d1, d2)))) goto yyL1;
   {
-# line 224 "MoveDescriptor.puma"
+/* line 205 "MoveDescriptor.puma" */
    MakeTemplateDescriptor (d1);
-# line 225 "MoveDescriptor.puma"
+/* line 206 "MoveDescriptor.puma" */
    MakeTemplateDescriptor (d2);
-# line 227 "MoveDescriptor.puma"
-
-# line 228 "MoveDescriptor.puma"
-
-# line 229 "MoveDescriptor.puma"
-
-# line 233 "MoveDescriptor.puma"
- found = true;
+/* line 214 "MoveDescriptor.puma" */
+ found = rtrue;
  
     sv->shift_rank = d2->formal_rank;
 
@@ -255,7 +244,7 @@ bool VDIsShifting
     for (top_dim = 0; top_dim < top_rank; top_dim++)
 
       { int  val;
-        bool ok;
+        rbool ok;
 
         VDIsTopDimAligned (d1, d2, top_dim+1, &ok, &val);
 
@@ -279,7 +268,7 @@ bool VDIsShifting
              case kANY_GEN_BLOCK_DIM :  sv->shift_pos[ind2-1] = val; break;
              case kANY_BLOCK_DIM     :  sv->shift_pos[ind2-1] = val; break;
 
-             default : ok = false;
+             default : ok = rfalse;
 
              } 
 
@@ -289,18 +278,18 @@ bool VDIsShifting
       }
  
   
-# line 280 "MoveDescriptor.puma"
+/* line 261 "MoveDescriptor.puma" */
    if (! ((found))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-bool VDIsSame
+rbool VDIsSame
 # if defined __STDC__ | defined __cplusplus
 (pvar d1, pvar d2)
 # else
@@ -309,22 +298,22 @@ bool VDIsSame
  pvar d2;
 # endif
 {
-# line 291 "MoveDescriptor.puma"
+/* line 272 "MoveDescriptor.puma" */
   {
-# line 293 "MoveDescriptor.puma"
+/* line 274 "MoveDescriptor.puma" */
    if (! ((VDIsSubSet (d1, d2)))) goto yyL1;
   {
-# line 294 "MoveDescriptor.puma"
+/* line 275 "MoveDescriptor.puma" */
    if (! ((VDIsSubSet (d2, d1)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-bool VDIsSubSet
+rbool VDIsSubSet
 # if defined __STDC__ | defined __cplusplus
 (pvar vard1, pvar vard2)
 # else
@@ -333,54 +322,48 @@ bool VDIsSubSet
  pvar vard2;
 # endif
 {
-# line 310 "MoveDescriptor.puma"
+/* line 291 "MoveDescriptor.puma" */
   {
-# line 313 "MoveDescriptor.puma"
+/* line 294 "MoveDescriptor.puma" */
    if (! ((IsReplicatedDescriptor (vard2)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 316 "MoveDescriptor.puma"
+/* line 297 "MoveDescriptor.puma" */
   {
-# line 318 "MoveDescriptor.puma"
+/* line 299 "MoveDescriptor.puma" */
    if (! ((IsNoDescriptor (vard1)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 321 "MoveDescriptor.puma"
+/* line 302 "MoveDescriptor.puma" */
   {
-# line 323 "MoveDescriptor.puma"
+/* line 304 "MoveDescriptor.puma" */
    if (! ((IsNoDescriptor (vard2)))) goto yyL3;
   {
-# line 325 "MoveDescriptor.puma"
-   return false;
+/* line 306 "MoveDescriptor.puma" */
+   return rfalse;
   }
   }
 yyL3:;
 
-# line 328 "MoveDescriptor.puma"
+/* line 309 "MoveDescriptor.puma" */
  {
-  bool okay;
+  rbool okay;
   int dim;
   int rank;
   {
-# line 330 "MoveDescriptor.puma"
+/* line 311 "MoveDescriptor.puma" */
    if (! ((SameDescriptorTopology (vard1, vard2)))) goto yyL4;
   {
-# line 334 "MoveDescriptor.puma"
+/* line 315 "MoveDescriptor.puma" */
    MakeTemplateDescriptor (vard1);
-# line 335 "MoveDescriptor.puma"
+/* line 316 "MoveDescriptor.puma" */
    MakeTemplateDescriptor (vard2);
-# line 337 "MoveDescriptor.puma"
-
-# line 338 "MoveDescriptor.puma"
-
-# line 339 "MoveDescriptor.puma"
-
-# line 341 "MoveDescriptor.puma"
- okay = true;
+/* line 322 "MoveDescriptor.puma" */
+ okay = rtrue;
 
 #ifdef DEBUG
      printf ("VDIsSubset vard1 of vard2\n");
@@ -414,81 +397,75 @@ yyL3:;
 #endif
 
    
-# line 376 "MoveDescriptor.puma"
+/* line 357 "MoveDescriptor.puma" */
    if (! ((okay))) goto yyL4;
   }
   }
-   return true;
+   return rtrue;
  }
 yyL4:;
 
-  return false;
+  return rfalse;
 }
 
 void VDIsTopDimAligned
 # if defined __STDC__ | defined __cplusplus
-(pvar vard1, pvar vard2, register int top_dim, register bool * yyP2, register int * yyP1)
+(pvar vard1, pvar vard2, register int top_dim, register rbool * yyP2, register int * yyP1)
 # else
 (vard1, vard2, top_dim, yyP2, yyP1)
  pvar vard1;
  pvar vard2;
  register int top_dim;
- register bool * yyP2;
+ register rbool * yyP2;
  register int * yyP1;
 # endif
 {
-# line 393 "MoveDescriptor.puma"
+/* line 374 "MoveDescriptor.puma" */
   {
-# line 397 "MoveDescriptor.puma"
+/* line 378 "MoveDescriptor.puma" */
    if (! ((vard2 -> on_index_dim [top_dim - 1] == 0))) goto yyL1;
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    * yyP1 = 0;
    return;
 yyL1:;
 
-# line 400 "MoveDescriptor.puma"
+/* line 381 "MoveDescriptor.puma" */
  {
   int idim2;
   {
-# line 402 "MoveDescriptor.puma"
-
-# line 404 "MoveDescriptor.puma"
+/* line 385 "MoveDescriptor.puma" */
    idim2 = vard2 -> on_index_dim [top_dim - 1];
-# line 406 "MoveDescriptor.puma"
+/* line 387 "MoveDescriptor.puma" */
    if (! (VDHasFullIndexDim (vard2, idim2))) goto yyL2;
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    * yyP1 = 0;
    return;
  }
 yyL2:;
 
-# line 409 "MoveDescriptor.puma"
+/* line 390 "MoveDescriptor.puma" */
  {
   int idim1;
   int idim2;
-  bool yyV1;
+  rbool yyV1;
   int yyV2;
   {
-# line 411 "MoveDescriptor.puma"
+/* line 392 "MoveDescriptor.puma" */
    if (! ((vard1 -> on_kind [top_dim - 1] == vard2 -> on_kind [top_dim - 1]))) goto yyL3;
   {
-# line 413 "MoveDescriptor.puma"
-
-# line 414 "MoveDescriptor.puma"
-
-# line 416 "MoveDescriptor.puma"
+/* line 397 "MoveDescriptor.puma" */
    idim1 = vard1 -> on_index_dim [top_dim - 1];
-# line 417 "MoveDescriptor.puma"
+/* line 398 "MoveDescriptor.puma" */
    idim2 = vard2 -> on_index_dim [top_dim - 1];
-# line 419 "MoveDescriptor.puma"
+/* line 400 "MoveDescriptor.puma" */
    if (! ((idim1 > 0))) goto yyL3;
   {
-# line 420 "MoveDescriptor.puma"
+/* line 401 "MoveDescriptor.puma" */
    if (! ((idim2 > 0))) goto yyL3;
   {
-# line 422 "MoveDescriptor.puma"
+/* line 403 "MoveDescriptor.puma" */
    IndexDifference (vard1, idim1, vard2, idim2, & yyV1, & yyV2);
   }
   }
@@ -500,8 +477,8 @@ yyL2:;
  }
 yyL3:;
 
-# line 425 "MoveDescriptor.puma"
-   * yyP2 = false;
+/* line 406 "MoveDescriptor.puma" */
+   * yyP2 = rfalse;
    * yyP1 = 0;
    return;
 
@@ -510,30 +487,26 @@ yyL3:;
 
 static void IndexDifference
 # if defined __STDC__ | defined __cplusplus
-(pvar vard1, register int idim1, pvar vard2, register int idim2, register bool * yyP4, register int * yyP3)
+(pvar vard1, register int idim1, pvar vard2, register int idim2, register rbool * yyP4, register int * yyP3)
 # else
 (vard1, idim1, vard2, idim2, yyP4, yyP3)
  pvar vard1;
  register int idim1;
  pvar vard2;
  register int idim2;
- register bool * yyP4;
+ register rbool * yyP4;
  register int * yyP3;
 # endif
 {
-# line 431 "MoveDescriptor.puma"
+/* line 412 "MoveDescriptor.puma" */
  {
-  bool found;
+  rbool found;
   int val;
   {
-# line 433 "MoveDescriptor.puma"
+/* line 414 "MoveDescriptor.puma" */
    if (! ((IndexRankPos (vard1, idim1) == IndexRankPos (vard2, idim2)))) goto yyL1;
   {
-# line 435 "MoveDescriptor.puma"
-
-# line 436 "MoveDescriptor.puma"
-
-# line 438 "MoveDescriptor.puma"
+/* line 419 "MoveDescriptor.puma" */
    GetSimpleAlignment (vard1, idim1, vard2, idim2, & found, & val);
   }
   }
@@ -543,15 +516,15 @@ static void IndexDifference
  }
 yyL1:;
 
-# line 441 "MoveDescriptor.puma"
-   * yyP4 = false;
+/* line 422 "MoveDescriptor.puma" */
+   * yyP4 = rfalse;
    * yyP3 = 0;
    return;
 
 ;
 }
 
-bool VDIsSubTopDim
+rbool VDIsSubTopDim
 # if defined __STDC__ | defined __cplusplus
 (pvar vard1, pvar vard2, register int top_dim)
 # else
@@ -561,77 +534,67 @@ bool VDIsSubTopDim
  register int top_dim;
 # endif
 {
-# line 452 "MoveDescriptor.puma"
+/* line 433 "MoveDescriptor.puma" */
   {
-# line 454 "MoveDescriptor.puma"
+/* line 435 "MoveDescriptor.puma" */
    if (! ((vard2 -> on_index_dim [top_dim - 1] == 0))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 457 "MoveDescriptor.puma"
+/* line 438 "MoveDescriptor.puma" */
  {
   int idim2;
   {
-# line 459 "MoveDescriptor.puma"
-
-# line 461 "MoveDescriptor.puma"
+/* line 442 "MoveDescriptor.puma" */
    idim2 = vard2 -> on_index_dim [top_dim - 1];
-# line 463 "MoveDescriptor.puma"
+/* line 444 "MoveDescriptor.puma" */
    if (! (VDHasFullIndexDim (vard2, idim2))) goto yyL2;
   }
-   return true;
+   return rtrue;
  }
 yyL2:;
 
-# line 466 "MoveDescriptor.puma"
+/* line 447 "MoveDescriptor.puma" */
  {
   int idim1;
   int idim2;
-  bool found;
+  rbool found;
   int val;
   {
-# line 468 "MoveDescriptor.puma"
+/* line 449 "MoveDescriptor.puma" */
    if (! ((vard1 -> on_kind [top_dim - 1] == vard2 -> on_kind [top_dim - 1]))) goto yyL3;
   {
-# line 470 "MoveDescriptor.puma"
-
-# line 471 "MoveDescriptor.puma"
-
-# line 473 "MoveDescriptor.puma"
+/* line 454 "MoveDescriptor.puma" */
    idim1 = vard1 -> on_index_dim [top_dim - 1];
-# line 474 "MoveDescriptor.puma"
+/* line 455 "MoveDescriptor.puma" */
    idim2 = vard2 -> on_index_dim [top_dim - 1];
-# line 476 "MoveDescriptor.puma"
+/* line 457 "MoveDescriptor.puma" */
    if (! ((idim1 > 0))) goto yyL3;
   {
-# line 477 "MoveDescriptor.puma"
+/* line 458 "MoveDescriptor.puma" */
    if (! ((idim2 > 0))) goto yyL3;
   {
-# line 479 "MoveDescriptor.puma"
-
-# line 480 "MoveDescriptor.puma"
-
-# line 482 "MoveDescriptor.puma"
+/* line 463 "MoveDescriptor.puma" */
    GetSimpleAlignment (vard1, idim1, vard2, idim2, & found, & val);
-# line 484 "MoveDescriptor.puma"
+/* line 465 "MoveDescriptor.puma" */
    if (! ((found))) goto yyL3;
   {
-# line 485 "MoveDescriptor.puma"
+/* line 466 "MoveDescriptor.puma" */
    if (! ((val == 0))) goto yyL3;
   }
   }
   }
   }
   }
-   return true;
+   return rtrue;
  }
 yyL3:;
 
-  return false;
+  return rfalse;
 }
 
-static bool VDHasFullIndexDim
+static rbool VDHasFullIndexDim
 # if defined __STDC__ | defined __cplusplus
 (pvar vard, register int index_dim)
 # else
@@ -640,15 +603,13 @@ static bool VDHasFullIndexDim
  register int index_dim;
 # endif
 {
-# line 496 "MoveDescriptor.puma"
+/* line 477 "MoveDescriptor.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 500 "MoveDescriptor.puma"
-
-# line 502 "MoveDescriptor.puma"
+/* line 483 "MoveDescriptor.puma" */
  tTree a_lb, a_ub, a_inc;
-     tTree f_lb, f_ub, f_inc;
+     tTree f_lb, f_ub;
 
      a_lb  = vard->actual_shape[index_dim-1][0];
      a_ub  = vard->actual_shape[index_dim-1][1];
@@ -662,17 +623,17 @@ static bool VDHasFullIndexDim
             IsStride1 (a_inc);
 
    
-# line 518 "MoveDescriptor.puma"
+/* line 499 "MoveDescriptor.puma" */
    if (! ((okay))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsTheFormalRange
+static rbool IsTheFormalRange
 # if defined __STDC__ | defined __cplusplus
 (register tTree actual, register tTree formal)
 # else
@@ -681,42 +642,42 @@ static bool IsTheFormalRange
  register tTree formal;
 # endif
 {
-# line 544 "MoveDescriptor.puma"
+/* line 525 "MoveDescriptor.puma" */
   {
-# line 546 "MoveDescriptor.puma"
+/* line 527 "MoveDescriptor.puma" */
    if (! ((formal == NoTree))) goto yyL1;
   {
-# line 548 "MoveDescriptor.puma"
-   return false;
+/* line 529 "MoveDescriptor.puma" */
+   return rfalse;
   }
   }
 yyL1:;
 
-# line 551 "MoveDescriptor.puma"
+/* line 532 "MoveDescriptor.puma" */
   {
-# line 553 "MoveDescriptor.puma"
+/* line 534 "MoveDescriptor.puma" */
    if (! ((actual == NoTree))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   if (actual->Kind == kDUMMY_EXP) {
-# line 556 "MoveDescriptor.puma"
-   return true;
+/* line 537 "MoveDescriptor.puma" */
+   return rtrue;
 
   }
-# line 559 "MoveDescriptor.puma"
+/* line 540 "MoveDescriptor.puma" */
   {
-# line 561 "MoveDescriptor.puma"
+/* line 542 "MoveDescriptor.puma" */
    if (! ((EqualExpression (actual, formal)))) goto yyL4;
   }
-   return true;
+   return rtrue;
 yyL4:;
 
-  return false;
+  return rfalse;
 }
 
-bool VDIsAligned
+rbool VDIsAligned
 # if defined __STDC__ | defined __cplusplus
 (pvar vard1, pvar vard2)
 # else
@@ -725,48 +686,44 @@ bool VDIsAligned
  pvar vard2;
 # endif
 {
-# line 581 "MoveDescriptor.puma"
+/* line 562 "MoveDescriptor.puma" */
   {
-# line 583 "MoveDescriptor.puma"
+/* line 564 "MoveDescriptor.puma" */
    if (! ((IsReplicatedDescriptor (vard1)))) goto yyL1;
   {
-# line 584 "MoveDescriptor.puma"
+/* line 565 "MoveDescriptor.puma" */
    if (! ((IsReplicatedDescriptor (vard2)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 587 "MoveDescriptor.puma"
+/* line 568 "MoveDescriptor.puma" */
   {
-# line 589 "MoveDescriptor.puma"
+/* line 570 "MoveDescriptor.puma" */
    if (! ((IsHostDescriptor (vard1)))) goto yyL2;
   {
-# line 590 "MoveDescriptor.puma"
+/* line 571 "MoveDescriptor.puma" */
    if (! ((IsHostDescriptor (vard2)))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 593 "MoveDescriptor.puma"
+/* line 574 "MoveDescriptor.puma" */
  {
-  bool okay;
+  rbool okay;
   int dim;
   {
-# line 595 "MoveDescriptor.puma"
+/* line 576 "MoveDescriptor.puma" */
    if (! ((SameDescriptorTopology (vard1, vard2)))) goto yyL3;
   {
-# line 601 "MoveDescriptor.puma"
+/* line 582 "MoveDescriptor.puma" */
    MakeTemplateDescriptor (vard1);
-# line 602 "MoveDescriptor.puma"
+/* line 583 "MoveDescriptor.puma" */
    MakeTemplateDescriptor (vard2);
-# line 604 "MoveDescriptor.puma"
-
-# line 605 "MoveDescriptor.puma"
-
-# line 607 "MoveDescriptor.puma"
- okay = true;
+/* line 588 "MoveDescriptor.puma" */
+ okay = rtrue;
 
      for (dim = 1; dim <= vard1->topology_rank; dim ++)
 
@@ -777,18 +734,18 @@ yyL2:;
       }
 
    
-# line 619 "MoveDescriptor.puma"
+/* line 600 "MoveDescriptor.puma" */
    if (! ((okay))) goto yyL3;
   }
   }
-   return true;
+   return rtrue;
  }
 yyL3:;
 
-  return false;
+  return rfalse;
 }
 
-static bool VDFullAlignedDim
+static rbool VDFullAlignedDim
 # if defined __STDC__ | defined __cplusplus
 (pvar vard1, pvar vard2, register int top_dim)
 # else
@@ -798,62 +755,58 @@ static bool VDFullAlignedDim
  register int top_dim;
 # endif
 {
-# line 624 "MoveDescriptor.puma"
+/* line 605 "MoveDescriptor.puma" */
   {
-# line 628 "MoveDescriptor.puma"
+/* line 609 "MoveDescriptor.puma" */
    if (! ((vard1 -> on_val [top_dim - 1] == NoTree))) goto yyL1;
   {
-# line 629 "MoveDescriptor.puma"
+/* line 610 "MoveDescriptor.puma" */
    if (! ((vard2 -> on_val [top_dim - 1] == NoTree))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 632 "MoveDescriptor.puma"
+/* line 613 "MoveDescriptor.puma" */
  {
   int idim1;
   int idim2;
-  bool yyV1;
+  rbool yyV1;
   int yyV2;
   {
-# line 634 "MoveDescriptor.puma"
+/* line 615 "MoveDescriptor.puma" */
    if (! ((SameDistribution (vard1, vard2, top_dim)))) goto yyL2;
   {
-# line 636 "MoveDescriptor.puma"
-
-# line 637 "MoveDescriptor.puma"
-
-# line 639 "MoveDescriptor.puma"
+/* line 620 "MoveDescriptor.puma" */
    idim1 = vard1 -> on_index_dim [top_dim - 1];
-# line 640 "MoveDescriptor.puma"
+/* line 621 "MoveDescriptor.puma" */
    idim2 = vard2 -> on_index_dim [top_dim - 1];
-# line 642 "MoveDescriptor.puma"
+/* line 623 "MoveDescriptor.puma" */
    if (! ((idim1 > 0))) goto yyL2;
   {
-# line 643 "MoveDescriptor.puma"
+/* line 624 "MoveDescriptor.puma" */
    if (! ((idim2 > 0))) goto yyL2;
   {
-# line 645 "MoveDescriptor.puma"
+/* line 626 "MoveDescriptor.puma" */
    IndexDifference (vard1, idim1, vard2, idim2, & yyV1, & yyV2);
-# line 647 "MoveDescriptor.puma"
+/* line 628 "MoveDescriptor.puma" */
    if (! ((yyV1))) goto yyL2;
   {
-# line 648 "MoveDescriptor.puma"
+/* line 629 "MoveDescriptor.puma" */
    if (! ((yyV2 == 0))) goto yyL2;
   }
   }
   }
   }
   }
-   return true;
+   return rtrue;
  }
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
-static bool SameDistribution
+static rbool SameDistribution
 # if defined __STDC__ | defined __cplusplus
 (pvar vard1, pvar vard2, register int top_dim)
 # else
@@ -863,42 +816,42 @@ static bool SameDistribution
  register int top_dim;
 # endif
 {
-# line 659 "MoveDescriptor.puma"
+/* line 640 "MoveDescriptor.puma" */
   {
-# line 661 "MoveDescriptor.puma"
+/* line 642 "MoveDescriptor.puma" */
    if (! ((vard1 -> template_obj == vard2 -> template_obj))) goto yyL1;
   {
-# line 662 "MoveDescriptor.puma"
+/* line 643 "MoveDescriptor.puma" */
    if (! ((vard1 -> template_obj != NoObject))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 665 "MoveDescriptor.puma"
+/* line 646 "MoveDescriptor.puma" */
   {
-# line 667 "MoveDescriptor.puma"
+/* line 648 "MoveDescriptor.puma" */
    if (! ((vard1 -> topology_obj == vard2 -> topology_obj))) goto yyL2;
   {
-# line 668 "MoveDescriptor.puma"
+/* line 649 "MoveDescriptor.puma" */
    if (! ((vard1 -> topology_obj != NoObject))) goto yyL2;
   {
-# line 670 "MoveDescriptor.puma"
+/* line 651 "MoveDescriptor.puma" */
    if (! ((SameDistributionKind (vard1 -> on_kind [top_dim - 1], vard2 -> on_kind [top_dim - 1])))) goto yyL2;
   {
-# line 673 "MoveDescriptor.puma"
+/* line 654 "MoveDescriptor.puma" */
    if (! ((SameDistributionSize (vard1 -> on_size [top_dim - 1], vard2 -> on_size [top_dim - 1])))) goto yyL2;
   }
   }
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
-static bool SameDistributionKind
+static rbool SameDistributionKind
 # if defined __STDC__ | defined __cplusplus
 (register int kind1, register int kind2)
 # else
@@ -907,58 +860,58 @@ static bool SameDistributionKind
  register int kind2;
 # endif
 {
-# line 687 "MoveDescriptor.puma"
+/* line 668 "MoveDescriptor.puma" */
   {
-# line 689 "MoveDescriptor.puma"
+/* line 670 "MoveDescriptor.puma" */
    if (! ((kind1 == kBLOCK_DIM))) goto yyL1;
   {
-# line 690 "MoveDescriptor.puma"
+/* line 671 "MoveDescriptor.puma" */
    if (! ((kind2 == kBLOCK_DIM))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 693 "MoveDescriptor.puma"
+/* line 674 "MoveDescriptor.puma" */
   {
-# line 695 "MoveDescriptor.puma"
+/* line 676 "MoveDescriptor.puma" */
    if (! ((kind1 == kCYCLIC_DIM))) goto yyL2;
   {
-# line 696 "MoveDescriptor.puma"
+/* line 677 "MoveDescriptor.puma" */
    if (! ((kind2 == kCYCLIC_DIM))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 699 "MoveDescriptor.puma"
+/* line 680 "MoveDescriptor.puma" */
   {
-# line 701 "MoveDescriptor.puma"
+/* line 682 "MoveDescriptor.puma" */
    if (! ((kind1 == kGEN_BLOCK_DIM))) goto yyL3;
   {
-# line 702 "MoveDescriptor.puma"
+/* line 683 "MoveDescriptor.puma" */
    if (! ((kind2 == kGEN_BLOCK_DIM))) goto yyL3;
   }
   }
-   return true;
+   return rtrue;
 yyL3:;
 
-# line 705 "MoveDescriptor.puma"
+/* line 686 "MoveDescriptor.puma" */
   {
-# line 707 "MoveDescriptor.puma"
+/* line 688 "MoveDescriptor.puma" */
    if (! ((kind1 == kINDIRECT_DIM))) goto yyL4;
   {
-# line 708 "MoveDescriptor.puma"
+/* line 689 "MoveDescriptor.puma" */
    if (! ((kind2 == kINDIRECT_DIM))) goto yyL4;
   }
   }
-   return true;
+   return rtrue;
 yyL4:;
 
-  return false;
+  return rfalse;
 }
 
-static bool SameDistributionSize
+static rbool SameDistributionSize
 # if defined __STDC__ | defined __cplusplus
 (register tTree size1, register tTree size2)
 # else
@@ -967,41 +920,41 @@ static bool SameDistributionSize
  register tTree size2;
 # endif
 {
-# line 721 "MoveDescriptor.puma"
+/* line 702 "MoveDescriptor.puma" */
   {
-# line 723 "MoveDescriptor.puma"
+/* line 704 "MoveDescriptor.puma" */
    if (! ((size1 == NoTree))) goto yyL1;
   {
-# line 724 "MoveDescriptor.puma"
+/* line 705 "MoveDescriptor.puma" */
    if (! ((size2 == NoTree))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 727 "MoveDescriptor.puma"
+/* line 708 "MoveDescriptor.puma" */
   {
-# line 729 "MoveDescriptor.puma"
+/* line 710 "MoveDescriptor.puma" */
    if (! ((size1 != NoTree))) goto yyL2;
   {
-# line 730 "MoveDescriptor.puma"
+/* line 711 "MoveDescriptor.puma" */
    if (! ((size2 != NoTree))) goto yyL2;
   {
-# line 731 "MoveDescriptor.puma"
+/* line 712 "MoveDescriptor.puma" */
    if (! ((EqualExpression (size1, size2)))) goto yyL2;
   }
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
-void BeginMoveDescriptor ()
+void BeginMoveDescriptor ARGS ((void))
 {
 }
 
-void CloseMoveDescriptor ()
+void CloseMoveDescriptor ARGS ((void))
 {
 }
