@@ -1,6 +1,13 @@
 # include "Nesting.h"
-# include "yyNesting.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,27 +15,24 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 57 "Nesting.puma"
+/* line 57 "Nesting.puma" */
 
 
 # undef DEBUG
@@ -106,11 +110,25 @@ tTree unit;
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* Nesting_Exit) () = yyExit;
+# include "yyNesting.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* Nesting_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -119,35 +137,36 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module Nesting, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module Nesting, routine %s failed\n",
+  yyFunction);
  Nesting_Exit ();
 }
 
-static void NestInit ARGS((tTree t));
-static void NestExit ARGS((tTree t));
-static bool ValidObject ARGS((tDefinitions obj, tTree t));
-void NestOpenUnit ARGS((tTree t));
-void NestOpenUnitObj ARGS((tDefinitions unit_obj));
-void NestCloseUnit ARGS((tTree t));
-void NestCloseUnitObj ARGS((tDefinitions unit_obj));
-static tDefinitions CloseUnitScope ARGS((tTree t));
-static void SetNewUnitEntries ARGS((tDefinitions unit_obj, tDefinitions new_entries));
-tTree GetCurrentUnit ARGS(());
-tDefinitions GetCurrentUnitObject ARGS(());
-int GetCurrentModel ARGS(());
-void NestOpenType ARGS((tTree t));
-void NestCloseType ARGS((tTree t));
-void IncLoopNesting ARGS((tTree loop));
-void DecLoopNesting ARGS((tTree loop));
-int OuterLoops ARGS(());
-tTree GetOuterLoop ARGS((int n));
-ptree GetAllOuterLoops ARGS(());
-bool IsCurrentLoopVar ARGS((tTree t));
-static bool IsLoopVarHere ARGS((tTree v, tTree loop));
-bool IsCountLoop ARGS((tTree loop));
-tTree GetLoopId ARGS((tTree loop));
-tTree GetLoopSlice ARGS((tTree loop));
-void GetLoopIncrement ARGS((tTree t, bool * yyP2, int * yyP1));
+static void NestInit ARGS ((tTree t));
+static void NestExit ARGS ((tTree t));
+static rbool ValidObject ARGS ((tDefinitions obj, tTree t));
+void NestOpenUnit ARGS ((tTree t));
+void NestOpenUnitObj ARGS ((tDefinitions unit_obj));
+void NestCloseUnit ARGS ((tTree t));
+void NestCloseUnitObj ARGS ((tDefinitions unit_obj));
+static tDefinitions CloseUnitScope ARGS ((tTree t));
+static void SetNewUnitEntries ARGS ((tDefinitions unit_obj, tDefinitions new_entries));
+tTree GetCurrentUnit ARGS ((void));
+tDefinitions GetCurrentUnitObject ARGS ((void));
+int GetCurrentModel ARGS ((void));
+void NestOpenType ARGS ((tTree t));
+void NestCloseType ARGS ((tTree t));
+void IncLoopNesting ARGS ((tTree loop));
+void DecLoopNesting ARGS ((tTree loop));
+int OuterLoops ARGS ((void));
+tTree GetOuterLoop ARGS ((int n));
+ptree GetAllOuterLoops ARGS ((void));
+rbool IsCurrentLoopVar ARGS ((tTree t));
+static rbool IsLoopVarHere ARGS ((tTree v, tTree loop));
+rbool IsCountLoop ARGS ((tTree loop));
+tTree GetLoopId ARGS ((tTree loop));
+tTree GetLoopSlice ARGS ((tTree loop));
+void GetLoopIncrement ARGS ((tTree t, rbool * yyP2, int * yyP1));
 
 static void NestInit
 # if defined __STDC__ | defined __cplusplus
@@ -157,13 +176,13 @@ static void NestInit
  register tTree t;
 # endif
 {
-# line 136 "Nesting.puma"
+/* line 136 "Nesting.puma" */
   {
-# line 138 "Nesting.puma"
+/* line 138 "Nesting.puma" */
  if (Nesting != 0)
          simple_error_protocol ("NestInit: nested loops not closed");
     
-# line 141 "Nesting.puma"
+/* line 141 "Nesting.puma" */
    Nesting = 0;
   }
    return;
@@ -179,13 +198,13 @@ static void NestExit
  register tTree t;
 # endif
 {
-# line 146 "Nesting.puma"
+/* line 146 "Nesting.puma" */
   {
-# line 148 "Nesting.puma"
+/* line 148 "Nesting.puma" */
  if (Nesting != 0)
          simple_error_protocol ("NestExit: nested loops not closed");
     
-# line 151 "Nesting.puma"
+/* line 151 "Nesting.puma" */
    Nesting = 0;
   }
    return;
@@ -193,7 +212,7 @@ static void NestExit
 ;
 }
 
-static bool ValidObject
+static rbool ValidObject
 # if defined __STDC__ | defined __cplusplus
 (register tDefinitions obj, register tTree t)
 # else
@@ -202,51 +221,51 @@ static bool ValidObject
  register tTree t;
 # endif
 {
-# line 162 "Nesting.puma"
+/* line 162 "Nesting.puma" */
   {
-# line 163 "Nesting.puma"
+/* line 163 "Nesting.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   {
-# line 164 "Nesting.puma"
-   return false;
+/* line 164 "Nesting.puma" */
+   return rfalse;
   }
   }
 yyL1:;
 
   if (obj->Kind == kProcObject) {
   if (t->Kind == kPROGRAM_DECL) {
-# line 167 "Nesting.puma"
-   return true;
+/* line 167 "Nesting.puma" */
+   return rtrue;
 
   }
   if (t->Kind == kPROC_DECL) {
-# line 170 "Nesting.puma"
-   return true;
+/* line 170 "Nesting.puma" */
+   return rtrue;
 
   }
   }
   if (obj->Kind == kFuncObject) {
   if (t->Kind == kFUNC_DECL) {
-# line 173 "Nesting.puma"
-   return true;
+/* line 173 "Nesting.puma" */
+   return rtrue;
 
   }
   }
   if (obj->Kind == kBlockObject) {
   if (t->Kind == kBLOCK_DATA_DECL) {
-# line 176 "Nesting.puma"
-   return true;
+/* line 176 "Nesting.puma" */
+   return rtrue;
 
   }
   }
   if (obj->Kind == kModuleObject) {
   if (t->Kind == kMODULE_DECL) {
-# line 179 "Nesting.puma"
-   return true;
+/* line 179 "Nesting.puma" */
+   return rtrue;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
 void NestOpenUnit
@@ -258,39 +277,39 @@ void NestOpenUnit
 # endif
 {
   if (Tree_IsType (t, kUNIT_NODE)) {
-# line 190 "Nesting.puma"
+/* line 190 "Nesting.puma" */
   {
-# line 192 "Nesting.puma"
+/* line 192 "Nesting.puma" */
    if (! ((ValidObject (t->UNIT_NODE.Object, t)))) goto yyL1;
   {
-# line 194 "Nesting.puma"
+/* line 194 "Nesting.puma" */
    NestOpenUnitObj (t->UNIT_NODE.Object);
   }
   }
    return;
 yyL1:;
 
-# line 197 "Nesting.puma"
+/* line 197 "Nesting.puma" */
   {
-# line 199 "Nesting.puma"
+/* line 199 "Nesting.puma" */
    tree_error_protocol ("no valid object for this unit", t);
-# line 200 "Nesting.puma"
+/* line 200 "Nesting.puma" */
    obj_error_protocol ("object for unit was : ", t->UNIT_NODE.Object);
-# line 202 "Nesting.puma"
+/* line 202 "Nesting.puma" */
    failure_protocol (MODULE, "GetUnitObject: not valid", t);
-# line 204 "Nesting.puma"
+/* line 204 "Nesting.puma" */
  t->UNIT_NODE.Object = mProcObject (t->UNIT_NODE.Ident, t, Default, NoObject,
                          UserRoutine, mENTRY_EMPTY ());
     
-# line 207 "Nesting.puma"
+/* line 207 "Nesting.puma" */
    NestOpenUnitObj (t->UNIT_NODE.Object);
   }
    return;
 
   }
-# line 210 "Nesting.puma"
+/* line 210 "Nesting.puma" */
   {
-# line 211 "Nesting.puma"
+/* line 211 "Nesting.puma" */
    failure_protocol (MODULE, "NestOpenUnit", t);
   }
    return;
@@ -307,68 +326,68 @@ void NestOpenUnitObj
 # endif
 {
   if (unit_obj->Kind == kProcObject) {
-# line 222 "Nesting.puma"
+/* line 222 "Nesting.puma" */
   {
-# line 224 "Nesting.puma"
+/* line 224 "Nesting.puma" */
    set_protocol_unit (unit_obj->ProcObject.decl);
-# line 225 "Nesting.puma"
+/* line 225 "Nesting.puma" */
    IncUnitNest (unit_obj->ProcObject.decl, unit_obj);
-# line 226 "Nesting.puma"
+/* line 226 "Nesting.puma" */
    OpenScope (unit_obj->ProcObject.Declarations);
-# line 227 "Nesting.puma"
+/* line 227 "Nesting.puma" */
    NestInit (unit_obj->ProcObject.decl);
   }
    return;
 
   }
   if (unit_obj->Kind == kFuncObject) {
-# line 230 "Nesting.puma"
+/* line 230 "Nesting.puma" */
   {
-# line 232 "Nesting.puma"
+/* line 232 "Nesting.puma" */
    set_protocol_unit (unit_obj->FuncObject.decl);
-# line 233 "Nesting.puma"
+/* line 233 "Nesting.puma" */
    IncUnitNest (unit_obj->FuncObject.decl, unit_obj);
-# line 234 "Nesting.puma"
+/* line 234 "Nesting.puma" */
    OpenScope (unit_obj->FuncObject.Declarations);
-# line 235 "Nesting.puma"
+/* line 235 "Nesting.puma" */
    NestInit (unit_obj->FuncObject.decl);
   }
    return;
 
   }
   if (unit_obj->Kind == kBlockObject) {
-# line 238 "Nesting.puma"
+/* line 238 "Nesting.puma" */
   {
-# line 240 "Nesting.puma"
+/* line 240 "Nesting.puma" */
    set_protocol_unit (unit_obj->BlockObject.decl);
-# line 241 "Nesting.puma"
+/* line 241 "Nesting.puma" */
    IncUnitNest (unit_obj->BlockObject.decl, unit_obj);
-# line 242 "Nesting.puma"
+/* line 242 "Nesting.puma" */
    OpenScope (unit_obj->BlockObject.Declarations);
-# line 243 "Nesting.puma"
+/* line 243 "Nesting.puma" */
    NestInit (unit_obj->BlockObject.decl);
   }
    return;
 
   }
   if (unit_obj->Kind == kModuleObject) {
-# line 246 "Nesting.puma"
+/* line 246 "Nesting.puma" */
   {
-# line 248 "Nesting.puma"
+/* line 248 "Nesting.puma" */
    set_protocol_unit (unit_obj->ModuleObject.decl);
-# line 249 "Nesting.puma"
+/* line 249 "Nesting.puma" */
    IncUnitNest (unit_obj->ModuleObject.decl, unit_obj);
-# line 250 "Nesting.puma"
+/* line 250 "Nesting.puma" */
    OpenScope (unit_obj->ModuleObject.Declarations);
-# line 251 "Nesting.puma"
+/* line 251 "Nesting.puma" */
    NestInit (unit_obj->ModuleObject.decl);
   }
    return;
 
   }
-# line 254 "Nesting.puma"
+/* line 254 "Nesting.puma" */
   {
-# line 255 "Nesting.puma"
+/* line 255 "Nesting.puma" */
    failure_protocol (MODULE, "NestOpenUnitObj", unit_obj->Object.decl);
   }
    return;
@@ -384,15 +403,13 @@ void NestCloseUnit
  register tTree t;
 # endif
 {
-# line 270 "Nesting.puma"
+/* line 270 "Nesting.puma" */
  {
   tDefinitions unit_obj;
   {
-# line 272 "Nesting.puma"
-
-# line 274 "Nesting.puma"
+/* line 274 "Nesting.puma" */
    unit_obj = GetCurrentUnitObject ();
-# line 276 "Nesting.puma"
+/* line 276 "Nesting.puma" */
    SetNewUnitEntries (unit_obj, CloseUnitScope (t));
   }
    return;
@@ -409,9 +426,9 @@ void NestCloseUnitObj
  register tDefinitions unit_obj;
 # endif
 {
-# line 287 "Nesting.puma"
+/* line 287 "Nesting.puma" */
   {
-# line 289 "Nesting.puma"
+/* line 289 "Nesting.puma" */
    SetNewUnitEntries (unit_obj, CloseUnitScope (unit_obj->Object.decl));
   }
    return;
@@ -427,24 +444,20 @@ static tDefinitions CloseUnitScope
  register tTree t;
 # endif
 {
-# line 307 "Nesting.puma"
+/* line 307 "Nesting.puma" */
  {
   tDefinitions Scope;
   {
-# line 309 "Nesting.puma"
-
-# line 311 "Nesting.puma"
+/* line 311 "Nesting.puma" */
    NestExit (t);
-# line 312 "Nesting.puma"
+/* line 312 "Nesting.puma" */
    Scope = GetCurrentScope ();
-# line 313 "Nesting.puma"
+/* line 313 "Nesting.puma" */
    CloseScope ();
-# line 314 "Nesting.puma"
+/* line 314 "Nesting.puma" */
    DecUnitNest (t);
   }
-  {
    return Scope;
-  }
  }
 
 }
@@ -459,44 +472,44 @@ static void SetNewUnitEntries
 # endif
 {
   if (unit_obj->Kind == kProcObject) {
-# line 327 "Nesting.puma"
+/* line 327 "Nesting.puma" */
   {
-# line 329 "Nesting.puma"
+/* line 329 "Nesting.puma" */
    unit_obj->ProcObject.Declarations = new_entries;
   }
    return;
 
   }
   if (unit_obj->Kind == kFuncObject) {
-# line 332 "Nesting.puma"
+/* line 332 "Nesting.puma" */
   {
-# line 334 "Nesting.puma"
+/* line 334 "Nesting.puma" */
    unit_obj->FuncObject.Declarations = new_entries;
   }
    return;
 
   }
   if (unit_obj->Kind == kBlockObject) {
-# line 337 "Nesting.puma"
+/* line 337 "Nesting.puma" */
   {
-# line 339 "Nesting.puma"
+/* line 339 "Nesting.puma" */
    unit_obj->BlockObject.Declarations = new_entries;
   }
    return;
 
   }
   if (unit_obj->Kind == kModuleObject) {
-# line 342 "Nesting.puma"
+/* line 342 "Nesting.puma" */
   {
-# line 344 "Nesting.puma"
+/* line 344 "Nesting.puma" */
    unit_obj->ModuleObject.Declarations = new_entries;
   }
    return;
 
   }
-# line 347 "Nesting.puma"
+/* line 347 "Nesting.puma" */
   {
-# line 348 "Nesting.puma"
+/* line 348 "Nesting.puma" */
    failure_protocol (MODULE, "SetNewUnitEntries", unit_obj->Object.decl);
   }
    return;
@@ -506,84 +519,92 @@ static void SetNewUnitEntries
 
 tTree GetCurrentUnit
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 359 "Nesting.puma"
+/* line 359 "Nesting.puma" */
   {
-# line 360 "Nesting.puma"
+/* line 360 "Nesting.puma" */
    if (! ((UnitNesting == 0))) goto yyL1;
   }
    return NoTree;
 yyL1:;
 
-# line 364 "Nesting.puma"
+/* line 364 "Nesting.puma" */
    return UnitNest [UnitNesting - 1];
 
 }
 
 tDefinitions GetCurrentUnitObject
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 370 "Nesting.puma"
+/* line 370 "Nesting.puma" */
   {
-# line 371 "Nesting.puma"
+/* line 371 "Nesting.puma" */
    if (! ((UnitNesting == 0))) goto yyL1;
   }
    return NoObject;
 yyL1:;
 
-# line 375 "Nesting.puma"
+/* line 375 "Nesting.puma" */
    return UnitObjNest [UnitNesting - 1];
 
 }
 
 int GetCurrentModel
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 381 "Nesting.puma"
+/* line 381 "Nesting.puma" */
   {
-# line 382 "Nesting.puma"
+/* line 382 "Nesting.puma" */
    if (! ((target_model == HPF_SERIAL))) goto yyL1;
   }
    return target_model;
 yyL1:;
 
-# line 386 "Nesting.puma"
+/* line 386 "Nesting.puma" */
   {
-# line 387 "Nesting.puma"
+/* line 387 "Nesting.puma" */
    if (! ((UnitNesting == 0))) goto yyL2;
   }
    return target_model;
 yyL2:;
 
-# line 391 "Nesting.puma"
+/* line 391 "Nesting.puma" */
   {
-# line 392 "Nesting.puma"
+/* line 392 "Nesting.puma" */
    if (! ((IsSerialUnit (UnitNest [UnitNesting - 1])))) goto yyL3;
   }
    return HPF_SERIAL;
 yyL3:;
 
-# line 396 "Nesting.puma"
+/* line 396 "Nesting.puma" */
   {
-# line 397 "Nesting.puma"
+/* line 397 "Nesting.puma" */
    if (! ((IsLocalUnit (UnitNest [UnitNesting - 1])))) goto yyL4;
   }
    return HPF_LOCAL;
 yyL4:;
 
-# line 401 "Nesting.puma"
+/* line 401 "Nesting.puma" */
+  {
+/* line 402 "Nesting.puma" */
+   if (! ((IsTaskUnit (UnitNest [UnitNesting - 1])))) goto yyL5;
+  }
+   return HPF_TASK;
+yyL5:;
+
+/* line 406 "Nesting.puma" */
    return HPF_GLOBAL;
 
 }
@@ -597,24 +618,22 @@ void NestOpenType
 # endif
 {
   if (t->Kind == kRECORD_TYPE) {
-# line 414 "Nesting.puma"
+/* line 419 "Nesting.puma" */
  {
   tDefinitions tobj;
   {
-# line 416 "Nesting.puma"
-
-# line 418 "Nesting.puma"
+/* line 423 "Nesting.puma" */
    tobj = GetGlobalObject (t->RECORD_TYPE.Name);
-# line 420 "Nesting.puma"
+/* line 425 "Nesting.puma" */
    OpenScope (tobj->TypeObject.Components);
   }
    return;
  }
 
   }
-# line 423 "Nesting.puma"
+/* line 428 "Nesting.puma" */
   {
-# line 424 "Nesting.puma"
+/* line 429 "Nesting.puma" */
    failure_protocol (MODULE, "NestOpenType", t);
   }
    return;
@@ -631,17 +650,17 @@ void NestCloseType
 # endif
 {
   if (t->Kind == kRECORD_TYPE) {
-# line 429 "Nesting.puma"
+/* line 434 "Nesting.puma" */
   {
-# line 431 "Nesting.puma"
+/* line 436 "Nesting.puma" */
    CloseScope ();
   }
    return;
 
   }
-# line 434 "Nesting.puma"
+/* line 439 "Nesting.puma" */
   {
-# line 435 "Nesting.puma"
+/* line 440 "Nesting.puma" */
    failure_protocol (MODULE, "NestCloseType", t);
   }
    return;
@@ -657,9 +676,9 @@ void IncLoopNesting
  register tTree loop;
 # endif
 {
-# line 448 "Nesting.puma"
+/* line 453 "Nesting.puma" */
   {
-# line 450 "Nesting.puma"
+/* line 455 "Nesting.puma" */
  if (Nesting >= MAXLoops)  
        simple_error_protocol ("to deep loop nesting");
       else
@@ -681,9 +700,9 @@ void DecLoopNesting
  register tTree loop;
 # endif
 {
-# line 461 "Nesting.puma"
+/* line 466 "Nesting.puma" */
   {
-# line 463 "Nesting.puma"
+/* line 468 "Nesting.puma" */
  if (Nesting == 0)  
        simple_error_protocol ("Nesting : no loop to be quit");
       else if (Nest[Nesting-1] != loop)
@@ -699,12 +718,12 @@ void DecLoopNesting
 
 int OuterLoops
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 474 "Nesting.puma"
+/* line 479 "Nesting.puma" */
    return Nesting;
 
 }
@@ -717,21 +736,21 @@ tTree GetOuterLoop
  register int n;
 # endif
 {
-# line 480 "Nesting.puma"
+/* line 485 "Nesting.puma" */
   {
-# line 481 "Nesting.puma"
+/* line 486 "Nesting.puma" */
    if (! ((n >= 1))) goto yyL1;
   {
-# line 482 "Nesting.puma"
+/* line 487 "Nesting.puma" */
    if (! ((n <= Nesting))) goto yyL1;
   }
   }
    return Nest [n - 1];
 yyL1:;
 
-# line 486 "Nesting.puma"
+/* line 491 "Nesting.puma" */
   {
-# line 487 "Nesting.puma"
+/* line 492 "Nesting.puma" */
    failure_protocol (MODULE, "GetOuterLoop : illegal index", NoTree);
   }
    return NoTree;
@@ -740,17 +759,17 @@ yyL1:;
 
 ptree GetAllOuterLoops
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 493 "Nesting.puma"
+/* line 498 "Nesting.puma" */
    return Nest;
 
 }
 
-bool IsCurrentLoopVar
+rbool IsCurrentLoopVar
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -759,42 +778,38 @@ bool IsCurrentLoopVar
 # endif
 {
   if (t->Kind == kUSED_VAR) {
-# line 507 "Nesting.puma"
+/* line 512 "Nesting.puma" */
  {
-  bool found;
+  rbool found;
   int i;
   {
-# line 509 "Nesting.puma"
-
-# line 510 "Nesting.puma"
-
-# line 512 "Nesting.puma"
+/* line 517 "Nesting.puma" */
  
 
-      found = false;
+      found = rfalse;
       i     = 0;
       while ((!found) && (i < Nesting))
          { found = IsLoopVarHere (t->USED_VAR.VARNAME, Nest[i]);
            i += 1;
          }
     
-# line 521 "Nesting.puma"
+/* line 526 "Nesting.puma" */
    if (! (found)) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
   }
   if (t->Kind == kLOOP_VAR) {
-# line 524 "Nesting.puma"
-   return true;
+/* line 529 "Nesting.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
-static bool IsLoopVarHere
+static rbool IsLoopVarHere
 # if defined __STDC__ | defined __cplusplus
 (register tTree v, register tTree loop)
 # else
@@ -805,56 +820,56 @@ static bool IsLoopVarHere
 {
   if (loop->Kind == kACF_DO) {
   if (loop->ACF_DO.DO_ID->Kind == kLOOP_VAR) {
-# line 535 "Nesting.puma"
+/* line 540 "Nesting.puma" */
   {
-# line 536 "Nesting.puma"
+/* line 541 "Nesting.puma" */
    if (! ((EqualExpression (v, loop->ACF_DO.DO_ID->LOOP_VAR.LOOP_VARNAME)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   }
   if (loop->Kind == kACF_FORALL) {
   if (loop->ACF_FORALL.FORALL_ID->Kind == kLOOP_VAR) {
-# line 539 "Nesting.puma"
+/* line 544 "Nesting.puma" */
   {
-# line 540 "Nesting.puma"
+/* line 545 "Nesting.puma" */
    if (! ((EqualExpression (v, loop->ACF_FORALL.FORALL_ID->LOOP_VAR.LOOP_VARNAME)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   }
   if (loop->Kind == kDO_EXP) {
   if (loop->DO_EXP.DO_ID->Kind == kLOOP_VAR) {
-# line 543 "Nesting.puma"
+/* line 548 "Nesting.puma" */
   {
-# line 544 "Nesting.puma"
+/* line 549 "Nesting.puma" */
    if (! ((EqualExpression (v, loop->DO_EXP.DO_ID->LOOP_VAR.LOOP_VARNAME)))) goto yyL3;
   }
-   return true;
+   return rtrue;
 yyL3:;
 
   }
   }
   if (loop->Kind == kDO_VAR) {
   if (loop->DO_VAR.DO_ID->Kind == kLOOP_VAR) {
-# line 547 "Nesting.puma"
+/* line 552 "Nesting.puma" */
   {
-# line 548 "Nesting.puma"
+/* line 553 "Nesting.puma" */
    if (! ((EqualExpression (v, loop->DO_VAR.DO_ID->LOOP_VAR.LOOP_VARNAME)))) goto yyL4;
   }
-   return true;
+   return rtrue;
 yyL4:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
-bool IsCountLoop
+rbool IsCountLoop
 # if defined __STDC__ | defined __cplusplus
 (register tTree loop)
 # else
@@ -863,26 +878,26 @@ bool IsCountLoop
 # endif
 {
   if (loop->Kind == kACF_DO) {
-# line 561 "Nesting.puma"
-   return true;
+/* line 566 "Nesting.puma" */
+   return rtrue;
 
   }
   if (loop->Kind == kACF_FORALL) {
-# line 562 "Nesting.puma"
-   return true;
+/* line 567 "Nesting.puma" */
+   return rtrue;
 
   }
   if (loop->Kind == kDO_EXP) {
-# line 563 "Nesting.puma"
-   return true;
+/* line 568 "Nesting.puma" */
+   return rtrue;
 
   }
   if (loop->Kind == kDO_VAR) {
-# line 564 "Nesting.puma"
-   return true;
+/* line 569 "Nesting.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
 tTree GetLoopId
@@ -894,28 +909,28 @@ tTree GetLoopId
 # endif
 {
   if (loop->Kind == kACF_DO) {
-# line 568 "Nesting.puma"
+/* line 573 "Nesting.puma" */
    return loop->ACF_DO.DO_ID;
 
   }
   if (loop->Kind == kACF_FORALL) {
-# line 569 "Nesting.puma"
+/* line 574 "Nesting.puma" */
    return loop->ACF_FORALL.FORALL_ID;
 
   }
   if (loop->Kind == kDO_EXP) {
-# line 570 "Nesting.puma"
+/* line 575 "Nesting.puma" */
    return loop->DO_EXP.DO_ID;
 
   }
   if (loop->Kind == kDO_VAR) {
-# line 571 "Nesting.puma"
+/* line 576 "Nesting.puma" */
    return loop->DO_VAR.DO_ID;
 
   }
-# line 573 "Nesting.puma"
+/* line 578 "Nesting.puma" */
   {
-# line 574 "Nesting.puma"
+/* line 579 "Nesting.puma" */
    failure_protocol (MODULE, "GetLoopId", loop);
   }
    return NoTree;
@@ -931,28 +946,28 @@ tTree GetLoopSlice
 # endif
 {
   if (loop->Kind == kACF_DO) {
-# line 580 "Nesting.puma"
+/* line 585 "Nesting.puma" */
    return loop->ACF_DO.DO_RANGE;
 
   }
   if (loop->Kind == kACF_FORALL) {
-# line 581 "Nesting.puma"
+/* line 586 "Nesting.puma" */
    return loop->ACF_FORALL.FORALL_RANGE;
 
   }
   if (loop->Kind == kDO_EXP) {
-# line 582 "Nesting.puma"
+/* line 587 "Nesting.puma" */
    return loop->DO_EXP.RANGE;
 
   }
   if (loop->Kind == kDO_VAR) {
-# line 583 "Nesting.puma"
+/* line 588 "Nesting.puma" */
    return loop->DO_VAR.RANGE;
 
   }
-# line 585 "Nesting.puma"
+/* line 590 "Nesting.puma" */
   {
-# line 586 "Nesting.puma"
+/* line 591 "Nesting.puma" */
    failure_protocol (MODULE, "GetLoopSlice", loop);
   }
    return NoTree;
@@ -961,63 +976,61 @@ tTree GetLoopSlice
 
 void GetLoopIncrement
 # if defined __STDC__ | defined __cplusplus
-(register tTree t, register bool * yyP2, register int * yyP1)
+(register tTree t, register rbool * yyP2, register int * yyP1)
 # else
 (t, yyP2, yyP1)
  register tTree t;
- register bool * yyP2;
+ register rbool * yyP2;
  register int * yyP1;
 # endif
 {
   if (t->Kind == kSLICE_EXP) {
-# line 598 "Nesting.puma"
+/* line 603 "Nesting.puma" */
   {
-# line 600 "Nesting.puma"
+/* line 605 "Nesting.puma" */
    if (! ((t->SLICE_EXP.INC == NoTree))) goto yyL1;
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    * yyP1 = 1;
    return;
 yyL1:;
 
   if (t->SLICE_EXP.INC->Kind == kDUMMY_EXP) {
-# line 603 "Nesting.puma"
-   * yyP2 = true;
+/* line 608 "Nesting.puma" */
+   * yyP2 = rtrue;
    * yyP1 = 1;
    return;
 
   }
-# line 606 "Nesting.puma"
+/* line 611 "Nesting.puma" */
  {
-  bool found;
+  rbool found;
   int val;
   {
-# line 608 "Nesting.puma"
-
-# line 610 "Nesting.puma"
+/* line 615 "Nesting.puma" */
    GetIntConstValue (t->SLICE_EXP.INC, & found, & val);
-# line 611 "Nesting.puma"
+/* line 616 "Nesting.puma" */
    if (! ((found))) goto yyL3;
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    * yyP1 = val;
    return;
  }
 yyL3:;
 
-# line 614 "Nesting.puma"
-   * yyP2 = false;
+/* line 619 "Nesting.puma" */
+   * yyP2 = rfalse;
    * yyP1 = 1;
    return;
 
   }
   if (t->Kind == kACF_DO) {
-# line 617 "Nesting.puma"
+/* line 622 "Nesting.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   int yyV2;
   {
-# line 619 "Nesting.puma"
+/* line 624 "Nesting.puma" */
    GetLoopIncrement (t->ACF_DO.DO_RANGE, & yyV1, & yyV2);
   }
    * yyP2 = yyV1;
@@ -1027,12 +1040,12 @@ yyL3:;
 
   }
   if (t->Kind == kACF_FORALL) {
-# line 622 "Nesting.puma"
+/* line 627 "Nesting.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   int yyV2;
   {
-# line 624 "Nesting.puma"
+/* line 629 "Nesting.puma" */
    GetLoopIncrement (t->ACF_FORALL.FORALL_RANGE, & yyV1, & yyV2);
   }
    * yyP2 = yyV1;
@@ -1041,22 +1054,22 @@ yyL3:;
  }
 
   }
-# line 627 "Nesting.puma"
+/* line 632 "Nesting.puma" */
   {
-# line 629 "Nesting.puma"
+/* line 634 "Nesting.puma" */
    failure_protocol (MODULE, "GetLoopIncrement", t);
   }
-   * yyP2 = false;
+   * yyP2 = rfalse;
    * yyP1 = 0;
    return;
 
 ;
 }
 
-void BeginNesting ()
+void BeginNesting ARGS ((void))
 {
 }
 
-void CloseNesting ()
+void CloseNesting ARGS ((void))
 {
 }
