@@ -1,6 +1,13 @@
 # include "NormalDefs.h"
-# include "yyNormalDefs.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,31 +15,28 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 28 "NormalDefs.puma"
+/* line 28 "NormalDefs.puma" */
 
 
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
@@ -40,6 +44,7 @@
 
 # include "Transform.h"
 # include "Types.h"         /* GetObjectType */
+# include "Rank.h"
 
 # include "Nesting.h"
 
@@ -55,18 +60,32 @@
 
 static tTree Entity;
 static tTree NewEntityDecls;   /* global use for translating an entity */
-static bool  IsParameterEntity;
+static rbool  HasParameterAttribute;
 static tTree InitialValue;
 
 static tTree stmtfuncs;
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* NormalDefs_Exit) () = yyExit;
+# include "yyNormalDefs.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* NormalDefs_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -75,26 +94,27 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module NormalDefs, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module NormalDefs, routine %s failed\n",
+  yyFunction);
  NormalDefs_Exit ();
 }
 
-static tTree ReverseDeclList ARGS((tTree list, tTree newlist));
-tTree Normal1Definitions ARGS((tTree t));
-static tTree TranslateCommonDECL ARGS((tTree idlist));
-static void TranslateEntityDecl ARGS((tIdent id, int pos, tTree attributes, tTree current_entity));
-static void UpdateEntityVal ARGS((tTree decl, tTree newval));
-static void UpdateEntityDims ARGS((tTree decl, tTree newdims));
-tTree Normal2Definitions ARGS((tTree t));
-void StatementFunctions ARGS((tTree body));
-static tTree ExtractStatementFunctions ARGS((tTree t));
-static bool IsStatementFunction ARGS((tTree t));
-static bool NoIndexAllowed ARGS((tDefinitions obj));
-static tTree MakeStmtFuncDecl ARGS((tTree var, tTree exp));
-static tTree MakeStmtFuncFormals ARGS((tTree Parameters));
-static bool IsDummyArgName ARGS((tTree t));
-static bool IsDummyArgNameList ARGS((tTree t));
-static tTree MakeDataItems ARGS((tTree val));
+static tTree ReverseDeclList ARGS ((tTree list, tTree newlist));
+tTree Normal1Definitions ARGS ((tTree t));
+static tTree TranslateCommonDECL ARGS ((tTree idlist));
+static void TranslateEntityDecl ARGS ((tIdent id, int pos, tTree attributes, tTree current_entity));
+static void UpdateEntityVal ARGS ((tTree decl, tTree newval));
+static void UpdateEntityDims ARGS ((tTree decl, tTree newdims));
+tTree Normal2Definitions ARGS ((tTree t));
+void StatementFunctions ARGS ((tTree body));
+static tTree ExtractStatementFunctions ARGS ((tTree t));
+static rbool IsStatementFunction ARGS ((tTree t));
+static rbool NoIndexAllowed ARGS ((tDefinitions obj));
+static tTree MakeStmtFuncDecl ARGS ((tTree var, tTree exp));
+static tTree MakeStmtFuncFormals ARGS ((tTree Parameters));
+static rbool IsDummyArgName ARGS ((tTree t));
+static rbool IsDummyArgNameList ARGS ((tTree t));
+static tTree MakeDataItems ARGS ((tTree val));
 
 static tTree ReverseDeclList
 # if defined __STDC__ | defined __cplusplus
@@ -105,33 +125,30 @@ static tTree ReverseDeclList
  register tTree newlist;
 # endif
 {
-# line 71 "NormalDefs.puma"
+/* line 72 "NormalDefs.puma" */
   {
-# line 72 "NormalDefs.puma"
+/* line 73 "NormalDefs.puma" */
    if (! ((list == NoTree))) goto yyL1;
   }
    return newlist;
 yyL1:;
 
   if (list->Kind == kDECL_LIST) {
-# line 76 "NormalDefs.puma"
+/* line 77 "NormalDefs.puma" */
  {
   tTree help;
   {
-# line 78 "NormalDefs.puma"
-
-# line 80 "NormalDefs.puma"
+/* line 81 "NormalDefs.puma" */
  help = list->DECL_LIST.Next;
      list->DECL_LIST.Next = newlist;
    
   }
-  {
    return ReverseDeclList (help, list);
-  }
  }
 
   }
  yyAbort ("ReverseDeclList");
+ { tTree yyDummy; return yyDummy; }
 }
 
 tTree Normal1Definitions
@@ -143,54 +160,50 @@ tTree Normal1Definitions
 # endif
 {
   if (t->Kind == kDECL_LIST) {
-# line 108 "NormalDefs.puma"
+/* line 113 "NormalDefs.puma" */
  {
   tTree newdecl;
   {
-# line 110 "NormalDefs.puma"
+/* line 117 "NormalDefs.puma" */
+ set_protocol_stmt (t->DECL_LIST.Elem);  
 
-# line 112 "NormalDefs.puma"
- newdecl = Normal1Definitions (t->DECL_LIST.Elem);
+     newdecl = Normal1Definitions (t->DECL_LIST.Elem);
      t->DECL_LIST.Next    = Normal1Definitions (t->DECL_LIST.Next);
      newdecl = ReplaceDECL (t, newdecl, t->DECL_LIST.Next);
    
   }
-  {
    return newdecl;
-  }
  }
 
   }
   if (t->Kind == kDECL_EMPTY) {
-# line 119 "NormalDefs.puma"
+/* line 127 "NormalDefs.puma" */
    return t;
 
   }
   if (t->Kind == kENTITY_DECL) {
-# line 131 "NormalDefs.puma"
+/* line 139 "NormalDefs.puma" */
   {
-# line 133 "NormalDefs.puma"
+/* line 141 "NormalDefs.puma" */
    Entity = NoTree;
-# line 134 "NormalDefs.puma"
+/* line 142 "NormalDefs.puma" */
    NewEntityDecls = NoTree;
-# line 135 "NormalDefs.puma"
-   IsParameterEntity = false;
-# line 136 "NormalDefs.puma"
+/* line 143 "NormalDefs.puma" */
+   HasParameterAttribute = rfalse;
+/* line 144 "NormalDefs.puma" */
    InitialValue = NoTree;
-# line 138 "NormalDefs.puma"
+/* line 146 "NormalDefs.puma" */
    TranslateEntityDecl (t->ENTITY_DECL.Ident, t->ENTITY_DECL.Line, t->ENTITY_DECL.ATTRIBUTES, t);
   }
    return NewEntityDecls;
 
   }
   if (t->Kind == kCOMMON_DECL) {
-# line 143 "NormalDefs.puma"
+/* line 151 "NormalDefs.puma" */
  {
   tTree newdecl;
   {
-# line 147 "NormalDefs.puma"
-
-# line 149 "NormalDefs.puma"
+/* line 157 "NormalDefs.puma" */
  newdecl = TranslateCommonDECL (t->COMMON_DECL.IDS);
      if (newdecl == NoTree)
         newdecl = t;
@@ -198,13 +211,11 @@ tTree Normal1Definitions
         newdecl = mDECL_LIST (t, newdecl);
    
   }
-  {
    return newdecl;
-  }
  }
 
   }
-# line 160 "NormalDefs.puma"
+/* line 168 "NormalDefs.puma" */
    return t;
 
 }
@@ -217,14 +228,14 @@ static tTree TranslateCommonDECL
  register tTree idlist;
 # endif
 {
-# line 178 "NormalDefs.puma"
+/* line 186 "NormalDefs.puma" */
 
 tTree newdecl;
 
   if (idlist->Kind == kDECL_LIST) {
-# line 182 "NormalDefs.puma"
+/* line 190 "NormalDefs.puma" */
   {
-# line 183 "NormalDefs.puma"
+/* line 191 "NormalDefs.puma" */
  newdecl = TranslateCommonDECL (idlist->DECL_LIST.Elem);
      if (newdecl == NoTree)
         newdecl = TranslateCommonDECL (idlist->DECL_LIST.Next);
@@ -236,15 +247,15 @@ tTree newdecl;
 
   }
   if (idlist->Kind == kDECL_EMPTY) {
-# line 192 "NormalDefs.puma"
+/* line 200 "NormalDefs.puma" */
    return NoTree;
 
   }
   if (idlist->Kind == kVAR_DECL) {
   if (idlist->VAR_DECL.VAL->Kind == kARRAY_TYPE) {
-# line 196 "NormalDefs.puma"
+/* line 204 "NormalDefs.puma" */
   {
-# line 197 "NormalDefs.puma"
+/* line 205 "NormalDefs.puma" */
  newdecl = mDIMENSION_DECL (idlist->VAR_DECL.Ident, idlist->VAR_DECL.Line, idlist->VAR_DECL.VAL->ARRAY_TYPE.ARRAY_INDEX_TYPES); 
      idlist->VAR_DECL.VAL = idlist->VAR_DECL.VAL->ARRAY_TYPE.ARRAY_COMP_TYPE;
    
@@ -252,11 +263,12 @@ tTree newdecl;
    return newdecl;
 
   }
-# line 205 "NormalDefs.puma"
+/* line 213 "NormalDefs.puma" */
    return NoTree;
 
   }
  yyAbort ("TranslateCommonDECL");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static void TranslateEntityDecl
@@ -270,17 +282,18 @@ static void TranslateEntityDecl
  register tTree current_entity;
 # endif
 {
-# line 220 "NormalDefs.puma"
+/* line 228 "NormalDefs.puma" */
 
 tTree newdecl;
 
+ yyRecursion:
   if (attributes->Kind == kDECL_EMPTY) {
-# line 226 "NormalDefs.puma"
+/* line 234 "NormalDefs.puma" */
   {
-# line 228 "NormalDefs.puma"
- if (IsParameterEntity && (InitialValue == NoTree))
-       tree_error_protocol ("Missing initial value for PARAMETER", 
-                             current_entity);
+/* line 238 "NormalDefs.puma" */
+ if (HasParameterAttribute && (InitialValue == NoTree))
+
+      error_protocol ("initialization must appear with PARAMETER attribute");
 
     
 
@@ -300,39 +313,39 @@ tTree newdecl;
 
   switch (attributes->DECL_LIST.Elem->Kind) {
   case kTYPESPEC_DECL:
-# line 244 "NormalDefs.puma"
+/* line 254 "NormalDefs.puma" */
   {
-# line 245 "NormalDefs.puma"
+/* line 256 "NormalDefs.puma" */
  if (Entity == NoTree)
       Entity = mVAR_DECL (id, pos, attributes->DECL_LIST.Elem->TYPESPEC_DECL.VAL);
      else 
       UpdateEntityVal (Entity, attributes->DECL_LIST.Elem->TYPESPEC_DECL.VAL);
   
-# line 250 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 261 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kDIMENSION_DECL:
-# line 253 "NormalDefs.puma"
+/* line 264 "NormalDefs.puma" */
   {
-# line 254 "NormalDefs.puma"
+/* line 266 "NormalDefs.puma" */
  if (Entity == NoTree)
       Entity = mVAR_DECL (id, pos, mARRAY_TYPE (attributes->DECL_LIST.Elem->DIMENSION_DECL.INDEXES, mDUMMY_TYPE()));
      else 
       UpdateEntityDims (Entity, attributes->DECL_LIST.Elem->DIMENSION_DECL.INDEXES);
   
-# line 259 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 271 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kINIT_DATA_DECL:
-# line 262 "NormalDefs.puma"
+/* line 274 "NormalDefs.puma" */
   {
-# line 263 "NormalDefs.puma"
+/* line 276 "NormalDefs.puma" */
   InitialValue = attributes->DECL_LIST.Elem->INIT_DATA_DECL.VAL;
-     if (IsParameterEntity)
+     if (HasParameterAttribute)
        { newdecl = mPARAMETER_DECL (id, pos, attributes->DECL_LIST.Elem->INIT_DATA_DECL.VAL);
          NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
        }
@@ -345,299 +358,299 @@ tTree newdecl;
          NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
        }
   
-# line 277 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 290 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kSAVE_DECL:
-# line 280 "NormalDefs.puma"
+/* line 293 "NormalDefs.puma" */
   {
-# line 281 "NormalDefs.puma"
+/* line 295 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->SAVE_DECL.Ident = id; attributes->DECL_LIST.Elem->SAVE_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 284 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 298 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kEXTERNAL_DECL:
-# line 287 "NormalDefs.puma"
+/* line 301 "NormalDefs.puma" */
   {
-# line 288 "NormalDefs.puma"
+/* line 303 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->EXTERNAL_DECL.Ident = id; attributes->DECL_LIST.Elem->EXTERNAL_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 291 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 306 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kINTRINSIC_DECL:
-# line 294 "NormalDefs.puma"
+/* line 309 "NormalDefs.puma" */
   {
-# line 295 "NormalDefs.puma"
+/* line 311 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->INTRINSIC_DECL.Ident = id; attributes->DECL_LIST.Elem->INTRINSIC_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 298 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 314 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kINTENT_DECL:
-# line 301 "NormalDefs.puma"
+/* line 317 "NormalDefs.puma" */
   {
-# line 302 "NormalDefs.puma"
+/* line 319 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->INTENT_DECL.Ident = id; attributes->DECL_LIST.Elem->INTENT_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 305 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 322 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kOPTIONAL_DECL:
-# line 308 "NormalDefs.puma"
+/* line 325 "NormalDefs.puma" */
   {
-# line 309 "NormalDefs.puma"
+/* line 327 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->OPTIONAL_DECL.Ident = id; attributes->DECL_LIST.Elem->OPTIONAL_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 312 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 330 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kALLOCATABLE_DECL:
-# line 315 "NormalDefs.puma"
+/* line 333 "NormalDefs.puma" */
   {
-# line 316 "NormalDefs.puma"
+/* line 335 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->ALLOCATABLE_DECL.Ident = id; attributes->DECL_LIST.Elem->ALLOCATABLE_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 319 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 338 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kPOINTER_DECL:
-# line 322 "NormalDefs.puma"
+/* line 341 "NormalDefs.puma" */
   {
-# line 323 "NormalDefs.puma"
+/* line 343 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->POINTER_DECL.Ident = id; attributes->DECL_LIST.Elem->POINTER_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 326 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 346 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kTARGET_DECL:
-# line 329 "NormalDefs.puma"
+/* line 349 "NormalDefs.puma" */
   {
-# line 330 "NormalDefs.puma"
+/* line 351 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->TARGET_DECL.Ident = id; attributes->DECL_LIST.Elem->TARGET_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 333 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 355 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kPUBLIC_DECL:
-# line 336 "NormalDefs.puma"
+/* line 358 "NormalDefs.puma" */
   {
-# line 337 "NormalDefs.puma"
+/* line 360 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->PUBLIC_DECL.Ident = id; attributes->DECL_LIST.Elem->PUBLIC_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 340 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 364 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kPRIVATE_DECL:
-# line 343 "NormalDefs.puma"
+/* line 367 "NormalDefs.puma" */
   {
-# line 344 "NormalDefs.puma"
+/* line 369 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->PRIVATE_DECL.Ident = id; attributes->DECL_LIST.Elem->PRIVATE_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 347 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 373 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kPARAMETER_DECL:
-# line 350 "NormalDefs.puma"
+/* line 376 "NormalDefs.puma" */
   {
-# line 351 "NormalDefs.puma"
- IsParameterEntity = true; 
-# line 352 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 378 "NormalDefs.puma" */
+ HasParameterAttribute = rtrue; 
+/* line 380 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kTEMPLATE_DECL:
-# line 355 "NormalDefs.puma"
+/* line 383 "NormalDefs.puma" */
   {
-# line 356 "NormalDefs.puma"
+/* line 385 "NormalDefs.puma" */
  if (Entity == NoTree)
       { Entity = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->TEMPLATE_DECL.Ident = id; attributes->DECL_LIST.Elem->TEMPLATE_DECL.Line = pos; }
      else 
       tree_error_protocol ("Illegal TEMPLATE", current_entity);
   
-# line 361 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 391 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kPROCESSORS_DECL:
-# line 364 "NormalDefs.puma"
+/* line 394 "NormalDefs.puma" */
   {
-# line 365 "NormalDefs.puma"
+/* line 396 "NormalDefs.puma" */
  if (Entity == NoTree)
       { Entity = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->PROCESSORS_DECL.Ident = id; attributes->DECL_LIST.Elem->PROCESSORS_DECL.Line = pos; }
      else 
       tree_error_protocol ("Illegal PROCESSORS", current_entity);
   
-# line 370 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 402 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kALIGN_DECL:
-# line 373 "NormalDefs.puma"
+/* line 405 "NormalDefs.puma" */
   {
-# line 374 "NormalDefs.puma"
+/* line 407 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->ALIGN_DECL.Ident = id; attributes->DECL_LIST.Elem->ALIGN_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 377 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 411 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kDYNAMIC_DECL:
-# line 380 "NormalDefs.puma"
+/* line 414 "NormalDefs.puma" */
   {
-# line 381 "NormalDefs.puma"
+/* line 416 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->DYNAMIC_DECL.Ident = id; attributes->DECL_LIST.Elem->DYNAMIC_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 384 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 420 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kDISTRIBUTE_DECL:
-# line 387 "NormalDefs.puma"
+/* line 423 "NormalDefs.puma" */
   {
-# line 388 "NormalDefs.puma"
+/* line 425 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->DISTRIBUTE_DECL.Ident = id; attributes->DECL_LIST.Elem->DISTRIBUTE_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 391 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 429 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
-  case kOVERLAP_DECL:
-# line 394 "NormalDefs.puma"
+  case kSHADOW_DECL:
+/* line 432 "NormalDefs.puma" */
   {
-# line 395 "NormalDefs.puma"
-  newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->OVERLAP_DECL.Ident = id; attributes->DECL_LIST.Elem->OVERLAP_DECL.Line = pos;
+/* line 434 "NormalDefs.puma" */
+  newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->SHADOW_DECL.Ident = id; attributes->DECL_LIST.Elem->SHADOW_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 398 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 438 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kINHERIT_DECL:
-# line 401 "NormalDefs.puma"
+/* line 441 "NormalDefs.puma" */
   {
-# line 402 "NormalDefs.puma"
+/* line 443 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->INHERIT_DECL.Ident = id; attributes->DECL_LIST.Elem->INHERIT_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 405 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 447 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kSHARED_DECL:
-# line 408 "NormalDefs.puma"
+/* line 450 "NormalDefs.puma" */
   {
-# line 409 "NormalDefs.puma"
+/* line 452 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->SHARED_DECL.Ident = id; attributes->DECL_LIST.Elem->SHARED_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 412 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 456 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kRANGE_DECL:
-# line 415 "NormalDefs.puma"
+/* line 459 "NormalDefs.puma" */
   {
-# line 416 "NormalDefs.puma"
+/* line 461 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->RANGE_DECL.Ident = id; attributes->DECL_LIST.Elem->RANGE_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 419 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 465 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kLAYOUT_DECL:
-# line 422 "NormalDefs.puma"
+/* line 468 "NormalDefs.puma" */
   {
-# line 423 "NormalDefs.puma"
+/* line 470 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->LAYOUT_DECL.Ident = id; attributes->DECL_LIST.Elem->LAYOUT_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 426 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 474 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kPASS_BY_DECL:
-# line 429 "NormalDefs.puma"
+/* line 477 "NormalDefs.puma" */
   {
-# line 430 "NormalDefs.puma"
+/* line 479 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->PASS_BY_DECL.Ident = id; attributes->DECL_LIST.Elem->PASS_BY_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 433 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 483 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kMAP_TO_DECL:
-# line 436 "NormalDefs.puma"
+/* line 486 "NormalDefs.puma" */
   {
-# line 437 "NormalDefs.puma"
+/* line 488 "NormalDefs.puma" */
   newdecl = attributes->DECL_LIST.Elem; attributes->DECL_LIST.Elem->MAP_TO_DECL.Ident = id; attributes->DECL_LIST.Elem->MAP_TO_DECL.Line = pos;
      NewEntityDecls = mDECL_LIST (newdecl, NewEntityDecls);
   
-# line 440 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 492 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
 
-# line 443 "NormalDefs.puma"
+/* line 495 "NormalDefs.puma" */
   {
-# line 444 "NormalDefs.puma"
+/* line 497 "NormalDefs.puma" */
    tree_error_protocol ("Unknown Attribute", attributes->DECL_LIST.Elem);
-# line 445 "NormalDefs.puma"
-   TranslateEntityDecl (id, pos, attributes->DECL_LIST.Next, current_entity);
+/* line 498 "NormalDefs.puma" */
+   attributes = attributes->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
 ;
@@ -654,17 +667,17 @@ static void UpdateEntityVal
 {
   if (decl->Kind == kVAR_DECL) {
   if (decl->VAR_DECL.VAL->Kind == kARRAY_TYPE) {
-# line 450 "NormalDefs.puma"
+/* line 503 "NormalDefs.puma" */
   {
-# line 451 "NormalDefs.puma"
+/* line 504 "NormalDefs.puma" */
  decl->VAR_DECL.VAL->ARRAY_TYPE.ARRAY_COMP_TYPE = newval; 
   }
    return;
 
   }
-# line 454 "NormalDefs.puma"
+/* line 507 "NormalDefs.puma" */
   {
-# line 455 "NormalDefs.puma"
+/* line 508 "NormalDefs.puma" */
  decl->VAR_DECL.VAL = newval; 
   }
    return;
@@ -684,35 +697,35 @@ static void UpdateEntityDims
 {
   if (decl->Kind == kVAR_DECL) {
   if (decl->VAR_DECL.VAL->Kind == kARRAY_TYPE) {
-# line 460 "NormalDefs.puma"
+/* line 513 "NormalDefs.puma" */
   {
-# line 461 "NormalDefs.puma"
+/* line 514 "NormalDefs.puma" */
  decl->VAR_DECL.VAL->ARRAY_TYPE.ARRAY_INDEX_TYPES = newdims; 
   }
    return;
 
   }
-# line 464 "NormalDefs.puma"
+/* line 517 "NormalDefs.puma" */
   {
-# line 465 "NormalDefs.puma"
+/* line 518 "NormalDefs.puma" */
  decl->VAR_DECL.VAL = mARRAY_TYPE (newdims, decl->VAR_DECL.VAL); 
   }
    return;
 
   }
   if (decl->Kind == kTEMPLATE_DECL) {
-# line 468 "NormalDefs.puma"
+/* line 521 "NormalDefs.puma" */
   {
-# line 469 "NormalDefs.puma"
+/* line 522 "NormalDefs.puma" */
  decl->TEMPLATE_DECL.DIMENSIONS = newdims; 
   }
    return;
 
   }
   if (decl->Kind == kPROCESSORS_DECL) {
-# line 472 "NormalDefs.puma"
+/* line 525 "NormalDefs.puma" */
   {
-# line 473 "NormalDefs.puma"
+/* line 526 "NormalDefs.puma" */
  decl->PROCESSORS_DECL.DIMENSIONS = newdims; 
   }
    return;
@@ -729,14 +742,14 @@ tTree Normal2Definitions
  register tTree t;
 # endif
 {
-# line 493 "NormalDefs.puma"
+/* line 546 "NormalDefs.puma" */
 
 tTree newdecl;
 
   if (t->Kind == kDECL_LIST) {
-# line 497 "NormalDefs.puma"
+/* line 550 "NormalDefs.puma" */
   {
-# line 498 "NormalDefs.puma"
+/* line 551 "NormalDefs.puma" */
  newdecl = Normal2Definitions (t->DECL_LIST.Elem);
      t->DECL_LIST.Next    = Normal2Definitions (t->DECL_LIST.Next);
      newdecl = ReplaceDECL (t, newdecl, t->DECL_LIST.Next);
@@ -746,49 +759,39 @@ tTree newdecl;
 
   }
   if (t->Kind == kDECL_EMPTY) {
-# line 505 "NormalDefs.puma"
+/* line 558 "NormalDefs.puma" */
    return t;
 
   }
   if (t->Kind == kDIMENSION_DECL) {
-# line 509 "NormalDefs.puma"
+/* line 562 "NormalDefs.puma" */
  {
   tTree type;
   {
-# line 513 "NormalDefs.puma"
-
-# line 515 "NormalDefs.puma"
+/* line 568 "NormalDefs.puma" */
  type = GetBaseType (GetObjectType (GetLocalObject (t->DIMENSION_DECL.Ident)));
       type = mARRAY_TYPE (t->DIMENSION_DECL.INDEXES, type);
       t->Kind = kVAR_DECL;
       t->VAR_DECL.VAL = type;
     
   }
-  {
    return t;
-  }
  }
 
   }
   if (t->Kind == kVAR_DECL) {
   if (t->VAR_DECL.VAL->Kind == kARRAY_TYPE) {
-# line 523 "NormalDefs.puma"
+/* line 576 "NormalDefs.puma" */
    return t;
 
   }
-# line 530 "NormalDefs.puma"
+/* line 583 "NormalDefs.puma" */
  {
   tTree newdecl;
   tDefinitions obj;
   int rank;
   {
-# line 534 "NormalDefs.puma"
-
-# line 535 "NormalDefs.puma"
-
-# line 536 "NormalDefs.puma"
-
-# line 538 "NormalDefs.puma"
+/* line 591 "NormalDefs.puma" */
  obj = GetLocalObject (t->VAR_DECL.Ident);
       if (obj != NoObject)
          rank = VarRank (obj);
@@ -800,13 +803,11 @@ tTree newdecl;
          newdecl = NoTree;
     
   }
-  {
    return newdecl;
-  }
  }
 
   }
-# line 553 "NormalDefs.puma"
+/* line 606 "NormalDefs.puma" */
    return t;
 
 }
@@ -820,9 +821,9 @@ void StatementFunctions
 # endif
 {
   if (body->Kind == kBODY_NODE) {
-# line 565 "NormalDefs.puma"
+/* line 618 "NormalDefs.puma" */
   {
-# line 567 "NormalDefs.puma"
+/* line 620 "NormalDefs.puma" */
  
       
 
@@ -850,7 +851,7 @@ static tTree ExtractStatementFunctions
  register tTree t;
 # endif
 {
-# line 596 "NormalDefs.puma"
+/* line 649 "NormalDefs.puma" */
 
 tTree StmtFuncDecl;
 tTree NextList;
@@ -858,16 +859,16 @@ tTree NextList;
   if (t->Kind == kACF_LIST) {
   if (t->ACF_LIST.Elem->Kind == kACF_BASIC) {
   if (t->ACF_LIST.Elem->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
-# line 601 "NormalDefs.puma"
+/* line 654 "NormalDefs.puma" */
   {
-# line 603 "NormalDefs.puma"
+/* line 656 "NormalDefs.puma" */
    if (! ((IsStatementFunction (t->ACF_LIST.Elem->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR)))) goto yyL1;
   {
-# line 605 "NormalDefs.puma"
+/* line 658 "NormalDefs.puma" */
    set_protocol_stmt (t->ACF_LIST.Elem);
-# line 607 "NormalDefs.puma"
+/* line 660 "NormalDefs.puma" */
    stmt_protocol ("this is a statement function");
-# line 613 "NormalDefs.puma"
+/* line 666 "NormalDefs.puma" */
  StmtFuncDecl = MakeStmtFuncDecl (t->ACF_LIST.Elem->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR, t->ACF_LIST.Elem->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_EXP);
       NextList     = ExtractStatementFunctions (t->ACF_LIST.Next);
       stmtfuncs    = mDECL_LIST (StmtFuncDecl, stmtfuncs);
@@ -879,28 +880,29 @@ yyL1:;
 
   }
   if (t->ACF_LIST.Elem->ACF_BASIC.BASIC_STMT->Kind == kFORMAT_STMT) {
-# line 621 "NormalDefs.puma"
+/* line 674 "NormalDefs.puma" */
   {
-# line 625 "NormalDefs.puma"
+/* line 678 "NormalDefs.puma" */
  t->ACF_LIST.Next = ExtractStatementFunctions (t->ACF_LIST.Next); 
   }
    return t;
 
   }
   }
-# line 630 "NormalDefs.puma"
+/* line 683 "NormalDefs.puma" */
    return t;
 
   }
   if (t->Kind == kACF_EMPTY) {
-# line 637 "NormalDefs.puma"
+/* line 690 "NormalDefs.puma" */
    return t;
 
   }
  yyAbort ("ExtractStatementFunctions");
+ { tTree yyDummy; return yyDummy; }
 }
 
-static bool IsStatementFunction
+static rbool IsStatementFunction
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -910,24 +912,24 @@ static bool IsStatementFunction
 {
   if (t->Kind == kINDEXED_VAR) {
   if (t->INDEXED_VAR.IND_VAR->Kind == kUSED_VAR) {
-# line 649 "NormalDefs.puma"
+/* line 702 "NormalDefs.puma" */
   {
-# line 656 "NormalDefs.puma"
+/* line 709 "NormalDefs.puma" */
    if (! ((IsDummyArgNameList (t->INDEXED_VAR.IND_EXPS)))) goto yyL1;
   {
-# line 657 "NormalDefs.puma"
+/* line 710 "NormalDefs.puma" */
    if (! ((NoIndexAllowed (GetGlobalObject (t->INDEXED_VAR.IND_VAR->USED_VAR.VARNAME->VAR_OBJ.Ident))))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
-static bool NoIndexAllowed
+static rbool NoIndexAllowed
 # if defined __STDC__ | defined __cplusplus
 (register tDefinitions obj)
 # else
@@ -935,35 +937,35 @@ static bool NoIndexAllowed
  register tDefinitions obj;
 # endif
 {
-# line 662 "NormalDefs.puma"
+/* line 715 "NormalDefs.puma" */
   {
-# line 664 "NormalDefs.puma"
+/* line 717 "NormalDefs.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   if (obj->Kind == kVarObject) {
   if (obj->VarObject.Kind->Kind == kVarLocal) {
-# line 667 "NormalDefs.puma"
+/* line 720 "NormalDefs.puma" */
   {
-# line 669 "NormalDefs.puma"
+/* line 722 "NormalDefs.puma" */
    if (! ((obj->VarObject.in == GetCurrentUnitObject ()))) goto yyL2;
   {
-# line 671 "NormalDefs.puma"
+/* line 724 "NormalDefs.puma" */
    if (! ((VarRank (obj) == 0))) goto yyL2;
   {
-# line 672 "NormalDefs.puma"
+/* line 725 "NormalDefs.puma" */
    if (! ((! IsStringType (GetObjectType (obj))))) goto yyL2;
   }
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
 static tTree MakeStmtFuncDecl
@@ -975,16 +977,16 @@ static tTree MakeStmtFuncDecl
  register tTree exp;
 # endif
 {
-# line 684 "NormalDefs.puma"
+/* line 737 "NormalDefs.puma" */
 
    tObject OldObj, NewObj;
    tTree   ResType, Decl, Formals;
 
   if (var->Kind == kINDEXED_VAR) {
   if (var->INDEXED_VAR.IND_VAR->Kind == kUSED_VAR) {
-# line 689 "NormalDefs.puma"
+/* line 742 "NormalDefs.puma" */
   {
-# line 691 "NormalDefs.puma"
+/* line 744 "NormalDefs.puma" */
  Formals = MakeStmtFuncFormals (var->INDEXED_VAR.IND_EXPS);
 
     OldObj = GetLocalObject (var->INDEXED_VAR.IND_VAR->USED_VAR.VARNAME->VAR_OBJ.Ident);
@@ -1015,6 +1017,7 @@ static tTree MakeStmtFuncDecl
   }
   }
  yyAbort ("MakeStmtFuncDecl");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static tTree MakeStmtFuncFormals
@@ -1025,44 +1028,43 @@ static tTree MakeStmtFuncFormals
  register tTree Parameters;
 # endif
 {
+ yyRecursion:
   if (Parameters->Kind == kBTE_LIST) {
   if (Parameters->BTE_LIST.Elem->Kind == kVAR_EXP) {
   if (Parameters->BTE_LIST.Elem->VAR_EXP.V->Kind == kUSED_VAR) {
-# line 726 "NormalDefs.puma"
+/* line 779 "NormalDefs.puma" */
  {
   tTree P;
   {
-# line 728 "NormalDefs.puma"
-
-# line 730 "NormalDefs.puma"
+/* line 783 "NormalDefs.puma" */
    P = mVAR_PARAM_DECL (Parameters->BTE_LIST.Elem->VAR_EXP.V->USED_VAR.VARNAME->VAR_OBJ.Ident, Parameters->BTE_LIST.Elem->VAR_EXP.V->USED_VAR.VARNAME->VAR_OBJ.Pos, mDUMMY_TYPE ());
   }
-  {
    return mDECL_LIST (P, MakeStmtFuncFormals (Parameters->BTE_LIST.Next));
-  }
  }
 
   }
   }
-# line 735 "NormalDefs.puma"
+/* line 788 "NormalDefs.puma" */
   {
-# line 739 "NormalDefs.puma"
+/* line 792 "NormalDefs.puma" */
    error_protocol ("Illegal Statement Function");
-# line 740 "NormalDefs.puma"
+/* line 793 "NormalDefs.puma" */
    tree_protocol ("Not a legal parameter : ", Parameters->BTE_LIST.Elem);
   }
-   return MakeStmtFuncFormals (Parameters->BTE_LIST.Next);
+   Parameters = Parameters->BTE_LIST.Next;
+   goto yyRecursion;
 
   }
   if (Parameters->Kind == kBTE_EMPTY) {
-# line 744 "NormalDefs.puma"
+/* line 797 "NormalDefs.puma" */
    return mDECL_EMPTY ();
 
   }
  yyAbort ("MakeStmtFuncFormals");
+ { tTree yyDummy; return yyDummy; }
 }
 
-static bool IsDummyArgName
+static rbool IsDummyArgName
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -1071,14 +1073,14 @@ static bool IsDummyArgName
 # endif
 {
   if (t->Kind == kUSED_VAR) {
-# line 756 "NormalDefs.puma"
-   return true;
+/* line 809 "NormalDefs.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
-static bool IsDummyArgNameList
+static rbool IsDummyArgNameList
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -1087,27 +1089,27 @@ static bool IsDummyArgNameList
 # endif
 {
   if (t->Kind == kBTE_EMPTY) {
-# line 763 "NormalDefs.puma"
-   return true;
+/* line 816 "NormalDefs.puma" */
+   return rtrue;
 
   }
   if (t->Kind == kBTE_LIST) {
   if (t->BTE_LIST.Elem->Kind == kVAR_EXP) {
-# line 766 "NormalDefs.puma"
+/* line 819 "NormalDefs.puma" */
   {
-# line 767 "NormalDefs.puma"
+/* line 820 "NormalDefs.puma" */
    if (! ((IsDummyArgName (t->BTE_LIST.Elem->VAR_EXP.V)))) goto yyL2;
   {
-# line 768 "NormalDefs.puma"
+/* line 821 "NormalDefs.puma" */
    if (! ((IsDummyArgNameList (t->BTE_LIST.Next)))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
 static tTree MakeDataItems
@@ -1119,19 +1121,19 @@ static tTree MakeDataItems
 # endif
 {
   if (val->Kind == kARRAY_EXP) {
-# line 785 "NormalDefs.puma"
+/* line 838 "NormalDefs.puma" */
    return val->ARRAY_EXP.ELEMENTS;
 
   }
-# line 790 "NormalDefs.puma"
+/* line 843 "NormalDefs.puma" */
    return mBTE_LIST (val, mBTE_EMPTY ());
 
 }
 
-void BeginNormalDefs ()
+void BeginNormalDefs ARGS ((void))
 {
 }
 
-void CloseNormalDefs ()
+void CloseNormalDefs ARGS ((void))
 {
 }
