@@ -1,6 +1,13 @@
 # include "Normalization.h"
-# include "yyNormalization.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,37 +15,35 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 34 "Normalization.puma"
+/* line 34 "Normalization.puma" */
 
 
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 # include "Nesting.h"
 
 # include "DefTable.h"
 
+# include "Objects.h"
 # include "ArrayData.h"       /* SetActualOverlap                       */
 # include "Shapes.h"          /* MakeFullShape, ...                     */
 
@@ -57,11 +62,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* Normalization_Exit) () = yyExit;
+# include "yyNormalization.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* Normalization_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -70,29 +89,30 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module Normalization, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module Normalization, routine %s failed\n",
+  yyFunction);
  Normalization_Exit ();
 }
 
-void Normalization ARGS((tTree t));
-static void NormalizationBody ARGS((tTree body, tTree unit));
-static tTree SortDeclarations ARGS((tTree t));
-static void TryMoveDeclaration ARGS((tTree decl, bool * yyP2, tTree * yyP1));
-static tIdent GetTopologyId ARGS((tDefinitions obj));
-static void MoveDecl ARGS((tIdent id, tTree decl));
-static bool FoundDeclaration ARGS((tIdent id, tTree list));
-static bool IsDeclaration ARGS((tIdent id, tTree decl));
-static void NormalShape ARGS((tTree body));
-static tTree NormalShapeAST ARGS((tTree t));
-static bool shape_pred ARGS((tTree t));
-static tTree shape_func ARGS((tTree t));
-static void MakeFullAllocParams ARGS((tTree params));
-static tTree MakeFullShapeFunc ARGS((tTree t));
-static void MakeFullShapeProc ARGS((tTree t));
-static void NormalNotFullParam ARGS((tTree t));
-static tTree MakeRaggedArrays ARGS((tTree t));
-static bool ragged_pred ARGS((tTree t));
-static tTree ragged_func ARGS((tTree t));
+void Normalization ARGS ((tTree t));
+static void NormalizationBody ARGS ((tTree body, tTree unit));
+static tTree SortDeclarations ARGS ((tTree t));
+static void TryMoveDeclaration ARGS ((tTree decl, rbool * yyP2, tTree * yyP1));
+static tIdent GetTopologyId ARGS ((tDefinitions obj));
+static void MoveDecl ARGS ((tIdent id, tTree decl));
+static rbool FoundDeclaration ARGS ((tIdent id, tTree list));
+static rbool IsDeclaration ARGS ((tIdent id, tTree decl));
+static void NormalShape ARGS ((tTree body));
+static tTree NormalShapeAST ARGS ((tTree t));
+static rbool shape_pred ARGS ((tTree t));
+static tTree shape_func ARGS ((tTree t));
+static void MakeFullAllocParams ARGS ((tTree params));
+static tTree MakeFullShapeFunc ARGS ((tTree t));
+static void MakeFullShapeProc ARGS ((tTree t));
+static void NormalNotFullParam ARGS ((tTree t));
+static tTree MakeRaggedArrays ARGS ((tTree t));
+static rbool ragged_pred ARGS ((tTree t));
+static tTree ragged_func ARGS ((tTree t));
 
 void Normalization
 # if defined __STDC__ | defined __cplusplus
@@ -102,99 +122,96 @@ void Normalization
  register tTree t;
 # endif
 {
+ yyRecursion:
 
   switch (t->Kind) {
   case kCOMP_UNIT:
-# line 72 "Normalization.puma"
+/* line 73 "Normalization.puma" */
   {
-# line 74 "Normalization.puma"
-   open_protocol ("adaptor.normal");
-# line 75 "Normalization.puma"
-   Normalization (t->COMP_UNIT.COMP_ELEMENTS);
-# line 76 "Normalization.puma"
-   close_protocol ();
+/* line 75 "Normalization.puma" */
+   t = t->COMP_UNIT.COMP_ELEMENTS;
+   goto yyRecursion;
   }
-   return;
 
   case kUNIT_EMPTY:
-# line 79 "Normalization.puma"
+/* line 78 "Normalization.puma" */
    return;
 
   case kUNIT_LIST:
-# line 82 "Normalization.puma"
+/* line 81 "Normalization.puma" */
   {
-# line 84 "Normalization.puma"
+/* line 83 "Normalization.puma" */
    Normalization (t->UNIT_LIST.Elem);
-# line 85 "Normalization.puma"
-   Normalization (t->UNIT_LIST.Next);
+/* line 84 "Normalization.puma" */
+   t = t->UNIT_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   case kPROGRAM_DECL:
-# line 88 "Normalization.puma"
+/* line 87 "Normalization.puma" */
   {
-# line 90 "Normalization.puma"
+/* line 89 "Normalization.puma" */
    NestOpenUnit (t);
-# line 91 "Normalization.puma"
+/* line 90 "Normalization.puma" */
    NormalizationBody (t->PROGRAM_DECL.PROGRAM_BODY, t);
-# line 92 "Normalization.puma"
+/* line 91 "Normalization.puma" */
    NestCloseUnit (t);
   }
    return;
 
   case kPROC_DECL:
-# line 95 "Normalization.puma"
+/* line 94 "Normalization.puma" */
   {
-# line 97 "Normalization.puma"
+/* line 96 "Normalization.puma" */
    NestOpenUnit (t);
-# line 98 "Normalization.puma"
+/* line 97 "Normalization.puma" */
    NormalizationBody (t->PROC_DECL.PROC_BODY, t);
-# line 99 "Normalization.puma"
+/* line 98 "Normalization.puma" */
    NestCloseUnit (t);
   }
    return;
 
   case kFUNC_DECL:
-# line 102 "Normalization.puma"
+/* line 101 "Normalization.puma" */
   {
-# line 104 "Normalization.puma"
+/* line 103 "Normalization.puma" */
    NestOpenUnit (t);
-# line 105 "Normalization.puma"
+/* line 104 "Normalization.puma" */
    NormalizationBody (t->FUNC_DECL.FUNC_BODY, t);
-# line 106 "Normalization.puma"
+/* line 105 "Normalization.puma" */
    NestCloseUnit (t);
   }
    return;
 
   case kBLOCK_DATA_DECL:
-# line 109 "Normalization.puma"
+/* line 108 "Normalization.puma" */
   {
-# line 111 "Normalization.puma"
+/* line 110 "Normalization.puma" */
    NestOpenUnit (t);
-# line 112 "Normalization.puma"
+/* line 111 "Normalization.puma" */
    NormalizationBody (t->BLOCK_DATA_DECL.DATA_BODY, t);
-# line 113 "Normalization.puma"
+/* line 112 "Normalization.puma" */
    NestCloseUnit (t);
   }
    return;
 
   case kMODULE_DECL:
-# line 116 "Normalization.puma"
+/* line 115 "Normalization.puma" */
   {
-# line 118 "Normalization.puma"
+/* line 117 "Normalization.puma" */
    NestOpenUnit (t);
-# line 119 "Normalization.puma"
+/* line 118 "Normalization.puma" */
    NormalizationBody (t->MODULE_DECL.MODULE_BODY, t);
-# line 120 "Normalization.puma"
+/* line 119 "Normalization.puma" */
    NestCloseUnit (t);
   }
    return;
 
   }
 
-# line 123 "Normalization.puma"
+/* line 122 "Normalization.puma" */
   {
-# line 124 "Normalization.puma"
+/* line 123 "Normalization.puma" */
    failure_protocol (MODULE, "Normalization", t);
   }
    return;
@@ -212,31 +229,29 @@ static void NormalizationBody
 # endif
 {
   if (body->Kind == kBODY_NODE) {
-# line 139 "Normalization.puma"
+/* line 138 "Normalization.puma" */
   {
-# line 141 "Normalization.puma"
+/* line 140 "Normalization.puma" */
  body->BODY_NODE.DECLS = SortDeclarations (body->BODY_NODE.DECLS); 
-# line 143 "Normalization.puma"
+/* line 142 "Normalization.puma" */
  body->BODY_NODE.DECLS = MakeRaggedArrays (body->BODY_NODE.DECLS);
       body->BODY_NODE.STATS = MakeRaggedArrays (body->BODY_NODE.STATS);
     
-# line 147 "Normalization.puma"
+/* line 146 "Normalization.puma" */
    ReachingAllocate (body);
-# line 151 "Normalization.puma"
+/* line 150 "Normalization.puma" */
    NormalShape (body);
-# line 155 "Normalization.puma"
-   tree_protocol ("translated ragged arrays : ", body);
-# line 157 "Normalization.puma"
+/* line 156 "Normalization.puma" */
    InitAllocate (body, unit);
-# line 159 "Normalization.puma"
+/* line 158 "Normalization.puma" */
    Normalization (body->BODY_NODE.INTERNALS);
   }
    return;
 
   }
-# line 162 "Normalization.puma"
+/* line 161 "Normalization.puma" */
   {
-# line 164 "Normalization.puma"
+/* line 163 "Normalization.puma" */
    failure_protocol (MODULE, "NormalizationBody", body);
   }
    return;
@@ -252,92 +267,89 @@ static tTree SortDeclarations
  register tTree t;
 # endif
 {
+ yyRecursion:
   if (t->Kind == kDECL_EMPTY) {
-# line 182 "Normalization.puma"
+/* line 181 "Normalization.puma" */
    return t;
 
   }
   if (t->Kind == kDECL_LIST) {
-# line 186 "Normalization.puma"
+/* line 185 "Normalization.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   tTree yyV2;
   {
-# line 187 "Normalization.puma"
+/* line 186 "Normalization.puma" */
    TryMoveDeclaration (t, & yyV1, & yyV2);
-# line 188 "Normalization.puma"
+/* line 187 "Normalization.puma" */
    if (! ((yyV1))) goto yyL2;
   }
-  {
-   return SortDeclarations (yyV2);
-  }
+   t = yyV2;
+   goto yyRecursion;
  }
 yyL2:;
 
-# line 192 "Normalization.puma"
+/* line 191 "Normalization.puma" */
   {
-# line 193 "Normalization.puma"
+/* line 192 "Normalization.puma" */
  t->DECL_LIST.Next = SortDeclarations (t->DECL_LIST.Next); 
   }
    return t;
 
   }
  yyAbort ("SortDeclarations");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static void TryMoveDeclaration
 # if defined __STDC__ | defined __cplusplus
-(register tTree decl, register bool * yyP2, register tTree * yyP1)
+(register tTree decl, register rbool * yyP2, register tTree * yyP1)
 # else
 (decl, yyP2, yyP1)
  register tTree decl;
- register bool * yyP2;
+ register rbool * yyP2;
  register tTree * yyP1;
 # endif
 {
   if (decl->Kind == kDECL_LIST) {
   if (decl->DECL_LIST.Elem->Kind == kDISTRIBUTE_DECL) {
-# line 205 "Normalization.puma"
+/* line 204 "Normalization.puma" */
   {
-# line 208 "Normalization.puma"
+/* line 207 "Normalization.puma" */
    if (! ((FoundDeclaration (decl->DECL_LIST.Elem->DISTRIBUTE_DECL.Ident, decl->DECL_LIST.Next)))) goto yyL1;
   {
-# line 209 "Normalization.puma"
+/* line 208 "Normalization.puma" */
    MoveDecl (decl->DECL_LIST.Elem->DISTRIBUTE_DECL.Ident, decl);
   }
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    * yyP1 = decl;
    return;
 yyL1:;
 
   }
   if (decl->DECL_LIST.Elem->Kind == kTEMPLATE_DECL) {
-# line 214 "Normalization.puma"
+/* line 213 "Normalization.puma" */
  {
   tDefinitions proc_obj;
   tIdent proc_id;
   {
-# line 217 "Normalization.puma"
-
-# line 218 "Normalization.puma"
-
-# line 220 "Normalization.puma"
+/* line 219 "Normalization.puma" */
    proc_obj = GetTopologyObj (GetGlobalObject (decl->DECL_LIST.Elem->TEMPLATE_DECL.Ident));
-# line 222 "Normalization.puma"
+/* line 221 "Normalization.puma" */
    proc_id = GetTopologyId (proc_obj);
-# line 224 "Normalization.puma"
+/* line 223 "Normalization.puma" */
    if (! ((proc_id != DefaultId ()))) goto yyL2;
   {
-# line 226 "Normalization.puma"
+/* line 225 "Normalization.puma" */
    if (! ((FoundDeclaration (proc_id, decl->DECL_LIST.Next)))) goto yyL2;
   {
-# line 227 "Normalization.puma"
+/* line 226 "Normalization.puma" */
    MoveDecl (proc_id, decl);
   }
   }
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    * yyP1 = decl;
    return;
  }
@@ -345,61 +357,53 @@ yyL2:;
 
   }
   if (decl->DECL_LIST.Elem->Kind == kVAR_DECL) {
-# line 232 "Normalization.puma"
+/* line 231 "Normalization.puma" */
  {
   tDefinitions proc_obj;
   tIdent proc_id;
   {
-# line 235 "Normalization.puma"
-
-# line 236 "Normalization.puma"
-
-# line 238 "Normalization.puma"
+/* line 237 "Normalization.puma" */
    proc_obj = GetTopologyObj (GetGlobalObject (decl->DECL_LIST.Elem->VAR_DECL.Ident));
-# line 240 "Normalization.puma"
+/* line 239 "Normalization.puma" */
    proc_id = GetTopologyId (proc_obj);
-# line 242 "Normalization.puma"
+/* line 241 "Normalization.puma" */
    if (! ((proc_id != DefaultId ()))) goto yyL3;
   {
-# line 243 "Normalization.puma"
+/* line 242 "Normalization.puma" */
    if (! ((FoundDeclaration (proc_id, decl->DECL_LIST.Next)))) goto yyL3;
   {
-# line 244 "Normalization.puma"
+/* line 243 "Normalization.puma" */
    MoveDecl (proc_id, decl);
   }
   }
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    * yyP1 = decl;
    return;
  }
 yyL3:;
 
-# line 258 "Normalization.puma"
+/* line 257 "Normalization.puma" */
  {
   tDefinitions temp_obj;
   tIdent temp_id;
   {
-# line 261 "Normalization.puma"
-
-# line 262 "Normalization.puma"
-
-# line 264 "Normalization.puma"
+/* line 263 "Normalization.puma" */
    temp_obj = GetTemplateObj (GetGlobalObject (decl->DECL_LIST.Elem->VAR_DECL.Ident));
-# line 265 "Normalization.puma"
+/* line 264 "Normalization.puma" */
    temp_id = temp_obj->Object.Ident;
-# line 267 "Normalization.puma"
+/* line 266 "Normalization.puma" */
    if (! (((temp_id != DefaultId ()) && (temp_id != decl->DECL_LIST.Elem->VAR_DECL.Ident)))) goto yyL5;
   {
-# line 268 "Normalization.puma"
+/* line 267 "Normalization.puma" */
    if (! ((FoundDeclaration (temp_id, decl->DECL_LIST.Next)))) goto yyL5;
   {
-# line 269 "Normalization.puma"
+/* line 268 "Normalization.puma" */
    MoveDecl (temp_id, decl);
   }
   }
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    * yyP1 = decl;
    return;
  }
@@ -407,24 +411,24 @@ yyL5:;
 
   }
   if (decl->DECL_LIST.Elem->Kind == kALIGN_DECL) {
-# line 249 "Normalization.puma"
+/* line 248 "Normalization.puma" */
   {
-# line 252 "Normalization.puma"
+/* line 251 "Normalization.puma" */
    if (! ((FoundDeclaration (decl->DECL_LIST.Elem->ALIGN_DECL.Ident, decl->DECL_LIST.Next)))) goto yyL4;
   {
-# line 253 "Normalization.puma"
+/* line 252 "Normalization.puma" */
    MoveDecl (decl->DECL_LIST.Elem->ALIGN_DECL.Ident, decl);
   }
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    * yyP1 = decl;
    return;
 yyL4:;
 
   }
   }
-# line 274 "Normalization.puma"
-   * yyP2 = false;
+/* line 273 "Normalization.puma" */
+   * yyP2 = rfalse;
    * yyP1 = decl;
    return;
 
@@ -439,35 +443,35 @@ static tIdent GetTopologyId
  register tDefinitions obj;
 # endif
 {
-# line 288 "Normalization.puma"
+/* line 287 "Normalization.puma" */
   {
-# line 290 "Normalization.puma"
+/* line 289 "Normalization.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   }
    return DefaultId ();
 yyL1:;
 
   if (obj->Kind == kTopologyObject) {
-# line 295 "Normalization.puma"
+/* line 294 "Normalization.puma" */
   {
-# line 297 "Normalization.puma"
+/* line 296 "Normalization.puma" */
    if (! ((obj->TopologyObject.Ident == DefaultId ()))) goto yyL2;
   }
    return obj->TopologyObject.Ident;
 yyL2:;
 
   if (obj->TopologyObject.decl->Kind == kPROCESSORS_DECL) {
-# line 302 "Normalization.puma"
+/* line 301 "Normalization.puma" */
    return obj->TopologyObject.Ident;
 
   }
   if (obj->TopologyObject.decl->Kind == kSUB_PROCS_DECL) {
-# line 307 "Normalization.puma"
+/* line 306 "Normalization.puma" */
    return obj->TopologyObject.decl->SUB_PROCS_DECL.TOP->TOP_OBJ.Ident;
 
   }
   }
-# line 312 "Normalization.puma"
+/* line 311 "Normalization.puma" */
    return DefaultId ();
 
 }
@@ -481,18 +485,17 @@ static void MoveDecl
  register tTree decl;
 # endif
 {
+ yyRecursion:
   if (decl->Kind == kDECL_LIST) {
   if (decl->DECL_LIST.Next->Kind == kDECL_LIST) {
-# line 327 "Normalization.puma"
+/* line 326 "Normalization.puma" */
  {
   tTree h;
   {
-# line 329 "Normalization.puma"
+/* line 328 "Normalization.puma" */
    if (! ((IsDeclaration (id, decl->DECL_LIST.Next->DECL_LIST.Elem)))) goto yyL1;
   {
-# line 331 "Normalization.puma"
-
-# line 333 "Normalization.puma"
+/* line 332 "Normalization.puma" */
  h = decl->DECL_LIST.Elem;
      decl->DECL_LIST.Elem = decl->DECL_LIST.Next->DECL_LIST.Elem;
      decl->DECL_LIST.Next->DECL_LIST.Elem = h;
@@ -503,28 +506,26 @@ static void MoveDecl
  }
 yyL1:;
 
-# line 339 "Normalization.puma"
+/* line 338 "Normalization.puma" */
  {
   tTree h;
   {
-# line 341 "Normalization.puma"
-
-# line 343 "Normalization.puma"
+/* line 342 "Normalization.puma" */
  h = decl->DECL_LIST.Elem;
      decl->DECL_LIST.Elem = decl->DECL_LIST.Next->DECL_LIST.Elem;
      decl->DECL_LIST.Next->DECL_LIST.Elem = h;
    
-# line 347 "Normalization.puma"
-   MoveDecl (id, decl->DECL_LIST.Next);
+/* line 346 "Normalization.puma" */
+   decl = decl->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
  }
 
   }
   }
-# line 350 "Normalization.puma"
+/* line 349 "Normalization.puma" */
   {
-# line 351 "Normalization.puma"
+/* line 350 "Normalization.puma" */
    failure_protocol (MODULE, "MoveDecl", decl);
   }
    return;
@@ -532,7 +533,7 @@ yyL1:;
 ;
 }
 
-static bool FoundDeclaration
+static rbool FoundDeclaration
 # if defined __STDC__ | defined __cplusplus
 (register tIdent id, register tTree list)
 # else
@@ -542,27 +543,27 @@ static bool FoundDeclaration
 # endif
 {
   if (list->Kind == kDECL_LIST) {
-# line 364 "Normalization.puma"
+/* line 363 "Normalization.puma" */
   {
-# line 365 "Normalization.puma"
+/* line 364 "Normalization.puma" */
    if (! ((IsDeclaration (id, list->DECL_LIST.Elem)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 368 "Normalization.puma"
+/* line 367 "Normalization.puma" */
   {
-# line 369 "Normalization.puma"
+/* line 368 "Normalization.puma" */
    if (! ((FoundDeclaration (id, list->DECL_LIST.Next)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
-  return false;
+  return rfalse;
 }
 
-static bool IsDeclaration
+static rbool IsDeclaration
 # if defined __STDC__ | defined __cplusplus
 (register tIdent id, register tTree decl)
 # else
@@ -572,36 +573,36 @@ static bool IsDeclaration
 # endif
 {
   if (decl->Kind == kTEMPLATE_DECL) {
-# line 380 "Normalization.puma"
+/* line 379 "Normalization.puma" */
   {
-# line 381 "Normalization.puma"
+/* line 380 "Normalization.puma" */
    if (! ((id == decl->TEMPLATE_DECL.Ident))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   if (decl->Kind == kPROCESSORS_DECL) {
-# line 384 "Normalization.puma"
+/* line 383 "Normalization.puma" */
   {
-# line 385 "Normalization.puma"
+/* line 384 "Normalization.puma" */
    if (! ((id == decl->PROCESSORS_DECL.Ident))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   if (decl->Kind == kVAR_DECL) {
-# line 388 "Normalization.puma"
+/* line 387 "Normalization.puma" */
   {
-# line 389 "Normalization.puma"
+/* line 388 "Normalization.puma" */
    if (! ((id == decl->VAR_DECL.Ident))) goto yyL3;
   }
-   return true;
+   return rtrue;
 yyL3:;
 
   }
-  return false;
+  return rfalse;
 }
 
 static void NormalShape
@@ -613,9 +614,9 @@ static void NormalShape
 # endif
 {
   if (body->Kind == kBODY_NODE) {
-# line 410 "Normalization.puma"
+/* line 409 "Normalization.puma" */
   {
-# line 412 "Normalization.puma"
+/* line 411 "Normalization.puma" */
  body->BODY_NODE.DECLS = NormalShapeAST (body->BODY_NODE.DECLS); 
       body->BODY_NODE.STATS = NormalShapeAST (body->BODY_NODE.STATS); 
     
@@ -623,9 +624,9 @@ static void NormalShape
    return;
 
   }
-# line 417 "Normalization.puma"
+/* line 416 "Normalization.puma" */
   {
-# line 419 "Normalization.puma"
+/* line 418 "Normalization.puma" */
    failure_protocol (MODULE, "NormalShape", body);
   }
    return;
@@ -641,12 +642,12 @@ static tTree NormalShapeAST
  register tTree t;
 # endif
 {
-# line 424 "Normalization.puma"
+/* line 423 "Normalization.puma" */
    return ReplaceAST (t, shape_pred, shape_func);
 
 }
 
-static bool shape_pred
+static rbool shape_pred
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -657,51 +658,55 @@ static bool shape_pred
 
   switch (t->Kind) {
   case kINDEXED_VAR:
-# line 443 "Normalization.puma"
-   return true;
+/* line 443 "Normalization.puma" */
+   return rtrue;
 
   case kSELECTED_VAR:
-# line 446 "Normalization.puma"
-   return true;
+/* line 446 "Normalization.puma" */
+   return rtrue;
 
   case kCALL_STMT:
-# line 449 "Normalization.puma"
-   return true;
+/* line 449 "Normalization.puma" */
+   return rtrue;
 
   case kIO_STMT:
-# line 452 "Normalization.puma"
-   return true;
+/* line 452 "Normalization.puma" */
+   return rtrue;
 
   case kFUNC_CALL_EXP:
-# line 455 "Normalization.puma"
-   return true;
+/* line 455 "Normalization.puma" */
+   return rtrue;
 
   case kDEALLOCATE_STMT:
-# line 458 "Normalization.puma"
-   return true;
+/* line 458 "Normalization.puma" */
+   return rtrue;
 
   case kALLOCATE_STMT:
-# line 461 "Normalization.puma"
-   return true;
+/* line 461 "Normalization.puma" */
+   return rtrue;
 
   case kCREATE_DSP_STMT:
-# line 464 "Normalization.puma"
-   return true;
+/* line 464 "Normalization.puma" */
+   return rtrue;
 
   case kINHERIT_DSP_STMT:
-# line 467 "Normalization.puma"
-   return true;
+/* line 467 "Normalization.puma" */
+   return rtrue;
 
   case kFREE_DSP_STMT:
-# line 470 "Normalization.puma"
-   return true;
+/* line 470 "Normalization.puma" */
+   return rtrue;
+
+  case kPTR_ASSIGN_STMT:
+/* line 473 "Normalization.puma" */
+   return rtrue;
 
   }
 
-# line 473 "Normalization.puma"
+/* line 476 "Normalization.puma" */
   {
-# line 474 "Normalization.puma"
-   return false;
+/* line 477 "Normalization.puma" */
+   return rfalse;
   }
 
 }
@@ -717,140 +722,146 @@ static tTree shape_func
 
   switch (t->Kind) {
   case kIO_STMT:
-# line 487 "Normalization.puma"
+/* line 490 "Normalization.puma" */
   {
-# line 489 "Normalization.puma"
+/* line 492 "Normalization.puma" */
    MakeFullShapeProc (t);
   }
    return t;
 
   case kCALL_STMT:
-# line 494 "Normalization.puma"
+/* line 497 "Normalization.puma" */
   {
-# line 496 "Normalization.puma"
+/* line 499 "Normalization.puma" */
    MakeFullShapeProc (t);
   }
    return t;
 
   case kUSED_VAR:
-# line 501 "Normalization.puma"
+/* line 504 "Normalization.puma" */
  {
   tTree new;
   {
-# line 503 "Normalization.puma"
-
-# line 505 "Normalization.puma"
+/* line 508 "Normalization.puma" */
    if (! ((TreeRank (t) > 0))) goto yyL3;
   {
-# line 507 "Normalization.puma"
+/* line 510 "Normalization.puma" */
    new = MakeFullShape (t);
   }
   }
-  {
    return new;
-  }
  }
 yyL3:;
 
   break;
   case kSELECTED_VAR:
-# line 512 "Normalization.puma"
+/* line 515 "Normalization.puma" */
   {
-# line 514 "Normalization.puma"
+/* line 517 "Normalization.puma" */
    if (! ((VarRank (t->SELECTED_VAR.SELECTOR->REC_COMP.Object) == 0))) goto yyL4;
   {
-# line 516 "Normalization.puma"
+/* line 519 "Normalization.puma" */
  t->SELECTED_VAR.SELEC_VAR = NormalShapeAST (t->SELECTED_VAR.SELEC_VAR); 
   }
   }
    return t;
 yyL4:;
 
-# line 521 "Normalization.puma"
+/* line 524 "Normalization.puma" */
  {
   tTree new;
   {
-# line 523 "Normalization.puma"
-
-# line 525 "Normalization.puma"
+/* line 528 "Normalization.puma" */
    tree_protocol ("make full shape of selected var : ", t);
-# line 527 "Normalization.puma"
+/* line 530 "Normalization.puma" */
  t->SELECTED_VAR.SELEC_VAR = NormalShapeAST (t->SELECTED_VAR.SELEC_VAR); 
-# line 529 "Normalization.puma"
+/* line 532 "Normalization.puma" */
    new = MakeFullShape (t);
-# line 531 "Normalization.puma"
+/* line 534 "Normalization.puma" */
    tree_protocol ("full shape of selected var  now : ", new);
   }
-  {
    return new;
-  }
  }
 
   case kINDEXED_VAR:
   if (t->INDEXED_VAR.IND_VAR->Kind == kUSED_VAR) {
-# line 536 "Normalization.puma"
+/* line 539 "Normalization.puma" */
  {
   tTree new;
   {
-# line 538 "Normalization.puma"
-
-# line 540 "Normalization.puma"
+/* line 543 "Normalization.puma" */
    tree_protocol ("make full shape of indexed var : ", t);
-# line 544 "Normalization.puma"
+/* line 547 "Normalization.puma" */
  t->INDEXED_VAR.IND_EXPS = NormalShapeAST (t->INDEXED_VAR.IND_EXPS); 
-# line 546 "Normalization.puma"
+/* line 549 "Normalization.puma" */
    new = MakeFullShape (t);
-# line 548 "Normalization.puma"
+/* line 551 "Normalization.puma" */
    tree_protocol ("full shape of indexed var now : ", new);
   }
-  {
    return new;
-  }
  }
 
   }
   if (t->INDEXED_VAR.IND_VAR->Kind == kSELECTED_VAR) {
-# line 553 "Normalization.puma"
+/* line 556 "Normalization.puma" */
  {
   tTree new;
   {
-# line 555 "Normalization.puma"
-
-# line 557 "Normalization.puma"
+/* line 560 "Normalization.puma" */
    tree_protocol ("make full shape of sel indexed var : ", t);
-# line 559 "Normalization.puma"
+/* line 562 "Normalization.puma" */
  t->INDEXED_VAR.IND_VAR->SELECTED_VAR.SELEC_VAR = NormalShapeAST (t->INDEXED_VAR.IND_VAR->SELECTED_VAR.SELEC_VAR);
      t->INDEXED_VAR.IND_EXPS = NormalShapeAST (t->INDEXED_VAR.IND_EXPS); 
    
-# line 563 "Normalization.puma"
+/* line 566 "Normalization.puma" */
    new = MakeFullShape (t);
-# line 565 "Normalization.puma"
+/* line 568 "Normalization.puma" */
    tree_protocol ("full shape of sel indexed var now : ", new);
   }
-  {
    return new;
-  }
  }
 
   }
   break;
   case kFUNC_CALL_EXP:
-# line 570 "Normalization.puma"
+/* line 573 "Normalization.puma" */
    return MakeFullShapeFunc (t);
 
   case kALLOCATE_STMT:
-# line 575 "Normalization.puma"
+/* line 578 "Normalization.puma" */
   {
-# line 577 "Normalization.puma"
+/* line 580 "Normalization.puma" */
  t->ALLOCATE_STMT.STATUS = NormalShapeAST (t->ALLOCATE_STMT.STATUS); 
      MakeFullAllocParams (t->ALLOCATE_STMT.PARAMS); 
    
   }
    return t;
 
+  case kPTR_ASSIGN_STMT:
+  if (t->PTR_ASSIGN_STMT.ASSIGN_VAR->Kind == kUSED_VAR) {
+/* line 587 "Normalization.puma" */
+  {
+/* line 589 "Normalization.puma" */
+ t->PTR_ASSIGN_STMT.ASSIGN_EXP = NormalShapeAST (t->PTR_ASSIGN_STMT.ASSIGN_EXP); 
+  }
+   return t;
+
+  }
+  if (t->PTR_ASSIGN_STMT.ASSIGN_VAR->Kind == kSELECTED_VAR) {
+/* line 594 "Normalization.puma" */
+  {
+/* line 596 "Normalization.puma" */
+ t->PTR_ASSIGN_STMT.ASSIGN_VAR->SELECTED_VAR.SELEC_VAR  = NormalShapeAST (t->PTR_ASSIGN_STMT.ASSIGN_VAR->SELECTED_VAR.SELEC_VAR); 
+     t->PTR_ASSIGN_STMT.ASSIGN_EXP = NormalShapeAST (t->PTR_ASSIGN_STMT.ASSIGN_EXP);
+   
+  }
+   return t;
+
+  }
+  break;
   }
 
-# line 584 "Normalization.puma"
+/* line 603 "Normalization.puma" */
    return t;
 
 }
@@ -863,26 +874,27 @@ static void MakeFullAllocParams
  register tTree params;
 # endif
 {
+ yyRecursion:
   if (params->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Elem->Kind == kVAR_PARAM) {
   if (params->BTP_LIST.Elem->VAR_PARAM.V->Kind == kUSED_VAR) {
-# line 590 "Normalization.puma"
+/* line 609 "Normalization.puma" */
   {
-# line 592 "Normalization.puma"
-   MakeFullAllocParams (params->BTP_LIST.Next);
+/* line 611 "Normalization.puma" */
+   params = params->BTP_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   }
-# line 595 "Normalization.puma"
+/* line 614 "Normalization.puma" */
   {
-# line 597 "Normalization.puma"
+/* line 616 "Normalization.puma" */
  params->BTP_LIST.Elem = NormalShapeAST (params->BTP_LIST.Elem); 
-# line 599 "Normalization.puma"
-   MakeFullAllocParams (params->BTP_LIST.Next);
+/* line 618 "Normalization.puma" */
+   params = params->BTP_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
 ;
@@ -897,39 +909,40 @@ static tTree MakeFullShapeFunc
 # endif
 {
   if (t->Kind == kFUNC_CALL_EXP) {
-# line 629 "Normalization.puma"
+/* line 648 "Normalization.puma" */
   {
-# line 631 "Normalization.puma"
+/* line 650 "Normalization.puma" */
    if (! ((IsIntrCall (t)))) goto yyL1;
   {
-# line 632 "Normalization.puma"
+/* line 651 "Normalization.puma" */
    if (! ((t->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Ident == IsIdent ("PRESENT")))) goto yyL1;
   }
   }
    return t;
 yyL1:;
 
-# line 637 "Normalization.puma"
+/* line 656 "Normalization.puma" */
   {
-# line 639 "Normalization.puma"
+/* line 658 "Normalization.puma" */
    if (! ((IsIntrCall (t)))) goto yyL2;
   {
-# line 641 "Normalization.puma"
+/* line 660 "Normalization.puma" */
  t->FUNC_CALL_EXP.FUNC_PARAMS = NormalShapeAST (t->FUNC_CALL_EXP.FUNC_PARAMS); 
   }
   }
    return TranslateInquiryCall (t);
 yyL2:;
 
-# line 646 "Normalization.puma"
+/* line 665 "Normalization.puma" */
   {
-# line 648 "Normalization.puma"
+/* line 667 "Normalization.puma" */
    NormalNotFullParam (t->FUNC_CALL_EXP.FUNC_PARAMS);
   }
    return t;
 
   }
  yyAbort ("MakeFullShapeFunc");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static void MakeFullShapeProc
@@ -941,18 +954,18 @@ static void MakeFullShapeProc
 # endif
 {
   if (t->Kind == kCALL_STMT) {
-# line 661 "Normalization.puma"
+/* line 680 "Normalization.puma" */
   {
-# line 662 "Normalization.puma"
+/* line 681 "Normalization.puma" */
    NormalNotFullParam (t->CALL_STMT.CALL_PARAMS);
   }
    return;
 
   }
   if (t->Kind == kIO_STMT) {
-# line 665 "Normalization.puma"
+/* line 684 "Normalization.puma" */
   {
-# line 666 "Normalization.puma"
+/* line 685 "Normalization.puma" */
    NormalNotFullParam (t->IO_STMT.IO_ITEMS);
   }
    return;
@@ -969,49 +982,50 @@ static void NormalNotFullParam
  register tTree t;
 # endif
 {
+ yyRecursion:
   if (t->Kind == kBTP_LIST) {
-# line 677 "Normalization.puma"
+/* line 696 "Normalization.puma" */
   {
-# line 678 "Normalization.puma"
+/* line 697 "Normalization.puma" */
    NormalNotFullParam (t->BTP_LIST.Elem);
-# line 679 "Normalization.puma"
-   NormalNotFullParam (t->BTP_LIST.Next);
+/* line 698 "Normalization.puma" */
+   t = t->BTP_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kBTP_EMPTY) {
-# line 682 "Normalization.puma"
+/* line 701 "Normalization.puma" */
    return;
 
   }
   if (t->Kind == kVAR_PARAM) {
-# line 697 "Normalization.puma"
+/* line 716 "Normalization.puma" */
   {
-# line 699 "Normalization.puma"
+/* line 718 "Normalization.puma" */
  t->VAR_PARAM.V = NormalShapeAST (t->VAR_PARAM.V); 
   }
    return;
 
   }
   if (t->Kind == kFUNC_PARAM) {
-# line 702 "Normalization.puma"
+/* line 721 "Normalization.puma" */
    return;
 
   }
   if (t->Kind == kPROC_PARAM) {
-# line 705 "Normalization.puma"
+/* line 724 "Normalization.puma" */
    return;
 
   }
   if (t->Kind == kNO_PARAM) {
-# line 708 "Normalization.puma"
+/* line 727 "Normalization.puma" */
    return;
 
   }
-# line 711 "Normalization.puma"
+/* line 730 "Normalization.puma" */
   {
-# line 712 "Normalization.puma"
+/* line 731 "Normalization.puma" */
    failure_protocol (MODULE, "NormalNotFullParam", t);
   }
    return;
@@ -1027,12 +1041,12 @@ static tTree MakeRaggedArrays
  register tTree t;
 # endif
 {
-# line 718 "Normalization.puma"
+/* line 737 "Normalization.puma" */
    return ReplaceAST (t, ragged_pred, ragged_func);
 
 }
 
-static bool ragged_pred
+static rbool ragged_pred
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -1040,27 +1054,27 @@ static bool ragged_pred
  register tTree t;
 # endif
 {
-# line 724 "Normalization.puma"
+/* line 743 "Normalization.puma" */
   {
-# line 725 "Normalization.puma"
+/* line 744 "Normalization.puma" */
    if (! ((IsTreeAccess (t)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   if (t->Kind == kACF_BASIC) {
   if (t->ACF_BASIC.BASIC_STMT->Kind == kALLOCATE_STMT) {
-# line 728 "Normalization.puma"
-   return true;
+/* line 747 "Normalization.puma" */
+   return rtrue;
 
   }
   if (t->ACF_BASIC.BASIC_STMT->Kind == kDEALLOCATE_STMT) {
-# line 731 "Normalization.puma"
-   return true;
+/* line 750 "Normalization.puma" */
+   return rtrue;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
 static tTree ragged_func
@@ -1072,19 +1086,19 @@ static tTree ragged_func
 # endif
 {
   if (t->Kind == kVAR_DECL) {
-# line 736 "Normalization.puma"
+/* line 755 "Normalization.puma" */
    return MakeRaggedDecls (t, GetLocalObject (t->VAR_DECL.Ident));
 
   }
   if (t->Kind == kTREE_DECL) {
-# line 741 "Normalization.puma"
+/* line 760 "Normalization.puma" */
    return NoTree;
 
   }
   if (Tree_IsType (t, kBT_VAR)) {
-# line 746 "Normalization.puma"
+/* line 765 "Normalization.puma" */
   {
-# line 748 "Normalization.puma"
+/* line 767 "Normalization.puma" */
    if (! ((IsTreeAccess (t)))) goto yyL3;
   }
    return MakeRaggedAccess (t);
@@ -1093,25 +1107,25 @@ yyL3:;
   }
   if (t->Kind == kACF_BASIC) {
   if (t->ACF_BASIC.BASIC_STMT->Kind == kALLOCATE_STMT) {
-# line 753 "Normalization.puma"
+/* line 772 "Normalization.puma" */
    return MakeRaggedAllocates (t);
 
   }
   if (t->ACF_BASIC.BASIC_STMT->Kind == kDEALLOCATE_STMT) {
-# line 758 "Normalization.puma"
+/* line 777 "Normalization.puma" */
    return MakeRaggedDeallocates (t);
 
   }
   }
-# line 763 "Normalization.puma"
+/* line 782 "Normalization.puma" */
    return t;
 
 }
 
-void BeginNormalization ()
+void BeginNormalization ARGS ((void))
 {
 }
 
-void CloseNormalization ()
+void CloseNormalization ARGS ((void))
 {
 }
