@@ -1,6 +1,13 @@
 # include "OverlapUpdate.h"
-# include "yyOverlapUpdate.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,33 +15,30 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 58 "OverlapUpdate.puma"
+/* line 58 "OverlapUpdate.puma" */
 
 
 # define MODULE "OverlapUpdate"
 
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 # include "DynArray.h"
 # include "protocol.h"
 
@@ -52,8 +56,8 @@
 *                                                                    * 
 *********************************************************************/
 
-static bool EqualSections ARGS((tTree v1, tTree v2));
-static bool IsSubSection ARGS((tTree v1, tTree v2));
+static rbool EqualSections ARGS((tTree v1, tTree v2));
+static rbool IsSubSection ARGS((tTree v1, tTree v2));
 static tTree MakeOverlapReset ARGS((tTree var));
 static tTree UnionSection ARGS((tTree v1, tTree v2));
  
@@ -115,13 +119,13 @@ static void InitUpdates ()
        *                                                     * 
        ******************************************************/
 
-static bool IsAnUpdate (sv)
+static rbool IsAnUpdate (sv)
 pshift sv;
 
-{ bool is;
+{ rbool is;
   int  i;
 
-  is = false;
+  is = rfalse;
   for (i=0; i<sv->shift_rank; i++)
      is = is || (sv->shift_pos[i] != 0);
  
@@ -173,27 +177,27 @@ overlap_update *op;
        *                                                      * 
        *  IsSmallerOverlap (overlap_update *op1, *op2)        *
        *                                                      * 
-       *   - returns true if op2 updates bigger than op1      * 
-       *   - also false if no analysis is possible            * 
+       *   - returns rtrue if op2 updates bigger than op1      * 
+       *   - also rfalse if no analysis is possible            * 
        *                                                      * 
        *******************************************************/
 
-static bool IsSmallerOverlap (op1, op2)
+static rbool IsSmallerOverlap (op1, op2)
 overlap_update *op1;
 overlap_update *op2;
 
-{ bool is;
+{ rbool is;
   int i, rank;
 
   if (!IsSubSection (op1->var, op2->var))
-     return (false);
+     return (rfalse);
 
   rank = op1->rank;
-  is   = true;
+  is   = rtrue;
 
   for (i=0; i<rank; i++)
-    { if (op1->left[i]  > op2->left[i])  is = false;
-      if (op1->right[i] > op2->right[i]) is = false;
+    { if (op1->left[i]  > op2->left[i])  is = rfalse;
+      if (op1->right[i] > op2->right[i]) is = rfalse;
     }
 
   return (is);
@@ -204,7 +208,7 @@ overlap_update *op2;
        *                                                      * 
        *  OrthogonalOverlaps (overlap_update *op1, *op2)      *
        *                                                      * 
-       *   - returns true if op1 and op2 have no              * 
+       *   - returns rtrue if op1 and op2 have no              * 
        *     overlap dimension in common                      * 
        *                                                      * 
        *   (1:1,0:0) and (0:0,1:1) are orthogonal             * 
@@ -212,32 +216,32 @@ overlap_update *op2;
        *                                                      * 
        *******************************************************/
 
-static bool OrthogonalOverlaps (op1, op2)
+static rbool OrthogonalOverlaps (op1, op2)
 overlap_update *op1;
 overlap_update *op2;
 
-{ bool is, nonzero1, nonzero2;
+{ rbool is, nonzero1, nonzero2;
   int i, rank;
 
   /* different variables are orthogonal in any case */
 
   if (TreeVarName (op1->var) != TreeVarName (op2->var)) 
-     return true;
+     return rtrue;
 
   /*
   if (!EqualSections (op1->var, op2->var))
-     return (true);
+     return (rtrue);
   */
 
   rank = op1->rank;
-  is   = true;
+  is   = rtrue;
 
   for (i=0; i<rank; i++)
 
     { nonzero1 = (op1->right[i] != 0) || (op1->left[i] != 0);
       nonzero2 = (op2->right[i] != 0) || (op2->left[i] != 0);
 
-      if (nonzero1 && nonzero2) is = false;
+      if (nonzero1 && nonzero2) is = rfalse;
     }
 
   return (is);
@@ -282,19 +286,19 @@ overlap_update *op2;
        *                                                     * 
        ******************************************************/
 
-static bool IsExistingOverlap (op)
+static rbool IsExistingOverlap (op)
 
 overlap_update *op;
 
 { int  i;
-  bool found;
+  rbool found;
 
-  found = false;
+  found = rfalse;
   i     = 0;
 
   while ( (!found) && (i<UpdateCounter) )
 
-    { /* found becomes true if it can be proved that the
+    { /* found becomes rtrue if it can be proved that the
          new overlap op does not already exist           */
 
       found = IsSmallerOverlap (op, all_updates+i);
@@ -319,9 +323,9 @@ static void CompressExistingOverlaps (op)
 overlap_update *op;
 
 { int  i, j;
-  bool found;
+  rbool found;
 
-  found = false;
+  found = rfalse;
   i     = 0;
 
   while ( (!found) && (i<UpdateCounter) )
@@ -461,7 +465,7 @@ static tTree GetArrayUpdates ()
 static tTree GetArrayResets ()
 
 { tTree reset, reset_list, stm;
-  bool found;
+  rbool found;
 
   overlap_update *ptr, *ptr1;
 
@@ -475,10 +479,10 @@ static tTree GetArrayResets ()
      { /* check whether there is another entry for this variable */
 
        ptr1 = ptr + 1;
-       found = false;
+       found = rfalse;
        for (j=i+1; j<UpdateCounter; j++)
           if (TreeVarName (ptr->var) == TreeVarName ((ptr1++)->var))
-             found = true;
+             found = rtrue;
 
        if (!found)
          reset_list = CombineACF (reset_list, MakeOverlapReset (ptr->var));
@@ -492,11 +496,25 @@ static tTree GetArrayResets ()
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* OverlapUpdate_Exit) () = yyExit;
+# include "yyOverlapUpdate.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* OverlapUpdate_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -505,44 +523,45 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module OverlapUpdate, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module OverlapUpdate, routine %s failed\n",
+  yyFunction);
  OverlapUpdate_Exit ();
 }
 
-void InitOverlapUpdates ARGS(());
-tTree GetOverlapUpdates ARGS(());
-tTree GetOverlapResets ARGS(());
-void SetOverlapUpdate ARGS((tTree v, pshift sv));
-void SetShiftOverlapUpdate ARGS((tTree v, pshift sv));
-bool IsOverlapUpdate ARGS((pshift sv));
-static void ShiftSection ARGS((tTree v, pshift sv));
-static void ShiftIndexes ARGS((tTree indexes, pshift sv, int pos));
-static void ShiftSection1 ARGS((tTree v, pshift sv));
-static void ShiftIndexes1 ARGS((tTree indexes, pshift sv, int pos));
-static void NormalizeSection ARGS((tTree exp));
-static bool EqualSections ARGS((tTree v1, tTree v2));
-static bool IsSubSection ARGS((tTree v1, tTree v2));
-static bool SubIndexes ARGS((tTree indexes1, tTree indexes2));
-static bool SubExpression ARGS((tTree e1, tTree e2));
-static bool IsSimpleExpression ARGS((tTree e));
-static bool ExpLE ARGS((tTree e1, tTree e2));
-static void CommonSliceIncrement ARGS((tTree slice1, tTree slice2, bool * yyP2, int * yyP1));
-static tTree UnionSection ARGS((tTree v1, tTree v2));
-static void UnionIndexes ARGS((tTree indexes1, tTree indexes2));
-static tTree UnionSlices ARGS((tTree e1, tTree e2));
-static tTree MakeOverlapReset ARGS((tTree var));
-void VectorizeOverlapUpdates ARGS((tTree id, tTree slice));
+void InitOverlapUpdates ARGS ((void));
+tTree GetOverlapUpdates ARGS ((void));
+tTree GetOverlapResets ARGS ((void));
+void SetOverlapUpdate ARGS ((tTree v, pshift sv));
+void SetShiftOverlapUpdate ARGS ((tTree v, pshift sv));
+rbool IsOverlapUpdate ARGS ((pshift sv));
+static void ShiftSection ARGS ((tTree v, pshift sv));
+static void ShiftIndexes ARGS ((tTree indexes, pshift sv, int pos));
+static void ShiftSection1 ARGS ((tTree v, pshift sv));
+static void ShiftIndexes1 ARGS ((tTree indexes, pshift sv, int pos));
+static void NormalizeSection ARGS ((tTree exp));
+static rbool EqualSections ARGS ((tTree v1, tTree v2));
+static rbool IsSubSection ARGS ((tTree v1, tTree v2));
+static rbool SubIndexes ARGS ((tTree indexes1, tTree indexes2));
+static rbool SubExpression ARGS ((tTree e1, tTree e2));
+static rbool IsSimpleExpression ARGS ((tTree e));
+static rbool ExpLE ARGS ((tTree e1, tTree e2));
+static void CommonSliceIncrement ARGS ((tTree slice1, tTree slice2, rbool * yyP2, int * yyP1));
+static tTree UnionSection ARGS ((tTree v1, tTree v2));
+static void UnionIndexes ARGS ((tTree indexes1, tTree indexes2));
+static tTree UnionSlices ARGS ((tTree e1, tTree e2));
+static tTree MakeOverlapReset ARGS ((tTree var));
+void VectorizeOverlapUpdates ARGS ((tTree id, tTree slice));
 
 void InitOverlapUpdates
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 530 "OverlapUpdate.puma"
+/* line 530 "OverlapUpdate.puma" */
   {
-# line 531 "OverlapUpdate.puma"
+/* line 531 "OverlapUpdate.puma" */
    InitUpdates ();
   }
    return;
@@ -552,24 +571,24 @@ void InitOverlapUpdates
 
 tTree GetOverlapUpdates
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 536 "OverlapUpdate.puma"
+/* line 536 "OverlapUpdate.puma" */
    return GetArrayUpdates ();
 
 }
 
 tTree GetOverlapResets
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 542 "OverlapUpdate.puma"
+/* line 542 "OverlapUpdate.puma" */
    return GetArrayResets ();
 
 }
@@ -584,32 +603,30 @@ void SetOverlapUpdate
 # endif
 {
   if (v->Kind == kINDEXED_VAR) {
-# line 548 "OverlapUpdate.puma"
+/* line 548 "OverlapUpdate.puma" */
  {
   tTree new_var;
   {
-# line 550 "OverlapUpdate.puma"
-
-# line 552 "OverlapUpdate.puma"
+/* line 552 "OverlapUpdate.puma" */
    new_var = CopyTree (v);
-# line 553 "OverlapUpdate.puma"
+/* line 553 "OverlapUpdate.puma" */
    NormalizeSection (new_var);
-# line 555 "OverlapUpdate.puma"
+/* line 555 "OverlapUpdate.puma" */
    tree_protocol ("SetOverlapUpdate, var = ", new_var);
-# line 557 "OverlapUpdate.puma"
+/* line 557 "OverlapUpdate.puma" */
    ShiftSection (new_var, sv);
-# line 559 "OverlapUpdate.puma"
+/* line 559 "OverlapUpdate.puma" */
    tree_protocol ("SetOverlapUpdate, shifted = ", new_var);
-# line 561 "OverlapUpdate.puma"
+/* line 561 "OverlapUpdate.puma" */
    SetUpdate (new_var, sv);
   }
    return;
  }
 
   }
-# line 564 "OverlapUpdate.puma"
+/* line 564 "OverlapUpdate.puma" */
   {
-# line 565 "OverlapUpdate.puma"
+/* line 565 "OverlapUpdate.puma" */
    failure_protocol (MODULE, "SetOverlapUpdate", v);
   }
    return;
@@ -627,30 +644,28 @@ void SetShiftOverlapUpdate
 # endif
 {
   if (v->Kind == kINDEXED_VAR) {
-# line 570 "OverlapUpdate.puma"
+/* line 570 "OverlapUpdate.puma" */
  {
   tTree new_var;
   {
-# line 572 "OverlapUpdate.puma"
-
-# line 574 "OverlapUpdate.puma"
+/* line 574 "OverlapUpdate.puma" */
    new_var = CopyTree (v);
-# line 576 "OverlapUpdate.puma"
+/* line 576 "OverlapUpdate.puma" */
    tree_protocol ("SetShiftOverlapUpdate, var = ", new_var);
-# line 578 "OverlapUpdate.puma"
+/* line 578 "OverlapUpdate.puma" */
    ShiftSection1 (new_var, sv);
-# line 580 "OverlapUpdate.puma"
+/* line 580 "OverlapUpdate.puma" */
    tree_protocol ("SetShiftOverlapUpdate, shifted = ", new_var);
-# line 582 "OverlapUpdate.puma"
+/* line 582 "OverlapUpdate.puma" */
    SetUpdate (new_var, sv);
   }
    return;
  }
 
   }
-# line 585 "OverlapUpdate.puma"
+/* line 585 "OverlapUpdate.puma" */
   {
-# line 586 "OverlapUpdate.puma"
+/* line 586 "OverlapUpdate.puma" */
    failure_protocol (MODULE, "SetShiftOverlapUpdate", v);
   }
    return;
@@ -658,7 +673,7 @@ void SetShiftOverlapUpdate
 ;
 }
 
-bool IsOverlapUpdate
+rbool IsOverlapUpdate
 # if defined __STDC__ | defined __cplusplus
 (pshift sv)
 # else
@@ -666,15 +681,15 @@ bool IsOverlapUpdate
  pshift sv;
 # endif
 {
-# line 591 "OverlapUpdate.puma"
+/* line 591 "OverlapUpdate.puma" */
   {
-# line 592 "OverlapUpdate.puma"
+/* line 592 "OverlapUpdate.puma" */
    if (! ((IsAnUpdate (sv)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
 static void ShiftSection
@@ -687,17 +702,17 @@ static void ShiftSection
 # endif
 {
   if (v->Kind == kINDEXED_VAR) {
-# line 603 "OverlapUpdate.puma"
+/* line 603 "OverlapUpdate.puma" */
   {
-# line 604 "OverlapUpdate.puma"
+/* line 604 "OverlapUpdate.puma" */
    ShiftIndexes (v->INDEXED_VAR.IND_EXPS, sv, 0);
   }
    return;
 
   }
-# line 607 "OverlapUpdate.puma"
+/* line 607 "OverlapUpdate.puma" */
   {
-# line 608 "OverlapUpdate.puma"
+/* line 608 "OverlapUpdate.puma" */
    failure_protocol (MODULE, "ShiftSection", v);
   }
    return;
@@ -715,25 +730,27 @@ static void ShiftIndexes
  register int pos;
 # endif
 {
+ yyRecursion:
   if (indexes->Kind == kBTE_EMPTY) {
-# line 614 "OverlapUpdate.puma"
+/* line 614 "OverlapUpdate.puma" */
    return;
 
   }
   if (indexes->Kind == kBTE_LIST) {
-# line 617 "OverlapUpdate.puma"
+/* line 617 "OverlapUpdate.puma" */
   {
-# line 619 "OverlapUpdate.puma"
+/* line 619 "OverlapUpdate.puma" */
  indexes->BTE_LIST.Elem = AddConstant (indexes->BTE_LIST.Elem, sv->shift_pos[pos]); 
-# line 621 "OverlapUpdate.puma"
-   ShiftIndexes (indexes->BTE_LIST.Next, sv, pos + 1);
+/* line 621 "OverlapUpdate.puma" */
+   indexes = indexes->BTE_LIST.Next;
+   pos = pos + 1;
+   goto yyRecursion;
   }
-   return;
 
   }
-# line 624 "OverlapUpdate.puma"
+/* line 624 "OverlapUpdate.puma" */
   {
-# line 625 "OverlapUpdate.puma"
+/* line 625 "OverlapUpdate.puma" */
    failure_protocol (MODULE, "ShiftIndexes", indexes);
   }
    return;
@@ -751,17 +768,17 @@ static void ShiftSection1
 # endif
 {
   if (v->Kind == kINDEXED_VAR) {
-# line 638 "OverlapUpdate.puma"
+/* line 638 "OverlapUpdate.puma" */
   {
-# line 639 "OverlapUpdate.puma"
+/* line 639 "OverlapUpdate.puma" */
    ShiftIndexes1 (v->INDEXED_VAR.IND_EXPS, sv, 0);
   }
    return;
 
   }
-# line 642 "OverlapUpdate.puma"
+/* line 642 "OverlapUpdate.puma" */
   {
-# line 643 "OverlapUpdate.puma"
+/* line 643 "OverlapUpdate.puma" */
    failure_protocol (MODULE, "ShiftSection1", v);
   }
    return;
@@ -779,69 +796,69 @@ static void ShiftIndexes1
  register int pos;
 # endif
 {
+ yyRecursion:
   if (indexes->Kind == kBTE_EMPTY) {
-# line 649 "OverlapUpdate.puma"
+/* line 649 "OverlapUpdate.puma" */
    return;
 
   }
   if (indexes->Kind == kBTE_LIST) {
   if (indexes->BTE_LIST.Elem->Kind == kSLICE_EXP) {
-# line 652 "OverlapUpdate.puma"
+/* line 652 "OverlapUpdate.puma" */
  {
   int shift;
   {
-# line 654 "OverlapUpdate.puma"
-
-# line 656 "OverlapUpdate.puma"
+/* line 656 "OverlapUpdate.puma" */
  shift = sv->shift_pos[pos]; 
-# line 658 "OverlapUpdate.puma"
+/* line 658 "OverlapUpdate.puma" */
    if (! ((shift > 0))) goto yyL2;
   {
-# line 660 "OverlapUpdate.puma"
+/* line 660 "OverlapUpdate.puma" */
  indexes->BTE_LIST.Elem->SLICE_EXP.STOP = AddConstant (indexes->BTE_LIST.Elem->SLICE_EXP.STOP, shift); 
-# line 662 "OverlapUpdate.puma"
-   ShiftIndexes1 (indexes->BTE_LIST.Next, sv, pos + 1);
+/* line 662 "OverlapUpdate.puma" */
+   indexes = indexes->BTE_LIST.Next;
+   pos = pos + 1;
+   goto yyRecursion;
   }
   }
-   return;
  }
 yyL2:;
 
-# line 665 "OverlapUpdate.puma"
+/* line 665 "OverlapUpdate.puma" */
  {
   int shift;
   {
-# line 667 "OverlapUpdate.puma"
-
-# line 669 "OverlapUpdate.puma"
+/* line 669 "OverlapUpdate.puma" */
  shift = sv->shift_pos[pos]; 
-# line 671 "OverlapUpdate.puma"
+/* line 671 "OverlapUpdate.puma" */
    if (! ((shift < 0))) goto yyL3;
   {
-# line 673 "OverlapUpdate.puma"
- indexes->BTE_LIST.Elem->SLICE_EXP.START = AddConstant (indexes->BTE_LIST.Elem->SLICE_EXP.START, shift); 
-# line 675 "OverlapUpdate.puma"
-   ShiftIndexes1 (indexes->BTE_LIST.Next, sv, pos + 1);
+/* line 673 "OverlapUpdate.puma" */
+ indexes->BTE_LIST.Elem->SLICE_EXP.FIRST = AddConstant (indexes->BTE_LIST.Elem->SLICE_EXP.FIRST, shift); 
+/* line 675 "OverlapUpdate.puma" */
+   indexes = indexes->BTE_LIST.Next;
+   pos = pos + 1;
+   goto yyRecursion;
   }
   }
-   return;
  }
 yyL3:;
 
   }
-# line 678 "OverlapUpdate.puma"
+/* line 678 "OverlapUpdate.puma" */
   {
-# line 680 "OverlapUpdate.puma"
+/* line 680 "OverlapUpdate.puma" */
  indexes->BTE_LIST.Elem = AddConstant (indexes->BTE_LIST.Elem, sv->shift_pos[pos]); 
-# line 682 "OverlapUpdate.puma"
-   ShiftIndexes1 (indexes->BTE_LIST.Next, sv, pos + 1);
+/* line 682 "OverlapUpdate.puma" */
+   indexes = indexes->BTE_LIST.Next;
+   pos = pos + 1;
+   goto yyRecursion;
   }
-   return;
 
   }
-# line 685 "OverlapUpdate.puma"
+/* line 685 "OverlapUpdate.puma" */
   {
-# line 686 "OverlapUpdate.puma"
+/* line 686 "OverlapUpdate.puma" */
    failure_protocol (MODULE, "ShiftIndexes", indexes);
   }
    return;
@@ -857,44 +874,41 @@ static void NormalizeSection
  register tTree exp;
 # endif
 {
+ yyRecursion:
   if (exp->Kind == kINDEXED_VAR) {
-# line 702 "OverlapUpdate.puma"
+/* line 702 "OverlapUpdate.puma" */
   {
-# line 704 "OverlapUpdate.puma"
+/* line 704 "OverlapUpdate.puma" */
    NormalizeSection (exp->INDEXED_VAR.IND_EXPS);
-# line 705 "OverlapUpdate.puma"
+/* line 705 "OverlapUpdate.puma" */
    MakeFullShape (exp);
   }
    return;
 
   }
   if (exp->Kind == kBTE_LIST) {
-# line 708 "OverlapUpdate.puma"
+/* line 708 "OverlapUpdate.puma" */
   {
-# line 709 "OverlapUpdate.puma"
+/* line 709 "OverlapUpdate.puma" */
    NormalizeSection (exp->BTE_LIST.Elem);
-# line 710 "OverlapUpdate.puma"
-   NormalizeSection (exp->BTE_LIST.Next);
+/* line 710 "OverlapUpdate.puma" */
+   exp = exp->BTE_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (exp->Kind == kSLICE_EXP) {
-# line 713 "OverlapUpdate.puma"
+/* line 713 "OverlapUpdate.puma" */
  {
-  bool found;
+  rbool found;
   int val;
   {
-# line 715 "OverlapUpdate.puma"
-
-# line 716 "OverlapUpdate.puma"
-
-# line 718 "OverlapUpdate.puma"
+/* line 718 "OverlapUpdate.puma" */
    SliceIncrement (exp, & found, & val);
-# line 720 "OverlapUpdate.puma"
+/* line 720 "OverlapUpdate.puma" */
  if (!found)
  
-        { exp->SLICE_EXP.START  = mDUMMY_EXP();
+        { exp->SLICE_EXP.FIRST  = mDUMMY_EXP();
           exp->SLICE_EXP.STOP  = mDUMMY_EXP();
           exp->SLICE_EXP.INC = mDUMMY_EXP();
         }
@@ -902,8 +916,8 @@ static void NormalizeSection
       else if (val < 0)
 
         { tTree h;
-          h   = exp->SLICE_EXP.START;
-          exp->SLICE_EXP.START  = exp->SLICE_EXP.STOP;
+          h   = exp->SLICE_EXP.FIRST;
+          exp->SLICE_EXP.FIRST  = exp->SLICE_EXP.STOP;
           exp->SLICE_EXP.STOP  = h;
           exp->SLICE_EXP.INC = MinusExpression (exp->SLICE_EXP.INC);
         }
@@ -916,7 +930,7 @@ static void NormalizeSection
 ;
 }
 
-static bool EqualSections
+static rbool EqualSections
 # if defined __STDC__ | defined __cplusplus
 (register tTree v1, register tTree v2)
 # else
@@ -927,24 +941,24 @@ static bool EqualSections
 {
   if (v1->Kind == kINDEXED_VAR) {
   if (v2->Kind == kINDEXED_VAR) {
-# line 748 "OverlapUpdate.puma"
+/* line 748 "OverlapUpdate.puma" */
   {
-# line 749 "OverlapUpdate.puma"
+/* line 749 "OverlapUpdate.puma" */
    if (! ((TreeVarName (v1->INDEXED_VAR.IND_VAR) == TreeVarName (v2->INDEXED_VAR.IND_VAR)))) goto yyL1;
   {
-# line 750 "OverlapUpdate.puma"
+/* line 750 "OverlapUpdate.puma" */
    if (! ((EqualIndexes (v1->INDEXED_VAR.IND_EXPS, v2->INDEXED_VAR.IND_EXPS)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
-static bool IsSubSection
+static rbool IsSubSection
 # if defined __STDC__ | defined __cplusplus
 (register tTree v1, register tTree v2)
 # else
@@ -955,24 +969,24 @@ static bool IsSubSection
 {
   if (v1->Kind == kINDEXED_VAR) {
   if (v2->Kind == kINDEXED_VAR) {
-# line 763 "OverlapUpdate.puma"
+/* line 763 "OverlapUpdate.puma" */
   {
-# line 764 "OverlapUpdate.puma"
+/* line 764 "OverlapUpdate.puma" */
    if (! ((TreeVarName (v1->INDEXED_VAR.IND_VAR) == TreeVarName (v2->INDEXED_VAR.IND_VAR)))) goto yyL1;
   {
-# line 765 "OverlapUpdate.puma"
+/* line 765 "OverlapUpdate.puma" */
    if (! ((SubIndexes (v1->INDEXED_VAR.IND_EXPS, v2->INDEXED_VAR.IND_EXPS)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
-static bool SubIndexes
+static rbool SubIndexes
 # if defined __STDC__ | defined __cplusplus
 (register tTree indexes1, register tTree indexes2)
 # else
@@ -983,31 +997,31 @@ static bool SubIndexes
 {
   if (indexes1->Kind == kBTE_LIST) {
   if (indexes2->Kind == kBTE_LIST) {
-# line 770 "OverlapUpdate.puma"
+/* line 770 "OverlapUpdate.puma" */
   {
-# line 771 "OverlapUpdate.puma"
+/* line 771 "OverlapUpdate.puma" */
    if (! ((SubExpression (indexes1->BTE_LIST.Elem, indexes2->BTE_LIST.Elem)))) goto yyL1;
   {
-# line 772 "OverlapUpdate.puma"
+/* line 772 "OverlapUpdate.puma" */
    if (! ((SubIndexes (indexes1->BTE_LIST.Next, indexes2->BTE_LIST.Next)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   }
   if (indexes1->Kind == kBTE_EMPTY) {
   if (indexes2->Kind == kBTE_EMPTY) {
-# line 775 "OverlapUpdate.puma"
-   return true;
+/* line 775 "OverlapUpdate.puma" */
+   return rtrue;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
-static bool SubExpression
+static rbool SubExpression
 # if defined __STDC__ | defined __cplusplus
 (register tTree e1, register tTree e2)
 # else
@@ -1018,90 +1032,90 @@ static bool SubExpression
 {
   if (e1->Kind == kSLICE_EXP) {
   if (e2->Kind == kSLICE_EXP) {
-# line 780 "OverlapUpdate.puma"
+/* line 780 "OverlapUpdate.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   int yyV2;
   {
-# line 784 "OverlapUpdate.puma"
+/* line 784 "OverlapUpdate.puma" */
    CommonSliceIncrement (e1, e2, & yyV1, & yyV2);
-# line 786 "OverlapUpdate.puma"
+/* line 786 "OverlapUpdate.puma" */
    if (! ((yyV1))) goto yyL1;
   {
-# line 787 "OverlapUpdate.puma"
+/* line 787 "OverlapUpdate.puma" */
    if (! ((yyV2 > 0))) goto yyL1;
   {
-# line 789 "OverlapUpdate.puma"
-   if (! ((ExpLE (e2->SLICE_EXP.START, e1->SLICE_EXP.START)))) goto yyL1;
+/* line 789 "OverlapUpdate.puma" */
+   if (! ((ExpLE (e2->SLICE_EXP.FIRST, e1->SLICE_EXP.FIRST)))) goto yyL1;
   {
-# line 790 "OverlapUpdate.puma"
+/* line 790 "OverlapUpdate.puma" */
    if (! ((ExpLE (e1->SLICE_EXP.STOP, e2->SLICE_EXP.STOP)))) goto yyL1;
   }
   }
   }
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-# line 793 "OverlapUpdate.puma"
+/* line 793 "OverlapUpdate.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   int yyV2;
   {
-# line 797 "OverlapUpdate.puma"
+/* line 797 "OverlapUpdate.puma" */
    CommonSliceIncrement (e1, e2, & yyV1, & yyV2);
-# line 799 "OverlapUpdate.puma"
+/* line 799 "OverlapUpdate.puma" */
    if (! ((yyV1))) goto yyL2;
   {
-# line 800 "OverlapUpdate.puma"
+/* line 800 "OverlapUpdate.puma" */
    if (! ((yyV2 < 0))) goto yyL2;
   {
-# line 802 "OverlapUpdate.puma"
-   if (! ((ExpLE (e1->SLICE_EXP.START, e2->SLICE_EXP.START)))) goto yyL2;
+/* line 802 "OverlapUpdate.puma" */
+   if (! ((ExpLE (e1->SLICE_EXP.FIRST, e2->SLICE_EXP.FIRST)))) goto yyL2;
   {
-# line 803 "OverlapUpdate.puma"
+/* line 803 "OverlapUpdate.puma" */
    if (! ((ExpLE (e2->SLICE_EXP.STOP, e1->SLICE_EXP.STOP)))) goto yyL2;
   }
   }
   }
   }
-   return true;
+   return rtrue;
  }
 yyL2:;
 
   }
   }
   if (e2->Kind == kSLICE_EXP) {
-# line 808 "OverlapUpdate.puma"
+/* line 808 "OverlapUpdate.puma" */
   {
-# line 809 "OverlapUpdate.puma"
+/* line 809 "OverlapUpdate.puma" */
    if (! ((IsSimpleExpression (e1)))) goto yyL3;
   {
-# line 810 "OverlapUpdate.puma"
-   if (! ((ExpLE (e2->SLICE_EXP.START, e1)))) goto yyL3;
+/* line 810 "OverlapUpdate.puma" */
+   if (! ((ExpLE (e2->SLICE_EXP.FIRST, e1)))) goto yyL3;
   {
-# line 811 "OverlapUpdate.puma"
+/* line 811 "OverlapUpdate.puma" */
    if (! ((ExpLE (e1, e2->SLICE_EXP.STOP)))) goto yyL3;
   }
   }
   }
-   return true;
+   return rtrue;
 yyL3:;
 
   }
-# line 814 "OverlapUpdate.puma"
+/* line 814 "OverlapUpdate.puma" */
   {
-# line 815 "OverlapUpdate.puma"
+/* line 815 "OverlapUpdate.puma" */
    if (! ((EqualExpression (e1, e2)))) goto yyL4;
   }
-   return true;
+   return rtrue;
 yyL4:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsSimpleExpression
+static rbool IsSimpleExpression
 # if defined __STDC__ | defined __cplusplus
 (register tTree e)
 # else
@@ -1110,19 +1124,19 @@ static bool IsSimpleExpression
 # endif
 {
   if (e->Kind == kSLICE_EXP) {
-# line 820 "OverlapUpdate.puma"
+/* line 820 "OverlapUpdate.puma" */
   {
-# line 821 "OverlapUpdate.puma"
-   return false;
+/* line 821 "OverlapUpdate.puma" */
+   return rfalse;
   }
 
   }
-# line 824 "OverlapUpdate.puma"
-   return true;
+/* line 824 "OverlapUpdate.puma" */
+   return rtrue;
 
 }
 
-static bool ExpLE
+static rbool ExpLE
 # if defined __STDC__ | defined __cplusplus
 (register tTree e1, register tTree e2)
 # else
@@ -1131,65 +1145,55 @@ static bool ExpLE
  register tTree e2;
 # endif
 {
-# line 829 "OverlapUpdate.puma"
+/* line 829 "OverlapUpdate.puma" */
  {
-  bool found;
+  rbool found;
   int val;
   {
-# line 831 "OverlapUpdate.puma"
-
-# line 832 "OverlapUpdate.puma"
-
-# line 834 "OverlapUpdate.puma"
+/* line 834 "OverlapUpdate.puma" */
    GetConstDifference (e1, e2, & found, & val);
-# line 835 "OverlapUpdate.puma"
+/* line 835 "OverlapUpdate.puma" */
    if (! ((found))) goto yyL1;
   {
-# line 836 "OverlapUpdate.puma"
+/* line 836 "OverlapUpdate.puma" */
    if (! ((val <= 0))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
 static void CommonSliceIncrement
 # if defined __STDC__ | defined __cplusplus
-(register tTree slice1, register tTree slice2, register bool * yyP2, register int * yyP1)
+(register tTree slice1, register tTree slice2, register rbool * yyP2, register int * yyP1)
 # else
 (slice1, slice2, yyP2, yyP1)
  register tTree slice1;
  register tTree slice2;
- register bool * yyP2;
+ register rbool * yyP2;
  register int * yyP1;
 # endif
 {
-# line 841 "OverlapUpdate.puma"
+/* line 841 "OverlapUpdate.puma" */
  {
-  bool found;
+  rbool found;
   int val1;
   int val2;
   {
-# line 843 "OverlapUpdate.puma"
-
-# line 844 "OverlapUpdate.puma"
-
-# line 845 "OverlapUpdate.puma"
-
-# line 847 "OverlapUpdate.puma"
+/* line 847 "OverlapUpdate.puma" */
    SliceIncrement (slice1, & found, & val1);
-# line 848 "OverlapUpdate.puma"
+/* line 848 "OverlapUpdate.puma" */
    if (! ((found))) goto yyL1;
   {
-# line 849 "OverlapUpdate.puma"
+/* line 849 "OverlapUpdate.puma" */
    SliceIncrement (slice2, & found, & val2);
-# line 850 "OverlapUpdate.puma"
+/* line 850 "OverlapUpdate.puma" */
    if (! ((found))) goto yyL1;
   {
-# line 851 "OverlapUpdate.puma"
+/* line 851 "OverlapUpdate.puma" */
    if (! ((val1 == val2))) goto yyL1;
   }
   }
@@ -1200,8 +1204,8 @@ static void CommonSliceIncrement
  }
 yyL1:;
 
-# line 854 "OverlapUpdate.puma"
-   * yyP2 = false;
+/* line 854 "OverlapUpdate.puma" */
+   * yyP2 = rfalse;
    * yyP1 = 0;
    return;
 
@@ -1219,12 +1223,12 @@ static tTree UnionSection
 {
   if (v1->Kind == kINDEXED_VAR) {
   if (v2->Kind == kINDEXED_VAR) {
-# line 873 "OverlapUpdate.puma"
+/* line 873 "OverlapUpdate.puma" */
   {
-# line 875 "OverlapUpdate.puma"
+/* line 875 "OverlapUpdate.puma" */
    if (! ((TreeVarName (v1) == TreeVarName (v2)))) goto yyL1;
   {
-# line 877 "OverlapUpdate.puma"
+/* line 877 "OverlapUpdate.puma" */
    UnionIndexes (v1->INDEXED_VAR.IND_EXPS, v2->INDEXED_VAR.IND_EXPS);
   }
   }
@@ -1233,9 +1237,9 @@ yyL1:;
 
   }
   }
-# line 882 "OverlapUpdate.puma"
+/* line 882 "OverlapUpdate.puma" */
   {
-# line 883 "OverlapUpdate.puma"
+/* line 883 "OverlapUpdate.puma" */
    failure2_protocol (MODULE, "UnionSection", v1, v2);
   }
    return NoTree;
@@ -1251,29 +1255,31 @@ static void UnionIndexes
  register tTree indexes2;
 # endif
 {
+ yyRecursion:
   if (indexes1->Kind == kBTE_LIST) {
   if (indexes2->Kind == kBTE_LIST) {
-# line 895 "OverlapUpdate.puma"
+/* line 895 "OverlapUpdate.puma" */
   {
-# line 897 "OverlapUpdate.puma"
+/* line 897 "OverlapUpdate.puma" */
  indexes1->BTE_LIST.Elem = UnionSlices (indexes1->BTE_LIST.Elem, indexes2->BTE_LIST.Elem); 
-# line 898 "OverlapUpdate.puma"
-   UnionIndexes (indexes1->BTE_LIST.Next, indexes2->BTE_LIST.Next);
+/* line 898 "OverlapUpdate.puma" */
+   indexes1 = indexes1->BTE_LIST.Next;
+   indexes2 = indexes2->BTE_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   }
   if (indexes1->Kind == kBTE_EMPTY) {
   if (indexes2->Kind == kBTE_EMPTY) {
-# line 901 "OverlapUpdate.puma"
+/* line 901 "OverlapUpdate.puma" */
    return;
 
   }
   }
-# line 904 "OverlapUpdate.puma"
+/* line 904 "OverlapUpdate.puma" */
   {
-# line 905 "OverlapUpdate.puma"
+/* line 905 "OverlapUpdate.puma" */
    failure2_protocol (MODULE, "UnionIndexes", indexes1, indexes2);
   }
    return;
@@ -1290,9 +1296,9 @@ static tTree UnionSlices
  register tTree e2;
 # endif
 {
-# line 916 "OverlapUpdate.puma"
+/* line 916 "OverlapUpdate.puma" */
   {
-# line 917 "OverlapUpdate.puma"
+/* line 917 "OverlapUpdate.puma" */
    if (! ((EqualExpression (e1, e2)))) goto yyL1;
   }
    return e1;
@@ -1300,25 +1306,25 @@ yyL1:;
 
   if (e1->Kind == kSLICE_EXP) {
   if (e2->Kind == kSLICE_EXP) {
-# line 921 "OverlapUpdate.puma"
+/* line 921 "OverlapUpdate.puma" */
   {
-# line 923 "OverlapUpdate.puma"
+/* line 923 "OverlapUpdate.puma" */
    if (! ((ExpLE (e1, e2)))) goto yyL2;
   {
-# line 924 "OverlapUpdate.puma"
+/* line 924 "OverlapUpdate.puma" */
  e1->SLICE_EXP.STOP = e2->SLICE_EXP.STOP; 
   }
   }
    return e1;
 yyL2:;
 
-# line 928 "OverlapUpdate.puma"
+/* line 928 "OverlapUpdate.puma" */
   {
-# line 930 "OverlapUpdate.puma"
+/* line 930 "OverlapUpdate.puma" */
    if (! ((ExpLE (e2, e1)))) goto yyL3;
   {
-# line 931 "OverlapUpdate.puma"
- e1->SLICE_EXP.START = e2->SLICE_EXP.START; 
+/* line 931 "OverlapUpdate.puma" */
+ e1->SLICE_EXP.FIRST = e2->SLICE_EXP.FIRST; 
   }
   }
    return e1;
@@ -1326,23 +1332,23 @@ yyL3:;
 
   }
   }
-# line 935 "OverlapUpdate.puma"
+/* line 935 "OverlapUpdate.puma" */
   {
-# line 936 "OverlapUpdate.puma"
+/* line 936 "OverlapUpdate.puma" */
    if (! ((ExpLE (e1, e2)))) goto yyL4;
   }
    return mSLICE_EXP (e1, e2, mDUMMY_EXP ());
 yyL4:;
 
-# line 940 "OverlapUpdate.puma"
+/* line 940 "OverlapUpdate.puma" */
   {
-# line 941 "OverlapUpdate.puma"
+/* line 941 "OverlapUpdate.puma" */
    if (! ((ExpLE (e2, e1)))) goto yyL5;
   }
    return mSLICE_EXP (e2, e1, mDUMMY_EXP ());
 yyL5:;
 
-# line 947 "OverlapUpdate.puma"
+/* line 947 "OverlapUpdate.puma" */
    return mSLICE_EXP (mDUMMY_EXP (), mDUMMY_EXP (), mDUMMY_EXP ());
 
 }
@@ -1356,27 +1362,23 @@ static tTree MakeOverlapReset
 # endif
 {
   if (var->Kind == kINDEXED_VAR) {
-# line 961 "OverlapUpdate.puma"
+/* line 961 "OverlapUpdate.puma" */
  {
   tTree reset;
   {
-# line 963 "OverlapUpdate.puma"
-
-# line 965 "OverlapUpdate.puma"
+/* line 965 "OverlapUpdate.puma" */
  reset = mBTE_EMPTY ();
      reset = mREG_SHADOW_GET (CopyTree (var->INDEXED_VAR.IND_VAR), reset);
      reset = mACF_BASIC (reset);
    
   }
-  {
    return reset;
-  }
  }
 
   }
-# line 972 "OverlapUpdate.puma"
+/* line 972 "OverlapUpdate.puma" */
   {
-# line 973 "OverlapUpdate.puma"
+/* line 973 "OverlapUpdate.puma" */
    failure_protocol (MODULE, "MakeOverlapReset", var);
   }
    return NoTree;
@@ -1392,13 +1394,11 @@ void VectorizeOverlapUpdates
  register tTree slice;
 # endif
 {
-# line 990 "OverlapUpdate.puma"
+/* line 990 "OverlapUpdate.puma" */
  {
   int i;
   {
-# line 994 "OverlapUpdate.puma"
-
-# line 996 "OverlapUpdate.puma"
+/* line 996 "OverlapUpdate.puma" */
  for (i=0; i<UpdateCounter; i++)
 
        { VectorizeVar (all_updates[i].var, id, slice);
@@ -1413,10 +1413,10 @@ void VectorizeOverlapUpdates
 ;
 }
 
-void BeginOverlapUpdate ()
+void BeginOverlapUpdate ARGS ((void))
 {
 }
 
-void CloseOverlapUpdate ()
+void CloseOverlapUpdate ARGS ((void))
 {
 }
