@@ -1,6 +1,13 @@
 # include "Ownership.h"
-# include "yyOwnership.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,33 +15,30 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 41 "Ownership.puma"
+/* line 41 "Ownership.puma" */
 
 
 # include <stdio.h>
 # include "protocol.h"
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "TreeOps.h"
 # include "Rank.h"
@@ -56,11 +60,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* Ownership_Exit) () = yyExit;
+# include "yyOwnership.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* Ownership_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -69,27 +87,28 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module Ownership, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module Ownership, routine %s failed\n",
+  yyFunction);
  Ownership_Exit ();
 }
 
-tTree MakeOnSelection ARGS((tTree stmt, pvar outer_home, pvar inner_home));
-void BetterOnSelection ARGS((tTree stmt, pvar inner_home, pvar outer_home));
-void OnIdUpdate ARGS((tTree stmt, pvar home, tTree id));
-static tTree GetOnVal ARGS((pvar vard, int topdim));
-tTree MakeOnVal ARGS((pvar vard, int topdim));
-tTree MakeOnTopology ARGS((pvar vard));
-tTree MakeOnSpecs ARGS((pvar vard));
-static tTree MakeOnSpecs1 ARGS((tTree old, pvar vard));
-static tTree MakeOnIdVals ARGS((pvar home, tTree id));
-static tTree AddOnIdRanges ARGS((tTree spec, pvar home, tTree id));
-static void FindLoopId ARGS((pvar owner, tTree id, bool * yyP3, int * yyP2, tTree * yyP1));
-static void FindShiftLoopId ARGS((pvar owner, tTree id, int * yyP7, int * yyP6, int * yyP5, int * yyP4));
-static tTree AddOnSpecs ARGS((tTree spec, pvar inner_home, pvar outer_home));
-void MakeOnLoop ARGS((tTree t, pvar home));
-static tTree MakeOnInfo ARGS((pvar vard, int dim, int base, int stride));
-static tTree SelectBody ARGS((tTree body, pvar home, tTree on_vals));
-static bool IsParallelInfo ARGS((tTree info));
+tTree MakeOnSelection ARGS ((tTree stmt, pvar outer_home, pvar inner_home));
+void BetterOnSelection ARGS ((tTree stmt, pvar inner_home, pvar outer_home));
+void OnIdUpdate ARGS ((tTree stmt, pvar home, tTree id));
+static tTree GetOnVal ARGS ((pvar vard, int topdim));
+tTree MakeOnVal ARGS ((pvar vard, int topdim));
+tTree MakeOnTopology ARGS ((pvar vard));
+tTree MakeOnSpecs ARGS ((pvar vard));
+static tTree MakeOnSpecs1 ARGS ((tTree old, pvar vard));
+static tTree MakeOnIdVals ARGS ((pvar home, tTree id));
+static tTree AddOnIdRanges ARGS ((tTree spec, pvar home, tTree id));
+static void FindLoopId ARGS ((pvar owner, tTree id, rbool * yyP3, int * yyP2, tTree * yyP1));
+static void FindShiftLoopId ARGS ((pvar owner, tTree id, int * yyP7, int * yyP6, int * yyP5, int * yyP4));
+static tTree AddOnSpecs ARGS ((tTree spec, pvar inner_home, pvar outer_home));
+void MakeOnLoop ARGS ((tTree t, pvar home));
+static tTree MakeOnInfo ARGS ((pvar vard, int dim, int base, int stride));
+static tTree SelectBody ARGS ((tTree body, pvar home, tTree on_vals));
+static rbool IsParallelInfo ARGS ((tTree info));
 
 tTree MakeOnSelection
 # if defined __STDC__ | defined __cplusplus
@@ -101,58 +120,52 @@ tTree MakeOnSelection
  pvar inner_home;
 # endif
 {
-# line 93 "Ownership.puma"
+/* line 93 "Ownership.puma" */
   {
-# line 95 "Ownership.puma"
+/* line 95 "Ownership.puma" */
    if (! ((GetCurrentModel () == HPF_SERIAL))) goto yyL1;
   }
    return stmt;
 yyL1:;
 
-# line 100 "Ownership.puma"
+/* line 100 "Ownership.puma" */
   {
-# line 102 "Ownership.puma"
+/* line 102 "Ownership.puma" */
    if (! ((IsReplicatedDescriptor (outer_home)))) goto yyL2;
   {
-# line 103 "Ownership.puma"
+/* line 103 "Ownership.puma" */
    if (! ((IsReplicatedDescriptor (inner_home)))) goto yyL2;
   }
   }
    return stmt;
 yyL2:;
 
-# line 108 "Ownership.puma"
+/* line 108 "Ownership.puma" */
   {
-# line 110 "Ownership.puma"
+/* line 110 "Ownership.puma" */
    if (! ((IsNoDescriptor (inner_home)))) goto yyL3;
   }
    return stmt;
 yyL3:;
 
-# line 115 "Ownership.puma"
+/* line 115 "Ownership.puma" */
   {
-# line 117 "Ownership.puma"
+/* line 117 "Ownership.puma" */
    if (! ((IsReplicatedDescriptor (outer_home)))) goto yyL4;
   }
    return mACF_ON (MakeOnTopology (inner_home), MakeOnSpecs (inner_home), MakeStmtList (stmt));
 yyL4:;
 
-# line 125 "Ownership.puma"
+/* line 125 "Ownership.puma" */
  {
   int dim;
   tTree spec;
   tTree new;
   {
-# line 129 "Ownership.puma"
+/* line 129 "Ownership.puma" */
    if (! ((SameDescriptorTopology (outer_home, inner_home)))) goto yyL5;
   {
-# line 133 "Ownership.puma"
-
-# line 134 "Ownership.puma"
-
-# line 135 "Ownership.puma"
-
-# line 137 "Ownership.puma"
+/* line 137 "Ownership.puma" */
  spec = NoTree;
 
      for (dim = 1; dim <= outer_home->topology_rank; dim ++)
@@ -184,19 +197,17 @@ yyL4:;
    
   }
   }
-  {
    return new;
-  }
  }
 yyL5:;
 
-# line 170 "Ownership.puma"
+/* line 170 "Ownership.puma" */
   {
-# line 172 "Ownership.puma"
+/* line 172 "Ownership.puma" */
    error_protocol ("could not make on selection");
-# line 173 "Ownership.puma"
+/* line 173 "Ownership.puma" */
    tree_protocol ("was home : ", PrintableDescriptorVar (outer_home));
-# line 174 "Ownership.puma"
+/* line 174 "Ownership.puma" */
    tree_protocol ("is  home : ", PrintableDescriptorVar (inner_home));
   }
    return stmt;
@@ -213,21 +224,21 @@ void BetterOnSelection
  pvar outer_home;
 # endif
 {
-# line 192 "Ownership.puma"
+/* line 192 "Ownership.puma" */
   {
-# line 194 "Ownership.puma"
+/* line 194 "Ownership.puma" */
    if (! ((VDIsSubSet (outer_home, inner_home)))) goto yyL1;
   }
    return;
 yyL1:;
 
   if (stmt->Kind == kREDUCTION_STMT) {
-# line 197 "Ownership.puma"
+/* line 197 "Ownership.puma" */
   {
-# line 199 "Ownership.puma"
+/* line 199 "Ownership.puma" */
    if (! ((IsReplicatedDescriptor (outer_home)))) goto yyL2;
   {
-# line 201 "Ownership.puma"
+/* line 201 "Ownership.puma" */
  stmt->REDUCTION_STMT.RED_TOP  = MakeOnTopology (inner_home);
      stmt->REDUCTION_STMT.RED_SPEC = MakeOnSpecs1 (stmt->REDUCTION_STMT.RED_SPEC, inner_home);
    
@@ -236,21 +247,21 @@ yyL1:;
    return;
 yyL2:;
 
-# line 206 "Ownership.puma"
+/* line 206 "Ownership.puma" */
   {
-# line 208 "Ownership.puma"
+/* line 208 "Ownership.puma" */
  stmt->REDUCTION_STMT.RED_SPEC = AddOnSpecs (stmt->REDUCTION_STMT.RED_SPEC, inner_home, outer_home);  
   }
    return;
 
   }
   if (stmt->Kind == kBROADCAST_STMT) {
-# line 211 "Ownership.puma"
+/* line 211 "Ownership.puma" */
   {
-# line 213 "Ownership.puma"
+/* line 213 "Ownership.puma" */
    if (! ((IsReplicatedDescriptor (outer_home)))) goto yyL4;
   {
-# line 215 "Ownership.puma"
+/* line 215 "Ownership.puma" */
  stmt->BROADCAST_STMT.OWNER_TOP  = MakeOnTopology (inner_home);
      stmt->BROADCAST_STMT.OWNER_SPEC = MakeOnSpecs1 (stmt->BROADCAST_STMT.OWNER_SPEC, inner_home);
    
@@ -259,17 +270,17 @@ yyL2:;
    return;
 yyL4:;
 
-# line 220 "Ownership.puma"
+/* line 220 "Ownership.puma" */
   {
-# line 222 "Ownership.puma"
+/* line 222 "Ownership.puma" */
  stmt->BROADCAST_STMT.OWNER_SPEC = AddOnSpecs (stmt->BROADCAST_STMT.OWNER_SPEC, inner_home, outer_home); 
   }
    return;
 
   }
-# line 225 "Ownership.puma"
+/* line 225 "Ownership.puma" */
   {
-# line 227 "Ownership.puma"
+/* line 227 "Ownership.puma" */
    failure_protocol (MODULE, "BetterOnSelection", stmt);
   }
    return;
@@ -288,26 +299,26 @@ void OnIdUpdate
 # endif
 {
   if (stmt->Kind == kREDUCTION_STMT) {
-# line 246 "Ownership.puma"
+/* line 246 "Ownership.puma" */
   {
-# line 248 "Ownership.puma"
+/* line 248 "Ownership.puma" */
  stmt->REDUCTION_STMT.RED_SPEC = AddOnIdRanges (stmt->REDUCTION_STMT.RED_SPEC, home, id); 
   }
    return;
 
   }
   if (stmt->Kind == kBROADCAST_STMT) {
-# line 251 "Ownership.puma"
+/* line 251 "Ownership.puma" */
   {
-# line 253 "Ownership.puma"
+/* line 253 "Ownership.puma" */
  stmt->BROADCAST_STMT.OWNER_SPEC = AddOnIdRanges (stmt->BROADCAST_STMT.OWNER_SPEC, home, id); 
   }
    return;
 
   }
-# line 256 "Ownership.puma"
+/* line 256 "Ownership.puma" */
   {
-# line 258 "Ownership.puma"
+/* line 258 "Ownership.puma" */
    failure_protocol (MODULE, "OnIdUpdates", stmt);
   }
    return;
@@ -324,23 +335,23 @@ static tTree GetOnVal
  register int topdim;
 # endif
 {
-# line 281 "Ownership.puma"
+/* line 281 "Ownership.puma" */
   {
-# line 283 "Ownership.puma"
+/* line 283 "Ownership.puma" */
    if (! ((vard -> on_val [topdim - 1] == NoTree))) goto yyL1;
   }
    return NoTree;
 yyL1:;
 
-# line 287 "Ownership.puma"
+/* line 287 "Ownership.puma" */
   {
-# line 289 "Ownership.puma"
+/* line 289 "Ownership.puma" */
    if (! ((TreeRank (vard -> on_val [topdim - 1]) > 0))) goto yyL2;
   }
    return NoTree;
 yyL2:;
 
-# line 293 "Ownership.puma"
+/* line 293 "Ownership.puma" */
    return (vard -> on_val [topdim - 1]);
 
 }
@@ -354,16 +365,12 @@ tTree MakeOnVal
  register int topdim;
 # endif
 {
-# line 308 "Ownership.puma"
+/* line 308 "Ownership.puma" */
  {
   tTree spec;
   tTree val;
   {
-# line 310 "Ownership.puma"
-
-# line 311 "Ownership.puma"
-
-# line 313 "Ownership.puma"
+/* line 313 "Ownership.puma" */
  val = vard->on_val [topdim - 1];
 
     if (val == NoTree)
@@ -375,9 +382,7 @@ tTree MakeOnVal
                     vard->on_index_dim [topdim-1], val);
   
   }
-  {
    return spec;
-  }
  }
 
 }
@@ -390,31 +395,31 @@ tTree MakeOnTopology
  pvar vard;
 # endif
 {
-# line 337 "Ownership.puma"
+/* line 337 "Ownership.puma" */
   {
-# line 338 "Ownership.puma"
+/* line 338 "Ownership.puma" */
    if (! ((IsReplicatedDescriptor (vard)))) goto yyL1;
   }
    return mON_ALL ();
 yyL1:;
 
-# line 342 "Ownership.puma"
+/* line 342 "Ownership.puma" */
   {
-# line 343 "Ownership.puma"
+/* line 343 "Ownership.puma" */
    if (! ((IsHostDescriptor (vard)))) goto yyL2;
   }
    return mON_HOST ();
 yyL2:;
 
-# line 347 "Ownership.puma"
+/* line 347 "Ownership.puma" */
   {
-# line 348 "Ownership.puma"
+/* line 348 "Ownership.puma" */
    if (! ((IsProcDescriptor (vard)))) goto yyL3;
   }
    return mON_PROCS (vard -> var_obj);
 yyL3:;
 
-# line 352 "Ownership.puma"
+/* line 352 "Ownership.puma" */
    return mON_VAR (vard -> topology_obj, vard -> template_obj, GetGlobalObject (TreeVarName (vard -> var_tree)));
 
 }
@@ -427,13 +432,11 @@ tTree MakeOnSpecs
  pvar vard;
 # endif
 {
-# line 368 "Ownership.puma"
+/* line 368 "Ownership.puma" */
  {
   tTree specs;
   {
-# line 370 "Ownership.puma"
-
-# line 372 "Ownership.puma"
+/* line 372 "Ownership.puma" */
  int dim, rank;
     specs = mON_EMPTY();
     rank  = vard->topology_rank;
@@ -442,9 +445,7 @@ tTree MakeOnSpecs
           specs = mON_LIST (MakeOnVal (vard, dim), specs);
   
   }
-  {
    return specs;
-  }
  }
 
 }
@@ -458,13 +459,11 @@ static tTree MakeOnSpecs1
  pvar vard;
 # endif
 {
-# line 384 "Ownership.puma"
+/* line 384 "Ownership.puma" */
  {
   tTree specs;
   {
-# line 386 "Ownership.puma"
-
-# line 388 "Ownership.puma"
+/* line 388 "Ownership.puma" */
  int dim, rank;
     specs = old;
     rank  = vard->topology_rank;
@@ -473,9 +472,7 @@ static tTree MakeOnSpecs1
           specs = mON_LIST (MakeOnVal (vard, dim), specs);
   
   }
-  {
    return specs;
-  }
  }
 
 }
@@ -489,42 +486,36 @@ static tTree MakeOnIdVals
  register tTree id;
 # endif
 {
-# line 400 "Ownership.puma"
+/* line 400 "Ownership.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   int yyV2;
   tTree yyV3;
   {
-# line 402 "Ownership.puma"
+/* line 402 "Ownership.puma" */
    FindLoopId (home, id, & yyV1, & yyV2, & yyV3);
-# line 403 "Ownership.puma"
+/* line 403 "Ownership.puma" */
    if (! ((! yyV1))) goto yyL1;
   }
-  {
    return mON_EMPTY ();
-  }
  }
 yyL1:;
 
-# line 408 "Ownership.puma"
+/* line 408 "Ownership.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   int yyV2;
   tTree yyV3;
   tTree on_val;
   {
-# line 410 "Ownership.puma"
+/* line 410 "Ownership.puma" */
    FindLoopId (home, id, & yyV1, & yyV2, & yyV3);
-# line 412 "Ownership.puma"
-
-# line 414 "Ownership.puma"
+/* line 414 "Ownership.puma" */
    on_val = MakeOnVal (home, yyV2);
-# line 416 "Ownership.puma"
+/* line 416 "Ownership.puma" */
  home->on_val[yyV2-1] = NoTree; 
   }
-  {
    return mON_LIST (on_val, MakeOnIdVals (home, id));
-  }
  }
 
 }
@@ -539,13 +530,11 @@ static tTree AddOnIdRanges
  register tTree id;
 # endif
 {
-# line 432 "Ownership.puma"
+/* line 432 "Ownership.puma" */
  {
   tTree new_spec;
   {
-# line 434 "Ownership.puma"
-
-# line 436 "Ownership.puma"
+/* line 436 "Ownership.puma" */
  int topdim, toprank;
      int found;
 
@@ -589,39 +578,31 @@ static tTree AddOnIdRanges
        } 
    
   }
-  {
    return new_spec;
-  }
  }
 
 }
 
 static void FindLoopId
 # if defined __STDC__ | defined __cplusplus
-(pvar owner, register tTree id, register bool * yyP3, register int * yyP2, register tTree * yyP1)
+(pvar owner, register tTree id, register rbool * yyP3, register int * yyP2, register tTree * yyP1)
 # else
 (owner, id, yyP3, yyP2, yyP1)
  pvar owner;
  register tTree id;
- register bool * yyP3;
+ register rbool * yyP3;
  register int * yyP2;
  register tTree * yyP1;
 # endif
 {
-# line 498 "Ownership.puma"
+/* line 498 "Ownership.puma" */
  {
-  bool found;
+  rbool found;
   int topdim;
   tTree val;
   {
-# line 500 "Ownership.puma"
-
-# line 501 "Ownership.puma"
-
-# line 502 "Ownership.puma"
-
-# line 504 "Ownership.puma"
- found  = false;
+/* line 504 "Ownership.puma" */
+ found  = rfalse;
      topdim = 0;
 
      while ((!found) && (topdim < owner->topology_rank))
@@ -660,29 +641,19 @@ static void FindShiftLoopId
  register int * yyP4;
 # endif
 {
-# line 524 "Ownership.puma"
+/* line 524 "Ownership.puma" */
  {
-  bool found;
+  rbool found;
   int topdim;
   int base;
   int stride;
   tTree val;
   {
-# line 526 "Ownership.puma"
-
-# line 527 "Ownership.puma"
-
-# line 528 "Ownership.puma"
-
-# line 529 "Ownership.puma"
-
-# line 530 "Ownership.puma"
-
-# line 532 "Ownership.puma"
+/* line 532 "Ownership.puma" */
  int toprank;
      tTree var;
 
-     found   = false;
+     found   = rfalse;
      topdim  = 0;
      toprank = owner->topology_rank;
 
@@ -722,29 +693,27 @@ static tTree AddOnSpecs
  pvar outer_home;
 # endif
 {
-# line 565 "Ownership.puma"
+/* line 565 "Ownership.puma" */
   {
-# line 569 "Ownership.puma"
+/* line 569 "Ownership.puma" */
    if (! ((! SameDescriptorTopology (inner_home, outer_home)))) goto yyL1;
   {
-# line 571 "Ownership.puma"
+/* line 571 "Ownership.puma" */
    error_protocol ("illegal call on add on specs");
-# line 572 "Ownership.puma"
+/* line 572 "Ownership.puma" */
    tree_protocol ("inner home : ", MakeDescriptorVar (inner_home));
-# line 573 "Ownership.puma"
+/* line 573 "Ownership.puma" */
    tree_protocol ("outer home : ", MakeDescriptorVar (outer_home));
   }
   }
    return spec;
 yyL1:;
 
-# line 578 "Ownership.puma"
+/* line 578 "Ownership.puma" */
  {
   tTree new_spec;
   {
-# line 580 "Ownership.puma"
-
-# line 582 "Ownership.puma"
+/* line 582 "Ownership.puma" */
  int dim, rank;
 
      new_spec = spec;
@@ -770,9 +739,7 @@ yyL1:;
 
    
   }
-  {
    return new_spec;
-  }
  }
 
 }
@@ -787,30 +754,30 @@ void MakeOnLoop
 # endif
 {
   if (t->Kind == kACF_DO) {
-# line 633 "Ownership.puma"
+/* line 633 "Ownership.puma" */
   {
-# line 635 "Ownership.puma"
+/* line 635 "Ownership.puma" */
    if (! ((GetCurrentModel () == HPF_SERIAL))) goto yyL1;
   {
-# line 637 "Ownership.puma"
+/* line 637 "Ownership.puma" */
  t->ACF_DO.DO_HOME_INFO = mNO_HOME_INFO (); 
   }
   }
    return;
 yyL1:;
 
-# line 640 "Ownership.puma"
+/* line 640 "Ownership.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   int yyV2;
   tTree yyV3;
   {
-# line 642 "Ownership.puma"
+/* line 642 "Ownership.puma" */
    FindLoopId (home, t->ACF_DO.DO_ID, & yyV1, & yyV2, & yyV3);
-# line 643 "Ownership.puma"
+/* line 643 "Ownership.puma" */
    if (! ((! yyV1))) goto yyL2;
   {
-# line 647 "Ownership.puma"
+/* line 647 "Ownership.puma" */
  t->ACF_DO.DO_HOME_INFO = mNO_HOME_INFO (); 
   }
   }
@@ -818,19 +785,19 @@ yyL1:;
  }
 yyL2:;
 
-# line 675 "Ownership.puma"
+/* line 675 "Ownership.puma" */
  {
   int yyV1;
   int yyV2;
   int yyV3;
   int yyV4;
   {
-# line 681 "Ownership.puma"
+/* line 681 "Ownership.puma" */
    FindShiftLoopId (home, t->ACF_DO.DO_ID, & yyV1, & yyV2, & yyV3, & yyV4);
-# line 683 "Ownership.puma"
+/* line 683 "Ownership.puma" */
    if (! ((yyV1))) goto yyL3;
   {
-# line 685 "Ownership.puma"
+/* line 685 "Ownership.puma" */
  t->ACF_DO.DO_HOME_INFO = MakeOnInfo (home, yyV2, yyV3, yyV4);
 
      home->on_val [yyV2-1] = NoTree;    
@@ -844,13 +811,13 @@ yyL2:;
  }
 yyL3:;
 
-# line 694 "Ownership.puma"
+/* line 694 "Ownership.puma" */
   {
-# line 696 "Ownership.puma"
+/* line 696 "Ownership.puma" */
    error_protocol ("loop variable does not appear linear in home");
-# line 697 "Ownership.puma"
+/* line 697 "Ownership.puma" */
    tree_protocol ("loop variable      : ", t->ACF_DO.DO_ID);
-# line 698 "Ownership.puma"
+/* line 698 "Ownership.puma" */
    tree_protocol ("home of iterations : ", PrintableDescriptorVar (home));
   }
    return;
@@ -870,25 +837,17 @@ static tTree MakeOnInfo
  register int stride;
 # endif
 {
-# line 715 "Ownership.puma"
+/* line 715 "Ownership.puma" */
  {
   int base1;
   int stride1;
   int index_dim;
+  int on_kind;
+  tTree on_size;
   int on_dim;
   tDefinitions on_id;
   {
-# line 717 "Ownership.puma"
-
-# line 718 "Ownership.puma"
-
-# line 720 "Ownership.puma"
-
-# line 721 "Ownership.puma"
-
-# line 722 "Ownership.puma"
-
-# line 724 "Ownership.puma"
+/* line 731 "Ownership.puma" */
  index_dim = vard->on_index_dim[dim-1];
 
     base1     = vard->align_add [index_dim-1];
@@ -903,13 +862,16 @@ static tTree MakeOnInfo
 
     if (on_id == NoObject) on_id = vard->var_obj;   
 
-    on_dim = vard->on_temp_dim [dim-1];
+    on_dim  = vard->on_temp_dim [dim-1];
+
+    
+
+    on_kind = vard->distribution_kind [index_dim-1];
+    on_size = vard->distribution_size [index_dim-1];
 
   
   }
-  {
-   return mON_INFO (on_id, on_dim, base1, stride1);
-  }
+   return mON_INFO (on_id, on_dim, on_kind, on_size, base1, stride1);
  }
 
 }
@@ -925,16 +887,16 @@ static tTree SelectBody
 # endif
 {
   if (on_vals->Kind == kON_EMPTY) {
-# line 757 "Ownership.puma"
+/* line 770 "Ownership.puma" */
    return body;
 
   }
-# line 762 "Ownership.puma"
+/* line 775 "Ownership.puma" */
    return mACF_ON (MakeOnTopology (home), on_vals, body);
 
 }
 
-static bool IsParallelInfo
+static rbool IsParallelInfo
 # if defined __STDC__ | defined __cplusplus
 (register tTree info)
 # else
@@ -943,24 +905,24 @@ static bool IsParallelInfo
 # endif
 {
   if (info->Kind == kINDEP_INFO) {
-# line 775 "Ownership.puma"
+/* line 788 "Ownership.puma" */
   {
-# line 777 "Ownership.puma"
+/* line 790 "Ownership.puma" */
    if (! ((info->INDEP_INFO.selection != 0))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
-# line 782 "Ownership.puma"
-   return false;
+/* line 795 "Ownership.puma" */
+   return rfalse;
 
 }
 
-void BeginOwnership ()
+void BeginOwnership ARGS ((void))
 {
 }
 
-void CloseOwnership ()
+void CloseOwnership ARGS ((void))
 {
 }
