@@ -1,6 +1,13 @@
 # include "Reaching.h"
-# include "yyReaching.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,34 +15,31 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 43 "Reaching.puma"
+/* line 43 "Reaching.puma" */
 
 
 # include "Idents.h"
 # include "protocol.h"
 
 # include "TreeOps.h"        /* IsDeferredShape  */
-# include "Objects.h"        /* ArrayFormals     */
+# include "Objects.h"        /* GetObjDimension  */
 # include "Types.h"          /* ArrayFormals     */
 # include "Transform.h"      /* ExpToVarParam    */
 # include "Expressions.h"    /* MakeConstant     */
@@ -96,10 +100,10 @@ static int GetAllocPosition (obj)
 tObject obj;
 
 { int pos;
-  bool found;
+  rbool found;
 
   pos = 0;
-  found = false;
+  found = rfalse;
 
   while ((pos < allocate_top) && (!found))
 
@@ -122,10 +126,10 @@ static int GetDistPosition (obj)
 tObject obj;
 
 { int pos;
-  bool found;
+  rbool found;
 
   pos = 0;
-  found = false;
+  found = rfalse;
 
   while ((pos < distribution_top) && (!found))
 
@@ -143,7 +147,7 @@ tObject obj;
        *                                                *
        *************************************************/
 
-static bool IsAllocated (obj)
+static rbool IsAllocated (obj)
 
 tObject obj;
 
@@ -219,11 +223,25 @@ tDefinitions distribution;
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* Reaching_Exit) () = yyExit;
+# include "yyReaching.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* Reaching_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -232,32 +250,33 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module Reaching, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module Reaching, routine %s failed\n",
+  yyFunction);
  Reaching_Exit ();
 }
 
-void ReachingAllocate ARGS((tTree t));
-static bool StopTraversing ARGS((tTree t));
-static void DoIt ARGS((tTree t));
-static tDefinitions GetTemplateObj ARGS((tDefinitions obj, tDefinitions dist));
-static void SetAllocateParameters ARGS((tTree params));
-static void SetAllocateParam ARGS((tTree var));
-static void ResetDeallocateParameters ARGS((tTree params));
-static void ResetDeallocateParam ARGS((tTree var));
-static void SetObjAllocated ARGS((tDefinitions obj, tTree indexes));
-static void SetObjDeallocated ARGS((tDefinitions obj));
-static void SetRemapping ARGS((tTree distributees, tDefinitions map_info));
-static tTree GetReachingInfo ARGS((tTree uv, tDefinitions obj));
-static tDefinitions GetReachingDistribution ARGS((tDefinitions obj));
-static tTree GetReachingAllocate ARGS((tTree uv, tDefinitions obj));
-static void NoReachingWarning ARGS((tDefinitions obj));
-static tTree MakeUsedVarTree ARGS((tTree uv, tDefinitions obj));
-static void DeallocateCheck ARGS(());
-static void DeallocateCheckObj ARGS((tDefinitions obj));
-static bool IsLocalObject ARGS((tDefinitions obj));
-static tTree MakeIndexBoundaries ARGS((tTree var, tTree formals, int n));
-static tTree MakeIndexBound ARGS((tTree var, tTree shape, int dim));
-void PrintReachingInfo ARGS((tTree uv));
+void ReachingAllocate ARGS ((tTree t));
+static rbool StopTraversing ARGS ((tTree t));
+static void DoIt ARGS ((tTree t));
+static tDefinitions GetTemplateObj ARGS ((tDefinitions obj, tDefinitions dist));
+static void SetAllocateParameters ARGS ((tTree params));
+static void SetAllocateParam ARGS ((tTree var));
+static void ResetDeallocateParameters ARGS ((tTree params));
+static void ResetDeallocateParam ARGS ((tTree var));
+static void SetObjAllocated ARGS ((tDefinitions obj, tTree indexes));
+static void SetObjDeallocated ARGS ((tDefinitions obj));
+static void SetRemapping ARGS ((tTree distributees, tDefinitions map_info));
+static tTree GetReachingInfo ARGS ((tTree uv, tDefinitions obj));
+static tDefinitions GetReachingDistribution ARGS ((tDefinitions obj));
+static tTree GetReachingAllocate ARGS ((tTree uv, tDefinitions obj));
+static void NoReachingWarning ARGS ((tDefinitions obj));
+static tTree MakeUsedVarTree ARGS ((tTree uv, tDefinitions obj));
+static void DeallocateCheck ARGS ((void));
+static void DeallocateCheckObj ARGS ((tDefinitions obj));
+static rbool IsLocalObject ARGS ((tDefinitions obj));
+static tTree MakeIndexBoundaries ARGS ((tTree var, tTree formals, int n));
+static tTree MakeIndexBound ARGS ((tTree var, tTree shape, int dim));
+void PrintReachingInfo ARGS ((tTree uv));
 
 void ReachingAllocate
 # if defined __STDC__ | defined __cplusplus
@@ -267,9 +286,9 @@ void ReachingAllocate
  register tTree t;
 # endif
 {
-# line 240 "Reaching.puma"
+/* line 240 "Reaching.puma" */
   {
-# line 241 "Reaching.puma"
+/* line 241 "Reaching.puma" */
    TraverseAST (t, StopTraversing, DoIt);
   }
    return;
@@ -277,7 +296,7 @@ void ReachingAllocate
 ;
 }
 
-static bool StopTraversing
+static rbool StopTraversing
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -286,21 +305,21 @@ static bool StopTraversing
 # endif
 {
   if (t->Kind == kBODY_NODE) {
-# line 252 "Reaching.puma"
-   return true;
+/* line 252 "Reaching.puma" */
+   return rtrue;
 
   }
   if (t->Kind == kALLOCATE_STMT) {
-# line 255 "Reaching.puma"
-   return true;
+/* line 255 "Reaching.puma" */
+   return rtrue;
 
   }
   if (t->Kind == kDEALLOCATE_STMT) {
-# line 258 "Reaching.puma"
-   return true;
+/* line 258 "Reaching.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
 static void DoIt
@@ -312,71 +331,71 @@ static void DoIt
 # endif
 {
   if (t->Kind == kBODY_NODE) {
-# line 269 "Reaching.puma"
+/* line 269 "Reaching.puma" */
   {
-# line 271 "Reaching.puma"
+/* line 271 "Reaching.puma" */
    allocate_top = 0;
-# line 272 "Reaching.puma"
+/* line 272 "Reaching.puma" */
    distribution_top = 0;
-# line 274 "Reaching.puma"
+/* line 274 "Reaching.puma" */
    ReachingAllocate (t->BODY_NODE.DECLS);
-# line 275 "Reaching.puma"
+/* line 275 "Reaching.puma" */
    ReachingAllocate (t->BODY_NODE.STATS);
-# line 279 "Reaching.puma"
+/* line 279 "Reaching.puma" */
    DeallocateCheck ();
-# line 281 "Reaching.puma"
+/* line 281 "Reaching.puma" */
    ReachingAllocate (t->BODY_NODE.INTERNALS);
   }
    return;
 
   }
   if (t->Kind == kALLOCATE_STMT) {
-# line 284 "Reaching.puma"
+/* line 284 "Reaching.puma" */
   {
-# line 286 "Reaching.puma"
+/* line 286 "Reaching.puma" */
    SetAllocateParameters (t->ALLOCATE_STMT.PARAMS);
-# line 287 "Reaching.puma"
+/* line 287 "Reaching.puma" */
    ReachingAllocate (t->ALLOCATE_STMT.STATUS);
   }
    return;
 
   }
   if (t->Kind == kDEALLOCATE_STMT) {
-# line 290 "Reaching.puma"
+/* line 290 "Reaching.puma" */
   {
-# line 292 "Reaching.puma"
+/* line 292 "Reaching.puma" */
    ResetDeallocateParameters (t->DEALLOCATE_STMT.PARAMS);
-# line 293 "Reaching.puma"
+/* line 293 "Reaching.puma" */
    ReachingAllocate (t->DEALLOCATE_STMT.STATUS);
   }
    return;
 
   }
   if (t->Kind == kREDISTRIBUTE_STMT) {
-# line 296 "Reaching.puma"
+/* line 296 "Reaching.puma" */
   {
-# line 298 "Reaching.puma"
+/* line 298 "Reaching.puma" */
    SetRemapping (t->REDISTRIBUTE_STMT.DISTRIBUTEE, t->REDISTRIBUTE_STMT.distribution);
   }
    return;
 
   }
   if (t->Kind == kREALIGN_STMT) {
-# line 301 "Reaching.puma"
+/* line 301 "Reaching.puma" */
   {
-# line 303 "Reaching.puma"
+/* line 303 "Reaching.puma" */
    SetRemapping (t->REALIGN_STMT.ALIGNEE, t->REALIGN_STMT.distribution);
   }
    return;
 
   }
   if (t->Kind == kUSED_VAR) {
-# line 306 "Reaching.puma"
+/* line 306 "Reaching.puma" */
   {
-# line 308 "Reaching.puma"
+/* line 308 "Reaching.puma" */
    if (! ((VarRank (t->USED_VAR.VARNAME->VAR_OBJ.Object) > 0))) goto yyL6;
   {
-# line 310 "Reaching.puma"
+/* line 310 "Reaching.puma" */
  t->USED_VAR.VARNAME->VAR_OBJ.Reaching = GetReachingInfo (t, t->USED_VAR.VARNAME->VAR_OBJ.Object); 
   }
   }
@@ -396,12 +415,12 @@ static tDefinitions GetTemplateObj
  register tDefinitions dist;
 # endif
 {
-  if (dist->Kind == kAlignDistribution) {
-# line 323 "Reaching.puma"
-   return dist->AlignDistribution.template_obj;
+  if (dist->Kind == kMapAlign) {
+/* line 323 "Reaching.puma" */
+   return dist->MapAlign.Alignment->Alignment.template_obj;
 
   }
-# line 328 "Reaching.puma"
+/* line 328 "Reaching.puma" */
    return obj;
 
 }
@@ -414,21 +433,22 @@ static void SetAllocateParameters
  register tTree params;
 # endif
 {
+ yyRecursion:
   if (params->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Elem->Kind == kVAR_PARAM) {
-# line 344 "Reaching.puma"
+/* line 344 "Reaching.puma" */
   {
-# line 346 "Reaching.puma"
+/* line 346 "Reaching.puma" */
    SetAllocateParam (params->BTP_LIST.Elem->VAR_PARAM.V);
-# line 347 "Reaching.puma"
-   SetAllocateParameters (params->BTP_LIST.Next);
+/* line 347 "Reaching.puma" */
+   params = params->BTP_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   }
   if (params->Kind == kBTP_EMPTY) {
-# line 350 "Reaching.puma"
+/* line 350 "Reaching.puma" */
    return;
 
   }
@@ -444,24 +464,24 @@ static void SetAllocateParam
 # endif
 {
   if (var->Kind == kINDEXED_VAR) {
-# line 361 "Reaching.puma"
+/* line 361 "Reaching.puma" */
   {
-# line 363 "Reaching.puma"
+/* line 363 "Reaching.puma" */
    ReachingAllocate (var->INDEXED_VAR.IND_EXPS);
-# line 364 "Reaching.puma"
+/* line 364 "Reaching.puma" */
    goto yyL1;
   }
 yyL1:;
 
   if (var->INDEXED_VAR.IND_VAR->Kind == kSELECTED_VAR) {
-# line 367 "Reaching.puma"
+/* line 367 "Reaching.puma" */
    return;
 
   }
   if (var->INDEXED_VAR.IND_VAR->Kind == kUSED_VAR) {
-# line 372 "Reaching.puma"
+/* line 372 "Reaching.puma" */
   {
-# line 374 "Reaching.puma"
+/* line 374 "Reaching.puma" */
    SetObjAllocated (var->INDEXED_VAR.IND_VAR->USED_VAR.VARNAME->VAR_OBJ.Object, var->INDEXED_VAR.IND_EXPS);
   }
    return;
@@ -469,13 +489,13 @@ yyL1:;
   }
   }
   if (var->Kind == kUSED_VAR) {
-# line 379 "Reaching.puma"
+/* line 379 "Reaching.puma" */
    return;
 
   }
-# line 382 "Reaching.puma"
+/* line 382 "Reaching.puma" */
   {
-# line 384 "Reaching.puma"
+/* line 384 "Reaching.puma" */
    failure_protocol (MODULE, "SetAllocateParam", var);
   }
    return;
@@ -491,21 +511,22 @@ static void ResetDeallocateParameters
  register tTree params;
 # endif
 {
+ yyRecursion:
   if (params->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Elem->Kind == kVAR_PARAM) {
-# line 395 "Reaching.puma"
+/* line 395 "Reaching.puma" */
   {
-# line 397 "Reaching.puma"
+/* line 397 "Reaching.puma" */
    ResetDeallocateParam (params->BTP_LIST.Elem->VAR_PARAM.V);
-# line 398 "Reaching.puma"
-   ResetDeallocateParameters (params->BTP_LIST.Next);
+/* line 398 "Reaching.puma" */
+   params = params->BTP_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   }
   if (params->Kind == kBTP_EMPTY) {
-# line 401 "Reaching.puma"
+/* line 401 "Reaching.puma" */
    return;
 
   }
@@ -521,22 +542,22 @@ static void ResetDeallocateParam
 # endif
 {
   if (var->Kind == kUSED_VAR) {
-# line 412 "Reaching.puma"
+/* line 412 "Reaching.puma" */
   {
-# line 414 "Reaching.puma"
+/* line 414 "Reaching.puma" */
    SetObjDeallocated (var->USED_VAR.VARNAME->VAR_OBJ.Object);
   }
    return;
 
   }
   if (var->Kind == kSELECTED_VAR) {
-# line 417 "Reaching.puma"
+/* line 417 "Reaching.puma" */
    return;
 
   }
-# line 422 "Reaching.puma"
+/* line 422 "Reaching.puma" */
   {
-# line 423 "Reaching.puma"
+/* line 423 "Reaching.puma" */
    failure_protocol (MODULE, "ResetDeallocateParam", var);
   }
    return;
@@ -553,55 +574,55 @@ static void SetObjAllocated
  register tTree indexes;
 # endif
 {
-# line 434 "Reaching.puma"
+/* line 434 "Reaching.puma" */
   {
-# line 436 "Reaching.puma"
+/* line 436 "Reaching.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   {
-# line 437 "Reaching.puma"
+/* line 437 "Reaching.puma" */
    failure_protocol (MODULE, "SetObjAllocated", NoTree);
   }
   }
    return;
 yyL1:;
 
-# line 441 "Reaching.puma"
+/* line 441 "Reaching.puma" */
   {
-# line 443 "Reaching.puma"
+/* line 443 "Reaching.puma" */
    if (! ((IsTreeObject (obj)))) goto yyL2;
   }
    return;
 yyL2:;
 
-# line 446 "Reaching.puma"
+/* line 446 "Reaching.puma" */
   {
-# line 448 "Reaching.puma"
+/* line 448 "Reaching.puma" */
    if (! ((IsAllocated (obj)))) goto yyL3;
   {
-# line 449 "Reaching.puma"
+/* line 449 "Reaching.puma" */
    error_protocol ("two reaching allocates for one object");
   }
   }
    return;
 yyL3:;
 
-# line 452 "Reaching.puma"
+/* line 452 "Reaching.puma" */
   {
-# line 454 "Reaching.puma"
+/* line 454 "Reaching.puma" */
    if (! ((allocate_top == MAX_ALLOCATES))) goto yyL4;
   {
-# line 455 "Reaching.puma"
+/* line 455 "Reaching.puma" */
    error_protocol ("too many allocates in one unit");
-# line 456 "Reaching.puma"
+/* line 456 "Reaching.puma" */
    kill_in_protocol ();
   }
   }
    return;
 yyL4:;
 
-# line 459 "Reaching.puma"
+/* line 459 "Reaching.puma" */
   {
-# line 461 "Reaching.puma"
+/* line 461 "Reaching.puma" */
    InsertObjAllocate (obj, indexes);
   }
    return;
@@ -617,41 +638,41 @@ static void SetObjDeallocated
  register tDefinitions obj;
 # endif
 {
-# line 472 "Reaching.puma"
+/* line 472 "Reaching.puma" */
   {
-# line 474 "Reaching.puma"
+/* line 474 "Reaching.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   {
-# line 475 "Reaching.puma"
+/* line 475 "Reaching.puma" */
    failure_protocol (MODULE, "SetObjDeallocated", NoTree);
   }
   }
    return;
 yyL1:;
 
-# line 478 "Reaching.puma"
+/* line 478 "Reaching.puma" */
   {
-# line 480 "Reaching.puma"
+/* line 480 "Reaching.puma" */
    if (! ((IsAllocated (obj)))) goto yyL2;
   {
-# line 481 "Reaching.puma"
+/* line 481 "Reaching.puma" */
    RemoveObjAllocate (obj);
   }
   }
    return;
 yyL2:;
 
-# line 484 "Reaching.puma"
+/* line 484 "Reaching.puma" */
   {
-# line 486 "Reaching.puma"
+/* line 486 "Reaching.puma" */
    if (! ((IsUsedObject (obj, GetCurrentUnitObject ())))) goto yyL3;
   }
    return;
 yyL3:;
 
-# line 489 "Reaching.puma"
+/* line 489 "Reaching.puma" */
   {
-# line 491 "Reaching.puma"
+/* line 491 "Reaching.puma" */
    warning_protocol ("DEALLOCATE: obj is not allocated");
   }
    return;
@@ -668,27 +689,28 @@ static void SetRemapping
  register tDefinitions map_info;
 # endif
 {
+ yyRecursion:
   if (distributees->Kind == kBTV_LIST) {
   if (distributees->BTV_LIST.Elem->Kind == kUSED_VAR) {
-# line 502 "Reaching.puma"
+/* line 502 "Reaching.puma" */
   {
-# line 504 "Reaching.puma"
+/* line 504 "Reaching.puma" */
    SetObjDistribution (distributees->BTV_LIST.Elem->USED_VAR.VARNAME->VAR_OBJ.Object, map_info);
-# line 505 "Reaching.puma"
-   SetRemapping (distributees->BTV_LIST.Next, map_info);
+/* line 505 "Reaching.puma" */
+   distributees = distributees->BTV_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   }
   if (distributees->Kind == kBTV_EMPTY) {
-# line 508 "Reaching.puma"
+/* line 508 "Reaching.puma" */
    return;
 
   }
-# line 511 "Reaching.puma"
+/* line 511 "Reaching.puma" */
   {
-# line 513 "Reaching.puma"
+/* line 513 "Reaching.puma" */
    failure_protocol (MODULE, "SetRemapping", distributees);
   }
    return;
@@ -705,19 +727,19 @@ static tTree GetReachingInfo
  register tDefinitions obj;
 # endif
 {
-# line 526 "Reaching.puma"
+/* line 526 "Reaching.puma" */
   {
-# line 528 "Reaching.puma"
+/* line 528 "Reaching.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   {
-# line 529 "Reaching.puma"
+/* line 529 "Reaching.puma" */
    failure_protocol (MODULE, "GetReachingAllocate", uv);
   }
   }
    return NoTree;
 yyL1:;
 
-# line 535 "Reaching.puma"
+/* line 535 "Reaching.puma" */
  {
   tDefinitions template_obj;
   tTree var_alloc;
@@ -725,30 +747,18 @@ yyL1:;
   tDefinitions var_dist;
   tDefinitions temp_dist;
   {
-# line 537 "Reaching.puma"
-
-# line 539 "Reaching.puma"
-
-# line 540 "Reaching.puma"
-
-# line 541 "Reaching.puma"
-
-# line 542 "Reaching.puma"
-
-# line 547 "Reaching.puma"
+/* line 547 "Reaching.puma" */
    var_dist = GetReachingDistribution (obj);
-# line 548 "Reaching.puma"
+/* line 548 "Reaching.puma" */
    var_alloc = GetReachingAllocate (uv, obj);
-# line 552 "Reaching.puma"
+/* line 552 "Reaching.puma" */
    template_obj = GetTemplateObj (obj, var_dist);
-# line 554 "Reaching.puma"
+/* line 554 "Reaching.puma" */
    temp_dist = GetReachingDistribution (template_obj);
-# line 555 "Reaching.puma"
+/* line 555 "Reaching.puma" */
    temp_alloc = GetReachingAllocate (NoTree, template_obj);
   }
-  {
    return mREACHING_INFO (var_alloc, temp_alloc, var_dist, temp_dist);
-  }
  }
 
 }
@@ -761,41 +771,37 @@ static tDefinitions GetReachingDistribution
  register tDefinitions obj;
 # endif
 {
-# line 568 "Reaching.puma"
+/* line 568 "Reaching.puma" */
  {
   int pos;
   {
-# line 570 "Reaching.puma"
-
-# line 572 "Reaching.puma"
+/* line 572 "Reaching.puma" */
    pos = GetDistPosition (obj);
-# line 574 "Reaching.puma"
+/* line 574 "Reaching.puma" */
    if (! ((pos < distribution_top))) goto yyL1;
   }
-  {
    return distribution_val [pos];
-  }
  }
 yyL1:;
 
   if (obj->Kind == kVarObject) {
-# line 579 "Reaching.puma"
-   return obj->VarObject.Dist;
+/* line 579 "Reaching.puma" */
+   return obj->VarObject.Dist->Mapping.spec;
 
   }
   if (obj->Kind == kTemplateObject) {
-# line 583 "Reaching.puma"
-   return obj->TemplateObject.Dist;
+/* line 584 "Reaching.puma" */
+   return obj->TemplateObject.Dist->Mapping.spec;
 
   }
   if (obj->Kind == kRaggedObject) {
-# line 587 "Reaching.puma"
-   return obj->RaggedObject.Dist;
+/* line 589 "Reaching.puma" */
+   return obj->RaggedObject.Dist->Mapping.spec;
 
   }
-# line 591 "Reaching.puma"
+/* line 594 "Reaching.puma" */
   {
-# line 592 "Reaching.puma"
+/* line 595 "Reaching.puma" */
    failure_protocol (MODULE, "GetReachingDistribution", obj->Object.decl);
   }
    return NoDefinitions;
@@ -811,73 +817,61 @@ static tTree GetReachingAllocate
  register tDefinitions obj;
 # endif
 {
-# line 606 "Reaching.puma"
+/* line 609 "Reaching.puma" */
   {
-# line 608 "Reaching.puma"
+/* line 611 "Reaching.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   {
-# line 609 "Reaching.puma"
+/* line 612 "Reaching.puma" */
    failure_protocol (MODULE, "GetReachingAllocate", uv);
   }
   }
    return NoTree;
 yyL1:;
 
-# line 613 "Reaching.puma"
+/* line 616 "Reaching.puma" */
  {
   tTree indexes;
   {
-# line 615 "Reaching.puma"
-
-# line 617 "Reaching.puma"
-   indexes = ArrayFormals (obj);
-# line 619 "Reaching.puma"
+/* line 620 "Reaching.puma" */
+   indexes = GetObjDimension (obj);
+/* line 622 "Reaching.puma" */
    if (! ((! IsDeferredShape (indexes)))) goto yyL2;
   {
-# line 620 "Reaching.puma"
+/* line 623 "Reaching.puma" */
    if (! ((! IsAssumedShape (indexes)))) goto yyL2;
   }
   }
-  {
    return indexes;
-  }
  }
 yyL2:;
 
-# line 627 "Reaching.puma"
+/* line 630 "Reaching.puma" */
  {
   int pos;
   {
-# line 629 "Reaching.puma"
-
-# line 631 "Reaching.puma"
+/* line 634 "Reaching.puma" */
    pos = GetAllocPosition (obj);
-# line 633 "Reaching.puma"
+/* line 636 "Reaching.puma" */
    if (! ((pos < allocate_top))) goto yyL3;
   }
-  {
    return allocate_val [pos];
-  }
  }
 yyL3:;
 
-# line 638 "Reaching.puma"
+/* line 641 "Reaching.puma" */
  {
   tTree indexes;
   {
-# line 640 "Reaching.puma"
-
-# line 642 "Reaching.puma"
+/* line 645 "Reaching.puma" */
    NoReachingWarning (obj);
-# line 644 "Reaching.puma"
+/* line 647 "Reaching.puma" */
  indexes = MakeIndexBoundaries (MakeUsedVarTree (uv, obj),
-                                    ArrayFormals    (obj), 1    );
+                                    GetObjDimension (obj), 1    );
      SetObjAllocated (obj, indexes);
    
   }
-  {
    return indexes;
-  }
  }
 
 }
@@ -890,50 +884,50 @@ static void NoReachingWarning
  register tDefinitions obj;
 # endif
 {
-# line 660 "Reaching.puma"
+/* line 663 "Reaching.puma" */
 
-char string [MAXID_LENGTH];
+char string [MAX_ID_LENGTH];
 char msg    [MAX_LINE_LENGTH];
 
-# line 665 "Reaching.puma"
+/* line 668 "Reaching.puma" */
   {
-# line 667 "Reaching.puma"
+/* line 670 "Reaching.puma" */
    if (! ((IsVarAssumedShape (obj)))) goto yyL1;
   {
-# line 669 "Reaching.puma"
+/* line 672 "Reaching.puma" */
    GetString (obj->Object.Ident, string);
-# line 670 "Reaching.puma"
+/* line 673 "Reaching.puma" */
    sprintf (msg, "no info about sizes of assumed shape array %s", string);
-# line 671 "Reaching.puma"
+/* line 674 "Reaching.puma" */
    warning_protocol (msg);
   }
   }
    return;
 yyL1:;
 
-# line 674 "Reaching.puma"
+/* line 677 "Reaching.puma" */
   {
-# line 676 "Reaching.puma"
+/* line 679 "Reaching.puma" */
    if (! ((IsUsedObject (obj, GetCurrentUnitObject ())))) goto yyL2;
   {
-# line 678 "Reaching.puma"
+/* line 681 "Reaching.puma" */
    GetString (obj->Object.Ident, string);
-# line 679 "Reaching.puma"
+/* line 682 "Reaching.puma" */
    sprintf (msg, "no info about sizes for global object %s", string);
-# line 680 "Reaching.puma"
+/* line 683 "Reaching.puma" */
    warning_protocol (msg);
   }
   }
    return;
 yyL2:;
 
-# line 683 "Reaching.puma"
+/* line 686 "Reaching.puma" */
   {
-# line 685 "Reaching.puma"
+/* line 688 "Reaching.puma" */
    GetString (obj->Object.Ident, string);
-# line 686 "Reaching.puma"
+/* line 689 "Reaching.puma" */
    sprintf (msg, "no reaching allocate for local object %s", string);
-# line 687 "Reaching.puma"
+/* line 690 "Reaching.puma" */
    warning_protocol (msg);
   }
    return;
@@ -950,42 +944,38 @@ static tTree MakeUsedVarTree
  register tDefinitions obj;
 # endif
 {
-# line 698 "Reaching.puma"
+/* line 701 "Reaching.puma" */
   {
-# line 700 "Reaching.puma"
+/* line 703 "Reaching.puma" */
    if (! ((uv != NoTree))) goto yyL1;
   }
    return uv;
 yyL1:;
 
-# line 705 "Reaching.puma"
+/* line 708 "Reaching.puma" */
  {
   tTree v;
   {
-# line 707 "Reaching.puma"
-
-# line 709 "Reaching.puma"
+/* line 712 "Reaching.puma" */
  v = mVAR_OBJ (0, obj->Object.Ident);
      v->VAR_OBJ.Object = obj;
    
   }
-  {
    return mUSED_VAR (v);
-  }
  }
 
 }
 
 static void DeallocateCheck
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 728 "Reaching.puma"
+/* line 731 "Reaching.puma" */
   {
-# line 730 "Reaching.puma"
+/* line 733 "Reaching.puma" */
  int i;
     for (i=allocate_top-1; i>=0; i--)
        DeallocateCheckObj (allocate_obj[i]);
@@ -1004,26 +994,26 @@ static void DeallocateCheckObj
  register tDefinitions obj;
 # endif
 {
-# line 738 "Reaching.puma"
+/* line 741 "Reaching.puma" */
   {
-# line 740 "Reaching.puma"
+/* line 743 "Reaching.puma" */
    if (! ((IsVarAssumedShape (obj)))) goto yyL1;
   }
    return;
 yyL1:;
 
-# line 743 "Reaching.puma"
+/* line 746 "Reaching.puma" */
   {
-# line 745 "Reaching.puma"
+/* line 748 "Reaching.puma" */
    if (! ((IsUsedObject (obj, GetCurrentUnitObject ())))) goto yyL2;
   }
    return;
 yyL2:;
 
-# line 748 "Reaching.puma"
+/* line 751 "Reaching.puma" */
   {
-# line 750 "Reaching.puma"
- char msg[MAX_LINE_LENGTH], name[MAXID_LENGTH];
+/* line 753 "Reaching.puma" */
+ char msg[MAX_LINE_LENGTH], name[MAX_ID_LENGTH];
 
      
 
@@ -1037,7 +1027,7 @@ yyL2:;
 ;
 }
 
-static bool IsLocalObject
+static rbool IsLocalObject
 # if defined __STDC__ | defined __cplusplus
 (register tDefinitions obj)
 # else
@@ -1045,15 +1035,15 @@ static bool IsLocalObject
  register tDefinitions obj;
 # endif
 {
-# line 768 "Reaching.puma"
+/* line 771 "Reaching.puma" */
   {
-# line 770 "Reaching.puma"
+/* line 773 "Reaching.puma" */
    if (! ((obj->Object.in == GetCurrentUnitObject ()))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
 static tTree MakeIndexBoundaries
@@ -1067,16 +1057,17 @@ static tTree MakeIndexBoundaries
 # endif
 {
   if (formals->Kind == kSHAPE_LIST) {
-# line 783 "Reaching.puma"
+/* line 786 "Reaching.puma" */
    return mBTE_LIST (MakeIndexBound (var, formals->SHAPE_LIST.Elem, n), MakeIndexBoundaries (var, formals->SHAPE_LIST.Next, n + 1));
 
   }
   if (formals->Kind == kSHAPE_EMPTY) {
-# line 789 "Reaching.puma"
+/* line 792 "Reaching.puma" */
    return mBTE_EMPTY ();
 
   }
  yyAbort ("MakeIndexBoundaries");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static tTree MakeIndexBound
@@ -1090,33 +1081,25 @@ static tTree MakeIndexBound
 # endif
 {
   if (shape->Kind == kASSUMED_SHAPE) {
-# line 800 "Reaching.puma"
+/* line 803 "Reaching.puma" */
  {
   int local;
   {
-# line 802 "Reaching.puma"
-
-# line 804 "Reaching.puma"
+/* line 807 "Reaching.puma" */
    local = 0;
   }
-  {
    return mSLICE_EXP (shape->ASSUMED_SHAPE.LOWER, mBOUND_EXP (var, dim, 1, local), mDUMMY_EXP ());
-  }
  }
 
   }
-# line 811 "Reaching.puma"
+/* line 814 "Reaching.puma" */
  {
   int local;
   {
-# line 813 "Reaching.puma"
-
-# line 815 "Reaching.puma"
+/* line 818 "Reaching.puma" */
    local = 0;
   }
-  {
    return mSLICE_EXP (mBOUND_EXP (var, dim, 0, local), mBOUND_EXP (var, dim, 1, local), mDUMMY_EXP ());
-  }
  }
 
 }
@@ -1131,48 +1114,48 @@ void PrintReachingInfo
 {
   if (uv->Kind == kUSED_VAR) {
   if (uv->USED_VAR.VARNAME->VAR_OBJ.Reaching->Kind == kREACHING_INFO) {
-# line 830 "Reaching.puma"
+/* line 833 "Reaching.puma" */
   {
-# line 832 "Reaching.puma"
+/* line 835 "Reaching.puma" */
    printf ("reaching info for ");
-# line 832 "Reaching.puma"
+/* line 835 "Reaching.puma" */
    FileUnparse (stdout, uv);
-# line 832 "Reaching.puma"
+/* line 835 "Reaching.puma" */
    printf ("\n");
-# line 833 "Reaching.puma"
+/* line 836 "Reaching.puma" */
    printf (" size var  ");
-# line 833 "Reaching.puma"
+/* line 836 "Reaching.puma" */
    FileUnparse (stdout, uv->USED_VAR.VARNAME->VAR_OBJ.Reaching->REACHING_INFO.var_allocate);
-# line 833 "Reaching.puma"
+/* line 836 "Reaching.puma" */
    printf ("\n");
-# line 834 "Reaching.puma"
+/* line 837 "Reaching.puma" */
    printf (" size temp ");
-# line 834 "Reaching.puma"
+/* line 837 "Reaching.puma" */
    FileUnparse (stdout, uv->USED_VAR.VARNAME->VAR_OBJ.Reaching->REACHING_INFO.temp_allocate);
-# line 834 "Reaching.puma"
+/* line 837 "Reaching.puma" */
    printf ("\n");
-# line 836 "Reaching.puma"
+/* line 839 "Reaching.puma" */
    SemFile = stdout;
-# line 838 "Reaching.puma"
+/* line 841 "Reaching.puma" */
    printf (" dist var  ");
-# line 838 "Reaching.puma"
-   ShowDistribution (uv->USED_VAR.VARNAME->VAR_OBJ.Reaching->REACHING_INFO.var_distribution);
-# line 838 "Reaching.puma"
+/* line 841 "Reaching.puma" */
+   ShowSpecifiedMapping (uv->USED_VAR.VARNAME->VAR_OBJ.Reaching->REACHING_INFO.var_distribution);
+/* line 841 "Reaching.puma" */
    printf ("\n");
-# line 839 "Reaching.puma"
+/* line 842 "Reaching.puma" */
    printf (" dist temp ");
-# line 839 "Reaching.puma"
-   ShowDistribution (uv->USED_VAR.VARNAME->VAR_OBJ.Reaching->REACHING_INFO.temp_distribution);
-# line 839 "Reaching.puma"
+/* line 842 "Reaching.puma" */
+   ShowSpecifiedMapping (uv->USED_VAR.VARNAME->VAR_OBJ.Reaching->REACHING_INFO.temp_distribution);
+/* line 842 "Reaching.puma" */
    printf ("\n");
   }
    return;
 
   }
   }
-# line 842 "Reaching.puma"
+/* line 845 "Reaching.puma" */
   {
-# line 844 "Reaching.puma"
+/* line 847 "Reaching.puma" */
    failure_protocol (MODULE, "PrintReachInfo", uv);
   }
    return;
@@ -1180,10 +1163,10 @@ void PrintReachingInfo
 ;
 }
 
-void BeginReaching ()
+void BeginReaching ARGS ((void))
 {
 }
 
-void CloseReaching ()
+void CloseReaching ARGS ((void))
 {
 }
