@@ -1,6 +1,13 @@
 # include "ReductionComm.h"
-# include "yyReductionComm.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,34 +15,31 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 21 "ReductionComm.puma"
+/* line 21 "ReductionComm.puma" */
 
 
 # undef DEBUG
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
@@ -49,16 +53,32 @@
 # include "Descriptor.h"   /* PrintableDescriptorVar      */
 # include "HomeDescriptor.h"   /* IsSerialDescriptor      */
 # include "MoveDescriptor.h"   /* IsSerialDescriptor      */
+# include "Nesting.h" 
+# include "TreeOps.h" 
 
 # define MODULE "ReductionComm"
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* ReductionComm_Exit) () = yyExit;
+# include "yyReductionComm.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* ReductionComm_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -67,24 +87,25 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module ReductionComm, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module ReductionComm, routine %s failed\n",
+  yyFunction);
  ReductionComm_Exit ();
 }
 
-void ReductionCommunication ARGS((tTree body));
-static tTree TranslateReduction ARGS((tTree stmt, pvar home));
-static tTree GenerateReduction ARGS((pvar active_home, pvar red_home, tTree func, tTree var));
-static tTree GenerateTopReduction ARGS((pvar active_home, pvar red_home, tTree func, tTree var));
-static tTree GenerateTopBroadcast ARGS((tTree var, pvar home));
-static tTree MakeRedDimSpecs ARGS((pvar active_home));
-static tTree MakeNewRedDimSpecs ARGS((pvar active_home, pvar sub_home));
-static tTree MakeReplicatedOnSpecs ARGS((pvar all_home, pvar red_home, pvar var_home));
-static void FindReductionHome ARGS((tTree t, pvar home, tTree red_var, pvar red_home, bool * yyP1));
-static void CombineReductionHome ARGS((pvar home1, bool found1, pvar home2, bool found2, bool * yyP2));
-static bool SameReductionVar ARGS((tTree vl1, tTree vl2));
-static bool SameReplication ARGS((pvar home1, pvar home2));
-static void MakeFullTopHome ARGS((pvar home));
-static bool OnMoreProcs ARGS((pvar home, int pdim));
+void ReductionCommunication ARGS ((tTree body));
+static tTree TranslateReduction ARGS ((tTree stmt, pvar home));
+static tTree GenerateReduction ARGS ((pvar active_home, pvar red_home, tTree func, tTree var));
+static tTree GenerateTopReduction ARGS ((pvar active_home, pvar red_home, tTree func, tTree var));
+static tTree GenerateTopBroadcast ARGS ((tTree var, pvar home));
+static tTree MakeRedDimSpecs ARGS ((pvar active_home));
+static tTree MakeNewRedDimSpecs ARGS ((pvar active_home, pvar sub_home));
+static tTree MakeReplicatedOnSpecs ARGS ((pvar all_home, pvar red_home, pvar var_home));
+static void FindReductionHome ARGS ((tTree t, pvar home, tTree red_var, pvar red_home, rbool * yyP1));
+static void CombineReductionHome ARGS ((pvar home1, rbool found1, pvar home2, rbool found2, rbool * yyP2));
+static rbool SameReductionVar ARGS ((tTree vl1, tTree vl2));
+static rbool SameReplication ARGS ((pvar home1, pvar home2));
+static void MakeFullTopHome ARGS ((pvar home));
+static rbool OnMoreProcs ARGS ((pvar home, int pdim));
 
 void ReductionCommunication
 # if defined __STDC__ | defined __cplusplus
@@ -95,24 +116,22 @@ void ReductionCommunication
 # endif
 {
   if (body->Kind == kBODY_NODE) {
-# line 61 "ReductionComm.puma"
+/* line 63 "ReductionComm.puma" */
  {
   var_descriptor home;
   {
-# line 63 "ReductionComm.puma"
-
-# line 65 "ReductionComm.puma"
+/* line 67 "ReductionComm.puma" */
    MakeReplicatedDescriptor (& home);
-# line 67 "ReductionComm.puma"
+/* line 69 "ReductionComm.puma" */
  body->BODY_NODE.STATS = HomeReplaceAST (body->BODY_NODE.STATS, &home, TranslateReduction); 
   }
    return;
  }
 
   }
-# line 70 "ReductionComm.puma"
+/* line 72 "ReductionComm.puma" */
   {
-# line 72 "ReductionComm.puma"
+/* line 74 "ReductionComm.puma" */
    failure_protocol (MODULE, "ReductionCommunication", body);
   }
    return;
@@ -130,44 +149,38 @@ static tTree TranslateReduction
 # endif
 {
   if (stmt->Kind == kACF_REDUCTION) {
-# line 122 "ReductionComm.puma"
+/* line 124 "ReductionComm.puma" */
  {
   tTree comm;
   var_descriptor red_home;
-  bool yyV1;
+  rbool yyV1;
   {
-# line 126 "ReductionComm.puma"
-
-# line 128 "ReductionComm.puma"
-
-# line 130 "ReductionComm.puma"
+/* line 132 "ReductionComm.puma" */
    set_protocol_stmt (stmt);
-# line 132 "ReductionComm.puma"
+/* line 134 "ReductionComm.puma" */
    FindReductionHome (stmt->ACF_REDUCTION.REDUCTION_BODY, home, stmt->ACF_REDUCTION.REDUCTION_VAR, & red_home, & yyV1);
-# line 134 "ReductionComm.puma"
+/* line 136 "ReductionComm.puma" */
  if (!yyV1)
-       { warning_protocol ("reduction variable not used");
+       { 
          MakeReplicatedDescriptor (&red_home);
        }
    
-# line 140 "ReductionComm.puma"
+/* line 142 "ReductionComm.puma" */
    tree_protocol ("generate reduction, var = ", stmt->ACF_REDUCTION.REDUCTION_VAR);
-# line 141 "ReductionComm.puma"
+/* line 143 "ReductionComm.puma" */
    tree_protocol ("home of reduction is    = ", PrintableDescriptorVar (home));
-# line 142 "ReductionComm.puma"
+/* line 144 "ReductionComm.puma" */
    tree_protocol ("home of reduce    is    = ", PrintableDescriptorVar (& red_home));
-# line 145 "ReductionComm.puma"
+/* line 147 "ReductionComm.puma" */
    comm = GenerateReduction (home, & red_home, stmt->ACF_REDUCTION.REDUCTION_FUNC, stmt->ACF_REDUCTION.REDUCTION_VAR);
-# line 147 "ReductionComm.puma"
+/* line 149 "ReductionComm.puma" */
    tree_protocol ("==> generated reduction = \n ", comm);
   }
-  {
    return CombineACF (stmt, comm);
-  }
  }
 
   }
-# line 152 "ReductionComm.puma"
+/* line 154 "ReductionComm.puma" */
    return stmt;
 
 }
@@ -183,64 +196,60 @@ static tTree GenerateReduction
  register tTree var;
 # endif
 {
-# line 183 "ReductionComm.puma"
+/* line 185 "ReductionComm.puma" */
   {
-# line 185 "ReductionComm.puma"
+/* line 187 "ReductionComm.puma" */
    if (! ((GetCurrentModel () == HPF_SERIAL))) goto yyL1;
   }
    return NoTree;
 yyL1:;
 
-# line 190 "ReductionComm.puma"
+/* line 192 "ReductionComm.puma" */
   {
-# line 192 "ReductionComm.puma"
+/* line 194 "ReductionComm.puma" */
    if (! ((IsReplicatedDescriptor (red_home)))) goto yyL2;
   }
    return NoTree;
 yyL2:;
 
-# line 200 "ReductionComm.puma"
+/* line 202 "ReductionComm.puma" */
   {
-# line 202 "ReductionComm.puma"
+/* line 204 "ReductionComm.puma" */
    if (! ((VDIsSame (active_home, red_home)))) goto yyL3;
   }
    return NoTree;
 yyL3:;
 
-# line 210 "ReductionComm.puma"
+/* line 212 "ReductionComm.puma" */
   {
-# line 212 "ReductionComm.puma"
+/* line 214 "ReductionComm.puma" */
    if (! ((TreeWriteDistribution (var) == 0))) goto yyL4;
   {
-# line 214 "ReductionComm.puma"
+/* line 216 "ReductionComm.puma" */
    if (! ((IsSerialDescriptor (red_home)))) goto yyL4;
   }
   }
    return mACF_BASIC (mREDUCTION_STMT (func, var, MakeOnTopology (active_home), MakeRedDimSpecs (active_home)));
 yyL4:;
 
-# line 223 "ReductionComm.puma"
+/* line 225 "ReductionComm.puma" */
  {
   var_descriptor full_top_home;
   {
-# line 225 "ReductionComm.puma"
+/* line 227 "ReductionComm.puma" */
    if (! ((IsReplicatedDescriptor (active_home)))) goto yyL5;
   {
-# line 229 "ReductionComm.puma"
-
-# line 231 "ReductionComm.puma"
+/* line 233 "ReductionComm.puma" */
  full_top_home = *red_home; 
-# line 233 "ReductionComm.puma"
+/* line 235 "ReductionComm.puma" */
    MakeFullTopHome (& full_top_home);
   }
   }
-  {
-   return CombineACF (GenerateTopBroadcast (var, & full_top_home), GenerateTopReduction (& full_top_home, red_home, func, var));
-  }
+   return CombineACF (GenerateTopReduction (& full_top_home, red_home, func, var), GenerateTopBroadcast (var, & full_top_home));
  }
 yyL5:;
 
-# line 243 "ReductionComm.puma"
+/* line 244 "ReductionComm.puma" */
    return GenerateTopReduction (active_home, red_home, func, var);
 
 }
@@ -256,57 +265,44 @@ static tTree GenerateTopReduction
  register tTree var;
 # endif
 {
-# line 276 "ReductionComm.puma"
+/* line 277 "ReductionComm.puma" */
  {
   tTree on_top;
   tTree on_spec;
   {
-# line 278 "ReductionComm.puma"
+/* line 279 "ReductionComm.puma" */
    if (! ((TreeWriteDistribution (var) == 0))) goto yyL1;
   {
-# line 280 "ReductionComm.puma"
-
-# line 281 "ReductionComm.puma"
-
-# line 283 "ReductionComm.puma"
+/* line 284 "ReductionComm.puma" */
    on_top = MakeOnTopology (active_home);
-# line 284 "ReductionComm.puma"
+/* line 285 "ReductionComm.puma" */
    on_spec = MakeNewRedDimSpecs (active_home, red_home);
   }
   }
-  {
    return mACF_BASIC (mREDUCTION_STMT (func, var, on_top, on_spec));
-  }
  }
 yyL1:;
 
   if (var->Kind == kBTV_LIST) {
-# line 289 "ReductionComm.puma"
+/* line 290 "ReductionComm.puma" */
  {
   var_descriptor var_home;
   tTree on_top;
   tTree on_spec;
   {
-# line 291 "ReductionComm.puma"
-
-# line 293 "ReductionComm.puma"
+/* line 294 "ReductionComm.puma" */
    SetVarDescriptor (var->BTV_LIST.Elem, & var_home);
-# line 295 "ReductionComm.puma"
-
-# line 296 "ReductionComm.puma"
-
-# line 298 "ReductionComm.puma"
+/* line 299 "ReductionComm.puma" */
    on_top = MakeOnTopology (active_home);
-# line 299 "ReductionComm.puma"
+/* line 300 "ReductionComm.puma" */
    on_spec = MakeReplicatedOnSpecs (active_home, red_home, & var_home);
   }
-  {
    return mACF_BASIC (mREDUCTION_STMT (func, var, on_top, on_spec));
-  }
  }
 
   }
  yyAbort ("GenerateTopReduction");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static tTree GenerateTopBroadcast
@@ -318,31 +314,31 @@ static tTree GenerateTopBroadcast
  pvar home;
 # endif
 {
-# line 315 "ReductionComm.puma"
+/* line 316 "ReductionComm.puma" */
   {
-# line 317 "ReductionComm.puma"
+/* line 318 "ReductionComm.puma" */
    if (! ((IsReplicatedDescriptor (home)))) goto yyL1;
   }
    return NoTree;
 yyL1:;
 
-# line 322 "ReductionComm.puma"
+/* line 323 "ReductionComm.puma" */
   {
-# line 324 "ReductionComm.puma"
+/* line 325 "ReductionComm.puma" */
    if (! ((IsHostDescriptor (home)))) goto yyL2;
   }
    return mACF_BASIC (mBROADCAST_STMT (CopyTree (var), mON_HOST (), mON_EMPTY ()));
 yyL2:;
 
-# line 330 "ReductionComm.puma"
+/* line 331 "ReductionComm.puma" */
   {
-# line 332 "ReductionComm.puma"
+/* line 333 "ReductionComm.puma" */
    if (! ((IsFullTopologyObj (home -> topology_obj)))) goto yyL3;
   }
    return NoTree;
 yyL3:;
 
-# line 337 "ReductionComm.puma"
+/* line 338 "ReductionComm.puma" */
    return mACF_BASIC (mBROADCAST_STMT (CopyTree (var), MakeOnTopology (home), mON_EMPTY ()));
 
 }
@@ -355,13 +351,11 @@ static tTree MakeRedDimSpecs
  pvar active_home;
 # endif
 {
-# line 355 "ReductionComm.puma"
+/* line 356 "ReductionComm.puma" */
  {
   tTree on_specs;
   {
-# line 357 "ReductionComm.puma"
-
-# line 359 "ReductionComm.puma"
+/* line 360 "ReductionComm.puma" */
  int pdim;
 
      on_specs = mON_EMPTY ();
@@ -371,9 +365,7 @@ static tTree MakeRedDimSpecs
           on_specs = mON_LIST (MakeOnVal (active_home, pdim), on_specs);
    
   }
-  {
    return on_specs;
-  }
  }
 
 }
@@ -387,16 +379,12 @@ static tTree MakeNewRedDimSpecs
  pvar sub_home;
 # endif
 {
-# line 383 "ReductionComm.puma"
+/* line 384 "ReductionComm.puma" */
  {
   tTree specs;
   int pdim;
   {
-# line 385 "ReductionComm.puma"
-
-# line 386 "ReductionComm.puma"
-
-# line 388 "ReductionComm.puma"
+/* line 389 "ReductionComm.puma" */
  specs = mON_EMPTY();
 
     for (pdim = 1; pdim <= active_home->topology_rank; pdim++)
@@ -407,9 +395,7 @@ static tTree MakeNewRedDimSpecs
              specs = mON_LIST (MakeOnVal (active_home, pdim), specs);
   
   }
-  {
    return specs;
-  }
  }
 
 }
@@ -424,16 +410,12 @@ static tTree MakeReplicatedOnSpecs
  pvar var_home;
 # endif
 {
-# line 410 "ReductionComm.puma"
+/* line 411 "ReductionComm.puma" */
  {
   tTree specs;
   int pdim;
   {
-# line 412 "ReductionComm.puma"
-
-# line 413 "ReductionComm.puma"
-
-# line 415 "ReductionComm.puma"
+/* line 416 "ReductionComm.puma" */
  specs = mON_EMPTY();
 
     for (pdim = 1; pdim <= all_home->topology_rank; pdim++)
@@ -448,62 +430,58 @@ static tTree MakeReplicatedOnSpecs
        }
   
   }
-  {
    return specs;
-  }
  }
 
 }
 
 static void FindReductionHome
 # if defined __STDC__ | defined __cplusplus
-(register tTree t, pvar home, register tTree red_var, pvar red_home, register bool * yyP1)
+(register tTree t, pvar home, register tTree red_var, pvar red_home, register rbool * yyP1)
 # else
 (t, home, red_var, red_home, yyP1)
  register tTree t;
  pvar home;
  register tTree red_var;
  pvar red_home;
- register bool * yyP1;
+ register rbool * yyP1;
 # endif
 {
 
   switch (t->Kind) {
   case kACF_BASIC:
   if (t->ACF_BASIC.BASIC_STMT->Kind == kREDUCE_STMT) {
-# line 447 "ReductionComm.puma"
+/* line 448 "ReductionComm.puma" */
   {
-# line 449 "ReductionComm.puma"
+/* line 450 "ReductionComm.puma" */
    if (! ((SameReductionVar (red_var, t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_PARAMS)))) goto yyL1;
   {
-# line 451 "ReductionComm.puma"
+/* line 452 "ReductionComm.puma" */
  *red_home = *home; 
   }
   }
-   * yyP1 = true;
+   * yyP1 = rtrue;
    return;
 yyL1:;
 
   }
-# line 454 "ReductionComm.puma"
-   * yyP1 = false;
+/* line 455 "ReductionComm.puma" */
+   * yyP1 = rfalse;
    return;
 
   case kACF_LIST:
-# line 457 "ReductionComm.puma"
+/* line 458 "ReductionComm.puma" */
  {
   var_descriptor red_home1;
-  bool yyV1;
-  bool yyV2;
-  bool yyV3;
+  rbool yyV1;
+  rbool yyV2;
+  rbool yyV3;
   {
-# line 459 "ReductionComm.puma"
-
-# line 461 "ReductionComm.puma"
+/* line 462 "ReductionComm.puma" */
    FindReductionHome (t->ACF_LIST.Elem, home, red_var, red_home, & yyV1);
-# line 462 "ReductionComm.puma"
+/* line 463 "ReductionComm.puma" */
    FindReductionHome (t->ACF_LIST.Next, home, red_var, & red_home1, & yyV2);
-# line 464 "ReductionComm.puma"
+/* line 465 "ReductionComm.puma" */
    CombineReductionHome (red_home, yyV1, & red_home1, yyV2, & yyV3);
   }
    * yyP1 = yyV3;
@@ -511,22 +489,25 @@ yyL1:;
  }
 
   case kACF_EMPTY:
-# line 467 "ReductionComm.puma"
-   * yyP1 = false;
+/* line 468 "ReductionComm.puma" */
+   * yyP1 = rfalse;
+   return;
+
+  case kACF_DUMMY:
+/* line 471 "ReductionComm.puma" */
+   * yyP1 = rfalse;
    return;
 
   case kACF_HOME:
   if (t->ACF_HOME.HOME_VAR->Kind == kON_HOST_CLAUSE) {
-# line 470 "ReductionComm.puma"
+/* line 474 "ReductionComm.puma" */
  {
   var_descriptor on_home;
-  bool yyV1;
+  rbool yyV1;
   {
-# line 473 "ReductionComm.puma"
-
-# line 475 "ReductionComm.puma"
+/* line 479 "ReductionComm.puma" */
    MakeHostDescriptor (& on_home);
-# line 477 "ReductionComm.puma"
+/* line 481 "ReductionComm.puma" */
    FindReductionHome (t->ACF_HOME.HOME_BODY, & on_home, red_var, red_home, & yyV1);
   }
    * yyP1 = yyV1;
@@ -535,16 +516,14 @@ yyL1:;
 
   }
   if (t->ACF_HOME.HOME_VAR->Kind == kON_VAR_CLAUSE) {
-# line 480 "ReductionComm.puma"
+/* line 484 "ReductionComm.puma" */
  {
   var_descriptor on_home;
-  bool yyV1;
+  rbool yyV1;
   {
-# line 483 "ReductionComm.puma"
-
-# line 485 "ReductionComm.puma"
+/* line 489 "ReductionComm.puma" */
    SetVarDescriptor (t->ACF_HOME.HOME_VAR->ON_VAR_CLAUSE.ON_VAR, & on_home);
-# line 487 "ReductionComm.puma"
+/* line 491 "ReductionComm.puma" */
    FindReductionHome (t->ACF_HOME.HOME_BODY, & on_home, red_var, red_home, & yyV1);
   }
    * yyP1 = yyV1;
@@ -554,11 +533,11 @@ yyL1:;
   }
   break;
   case kACF_NEW:
-# line 490 "ReductionComm.puma"
+/* line 494 "ReductionComm.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   {
-# line 492 "ReductionComm.puma"
+/* line 496 "ReductionComm.puma" */
    FindReductionHome (t->ACF_NEW.NEW_BODY, home, red_var, red_home, & yyV1);
   }
    * yyP1 = yyV1;
@@ -566,11 +545,11 @@ yyL1:;
  }
 
   case kACF_RESIDENT:
-# line 495 "ReductionComm.puma"
+/* line 499 "ReductionComm.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   {
-# line 497 "ReductionComm.puma"
+/* line 501 "ReductionComm.puma" */
    FindReductionHome (t->ACF_RESIDENT.RESIDENT_BODY, home, red_var, red_home, & yyV1);
   }
    * yyP1 = yyV1;
@@ -578,11 +557,11 @@ yyL1:;
  }
 
   case kACF_REDUCTION:
-# line 500 "ReductionComm.puma"
+/* line 504 "ReductionComm.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   {
-# line 502 "ReductionComm.puma"
+/* line 506 "ReductionComm.puma" */
    FindReductionHome (t->ACF_REDUCTION.REDUCTION_BODY, home, red_var, red_home, & yyV1);
   }
    * yyP1 = yyV1;
@@ -590,11 +569,11 @@ yyL1:;
  }
 
   case kACF_DO:
-# line 505 "ReductionComm.puma"
+/* line 509 "ReductionComm.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   {
-# line 507 "ReductionComm.puma"
+/* line 511 "ReductionComm.puma" */
    FindReductionHome (t->ACF_DO.DO_BODY, home, red_var, red_home, & yyV1);
   }
    * yyP1 = yyV1;
@@ -602,11 +581,11 @@ yyL1:;
  }
 
   case kACF_WHILE:
-# line 510 "ReductionComm.puma"
+/* line 514 "ReductionComm.puma" */
  {
-  bool yyV1;
+  rbool yyV1;
   {
-# line 512 "ReductionComm.puma"
+/* line 516 "ReductionComm.puma" */
    FindReductionHome (t->ACF_WHILE.WHILE_BODY, home, red_var, red_home, & yyV1);
   }
    * yyP1 = yyV1;
@@ -614,20 +593,18 @@ yyL1:;
  }
 
   case kACF_IF:
-# line 515 "ReductionComm.puma"
+/* line 519 "ReductionComm.puma" */
  {
   var_descriptor red_home1;
-  bool yyV1;
-  bool yyV2;
-  bool yyV3;
+  rbool yyV1;
+  rbool yyV2;
+  rbool yyV3;
   {
-# line 517 "ReductionComm.puma"
-
-# line 519 "ReductionComm.puma"
+/* line 523 "ReductionComm.puma" */
    FindReductionHome (t->ACF_IF.THEN_PART, home, red_var, red_home, & yyV1);
-# line 520 "ReductionComm.puma"
+/* line 524 "ReductionComm.puma" */
    FindReductionHome (t->ACF_IF.ELSE_PART, home, red_var, & red_home1, & yyV2);
-# line 522 "ReductionComm.puma"
+/* line 526 "ReductionComm.puma" */
    CombineReductionHome (red_home, yyV1, & red_home1, yyV2, & yyV3);
   }
    * yyP1 = yyV3;
@@ -636,12 +613,12 @@ yyL1:;
 
   }
 
-# line 525 "ReductionComm.puma"
+/* line 529 "ReductionComm.puma" */
   {
-# line 527 "ReductionComm.puma"
+/* line 531 "ReductionComm.puma" */
    failure_protocol (MODULE, "FindReductionHome", t);
   }
-   * yyP1 = false;
+   * yyP1 = rfalse;
    return;
 
 ;
@@ -649,31 +626,31 @@ yyL1:;
 
 static void CombineReductionHome
 # if defined __STDC__ | defined __cplusplus
-(pvar home1, register bool found1, pvar home2, register bool found2, register bool * yyP2)
+(pvar home1, register rbool found1, pvar home2, register rbool found2, register rbool * yyP2)
 # else
 (home1, found1, home2, found2, yyP2)
  pvar home1;
- register bool found1;
+ register rbool found1;
  pvar home2;
- register bool found2;
- register bool * yyP2;
+ register rbool found2;
+ register rbool * yyP2;
 # endif
 {
-# line 543 "ReductionComm.puma"
+/* line 547 "ReductionComm.puma" */
   {
-# line 545 "ReductionComm.puma"
+/* line 549 "ReductionComm.puma" */
    if (! ((! found2))) goto yyL1;
   }
    * yyP2 = found1;
    return;
 yyL1:;
 
-# line 548 "ReductionComm.puma"
+/* line 552 "ReductionComm.puma" */
   {
-# line 550 "ReductionComm.puma"
+/* line 554 "ReductionComm.puma" */
    if (! ((! found1))) goto yyL2;
   {
-# line 551 "ReductionComm.puma"
+/* line 555 "ReductionComm.puma" */
  *home1 = *home2; 
   }
   }
@@ -681,37 +658,37 @@ yyL1:;
    return;
 yyL2:;
 
-# line 554 "ReductionComm.puma"
+/* line 558 "ReductionComm.puma" */
   {
-# line 556 "ReductionComm.puma"
+/* line 560 "ReductionComm.puma" */
    if (! ((SameReplication (home1, home2)))) goto yyL3;
   {
-# line 558 "ReductionComm.puma"
+/* line 562 "ReductionComm.puma" */
    tree_protocol ("home 1 is : ", PrintableDescriptorVar (home1));
-# line 559 "ReductionComm.puma"
+/* line 563 "ReductionComm.puma" */
    tree_protocol ("home 2 is : ", PrintableDescriptorVar (home2));
   }
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    return;
 yyL3:;
 
-# line 562 "ReductionComm.puma"
+/* line 566 "ReductionComm.puma" */
   {
-# line 564 "ReductionComm.puma"
+/* line 568 "ReductionComm.puma" */
    error_protocol ("two different homes for reductions");
-# line 565 "ReductionComm.puma"
+/* line 569 "ReductionComm.puma" */
    tree_protocol ("home 1 is : ", PrintableDescriptorVar (home1));
-# line 566 "ReductionComm.puma"
+/* line 570 "ReductionComm.puma" */
    tree_protocol ("home 2 is : ", PrintableDescriptorVar (home2));
   }
-   * yyP2 = true;
+   * yyP2 = rtrue;
    return;
 
 ;
 }
 
-static bool SameReductionVar
+static rbool SameReductionVar
 # if defined __STDC__ | defined __cplusplus
 (register tTree vl1, register tTree vl2)
 # else
@@ -723,21 +700,21 @@ static bool SameReductionVar
   if (vl1->Kind == kBTV_LIST) {
   if (vl2->Kind == kBTP_LIST) {
   if (vl2->BTP_LIST.Elem->Kind == kVAR_PARAM) {
-# line 577 "ReductionComm.puma"
+/* line 581 "ReductionComm.puma" */
   {
-# line 579 "ReductionComm.puma"
+/* line 583 "ReductionComm.puma" */
    if (! ((TreeVarName (vl1->BTV_LIST.Elem) == TreeVarName (vl2->BTP_LIST.Elem->VAR_PARAM.V)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   }
   }
-  return false;
+  return rfalse;
 }
 
-static bool SameReplication
+static rbool SameReplication
 # if defined __STDC__ | defined __cplusplus
 (pvar home1, pvar home2)
 # else
@@ -746,76 +723,74 @@ static bool SameReplication
  pvar home2;
 # endif
 {
-# line 590 "ReductionComm.puma"
+/* line 594 "ReductionComm.puma" */
   {
-# line 592 "ReductionComm.puma"
+/* line 596 "ReductionComm.puma" */
    if (! ((IsReplicatedDescriptor (home1)))) goto yyL1;
   {
-# line 593 "ReductionComm.puma"
+/* line 597 "ReductionComm.puma" */
    if (! ((IsReplicatedDescriptor (home2)))) goto yyL1;
   {
-# line 595 "ReductionComm.puma"
+/* line 599 "ReductionComm.puma" */
    print_protocol ("both homes are full replicated");
   }
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 598 "ReductionComm.puma"
+/* line 602 "ReductionComm.puma" */
   {
-# line 600 "ReductionComm.puma"
+/* line 604 "ReductionComm.puma" */
    if (! ((IsSerialDescriptor (home1)))) goto yyL2;
   {
-# line 601 "ReductionComm.puma"
+/* line 605 "ReductionComm.puma" */
    if (! ((IsSerialDescriptor (home2)))) goto yyL2;
   {
-# line 603 "ReductionComm.puma"
+/* line 607 "ReductionComm.puma" */
    print_protocol ("both homes are are on a single processor");
   }
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 606 "ReductionComm.puma"
+/* line 610 "ReductionComm.puma" */
   {
-# line 608 "ReductionComm.puma"
+/* line 612 "ReductionComm.puma" */
    if (! ((home1 -> topology_obj != home2 -> topology_obj))) goto yyL3;
   {
-# line 609 "ReductionComm.puma"
-   return false;
+/* line 613 "ReductionComm.puma" */
+   return rfalse;
   }
   }
 yyL3:;
 
-# line 612 "ReductionComm.puma"
+/* line 616 "ReductionComm.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 614 "ReductionComm.puma"
-
-# line 616 "ReductionComm.puma"
+/* line 620 "ReductionComm.puma" */
  int pdim;
 
-      okay = true;
+      okay = rtrue;
 
       for (pdim = 1; pdim <= home1->topology_rank; pdim++)
 
        { if (OnMoreProcs (home1, pdim) && !OnMoreProcs(home2, pdim))
-            okay = false;
+            okay = rfalse;
          if (OnMoreProcs (home2, pdim) && !OnMoreProcs(home1, pdim))
-            okay = false;
+            okay = rfalse;
        }
     
-# line 629 "ReductionComm.puma"
+/* line 633 "ReductionComm.puma" */
    if (! ((okay))) goto yyL4;
   }
-   return true;
+   return rtrue;
  }
 yyL4:;
 
-  return false;
+  return rfalse;
 }
 
 static void MakeFullTopHome
@@ -826,9 +801,9 @@ static void MakeFullTopHome
  pvar home;
 # endif
 {
-# line 643 "ReductionComm.puma"
+/* line 647 "ReductionComm.puma" */
   {
-# line 645 "ReductionComm.puma"
+/* line 649 "ReductionComm.puma" */
  int pdim;
 
      for (pdim = 1; pdim <= home->topology_rank; pdim++)
@@ -842,7 +817,7 @@ static void MakeFullTopHome
 ;
 }
 
-static bool OnMoreProcs
+static rbool OnMoreProcs
 # if defined __STDC__ | defined __cplusplus
 (pvar home, register int pdim)
 # else
@@ -851,29 +826,29 @@ static bool OnMoreProcs
  register int pdim;
 # endif
 {
-# line 665 "ReductionComm.puma"
+/* line 669 "ReductionComm.puma" */
   {
-# line 667 "ReductionComm.puma"
+/* line 671 "ReductionComm.puma" */
    if (! ((home -> on_val [pdim - 1] == NoTree))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 670 "ReductionComm.puma"
+/* line 674 "ReductionComm.puma" */
   {
-# line 672 "ReductionComm.puma"
+/* line 676 "ReductionComm.puma" */
    if (! ((TreeRank (home -> on_val [pdim - 1]) > 0))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
-void BeginReductionComm ()
+void BeginReductionComm ARGS ((void))
 {
 }
 
-void CloseReductionComm ()
+void CloseReductionComm ARGS ((void))
 {
 }
