@@ -1,6 +1,13 @@
 # include "Reductions.h"
-# include "yyReductions.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,27 +15,24 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 35 "Reductions.puma"
+/* line 35 "Reductions.puma" */
 
 
 # define  MODULE "Reductions"
@@ -39,7 +43,7 @@
 # include "protocol.h"
 # include "DefTable.h"
 
-# include "StringMem.h"
+# include "StringM.h"
 # include "Definitions.h"
 # include "Types.h"
 # include "Transform.h"    /* ExpToVarParam */
@@ -49,11 +53,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* Reductions_Exit) () = yyExit;
+# include "yyReductions.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* Reductions_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -62,28 +80,29 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module Reductions, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module Reductions, routine %s failed\n",
+  yyFunction);
  Reductions_Exit ();
 }
 
-bool IsReduction ARGS((tTree t));
-bool IsLocReduction ARGS((tTree t));
-void SplitReduction ARGS((tTree params, tTree * reduction_array, int * reduction_dim, tTree * reduction_mask));
-int GetReductionDim ARGS((tTree params));
-tTree InitReductionStmt ARGS((tTree var, tTree vtype, tTree func));
-static int GetSizeVal ARGS((tTree type));
-static int GetVal ARGS((tTree kind));
-tTree GenRedAssign ARGS((tTree var, tTree exp, tIdent id));
-static tTree MakeOpRedCall ARGS((tTree op, tTree var, tTree exp));
-static tTree MakeIntrRedCall ARGS((tIdent fname, tTree var, tTree exp));
-tTree ResolveReduce ARGS((tTree t));
-static tTree ResolveDoIt ARGS((tTree t, tIdent func, tTree var, tTree exp, tTree other_stmts));
-static tTree LocationStmts ARGS((tTree params));
-int GetGlobalOp ARGS((tTree type, tIdent redfunc));
-static int GetGlobalOp1 ARGS((tTree type, int size, tIdent redfunc));
-tTree GetReductionZero ARGS((tTree vtype, tIdent ident));
+rbool IsReduction ARGS ((tTree t));
+rbool IsLocReduction ARGS ((tTree t));
+void SplitReduction ARGS ((tTree params, tTree * reduction_array, int * reduction_dim, tTree * reduction_mask));
+int GetReductionDim ARGS ((tTree params));
+tTree InitReductionStmt ARGS ((tTree var, tTree vtype, tTree func));
+static int GetSizeVal ARGS ((tTree type));
+static int GetVal ARGS ((tTree kind));
+tTree GenRedAssign ARGS ((tTree var, tTree exp, tIdent id));
+static tTree MakeOpRedCall ARGS ((tTree op, tTree var, tTree exp));
+static tTree MakeIntrRedCall ARGS ((tIdent fname, tTree var, tTree exp));
+tTree ResolveReduce ARGS ((tTree t));
+static tTree ResolveDoIt ARGS ((tTree t, tIdent func, tTree var, tTree exp, tTree other_stmts));
+static tTree LocationStmts ARGS ((tTree params));
+int GetGlobalOp ARGS ((tTree type, tIdent redfunc));
+static int GetGlobalOp1 ARGS ((tTree type, int size, tIdent redfunc));
+tTree GetReductionZero ARGS ((tTree vtype, tIdent ident));
 
-bool IsReduction
+rbool IsReduction
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -92,35 +111,35 @@ bool IsReduction
 # endif
 {
   if (t->Kind == kACF_BASIC) {
-# line 65 "Reductions.puma"
+/* line 65 "Reductions.puma" */
   {
-# line 67 "Reductions.puma"
+/* line 67 "Reductions.puma" */
    if (! ((IsReduction (t->ACF_BASIC.BASIC_STMT)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   if (t->Kind == kASSIGN_STMT) {
   if (t->ASSIGN_STMT.ASSIGN_EXP->Kind == kFUNC_CALL_EXP) {
-# line 70 "Reductions.puma"
+/* line 70 "Reductions.puma" */
   {
-# line 72 "Reductions.puma"
+/* line 72 "Reductions.puma" */
    if (! ((IsIntrCall (t->ASSIGN_STMT.ASSIGN_EXP)))) goto yyL2;
   {
-# line 73 "Reductions.puma"
+/* line 73 "Reductions.puma" */
    if (! ((IntrFuncRed (t->ASSIGN_STMT.ASSIGN_EXP->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Ident)))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
-bool IsLocReduction
+rbool IsLocReduction
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -129,32 +148,32 @@ bool IsLocReduction
 # endif
 {
   if (t->Kind == kACF_BASIC) {
-# line 86 "Reductions.puma"
+/* line 86 "Reductions.puma" */
   {
-# line 88 "Reductions.puma"
+/* line 88 "Reductions.puma" */
    if (! ((IsLocReduction (t->ACF_BASIC.BASIC_STMT)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   if (t->Kind == kASSIGN_STMT) {
   if (t->ASSIGN_STMT.ASSIGN_EXP->Kind == kFUNC_CALL_EXP) {
-# line 91 "Reductions.puma"
+/* line 91 "Reductions.puma" */
   {
-# line 93 "Reductions.puma"
+/* line 93 "Reductions.puma" */
    if (! ((IsIntrCall (t->ASSIGN_STMT.ASSIGN_EXP)))) goto yyL2;
   {
-# line 94 "Reductions.puma"
+/* line 94 "Reductions.puma" */
    if (! (((t->ASSIGN_STMT.ASSIGN_EXP->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Ident == IsIdent ("MINLOC")) || (t->ASSIGN_STMT.ASSIGN_EXP->FUNC_CALL_EXP.FUNC_ID->PROC_OBJ.Ident == IsIdent ("MAXLOC"))))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
 void SplitReduction
@@ -173,7 +192,7 @@ void SplitReduction
   if (params->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Next->BTP_LIST.Next->BTP_LIST.Elem->Kind == kNO_PARAM) {
   if (params->BTP_LIST.Next->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_EMPTY) {
-# line 113 "Reductions.puma"
+/* line 113 "Reductions.puma" */
    * reduction_array = params->BTP_LIST.Elem;
    * reduction_dim = GetReductionDim (params);
    * reduction_mask = NoTree;
@@ -182,7 +201,7 @@ void SplitReduction
   }
   }
   if (params->BTP_LIST.Next->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_EMPTY) {
-# line 117 "Reductions.puma"
+/* line 117 "Reductions.puma" */
    * reduction_array = params->BTP_LIST.Elem;
    * reduction_dim = GetReductionDim (params);
    * reduction_mask = params->BTP_LIST.Next->BTP_LIST.Next->BTP_LIST.Elem;
@@ -192,9 +211,9 @@ void SplitReduction
   }
   }
   }
-# line 121 "Reductions.puma"
+/* line 121 "Reductions.puma" */
   {
-# line 123 "Reductions.puma"
+/* line 123 "Reductions.puma" */
    failure_protocol ("Reductions", "SplitReduction", params);
   }
    * reduction_array = NoTree;
@@ -218,7 +237,7 @@ int GetReductionDim
   if (params->BTP_LIST.Next->BTP_LIST.Elem->Kind == kNO_PARAM) {
   if (params->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Next->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_EMPTY) {
-# line 128 "Reductions.puma"
+/* line 128 "Reductions.puma" */
    return 0;
 
   }
@@ -226,23 +245,17 @@ int GetReductionDim
   }
   if (params->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Next->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_EMPTY) {
-# line 133 "Reductions.puma"
+/* line 133 "Reductions.puma" */
  {
   int idim;
-  bool found;
+  rbool found;
   {
-# line 135 "Reductions.puma"
-
-# line 136 "Reductions.puma"
-
-# line 138 "Reductions.puma"
+/* line 138 "Reductions.puma" */
  GetIntConstValue (params->BTP_LIST.Next->BTP_LIST.Elem, &found, &idim);
      if (!found) idim = -1;
    
   }
-  {
    return idim;
-  }
  }
 
   }
@@ -250,6 +263,7 @@ int GetReductionDim
   }
   }
  yyAbort ("GetReductionDim");
+ { int yyDummy; return yyDummy; }
 }
 
 tTree InitReductionStmt
@@ -262,24 +276,24 @@ tTree InitReductionStmt
  register tTree func;
 # endif
 {
-# line 163 "Reductions.puma"
+/* line 163 "Reductions.puma" */
 
 tTree t;
 
   if (func->Kind == kPROC_OBJ) {
-# line 167 "Reductions.puma"
+/* line 167 "Reductions.puma" */
   {
-# line 169 "Reductions.puma"
+/* line 169 "Reductions.puma" */
  t = mCONST_EXP (GetReductionZero (vtype, func->PROC_OBJ.Ident));
      t = mASSIGN_STMT (var, t);
      t = mACF_BASIC (t);
-     SetACFNode (t, 0, 0);
    
   }
    return t;
 
   }
  yyAbort ("InitReductionStmt");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static int GetSizeVal
@@ -291,26 +305,27 @@ static int GetSizeVal
 # endif
 {
   if (type->Kind == kINTEGER_TYPE) {
-# line 179 "Reductions.puma"
+/* line 178 "Reductions.puma" */
    return GetVal (type->INTEGER_TYPE.SIZE);
 
   }
   if (type->Kind == kREAL_TYPE) {
-# line 180 "Reductions.puma"
+/* line 179 "Reductions.puma" */
    return GetVal (type->REAL_TYPE.SIZE);
 
   }
   if (type->Kind == kBOOLEAN_TYPE) {
-# line 181 "Reductions.puma"
+/* line 180 "Reductions.puma" */
    return GetVal (type->BOOLEAN_TYPE.SIZE);
 
   }
   if (type->Kind == kCOMPLEX_TYPE) {
-# line 182 "Reductions.puma"
+/* line 181 "Reductions.puma" */
    return GetVal (type->COMPLEX_TYPE.SIZE);
 
   }
  yyAbort ("GetSizeVal");
+ { int yyDummy; return yyDummy; }
 }
 
 static int GetVal
@@ -323,12 +338,13 @@ static int GetVal
 {
   if (kind->Kind == kCONST_EXP) {
   if (kind->CONST_EXP.C->Kind == kINT_CONSTANT) {
-# line 186 "Reductions.puma"
+/* line 185 "Reductions.puma" */
    return kind->CONST_EXP.C->INT_CONSTANT.value;
 
   }
   }
  yyAbort ("GetVal");
+ { int yyDummy; return yyDummy; }
 }
 
 tTree GenRedAssign
@@ -342,58 +358,58 @@ tTree GenRedAssign
 # endif
 {
   if (equaltIdent (id, IsIdent ("SUM"))) {
-# line 199 "Reductions.puma"
+/* line 198 "Reductions.puma" */
    return MakeOpRedCall (mOP_PLUS (), var, exp);
 
   }
   if (equaltIdent (id, IsIdent ("ANY"))) {
-# line 204 "Reductions.puma"
+/* line 203 "Reductions.puma" */
    return MakeOpRedCall (mOP_OR (), var, exp);
 
   }
   if (equaltIdent (id, IsIdent ("PARITY"))) {
-# line 209 "Reductions.puma"
+/* line 208 "Reductions.puma" */
    return MakeOpRedCall (mOP_NEQV (), var, exp);
 
   }
   if (equaltIdent (id, IsIdent ("ALL"))) {
-# line 214 "Reductions.puma"
+/* line 213 "Reductions.puma" */
    return MakeOpRedCall (mOP_AND (), var, exp);
 
   }
   if (equaltIdent (id, IsIdent ("PRODUCT"))) {
-# line 219 "Reductions.puma"
+/* line 218 "Reductions.puma" */
    return MakeOpRedCall (mOP_TIMES (), var, exp);
 
   }
   if (equaltIdent (id, IsIdent ("IALL"))) {
-# line 224 "Reductions.puma"
+/* line 223 "Reductions.puma" */
    return MakeIntrRedCall (IsIdent ("IAND"), var, exp);
 
   }
   if (equaltIdent (id, IsIdent ("IANY"))) {
-# line 229 "Reductions.puma"
+/* line 228 "Reductions.puma" */
    return MakeIntrRedCall (IsIdent ("IOR"), var, exp);
 
   }
   if (equaltIdent (id, IsIdent ("IPARITY"))) {
-# line 234 "Reductions.puma"
+/* line 233 "Reductions.puma" */
    return MakeIntrRedCall (IsIdent ("IEOR"), var, exp);
 
   }
   if (equaltIdent (id, IsIdent ("MINVAL"))) {
-# line 239 "Reductions.puma"
+/* line 238 "Reductions.puma" */
    return MakeIntrRedCall (IsIdent ("MIN"), var, exp);
 
   }
   if (equaltIdent (id, IsIdent ("MAXVAL"))) {
-# line 244 "Reductions.puma"
+/* line 243 "Reductions.puma" */
    return MakeIntrRedCall (IsIdent ("MAX"), var, exp);
 
   }
-# line 249 "Reductions.puma"
+/* line 248 "Reductions.puma" */
   {
-# line 251 "Reductions.puma"
+/* line 250 "Reductions.puma" */
    failure_protocol (MODULE, "GenRedAssign", mPROC_OBJ (id));
   }
    return NoTree;
@@ -410,20 +426,16 @@ static tTree MakeOpRedCall
  register tTree exp;
 # endif
 {
-# line 263 "Reductions.puma"
+/* line 262 "Reductions.puma" */
  {
   tTree stmt;
   {
-# line 265 "Reductions.puma"
-
-# line 267 "Reductions.puma"
+/* line 266 "Reductions.puma" */
  stmt = mOP_EXP (op, mVAR_EXP (CopyTree(var)), exp);
       stmt = mASSIGN_STMT (var, stmt);
     
   }
-  {
    return stmt;
-  }
  }
 
 }
@@ -438,16 +450,12 @@ static tTree MakeIntrRedCall
  register tTree exp;
 # endif
 {
-# line 282 "Reductions.puma"
+/* line 281 "Reductions.puma" */
  {
   tTree params;
   tTree stmt;
   {
-# line 284 "Reductions.puma"
-
-# line 285 "Reductions.puma"
-
-# line 287 "Reductions.puma"
+/* line 286 "Reductions.puma" */
  params = mBTP_EMPTY ();
       params = mBTP_LIST (ExpToVarParam (exp), params);
       params = mBTP_LIST (mVAR_PARAM (CopyTree(var)), params);
@@ -457,9 +465,7 @@ static tTree MakeIntrRedCall
       stmt = mASSIGN_STMT (var, stmt);
     
   }
-  {
    return stmt;
-  }
  }
 
 }
@@ -479,11 +485,11 @@ tTree ResolveReduce
   if (t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->Kind == kBTP_LIST) {
   if (t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->BTP_LIST.Elem->Kind == kVAR_PARAM) {
   if (t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->BTP_LIST.Elem->VAR_PARAM.V->Kind == kADDR) {
-# line 321 "Reductions.puma"
+/* line 320 "Reductions.puma" */
    return ResolveDoIt (t, t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_FUNC->PROC_OBJ.Ident, t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_PARAMS->BTP_LIST.Elem->VAR_PARAM.V, t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->BTP_LIST.Elem->VAR_PARAM.V->ADDR.E, LocationStmts (t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->BTP_LIST.Next));
 
   }
-# line 332 "Reductions.puma"
+/* line 332 "Reductions.puma" */
    return ResolveDoIt (t, t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_FUNC->PROC_OBJ.Ident, t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_PARAMS->BTP_LIST.Elem->VAR_PARAM.V, mVAR_EXP (t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->BTP_LIST.Elem->VAR_PARAM.V), LocationStmts (t->ACF_BASIC.BASIC_STMT->REDUCE_STMT.RED_PARAMS->BTP_LIST.Next->BTP_LIST.Next));
 
   }
@@ -492,9 +498,9 @@ tTree ResolveReduce
   }
   }
   }
-# line 342 "Reductions.puma"
+/* line 343 "Reductions.puma" */
   {
-# line 344 "Reductions.puma"
+/* line 345 "Reductions.puma" */
    failure_protocol (MODULE, "ResolveReduce", t);
   }
    return NoTree;
@@ -515,9 +521,9 @@ static tTree ResolveDoIt
 {
   if (t->Kind == kACF_BASIC) {
   if (equaltIdent (func, IsIdent ("COUNT"))) {
-# line 351 "Reductions.puma"
+/* line 352 "Reductions.puma" */
   {
-# line 355 "Reductions.puma"
+/* line 356 "Reductions.puma" */
  
 
       t->ACF_BASIC.BASIC_STMT = MakeOpRedCall (mOP_PLUS (), var, MakeConstant(1));
@@ -528,9 +534,9 @@ static tTree ResolveDoIt
 
   }
   if (other_stmts->Kind == kACF_EMPTY) {
-# line 364 "Reductions.puma"
+/* line 365 "Reductions.puma" */
   {
-# line 368 "Reductions.puma"
+/* line 369 "Reductions.puma" */
  t->ACF_BASIC.BASIC_STMT = GenRedAssign (var, exp, func); 
   }
    return t;
@@ -540,13 +546,9 @@ static tTree ResolveDoIt
   tTree cond;
   tTree stmt;
   if (equaltIdent (func, IsIdent ("MINVAL"))) {
-# line 373 "Reductions.puma"
+/* line 374 "Reductions.puma" */
   {
-# line 377 "Reductions.puma"
-
-# line 378 "Reductions.puma"
-
-# line 380 "Reductions.puma"
+/* line 381 "Reductions.puma" */
  cond = mOP_EXP (mOP_LT(), CopyTree(exp), mVAR_EXP (CopyTree(var)));
       t->ACF_BASIC.BASIC_STMT = mASSIGN_STMT (var, exp);
 
@@ -556,9 +558,7 @@ static tTree ResolveDoIt
       stmt = mACF_IF (cond, stmt, mACF_EMPTY ());
     
   }
-  {
    return stmt;
-  }
 
   }
  }
@@ -566,13 +566,9 @@ static tTree ResolveDoIt
   tTree cond;
   tTree stmt;
   if (equaltIdent (func, IsIdent ("MAXVAL"))) {
-# line 392 "Reductions.puma"
+/* line 393 "Reductions.puma" */
   {
-# line 396 "Reductions.puma"
-
-# line 397 "Reductions.puma"
-
-# line 399 "Reductions.puma"
+/* line 400 "Reductions.puma" */
  cond = mOP_EXP (mOP_GT(), CopyTree(exp), mVAR_EXP (CopyTree(var)));
       t->ACF_BASIC.BASIC_STMT = mASSIGN_STMT (var, exp);
 
@@ -582,16 +578,14 @@ static tTree ResolveDoIt
       stmt = mACF_IF (cond, stmt, mACF_EMPTY ());
     
   }
-  {
    return stmt;
-  }
 
   }
  }
   }
-# line 410 "Reductions.puma"
+/* line 411 "Reductions.puma" */
   {
-# line 412 "Reductions.puma"
+/* line 413 "Reductions.puma" */
    failure_protocol (MODULE, "ResolveReduce", t);
   }
    return NoTree;
@@ -606,12 +600,12 @@ static tTree LocationStmts
  register tTree params;
 # endif
 {
-# line 428 "Reductions.puma"
+/* line 429 "Reductions.puma" */
 
 tTree stmt;
 
   if (params->Kind == kBTP_EMPTY) {
-# line 432 "Reductions.puma"
+/* line 433 "Reductions.puma" */
    return mACF_EMPTY ();
 
   }
@@ -620,9 +614,9 @@ tTree stmt;
   if (params->BTP_LIST.Next->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Next->BTP_LIST.Elem->Kind == kVAR_PARAM) {
   if (params->BTP_LIST.Next->BTP_LIST.Elem->VAR_PARAM.V->Kind == kADDR) {
-# line 437 "Reductions.puma"
+/* line 438 "Reductions.puma" */
   {
-# line 439 "Reductions.puma"
+/* line 440 "Reductions.puma" */
  stmt = mASSIGN_STMT (params->BTP_LIST.Elem->VAR_PARAM.V, params->BTP_LIST.Next->BTP_LIST.Elem->VAR_PARAM.V->ADDR.E);
       stmt = mACF_BASIC (stmt);
     
@@ -630,9 +624,9 @@ tTree stmt;
    return mACF_LIST (stmt, LocationStmts (params->BTP_LIST.Next->BTP_LIST.Next));
 
   }
-# line 446 "Reductions.puma"
+/* line 447 "Reductions.puma" */
   {
-# line 448 "Reductions.puma"
+/* line 449 "Reductions.puma" */
  stmt = mASSIGN_STMT (params->BTP_LIST.Elem->VAR_PARAM.V, mVAR_EXP (params->BTP_LIST.Next->BTP_LIST.Elem->VAR_PARAM.V));
       stmt = mACF_BASIC (stmt);
     
@@ -644,6 +638,7 @@ tTree stmt;
   }
   }
  yyAbort ("LocationStmts");
+ { tTree yyDummy; return yyDummy; }
 }
 
 int GetGlobalOp
@@ -655,7 +650,7 @@ int GetGlobalOp
  register tIdent redfunc;
 # endif
 {
-# line 465 "Reductions.puma"
+/* line 466 "Reductions.puma" */
    return GetGlobalOp1 (type, GetSizeVal (type), redfunc);
 
 }
@@ -673,21 +668,21 @@ static int GetGlobalOp1
   if (type->Kind == kBOOLEAN_TYPE) {
   if (equalint (size, 1 * default_int_size)) {
   if (equaltIdent (redfunc, IsIdent ("ANY"))) {
-# line 472 "Reductions.puma"
+/* line 473 "Reductions.puma" */
    return 17;
 
   }
   }
   if (equalint (size, 1 * default_int_size)) {
   if (equaltIdent (redfunc, IsIdent ("ALL"))) {
-# line 476 "Reductions.puma"
+/* line 477 "Reductions.puma" */
    return 16;
 
   }
   }
   if (equalint (size, 1 * default_int_size)) {
   if (equaltIdent (redfunc, IsIdent ("PARITY"))) {
-# line 480 "Reductions.puma"
+/* line 481 "Reductions.puma" */
    return 18;
 
   }
@@ -696,56 +691,56 @@ static int GetGlobalOp1
   if (type->Kind == kINTEGER_TYPE) {
   if (equalint (size, 1 * default_int_size)) {
   if (equaltIdent (redfunc, IsIdent ("SUM"))) {
-# line 484 "Reductions.puma"
+/* line 485 "Reductions.puma" */
    return 7;
 
   }
   }
   if (equalint (size, 1 * default_int_size)) {
   if (equaltIdent (redfunc, IsIdent ("PRODUCT"))) {
-# line 488 "Reductions.puma"
+/* line 489 "Reductions.puma" */
    return 10;
 
   }
   }
   if (equalint (size, 1 * default_int_size)) {
   if (equaltIdent (redfunc, IsIdent ("MINVAL"))) {
-# line 492 "Reductions.puma"
+/* line 493 "Reductions.puma" */
    return 1;
 
   }
   }
   if (equalint (size, 1 * default_int_size)) {
   if (equaltIdent (redfunc, IsIdent ("MAXVAL"))) {
-# line 496 "Reductions.puma"
+/* line 497 "Reductions.puma" */
    return default_int_size;
 
   }
   }
   if (equalint (size, 1 * default_int_size)) {
   if (equaltIdent (redfunc, IsIdent ("IALL"))) {
-# line 500 "Reductions.puma"
+/* line 501 "Reductions.puma" */
    return 13;
 
   }
   }
   if (equalint (size, 1 * default_int_size)) {
   if (equaltIdent (redfunc, IsIdent ("IANY"))) {
-# line 504 "Reductions.puma"
+/* line 505 "Reductions.puma" */
    return 14;
 
   }
   }
   if (equalint (size, 1 * default_int_size)) {
   if (equaltIdent (redfunc, IsIdent ("IPARITY"))) {
-# line 508 "Reductions.puma"
+/* line 509 "Reductions.puma" */
    return 15;
 
   }
   }
   if (equalint (size, 1 * default_int_size)) {
   if (equaltIdent (redfunc, IsIdent ("COUNT"))) {
-# line 512 "Reductions.puma"
+/* line 513 "Reductions.puma" */
    return 7;
 
   }
@@ -754,56 +749,56 @@ static int GetGlobalOp1
   if (type->Kind == kREAL_TYPE) {
   if (equalint (size, 1 * default_real_size)) {
   if (equaltIdent (redfunc, IsIdent ("SUM"))) {
-# line 516 "Reductions.puma"
+/* line 517 "Reductions.puma" */
    return 8;
 
   }
   }
   if (equalint (size, 1 * default_real_size)) {
   if (equaltIdent (redfunc, IsIdent ("PRODUCT"))) {
-# line 520 "Reductions.puma"
+/* line 521 "Reductions.puma" */
    return 11;
 
   }
   }
   if (equalint (size, 1 * default_real_size)) {
   if (equaltIdent (redfunc, IsIdent ("MINVAL"))) {
-# line 524 "Reductions.puma"
+/* line 525 "Reductions.puma" */
    return 2;
 
   }
   }
   if (equalint (size, 1 * default_real_size)) {
   if (equaltIdent (redfunc, IsIdent ("MAXVAL"))) {
-# line 528 "Reductions.puma"
+/* line 529 "Reductions.puma" */
    return 5;
 
   }
   }
   if (equalint (size, 2 * default_real_size)) {
   if (equaltIdent (redfunc, IsIdent ("SUM"))) {
-# line 532 "Reductions.puma"
+/* line 533 "Reductions.puma" */
    return 9;
 
   }
   }
   if (equalint (size, 2 * default_real_size)) {
   if (equaltIdent (redfunc, IsIdent ("PRODUCT"))) {
-# line 536 "Reductions.puma"
+/* line 537 "Reductions.puma" */
    return 12;
 
   }
   }
   if (equalint (size, 2 * default_real_size)) {
   if (equaltIdent (redfunc, IsIdent ("MINVAL"))) {
-# line 540 "Reductions.puma"
+/* line 541 "Reductions.puma" */
    return 3;
 
   }
   }
   if (equalint (size, 2 * default_real_size)) {
   if (equaltIdent (redfunc, IsIdent ("MAXVAL"))) {
-# line 544 "Reductions.puma"
+/* line 545 "Reductions.puma" */
    return 6;
 
   }
@@ -812,24 +807,24 @@ static int GetGlobalOp1
   if (type->Kind == kCOMPLEX_TYPE) {
   if (equalint (size, 2 * default_real_size)) {
   if (equaltIdent (redfunc, IsIdent ("SUM"))) {
-# line 548 "Reductions.puma"
+/* line 549 "Reductions.puma" */
    return 19;
 
   }
   }
   if (equalint (size, 2 * default_real_size)) {
   if (equaltIdent (redfunc, IsIdent ("PRODUCT"))) {
-# line 552 "Reductions.puma"
+/* line 553 "Reductions.puma" */
    return 20;
 
   }
   }
   }
-# line 556 "Reductions.puma"
+/* line 557 "Reductions.puma" */
   {
-# line 557 "Reductions.puma"
+/* line 558 "Reductions.puma" */
    error_protocol ("This reduction is not handled within ADAPTOR");
-# line 558 "Reductions.puma"
+/* line 559 "Reductions.puma" */
    tree_protocol ("type is ", type);
   }
    return - 1;
@@ -846,48 +841,48 @@ tTree GetReductionZero
 # endif
 {
   if (vtype->Kind == kBOOLEAN_TYPE) {
-# line 570 "Reductions.puma"
+/* line 571 "Reductions.puma" */
   {
-# line 572 "Reductions.puma"
+/* line 573 "Reductions.puma" */
    if (! ((GetVal (vtype->BOOLEAN_TYPE.SIZE) == default_int_size))) goto yyL1;
   {
-# line 573 "Reductions.puma"
+/* line 574 "Reductions.puma" */
    if (! ((ident == IsIdent ("ANY")))) goto yyL1;
   }
   }
    return mBOOL_CONSTANT (0);
 yyL1:;
 
-# line 578 "Reductions.puma"
+/* line 579 "Reductions.puma" */
   {
-# line 580 "Reductions.puma"
+/* line 581 "Reductions.puma" */
    if (! ((GetVal (vtype->BOOLEAN_TYPE.SIZE) == default_int_size))) goto yyL2;
   {
-# line 581 "Reductions.puma"
+/* line 582 "Reductions.puma" */
    if (! ((ident == IsIdent ("ALL")))) goto yyL2;
   }
   }
    return mBOOL_CONSTANT (1);
 yyL2:;
 
-# line 586 "Reductions.puma"
+/* line 587 "Reductions.puma" */
   {
-# line 588 "Reductions.puma"
+/* line 589 "Reductions.puma" */
    if (! ((GetVal (vtype->BOOLEAN_TYPE.SIZE) == default_int_size))) goto yyL3;
   {
-# line 589 "Reductions.puma"
+/* line 590 "Reductions.puma" */
    if (! ((ident == IsIdent ("PARITY")))) goto yyL3;
   }
   }
    return mBOOL_CONSTANT (0);
 yyL3:;
 
-# line 604 "Reductions.puma"
+/* line 605 "Reductions.puma" */
   {
-# line 606 "Reductions.puma"
+/* line 607 "Reductions.puma" */
    if (! ((GetVal (vtype->BOOLEAN_TYPE.SIZE) == default_int_size))) goto yyL5;
   {
-# line 607 "Reductions.puma"
+/* line 608 "Reductions.puma" */
    if (! ((ident == IsIdent ("COUNT")))) goto yyL5;
   }
   }
@@ -896,96 +891,96 @@ yyL5:;
 
   }
   if (vtype->Kind == kINTEGER_TYPE) {
-# line 596 "Reductions.puma"
+/* line 597 "Reductions.puma" */
   {
-# line 598 "Reductions.puma"
+/* line 599 "Reductions.puma" */
    if (! ((GetVal (vtype->INTEGER_TYPE.SIZE) == default_int_size))) goto yyL4;
   {
-# line 599 "Reductions.puma"
+/* line 600 "Reductions.puma" */
    if (! ((ident == IsIdent ("COUNT")))) goto yyL4;
   }
   }
    return mINT_CONSTANT (0);
 yyL4:;
 
-# line 612 "Reductions.puma"
+/* line 613 "Reductions.puma" */
   {
-# line 614 "Reductions.puma"
+/* line 615 "Reductions.puma" */
    if (! ((GetVal (vtype->INTEGER_TYPE.SIZE) == default_int_size))) goto yyL6;
   {
-# line 615 "Reductions.puma"
+/* line 616 "Reductions.puma" */
    if (! ((ident == IsIdent ("SUM")))) goto yyL6;
   }
   }
    return mINT_CONSTANT (0);
 yyL6:;
 
-# line 620 "Reductions.puma"
+/* line 621 "Reductions.puma" */
   {
-# line 622 "Reductions.puma"
+/* line 623 "Reductions.puma" */
    if (! ((GetVal (vtype->INTEGER_TYPE.SIZE) == default_int_size))) goto yyL7;
   {
-# line 623 "Reductions.puma"
+/* line 624 "Reductions.puma" */
    if (! ((ident == IsIdent ("PRODUCT")))) goto yyL7;
   }
   }
    return mINT_CONSTANT (1);
 yyL7:;
 
-# line 628 "Reductions.puma"
+/* line 629 "Reductions.puma" */
   {
-# line 630 "Reductions.puma"
+/* line 631 "Reductions.puma" */
    if (! ((GetVal (vtype->INTEGER_TYPE.SIZE) == default_int_size))) goto yyL8;
   {
-# line 631 "Reductions.puma"
+/* line 632 "Reductions.puma" */
    if (! ((ident == IsIdent ("MAXVAL")))) goto yyL8;
   }
   }
    return mINT_CONSTANT (- 2147483647);
 yyL8:;
 
-# line 636 "Reductions.puma"
+/* line 637 "Reductions.puma" */
   {
-# line 638 "Reductions.puma"
+/* line 639 "Reductions.puma" */
    if (! ((GetVal (vtype->INTEGER_TYPE.SIZE) == default_int_size))) goto yyL9;
   {
-# line 639 "Reductions.puma"
+/* line 640 "Reductions.puma" */
    if (! ((ident == IsIdent ("MINVAL")))) goto yyL9;
   }
   }
    return mINT_CONSTANT (2147483647);
 yyL9:;
 
-# line 644 "Reductions.puma"
+/* line 645 "Reductions.puma" */
   {
-# line 646 "Reductions.puma"
+/* line 647 "Reductions.puma" */
    if (! ((GetVal (vtype->INTEGER_TYPE.SIZE) == default_int_size))) goto yyL10;
   {
-# line 647 "Reductions.puma"
+/* line 648 "Reductions.puma" */
    if (! ((ident == IsIdent ("IALL")))) goto yyL10;
   }
   }
    return mINT_CONSTANT (- 1);
 yyL10:;
 
-# line 652 "Reductions.puma"
+/* line 653 "Reductions.puma" */
   {
-# line 654 "Reductions.puma"
+/* line 655 "Reductions.puma" */
    if (! ((GetVal (vtype->INTEGER_TYPE.SIZE) == default_int_size))) goto yyL11;
   {
-# line 655 "Reductions.puma"
+/* line 656 "Reductions.puma" */
    if (! ((ident == IsIdent ("IANY")))) goto yyL11;
   }
   }
    return mINT_CONSTANT (0);
 yyL11:;
 
-# line 660 "Reductions.puma"
+/* line 661 "Reductions.puma" */
   {
-# line 662 "Reductions.puma"
+/* line 663 "Reductions.puma" */
    if (! ((GetVal (vtype->INTEGER_TYPE.SIZE) == default_int_size))) goto yyL12;
   {
-# line 663 "Reductions.puma"
+/* line 664 "Reductions.puma" */
    if (! ((ident == IsIdent ("IPARITY")))) goto yyL12;
   }
   }
@@ -994,96 +989,96 @@ yyL12:;
 
   }
   if (vtype->Kind == kREAL_TYPE) {
-# line 668 "Reductions.puma"
+/* line 669 "Reductions.puma" */
   {
-# line 670 "Reductions.puma"
+/* line 671 "Reductions.puma" */
    if (! ((GetVal (vtype->REAL_TYPE.SIZE) == default_real_size))) goto yyL13;
   {
-# line 671 "Reductions.puma"
+/* line 672 "Reductions.puma" */
    if (! ((ident == IsIdent ("SUM")))) goto yyL13;
   }
   }
    return mREAL_CONSTANT (PutString ("0.0", 3), default_real_size);
 yyL13:;
 
-# line 676 "Reductions.puma"
+/* line 677 "Reductions.puma" */
   {
-# line 678 "Reductions.puma"
+/* line 679 "Reductions.puma" */
    if (! ((GetVal (vtype->REAL_TYPE.SIZE) == default_real_size))) goto yyL14;
   {
-# line 679 "Reductions.puma"
+/* line 680 "Reductions.puma" */
    if (! ((ident == IsIdent ("PRODUCT")))) goto yyL14;
   }
   }
    return mREAL_CONSTANT (PutString ("1.0", 3), default_real_size);
 yyL14:;
 
-# line 684 "Reductions.puma"
+/* line 685 "Reductions.puma" */
   {
-# line 686 "Reductions.puma"
+/* line 687 "Reductions.puma" */
    if (! ((GetVal (vtype->REAL_TYPE.SIZE) == default_real_size))) goto yyL15;
   {
-# line 687 "Reductions.puma"
+/* line 688 "Reductions.puma" */
    if (! ((ident == IsIdent ("MINVAL")))) goto yyL15;
   }
   }
    return mREAL_CONSTANT (PutString ("3.4028235E+38", 13), default_real_size);
 yyL15:;
 
-# line 692 "Reductions.puma"
+/* line 693 "Reductions.puma" */
   {
-# line 694 "Reductions.puma"
+/* line 695 "Reductions.puma" */
    if (! ((GetVal (vtype->REAL_TYPE.SIZE) == default_real_size))) goto yyL16;
   {
-# line 695 "Reductions.puma"
+/* line 696 "Reductions.puma" */
    if (! ((ident == IsIdent ("MAXVAL")))) goto yyL16;
   }
   }
    return mREAL_CONSTANT (PutString ("-3.4028235E+38", 14), default_real_size);
 yyL16:;
 
-# line 700 "Reductions.puma"
+/* line 701 "Reductions.puma" */
   {
-# line 702 "Reductions.puma"
+/* line 703 "Reductions.puma" */
    if (! ((GetVal (vtype->REAL_TYPE.SIZE) == 2 * default_real_size))) goto yyL17;
   {
-# line 703 "Reductions.puma"
+/* line 704 "Reductions.puma" */
    if (! ((ident == IsIdent ("SUM")))) goto yyL17;
   }
   }
    return mREAL_CONSTANT (PutString ("0.0d0", 5), 2 * default_real_size);
 yyL17:;
 
-# line 708 "Reductions.puma"
+/* line 709 "Reductions.puma" */
   {
-# line 710 "Reductions.puma"
+/* line 711 "Reductions.puma" */
    if (! ((GetVal (vtype->REAL_TYPE.SIZE) == 2 * default_real_size))) goto yyL18;
   {
-# line 711 "Reductions.puma"
+/* line 712 "Reductions.puma" */
    if (! ((ident == IsIdent ("PRODUCT")))) goto yyL18;
   }
   }
    return mREAL_CONSTANT (PutString ("1.0d0", 5), 2 * default_real_size);
 yyL18:;
 
-# line 716 "Reductions.puma"
+/* line 717 "Reductions.puma" */
   {
-# line 718 "Reductions.puma"
+/* line 719 "Reductions.puma" */
    if (! ((GetVal (vtype->REAL_TYPE.SIZE) == 2 * default_real_size))) goto yyL19;
   {
-# line 719 "Reductions.puma"
+/* line 720 "Reductions.puma" */
    if (! ((ident == IsIdent ("MINVAL")))) goto yyL19;
   }
   }
    return mREAL_CONSTANT (PutString ("1.797693134862313E+308", 22), 2 * default_real_size);
 yyL19:;
 
-# line 725 "Reductions.puma"
+/* line 726 "Reductions.puma" */
   {
-# line 727 "Reductions.puma"
+/* line 728 "Reductions.puma" */
    if (! ((GetVal (vtype->REAL_TYPE.SIZE) == 2 * default_real_size))) goto yyL20;
   {
-# line 728 "Reductions.puma"
+/* line 729 "Reductions.puma" */
    if (! ((ident == IsIdent ("MAXVAL")))) goto yyL20;
   }
   }
@@ -1092,24 +1087,24 @@ yyL20:;
 
   }
   if (vtype->Kind == kCOMPLEX_TYPE) {
-# line 734 "Reductions.puma"
+/* line 735 "Reductions.puma" */
   {
-# line 736 "Reductions.puma"
+/* line 737 "Reductions.puma" */
    if (! ((GetVal (vtype->COMPLEX_TYPE.SIZE) == 2 * default_real_size))) goto yyL21;
   {
-# line 737 "Reductions.puma"
+/* line 738 "Reductions.puma" */
    if (! ((ident == IsIdent ("SUM")))) goto yyL21;
   }
   }
    return mCOMPLEX_CONSTANT (PutString ("0.0", 3), PutString ("0.0", 3));
 yyL21:;
 
-# line 742 "Reductions.puma"
+/* line 743 "Reductions.puma" */
   {
-# line 744 "Reductions.puma"
+/* line 745 "Reductions.puma" */
    if (! ((GetVal (vtype->COMPLEX_TYPE.SIZE) == 2 * default_real_size))) goto yyL22;
   {
-# line 745 "Reductions.puma"
+/* line 746 "Reductions.puma" */
    if (! ((ident == IsIdent ("PRODUCT")))) goto yyL22;
   }
   }
@@ -1117,21 +1112,21 @@ yyL21:;
 yyL22:;
 
   }
-# line 750 "Reductions.puma"
+/* line 751 "Reductions.puma" */
   {
-# line 752 "Reductions.puma"
+/* line 753 "Reductions.puma" */
    error_protocol ("illegal reduction type");
-# line 753 "Reductions.puma"
+/* line 754 "Reductions.puma" */
    tree_protocol ("type : ", vtype);
   }
    return mINT_CONSTANT (1);
 
 }
 
-void BeginReductions ()
+void BeginReductions ARGS ((void))
 {
 }
 
-void CloseReductions ()
+void CloseReductions ARGS ((void))
 {
 }
