@@ -1,6 +1,13 @@
 # include "SMParallel.h"
-# include "yySMParallel.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,37 +15,37 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 25 "SMParallel.puma"
+/* line 25 "SMParallel.puma" */
 
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
 # include "Types.h"
 # include "Transform.h"      /* CombineACF, CombineBTV, ... */
+# include "TreeOps.h"       
+# include "Rank.h"           
+# include "Objects.h"           
 # include "Traverse.h"       /* ReplaceAST                  */
 # include "VarDescriptor.h"  /* SetVarDescriptor            */
 
@@ -55,11 +62,25 @@ static tTree local1, local2, global;
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* SMParallel_Exit) () = yyExit;
+# include "yySMParallel.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* SMParallel_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -68,34 +89,35 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module SMParallel, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module SMParallel, routine %s failed\n",
+  yyFunction);
  SMParallel_Exit ();
 }
 
-static bool IsSMParallelLoop ARGS((tTree t));
-void FindSMParallel ARGS((tTree t));
-static bool StopFind ARGS((tTree t));
-static tTree FindPARALLEL ARGS((tTree t));
-static bool DoItParallel ARGS((tTree t));
-static void SetItParallel ARGS((tTree t));
-static bool IsDistributedId ARGS((tTree home_var, tTree id));
-void MakeSMParallel ARGS((tTree t));
-static bool StopPARALLEL ARGS((tTree t));
-static tTree TranslatePARALLEL ARGS((tTree t));
-tTree MakeParallel ARGS((tTree t));
-static tTree TranslateParBody ARGS((tTree t));
-static bool StopTranslation ARGS((tTree t));
-static tTree DoTranslation ARGS((tTree t));
-static void MakeLocalLoopVar ARGS((tTree t));
-static tTree ApplySMReduction ARGS((tTree reduction, tTree stmts));
-static bool StopREDUCTION ARGS((tTree t));
-static tTree TranslateREDUCTION ARGS((tTree t));
-static tTree GetRedVar ARGS((tTree t));
-void AutoScope ARGS((tTree t));
-static void AddSharedVariable ARGS((tTree t));
-tTree FinalSM ARGS((tTree t));
+static rbool IsSMParallelLoop ARGS ((tTree t));
+void FindSMParallel ARGS ((tTree t));
+static rbool StopFind ARGS ((tTree t));
+static tTree FindPARALLEL ARGS ((tTree t));
+static rbool DoItParallel ARGS ((tTree t));
+static void SetItParallel ARGS ((tTree t));
+static rbool IsDistributedId ARGS ((tTree home_var, tTree id));
+void MakeSMParallel ARGS ((tTree t));
+static rbool StopPARALLEL ARGS ((tTree t));
+static tTree TranslatePARALLEL ARGS ((tTree t));
+tTree MakeParallel ARGS ((tTree t));
+static tTree TranslateParBody ARGS ((tTree t));
+static rbool StopTranslation ARGS ((tTree t));
+static tTree DoTranslation ARGS ((tTree t));
+static void MakeLocalLoopVar ARGS ((tTree t));
+static tTree ApplySMReduction ARGS ((tTree reduction, tTree stmts));
+static rbool StopREDUCTION ARGS ((tTree t));
+static tTree TranslateREDUCTION ARGS ((tTree t));
+static tTree GetRedVar ARGS ((tTree t));
+void AutoScope ARGS ((tTree t));
+static void AddSharedVariable ARGS ((tTree t));
+tTree FinalSM ARGS ((tTree t));
 
-static bool IsSMParallelLoop
+static rbool IsSMParallelLoop
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -105,17 +127,17 @@ static bool IsSMParallelLoop
 {
   if (t->Kind == kACF_DO) {
   if (t->ACF_DO.DO_DEP_INFO->Kind == kINDEP_INFO) {
-# line 61 "SMParallel.puma"
+/* line 64 "SMParallel.puma" */
   {
-# line 63 "SMParallel.puma"
+/* line 66 "SMParallel.puma" */
    if (! ((t->ACF_DO.DO_DEP_INFO->INDEP_INFO.selection & kSELECT_CONCUR))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
 void FindSMParallel
@@ -127,25 +149,25 @@ void FindSMParallel
 # endif
 {
   if (t->Kind == kBODY_NODE) {
-# line 76 "SMParallel.puma"
+/* line 79 "SMParallel.puma" */
   {
-# line 78 "SMParallel.puma"
+/* line 81 "SMParallel.puma" */
    if (! ((sm_parallelization))) goto yyL1;
   {
-# line 80 "SMParallel.puma"
+/* line 83 "SMParallel.puma" */
  t->BODY_NODE.STATS = ReplaceAST (t->BODY_NODE.STATS, StopFind, FindPARALLEL); 
   }
   }
    return;
 yyL1:;
 
-# line 83 "SMParallel.puma"
+/* line 86 "SMParallel.puma" */
    return;
 
   }
-# line 86 "SMParallel.puma"
+/* line 89 "SMParallel.puma" */
   {
-# line 87 "SMParallel.puma"
+/* line 90 "SMParallel.puma" */
    failure_protocol (MODULE, "FindSMParallel", t);
   }
    return;
@@ -153,7 +175,7 @@ yyL1:;
 ;
 }
 
-static bool StopFind
+static rbool StopFind
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -162,11 +184,11 @@ static bool StopFind
 # endif
 {
   if (t->Kind == kACF_DO) {
-# line 101 "SMParallel.puma"
-   return true;
+/* line 104 "SMParallel.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
 static tTree FindPARALLEL
@@ -178,32 +200,32 @@ static tTree FindPARALLEL
 # endif
 {
   if (t->Kind == kACF_DO) {
-# line 114 "SMParallel.puma"
+/* line 117 "SMParallel.puma" */
   {
-# line 116 "SMParallel.puma"
+/* line 119 "SMParallel.puma" */
    if (! ((DoItParallel (t)))) goto yyL1;
   {
-# line 118 "SMParallel.puma"
+/* line 121 "SMParallel.puma" */
    SetItParallel (t);
   }
   }
    return t;
 yyL1:;
 
-# line 123 "SMParallel.puma"
+/* line 126 "SMParallel.puma" */
   {
-# line 127 "SMParallel.puma"
+/* line 130 "SMParallel.puma" */
  t->ACF_DO.DO_BODY = ReplaceAST (t->ACF_DO.DO_BODY, StopFind, FindPARALLEL); 
   }
    return t;
 
   }
-# line 132 "SMParallel.puma"
+/* line 135 "SMParallel.puma" */
    return t;
 
 }
 
-static bool DoItParallel
+static rbool DoItParallel
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -213,48 +235,48 @@ static bool DoItParallel
 {
   if (t->Kind == kACF_DO) {
   if (t->ACF_DO.DO_DEP_INFO->Kind == kINDEP_INFO) {
-# line 147 "SMParallel.puma"
+/* line 150 "SMParallel.puma" */
   {
-# line 149 "SMParallel.puma"
+/* line 152 "SMParallel.puma" */
    if (! ((sm_parallelization))) goto yyL1;
   {
-# line 150 "SMParallel.puma"
+/* line 153 "SMParallel.puma" */
    if (! ((t->ACF_DO.DO_DEP_INFO->INDEP_INFO.user_independent))) goto yyL1;
   {
-# line 152 "SMParallel.puma"
+/* line 155 "SMParallel.puma" */
    tree_protocol ("INDEPENDEDNT loop will be used for SM parallelism :\n", t);
   }
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   if (t->ACF_DO.DO_HOME_INFO->Kind == kCOMM_INFO) {
-# line 155 "SMParallel.puma"
+/* line 158 "SMParallel.puma" */
   {
-# line 157 "SMParallel.puma"
+/* line 160 "SMParallel.puma" */
    if (! ((sm_parallelization))) goto yyL2;
   {
-# line 158 "SMParallel.puma"
+/* line 161 "SMParallel.puma" */
    if (! ((IsDistributedId (t->ACF_DO.DO_HOME_INFO->COMM_INFO.home_var, t->ACF_DO.DO_ID)))) goto yyL2;
   {
-# line 160 "SMParallel.puma"
+/* line 163 "SMParallel.puma" */
    tree_protocol ("DO loop over distributed dimension will be used for SM parallelism :\n", t);
   }
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   }
-# line 164 "SMParallel.puma"
+/* line 167 "SMParallel.puma" */
   {
-# line 165 "SMParallel.puma"
+/* line 168 "SMParallel.puma" */
    tree_protocol ("DO loop not used for SM parallelism :\n", t);
-# line 166 "SMParallel.puma"
-   return false;
+/* line 169 "SMParallel.puma" */
+   return rfalse;
   }
 
 }
@@ -269,10 +291,12 @@ static void SetItParallel
 {
   if (t->Kind == kACF_DO) {
   if (t->ACF_DO.DO_DEP_INFO->Kind == kINDEP_INFO) {
-# line 179 "SMParallel.puma"
+/* line 182 "SMParallel.puma" */
   {
-# line 181 "SMParallel.puma"
- t->ACF_DO.DO_DEP_INFO->INDEP_INFO.selection = 1; 
+/* line 184 "SMParallel.puma" */
+ t->ACF_DO.DO_DEP_INFO->INDEP_INFO.selection = t->ACF_DO.DO_DEP_INFO->INDEP_INFO.selection | kSELECT_CONCUR; 
+/* line 186 "SMParallel.puma" */
+   FileUnparse (stdout, t);
   }
    return;
 
@@ -281,7 +305,7 @@ static void SetItParallel
 ;
 }
 
-static bool IsDistributedId
+static rbool IsDistributedId
 # if defined __STDC__ | defined __cplusplus
 (register tTree home_var, register tTree id)
 # else
@@ -290,25 +314,17 @@ static bool IsDistributedId
  register tTree id;
 # endif
 {
-# line 195 "SMParallel.puma"
+/* line 200 "SMParallel.puma" */
  {
   var_descriptor vard;
-  bool found;
+  rbool found;
   int topdim;
   tTree val;
   {
-# line 197 "SMParallel.puma"
-
-# line 198 "SMParallel.puma"
-
-# line 199 "SMParallel.puma"
-
-# line 200 "SMParallel.puma"
-
-# line 202 "SMParallel.puma"
+/* line 207 "SMParallel.puma" */
    SetVarDescriptor (home_var, & vard);
-# line 204 "SMParallel.puma"
- found  = false;
+/* line 209 "SMParallel.puma" */
+ found  = rfalse;
      topdim = 0;
 
      while ((!found) && (topdim < vard.topology_rank))
@@ -324,14 +340,14 @@ static bool IsDistributedId
      
 
    
-# line 221 "SMParallel.puma"
+/* line 226 "SMParallel.puma" */
    if (! ((found))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
 void MakeSMParallel
@@ -343,21 +359,21 @@ void MakeSMParallel
 # endif
 {
   if (t->Kind == kBODY_NODE) {
-# line 237 "SMParallel.puma"
+/* line 242 "SMParallel.puma" */
   {
-# line 239 "SMParallel.puma"
+/* line 244 "SMParallel.puma" */
    tree_protocol ("body before SM parallelism : \n", t->BODY_NODE.STATS);
-# line 241 "SMParallel.puma"
+/* line 246 "SMParallel.puma" */
  t->BODY_NODE.STATS = ReplaceAST (t->BODY_NODE.STATS, StopPARALLEL, TranslatePARALLEL); 
-# line 243 "SMParallel.puma"
+/* line 248 "SMParallel.puma" */
    tree_protocol ("body after SM parallelism : \n", t->BODY_NODE.STATS);
   }
    return;
 
   }
-# line 246 "SMParallel.puma"
+/* line 251 "SMParallel.puma" */
   {
-# line 247 "SMParallel.puma"
+/* line 252 "SMParallel.puma" */
    failure_protocol (MODULE, "MakeSMParallel", t);
   }
    return;
@@ -365,7 +381,7 @@ void MakeSMParallel
 ;
 }
 
-static bool StopPARALLEL
+static rbool StopPARALLEL
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -374,16 +390,16 @@ static bool StopPARALLEL
 # endif
 {
   if (t->Kind == kACF_DO) {
-# line 260 "SMParallel.puma"
+/* line 265 "SMParallel.puma" */
   {
-# line 262 "SMParallel.puma"
+/* line 267 "SMParallel.puma" */
    if (! ((IsSMParallelLoop (t)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
-  return false;
+  return rfalse;
 }
 
 static tTree TranslatePARALLEL
@@ -395,47 +411,43 @@ static tTree TranslatePARALLEL
 # endif
 {
   if (t->Kind == kACF_DO) {
-# line 273 "SMParallel.puma"
+/* line 278 "SMParallel.puma" */
  {
   tTree new;
   {
-# line 275 "SMParallel.puma"
+/* line 280 "SMParallel.puma" */
    if (! ((IsSMParallelLoop (t)))) goto yyL1;
   {
-# line 277 "SMParallel.puma"
-
-# line 279 "SMParallel.puma"
+/* line 284 "SMParallel.puma" */
    new = MakeParallel (t);
-# line 281 "SMParallel.puma"
+/* line 286 "SMParallel.puma" */
    tree_protocol ("translation of SM parallel loops :\n", new);
   }
   }
-  {
    return new;
-  }
  }
 yyL1:;
 
   }
   if (t->Kind == kACF_NEW) {
-# line 286 "SMParallel.puma"
+/* line 291 "SMParallel.puma" */
    return t->ACF_NEW.NEW_BODY;
 
   }
   if (t->Kind == kACF_REDUCTION) {
-# line 291 "SMParallel.puma"
+/* line 296 "SMParallel.puma" */
   {
-# line 293 "SMParallel.puma"
+/* line 298 "SMParallel.puma" */
    if (! ((sm_parallelization == NO_MP))) goto yyL3;
   }
    return t->ACF_REDUCTION.REDUCTION_BODY;
 yyL3:;
 
-# line 298 "SMParallel.puma"
+/* line 303 "SMParallel.puma" */
    return ApplySMReduction (t, t->ACF_REDUCTION.REDUCTION_BODY);
 
   }
-# line 303 "SMParallel.puma"
+/* line 308 "SMParallel.puma" */
    return t;
 
 }
@@ -449,13 +461,11 @@ tTree MakeParallel
 # endif
 {
   if (t->Kind == kACF_DO) {
-# line 318 "SMParallel.puma"
+/* line 323 "SMParallel.puma" */
  {
   tTree new;
   {
-# line 320 "SMParallel.puma"
-
-# line 322 "SMParallel.puma"
+/* line 327 "SMParallel.puma" */
  new_vars  = mBTV_EMPTY ();    
      par_loops = 1;
 
@@ -470,16 +480,14 @@ tTree MakeParallel
 
    
   }
-  {
    return new;
-  }
  }
 
   }
   if (t->Kind == kACF_PARALLEL) {
-# line 339 "SMParallel.puma"
+/* line 344 "SMParallel.puma" */
   {
-# line 341 "SMParallel.puma"
+/* line 346 "SMParallel.puma" */
  new_vars = t->ACF_PARALLEL.LOCAL_VARS;
 
      par_loops = 0;
@@ -493,9 +501,9 @@ tTree MakeParallel
    return t;
 
   }
-# line 354 "SMParallel.puma"
+/* line 359 "SMParallel.puma" */
   {
-# line 356 "SMParallel.puma"
+/* line 361 "SMParallel.puma" */
    failure_protocol (MODULE, "MakeParallel", t);
   }
    return t;
@@ -510,12 +518,12 @@ static tTree TranslateParBody
  register tTree t;
 # endif
 {
-# line 367 "SMParallel.puma"
+/* line 372 "SMParallel.puma" */
    return ReplaceAST (t, StopTranslation, DoTranslation);
 
 }
 
-static bool StopTranslation
+static rbool StopTranslation
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -524,21 +532,21 @@ static bool StopTranslation
 # endif
 {
   if (t->Kind == kACF_BASIC) {
-# line 374 "SMParallel.puma"
-   return true;
+/* line 379 "SMParallel.puma" */
+   return rtrue;
 
   }
   if (t->Kind == kACF_DO) {
-# line 377 "SMParallel.puma"
+/* line 382 "SMParallel.puma" */
   {
-# line 379 "SMParallel.puma"
+/* line 384 "SMParallel.puma" */
    if (! ((IsSMParallelLoop (t)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
-  return false;
+  return rfalse;
 }
 
 static tTree DoTranslation
@@ -550,36 +558,36 @@ static tTree DoTranslation
 # endif
 {
   if (t->Kind == kACF_HOME) {
-# line 384 "SMParallel.puma"
+/* line 389 "SMParallel.puma" */
    return t->ACF_HOME.HOME_BODY;
 
   }
   if (t->Kind == kACF_NEW) {
-# line 389 "SMParallel.puma"
+/* line 394 "SMParallel.puma" */
   {
-# line 391 "SMParallel.puma"
+/* line 396 "SMParallel.puma" */
  new_vars = ConcatVars (t->ACF_NEW.NEW_VAR, new_vars); 
   }
    return t->ACF_NEW.NEW_BODY;
 
   }
   if (t->Kind == kACF_RESIDENT) {
-# line 396 "SMParallel.puma"
+/* line 401 "SMParallel.puma" */
    return t->ACF_RESIDENT.RESIDENT_BODY;
 
   }
   if (t->Kind == kACF_REDUCTION) {
-# line 401 "SMParallel.puma"
+/* line 406 "SMParallel.puma" */
    return t->ACF_REDUCTION.REDUCTION_BODY;
 
   }
   if (t->Kind == kACF_DO) {
   if (t->ACF_DO.DO_DEP_INFO->Kind == kPARDO_INFO) {
-# line 406 "SMParallel.puma"
+/* line 411 "SMParallel.puma" */
   {
-# line 408 "SMParallel.puma"
+/* line 413 "SMParallel.puma" */
    MakeLocalLoopVar (t);
-# line 410 "SMParallel.puma"
+/* line 415 "SMParallel.puma" */
  par_loops ++;
 
      t->ACF_DO.DO_BODY = TranslateParBody (t->ACF_DO.DO_BODY);
@@ -594,9 +602,9 @@ static tTree DoTranslation
 
   }
   }
-# line 423 "SMParallel.puma"
+/* line 428 "SMParallel.puma" */
   {
-# line 425 "SMParallel.puma"
+/* line 430 "SMParallel.puma" */
    MakeLocalLoopVar (t);
   }
    return t;
@@ -612,9 +620,9 @@ static void MakeLocalLoopVar
 # endif
 {
   if (t->Kind == kACF_DO) {
-# line 432 "SMParallel.puma"
+/* line 437 "SMParallel.puma" */
   {
-# line 434 "SMParallel.puma"
+/* line 439 "SMParallel.puma" */
  new_vars = mBTV_LIST (CopyTree(t->ACF_DO.DO_ID), new_vars); 
   }
    return;
@@ -635,16 +643,16 @@ static tTree ApplySMReduction
   if (reduction->Kind == kACF_REDUCTION) {
   if (reduction->ACF_REDUCTION.REDUCTION_VAR->Kind == kBTV_LIST) {
   if (reduction->ACF_REDUCTION.REDUCTION_VAR->BTV_LIST.Next->Kind == kBTV_EMPTY) {
-# line 447 "SMParallel.puma"
+/* line 452 "SMParallel.puma" */
   {
-# line 449 "SMParallel.puma"
+/* line 454 "SMParallel.puma" */
    if (! ((TreeRank (reduction->ACF_REDUCTION.REDUCTION_VAR->BTV_LIST.Elem) > 0))) goto yyL1;
   {
-# line 451 "SMParallel.puma"
+/* line 456 "SMParallel.puma" */
    set_protocol_stmt (reduction);
-# line 453 "SMParallel.puma"
+/* line 458 "SMParallel.puma" */
    error_protocol ("SM reduction only for scalar variables");
-# line 454 "SMParallel.puma"
+/* line 459 "SMParallel.puma" */
    tree_protocol ("illegal reduction variable : ", reduction->ACF_REDUCTION.REDUCTION_VAR->BTV_LIST.Elem);
   }
   }
@@ -654,13 +662,11 @@ yyL1:;
   }
   }
   }
-# line 459 "SMParallel.puma"
+/* line 464 "SMParallel.puma" */
  {
   tTree new;
   {
-# line 461 "SMParallel.puma"
-
-# line 463 "SMParallel.puma"
+/* line 468 "SMParallel.puma" */
  global_reduction = reduction; 
 
      tree_protocol ("apply reduction : \n", reduction);
@@ -671,14 +677,12 @@ yyL1:;
 
    
   }
-  {
    return new;
-  }
  }
 
 }
 
-static bool StopREDUCTION
+static rbool StopREDUCTION
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -687,21 +691,21 @@ static bool StopREDUCTION
 # endif
 {
   if (t->Kind == kACF_DO) {
-# line 486 "SMParallel.puma"
+/* line 491 "SMParallel.puma" */
   {
-# line 488 "SMParallel.puma"
+/* line 493 "SMParallel.puma" */
    if (! ((IsSMParallelLoop (t)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   if (t->Kind == kACF_PARALLEL) {
-# line 491 "SMParallel.puma"
-   return true;
+/* line 496 "SMParallel.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
 static tTree TranslateREDUCTION
@@ -713,15 +717,15 @@ static tTree TranslateREDUCTION
 # endif
 {
   if (t->Kind == kACF_PARALLEL) {
-# line 502 "SMParallel.puma"
+/* line 507 "SMParallel.puma" */
   {
-# line 504 "SMParallel.puma"
+/* line 509 "SMParallel.puma" */
  t->ACF_PARALLEL.REDUCTION_VARS = CombineBTV (GetRedVar(global_reduction), t->ACF_PARALLEL.REDUCTION_VARS); 
   }
    return t;
 
   }
-# line 509 "SMParallel.puma"
+/* line 514 "SMParallel.puma" */
    return t;
 
 }
@@ -735,13 +739,13 @@ static tTree GetRedVar
 # endif
 {
   if (t->Kind == kACF_REDUCTION) {
-# line 515 "SMParallel.puma"
+/* line 520 "SMParallel.puma" */
    return t->ACF_REDUCTION.REDUCTION_VAR;
 
   }
-# line 522 "SMParallel.puma"
+/* line 527 "SMParallel.puma" */
   {
-# line 524 "SMParallel.puma"
+/* line 529 "SMParallel.puma" */
    failure_protocol (MODULE, "GetRedVar", t);
   }
    return NoTree;
@@ -757,32 +761,32 @@ void AutoScope
 # endif
 {
   if (t->Kind == kACF_PARALLEL) {
-# line 539 "SMParallel.puma"
+/* line 544 "SMParallel.puma" */
   {
-# line 542 "SMParallel.puma"
+/* line 547 "SMParallel.puma" */
  local1 = t->ACF_PARALLEL.REDUCTION_VARS;
      local2 = t->ACF_PARALLEL.LOCAL_VARS;
      global = t->ACF_PARALLEL.SHARED_VARS;
    
-# line 547 "SMParallel.puma"
+/* line 552 "SMParallel.puma" */
    tree_protocol ("AutoScope : local vars = ", t->ACF_PARALLEL.LOCAL_VARS);
-# line 548 "SMParallel.puma"
+/* line 553 "SMParallel.puma" */
    tree_protocol ("AutoScope : shared vars = ", t->ACF_PARALLEL.SHARED_VARS);
-# line 549 "SMParallel.puma"
+/* line 554 "SMParallel.puma" */
    tree_protocol ("AutoScope : red vars = ", t->ACF_PARALLEL.REDUCTION_VARS);
-# line 551 "SMParallel.puma"
+/* line 556 "SMParallel.puma" */
    FullTraverseAST (t->ACF_PARALLEL.PARALLEL_BODY, AddSharedVariable);
-# line 553 "SMParallel.puma"
+/* line 558 "SMParallel.puma" */
    tree_protocol ("AutoScope : new shared vars = ", global);
-# line 555 "SMParallel.puma"
+/* line 560 "SMParallel.puma" */
  t->ACF_PARALLEL.SHARED_VARS = global; 
   }
    return;
 
   }
-# line 558 "SMParallel.puma"
+/* line 563 "SMParallel.puma" */
   {
-# line 559 "SMParallel.puma"
+/* line 564 "SMParallel.puma" */
    failure_protocol (MODULE, "AutoScope", t);
   }
    return;
@@ -799,30 +803,30 @@ static void AddSharedVariable
 # endif
 {
   if (t->Kind == kUSED_VAR) {
-# line 565 "SMParallel.puma"
+/* line 570 "SMParallel.puma" */
   {
-# line 567 "SMParallel.puma"
+/* line 572 "SMParallel.puma" */
    if (! ((t->USED_VAR.VARNAME->VAR_OBJ.Object != NoObject))) goto yyL1;
   {
-# line 568 "SMParallel.puma"
+/* line 573 "SMParallel.puma" */
    if (! ((IsVarParameter (t->USED_VAR.VARNAME->VAR_OBJ.Object)))) goto yyL1;
   }
   }
    return;
 yyL1:;
 
-# line 571 "SMParallel.puma"
+/* line 576 "SMParallel.puma" */
   {
-# line 573 "SMParallel.puma"
+/* line 578 "SMParallel.puma" */
    if (! ((! IsVarInExp (t->USED_VAR.VARNAME->VAR_OBJ.Ident, local1)))) goto yyL2;
   {
-# line 574 "SMParallel.puma"
+/* line 579 "SMParallel.puma" */
    if (! ((! IsVarInExp (t->USED_VAR.VARNAME->VAR_OBJ.Ident, local2)))) goto yyL2;
   {
-# line 575 "SMParallel.puma"
+/* line 580 "SMParallel.puma" */
    if (! ((! IsVarInExp (t->USED_VAR.VARNAME->VAR_OBJ.Ident, global)))) goto yyL2;
   {
-# line 577 "SMParallel.puma"
+/* line 582 "SMParallel.puma" */
  global = mBTV_LIST (CopyTree (t), global); 
   }
   }
@@ -833,18 +837,18 @@ yyL2:;
 
   }
   if (t->Kind == kLOOP_VAR) {
-# line 582 "SMParallel.puma"
+/* line 587 "SMParallel.puma" */
   {
-# line 584 "SMParallel.puma"
+/* line 589 "SMParallel.puma" */
    if (! ((! IsVarInExp (t->LOOP_VAR.LOOP_VARNAME->VAR_OBJ.Ident, local1)))) goto yyL3;
   {
-# line 585 "SMParallel.puma"
+/* line 590 "SMParallel.puma" */
    if (! ((! IsVarInExp (t->LOOP_VAR.LOOP_VARNAME->VAR_OBJ.Ident, local2)))) goto yyL3;
   {
-# line 586 "SMParallel.puma"
+/* line 591 "SMParallel.puma" */
    if (! ((! IsVarInExp (t->LOOP_VAR.LOOP_VARNAME->VAR_OBJ.Ident, global)))) goto yyL3;
   {
-# line 588 "SMParallel.puma"
+/* line 593 "SMParallel.puma" */
  global = mBTV_LIST (CopyTree (t), global); 
   }
   }
@@ -867,41 +871,41 @@ tTree FinalSM
 {
   if (t->Kind == kACF_PARALLEL) {
   if (t->ACF_PARALLEL.REDUCTION_VARS->Kind == kBTV_EMPTY) {
-# line 601 "SMParallel.puma"
+/* line 606 "SMParallel.puma" */
    return t;
 
   }
-# line 606 "SMParallel.puma"
+/* line 611 "SMParallel.puma" */
   {
-# line 608 "SMParallel.puma"
+/* line 613 "SMParallel.puma" */
    if (! ((sm_parallelization == CRAY_MP))) goto yyL2;
   {
-# line 610 "SMParallel.puma"
+/* line 615 "SMParallel.puma" */
    set_protocol_stmt (t);
-# line 612 "SMParallel.puma"
+/* line 617 "SMParallel.puma" */
    serious_warning_protocol ("REDUCTION not supported here");
   }
   }
    return t->ACF_PARALLEL.PARALLEL_BODY;
 yyL2:;
 
-# line 617 "SMParallel.puma"
+/* line 622 "SMParallel.puma" */
    return t;
 
   }
-# line 622 "SMParallel.puma"
+/* line 627 "SMParallel.puma" */
   {
-# line 624 "SMParallel.puma"
+/* line 629 "SMParallel.puma" */
    failure_protocol (MODULE, "FinalSM", t);
   }
    return NoTree;
 
 }
 
-void BeginSMParallel ()
+void BeginSMParallel ARGS ((void))
 {
 }
 
-void CloseSMParallel ()
+void CloseSMParallel ARGS ((void))
 {
 }
