@@ -1,6 +1,13 @@
 # include "Scatter.h"
-# include "yyScatter.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,34 +15,31 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 31 "Scatter.puma"
+/* line 32 "Scatter.puma" */
 
 
 # define MODULE "Scatter"
 
 # define MAX_INDIRECT_RANK 4
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 # include "Types.h"      /* IntrFuncRed */
 # include "protocol.h"
 
@@ -61,11 +65,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* Scatter_Exit) () = yyExit;
+# include "yyScatter.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* Scatter_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -74,24 +92,27 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module Scatter, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module Scatter, routine %s failed\n",
+  yyFunction);
  Scatter_Exit ();
 }
 
-static tTree VarTopParam ARGS((tTree A));
-static void MakeIndDefine ARGS((tTree s_id, tTree A, tTree B, tTree M, tTree indexes, tTree * yyP2, tTree * yyP1));
-static void MakeScatterCall ARGS((int op, tTree A, tTree B, tTree M, tTree indexes, tTree * yyP3));
-static void MakeIndexParams ARGS((tTree indexes, int nr, int snr, tTree * yyP6, tTree * yyP5, tTree * yyP4));
-static tTree MakeDescriptorParam ARGS((tTree var, int nr));
-static void GetScatterParams ARGS((tTree params, tTree * yyP10, tTree * yyP9, tTree * yyP8, tTree * yyP7));
-static void SplitTail ARGS((tTree params, tTree * yyP12, tTree * yyP11));
-tTree MakeScatterStmt ARGS((tTree call));
-static bool BaseIsVar ARGS((tTree base, tTree var));
-static int GetScatterOp ARGS((tTree type, tIdent name));
-tTree CodeScatterStmt ARGS((tTree stmt));
-tTree CodeIndShadow ARGS((tTree s));
-static tTree CodeVarList ARGS((tTree vars, tTree params));
-static tTree DalibScheduleId ARGS((int id));
+static tTree VarTopParam ARGS ((tTree A));
+static void MakeIndDefine ARGS ((tTree s_id, tTree A, tTree B, tTree M, tTree indexes, tTree * yyP2, tTree * yyP1));
+static void MakeScatterCall ARGS ((int op, tTree A, tTree B, tTree M, tTree indexes, tTree * yyP3));
+static void MakeIndexParams ARGS ((tTree indexes, int nr, int snr, tTree * yyP6, tTree * yyP5, tTree * yyP4));
+static tTree MakeDescriptorParam ARGS ((tTree var, int nr));
+static void GetScatterParams ARGS ((tTree params, tTree * yyP10, tTree * yyP9, tTree * yyP8, tTree * yyP7));
+static void SplitTail ARGS ((tTree params, tTree * yyP12, tTree * yyP11));
+tTree MakeScatterStmt ARGS ((tTree call));
+static rbool BaseIsVar ARGS ((tTree base, tTree var));
+static int GetScatterOp ARGS ((tTree type, tIdent name));
+tTree CodeScatterStmt ARGS ((tTree stmt));
+rbool IsIndShadowStmt ARGS ((tTree s));
+tTree CodeIndShadow ARGS ((tTree s));
+static tTree CodeVarList ARGS ((tTree vars, int ptr, int dyn, tTree params));
+static tTree DalibScheduleId ARGS ((int id));
+static void SetParamAtt ARGS ((tTree param, int pass_by, int pointer, int dynamic));
 
 static tTree VarTopParam
 # if defined __STDC__ | defined __cplusplus
@@ -101,17 +122,20 @@ static tTree VarTopParam
  register tTree A;
 # endif
 {
+ yyRecursion:
   if (A->Kind == kINDEXED_VAR) {
-# line 70 "Scatter.puma"
-   return VarTopParam (A->INDEXED_VAR.IND_VAR);
+/* line 71 "Scatter.puma" */
+   A = A->INDEXED_VAR.IND_VAR;
+   goto yyRecursion;
 
   }
   if (A->Kind == kUSED_VAR) {
-# line 75 "Scatter.puma"
+/* line 76 "Scatter.puma" */
    return MakeTopologyParam (GetTopologyObj (A->USED_VAR.VARNAME->VAR_OBJ.Object));
 
   }
  yyAbort ("VarTopParam");
+ { tTree yyDummy; return yyDummy; }
 }
 
 static void MakeIndDefine
@@ -128,7 +152,7 @@ static void MakeIndDefine
  register tTree * yyP1;
 # endif
 {
-# line 92 "Scatter.puma"
+/* line 93 "Scatter.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -138,19 +162,11 @@ static void MakeIndDefine
   tTree pre1;
   tTree post1;
   {
-# line 94 "Scatter.puma"
+/* line 95 "Scatter.puma" */
    MakeIndexParams (indexes, TreeRank (B), INDEX_SEC_ID, & yyV1, & yyV2, & yyV3);
-# line 97 "Scatter.puma"
+/* line 98 "Scatter.puma" */
    yyV2 = CompleteBTPs (yyV2);
-# line 99 "Scatter.puma"
-
-# line 100 "Scatter.puma"
-
-# line 101 "Scatter.puma"
-
-# line 102 "Scatter.puma"
-
-# line 104 "Scatter.puma"
+/* line 105 "Scatter.puma" */
  if (M != NoTree)
 
        {  MakeIndexParams (M, 1, MASK_SEC_ID, &pre1, &param, &post1);
@@ -204,7 +220,7 @@ static void MakeScatterCall
  register tTree * yyP3;
 # endif
 {
-# line 145 "Scatter.puma"
+/* line 146 "Scatter.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -214,19 +230,11 @@ static void MakeScatterCall
   tTree pre1;
   tTree post1;
   {
-# line 147 "Scatter.puma"
+/* line 148 "Scatter.puma" */
    MakeIndexParams (indexes, TreeRank (B), INDEX_SEC_ID, & yyV1, & yyV2, & yyV3);
-# line 150 "Scatter.puma"
+/* line 151 "Scatter.puma" */
    yyV2 = CompleteBTPs (yyV2);
-# line 152 "Scatter.puma"
-
-# line 153 "Scatter.puma"
-
-# line 154 "Scatter.puma"
-
-# line 155 "Scatter.puma"
-
-# line 157 "Scatter.puma"
+/* line 158 "Scatter.puma" */
  if (M != NoTree)
  
        {  MakeIndexParams (M, 1, MASK_SEC_ID, &pre1, &param, &post1);
@@ -281,9 +289,9 @@ static void MakeIndexParams
 # endif
 {
   if (indexes->Kind == kBTP_LIST) {
-# line 200 "Scatter.puma"
+/* line 201 "Scatter.puma" */
   {
-# line 201 "Scatter.puma"
+/* line 202 "Scatter.puma" */
    if (! ((nr == 0))) goto yyL1;
   }
    * yyP6 = NoTree;
@@ -292,7 +300,7 @@ static void MakeIndexParams
    return;
 yyL1:;
 
-# line 204 "Scatter.puma"
+/* line 205 "Scatter.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -301,9 +309,9 @@ yyL1:;
   tTree yyV5;
   tTree yyV6;
   {
-# line 208 "Scatter.puma"
+/* line 209 "Scatter.puma" */
    MakeIndexParams (indexes->BTP_LIST.Elem, nr, snr, & yyV1, & yyV2, & yyV3);
-# line 209 "Scatter.puma"
+/* line 210 "Scatter.puma" */
    MakeIndexParams (indexes->BTP_LIST.Next, nr - 1, snr + 1, & yyV4, & yyV5, & yyV6);
   }
    * yyP6 = CombineACF (yyV1, yyV4);
@@ -314,7 +322,7 @@ yyL1:;
 
   }
   if (indexes->Kind == kBTP_EMPTY) {
-# line 212 "Scatter.puma"
+/* line 213 "Scatter.puma" */
    * yyP6 = NoTree;
    * yyP5 = NoTree;
    * yyP4 = NoTree;
@@ -322,13 +330,11 @@ yyL1:;
 
   }
   if (indexes->Kind == kNO_PARAM) {
-# line 215 "Scatter.puma"
+/* line 216 "Scatter.puma" */
  {
   tTree params;
   {
-# line 217 "Scatter.puma"
-
-# line 218 "Scatter.puma"
+/* line 219 "Scatter.puma" */
    params = mBTP_LIST (indexes, mBTP_LIST (indexes, NoTree));
   }
    * yyP6 = NoTree;
@@ -340,13 +346,11 @@ yyL1:;
   }
   if (indexes->Kind == kVAR_PARAM) {
   if (indexes->VAR_PARAM.V->Kind == kADDR) {
-# line 221 "Scatter.puma"
+/* line 222 "Scatter.puma" */
  {
   tTree params;
   {
-# line 223 "Scatter.puma"
-
-# line 224 "Scatter.puma"
+/* line 225 "Scatter.puma" */
    params = mBTP_LIST (indexes, mBTP_LIST (mNO_PARAM (kDUMMY_TYPE), NoTree));
   }
    * yyP6 = NoTree;
@@ -356,19 +360,17 @@ yyL1:;
  }
 
   }
-# line 227 "Scatter.puma"
+/* line 228 "Scatter.puma" */
  {
   tTree params;
   {
-# line 229 "Scatter.puma"
+/* line 230 "Scatter.puma" */
    if (! ((TreeRank (indexes->VAR_PARAM.V) == 0))) goto yyL6;
   {
-# line 230 "Scatter.puma"
+/* line 231 "Scatter.puma" */
    if (! ((TreeReadDistribution (indexes) == 0))) goto yyL6;
   {
-# line 232 "Scatter.puma"
-
-# line 233 "Scatter.puma"
+/* line 234 "Scatter.puma" */
    params = mBTP_LIST (indexes, mBTP_LIST (mNO_PARAM (kDUMMY_TYPE), NoTree));
   }
   }
@@ -380,13 +382,13 @@ yyL1:;
  }
 yyL6:;
 
-# line 236 "Scatter.puma"
+/* line 237 "Scatter.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 238 "Scatter.puma"
+/* line 239 "Scatter.puma" */
    MakeIndexParams (indexes->VAR_PARAM.V, nr, snr, & yyV1, & yyV2, & yyV3);
   }
    * yyP6 = yyV1;
@@ -397,21 +399,15 @@ yyL6:;
 
   }
   if (indexes->Kind == kINDEXED_VAR) {
-# line 241 "Scatter.puma"
+/* line 242 "Scatter.puma" */
  {
   tTree pre;
   tTree post;
   tTree params;
   {
-# line 243 "Scatter.puma"
-
-# line 244 "Scatter.puma"
-
-# line 245 "Scatter.puma"
-
-# line 247 "Scatter.puma"
+/* line 248 "Scatter.puma" */
    DalibSectionTranslation (snr, indexes, & pre, & params, & post);
-# line 248 "Scatter.puma"
+/* line 249 "Scatter.puma" */
    params = mBTP_LIST (mVAR_PARAM (FirstArrayElement (indexes)), mBTP_LIST (params, NoTree));
   }
    * yyP6 = pre;
@@ -421,9 +417,9 @@ yyL6:;
  }
 
   }
-# line 252 "Scatter.puma"
+/* line 253 "Scatter.puma" */
   {
-# line 254 "Scatter.puma"
+/* line 255 "Scatter.puma" */
    failure_protocol (MODULE, "MakeIndexParams", indexes);
   }
    * yyP6 = NoTree;
@@ -444,26 +440,26 @@ static tTree MakeDescriptorParam
 # endif
 {
   if (var->Kind == kUSED_VAR) {
-# line 268 "Scatter.puma"
+/* line 269 "Scatter.puma" */
    return mVAR_PARAM (MakeUsedVarA (TreeVarName (var->USED_VAR.VARNAME), "_DSP"));
 
   }
   if (var->Kind == kINDEXED_VAR) {
-# line 273 "Scatter.puma"
+/* line 274 "Scatter.puma" */
   {
-# line 275 "Scatter.puma"
+/* line 276 "Scatter.puma" */
    if (! ((IsWholeVar (var)))) goto yyL2;
   }
    return mVAR_PARAM (MakeUsedVarA (TreeVarName (var->INDEXED_VAR.IND_VAR), "_DSP"));
 yyL2:;
 
-# line 280 "Scatter.puma"
+/* line 281 "Scatter.puma" */
    return DalibSectionDescriptor (nr);
 
   }
-# line 285 "Scatter.puma"
+/* line 286 "Scatter.puma" */
   {
-# line 287 "Scatter.puma"
+/* line 288 "Scatter.puma" */
    failure_protocol (MODULE, "MakeDescriptorParam", var);
   }
    return NoTree;
@@ -486,12 +482,12 @@ static void GetScatterParams
   if (params->BTP_LIST.Elem->Kind == kVAR_PARAM) {
   if (params->BTP_LIST.Next->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Next->BTP_LIST.Elem->Kind == kVAR_PARAM) {
-# line 299 "Scatter.puma"
+/* line 300 "Scatter.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 302 "Scatter.puma"
+/* line 303 "Scatter.puma" */
    SplitTail (params->BTP_LIST.Next->BTP_LIST.Next, & yyV1, & yyV2);
   }
    * yyP10 = params->BTP_LIST.Elem->VAR_PARAM.V;
@@ -505,9 +501,9 @@ static void GetScatterParams
   }
   }
   }
-# line 305 "Scatter.puma"
+/* line 306 "Scatter.puma" */
   {
-# line 307 "Scatter.puma"
+/* line 308 "Scatter.puma" */
    failure_protocol (MODULE, "GetScatterParams", params);
   }
    * yyP10 = NoTree;
@@ -532,13 +528,11 @@ static void SplitTail
   if (params->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Next->Kind == kBTP_LIST) {
   if (params->BTP_LIST.Next->BTP_LIST.Next->Kind == kBTP_EMPTY) {
-# line 318 "Scatter.puma"
+/* line 319 "Scatter.puma" */
  {
   tTree save_mask;
   {
-# line 320 "Scatter.puma"
-
-# line 321 "Scatter.puma"
+/* line 322 "Scatter.puma" */
  save_mask = params->BTP_LIST.Next->BTP_LIST.Elem;
       params->BTP_LIST.Next = params->BTP_LIST.Next->BTP_LIST.Next;          
     
@@ -550,12 +544,12 @@ static void SplitTail
 
   }
   }
-# line 326 "Scatter.puma"
+/* line 327 "Scatter.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 328 "Scatter.puma"
+/* line 329 "Scatter.puma" */
    SplitTail (params->BTP_LIST.Next, & yyV1, & yyV2);
   }
    * yyP12 = params;
@@ -564,9 +558,9 @@ static void SplitTail
  }
 
   }
-# line 331 "Scatter.puma"
+/* line 332 "Scatter.puma" */
   {
-# line 333 "Scatter.puma"
+/* line 334 "Scatter.puma" */
    failure_protocol (MODULE, "SplitTail scatter params", params);
   }
    * yyP12 = NoTree;
@@ -587,16 +581,16 @@ tTree MakeScatterStmt
   if (call->Kind == kACF_BASIC) {
   if (call->ACF_BASIC.BASIC_STMT->Kind == kASSIGN_STMT) {
   if (call->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_EXP->Kind == kFUNC_CALL_EXP) {
-# line 346 "Scatter.puma"
+/* line 347 "Scatter.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   tTree yyV4;
   {
-# line 349 "Scatter.puma"
+/* line 350 "Scatter.puma" */
    GetScatterParams (call->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_EXP->FUNC_CALL_EXP.FUNC_PARAMS, & yyV1, & yyV2, & yyV3, & yyV4);
-# line 353 "Scatter.puma"
+/* line 354 "Scatter.puma" */
  if (!BaseIsVar (yyV2, call->ACF_BASIC.BASIC_STMT->ASSIGN_STMT.ASSIGN_VAR))
          error_protocol ("base of scatter must be lhs var");
 
@@ -604,24 +598,22 @@ tTree MakeScatterStmt
 
     
   }
-  {
    return call;
-  }
  }
 
   }
   }
   }
-# line 363 "Scatter.puma"
+/* line 364 "Scatter.puma" */
   {
-# line 364 "Scatter.puma"
+/* line 365 "Scatter.puma" */
    failure_protocol (MODULE, "MakeScatterStmt", call);
   }
    return NoTree;
 
 }
 
-static bool BaseIsVar
+static rbool BaseIsVar
 # if defined __STDC__ | defined __cplusplus
 (register tTree base, register tTree var)
 # else
@@ -632,21 +624,21 @@ static bool BaseIsVar
 {
   if (base->Kind == kINDEXED_VAR) {
   if (var->Kind == kINDEXED_VAR) {
-# line 370 "Scatter.puma"
+/* line 371 "Scatter.puma" */
   {
-# line 372 "Scatter.puma"
+/* line 373 "Scatter.puma" */
    if (! ((TreeVarName (base->INDEXED_VAR.IND_VAR) == TreeVarName (var->INDEXED_VAR.IND_VAR)))) goto yyL1;
   {
-# line 373 "Scatter.puma"
+/* line 374 "Scatter.puma" */
    if (! ((EqualIndexes (base->INDEXED_VAR.IND_EXPS, var->INDEXED_VAR.IND_EXPS)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
 static int GetScatterOp
@@ -658,21 +650,21 @@ static int GetScatterOp
  register tIdent name;
 # endif
 {
-# line 384 "Scatter.puma"
+/* line 385 "Scatter.puma" */
 
 char str[100];
 
-# line 388 "Scatter.puma"
+/* line 389 "Scatter.puma" */
   {
-# line 390 "Scatter.puma"
+/* line 391 "Scatter.puma" */
    if (! ((name == MakeIdent ("COPY_SCATTER", 12)))) goto yyL1;
   }
    return 0;
 yyL1:;
 
-# line 394 "Scatter.puma"
+/* line 395 "Scatter.puma" */
   {
-# line 398 "Scatter.puma"
+/* line 399 "Scatter.puma" */
    GetString (name, str);
   }
    return GetGlobalOp (type, MakeIdent (str, strlen (str) - 8));
@@ -688,7 +680,7 @@ tTree CodeScatterStmt
 # endif
 {
   if (stmt->Kind == kSCATTER_STMT) {
-# line 410 "Scatter.puma"
+/* line 411 "Scatter.puma" */
  {
   tTree stmtlist;
   tTree s_id;
@@ -698,24 +690,10 @@ tTree CodeScatterStmt
   tTree call;
   int nop;
   {
-# line 412 "Scatter.puma"
+/* line 413 "Scatter.puma" */
    if (! ((stmt->SCATTER_STMT.func == MakeIdent ("GET_SCATTER", 11)))) goto yyL1;
   {
-# line 414 "Scatter.puma"
-
-# line 415 "Scatter.puma"
-
-# line 416 "Scatter.puma"
-
-# line 417 "Scatter.puma"
-
-# line 418 "Scatter.puma"
-
-# line 419 "Scatter.puma"
-
-# line 420 "Scatter.puma"
-
-# line 422 "Scatter.puma"
+/* line 423 "Scatter.puma" */
  s_id = mVAR_PARAM (MakeUsedVarAn (MakeIdent ("IS",2), "_", 1));
      DefineNewIntVar (TreeVarName (s_id));
  
@@ -758,25 +736,19 @@ tTree CodeScatterStmt
    
   }
   }
-  {
    return stmtlist;
-  }
  }
 yyL1:;
 
-# line 466 "Scatter.puma"
+/* line 467 "Scatter.puma" */
  {
   tTree stmtlist;
   int nop;
   {
-# line 470 "Scatter.puma"
+/* line 471 "Scatter.puma" */
    if (! ((GetCurrentModel () == HPF_SERIAL))) goto yyL2;
   {
-# line 474 "Scatter.puma"
-
-# line 475 "Scatter.puma"
-
-# line 477 "Scatter.puma"
+/* line 478 "Scatter.puma" */
  nop = GetScatterOp (TreeType (stmt->SCATTER_STMT.BASE), stmt->SCATTER_STMT.func);
 
      MakeScatterCall (nop, stmt->SCATTER_STMT.ARRAY, stmt->SCATTER_STMT.BASE, stmt->SCATTER_STMT.MASK, stmt->SCATTER_STMT.INDEXES, &stmtlist);
@@ -784,13 +756,11 @@ yyL1:;
    
   }
   }
-  {
    return stmtlist;
-  }
  }
 yyL2:;
 
-# line 486 "Scatter.puma"
+/* line 487 "Scatter.puma" */
  {
   tTree stmtlist;
   tTree s_id;
@@ -800,21 +770,7 @@ yyL2:;
   tTree call;
   int nop;
   {
-# line 490 "Scatter.puma"
-
-# line 491 "Scatter.puma"
-
-# line 492 "Scatter.puma"
-
-# line 493 "Scatter.puma"
-
-# line 494 "Scatter.puma"
-
-# line 495 "Scatter.puma"
-
-# line 496 "Scatter.puma"
-
-# line 498 "Scatter.puma"
+/* line 499 "Scatter.puma" */
  s_id = mVAR_PARAM (MakeUsedVarAn (MakeIdent ("IS",2), "_", 1));
      DefineNewIntVar (TreeVarName (s_id));
 
@@ -859,21 +815,55 @@ yyL2:;
 
    
   }
-  {
    return stmtlist;
-  }
  }
 
   }
-# line 545 "Scatter.puma"
+/* line 546 "Scatter.puma" */
   {
-# line 546 "Scatter.puma"
+/* line 547 "Scatter.puma" */
    failure_protocol (MODULE, "CodeScatterStmt", stmt);
   }
    return NoTree;
 
 }
 
+rbool IsIndShadowStmt
+# if defined __STDC__ | defined __cplusplus
+(register tTree s)
+# else
+(s)
+ register tTree s;
+# endif
+{
+  if (s->Kind == kIND_SHADOW_CREATE) {
+/* line 558 "Scatter.puma" */
+   return rtrue;
+
+  }
+  if (s->Kind == kIND_SHADOW_FREE) {
+/* line 559 "Scatter.puma" */
+   return rtrue;
+
+  }
+  if (s->Kind == kIND_SHADOW_PUT) {
+/* line 560 "Scatter.puma" */
+   return rtrue;
+
+  }
+  if (s->Kind == kIND_SHADOW_GET) {
+/* line 561 "Scatter.puma" */
+   return rtrue;
+
+  }
+  if (s->Kind == kIND_SHADOW_SET) {
+/* line 562 "Scatter.puma" */
+   return rtrue;
+
+  }
+  return rfalse;
+}
+
 tTree CodeIndShadow
 # if defined __STDC__ | defined __cplusplus
 (register tTree s)
@@ -883,178 +873,150 @@ tTree CodeIndShadow
 # endif
 {
   if (s->Kind == kIND_SHADOW_CREATE) {
-# line 570 "Scatter.puma"
+/* line 587 "Scatter.puma" */
  {
   tTree call;
   tTree param;
   tTree params;
   {
-# line 572 "Scatter.puma"
-
-# line 573 "Scatter.puma"
-
-# line 574 "Scatter.puma"
-
-# line 576 "Scatter.puma"
+/* line 593 "Scatter.puma" */
    params = mBTP_EMPTY ();
-# line 577 "Scatter.puma"
-   params = CodeVarList (s->IND_SHADOW_CREATE.MASK, params);
-# line 578 "Scatter.puma"
-   params = CodeVarList (s->IND_SHADOW_CREATE.INDEX_VAR, params);
-# line 579 "Scatter.puma"
-   params = CodeVarList (s->IND_SHADOW_CREATE.TMP_INDEX_VAR, params);
-# line 580 "Scatter.puma"
-   params = CodeVarList (s->IND_SHADOW_CREATE.SHADOW_VAR, params);
-# line 581 "Scatter.puma"
+/* line 594 "Scatter.puma" */
+   params = CodeVarList (s->IND_SHADOW_CREATE.MASK, kIS_POINTER, kHPF_NOT_DYNAMIC, params);
+/* line 595 "Scatter.puma" */
+   params = CodeVarList (s->IND_SHADOW_CREATE.INDEX_VAR, kNOT_POINTER, kHPF_NOT_DYNAMIC, params);
+/* line 596 "Scatter.puma" */
+   params = CodeVarList (s->IND_SHADOW_CREATE.TMP_INDEX_VAR, kNOT_POINTER, kHPF_NOT_DYNAMIC, params);
+/* line 597 "Scatter.puma" */
+   params = CodeVarList (s->IND_SHADOW_CREATE.SHADOW_VAR, kNOT_POINTER, kHPF_IS_DYNAMIC, params);
+/* line 598 "Scatter.puma" */
    param = DalibScheduleId (s->IND_SHADOW_CREATE.id);
-# line 582 "Scatter.puma"
+/* line 599 "Scatter.puma" */
    params = mBTP_LIST (param, params);
-# line 583 "Scatter.puma"
+/* line 600 "Scatter.puma" */
    call = mPROC_OBJ (MakeDalibId ("insp_shadow_get"));
-# line 584 "Scatter.puma"
-   call = mACF_BASIC (mCALL_STMT (call, params));
+/* line 601 "Scatter.puma" */
+   call = mCALL_STMT (call, params);
   }
-  {
    return call;
-  }
  }
 
   }
   if (s->Kind == kIND_SHADOW_FREE) {
-# line 589 "Scatter.puma"
+/* line 606 "Scatter.puma" */
  {
   tTree call;
   tTree param;
   tTree params;
   {
-# line 591 "Scatter.puma"
-
-# line 592 "Scatter.puma"
-
-# line 593 "Scatter.puma"
-
-# line 595 "Scatter.puma"
+/* line 612 "Scatter.puma" */
    params = mBTP_EMPTY ();
-# line 596 "Scatter.puma"
-   params = CodeVarList (s->IND_SHADOW_FREE.INDEX_VAR, params);
-# line 597 "Scatter.puma"
+/* line 613 "Scatter.puma" */
+   params = CodeVarList (s->IND_SHADOW_FREE.INDEX_VAR, kNOT_POINTER, kHPF_NOT_DYNAMIC, params);
+/* line 614 "Scatter.puma" */
    param = DalibScheduleId (s->IND_SHADOW_FREE.id);
-# line 598 "Scatter.puma"
+/* line 615 "Scatter.puma" */
    params = mBTP_LIST (param, params);
-# line 599 "Scatter.puma"
+/* line 616 "Scatter.puma" */
    call = mPROC_OBJ (MakeDalibId ("insp_shadow_release"));
-# line 600 "Scatter.puma"
-   call = mACF_BASIC (mCALL_STMT (call, params));
+/* line 617 "Scatter.puma" */
+   call = mCALL_STMT (call, params);
   }
-  {
    return call;
-  }
  }
 
   }
   if (s->Kind == kIND_SHADOW_GET) {
-# line 605 "Scatter.puma"
+/* line 622 "Scatter.puma" */
  {
   tTree call;
   tTree params;
   tTree param;
   {
-# line 607 "Scatter.puma"
-
-# line 608 "Scatter.puma"
-
-# line 609 "Scatter.puma"
-
-# line 611 "Scatter.puma"
+/* line 628 "Scatter.puma" */
    params = mBTP_EMPTY ();
-# line 612 "Scatter.puma"
-   params = mBTP_LIST (MakeDescriptorParam (s->IND_SHADOW_GET.SHADOW_VAR, 0), params);
-# line 613 "Scatter.puma"
-   param = DalibScheduleId (s->IND_SHADOW_GET.id);
-# line 614 "Scatter.puma"
+/* line 629 "Scatter.puma" */
+   param = mVAR_PARAM (s->IND_SHADOW_GET.SHADOW_VAR);
+/* line 630 "Scatter.puma" */
+   SetParamAtt (param, kHPF_HANDLE_PASS_BY, kNOT_POINTER, kHPF_NOT_DYNAMIC);
+/* line 631 "Scatter.puma" */
    params = mBTP_LIST (param, params);
-# line 615 "Scatter.puma"
+/* line 632 "Scatter.puma" */
+   param = DalibScheduleId (s->IND_SHADOW_GET.id);
+/* line 633 "Scatter.puma" */
+   params = mBTP_LIST (param, params);
+/* line 634 "Scatter.puma" */
    call = mPROC_OBJ (MakeDalibId ("exec_shadow_get"));
-# line 616 "Scatter.puma"
-   call = mACF_BASIC (mCALL_STMT (call, params));
+/* line 635 "Scatter.puma" */
+   call = mCALL_STMT (call, params);
   }
-  {
    return call;
-  }
  }
 
   }
   if (s->Kind == kIND_SHADOW_SET) {
-# line 621 "Scatter.puma"
+/* line 640 "Scatter.puma" */
  {
   tTree call;
   tTree params;
   tTree param;
   {
-# line 623 "Scatter.puma"
-
-# line 624 "Scatter.puma"
-
-# line 625 "Scatter.puma"
-
-# line 627 "Scatter.puma"
+/* line 646 "Scatter.puma" */
    params = mBTP_EMPTY ();
-# line 628 "Scatter.puma"
+/* line 647 "Scatter.puma" */
    params = mBTP_LIST (ExpToVarParam (s->IND_SHADOW_SET.VAL), params);
-# line 629 "Scatter.puma"
-   params = mBTP_LIST (MakeDescriptorParam (s->IND_SHADOW_SET.SHADOW_VAR, 0), params);
-# line 630 "Scatter.puma"
-   param = DalibScheduleId (s->IND_SHADOW_SET.id);
-# line 631 "Scatter.puma"
+/* line 648 "Scatter.puma" */
+   param = mVAR_PARAM (s->IND_SHADOW_SET.SHADOW_VAR);
+/* line 649 "Scatter.puma" */
+   SetParamAtt (param, kHPF_HANDLE_PASS_BY, kNOT_POINTER, kHPF_NOT_DYNAMIC);
+/* line 650 "Scatter.puma" */
    params = mBTP_LIST (param, params);
-# line 632 "Scatter.puma"
+/* line 651 "Scatter.puma" */
+   param = DalibScheduleId (s->IND_SHADOW_SET.id);
+/* line 652 "Scatter.puma" */
+   params = mBTP_LIST (param, params);
+/* line 653 "Scatter.puma" */
    call = mPROC_OBJ (MakeDalibId ("exec_shadow_set"));
-# line 633 "Scatter.puma"
-   call = mACF_BASIC (mCALL_STMT (call, params));
+/* line 654 "Scatter.puma" */
+   call = mCALL_STMT (call, params);
   }
-  {
    return call;
-  }
  }
 
   }
   if (s->Kind == kIND_SHADOW_PUT) {
-# line 638 "Scatter.puma"
+/* line 659 "Scatter.puma" */
  {
   tTree call;
   tTree params;
   tTree param;
   {
-# line 640 "Scatter.puma"
-
-# line 641 "Scatter.puma"
-
-# line 642 "Scatter.puma"
-
-# line 644 "Scatter.puma"
+/* line 665 "Scatter.puma" */
    params = mBTP_EMPTY ();
-# line 645 "Scatter.puma"
+/* line 666 "Scatter.puma" */
    params = mBTP_LIST (ExpToVarParam (MakeConstant (s->IND_SHADOW_PUT.op)), params);
-# line 646 "Scatter.puma"
-   params = mBTP_LIST (MakeDescriptorParam (s->IND_SHADOW_PUT.SHADOW_VAR, 0), params);
-# line 647 "Scatter.puma"
-   param = DalibScheduleId (s->IND_SHADOW_PUT.id);
-# line 648 "Scatter.puma"
+/* line 667 "Scatter.puma" */
+   param = mVAR_PARAM (s->IND_SHADOW_PUT.SHADOW_VAR);
+/* line 668 "Scatter.puma" */
+   SetParamAtt (param, kHPF_HANDLE_PASS_BY, kNOT_POINTER, kHPF_NOT_DYNAMIC);
+/* line 669 "Scatter.puma" */
    params = mBTP_LIST (param, params);
-# line 649 "Scatter.puma"
+/* line 670 "Scatter.puma" */
+   param = DalibScheduleId (s->IND_SHADOW_PUT.id);
+/* line 671 "Scatter.puma" */
+   params = mBTP_LIST (param, params);
+/* line 672 "Scatter.puma" */
    call = mPROC_OBJ (MakeDalibId ("exec_shadow_put"));
-# line 650 "Scatter.puma"
-   call = mACF_BASIC (mCALL_STMT (call, params));
+/* line 673 "Scatter.puma" */
+   call = mCALL_STMT (call, params);
   }
-  {
    return call;
-  }
  }
 
   }
-# line 655 "Scatter.puma"
+/* line 678 "Scatter.puma" */
   {
-# line 657 "Scatter.puma"
+/* line 680 "Scatter.puma" */
    failure_protocol (MODULE, "CodeIndShadow", s);
   }
    return NoTree;
@@ -1063,45 +1025,48 @@ tTree CodeIndShadow
 
 static tTree CodeVarList
 # if defined __STDC__ | defined __cplusplus
-(register tTree vars, register tTree params)
+(register tTree vars, register int ptr, register int dyn, register tTree params)
 # else
-(vars, params)
+(vars, ptr, dyn, params)
  register tTree vars;
+ register int ptr;
+ register int dyn;
  register tTree params;
 # endif
 {
   if (vars->Kind == kBTV_EMPTY) {
-# line 670 "Scatter.puma"
+/* line 693 "Scatter.puma" */
    return params;
 
   }
   if (vars->Kind == kBTV_LIST) {
-# line 675 "Scatter.puma"
-   return CodeVarList (vars->BTV_LIST.Elem, CodeVarList (vars->BTV_LIST.Next, params));
+/* line 698 "Scatter.puma" */
+   return CodeVarList (vars->BTV_LIST.Elem, ptr, dyn, CodeVarList (vars->BTV_LIST.Next, ptr, dyn, params));
 
   }
   if (vars->Kind == kDUMMY_VAR) {
-# line 680 "Scatter.puma"
+/* line 703 "Scatter.puma" */
    return mBTP_LIST (mNO_PARAM (kDUMMY_TYPE), mBTP_LIST (mNO_PARAM (kDUMMY_TYPE), params));
 
   }
-# line 686 "Scatter.puma"
+/* line 709 "Scatter.puma" */
  {
+  tTree full_var;
   tTree param_data;
   tTree param_dsp;
   {
-# line 688 "Scatter.puma"
-
-# line 689 "Scatter.puma"
-
-# line 691 "Scatter.puma"
-   param_dsp = mVAR_PARAM (MakeUsedVarA (TreeVarName (vars), "_DSP"));
-# line 692 "Scatter.puma"
-   param_data = mVAR_PARAM (FirstArrayElement (vars));
+/* line 715 "Scatter.puma" */
+   full_var = MakeFullShape (vars);
+/* line 717 "Scatter.puma" */
+   param_dsp = mVAR_PARAM (CopyTree (full_var));
+/* line 718 "Scatter.puma" */
+   SetParamAtt (param_dsp, kHPF_HANDLE_PASS_BY, ptr, dyn);
+/* line 720 "Scatter.puma" */
+   param_data = mVAR_PARAM (full_var);
+/* line 721 "Scatter.puma" */
+   SetParamAtt (param_data, kDATA_PASS_BY, kNOT_POINTER, kHPF_NOT_DYNAMIC);
   }
-  {
    return mBTP_LIST (param_data, mBTP_LIST (param_dsp, params));
-  }
  }
 
 }
@@ -1114,28 +1079,66 @@ static tTree DalibScheduleId
  register int id;
 # endif
 {
-# line 705 "Scatter.puma"
+/* line 734 "Scatter.puma" */
  {
-  tTree var;
+  tTree param;
   {
-# line 707 "Scatter.puma"
-
-# line 709 "Scatter.puma"
-   var = MakeUsedVarAn (IsIdent ("S"), "_ID", id);
-# line 710 "Scatter.puma"
-   DefineNewIntVar (TreeVarName (var));
-  }
-  {
-   return mVAR_PARAM (var);
+/* line 738 "Scatter.puma" */
+   param = MakeUsedVarAn (IsIdent ("S"), "_ID", id);
+/* line 739 "Scatter.puma" */
+   DefineNewIntVar (TreeVarName (param));
+/* line 740 "Scatter.puma" */
+   param = mVAR_PARAM (param);
+/* line 741 "Scatter.puma" */
+   SetParamAtt (param, kDATA_PASS_BY, 0, 2);
   }
+   return param;
  }
 
 }
 
-void BeginScatter ()
+static void SetParamAtt
+# if defined __STDC__ | defined __cplusplus
+(register tTree param, register int pass_by, register int pointer, register int dynamic)
+# else
+(param, pass_by, pointer, dynamic)
+ register tTree param;
+ register int pass_by;
+ register int pointer;
+ register int dynamic;
+# endif
+{
+  if (param->Kind == kVAR_PARAM) {
+/* line 759 "Scatter.puma" */
+  {
+/* line 763 "Scatter.puma" */
+ param->VAR_PARAM.pass_by = pass_by;
+     param->VAR_PARAM.pointer = pointer;
+     param->VAR_PARAM.dynamic = dynamic;
+   
+  }
+   return;
+
+  }
+  if (param->Kind == kNO_PARAM) {
+/* line 769 "Scatter.puma" */
+   return;
+
+  }
+/* line 772 "Scatter.puma" */
+  {
+/* line 774 "Scatter.puma" */
+   failure_protocol (MODULE, "SetParamAtt", param);
+  }
+   return;
+
+;
+}
+
+void BeginScatter ARGS ((void))
 {
 }
 
-void CloseScatter ()
+void CloseScatter ARGS ((void))
 {
 }
