@@ -1,6 +1,13 @@
 # include "SemMap.h"
-# include "yySemMap.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,31 +15,28 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 40 "SemMap.puma"
+/* line 39 "SemMap.puma" */
 
 
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 # include "Types.h"
 
 # include "DefTable.h"
@@ -53,11 +57,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* SemMap_Exit) () = yyExit;
+# include "yySemMap.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* SemMap_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -66,36 +84,36 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module SemMap, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module SemMap, routine %s failed\n",
+  yyFunction);
  SemMap_Exit ();
 }
 
-void EvalObjDistribution ARGS((tDefinitions t));
-static tDefinitions EvalDistribution ARGS((tDefinitions o, tDefinitions d));
-static tDefinitions EvalDistributionDecl ARGS((tTree layout_decl));
-tDefinitions EvalDistributionStmt ARGS((tTree layout_stmt));
-static tDefinitions EvalRangeDecl ARGS((tTree layout_decl));
-static void GetObjectInfo ARGS((tTree var, tIdent * yyP2, tDefinitions * yyP1));
-static tDefinitions EvalDistributionSpec ARGS((tTree d_spec, tTree p_spec));
-static tDefinitions EvalRangeSpec ARGS((tIdent tid, tTree d_spec));
-static bool IsReplicatedDistribution ARGS((tTree t));
-static tDefinitions ComputeNodeDistribution ARGS((tTree mapping, tDefinitions p_obj));
-static tDefinitions ComputeRangeDistribution ARGS((tTree mapping));
-static tDefinitions MakeDistributionList ARGS((tTree mapping, int top_dim, int temp_dim));
-static void CodeDistribution ARGS((tTree distribution, int * yyP6, tTree * yyP5, tTree * yyP4, tTree * yyP3));
-static bool IsLegalIntArray ARGS((tTree int_array));
-static int NoDistributedDims ARGS((tDefinitions dlist));
-static tDefinitions EvalAlignmentDecl ARGS((tTree align_decl));
-tDefinitions EvalAlignmentStmt ARGS((tTree align_stmt));
-static tDefinitions GetObjProcDistribution ARGS((tDefinitions obj));
-static tDefinitions ComputeAlignDistribution ARGS((tTree source, tTree target, tDefinitions temp_obj));
-static tDefinitions MakeAlignTargetDimList ARGS((int temp_dim, tTree source, tTree target));
-static tDefinitions MakeAlignSourceDimList ARGS((int i, int n, tDefinitions target_list));
-static void FindEntry ARGS((tDefinitions target_list, int index, tDefinitions * yyP7));
-void MakeObjReplicated ARGS((tDefinitions t));
-static tDefinitions MakeReplicatedDistribution ARGS((tDefinitions d));
+void EvalObjSpecMapping ARGS ((tDefinitions t));
+void EvalObjRangeMapping ARGS ((tDefinitions t));
+static tDefinitions EvalSpecMapDecl ARGS ((tDefinitions obj, tDefinitions d));
+static tDefinitions EvalRangeMapDecl ARGS ((tDefinitions obj, tDefinitions d));
+static tDefinitions EvalDistributionDecl ARGS ((tTree layout_decl));
+tDefinitions EvalDistributionStmt ARGS ((tTree layout_stmt));
+static tDefinitions EvalRangeDecl ARGS ((tTree layout_decl));
+static void GetObjectInfo ARGS ((tTree var, tIdent * yyP2, tDefinitions * yyP1));
+static tDefinitions EvalDistributionSpec ARGS ((tTree d_spec, tTree p_spec));
+static tDefinitions EvalRangeSpec ARGS ((tIdent tid, tTree d_spec));
+static rbool IsReplicatedDistribution ARGS ((tTree t));
+static tDefinitions ComputeNodeDistribution ARGS ((tTree mapping, tDefinitions p_obj));
+static tDefinitions ComputeRangeDistribution ARGS ((tTree mapping));
+static tDefinitions MakeDistDimList ARGS ((tTree mapping, int top_dim, int temp_dim));
+static void CodeDistribution ARGS ((tTree distribution, int * yyP6, tTree * yyP5, tTree * yyP4, tTree * yyP3));
+static rbool IsLegalIntArray ARGS ((tTree int_array));
+static int NoDistributedDims ARGS ((tDefinitions dlist));
+static tDefinitions EvalAlignmentDecl ARGS ((tTree align_decl));
+tDefinitions EvalAlignmentStmt ARGS ((tTree align_stmt));
+static tDefinitions ComputeAlignment ARGS ((tTree source, tTree target, tDefinitions temp_obj));
+static tDefinitions MakeAlignTargetDimList ARGS ((int temp_dim, tTree source, tTree target));
+static tDefinitions MakeAlignSourceDimList ARGS ((int i, int n, tDefinitions target_list));
+static void FindEntry ARGS ((tDefinitions target_list, int index, tDefinitions * yyP7));
 
-void EvalObjDistribution
+void EvalObjSpecMapping
 # if defined __STDC__ | defined __cplusplus
 (register tDefinitions t)
 # else
@@ -104,130 +122,154 @@ void EvalObjDistribution
 # endif
 {
   if (t->Kind == kTemplateObject) {
-# line 76 "SemMap.puma"
+  if (t->TemplateObject.Dist->Mapping.spec->Kind == kMapDecl) {
+/* line 75 "SemMap.puma" */
   {
-# line 78 "SemMap.puma"
-   t->TemplateObject.Dist = EvalDistribution (t, t->TemplateObject.Dist);
+/* line 77 "SemMap.puma" */
+ t->TemplateObject.Dist->Mapping.spec = EvalSpecMapDecl (t, t->TemplateObject.Dist->Mapping.spec); 
   }
    return;
 
+  }
   }
   if (t->Kind == kVarObject) {
-# line 81 "SemMap.puma"
+  if (t->VarObject.Dist->Mapping.spec->Kind == kMapDecl) {
+/* line 80 "SemMap.puma" */
   {
-# line 83 "SemMap.puma"
-   t->VarObject.Dist = EvalDistribution (t, t->VarObject.Dist);
+/* line 82 "SemMap.puma" */
+ t->VarObject.Dist->Mapping.spec = EvalSpecMapDecl (t, t->VarObject.Dist->Mapping.spec); 
   }
    return;
 
   }
-# line 86 "SemMap.puma"
+  }
+/* line 85 "SemMap.puma" */
   {
-# line 87 "SemMap.puma"
-   obj_error_protocol ("EvalObjDistribution illegal", t);
+/* line 87 "SemMap.puma" */
+   obj_error_protocol ("EvalObjSpecMapping illegal", t);
   }
    return;
 
 ;
 }
 
-static tDefinitions EvalDistribution
+void EvalObjRangeMapping
 # if defined __STDC__ | defined __cplusplus
-(register tDefinitions o, register tDefinitions d)
+(register tDefinitions t)
 # else
-(o, d)
- register tDefinitions o;
+(t)
+ register tDefinitions t;
+# endif
+{
+  if (t->Kind == kTemplateObject) {
+  if (t->TemplateObject.Dist->Mapping.range->Kind == kRangeDecl) {
+/* line 92 "SemMap.puma" */
+  {
+/* line 94 "SemMap.puma" */
+ t->TemplateObject.Dist->Mapping.range = EvalRangeMapDecl (t, t->TemplateObject.Dist->Mapping.range); 
+  }
+   return;
+
+  }
+  }
+  if (t->Kind == kVarObject) {
+  if (t->VarObject.Dist->Mapping.range->Kind == kRangeDecl) {
+/* line 97 "SemMap.puma" */
+  {
+/* line 99 "SemMap.puma" */
+ t->VarObject.Dist->Mapping.range = EvalRangeMapDecl (t, t->VarObject.Dist->Mapping.range); 
+  }
+   return;
+
+  }
+  }
+/* line 102 "SemMap.puma" */
+  {
+/* line 104 "SemMap.puma" */
+   obj_error_protocol ("EvalObjSpecMapping illegal", t);
+  }
+   return;
+
+;
+}
+
+static tDefinitions EvalSpecMapDecl
+# if defined __STDC__ | defined __cplusplus
+(register tDefinitions obj, register tDefinitions d)
+# else
+(obj, d)
+ register tDefinitions obj;
  register tDefinitions d;
 # endif
 {
-  if (d->Kind == kDeclDistribution) {
-  if (d->DeclDistribution.decl->Kind == kDISTRIBUTE_DECL) {
-# line 100 "SemMap.puma"
- {
-  tDefinitions d1;
+  if (d->Kind == kMapDecl) {
+  if (d->MapDecl.decl->Kind == kDISTRIBUTE_DECL) {
+/* line 115 "SemMap.puma" */
   {
-# line 102 "SemMap.puma"
-   set_protocol_stmt (d->DeclDistribution.decl);
-# line 104 "SemMap.puma"
-   ReplaceDefaultDistributions (o, d->DeclDistribution.decl);
-# line 105 "SemMap.puma"
-   ControlDistributeDecl (o, d->DeclDistribution.decl);
-# line 107 "SemMap.puma"
-
-# line 109 "SemMap.puma"
-   d1 = EvalDistributionDecl (d->DeclDistribution.decl);
-# line 111 "SemMap.puma"
-   d1->Distribution.shared = d->Distribution.shared;
-# line 112 "SemMap.puma"
-   d1->Distribution.dynamic = d->Distribution.dynamic;
-# line 113 "SemMap.puma"
-   d1->Distribution.inherited = d->Distribution.inherited;
+/* line 117 "SemMap.puma" */
+   set_protocol_stmt (d->MapDecl.decl);
+/* line 119 "SemMap.puma" */
+   ReplaceDefaultDistributions (obj, d->MapDecl.decl);
+/* line 120 "SemMap.puma" */
+   ControlDistributeDecl (obj, d->MapDecl.decl);
   }
-  {
-   return d1;
-  }
- }
+   return EvalDistributionDecl (d->MapDecl.decl);
 
   }
-  if (d->DeclDistribution.decl->Kind == kRANGE_DECL) {
-# line 118 "SemMap.puma"
- {
-  tDefinitions d1;
+  if (d->MapDecl.decl->Kind == kALIGN_DECL) {
+/* line 125 "SemMap.puma" */
   {
-# line 120 "SemMap.puma"
-   set_protocol_stmt (d->DeclDistribution.decl);
-# line 122 "SemMap.puma"
-
-# line 124 "SemMap.puma"
-   d1 = EvalRangeDecl (d->DeclDistribution.decl);
-# line 126 "SemMap.puma"
-   d1->Distribution.shared = d->Distribution.shared;
-# line 127 "SemMap.puma"
-   d1->Distribution.dynamic = d->Distribution.dynamic;
-# line 128 "SemMap.puma"
-   d1->Distribution.inherited = d->Distribution.inherited;
+/* line 127 "SemMap.puma" */
+   set_protocol_stmt (d->MapDecl.decl);
   }
-  {
-   return d1;
-  }
- }
+   return EvalAlignmentDecl (d->MapDecl.decl);
 
   }
-  if (d->DeclDistribution.decl->Kind == kALIGN_DECL) {
-# line 133 "SemMap.puma"
- {
-  tDefinitions d1;
+/* line 132 "SemMap.puma" */
   {
-# line 135 "SemMap.puma"
-   set_protocol_stmt (d->DeclDistribution.decl);
-# line 137 "SemMap.puma"
-
-# line 139 "SemMap.puma"
-   d1 = EvalAlignmentDecl (d->DeclDistribution.decl);
-# line 141 "SemMap.puma"
-   d1->Distribution.shared = d->Distribution.shared;
-# line 142 "SemMap.puma"
-   d1->Distribution.dynamic = d->Distribution.dynamic;
-# line 143 "SemMap.puma"
-   d1->Distribution.inherited = d->Distribution.inherited;
-  }
-  {
-   return d1;
-  }
- }
-
-  }
-# line 148 "SemMap.puma"
-  {
-# line 150 "SemMap.puma"
-   set_protocol_stmt (d->DeclDistribution.decl);
-# line 152 "SemMap.puma"
-   failure_protocol (MODULE, "EvalDistribution", d->DeclDistribution.decl);
+/* line 134 "SemMap.puma" */
+   set_protocol_stmt (d->MapDecl.decl);
+/* line 136 "SemMap.puma" */
+   failure_protocol (MODULE, "EvalDistribution", d->MapDecl.decl);
   }
    return d;
 
   }
-# line 157 "SemMap.puma"
+/* line 141 "SemMap.puma" */
+   return d;
+
+}
+
+static tDefinitions EvalRangeMapDecl
+# if defined __STDC__ | defined __cplusplus
+(register tDefinitions obj, register tDefinitions d)
+# else
+(obj, d)
+ register tDefinitions obj;
+ register tDefinitions d;
+# endif
+{
+  if (d->Kind == kRangeDecl) {
+  if (d->RangeDecl.decl->Kind == kRANGE_DECL) {
+/* line 154 "SemMap.puma" */
+  {
+/* line 156 "SemMap.puma" */
+   set_protocol_stmt (d->RangeDecl.decl);
+  }
+   return EvalRangeDecl (d->RangeDecl.decl);
+
+  }
+/* line 161 "SemMap.puma" */
+  {
+/* line 163 "SemMap.puma" */
+   set_protocol_stmt (d->RangeDecl.decl);
+/* line 165 "SemMap.puma" */
+   failure_protocol (MODULE, "EvalRangeMapDecl", d->RangeDecl.decl);
+  }
+   return d;
+
+  }
+/* line 170 "SemMap.puma" */
    return d;
 
 }
@@ -241,16 +283,16 @@ static tDefinitions EvalDistributionDecl
 # endif
 {
   if (layout_decl->Kind == kDISTRIBUTE_DECL) {
-# line 173 "SemMap.puma"
+/* line 186 "SemMap.puma" */
    return EvalDistributionSpec (layout_decl->DISTRIBUTE_DECL.DISTRIBUTION, layout_decl->DISTRIBUTE_DECL.TARGET);
 
   }
-# line 178 "SemMap.puma"
+/* line 191 "SemMap.puma" */
   {
-# line 179 "SemMap.puma"
+/* line 193 "SemMap.puma" */
    failure_protocol (MODULE, "EvalDistributionDecl", layout_decl);
   }
-   return mReplicatedDistribution (0, 0, 0);
+   return mMapReplicated ();
 
 }
 
@@ -264,31 +306,29 @@ tDefinitions EvalDistributionStmt
 {
   if (layout_stmt->Kind == kREDISTRIBUTE_STMT) {
   if (layout_stmt->REDISTRIBUTE_STMT.DISTRIBUTEE->Kind == kBTV_LIST) {
-# line 188 "SemMap.puma"
+/* line 202 "SemMap.puma" */
  {
   tIdent yyV1;
   tDefinitions yyV2;
   {
-# line 191 "SemMap.puma"
+/* line 205 "SemMap.puma" */
    GetObjectInfo (layout_stmt->REDISTRIBUTE_STMT.DISTRIBUTEE->BTV_LIST.Elem, & yyV1, & yyV2);
-# line 193 "SemMap.puma"
+/* line 207 "SemMap.puma" */
    ReplaceDefaultDistributions (yyV2, layout_stmt);
-# line 195 "SemMap.puma"
+/* line 209 "SemMap.puma" */
    ControlRedistributeStmt (layout_stmt);
   }
-  {
    return EvalDistributionSpec (layout_stmt->REDISTRIBUTE_STMT.DISTRIBUTION, layout_stmt->REDISTRIBUTE_STMT.TARGET);
-  }
  }
 
   }
   }
-# line 200 "SemMap.puma"
+/* line 214 "SemMap.puma" */
   {
-# line 201 "SemMap.puma"
+/* line 216 "SemMap.puma" */
    failure_protocol (MODULE, "EvalDistributionStmt", layout_stmt);
   }
-   return mReplicatedDistribution (0, 0, 0);
+   return mMapReplicated ();
 
 }
 
@@ -303,18 +343,25 @@ static tDefinitions EvalRangeDecl
   if (layout_decl->Kind == kRANGE_DECL) {
   if (layout_decl->RANGE_DECL.RANGES->Kind == kRANGE_LIST) {
   if (layout_decl->RANGE_DECL.RANGES->RANGE_LIST.Next->Kind == kRANGE_EMPTY) {
-# line 217 "SemMap.puma"
-   return EvalRangeSpec (layout_decl->RANGE_DECL.Ident, layout_decl->RANGE_DECL.RANGES->RANGE_LIST.Elem);
+/* line 232 "SemMap.puma" */
+ {
+  tDefinitions selector;
+  {
+/* line 236 "SemMap.puma" */
+   selector = EvalRangeSpec (layout_decl->RANGE_DECL.Ident, layout_decl->RANGE_DECL.RANGES->RANGE_LIST.Elem);
+  }
+   return mRangeSelection (mRangeList (selector, mRangeEmpty ()));
+ }
 
   }
   }
   }
-# line 222 "SemMap.puma"
+/* line 241 "SemMap.puma" */
   {
-# line 223 "SemMap.puma"
+/* line 243 "SemMap.puma" */
    failure_protocol (MODULE, "EvalRangeDecl", layout_decl);
   }
-   return mReplicatedDistribution (0, 0, 0);
+   return mRangeDefault ();
 
 }
 
@@ -329,19 +376,19 @@ static void GetObjectInfo
 # endif
 {
   if (var->Kind == kVAR_OBJ) {
-# line 236 "SemMap.puma"
+/* line 256 "SemMap.puma" */
    * yyP2 = var->VAR_OBJ.Ident;
    * yyP1 = var->VAR_OBJ.Object;
    return;
 
   }
   if (var->Kind == kUSED_VAR) {
-# line 239 "SemMap.puma"
+/* line 259 "SemMap.puma" */
  {
   tIdent yyV1;
   tDefinitions yyV2;
   {
-# line 241 "SemMap.puma"
+/* line 261 "SemMap.puma" */
    GetObjectInfo (var->USED_VAR.VARNAME, & yyV1, & yyV2);
   }
    * yyP2 = yyV1;
@@ -351,15 +398,15 @@ static void GetObjectInfo
 
   }
   if (var->Kind == kSELECTED_VAR) {
-# line 244 "SemMap.puma"
+/* line 264 "SemMap.puma" */
    * yyP2 = var->SELECTED_VAR.SELECTOR->REC_COMP.Ident;
    * yyP1 = var->SELECTED_VAR.SELECTOR->REC_COMP.Object;
    return;
 
   }
-# line 247 "SemMap.puma"
+/* line 267 "SemMap.puma" */
   {
-# line 249 "SemMap.puma"
+/* line 269 "SemMap.puma" */
    failure_protocol (MODULE, "GetObjectInfo", var);
   }
    * yyP2 = DefaultId ();
@@ -379,46 +426,46 @@ static tDefinitions EvalDistributionSpec
 # endif
 {
   if (d_spec->Kind == kANY_DISTRIBUTION) {
-# line 260 "SemMap.puma"
-   return mReplicatedDistribution (0, 0, 0);
+/* line 280 "SemMap.puma" */
+   return mMapReplicated ();
 
   }
   if (d_spec->Kind == kREPL_DISTRIBUTION) {
-# line 265 "SemMap.puma"
-   return mReplicatedDistribution (0, 0, 0);
+/* line 285 "SemMap.puma" */
+   return mMapReplicated ();
 
   }
   if (d_spec->Kind == kNODE_DISTRIBUTION) {
-# line 270 "SemMap.puma"
+/* line 290 "SemMap.puma" */
   {
-# line 272 "SemMap.puma"
+/* line 292 "SemMap.puma" */
    if (! ((IsReplicatedDistribution (d_spec->NODE_DISTRIBUTION.MAPPING)))) goto yyL3;
   }
-   return mReplicatedDistribution (0, 0, 0);
+   return mMapReplicated ();
 yyL3:;
 
   if (p_spec->Kind == kPROCESSOR_ARRAY) {
-# line 276 "SemMap.puma"
+/* line 297 "SemMap.puma" */
    return ComputeNodeDistribution (d_spec->NODE_DISTRIBUTION.MAPPING, p_spec->PROCESSOR_ARRAY.TOPNAME->TOP_OBJ.Object);
 
   }
   if (p_spec->Kind == kPROCESSOR_SUBSET) {
-# line 281 "SemMap.puma"
+/* line 302 "SemMap.puma" */
    return ComputeNodeDistribution (d_spec->NODE_DISTRIBUTION.MAPPING, p_spec->PROCESSOR_SUBSET.SUBTOP->TOP_OBJ.Object);
 
   }
   if (p_spec->Kind == kANY_PROCESSOR_ARRAY) {
-# line 286 "SemMap.puma"
+/* line 307 "SemMap.puma" */
    return ComputeNodeDistribution (d_spec->NODE_DISTRIBUTION.MAPPING, NoObject);
 
   }
   }
-# line 293 "SemMap.puma"
+/* line 314 "SemMap.puma" */
   {
-# line 295 "SemMap.puma"
+/* line 316 "SemMap.puma" */
    failure_protocol (MODULE, "EvalDistributionSpec", d_spec);
   }
-   return mReplicatedDistribution (0, 0, 0);
+   return mMapReplicated ();
 
 }
 
@@ -432,28 +479,28 @@ static tDefinitions EvalRangeSpec
 # endif
 {
   if (d_spec->Kind == kNODE_DISTRIBUTION) {
-# line 307 "SemMap.puma"
+/* line 329 "SemMap.puma" */
   {
-# line 309 "SemMap.puma"
+/* line 331 "SemMap.puma" */
    if (! ((IsReplicatedDistribution (d_spec->NODE_DISTRIBUTION.MAPPING)))) goto yyL1;
   }
-   return mReplicatedDistribution (0, 0, 0);
+   return mRangeReplicated ();
 yyL1:;
 
-# line 313 "SemMap.puma"
+/* line 336 "SemMap.puma" */
    return ComputeRangeDistribution (d_spec->NODE_DISTRIBUTION.MAPPING);
 
   }
-# line 318 "SemMap.puma"
+/* line 341 "SemMap.puma" */
   {
-# line 319 "SemMap.puma"
+/* line 343 "SemMap.puma" */
    failure_protocol (MODULE, "EvalRangeSpec", d_spec);
   }
-   return mReplicatedDistribution (0, 0, 0);
+   return mRangeReplicated ();
 
 }
 
-static bool IsReplicatedDistribution
+static rbool IsReplicatedDistribution
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -461,24 +508,24 @@ static bool IsReplicatedDistribution
  register tTree t;
 # endif
 {
+ yyRecursion:
   if (t->Kind == kDIST_EMPTY) {
-# line 334 "SemMap.puma"
-   return true;
+/* line 359 "SemMap.puma" */
+   return rtrue;
 
   }
   if (t->Kind == kDIST_LIST) {
   if (t->DIST_LIST.Elem->Kind == kSERIAL_DISTRIBUTION) {
-# line 337 "SemMap.puma"
+/* line 362 "SemMap.puma" */
   {
-# line 338 "SemMap.puma"
-   if (! (IsReplicatedDistribution (t->DIST_LIST.Next))) goto yyL2;
+/* line 364 "SemMap.puma" */
+   t = t->DIST_LIST.Next;
+   goto yyRecursion;
   }
-   return true;
-yyL2:;
 
   }
   }
-  return false;
+  return rfalse;
 }
 
 static tDefinitions ComputeNodeDistribution
@@ -490,18 +537,14 @@ static tDefinitions ComputeNodeDistribution
  register tDefinitions p_obj;
 # endif
 {
-# line 357 "SemMap.puma"
+/* line 384 "SemMap.puma" */
  {
   tDefinitions array_list;
   {
-# line 359 "SemMap.puma"
-
-# line 361 "SemMap.puma"
-   array_list = MakeDistributionList (mapping, 1, 1);
-  }
-  {
-   return mNodeDistribution (0, 0, 0, p_obj, array_list);
+/* line 388 "SemMap.puma" */
+   array_list = MakeDistDimList (mapping, 1, 1);
   }
+   return mMapDist (mDistribution (p_obj, array_list));
  }
 
 }
@@ -514,23 +557,19 @@ static tDefinitions ComputeRangeDistribution
  register tTree mapping;
 # endif
 {
-# line 368 "SemMap.puma"
+/* line 395 "SemMap.puma" */
  {
   tDefinitions array_list;
   {
-# line 370 "SemMap.puma"
-
-# line 372 "SemMap.puma"
-   array_list = MakeDistributionList (mapping, 1, 1);
-  }
-  {
-   return mRangeDistribution (0, 0, 0, array_list);
+/* line 399 "SemMap.puma" */
+   array_list = MakeDistDimList (mapping, 1, 1);
   }
+   return mRangeDist (array_list);
  }
 
 }
 
-static tDefinitions MakeDistributionList
+static tDefinitions MakeDistDimList
 # if defined __STDC__ | defined __cplusplus
 (register tTree mapping, register int top_dim, register int temp_dim)
 # else
@@ -541,36 +580,34 @@ static tDefinitions MakeDistributionList
 # endif
 {
   if (mapping->Kind == kDIST_EMPTY) {
-# line 395 "SemMap.puma"
+/* line 422 "SemMap.puma" */
    return mDIM_EMPTY ();
 
   }
   if (mapping->Kind == kDIST_LIST) {
   if (mapping->DIST_LIST.Elem->Kind == kSERIAL_DISTRIBUTION) {
-# line 400 "SemMap.puma"
-   return mDIM_LIST (mSerialDimension (temp_dim), MakeDistributionList (mapping->DIST_LIST.Next, top_dim, temp_dim + 1));
+/* line 427 "SemMap.puma" */
+   return mDIM_LIST (mSerialDimension (temp_dim), MakeDistDimList (mapping->DIST_LIST.Next, top_dim, temp_dim + 1));
 
   }
-# line 408 "SemMap.puma"
+/* line 435 "SemMap.puma" */
  {
   int yyV1;
   tTree yyV2;
   tTree yyV3;
   tTree yyV4;
   {
-# line 412 "SemMap.puma"
+/* line 439 "SemMap.puma" */
    CodeDistribution (mapping->DIST_LIST.Elem, & yyV1, & yyV2, & yyV3, & yyV4);
   }
-  {
-   return mDIM_LIST (mDistributedDimension (temp_dim, top_dim, yyV1, yyV2, yyV3, yyV4), MakeDistributionList (mapping->DIST_LIST.Next, top_dim + 1, temp_dim + 1));
-  }
+   return mDIM_LIST (mDistributedDimension (temp_dim, top_dim, yyV1, yyV2, yyV3, yyV4), MakeDistDimList (mapping->DIST_LIST.Next, top_dim + 1, temp_dim + 1));
  }
 
   }
-# line 419 "SemMap.puma"
+/* line 446 "SemMap.puma" */
   {
-# line 421 "SemMap.puma"
-   failure_protocol (MODULE, "MakeDistributionList", mapping);
+/* line 448 "SemMap.puma" */
+   failure_protocol (MODULE, "MakeDistDimList", mapping);
   }
    return mDIM_EMPTY ();
 
@@ -592,7 +629,7 @@ static void CodeDistribution
   switch (distribution->Kind) {
   case kBLOCK_DISTRIBUTION:
   if (distribution->BLOCK_DISTRIBUTION.SIZE->Kind == kDUMMY_EXP) {
-# line 435 "SemMap.puma"
+/* line 462 "SemMap.puma" */
    * yyP6 = kBLOCK_DIM;
    * yyP5 = NoTree;
    * yyP4 = NoTree;
@@ -600,9 +637,9 @@ static void CodeDistribution
    return;
 
   }
-# line 438 "SemMap.puma"
+/* line 465 "SemMap.puma" */
   {
-# line 440 "SemMap.puma"
+/* line 467 "SemMap.puma" */
  if (TreeRank (distribution->BLOCK_DISTRIBUTION.SIZE) != 0)
        { error_protocol ("size of BLOCK must be scalar");
          tree_protocol ("illegal size expression is : ", distribution->BLOCK_DISTRIBUTION.SIZE);
@@ -617,7 +654,7 @@ static void CodeDistribution
 
   case kCYCLIC_DISTRIBUTION:
   if (distribution->CYCLIC_DISTRIBUTION.SIZE->Kind == kDUMMY_EXP) {
-# line 447 "SemMap.puma"
+/* line 474 "SemMap.puma" */
    * yyP6 = kCYCLIC_DIM;
    * yyP5 = NoTree;
    * yyP4 = NoTree;
@@ -625,9 +662,9 @@ static void CodeDistribution
    return;
 
   }
-# line 450 "SemMap.puma"
+/* line 477 "SemMap.puma" */
   {
-# line 452 "SemMap.puma"
+/* line 479 "SemMap.puma" */
  if (TreeRank (distribution->CYCLIC_DISTRIBUTION.SIZE) != 0)
        { error_protocol ("size of CYCLIC must be scalar");
          tree_protocol ("illegal size expression is : ", distribution->CYCLIC_DISTRIBUTION.SIZE);
@@ -641,9 +678,9 @@ static void CodeDistribution
    return;
 
   case kGEN_BLOCK_DISTRIBUTION:
-# line 459 "SemMap.puma"
+/* line 486 "SemMap.puma" */
   {
-# line 462 "SemMap.puma"
+/* line 489 "SemMap.puma" */
  if (!IsLegalIntArray (distribution->GEN_BLOCK_DISTRIBUTION.ARRAY))
 
        { error_protocol ("size array in GEN_BLOCK must have rank 1");
@@ -658,9 +695,9 @@ static void CodeDistribution
    return;
 
   case kINDIRECT_DISTRIBUTION:
-# line 470 "SemMap.puma"
+/* line 497 "SemMap.puma" */
   {
-# line 473 "SemMap.puma"
+/* line 500 "SemMap.puma" */
  if (!IsLegalIntArray (distribution->INDIRECT_DISTRIBUTION.ARRAY))
 
        { error_protocol ("map array in INDIRECT must have rank 1");
@@ -675,9 +712,9 @@ static void CodeDistribution
    return;
 
   case kARBITRARY_DISTRIBUTION:
-# line 481 "SemMap.puma"
+/* line 508 "SemMap.puma" */
   {
-# line 484 "SemMap.puma"
+/* line 511 "SemMap.puma" */
  if (TreeRank (distribution->ARBITRARY_DISTRIBUTION.SIZE) != 0)
        { error_protocol ("size of ARBITRARY must be scalar");
          tree_protocol ("illegal size expression is : ", distribution->ARBITRARY_DISTRIBUTION.SIZE);
@@ -703,7 +740,7 @@ static void CodeDistribution
    return;
 
   case kANY_BLOCK_DISTRIBUTION:
-# line 503 "SemMap.puma"
+/* line 530 "SemMap.puma" */
    * yyP6 = kANY_BLOCK_DIM;
    * yyP5 = NoTree;
    * yyP4 = NoTree;
@@ -711,7 +748,7 @@ static void CodeDistribution
    return;
 
   case kANY_CYCLIC_DISTRIBUTION:
-# line 506 "SemMap.puma"
+/* line 533 "SemMap.puma" */
    * yyP6 = kANY_CYCLIC_DIM;
    * yyP5 = NoTree;
    * yyP4 = NoTree;
@@ -719,7 +756,7 @@ static void CodeDistribution
    return;
 
   case kANY_GEN_BLOCK_DISTRIBUTION:
-# line 509 "SemMap.puma"
+/* line 536 "SemMap.puma" */
    * yyP6 = kANY_GEN_BLOCK_DIM;
    * yyP5 = NoTree;
    * yyP4 = NoTree;
@@ -727,7 +764,7 @@ static void CodeDistribution
    return;
 
   case kANY_INDIRECT_DISTRIBUTION:
-# line 512 "SemMap.puma"
+/* line 539 "SemMap.puma" */
    * yyP6 = kANY_INDIRECT_DIM;
    * yyP5 = NoTree;
    * yyP4 = NoTree;
@@ -735,7 +772,7 @@ static void CodeDistribution
    return;
 
   case kANY_ARBITRARY_DISTRIBUTION:
-# line 515 "SemMap.puma"
+/* line 542 "SemMap.puma" */
    * yyP6 = kANY_ARBITRARY_DIM;
    * yyP5 = NoTree;
    * yyP4 = NoTree;
@@ -743,7 +780,7 @@ static void CodeDistribution
    return;
 
   case kANY_DIM_DISTRIBUTION:
-# line 518 "SemMap.puma"
+/* line 545 "SemMap.puma" */
    * yyP6 = kANY_DISTRIBUTED_DIM;
    * yyP5 = NoTree;
    * yyP4 = NoTree;
@@ -752,11 +789,11 @@ static void CodeDistribution
 
   }
 
-# line 521 "SemMap.puma"
+/* line 548 "SemMap.puma" */
   {
-# line 523 "SemMap.puma"
+/* line 550 "SemMap.puma" */
    error_protocol ("illegal distribution format");
-# line 524 "SemMap.puma"
+/* line 551 "SemMap.puma" */
    tree_protocol ("illegal distribution is ", distribution);
   }
    * yyP6 = kANY_DISTRIBUTED_DIM;
@@ -768,7 +805,7 @@ static void CodeDistribution
 ;
 }
 
-static bool IsLegalIntArray
+static rbool IsLegalIntArray
 # if defined __STDC__ | defined __cplusplus
 (register tTree int_array)
 # else
@@ -777,16 +814,16 @@ static bool IsLegalIntArray
 # endif
 {
   if (int_array->Kind == kUSED_VAR) {
-# line 535 "SemMap.puma"
+/* line 562 "SemMap.puma" */
   {
-# line 537 "SemMap.puma"
+/* line 564 "SemMap.puma" */
    if (! ((VarRank (int_array->USED_VAR.VARNAME->VAR_OBJ.Object) == 1))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
-  return false;
+  return rfalse;
 }
 
 static int NoDistributedDims
@@ -797,22 +834,25 @@ static int NoDistributedDims
  register tDefinitions dlist;
 # endif
 {
+ yyRecursion:
   if (dlist->Kind == kDIM_EMPTY) {
-# line 548 "SemMap.puma"
+/* line 575 "SemMap.puma" */
    return 0;
 
   }
   if (dlist->Kind == kDIM_LIST) {
   if (dlist->DIM_LIST.Elem->Kind == kDistributedDimension) {
-# line 553 "SemMap.puma"
+/* line 580 "SemMap.puma" */
    return NoDistributedDims (dlist->DIM_LIST.Next) + 1;
 
   }
-# line 558 "SemMap.puma"
-   return NoDistributedDims (dlist->DIM_LIST.Next);
+/* line 585 "SemMap.puma" */
+   dlist = dlist->DIM_LIST.Next;
+   goto yyRecursion;
 
   }
  yyAbort ("NoDistributedDims");
+ { int yyDummy; return yyDummy; }
 }
 
 static tDefinitions EvalAlignmentDecl
@@ -824,16 +864,16 @@ static tDefinitions EvalAlignmentDecl
 # endif
 {
   if (align_decl->Kind == kALIGN_DECL) {
-# line 574 "SemMap.puma"
-   return ComputeAlignDistribution (align_decl->ALIGN_DECL.ALIGN_SOURCE, align_decl->ALIGN_DECL.ALIGN_TARGET->ALIGN_SPEC.SUBSCRIPTS, align_decl->ALIGN_DECL.ALIGN_TARGET->ALIGN_SPEC.TEMPNAME->TEMP_OBJ.Object);
+/* line 601 "SemMap.puma" */
+   return mMapAlign (ComputeAlignment (align_decl->ALIGN_DECL.ALIGN_SOURCE, align_decl->ALIGN_DECL.ALIGN_TARGET->ALIGN_SPEC.SUBSCRIPTS, align_decl->ALIGN_DECL.ALIGN_TARGET->ALIGN_SPEC.TEMPNAME->TEMP_OBJ.Object));
 
   }
-# line 580 "SemMap.puma"
+/* line 607 "SemMap.puma" */
   {
-# line 582 "SemMap.puma"
+/* line 609 "SemMap.puma" */
    failure_protocol ("Distributions", "EvalAlignmentDecl", align_decl);
   }
-   return NoObject;
+   return mMapReplicated ();
 
 }
 
@@ -846,76 +886,20 @@ tDefinitions EvalAlignmentStmt
 # endif
 {
   if (align_stmt->Kind == kREALIGN_STMT) {
-# line 597 "SemMap.puma"
-   return ComputeAlignDistribution (align_stmt->REALIGN_STMT.ALIGN_SOURCE, align_stmt->REALIGN_STMT.ALIGN_TARGET->ALIGN_SPEC.SUBSCRIPTS, align_stmt->REALIGN_STMT.ALIGN_TARGET->ALIGN_SPEC.TEMPNAME->TEMP_OBJ.Object);
+/* line 625 "SemMap.puma" */
+   return mMapAlign (ComputeAlignment (align_stmt->REALIGN_STMT.ALIGN_SOURCE, align_stmt->REALIGN_STMT.ALIGN_TARGET->ALIGN_SPEC.SUBSCRIPTS, align_stmt->REALIGN_STMT.ALIGN_TARGET->ALIGN_SPEC.TEMPNAME->TEMP_OBJ.Object));
 
   }
-# line 603 "SemMap.puma"
+/* line 632 "SemMap.puma" */
   {
-# line 605 "SemMap.puma"
+/* line 634 "SemMap.puma" */
    failure_protocol (MODULE, "EvalAlignmentStmt", align_stmt);
   }
    return NoObject;
 
 }
 
-static tDefinitions GetObjProcDistribution
-# if defined __STDC__ | defined __cplusplus
-(register tDefinitions obj)
-# else
-(obj)
- register tDefinitions obj;
-# endif
-{
-  if (obj->Kind == kTemplateObject) {
-  if (obj->TemplateObject.Dist->Kind == kDeclDistribution) {
-# line 617 "SemMap.puma"
-  {
-# line 619 "SemMap.puma"
-   EvalObjDistribution (obj);
-  }
-   return obj->TemplateObject.Dist;
-
-  }
-# line 636 "SemMap.puma"
-   return obj->TemplateObject.Dist;
-
-  }
-  if (obj->Kind == kVarObject) {
-  if (obj->VarObject.Dist->Kind == kDeclDistribution) {
-  if (obj->VarObject.Dist->DeclDistribution.decl->Kind == kALIGN_DECL) {
-# line 623 "SemMap.puma"
-  {
-# line 625 "SemMap.puma"
-   obj_error_protocol ("Alignment to aligned object not possible (indirect alignment)", obj);
-  }
-   return mReplicatedDistribution (0, 0, 0);
-
-  }
-# line 630 "SemMap.puma"
-  {
-# line 632 "SemMap.puma"
-   EvalObjDistribution (obj);
-  }
-   return obj->VarObject.Dist;
-
-  }
-# line 640 "SemMap.puma"
-   return obj->VarObject.Dist;
-
-  }
-# line 644 "SemMap.puma"
-  {
-# line 645 "SemMap.puma"
-   obj_error_protocol ("GetObjProcDistribution illegal\n", obj);
-# line 646 "SemMap.puma"
-   kill_in_protocol ();
-  }
-   return NoObject;
-
-}
-
-static tDefinitions ComputeAlignDistribution
+static tDefinitions ComputeAlignment
 # if defined __STDC__ | defined __cplusplus
 (register tTree source, register tTree target, register tDefinitions temp_obj)
 # else
@@ -925,30 +909,22 @@ static tDefinitions ComputeAlignDistribution
  register tDefinitions temp_obj;
 # endif
 {
-# line 673 "SemMap.puma"
+/* line 657 "SemMap.puma" */
  {
   tDefinitions target_list;
   tDefinitions source_list;
   tDefinitions newtop_list;
   {
-# line 675 "SemMap.puma"
-
-# line 676 "SemMap.puma"
-
-# line 677 "SemMap.puma"
-
-# line 681 "SemMap.puma"
+/* line 665 "SemMap.puma" */
  if (IsAlignedObj (temp_obj))
          error_protocol ("aligned target in ALIGN not allowed");
     
-# line 685 "SemMap.puma"
+/* line 669 "SemMap.puma" */
    target_list = MakeAlignTargetDimList (1, source, target);
-# line 690 "SemMap.puma"
+/* line 674 "SemMap.puma" */
    source_list = MakeAlignSourceDimList (1, TreeListLength (source), target_list);
   }
-  {
-   return mAlignDistribution (0, 0, 0, temp_obj, target_list, source_list);
-  }
+   return mAlignment (temp_obj, target_list, source_list);
  }
 
 }
@@ -965,11 +941,11 @@ static tDefinitions MakeAlignTargetDimList
 {
   if (target->Kind == kBTE_LIST) {
   if (target->BTE_LIST.Elem->Kind == kDUMMY_EXP) {
-# line 723 "SemMap.puma"
+/* line 706 "SemMap.puma" */
    return mDIM_LIST (mReplicatedDimension (temp_dim), MakeAlignTargetDimList (temp_dim + 1, source, target->BTE_LIST.Next));
 
   }
-# line 729 "SemMap.puma"
+/* line 712 "SemMap.puma" */
  {
   tDefinitions dim;
   int index_dim;
@@ -977,28 +953,14 @@ static tDefinitions MakeAlignTargetDimList
   int mult_val;
   tTree add_tree;
   int add_val;
-  bool done;
+  rbool done;
   {
-# line 731 "SemMap.puma"
-
-# line 733 "SemMap.puma"
-
-# line 734 "SemMap.puma"
-
-# line 735 "SemMap.puma"
-
-# line 736 "SemMap.puma"
-
-# line 737 "SemMap.puma"
-
-# line 738 "SemMap.puma"
-
-# line 740 "SemMap.puma"
+/* line 723 "SemMap.puma" */
    AnalTargetExp (target->BTE_LIST.Elem, source, & index_dim, & done, & mult_tree, & mult_val, & add_tree, & add_val);
-# line 743 "SemMap.puma"
+/* line 726 "SemMap.puma" */
    if (! ((index_dim > 0))) goto yyL2;
   {
-# line 751 "SemMap.puma"
+/* line 734 "SemMap.puma" */
  if (mult_tree == NoTree)
         mult_tree = MakeConstant (mult_val);
       else
@@ -1010,24 +972,22 @@ static tDefinitions MakeAlignTargetDimList
    
   }
   }
-  {
    return mDIM_LIST (mAlignedDimension (index_dim, temp_dim, mult_tree, add_tree), MakeAlignTargetDimList (temp_dim + 1, source, target->BTE_LIST.Next));
-  }
  }
 yyL2:;
 
-# line 766 "SemMap.puma"
+/* line 749 "SemMap.puma" */
    return mDIM_LIST (mEmbeddedDimension (temp_dim, target->BTE_LIST.Elem), MakeAlignTargetDimList (temp_dim + 1, source, target->BTE_LIST.Next));
 
   }
   if (target->Kind == kBTE_EMPTY) {
-# line 772 "SemMap.puma"
+/* line 755 "SemMap.puma" */
    return mDIM_EMPTY ();
 
   }
-# line 777 "SemMap.puma"
+/* line 760 "SemMap.puma" */
   {
-# line 779 "SemMap.puma"
+/* line 762 "SemMap.puma" */
    failure_protocol (MODULE, "MakeAlignTargetDimList", target);
   }
    return mDIM_EMPTY ();
@@ -1044,24 +1004,22 @@ static tDefinitions MakeAlignSourceDimList
  register tDefinitions target_list;
 # endif
 {
-# line 803 "SemMap.puma"
+/* line 786 "SemMap.puma" */
   {
-# line 805 "SemMap.puma"
+/* line 788 "SemMap.puma" */
    if (! ((i > n))) goto yyL1;
   }
    return mDIM_EMPTY ();
 yyL1:;
 
-# line 809 "SemMap.puma"
+/* line 792 "SemMap.puma" */
  {
   tDefinitions yyV1;
   {
-# line 811 "SemMap.puma"
+/* line 794 "SemMap.puma" */
    FindEntry (target_list, i, & yyV1);
   }
-  {
    return mDIM_LIST (yyV1, MakeAlignSourceDimList (i + 1, n, target_list));
-  }
  }
 
 }
@@ -1077,13 +1035,11 @@ static void FindEntry
 # endif
 {
   if (target_list->Kind == kDIM_EMPTY) {
-# line 824 "SemMap.puma"
+/* line 807 "SemMap.puma" */
  {
   tDefinitions dim;
   {
-# line 826 "SemMap.puma"
-
-# line 828 "SemMap.puma"
+/* line 811 "SemMap.puma" */
    dim = mSerialDimension (index);
   }
    * yyP7 = dim;
@@ -1093,9 +1049,9 @@ static void FindEntry
   }
   if (target_list->Kind == kDIM_LIST) {
   if (target_list->DIM_LIST.Elem->Kind == kAlignedDimension) {
-# line 831 "SemMap.puma"
+/* line 814 "SemMap.puma" */
   {
-# line 833 "SemMap.puma"
+/* line 816 "SemMap.puma" */
    if (! ((target_list->DIM_LIST.Elem->AlignedDimension.dimension == index))) goto yyL2;
   }
    * yyP7 = target_list->DIM_LIST.Elem;
@@ -1103,11 +1059,11 @@ static void FindEntry
 yyL2:;
 
   }
-# line 836 "SemMap.puma"
+/* line 819 "SemMap.puma" */
  {
   tDefinitions yyV1;
   {
-# line 838 "SemMap.puma"
+/* line 821 "SemMap.puma" */
    FindEntry (target_list->DIM_LIST.Next, index, & yyV1);
   }
    * yyP7 = yyV1;
@@ -1118,59 +1074,10 @@ yyL2:;
 ;
 }
 
-void MakeObjReplicated
-# if defined __STDC__ | defined __cplusplus
-(register tDefinitions t)
-# else
-(t)
- register tDefinitions t;
-# endif
-{
-  if (t->Kind == kVarObject) {
-# line 851 "SemMap.puma"
-  {
-# line 853 "SemMap.puma"
- t->VarObject.Dist = MakeReplicatedDistribution (t->VarObject.Dist); 
-  }
-   return;
-
-  }
-  if (t->Kind == kTemplateObject) {
-# line 856 "SemMap.puma"
-  {
-# line 858 "SemMap.puma"
- t->TemplateObject.Dist = MakeReplicatedDistribution (t->TemplateObject.Dist); 
-  }
-   return;
-
-  }
-# line 861 "SemMap.puma"
-  {
-# line 862 "SemMap.puma"
-   failure_protocol (MODULE, "MakeObjReplicated", t->Object.decl);
-  }
-   return;
-
-;
-}
-
-static tDefinitions MakeReplicatedDistribution
-# if defined __STDC__ | defined __cplusplus
-(register tDefinitions d)
-# else
-(d)
- register tDefinitions d;
-# endif
-{
-# line 873 "SemMap.puma"
-   return mReplicatedDistribution (d->Distribution.dynamic, d->Distribution.shared, d->Distribution.inherited);
-
-}
-
-void BeginSemMap ()
+void BeginSemMap ARGS ((void))
 {
 }
 
-void CloseSemMap ()
+void CloseSemMap ARGS ((void))
 {
 }
