@@ -1,6 +1,13 @@
 # include "SemPreds.h"
-# include "yySemPreds.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,33 +15,30 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 58 "SemPreds.puma"
+/* line 58 "SemPreds.puma" */
 
 
 # define MODULE "SemPreds"
 
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
 # include "protocol.h"
 
 # include "Types.h"
@@ -47,11 +51,25 @@ char type_error_msg[100];
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* SemPreds_Exit) () = yyExit;
+# include "yySemPreds.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* SemPreds_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -60,43 +78,44 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module SemPreds, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module SemPreds, routine %s failed\n",
+  yyFunction);
  SemPreds_Exit ();
 }
 
-bool ScalarExpr ARGS((tTree exp));
-static bool DummyExpr ARGS((tTree exp));
-bool IntExpr ARGS((tTree exp));
-bool LogicalExpr ARGS((tTree exp));
-bool NormalRealExpr ARGS((tTree exp));
-bool DoubleRealExpr ARGS((tTree exp));
-bool ComplexExpr ARGS((tTree exp));
-bool StringExpr ARGS((tTree exp));
-bool RealExpr ARGS((tTree exp));
-bool NumExpr ARGS((tTree exp));
-bool CaseExpr ARGS((tTree exp));
-bool SortExpr ARGS((tTree exp));
-bool ScalarLogicalExpr ARGS((tTree exp));
-bool ScalarIntExpr ARGS((tTree exp));
-bool ScalarNumExpr ARGS((tTree exp));
-bool ScalarCaseExpr ARGS((tTree exp));
-bool ScalarVariable ARGS((tTree var));
-bool DoVariable ARGS((tTree var));
-bool ScalarIntVariable ARGS((tTree var));
-bool PointerObject ARGS((tTree var));
-bool AllocateObject ARGS((tTree var));
-bool InitializationExpr ARGS((tTree exp));
-bool Expr ARGS((tTree exp));
-bool InitIntExpr ARGS((tTree exp));
-bool ArrayExpr ARGS((tTree exp));
-bool DummyHPFArray ARGS((tTree exp));
-bool IsLegalOutVar ARGS((tTree var));
-bool IsDefinableParam ARGS((tTree t));
-static bool IsParameterVariable ARGS((tTree var));
-static bool IsVectorVariable ARGS((tTree var));
-static bool HasVectorIndex ARGS((tTree exp_list));
+rbool ScalarExpr ARGS ((tTree exp));
+static rbool DummyExpr ARGS ((tTree exp));
+rbool IntExpr ARGS ((tTree exp));
+rbool LogicalExpr ARGS ((tTree exp));
+rbool NormalRealExpr ARGS ((tTree exp));
+rbool DoubleRealExpr ARGS ((tTree exp));
+rbool ComplexExpr ARGS ((tTree exp));
+rbool StringExpr ARGS ((tTree exp));
+rbool RealExpr ARGS ((tTree exp));
+rbool NumExpr ARGS ((tTree exp));
+rbool CaseExpr ARGS ((tTree exp));
+rbool SortExpr ARGS ((tTree exp));
+rbool ScalarLogicalExpr ARGS ((tTree exp));
+rbool ScalarIntExpr ARGS ((tTree exp));
+rbool ScalarNumExpr ARGS ((tTree exp));
+rbool ScalarCaseExpr ARGS ((tTree exp));
+rbool ScalarVariable ARGS ((tTree var));
+rbool DoVariable ARGS ((tTree var));
+rbool ScalarIntVariable ARGS ((tTree var));
+rbool PointerObject ARGS ((tTree var));
+rbool AllocateObject ARGS ((tTree var));
+rbool InitializationExpr ARGS ((tTree exp));
+rbool Expr ARGS ((tTree exp));
+rbool InitIntExpr ARGS ((tTree exp));
+rbool ArrayExpr ARGS ((tTree exp));
+rbool DummyHPFArray ARGS ((tTree exp));
+rbool IsLegalOutVar ARGS ((tTree var));
+rbool IsDefinableParam ARGS ((tTree t));
+static rbool IsParameterVariable ARGS ((tTree var));
+static rbool IsVectorVariable ARGS ((tTree var));
+static rbool HasVectorIndex ARGS ((tTree exp_list));
 
-bool ScalarExpr
+rbool ScalarExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -104,18 +123,18 @@ bool ScalarExpr
  register tTree exp;
 # endif
 {
-# line 84 "SemPreds.puma"
+/* line 84 "SemPreds.puma" */
   {
-# line 85 "SemPreds.puma"
+/* line 85 "SemPreds.puma" */
    if (! ((TreeRank (exp) == 0))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool DummyExpr
+static rbool DummyExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -123,25 +142,23 @@ static bool DummyExpr
  register tTree exp;
 # endif
 {
-# line 104 "SemPreds.puma"
+/* line 104 "SemPreds.puma" */
  {
   type_rec t;
   {
-# line 106 "SemPreds.puma"
-
-# line 108 "SemPreds.puma"
+/* line 108 "SemPreds.puma" */
    GetExpType (exp, & t);
-# line 109 "SemPreds.puma"
+/* line 109 "SemPreds.puma" */
    if (! ((t . type_kind == kDUMMY_TYPE))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-bool IntExpr
+rbool IntExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -149,32 +166,30 @@ bool IntExpr
  register tTree exp;
 # endif
 {
-# line 118 "SemPreds.puma"
+/* line 118 "SemPreds.puma" */
  {
   type_rec t;
   {
-# line 120 "SemPreds.puma"
-
-# line 122 "SemPreds.puma"
+/* line 122 "SemPreds.puma" */
    GetExpType (exp, & t);
-# line 123 "SemPreds.puma"
+/* line 123 "SemPreds.puma" */
    if (! (((t . type_kind == kINTEGER_TYPE) || (t . type_kind == kDUMMY_TYPE)))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-# line 126 "SemPreds.puma"
+/* line 126 "SemPreds.puma" */
   {
-# line 127 "SemPreds.puma"
+/* line 127 "SemPreds.puma" */
    sprintf (type_error_msg, "is not integer");
-# line 128 "SemPreds.puma"
-   return false;
+/* line 128 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool LogicalExpr
+rbool LogicalExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -182,32 +197,30 @@ bool LogicalExpr
  register tTree exp;
 # endif
 {
-# line 137 "SemPreds.puma"
+/* line 137 "SemPreds.puma" */
  {
   type_rec t;
   {
-# line 139 "SemPreds.puma"
-
-# line 141 "SemPreds.puma"
+/* line 141 "SemPreds.puma" */
    GetExpType (exp, & t);
-# line 142 "SemPreds.puma"
+/* line 142 "SemPreds.puma" */
    if (! (((t . type_kind == kDUMMY_TYPE) || (t . type_kind == kBOOLEAN_TYPE)))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-# line 145 "SemPreds.puma"
+/* line 145 "SemPreds.puma" */
   {
-# line 146 "SemPreds.puma"
+/* line 146 "SemPreds.puma" */
    sprintf (type_error_msg, "is not logical");
-# line 147 "SemPreds.puma"
-   return false;
+/* line 147 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool NormalRealExpr
+rbool NormalRealExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -215,44 +228,42 @@ bool NormalRealExpr
  register tTree exp;
 # endif
 {
-# line 156 "SemPreds.puma"
+/* line 156 "SemPreds.puma" */
   {
-# line 157 "SemPreds.puma"
+/* line 157 "SemPreds.puma" */
    if (! ((DummyExpr (exp)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 160 "SemPreds.puma"
+/* line 160 "SemPreds.puma" */
  {
   type_rec t;
   {
-# line 162 "SemPreds.puma"
-
-# line 164 "SemPreds.puma"
+/* line 164 "SemPreds.puma" */
    GetExpType (exp, & t);
-# line 165 "SemPreds.puma"
+/* line 165 "SemPreds.puma" */
    if (! ((t . type_kind == kREAL_TYPE))) goto yyL2;
   {
-# line 166 "SemPreds.puma"
+/* line 166 "SemPreds.puma" */
    if (! ((t . type_size == default_real_size))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
  }
 yyL2:;
 
-# line 169 "SemPreds.puma"
+/* line 169 "SemPreds.puma" */
   {
-# line 170 "SemPreds.puma"
+/* line 170 "SemPreds.puma" */
    sprintf (type_error_msg, "is not default real");
-# line 171 "SemPreds.puma"
-   return false;
+/* line 171 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool DoubleRealExpr
+rbool DoubleRealExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -260,44 +271,42 @@ bool DoubleRealExpr
  register tTree exp;
 # endif
 {
-# line 180 "SemPreds.puma"
+/* line 180 "SemPreds.puma" */
   {
-# line 181 "SemPreds.puma"
+/* line 181 "SemPreds.puma" */
    if (! ((DummyExpr (exp)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 184 "SemPreds.puma"
+/* line 184 "SemPreds.puma" */
  {
   type_rec t;
   {
-# line 186 "SemPreds.puma"
-
-# line 188 "SemPreds.puma"
+/* line 188 "SemPreds.puma" */
    GetExpType (exp, & t);
-# line 189 "SemPreds.puma"
+/* line 189 "SemPreds.puma" */
    if (! ((t . type_kind == kREAL_TYPE))) goto yyL2;
   {
-# line 190 "SemPreds.puma"
+/* line 190 "SemPreds.puma" */
    if (! ((t . type_size == 2 * default_real_size))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
  }
 yyL2:;
 
-# line 193 "SemPreds.puma"
+/* line 193 "SemPreds.puma" */
   {
-# line 194 "SemPreds.puma"
+/* line 194 "SemPreds.puma" */
    sprintf (type_error_msg, "is not double real");
-# line 195 "SemPreds.puma"
-   return false;
+/* line 195 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool ComplexExpr
+rbool ComplexExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -305,40 +314,38 @@ bool ComplexExpr
  register tTree exp;
 # endif
 {
-# line 204 "SemPreds.puma"
+/* line 204 "SemPreds.puma" */
   {
-# line 205 "SemPreds.puma"
+/* line 205 "SemPreds.puma" */
    if (! ((DummyExpr (exp)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 208 "SemPreds.puma"
+/* line 208 "SemPreds.puma" */
  {
   type_rec t;
   {
-# line 210 "SemPreds.puma"
-
-# line 212 "SemPreds.puma"
+/* line 212 "SemPreds.puma" */
    GetExpType (exp, & t);
-# line 213 "SemPreds.puma"
+/* line 213 "SemPreds.puma" */
    if (! ((t . type_kind == kCOMPLEX_TYPE))) goto yyL2;
   }
-   return true;
+   return rtrue;
  }
 yyL2:;
 
-# line 216 "SemPreds.puma"
+/* line 216 "SemPreds.puma" */
   {
-# line 217 "SemPreds.puma"
+/* line 217 "SemPreds.puma" */
    sprintf (type_error_msg, "is not complex");
-# line 218 "SemPreds.puma"
-   return false;
+/* line 218 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool StringExpr
+rbool StringExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -346,40 +353,38 @@ bool StringExpr
  register tTree exp;
 # endif
 {
-# line 227 "SemPreds.puma"
+/* line 227 "SemPreds.puma" */
   {
-# line 228 "SemPreds.puma"
+/* line 228 "SemPreds.puma" */
    if (! ((DummyExpr (exp)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 231 "SemPreds.puma"
+/* line 231 "SemPreds.puma" */
  {
   type_rec t;
   {
-# line 233 "SemPreds.puma"
-
-# line 235 "SemPreds.puma"
+/* line 235 "SemPreds.puma" */
    GetExpType (exp, & t);
-# line 236 "SemPreds.puma"
+/* line 236 "SemPreds.puma" */
    if (! ((t . type_kind == kSTRING_TYPE))) goto yyL2;
   }
-   return true;
+   return rtrue;
  }
 yyL2:;
 
-# line 239 "SemPreds.puma"
+/* line 239 "SemPreds.puma" */
   {
-# line 240 "SemPreds.puma"
+/* line 240 "SemPreds.puma" */
    sprintf (type_error_msg, "is not string");
-# line 241 "SemPreds.puma"
-   return false;
+/* line 241 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool RealExpr
+rbool RealExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -387,32 +392,30 @@ bool RealExpr
  register tTree exp;
 # endif
 {
-# line 250 "SemPreds.puma"
+/* line 250 "SemPreds.puma" */
  {
   type_rec t;
   {
-# line 252 "SemPreds.puma"
-
-# line 254 "SemPreds.puma"
+/* line 254 "SemPreds.puma" */
    GetExpType (exp, & t);
-# line 255 "SemPreds.puma"
+/* line 255 "SemPreds.puma" */
    if (! (((t . type_kind == kDUMMY_TYPE) || (t . type_kind == kREAL_TYPE)))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-# line 258 "SemPreds.puma"
+/* line 258 "SemPreds.puma" */
   {
-# line 259 "SemPreds.puma"
+/* line 259 "SemPreds.puma" */
    sprintf (type_error_msg, "is not real");
-# line 260 "SemPreds.puma"
-   return false;
+/* line 260 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool NumExpr
+rbool NumExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -420,32 +423,30 @@ bool NumExpr
  register tTree exp;
 # endif
 {
-# line 269 "SemPreds.puma"
+/* line 269 "SemPreds.puma" */
  {
   type_rec t;
   {
-# line 271 "SemPreds.puma"
-
-# line 273 "SemPreds.puma"
+/* line 273 "SemPreds.puma" */
    GetExpType (exp, & t);
-# line 275 "SemPreds.puma"
+/* line 275 "SemPreds.puma" */
    if (! (((t . type_kind == kDUMMY_TYPE) || (t . type_kind == kINTEGER_TYPE) || (t . type_kind == kREAL_TYPE) || (t . type_kind == kCOMPLEX_TYPE)))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-# line 281 "SemPreds.puma"
+/* line 281 "SemPreds.puma" */
   {
-# line 282 "SemPreds.puma"
+/* line 282 "SemPreds.puma" */
    sprintf (type_error_msg, "is not numerical (int/real/complex)");
-# line 283 "SemPreds.puma"
-   return false;
+/* line 283 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool CaseExpr
+rbool CaseExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -453,32 +454,30 @@ bool CaseExpr
  register tTree exp;
 # endif
 {
-# line 292 "SemPreds.puma"
+/* line 292 "SemPreds.puma" */
  {
   type_rec t;
   {
-# line 294 "SemPreds.puma"
-
-# line 296 "SemPreds.puma"
+/* line 296 "SemPreds.puma" */
    GetExpType (exp, & t);
-# line 298 "SemPreds.puma"
+/* line 298 "SemPreds.puma" */
    if (! (((t . type_kind == kDUMMY_TYPE) || (t . type_kind == kINTEGER_TYPE) || (t . type_kind == kBOOLEAN_TYPE) || (t . type_kind == kSTRING_TYPE)))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-# line 305 "SemPreds.puma"
+/* line 305 "SemPreds.puma" */
   {
-# line 306 "SemPreds.puma"
+/* line 306 "SemPreds.puma" */
    sprintf (type_error_msg, "is not base type");
-# line 307 "SemPreds.puma"
-   return false;
+/* line 307 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool SortExpr
+rbool SortExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -486,32 +485,30 @@ bool SortExpr
  register tTree exp;
 # endif
 {
-# line 316 "SemPreds.puma"
+/* line 316 "SemPreds.puma" */
  {
   type_rec t;
   {
-# line 318 "SemPreds.puma"
-
-# line 320 "SemPreds.puma"
+/* line 320 "SemPreds.puma" */
    GetExpType (exp, & t);
-# line 322 "SemPreds.puma"
+/* line 322 "SemPreds.puma" */
    if (! (((t . type_kind == kDUMMY_TYPE) || (t . type_kind == kINTEGER_TYPE) || (t . type_kind == kREAL_TYPE) || (t . type_kind == kSTRING_TYPE)))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-# line 328 "SemPreds.puma"
+/* line 328 "SemPreds.puma" */
   {
-# line 329 "SemPreds.puma"
+/* line 329 "SemPreds.puma" */
    sprintf (type_error_msg, "is not sortable (int/real/character)");
-# line 330 "SemPreds.puma"
-   return false;
+/* line 330 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool ScalarLogicalExpr
+rbool ScalarLogicalExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -519,29 +516,29 @@ bool ScalarLogicalExpr
  register tTree exp;
 # endif
 {
-# line 350 "SemPreds.puma"
+/* line 350 "SemPreds.puma" */
   {
-# line 351 "SemPreds.puma"
+/* line 351 "SemPreds.puma" */
    if (! ((LogicalExpr (exp)))) goto yyL1;
   {
-# line 352 "SemPreds.puma"
+/* line 352 "SemPreds.puma" */
    if (! ((ScalarExpr (exp)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 355 "SemPreds.puma"
+/* line 355 "SemPreds.puma" */
   {
-# line 356 "SemPreds.puma"
+/* line 356 "SemPreds.puma" */
    sprintf (type_error_msg, "is not scalar logical");
-# line 357 "SemPreds.puma"
-   return false;
+/* line 357 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool ScalarIntExpr
+rbool ScalarIntExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -550,33 +547,33 @@ bool ScalarIntExpr
 # endif
 {
   if (exp->Kind == kDUMMY_EXP) {
-# line 366 "SemPreds.puma"
-   return true;
+/* line 366 "SemPreds.puma" */
+   return rtrue;
 
   }
-# line 369 "SemPreds.puma"
+/* line 369 "SemPreds.puma" */
   {
-# line 370 "SemPreds.puma"
+/* line 370 "SemPreds.puma" */
    if (! ((IntExpr (exp)))) goto yyL2;
   {
-# line 371 "SemPreds.puma"
+/* line 371 "SemPreds.puma" */
    if (! ((ScalarExpr (exp)))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 374 "SemPreds.puma"
+/* line 374 "SemPreds.puma" */
   {
-# line 375 "SemPreds.puma"
+/* line 375 "SemPreds.puma" */
    sprintf (type_error_msg, "is not scalar integer");
-# line 376 "SemPreds.puma"
-   return false;
+/* line 376 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool ScalarNumExpr
+rbool ScalarNumExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -584,29 +581,29 @@ bool ScalarNumExpr
  register tTree exp;
 # endif
 {
-# line 385 "SemPreds.puma"
+/* line 385 "SemPreds.puma" */
   {
-# line 386 "SemPreds.puma"
+/* line 386 "SemPreds.puma" */
    if (! (NumExpr (exp))) goto yyL1;
   {
-# line 387 "SemPreds.puma"
+/* line 387 "SemPreds.puma" */
    if (! (ScalarExpr (exp))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 390 "SemPreds.puma"
+/* line 390 "SemPreds.puma" */
   {
-# line 391 "SemPreds.puma"
+/* line 391 "SemPreds.puma" */
    sprintf (type_error_msg, "is not scalar numerical");
-# line 392 "SemPreds.puma"
-   return false;
+/* line 392 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool ScalarCaseExpr
+rbool ScalarCaseExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -614,29 +611,29 @@ bool ScalarCaseExpr
  register tTree exp;
 # endif
 {
-# line 401 "SemPreds.puma"
+/* line 401 "SemPreds.puma" */
   {
-# line 402 "SemPreds.puma"
+/* line 402 "SemPreds.puma" */
    if (! ((CaseExpr (exp)))) goto yyL1;
   {
-# line 403 "SemPreds.puma"
+/* line 403 "SemPreds.puma" */
    if (! ((ScalarExpr (exp)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 406 "SemPreds.puma"
+/* line 406 "SemPreds.puma" */
   {
-# line 407 "SemPreds.puma"
+/* line 407 "SemPreds.puma" */
    sprintf (type_error_msg, "is not scalar case expr");
-# line 408 "SemPreds.puma"
-   return false;
+/* line 408 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool ScalarVariable
+rbool ScalarVariable
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -644,25 +641,25 @@ bool ScalarVariable
  register tTree var;
 # endif
 {
-# line 421 "SemPreds.puma"
+/* line 421 "SemPreds.puma" */
   {
-# line 422 "SemPreds.puma"
+/* line 422 "SemPreds.puma" */
    if (! ((TreeRank (var) == 0))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 425 "SemPreds.puma"
+/* line 425 "SemPreds.puma" */
   {
-# line 426 "SemPreds.puma"
+/* line 426 "SemPreds.puma" */
    sprintf (type_error_msg, "is not scalar variable");
-# line 427 "SemPreds.puma"
-   return false;
+/* line 427 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool DoVariable
+rbool DoVariable
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -670,41 +667,41 @@ bool DoVariable
  register tTree var;
 # endif
 {
-# line 438 "SemPreds.puma"
+/* line 438 "SemPreds.puma" */
   {
-# line 439 "SemPreds.puma"
+/* line 439 "SemPreds.puma" */
    if (! ((ScalarVariable (var)))) goto yyL1;
   {
-# line 440 "SemPreds.puma"
+/* line 440 "SemPreds.puma" */
    if (! ((IntExpr (var)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 443 "SemPreds.puma"
+/* line 443 "SemPreds.puma" */
   {
-# line 444 "SemPreds.puma"
+/* line 444 "SemPreds.puma" */
    if (! ((ScalarVariable (var)))) goto yyL2;
   {
-# line 445 "SemPreds.puma"
+/* line 445 "SemPreds.puma" */
    if (! ((RealExpr (var)))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 448 "SemPreds.puma"
+/* line 448 "SemPreds.puma" */
   {
-# line 449 "SemPreds.puma"
+/* line 449 "SemPreds.puma" */
    sprintf (type_error_msg, "is illegal do variable");
-# line 450 "SemPreds.puma"
-   return false;
+/* line 450 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool ScalarIntVariable
+rbool ScalarIntVariable
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -712,29 +709,29 @@ bool ScalarIntVariable
  register tTree var;
 # endif
 {
-# line 461 "SemPreds.puma"
+/* line 461 "SemPreds.puma" */
   {
-# line 462 "SemPreds.puma"
+/* line 462 "SemPreds.puma" */
    if (! ((ScalarVariable (var)))) goto yyL1;
   {
-# line 463 "SemPreds.puma"
+/* line 463 "SemPreds.puma" */
    if (! ((IntExpr (var)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 466 "SemPreds.puma"
+/* line 466 "SemPreds.puma" */
   {
-# line 467 "SemPreds.puma"
+/* line 467 "SemPreds.puma" */
    sprintf (type_error_msg, "is not scalar integer variable");
-# line 468 "SemPreds.puma"
-   return false;
+/* line 468 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool PointerObject
+rbool PointerObject
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -743,46 +740,46 @@ bool PointerObject
 # endif
 {
   if (var->Kind == kVAR_PARAM) {
-# line 480 "SemPreds.puma"
+/* line 480 "SemPreds.puma" */
   {
-# line 482 "SemPreds.puma"
+/* line 482 "SemPreds.puma" */
    if (! ((PointerObject (var->VAR_PARAM.V)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   if (var->Kind == kUSED_VAR) {
-# line 485 "SemPreds.puma"
+/* line 485 "SemPreds.puma" */
   {
-# line 487 "SemPreds.puma"
-   if (! ((IsPointerType (GetObjectType (var->USED_VAR.VARNAME->VAR_OBJ.Object))))) goto yyL2;
+/* line 487 "SemPreds.puma" */
+   if (! ((IsPointerObject (var->USED_VAR.VARNAME->VAR_OBJ.Object)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   if (var->Kind == kSELECTED_VAR) {
-# line 490 "SemPreds.puma"
+/* line 490 "SemPreds.puma" */
   {
-# line 492 "SemPreds.puma"
-   if (! ((IsPointerType (GetObjectType (var->SELECTED_VAR.SELECTOR->REC_COMP.Object))))) goto yyL3;
+/* line 492 "SemPreds.puma" */
+   if (! ((IsPointerObject (var->SELECTED_VAR.SELECTOR->REC_COMP.Object)))) goto yyL3;
   }
-   return true;
+   return rtrue;
 yyL3:;
 
   }
-# line 495 "SemPreds.puma"
+/* line 495 "SemPreds.puma" */
   {
-# line 496 "SemPreds.puma"
+/* line 496 "SemPreds.puma" */
    sprintf (type_error_msg, "is not pointer object");
-# line 497 "SemPreds.puma"
-   return false;
+/* line 497 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool AllocateObject
+rbool AllocateObject
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -791,36 +788,36 @@ bool AllocateObject
 # endif
 {
   if (var->Kind == kVAR_PARAM) {
-# line 511 "SemPreds.puma"
+/* line 511 "SemPreds.puma" */
   {
-# line 513 "SemPreds.puma"
+/* line 513 "SemPreds.puma" */
    if (! ((AllocateObject (var->VAR_PARAM.V)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   if (var->Kind == kUSED_VAR) {
-# line 516 "SemPreds.puma"
-   return true;
+/* line 516 "SemPreds.puma" */
+   return rtrue;
 
   }
   if (var->Kind == kSELECTED_VAR) {
-# line 519 "SemPreds.puma"
-   return true;
+/* line 519 "SemPreds.puma" */
+   return rtrue;
 
   }
-# line 522 "SemPreds.puma"
+/* line 522 "SemPreds.puma" */
   {
-# line 523 "SemPreds.puma"
+/* line 523 "SemPreds.puma" */
    sprintf (type_error_msg, "is not allocatable object");
-# line 524 "SemPreds.puma"
-   return false;
+/* line 524 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool InitializationExpr
+rbool InitializationExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -828,12 +825,12 @@ bool InitializationExpr
  register tTree exp;
 # endif
 {
-# line 529 "SemPreds.puma"
-   return true;
+/* line 529 "SemPreds.puma" */
+   return rtrue;
 
 }
 
-bool Expr
+rbool Expr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -841,12 +838,12 @@ bool Expr
  register tTree exp;
 # endif
 {
-# line 539 "SemPreds.puma"
-   return true;
+/* line 539 "SemPreds.puma" */
+   return rtrue;
 
 }
 
-bool InitIntExpr
+rbool InitIntExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -854,29 +851,29 @@ bool InitIntExpr
  register tTree exp;
 # endif
 {
-# line 544 "SemPreds.puma"
+/* line 544 "SemPreds.puma" */
   {
-# line 545 "SemPreds.puma"
+/* line 545 "SemPreds.puma" */
    if (! ((InitializationExpr (exp)))) goto yyL1;
   {
-# line 546 "SemPreds.puma"
+/* line 546 "SemPreds.puma" */
    if (! ((IntExpr (exp)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 549 "SemPreds.puma"
+/* line 549 "SemPreds.puma" */
   {
-# line 550 "SemPreds.puma"
+/* line 550 "SemPreds.puma" */
    sprintf (type_error_msg, "not integer initialization expr");
-# line 551 "SemPreds.puma"
-   return false;
+/* line 551 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool ArrayExpr
+rbool ArrayExpr
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -884,25 +881,25 @@ bool ArrayExpr
  register tTree exp;
 # endif
 {
-# line 556 "SemPreds.puma"
+/* line 556 "SemPreds.puma" */
   {
-# line 557 "SemPreds.puma"
+/* line 557 "SemPreds.puma" */
    if (! ((TreeRank (exp) > 0))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 560 "SemPreds.puma"
+/* line 560 "SemPreds.puma" */
   {
-# line 561 "SemPreds.puma"
+/* line 561 "SemPreds.puma" */
    sprintf (type_error_msg, "is scalar");
-# line 562 "SemPreds.puma"
-   return false;
+/* line 562 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool DummyHPFArray
+rbool DummyHPFArray
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
 # else
@@ -910,12 +907,12 @@ bool DummyHPFArray
  register tTree exp;
 # endif
 {
-# line 567 "SemPreds.puma"
-   return true;
+/* line 567 "SemPreds.puma" */
+   return rtrue;
 
 }
 
-bool IsLegalOutVar
+rbool IsLegalOutVar
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -924,72 +921,72 @@ bool IsLegalOutVar
 # endif
 {
   if (var->Kind == kINDEXED_VAR) {
-# line 585 "SemPreds.puma"
+/* line 585 "SemPreds.puma" */
   {
-# line 587 "SemPreds.puma"
+/* line 587 "SemPreds.puma" */
    if (! ((IsLegalOutVar (var->INDEXED_VAR.IND_VAR)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   if (var->Kind == kSELECTED_VAR) {
-# line 590 "SemPreds.puma"
+/* line 590 "SemPreds.puma" */
   {
-# line 592 "SemPreds.puma"
+/* line 592 "SemPreds.puma" */
    if (! ((IsLegalOutVar (var->SELECTED_VAR.SELEC_VAR)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   if (var->Kind == kSUBSTRING_VAR) {
-# line 595 "SemPreds.puma"
+/* line 595 "SemPreds.puma" */
   {
-# line 597 "SemPreds.puma"
+/* line 597 "SemPreds.puma" */
    if (! ((IsLegalOutVar (var->SUBSTRING_VAR.IND_VAR)))) goto yyL3;
   }
-   return true;
+   return rtrue;
 yyL3:;
 
   }
   if (var->Kind == kUSED_VAR) {
   if (var->USED_VAR.VARNAME->VAR_OBJ.Object->Kind == kVarObject) {
   if (var->USED_VAR.VARNAME->VAR_OBJ.Object->VarObject.Kind->Kind == kVarParameter) {
-# line 600 "SemPreds.puma"
+/* line 600 "SemPreds.puma" */
   {
-# line 602 "SemPreds.puma"
+/* line 602 "SemPreds.puma" */
    sprintf (type_error_msg, "left hand side of assignment must not be parameter");
-# line 604 "SemPreds.puma"
-   return false;
+/* line 604 "SemPreds.puma" */
+   return rfalse;
   }
 
   }
-# line 607 "SemPreds.puma"
-   return true;
+/* line 607 "SemPreds.puma" */
+   return rtrue;
 
   }
   }
   if (var->Kind == kLOOP_VAR) {
-# line 610 "SemPreds.puma"
+/* line 610 "SemPreds.puma" */
   {
-# line 612 "SemPreds.puma"
+/* line 612 "SemPreds.puma" */
    sprintf (type_error_msg, "loop variable cannot appear as lhs of assignment");
   }
-   return true;
+   return rtrue;
 
   }
-# line 616 "SemPreds.puma"
+/* line 616 "SemPreds.puma" */
   {
-# line 617 "SemPreds.puma"
+/* line 617 "SemPreds.puma" */
    sprintf (type_error_msg, "illegal lhs in assignment");
-# line 618 "SemPreds.puma"
-   return false;
+/* line 618 "SemPreds.puma" */
+   return rfalse;
   }
 
 }
 
-bool IsDefinableParam
+rbool IsDefinableParam
 # if defined __STDC__ | defined __cplusplus
 (register tTree t)
 # else
@@ -999,43 +996,43 @@ bool IsDefinableParam
 {
   if (t->Kind == kVAR_PARAM) {
   if (t->VAR_PARAM.V->Kind == kADDR) {
-# line 629 "SemPreds.puma"
+/* line 629 "SemPreds.puma" */
   {
-# line 631 "SemPreds.puma"
-   return false;
+/* line 631 "SemPreds.puma" */
+   return rfalse;
   }
 
   }
-# line 634 "SemPreds.puma"
+/* line 634 "SemPreds.puma" */
   {
-# line 636 "SemPreds.puma"
+/* line 636 "SemPreds.puma" */
    if (! ((IsParameterVariable (t->VAR_PARAM.V)))) goto yyL2;
   {
-# line 638 "SemPreds.puma"
-   return false;
+/* line 638 "SemPreds.puma" */
+   return rfalse;
   }
   }
 yyL2:;
 
-# line 641 "SemPreds.puma"
+/* line 641 "SemPreds.puma" */
   {
-# line 643 "SemPreds.puma"
+/* line 643 "SemPreds.puma" */
    if (! ((IsVectorVariable (t->VAR_PARAM.V)))) goto yyL3;
   {
-# line 645 "SemPreds.puma"
-   return false;
+/* line 645 "SemPreds.puma" */
+   return rfalse;
   }
   }
 yyL3:;
 
-# line 648 "SemPreds.puma"
-   return true;
+/* line 648 "SemPreds.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
-static bool IsParameterVariable
+static rbool IsParameterVariable
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -1044,39 +1041,39 @@ static bool IsParameterVariable
 # endif
 {
   if (var->Kind == kVAR_OBJ) {
-# line 661 "SemPreds.puma"
+/* line 661 "SemPreds.puma" */
   {
-# line 663 "SemPreds.puma"
+/* line 663 "SemPreds.puma" */
    if (! ((IsVarParameter (var->VAR_OBJ.Object)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
   if (var->Kind == kUSED_VAR) {
-# line 666 "SemPreds.puma"
+/* line 666 "SemPreds.puma" */
   {
-# line 668 "SemPreds.puma"
+/* line 668 "SemPreds.puma" */
    if (! ((IsParameterVariable (var->USED_VAR.VARNAME)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
   }
   if (var->Kind == kINDEXED_VAR) {
-# line 671 "SemPreds.puma"
+/* line 671 "SemPreds.puma" */
   {
-# line 673 "SemPreds.puma"
+/* line 673 "SemPreds.puma" */
    if (! ((IsParameterVariable (var->INDEXED_VAR.IND_VAR)))) goto yyL3;
   }
-   return true;
+   return rtrue;
 yyL3:;
 
   }
-  return false;
+  return rfalse;
 }
 
-static bool IsVectorVariable
+static rbool IsVectorVariable
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -1085,19 +1082,19 @@ static bool IsVectorVariable
 # endif
 {
   if (var->Kind == kINDEXED_VAR) {
-# line 679 "SemPreds.puma"
+/* line 679 "SemPreds.puma" */
   {
-# line 681 "SemPreds.puma"
+/* line 681 "SemPreds.puma" */
    if (! ((HasVectorIndex (var->INDEXED_VAR.IND_EXPS)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   }
-  return false;
+  return rfalse;
 }
 
-static bool HasVectorIndex
+static rbool HasVectorIndex
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp_list)
 # else
@@ -1106,46 +1103,46 @@ static bool HasVectorIndex
 # endif
 {
   if (exp_list->Kind == kBTE_LIST) {
-# line 686 "SemPreds.puma"
+/* line 686 "SemPreds.puma" */
   {
-# line 688 "SemPreds.puma"
+/* line 688 "SemPreds.puma" */
    if (! ((HasVectorIndex (exp_list->BTE_LIST.Next)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   if (exp_list->BTE_LIST.Elem->Kind == kSLICE_EXP) {
-# line 691 "SemPreds.puma"
+/* line 691 "SemPreds.puma" */
   {
-# line 693 "SemPreds.puma"
+/* line 693 "SemPreds.puma" */
    if (! ((HasVectorIndex (exp_list->BTE_LIST.Next)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 696 "SemPreds.puma"
+/* line 696 "SemPreds.puma" */
   {
-# line 698 "SemPreds.puma"
-   return false;
+/* line 698 "SemPreds.puma" */
+   return rfalse;
   }
 
   }
-# line 701 "SemPreds.puma"
+/* line 701 "SemPreds.puma" */
   {
-# line 703 "SemPreds.puma"
+/* line 703 "SemPreds.puma" */
    if (! ((TreeRank (exp_list->BTE_LIST.Elem) > 0))) goto yyL4;
   }
-   return true;
+   return rtrue;
 yyL4:;
 
   }
-  return false;
+  return rfalse;
 }
 
-void BeginSemPreds ()
+void BeginSemPreds ARGS ((void))
 {
 }
 
-void CloseSemPreds ()
+void CloseSemPreds ARGS ((void))
 {
 }
