@@ -1,6 +1,13 @@
 # include "SpecialMap.h"
-# include "yySpecialMap.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,32 +15,29 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 27 "SpecialMap.puma"
+/* line 27 "SpecialMap.puma" */
 
 
 # include <stdio.h>
 # include "Idents.h"
-# include "StringMem.h"
+# include "StringM.h"
  
 # include "protocol.h"
 
@@ -43,11 +47,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* SpecialMap_Exit) () = yyExit;
+# include "yySpecialMap.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* SpecialMap_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -56,30 +74,31 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module SpecialMap, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module SpecialMap, routine %s failed\n",
+  yyFunction);
  SpecialMap_Exit ();
 }
 
-bool IsSpecialisation ARGS((pvar actual_vard, pvar dummy_vard));
-static bool IsTopSpecialisation ARGS((tDefinitions actual_topology, tDefinitions dummy_topology));
-static bool IsDistFormatSpecialisation ARGS((int actual_dist_kind, tTree actual_dist_size, int dummy_dist_kind, tTree dummy_dist_size));
-static bool IsRangeFormatSpecialisation ARGS((int actual_dist_kind, tTree actual_dist_size, int dummy_range_kind, tTree dummy_range_size));
-static bool IsSameSize ARGS((tTree size1, tTree size2));
-static bool IsDistSpecialisation ARGS((pvar actual_vard, pvar dummy_vard));
-static bool IsRangeSpecialisation ARGS((pvar actual_vard, pvar dummy_vard));
-static bool IsMapSpecialisation ARGS((pvar actual_vard, pvar dummy_vard));
-static bool IsRegularAlignment ARGS((pvar vard));
-static bool IsNormalAlignment ARGS((pvar vard));
-static bool HasNoPermutation ARGS((pvar vard));
-static bool IsNoStrideAlignment ARGS((pvar vard));
-static bool IsFullFormalShape ARGS((pvar vard));
-static bool IsFullFormalDimShape ARGS((tTree lb, tTree ub, tTree str, tTree formal_lb, tTree formal_ub));
-static bool MyEqualExpression ARGS((tTree actual, tTree formal));
-static bool IsDistFormatUnderspecified ARGS((int dist_kind));
-static bool IsDistUnderspecified ARGS((pvar vard));
-static bool IsMapUnderspecified ARGS((pvar vard));
+rbool IsSpecialisation ARGS ((pvar actual_vard, pvar dummy_vard));
+static rbool IsTopSpecialisation ARGS ((tDefinitions actual_topology, tDefinitions dummy_topology));
+static rbool IsDistFormatSpecialisation ARGS ((int actual_dist_kind, tTree actual_dist_size, int dummy_dist_kind, tTree dummy_dist_size));
+static rbool IsRangeFormatSpecialisation ARGS ((int actual_dist_kind, tTree actual_dist_size, int dummy_range_kind, tTree dummy_range_size));
+static rbool IsSameSize ARGS ((tTree size1, tTree size2));
+static rbool IsDistSpecialisation ARGS ((pvar actual_vard, pvar dummy_vard));
+static rbool IsRangeSpecialisation ARGS ((pvar actual_vard, pvar dummy_vard));
+static rbool IsMapSpecialisation ARGS ((pvar actual_vard, pvar dummy_vard));
+static rbool IsRegularAlignment ARGS ((pvar vard));
+static rbool IsNormalAlignment ARGS ((pvar vard));
+static rbool HasNoPermutation ARGS ((pvar vard));
+static rbool IsNoStrideAlignment ARGS ((pvar vard));
+static rbool IsFullFormalShape ARGS ((pvar vard));
+static rbool IsFullFormalDimShape ARGS ((tTree lb, tTree ub, tTree str, tTree formal_lb, tTree formal_ub));
+static rbool MyEqualExpression ARGS ((tTree actual, tTree formal));
+static rbool IsDistFormatUnderspecified ARGS ((int dist_kind));
+static rbool IsDistUnderspecified ARGS ((pvar vard));
+static rbool IsMapUnderspecified ARGS ((pvar vard));
 
-bool IsSpecialisation
+rbool IsSpecialisation
 # if defined __STDC__ | defined __cplusplus
 (pvar actual_vard, pvar dummy_vard)
 # else
@@ -88,22 +107,22 @@ bool IsSpecialisation
  pvar dummy_vard;
 # endif
 {
-# line 49 "SpecialMap.puma"
+/* line 49 "SpecialMap.puma" */
   {
-# line 51 "SpecialMap.puma"
+/* line 51 "SpecialMap.puma" */
    if (! ((IsTopSpecialisation (actual_vard -> topology_obj, dummy_vard -> topology_obj)))) goto yyL1;
   {
-# line 52 "SpecialMap.puma"
+/* line 52 "SpecialMap.puma" */
    if (! ((IsMapSpecialisation (actual_vard, dummy_vard)))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsTopSpecialisation
+static rbool IsTopSpecialisation
 # if defined __STDC__ | defined __cplusplus
 (register tDefinitions actual_topology, register tDefinitions dummy_topology)
 # else
@@ -112,30 +131,30 @@ static bool IsTopSpecialisation
  register tDefinitions dummy_topology;
 # endif
 {
-# line 64 "SpecialMap.puma"
+/* line 64 "SpecialMap.puma" */
   {
-# line 66 "SpecialMap.puma"
+/* line 66 "SpecialMap.puma" */
    if (! ((dummy_topology == NoObject))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 69 "SpecialMap.puma"
+/* line 69 "SpecialMap.puma" */
   {
-# line 71 "SpecialMap.puma"
+/* line 71 "SpecialMap.puma" */
    if (! ((actual_topology != NoObject))) goto yyL2;
   {
-# line 72 "SpecialMap.puma"
+/* line 72 "SpecialMap.puma" */
    if (! ((SameTopologyObject (actual_topology, dummy_topology)))) goto yyL2;
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsDistFormatSpecialisation
+static rbool IsDistFormatSpecialisation
 # if defined __STDC__ | defined __cplusplus
 (register int actual_dist_kind, register tTree actual_dist_size, register int dummy_dist_kind, register tTree dummy_dist_size)
 # else
@@ -146,9 +165,9 @@ static bool IsDistFormatSpecialisation
  register tTree dummy_dist_size;
 # endif
 {
-# line 97 "SpecialMap.puma"
+/* line 97 "SpecialMap.puma" */
   {
-# line 99 "SpecialMap.puma"
+/* line 99 "SpecialMap.puma" */
 
 #ifdef DEBUG
     printf ("IsDistFormatSpecialisation, act_kind = %d, act_size =", 
@@ -159,147 +178,147 @@ static bool IsDistFormatSpecialisation
     printf ("\n");
 #endif
    
-# line 110 "SpecialMap.puma"
+/* line 110 "SpecialMap.puma" */
    goto yyL1;
   }
 yyL1:;
 
-# line 113 "SpecialMap.puma"
+/* line 113 "SpecialMap.puma" */
   {
-# line 115 "SpecialMap.puma"
+/* line 115 "SpecialMap.puma" */
    if (! ((actual_dist_kind == kBLOCK_DIM))) goto yyL2;
   {
-# line 116 "SpecialMap.puma"
+/* line 116 "SpecialMap.puma" */
    if (! ((dummy_dist_kind == kBLOCK_DIM))) goto yyL2;
   {
-# line 117 "SpecialMap.puma"
+/* line 117 "SpecialMap.puma" */
    if (! ((IsSameSize (actual_dist_size, dummy_dist_size)))) goto yyL2;
   }
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 122 "SpecialMap.puma"
+/* line 122 "SpecialMap.puma" */
   {
-# line 124 "SpecialMap.puma"
+/* line 124 "SpecialMap.puma" */
    if (! (((actual_dist_kind == kBLOCK_DIM) || (actual_dist_kind == kANY_BLOCK_DIM)))) goto yyL3;
   {
-# line 125 "SpecialMap.puma"
+/* line 125 "SpecialMap.puma" */
    if (! ((dummy_dist_kind == kANY_BLOCK_DIM))) goto yyL3;
   }
   }
-   return true;
+   return rtrue;
 yyL3:;
 
-# line 130 "SpecialMap.puma"
+/* line 130 "SpecialMap.puma" */
   {
-# line 132 "SpecialMap.puma"
+/* line 132 "SpecialMap.puma" */
    if (! ((actual_dist_kind == kCYCLIC_DIM))) goto yyL4;
   {
-# line 133 "SpecialMap.puma"
+/* line 133 "SpecialMap.puma" */
    if (! ((dummy_dist_kind == kCYCLIC_DIM))) goto yyL4;
   {
-# line 134 "SpecialMap.puma"
+/* line 134 "SpecialMap.puma" */
    if (! ((IsSameSize (actual_dist_size, dummy_dist_size)))) goto yyL4;
   }
   }
   }
-   return true;
+   return rtrue;
 yyL4:;
 
-# line 139 "SpecialMap.puma"
+/* line 139 "SpecialMap.puma" */
   {
-# line 141 "SpecialMap.puma"
+/* line 141 "SpecialMap.puma" */
    if (! (((actual_dist_kind == kCYCLIC_DIM) || (actual_dist_kind == kANY_CYCLIC_DIM)))) goto yyL5;
   {
-# line 143 "SpecialMap.puma"
+/* line 143 "SpecialMap.puma" */
    if (! ((dummy_dist_kind == kANY_CYCLIC_DIM))) goto yyL5;
   }
   }
-   return true;
+   return rtrue;
 yyL5:;
 
-# line 148 "SpecialMap.puma"
+/* line 148 "SpecialMap.puma" */
   {
-# line 150 "SpecialMap.puma"
+/* line 150 "SpecialMap.puma" */
    if (! ((actual_dist_kind == kGEN_BLOCK_DIM))) goto yyL6;
   {
-# line 151 "SpecialMap.puma"
+/* line 151 "SpecialMap.puma" */
    if (! ((dummy_dist_kind == kGEN_BLOCK_DIM))) goto yyL6;
   {
-# line 152 "SpecialMap.puma"
+/* line 152 "SpecialMap.puma" */
    if (! ((IsSameSize (actual_dist_size, dummy_dist_size)))) goto yyL6;
   }
   }
   }
-   return true;
+   return rtrue;
 yyL6:;
 
-# line 157 "SpecialMap.puma"
+/* line 157 "SpecialMap.puma" */
   {
-# line 159 "SpecialMap.puma"
+/* line 159 "SpecialMap.puma" */
    if (! (((actual_dist_kind == kGEN_BLOCK_DIM) || (actual_dist_kind == kANY_GEN_BLOCK_DIM)))) goto yyL7;
   {
-# line 161 "SpecialMap.puma"
+/* line 161 "SpecialMap.puma" */
    if (! ((dummy_dist_kind == kANY_GEN_BLOCK_DIM))) goto yyL7;
   }
   }
-   return true;
+   return rtrue;
 yyL7:;
 
-# line 166 "SpecialMap.puma"
+/* line 166 "SpecialMap.puma" */
   {
-# line 168 "SpecialMap.puma"
+/* line 168 "SpecialMap.puma" */
    if (! ((actual_dist_kind == kINDIRECT_DIM))) goto yyL8;
   {
-# line 169 "SpecialMap.puma"
+/* line 169 "SpecialMap.puma" */
    if (! ((dummy_dist_kind == kINDIRECT_DIM))) goto yyL8;
   {
-# line 170 "SpecialMap.puma"
+/* line 170 "SpecialMap.puma" */
    if (! ((IsSameSize (actual_dist_size, dummy_dist_size)))) goto yyL8;
   }
   }
   }
-   return true;
+   return rtrue;
 yyL8:;
 
-# line 175 "SpecialMap.puma"
+/* line 175 "SpecialMap.puma" */
   {
-# line 177 "SpecialMap.puma"
+/* line 177 "SpecialMap.puma" */
    if (! (((actual_dist_kind == kINDIRECT_DIM) || (actual_dist_kind == kANY_INDIRECT_DIM)))) goto yyL9;
   {
-# line 179 "SpecialMap.puma"
+/* line 179 "SpecialMap.puma" */
    if (! ((dummy_dist_kind == kANY_INDIRECT_DIM))) goto yyL9;
   }
   }
-   return true;
+   return rtrue;
 yyL9:;
 
-# line 184 "SpecialMap.puma"
+/* line 184 "SpecialMap.puma" */
   {
-# line 186 "SpecialMap.puma"
+/* line 186 "SpecialMap.puma" */
    if (! ((actual_dist_kind == kSERIAL_DIM))) goto yyL10;
   {
-# line 187 "SpecialMap.puma"
+/* line 187 "SpecialMap.puma" */
    if (! ((dummy_dist_kind == kSERIAL_DIM))) goto yyL10;
   }
   }
-   return true;
+   return rtrue;
 yyL10:;
 
-# line 192 "SpecialMap.puma"
+/* line 192 "SpecialMap.puma" */
   {
-# line 194 "SpecialMap.puma"
+/* line 194 "SpecialMap.puma" */
    if (! ((dummy_dist_kind == kANY_DISTRIBUTED_DIM))) goto yyL11;
   }
-   return true;
+   return rtrue;
 yyL11:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsRangeFormatSpecialisation
+static rbool IsRangeFormatSpecialisation
 # if defined __STDC__ | defined __cplusplus
 (register int actual_dist_kind, register tTree actual_dist_size, register int dummy_range_kind, register tTree dummy_range_size)
 # else
@@ -310,26 +329,26 @@ static bool IsRangeFormatSpecialisation
  register tTree dummy_range_size;
 # endif
 {
-# line 208 "SpecialMap.puma"
+/* line 208 "SpecialMap.puma" */
   {
-# line 210 "SpecialMap.puma"
+/* line 210 "SpecialMap.puma" */
    if (! ((IsDistFormatSpecialisation (actual_dist_kind, actual_dist_size, dummy_range_kind, dummy_range_size)))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 216 "SpecialMap.puma"
+/* line 216 "SpecialMap.puma" */
   {
-# line 218 "SpecialMap.puma"
+/* line 218 "SpecialMap.puma" */
    if (! ((actual_dist_kind == kSERIAL_DIM))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsSameSize
+static rbool IsSameSize
 # if defined __STDC__ | defined __cplusplus
 (register tTree size1, register tTree size2)
 # else
@@ -338,38 +357,38 @@ static bool IsSameSize
  register tTree size2;
 # endif
 {
-# line 228 "SpecialMap.puma"
+/* line 228 "SpecialMap.puma" */
   {
-# line 230 "SpecialMap.puma"
+/* line 230 "SpecialMap.puma" */
    if (! ((size1 == NoTree))) goto yyL1;
   {
-# line 231 "SpecialMap.puma"
+/* line 231 "SpecialMap.puma" */
    if (! ((size2 == NoTree))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 234 "SpecialMap.puma"
+/* line 234 "SpecialMap.puma" */
   {
-# line 236 "SpecialMap.puma"
+/* line 236 "SpecialMap.puma" */
    if (! ((size1 != NoTree))) goto yyL2;
   {
-# line 237 "SpecialMap.puma"
+/* line 237 "SpecialMap.puma" */
    if (! ((size2 != NoTree))) goto yyL2;
   {
-# line 238 "SpecialMap.puma"
+/* line 238 "SpecialMap.puma" */
    if (! ((EqualExpression (size1, size2)))) goto yyL2;
   }
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsDistSpecialisation
+static rbool IsDistSpecialisation
 # if defined __STDC__ | defined __cplusplus
 (pvar actual_vard, pvar dummy_vard)
 # else
@@ -378,16 +397,14 @@ static bool IsDistSpecialisation
  pvar dummy_vard;
 # endif
 {
-# line 249 "SpecialMap.puma"
+/* line 249 "SpecialMap.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 251 "SpecialMap.puma"
-
-# line 253 "SpecialMap.puma"
+/* line 253 "SpecialMap.puma" */
  int i, rank;
 
-     okay = true; rank = actual_vard->formal_rank;
+     okay = rtrue; rank = actual_vard->formal_rank;
 
      for (i=0; i<rank; i++)
 
@@ -396,19 +413,19 @@ static bool IsDistSpecialisation
           actual_vard->distribution_kind[i], actual_vard->distribution_size[i],
           dummy_vard->distribution_kind[i], dummy_vard->distribution_size[i]) )
 
-             okay = false;
+             okay = rfalse;
    
-# line 267 "SpecialMap.puma"
+/* line 267 "SpecialMap.puma" */
    if (! ((okay))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsRangeSpecialisation
+static rbool IsRangeSpecialisation
 # if defined __STDC__ | defined __cplusplus
 (pvar actual_vard, pvar dummy_vard)
 # else
@@ -417,16 +434,14 @@ static bool IsRangeSpecialisation
  pvar dummy_vard;
 # endif
 {
-# line 278 "SpecialMap.puma"
+/* line 278 "SpecialMap.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 280 "SpecialMap.puma"
-
-# line 282 "SpecialMap.puma"
+/* line 282 "SpecialMap.puma" */
  int i, rank;
 
-     okay = true; rank = actual_vard->formal_rank;
+     okay = rtrue; rank = actual_vard->formal_rank;
 
      for (i=0; i<rank; i++)
 
@@ -435,19 +450,19 @@ static bool IsRangeSpecialisation
           actual_vard->distribution_kind[i], actual_vard->distribution_size[i],
           dummy_vard->distribution_kind[i], dummy_vard->distribution_size[i]) )
 
-            okay = false;
+            okay = rfalse;
    
-# line 296 "SpecialMap.puma"
+/* line 296 "SpecialMap.puma" */
    if (! ((okay))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsMapSpecialisation
+static rbool IsMapSpecialisation
 # if defined __STDC__ | defined __cplusplus
 (pvar actual_vard, pvar dummy_vard)
 # else
@@ -456,122 +471,122 @@ static bool IsMapSpecialisation
  pvar dummy_vard;
 # endif
 {
-# line 307 "SpecialMap.puma"
+/* line 307 "SpecialMap.puma" */
   {
-# line 317 "SpecialMap.puma"
+/* line 317 "SpecialMap.puma" */
    if (! ((actual_vard -> formal_rank != dummy_vard -> formal_rank))) goto yyL1;
   {
-# line 319 "SpecialMap.puma"
+/* line 319 "SpecialMap.puma" */
    warning_protocol ("rank mismatch between actual and dummy");
-# line 321 "SpecialMap.puma"
-   return false;
+/* line 321 "SpecialMap.puma" */
+   return rfalse;
   }
   }
 yyL1:;
 
-# line 332 "SpecialMap.puma"
+/* line 332 "SpecialMap.puma" */
   {
-# line 334 "SpecialMap.puma"
+/* line 334 "SpecialMap.puma" */
    if (! ((dummy_vard -> var_obj == dummy_vard -> template_obj))) goto yyL2;
   {
-# line 335 "SpecialMap.puma"
+/* line 335 "SpecialMap.puma" */
    if (! ((dummy_vard -> topology_obj != NoObject))) goto yyL2;
   {
-# line 337 "SpecialMap.puma"
+/* line 337 "SpecialMap.puma" */
    if (! ((IsRegularAlignment (actual_vard)))) goto yyL2;
   {
-# line 338 "SpecialMap.puma"
+/* line 338 "SpecialMap.puma" */
    if (! ((IsDistSpecialisation (actual_vard, dummy_vard)))) goto yyL2;
   }
   }
   }
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 349 "SpecialMap.puma"
+/* line 349 "SpecialMap.puma" */
   {
-# line 351 "SpecialMap.puma"
+/* line 351 "SpecialMap.puma" */
    if (! ((dummy_vard -> var_obj == dummy_vard -> template_obj))) goto yyL3;
   {
-# line 352 "SpecialMap.puma"
+/* line 352 "SpecialMap.puma" */
    if (! ((dummy_vard -> topology_obj == NoObject))) goto yyL3;
   {
-# line 353 "SpecialMap.puma"
+/* line 353 "SpecialMap.puma" */
    if (! ((! dummy_vard -> template_inherited))) goto yyL3;
   {
-# line 355 "SpecialMap.puma"
+/* line 355 "SpecialMap.puma" */
    if (! ((IsNoStrideAlignment (actual_vard)))) goto yyL3;
   {
-# line 356 "SpecialMap.puma"
+/* line 356 "SpecialMap.puma" */
    if (! ((IsRangeSpecialisation (actual_vard, dummy_vard)))) goto yyL3;
   }
   }
   }
   }
   }
-   return true;
+   return rtrue;
 yyL3:;
 
-# line 367 "SpecialMap.puma"
+/* line 367 "SpecialMap.puma" */
   {
-# line 369 "SpecialMap.puma"
+/* line 369 "SpecialMap.puma" */
    if (! ((dummy_vard -> var_obj == dummy_vard -> template_obj))) goto yyL4;
   {
-# line 370 "SpecialMap.puma"
+/* line 370 "SpecialMap.puma" */
    if (! ((dummy_vard -> topology_obj == NoObject))) goto yyL4;
   {
-# line 371 "SpecialMap.puma"
+/* line 371 "SpecialMap.puma" */
    if (! ((dummy_vard -> template_inherited))) goto yyL4;
   {
-# line 373 "SpecialMap.puma"
+/* line 373 "SpecialMap.puma" */
    if (! ((IsRangeSpecialisation (actual_vard, dummy_vard)))) goto yyL4;
   }
   }
   }
   }
-   return true;
+   return rtrue;
 yyL4:;
 
-# line 387 "SpecialMap.puma"
+/* line 387 "SpecialMap.puma" */
   {
-# line 389 "SpecialMap.puma"
+/* line 389 "SpecialMap.puma" */
    if (! ((dummy_vard -> var_obj != dummy_vard -> template_obj))) goto yyL5;
   {
-# line 391 "SpecialMap.puma"
+/* line 391 "SpecialMap.puma" */
    if (! ((IsRegularAlignment (dummy_vard)))) goto yyL5;
   {
-# line 392 "SpecialMap.puma"
+/* line 392 "SpecialMap.puma" */
    if (! ((! IsMapUnderspecified (dummy_vard)))) goto yyL5;
   {
-# line 393 "SpecialMap.puma"
+/* line 393 "SpecialMap.puma" */
    if (! ((IsRegularAlignment (actual_vard)))) goto yyL5;
   {
-# line 394 "SpecialMap.puma"
+/* line 394 "SpecialMap.puma" */
    if (! ((IsDistSpecialisation (actual_vard, dummy_vard)))) goto yyL5;
   }
   }
   }
   }
   }
-   return true;
+   return rtrue;
 yyL5:;
 
-# line 405 "SpecialMap.puma"
+/* line 405 "SpecialMap.puma" */
   {
-# line 407 "SpecialMap.puma"
+/* line 407 "SpecialMap.puma" */
    if (! ((dummy_vard -> var_obj != dummy_vard -> template_obj))) goto yyL6;
   {
-# line 411 "SpecialMap.puma"
-   return false;
+/* line 411 "SpecialMap.puma" */
+   return rfalse;
   }
   }
 yyL6:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsRegularAlignment
+static rbool IsRegularAlignment
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -579,30 +594,30 @@ static bool IsRegularAlignment
  pvar vard;
 # endif
 {
-# line 426 "SpecialMap.puma"
+/* line 426 "SpecialMap.puma" */
   {
-# line 428 "SpecialMap.puma"
+/* line 428 "SpecialMap.puma" */
    if (! ((IsNormalAlignment (vard)))) goto yyL1;
   {
-# line 429 "SpecialMap.puma"
+/* line 429 "SpecialMap.puma" */
    if (! ((HasNoPermutation (vard)))) goto yyL1;
   {
-# line 430 "SpecialMap.puma"
+/* line 430 "SpecialMap.puma" */
    if (! ((IsNoStrideAlignment (vard)))) goto yyL1;
   {
-# line 431 "SpecialMap.puma"
+/* line 431 "SpecialMap.puma" */
    if (! ((IsFullFormalShape (vard)))) goto yyL1;
   }
   }
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsNormalAlignment
+static rbool IsNormalAlignment
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -610,35 +625,33 @@ static bool IsNormalAlignment
  pvar vard;
 # endif
 {
-# line 446 "SpecialMap.puma"
+/* line 446 "SpecialMap.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 448 "SpecialMap.puma"
-
-# line 450 "SpecialMap.puma"
+/* line 450 "SpecialMap.puma" */
  int i, rank;
 
-     okay = true; rank = vard->topology_rank;
+     okay = rtrue; rank = vard->topology_rank;
 
      for (i=0; i<rank; i++)
 
         if (vard->on_index_dim[i] == 0)
 
-           okay = false;
+           okay = rfalse;
 
    
-# line 462 "SpecialMap.puma"
+/* line 462 "SpecialMap.puma" */
    if (! ((okay))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool HasNoPermutation
+static rbool HasNoPermutation
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -646,16 +659,14 @@ static bool HasNoPermutation
  pvar vard;
 # endif
 {
-# line 475 "SpecialMap.puma"
+/* line 475 "SpecialMap.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 477 "SpecialMap.puma"
-
-# line 479 "SpecialMap.puma"
+/* line 479 "SpecialMap.puma" */
  int i, rank, last_dim;
 
-     okay = true; last_dim = 0;
+     okay = rtrue; last_dim = 0;
 
      rank = vard->topology_rank;
 
@@ -669,7 +680,7 @@ static bool HasNoPermutation
 
              { if (index_dim < last_dim)
 
-                  okay = false;
+                  okay = rfalse;
 
                  else
 
@@ -680,17 +691,17 @@ static bool HasNoPermutation
         } 
 
    
-# line 507 "SpecialMap.puma"
+/* line 507 "SpecialMap.puma" */
    if (! ((okay))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsNoStrideAlignment
+static rbool IsNoStrideAlignment
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -698,38 +709,36 @@ static bool IsNoStrideAlignment
  pvar vard;
 # endif
 {
-# line 522 "SpecialMap.puma"
+/* line 522 "SpecialMap.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 524 "SpecialMap.puma"
-
-# line 526 "SpecialMap.puma"
+/* line 526 "SpecialMap.puma" */
  int i, rank;
 
-     okay = true; rank = vard->formal_rank;
+     okay = rtrue; rank = vard->formal_rank;
 
      for (i=0; i<rank; i++)
 
-       {  if (vard->align_mult[i] != 1) okay = false;
+       {  if (vard->align_mult[i] != 1) okay = rfalse;
 
           if (vard->topology_dim[i] > 0)  
 
-            if (!IsStride1 (vard->actual_shape[i][2])) okay = false;
+            if (!IsStride1 (vard->actual_shape[i][2])) okay = rfalse;
        }
 
    
-# line 541 "SpecialMap.puma"
+/* line 541 "SpecialMap.puma" */
    if (! ((okay))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsFullFormalShape
+static rbool IsFullFormalShape
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -737,16 +746,14 @@ static bool IsFullFormalShape
  pvar vard;
 # endif
 {
-# line 554 "SpecialMap.puma"
+/* line 554 "SpecialMap.puma" */
  {
-  bool okay;
+  rbool okay;
   {
-# line 556 "SpecialMap.puma"
-
-# line 558 "SpecialMap.puma"
+/* line 558 "SpecialMap.puma" */
  int i, rank;
 
-     okay = true; rank = vard->formal_rank;
+     okay = rtrue; rank = vard->formal_rank;
 
      for (i=0; i<rank; i++)
 
@@ -759,20 +766,20 @@ static bool IsFullFormalShape
                 vard->actual_shape[i][2],
                 vard->formal_shape[i][0], vard->formal_shape[i][1]))
      
-            okay = false;
+            okay = rfalse;
        }
    
-# line 577 "SpecialMap.puma"
+/* line 577 "SpecialMap.puma" */
    if (! ((okay))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsFullFormalDimShape
+static rbool IsFullFormalDimShape
 # if defined __STDC__ | defined __cplusplus
 (register tTree lb, register tTree ub, register tTree str, register tTree formal_lb, register tTree formal_ub)
 # else
@@ -784,44 +791,44 @@ static bool IsFullFormalDimShape
  register tTree formal_ub;
 # endif
 {
-# line 583 "SpecialMap.puma"
+/* line 583 "SpecialMap.puma" */
   {
-# line 585 "SpecialMap.puma"
+/* line 585 "SpecialMap.puma" */
    if (! ((lb == ub))) goto yyL1;
   {
-# line 586 "SpecialMap.puma"
-   return false;
+/* line 586 "SpecialMap.puma" */
+   return rfalse;
   }
   }
 yyL1:;
 
-# line 589 "SpecialMap.puma"
+/* line 589 "SpecialMap.puma" */
   {
-# line 591 "SpecialMap.puma"
+/* line 591 "SpecialMap.puma" */
    if (! ((! IsStride1 (str)))) goto yyL2;
   {
-# line 592 "SpecialMap.puma"
-   return false;
+/* line 592 "SpecialMap.puma" */
+   return rfalse;
   }
   }
 yyL2:;
 
-# line 595 "SpecialMap.puma"
+/* line 595 "SpecialMap.puma" */
   {
-# line 597 "SpecialMap.puma"
+/* line 597 "SpecialMap.puma" */
    if (! ((MyEqualExpression (lb, formal_lb)))) goto yyL3;
   {
-# line 598 "SpecialMap.puma"
+/* line 598 "SpecialMap.puma" */
    if (! ((MyEqualExpression (ub, formal_ub)))) goto yyL3;
   }
   }
-   return true;
+   return rtrue;
 yyL3:;
 
-  return false;
+  return rfalse;
 }
 
-static bool MyEqualExpression
+static rbool MyEqualExpression
 # if defined __STDC__ | defined __cplusplus
 (register tTree actual, register tTree formal)
 # else
@@ -830,52 +837,52 @@ static bool MyEqualExpression
  register tTree formal;
 # endif
 {
-# line 603 "SpecialMap.puma"
+/* line 603 "SpecialMap.puma" */
   {
-# line 605 "SpecialMap.puma"
+/* line 605 "SpecialMap.puma" */
    if (! ((actual == NoTree))) goto yyL1;
   {
-# line 606 "SpecialMap.puma"
+/* line 606 "SpecialMap.puma" */
    if (! ((formal == NoTree))) goto yyL1;
   }
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 609 "SpecialMap.puma"
+/* line 609 "SpecialMap.puma" */
   {
-# line 611 "SpecialMap.puma"
+/* line 611 "SpecialMap.puma" */
    if (! ((actual == NoTree))) goto yyL2;
   {
-# line 612 "SpecialMap.puma"
-   return false;
+/* line 612 "SpecialMap.puma" */
+   return rfalse;
   }
   }
 yyL2:;
 
-# line 614 "SpecialMap.puma"
+/* line 614 "SpecialMap.puma" */
   {
-# line 616 "SpecialMap.puma"
+/* line 616 "SpecialMap.puma" */
    if (! ((formal == NoTree))) goto yyL3;
   {
-# line 617 "SpecialMap.puma"
-   return false;
+/* line 617 "SpecialMap.puma" */
+   return rfalse;
   }
   }
 yyL3:;
 
-# line 620 "SpecialMap.puma"
+/* line 620 "SpecialMap.puma" */
   {
-# line 622 "SpecialMap.puma"
+/* line 622 "SpecialMap.puma" */
    if (! ((EqualExpression (actual, formal)))) goto yyL4;
   }
-   return true;
+   return rtrue;
 yyL4:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsDistFormatUnderspecified
+static rbool IsDistFormatUnderspecified
 # if defined __STDC__ | defined __cplusplus
 (register int dist_kind)
 # else
@@ -883,50 +890,50 @@ static bool IsDistFormatUnderspecified
  register int dist_kind;
 # endif
 {
-# line 636 "SpecialMap.puma"
+/* line 636 "SpecialMap.puma" */
   {
-# line 637 "SpecialMap.puma"
+/* line 637 "SpecialMap.puma" */
    if (! ((dist_kind == kANY_BLOCK_DIM))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 640 "SpecialMap.puma"
+/* line 640 "SpecialMap.puma" */
   {
-# line 641 "SpecialMap.puma"
+/* line 641 "SpecialMap.puma" */
    if (! ((dist_kind == kANY_CYCLIC_DIM))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-# line 644 "SpecialMap.puma"
+/* line 644 "SpecialMap.puma" */
   {
-# line 645 "SpecialMap.puma"
+/* line 645 "SpecialMap.puma" */
    if (! ((dist_kind == kANY_GEN_BLOCK_DIM))) goto yyL3;
   }
-   return true;
+   return rtrue;
 yyL3:;
 
-# line 648 "SpecialMap.puma"
+/* line 648 "SpecialMap.puma" */
   {
-# line 649 "SpecialMap.puma"
+/* line 649 "SpecialMap.puma" */
    if (! ((dist_kind == kANY_INDIRECT_DIM))) goto yyL4;
   }
-   return true;
+   return rtrue;
 yyL4:;
 
-# line 652 "SpecialMap.puma"
+/* line 652 "SpecialMap.puma" */
   {
-# line 653 "SpecialMap.puma"
+/* line 653 "SpecialMap.puma" */
    if (! ((dist_kind == kANY_DISTRIBUTED_DIM))) goto yyL5;
   }
-   return true;
+   return rtrue;
 yyL5:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsDistUnderspecified
+static rbool IsDistUnderspecified
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -934,16 +941,14 @@ static bool IsDistUnderspecified
  pvar vard;
 # endif
 {
-# line 664 "SpecialMap.puma"
+/* line 664 "SpecialMap.puma" */
  {
-  bool under_spec;
+  rbool under_spec;
   {
-# line 666 "SpecialMap.puma"
-
-# line 668 "SpecialMap.puma"
+/* line 668 "SpecialMap.puma" */
  int i, rank;
 
-     under_spec = false;
+     under_spec = rfalse;
 
      rank = vard->formal_rank;
 
@@ -951,19 +956,19 @@ static bool IsDistUnderspecified
 
         if (IsDistFormatUnderspecified (vard->distribution_kind[i]))
 
-           under_spec = true;
+           under_spec = rtrue;
    
-# line 681 "SpecialMap.puma"
+/* line 681 "SpecialMap.puma" */
    if (! ((under_spec))) goto yyL1;
   }
-   return true;
+   return rtrue;
  }
 yyL1:;
 
-  return false;
+  return rfalse;
 }
 
-static bool IsMapUnderspecified
+static rbool IsMapUnderspecified
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -971,29 +976,29 @@ static bool IsMapUnderspecified
  pvar vard;
 # endif
 {
-# line 692 "SpecialMap.puma"
+/* line 692 "SpecialMap.puma" */
   {
-# line 694 "SpecialMap.puma"
+/* line 694 "SpecialMap.puma" */
    if (! ((vard -> topology_obj == NoObject))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
-# line 697 "SpecialMap.puma"
+/* line 697 "SpecialMap.puma" */
   {
-# line 698 "SpecialMap.puma"
+/* line 698 "SpecialMap.puma" */
    if (! ((IsDistUnderspecified (vard)))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
-void BeginSpecialMap ()
+void BeginSpecialMap ARGS ((void))
 {
 }
 
-void CloseSpecialMap ()
+void CloseSpecialMap ARGS ((void))
 {
 }
