@@ -20,7 +20,7 @@
 *                                                                    *
 *********************************************************************/
 
-# include "ratc.h"           /* needed for bool */
+# include "ratc.h"           /* needed for rbool */
 # include "global.h"
 # include "SrcLines.h"
 
@@ -55,7 +55,7 @@
   static int  Buffer_NewLines;
   static int  Buffer_State;
   static int  Buffer_ParNest;        /* nesting of parentheses ) and ( */
-  static bool Buffer_Continue;       /* true if Buffer ends with &     */
+  static rbool Buffer_Continue;       /* rtrue if Buffer ends with &     */
 
 
 #define BUFFER_NO_ERROR      0
@@ -138,12 +138,12 @@ char c;
     return c;
 }
 
-static bool my_compare (str1, str2)
+static rbool my_compare (str1, str2)
 char *str1, *str2;
 
-{  bool equal;
+{  rbool equal;
 
-   equal = true;
+   equal = rtrue;
 
    while (equal && *str2)
      equal = (my_upper (*str1++) == my_upper (*str2++));
@@ -167,36 +167,36 @@ char *str1, *str2;
 #define DIRECTIVE_LINE    5
 #define CONT_DIR_LINE     6
 
-static bool IsAnyCharacter (c)
+static rbool IsAnyCharacter (c)
 char c;
 
-{ if (c == ' ')  return (false);
-  if (c == '\t') return (false);
-  if (c == '\n') return (false);
-  return (true);
+{ if (c == ' ')  return (rfalse);
+  if (c == '\t') return (rfalse);
+  if (c == '\n') return (rfalse);
+  return (rtrue);
 
 } /* IsAnyCharacter */
 
-static bool IsDigit (c)
+static rbool IsDigit (c)
 char c;
-{ if (c < '0') return (false);
-  if (c > '9') return (false);
-  return (true);
+{ if (c < '0') return (rfalse);
+  if (c > '9') return (rfalse);
+  return (rtrue);
 }
 
-static bool IsFixedComment (c)
+static rbool IsFixedComment (c)
 char c;
-{ if (c == 'c') return (true);
-  if (c == 'C') return (true);
-  if (c == '!') return (true);
-  if (c == '$') return (true);
-  if (c == '*') return (true);
+{ if (c == 'c') return (rtrue);
+  if (c == 'C') return (rtrue);
+  if (c == '!') return (rtrue);
+  if (c == '$') return (rtrue);
+  if (c == '*') return (rtrue);
 }
 
-/* the following predicate returns true if the line does not
+/* the following predicate returns rtrue if the line does not
    satisfy the conditions of a fixed line                     */
 
-static bool IsFixedLine (NewLine)
+static rbool IsFixedLine (NewLine)
 SourceLine *NewLine;
 
 {  int  pos, nr;
@@ -209,7 +209,7 @@ SourceLine *NewLine;
 
    if (nr > 1)
      { if (IsFixedComment (*ptr))
-          return (true);
+          return (rtrue);
      }
 
    pos = 0;
@@ -217,44 +217,44 @@ SourceLine *NewLine;
 
     {  c = *ptr++;
        if (IsAnyCharacter (c) && !IsDigit(c))
-          return (false);
+          return (rfalse);
        if (c == '\t')
           pos += 8;
        pos ++;
     }
-   return (true);
+   return (rtrue);
 
 } /* IsFixedLine */
 
 /*******************************************************************
 *                                                                  *
-* bool IsCommentLine (SourceLine *NewLine)                         *
+* rbool IsCommentLine (SourceLine *NewLine)                        *
 *                                                                  *
 *******************************************************************/
 
-static bool IsCommentLine (NewLine)
+static rbool IsCommentLine (NewLine)
 SourceLine *NewLine;
 
 {  int  pos, nr;
    char *ptr, c;
-   bool stop;
+   rbool stop;
 
    ptr = NewLine->content;
    nr = NewLine->counter;
 
-   if (nr <= 0) return (false);
+   if (nr <= 0) return (rfalse);
 
    c = *ptr;
 
    if (!IsFreeFormat)
-     { if (c == 'c') return (true);
-       if (c == 'C') return (true);
-       if (c == '*') return (true);
+     { if (c == 'c') return (rtrue);
+       if (c == 'C') return (rtrue);
+       if (c == '*') return (rtrue);
      }
 
    /* now look at the first non blank character */
 
-   pos = 0; stop = false;
+   pos = 0; stop = rfalse;
 
    while (!stop) 
 
@@ -263,20 +263,20 @@ SourceLine *NewLine;
        if (pos < nr) 
           { if (!stop) pos++; }
         else
-          stop = true;
+          stop = rtrue;
     }
    
    /* case 1 : no non-blank character found */
 
-   if (pos >= nr) return (false);
+   if (pos >= nr) return (rfalse);
 
    /* case 2 : first character is !  */
 
-   if (c == '!') return (true);
+   if (c == '!') return (rtrue);
 
    /* case 3 : first charcter is not ! */
 
-   return (false);
+   return (rfalse);
 
 } /* IsCommentLine */
 
@@ -395,7 +395,7 @@ SourceLine *Line;
 { int  i;
   int  pos;        /* pos will be position of first relevant character */
   int  nr;
-  int  stop;       /* stop becomes true at first relevant character    */
+  int  stop;       /* stop becomes rtrue at first relevant character    */
   char *ptr;
   char c;
 
@@ -500,7 +500,7 @@ int  MaxSize;
   Buffer_Counter  = 0;
   Buffer_NewLines = 0;
   Buffer_ParNest  = 0;
-  Buffer_Continue = false;
+  Buffer_Continue = rfalse;
   Buffer_State    = BUFFER_NORMAL;
   Buffer_Error    = BUFFER_NO_ERROR;
 }
@@ -516,7 +516,7 @@ SourceLine *NewLine;
 
 { char *ptr, c;
   int i, n;
-  bool found;
+  rbool found;
   int  last_pos;
 
   ptr  = NewLine->content;
@@ -541,7 +541,7 @@ SourceLine *NewLine;
     { c = *(ptr++);
 
       if (IsAnyCharacter(c))
-         Buffer_Continue = false;   /* continuation is satisfied */
+         Buffer_Continue = rfalse;   /* continuation is satisfied */
 
       if (c == '\n')
          Buffer_NewLines ++;
@@ -587,7 +587,7 @@ SourceLine *NewLine;
     /* check for Continuation '&    ' */
 
     if ((!Buffer_Continue) && (IsFreeFormat))
-     {  found    = false;
+     {  found    = rfalse;
         last_pos = Buffer_Counter-1;
         while ((last_pos >= 0) && !found)
           { found = (IsAnyCharacter (Buffer_Data[last_pos]));
@@ -596,7 +596,7 @@ SourceLine *NewLine;
         if (found) 
            Buffer_Continue = (Buffer_Data[last_pos] == '&');
           else
-           Buffer_Continue = false;
+           Buffer_Continue = rfalse;
         if (Buffer_Continue)
            Buffer_Counter = last_pos;
      }
@@ -616,7 +616,7 @@ SourceLine *NewLine;
       *                                                     *
       ******************************************************/
 
-bool LineError ()
+rbool LineError ()
 {
   return (Buffer_Error != BUFFER_NO_ERROR);
 }
