@@ -1,6 +1,13 @@
 # include "TempExp.h"
-# include "yyTempExp.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,27 +15,24 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 35 "TempExp.puma"
+/* line 35 "TempExp.puma" */
 
 
 # include "protocol.h"
@@ -64,11 +68,25 @@
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* TempExp_Exit) () = yyExit;
+# include "yyTempExp.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* TempExp_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -77,27 +95,27 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module TempExp, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module TempExp, routine %s failed\n",
+  yyFunction);
  TempExp_Exit ();
 }
 
-void MakeLocalExp ARGS((tTree exp, pvar home_vard, tTree * yyP3, tTree * yyP2, tTree * yyP1));
-void MakeHostExp ARGS((tTree exp, tTree * yyP6, tTree * yyP5, tTree * yyP4));
-void MakeReplExp ARGS((tTree exp, tTree * yyP9, tTree * yyP8, tTree * yyP7));
-static void LocalizeExp ARGS((tTree exp, pvar home_vard, tTree * yyP12, tTree * yyP11, tTree * yyP10));
-static void LocalizeVarIndexes ARGS((tTree var, pvar home_vard, tTree * yyP14, tTree * yyP13));
-void MakeLocalParams ARGS((tTree args, pvar home_vard, tTree * yyP16, tTree * yyP15));
-void MakeLocalVarParam ARGS((tTree var, int intent, pvar home_vard, tTree * yyP19, tTree * yyP18, tTree * yyP17));
-void MakeLocalVarAccess ARGS((tTree var, pvar home_vard, int intent, tTree * yyP22, tTree * yyP21, tTree * yyP20));
-void MakeFullLocalIndexes ARGS((tTree var, pvar home_vard, tTree * yyP24, tTree * yyP23));
-static bool IsSpreadAssign ARGS((pvar home_vard, tTree exp));
-static void MakeSpreadDescriptor ARGS((pvar home, tTree exp, var_descriptor * yyP25));
-static void SetParamExp ARGS((tTree param, tTree exp));
-static void MakeLocalIndex ARGS((tTree index, pvar home, tTree * yyP28, tTree * yyP27, tTree * yyP26));
-static void MyMakeTemporaryStmt ARGS((tTree exp, pvar vard, tTree * yyP32, tTree * yyP31, tTree * yyP30, tTree * yyP29));
-static bool IsFullBroadcast ARGS((tTree var));
-static int ImpliedDos ARGS(());
-static bool ImpliedDoLoop ARGS((tTree loop));
+void MakeLocalExp ARGS ((tTree exp, pvar home_vard, tTree * yyP3, tTree * yyP2, tTree * yyP1));
+void MakeHostExp ARGS ((tTree exp, tTree * yyP6, tTree * yyP5, tTree * yyP4));
+void MakeReplExp ARGS ((tTree exp, tTree * yyP9, tTree * yyP8, tTree * yyP7));
+static void LocalizeExp ARGS ((tTree exp, pvar home_vard, tTree * yyP12, tTree * yyP11, tTree * yyP10));
+static void LocalizeVarIndexes ARGS ((tTree var, pvar home_vard, tTree * yyP14, tTree * yyP13));
+void MakeLocalParams ARGS ((tTree args, pvar home_vard, tTree * yyP16, tTree * yyP15));
+void MakeLocalVarParam ARGS ((tTree var, int intent, pvar home_vard, tTree * yyP19, tTree * yyP18, tTree * yyP17));
+void MakeLocalVarAccess ARGS ((tTree var, pvar home_vard, int intent, tTree * yyP22, tTree * yyP21, tTree * yyP20));
+void MakeFullLocalIndexes ARGS ((tTree var, pvar home_vard, tTree * yyP24, tTree * yyP23));
+static rbool IsSpreadAssign ARGS ((pvar home_vard, tTree exp));
+static void SetParamExp ARGS ((tTree param, tTree exp));
+static void MakeLocalIndex ARGS ((tTree index, pvar home, tTree * yyP27, tTree * yyP26, tTree * yyP25));
+static void MyMakeTemporaryStmt ARGS ((tTree exp, pvar vard, tTree * yyP31, tTree * yyP30, tTree * yyP29, tTree * yyP28));
+static rbool IsFullBroadcast ARGS ((tTree var));
+static int ImpliedDos ARGS ((void));
+static rbool ImpliedDoLoop ARGS ((tTree loop));
 
 void MakeLocalExp
 # if defined __STDC__ | defined __cplusplus
@@ -111,13 +129,13 @@ void MakeLocalExp
  register tTree * yyP1;
 # endif
 {
-# line 90 "TempExp.puma"
+/* line 90 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 92 "TempExp.puma"
+/* line 92 "TempExp.puma" */
    LocalizeExp (exp, home_vard, & yyV1, & yyV2, & yyV3);
   }
    * yyP3 = yyV1;
@@ -140,7 +158,7 @@ void MakeHostExp
  register tTree * yyP4;
 # endif
 {
-# line 114 "TempExp.puma"
+/* line 114 "TempExp.puma" */
  {
   var_descriptor vard;
   int dist;
@@ -148,15 +166,11 @@ void MakeHostExp
   tTree yyV2;
   tTree yyV3;
   {
-# line 116 "TempExp.puma"
-
-# line 117 "TempExp.puma"
-
-# line 119 "TempExp.puma"
+/* line 119 "TempExp.puma" */
    MakeHostDescriptor (& vard);
-# line 121 "TempExp.puma"
+/* line 121 "TempExp.puma" */
    LocalizeExp (exp, & vard, & yyV1, & yyV2, & yyV3);
-# line 123 "TempExp.puma"
+/* line 123 "TempExp.puma" */
  dist = TreeReadDistribution (yyV1);
     if ((dist < -1) || (dist > 0))
        { error_protocol ("failed to make host expression");
@@ -186,7 +200,7 @@ void MakeReplExp
  register tTree * yyP7;
 # endif
 {
-# line 151 "TempExp.puma"
+/* line 151 "TempExp.puma" */
  {
   var_descriptor vard;
   int dist;
@@ -194,15 +208,11 @@ void MakeReplExp
   tTree yyV2;
   tTree yyV3;
   {
-# line 153 "TempExp.puma"
-
-# line 154 "TempExp.puma"
-
-# line 156 "TempExp.puma"
+/* line 156 "TempExp.puma" */
    MakeReplicatedDescriptor (& vard);
-# line 158 "TempExp.puma"
+/* line 158 "TempExp.puma" */
    LocalizeExp (exp, & vard, & yyV1, & yyV2, & yyV3);
-# line 160 "TempExp.puma"
+/* line 160 "TempExp.puma" */
  dist = TreeReadDistribution (yyV1);
 
     if (dist != 0)
@@ -238,35 +248,35 @@ static void LocalizeExp
 
   switch (exp->Kind) {
   case kDUMMY_EXP:
-# line 187 "TempExp.puma"
+/* line 187 "TempExp.puma" */
    * yyP12 = exp;
    * yyP11 = NoTree;
    * yyP10 = NoTree;
    return;
 
   case kCONST_EXP:
-# line 190 "TempExp.puma"
+/* line 190 "TempExp.puma" */
    * yyP12 = exp;
    * yyP11 = NoTree;
    * yyP10 = NoTree;
    return;
 
   case kBOUND_EXP:
-# line 193 "TempExp.puma"
+/* line 193 "TempExp.puma" */
    * yyP12 = exp;
    * yyP11 = NoTree;
    * yyP10 = NoTree;
    return;
 
   case kRANK_EXP:
-# line 196 "TempExp.puma"
+/* line 196 "TempExp.puma" */
    * yyP12 = exp;
    * yyP11 = NoTree;
    * yyP10 = NoTree;
    return;
 
   case kOP_EXP:
-# line 205 "TempExp.puma"
+/* line 205 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -275,11 +285,11 @@ static void LocalizeExp
   tTree yyV5;
   tTree yyV6;
   {
-# line 208 "TempExp.puma"
+/* line 208 "TempExp.puma" */
    LocalizeExp (exp->OP_EXP.OPND1, home_vard, & yyV1, & yyV2, & yyV3);
-# line 209 "TempExp.puma"
+/* line 209 "TempExp.puma" */
    LocalizeExp (exp->OP_EXP.OPND2, home_vard, & yyV4, & yyV5, & yyV6);
-# line 211 "TempExp.puma"
+/* line 211 "TempExp.puma" */
  exp->OP_EXP.OPND1 = yyV1; 
      exp->OP_EXP.OPND2 = yyV4;
    
@@ -291,13 +301,13 @@ static void LocalizeExp
  }
 
   case kOP1_EXP:
-# line 223 "TempExp.puma"
+/* line 223 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 225 "TempExp.puma"
+/* line 225 "TempExp.puma" */
    LocalizeExp (exp->OP1_EXP.OPND, home_vard, & yyV1, & yyV2, & yyV3);
   }
    * yyP12 = exp;
@@ -307,12 +317,12 @@ static void LocalizeExp
  }
 
   case kFUNC_CALL_EXP:
-# line 234 "TempExp.puma"
+/* line 234 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 242 "TempExp.puma"
+/* line 242 "TempExp.puma" */
    MakeLocalParams (exp->FUNC_CALL_EXP.FUNC_PARAMS, home_vard, & yyV1, & yyV2);
   }
    * yyP12 = exp;
@@ -322,15 +332,15 @@ static void LocalizeExp
  }
 
   case kTYPE_EXP:
-# line 251 "TempExp.puma"
+/* line 251 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 253 "TempExp.puma"
+/* line 253 "TempExp.puma" */
    LocalizeExp (exp->TYPE_EXP.ELEMENTS, home_vard, & yyV1, & yyV2, & yyV3);
-# line 255 "TempExp.puma"
+/* line 255 "TempExp.puma" */
  exp->TYPE_EXP.ELEMENTS = yyV1; 
   }
    * yyP12 = exp;
@@ -340,15 +350,15 @@ static void LocalizeExp
  }
 
   case kARRAY_EXP:
-# line 264 "TempExp.puma"
+/* line 264 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 266 "TempExp.puma"
+/* line 266 "TempExp.puma" */
    LocalizeExp (exp->ARRAY_EXP.ELEMENTS, home_vard, & yyV1, & yyV2, & yyV3);
-# line 268 "TempExp.puma"
+/* line 268 "TempExp.puma" */
  exp->ARRAY_EXP.ELEMENTS = yyV1; 
   }
    * yyP12 = exp;
@@ -358,7 +368,7 @@ static void LocalizeExp
  }
 
   case kBTE_LIST:
-# line 271 "TempExp.puma"
+/* line 271 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -367,15 +377,15 @@ static void LocalizeExp
   tTree yyV5;
   tTree yyV6;
   {
-# line 273 "TempExp.puma"
+/* line 273 "TempExp.puma" */
    LocalizeExp (exp->BTE_LIST.Elem, home_vard, & yyV1, & yyV2, & yyV3);
-# line 274 "TempExp.puma"
+/* line 274 "TempExp.puma" */
    LocalizeExp (exp->BTE_LIST.Next, home_vard, & yyV4, & yyV5, & yyV6);
-# line 276 "TempExp.puma"
+/* line 276 "TempExp.puma" */
    yyV2 = CombineACF (yyV2, yyV5);
-# line 277 "TempExp.puma"
+/* line 277 "TempExp.puma" */
    yyV3 = CombineACF (yyV6, yyV3);
-# line 279 "TempExp.puma"
+/* line 279 "TempExp.puma" */
  exp->BTE_LIST.Elem = yyV1; exp->BTE_LIST.Next = yyV4; 
   }
    * yyP12 = exp;
@@ -385,14 +395,14 @@ static void LocalizeExp
  }
 
   case kBTE_EMPTY:
-# line 282 "TempExp.puma"
+/* line 282 "TempExp.puma" */
    * yyP12 = exp;
    * yyP11 = NoTree;
    * yyP10 = NoTree;
    return;
 
   case kSLICE_EXP:
-# line 285 "TempExp.puma"
+/* line 285 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -404,17 +414,17 @@ static void LocalizeExp
   tTree yyV8;
   tTree yyV9;
   {
-# line 287 "TempExp.puma"
-   LocalizeExp (exp->SLICE_EXP.START, home_vard, & yyV1, & yyV2, & yyV3);
-# line 288 "TempExp.puma"
+/* line 287 "TempExp.puma" */
+   LocalizeExp (exp->SLICE_EXP.FIRST, home_vard, & yyV1, & yyV2, & yyV3);
+/* line 288 "TempExp.puma" */
    LocalizeExp (exp->SLICE_EXP.STOP, home_vard, & yyV4, & yyV5, & yyV6);
-# line 289 "TempExp.puma"
+/* line 289 "TempExp.puma" */
    LocalizeExp (exp->SLICE_EXP.INC, home_vard, & yyV7, & yyV8, & yyV9);
-# line 291 "TempExp.puma"
- exp->SLICE_EXP.START = yyV1; exp->SLICE_EXP.STOP = yyV4; exp->SLICE_EXP.INC = yyV7;  
-# line 293 "TempExp.puma"
+/* line 291 "TempExp.puma" */
+ exp->SLICE_EXP.FIRST = yyV1; exp->SLICE_EXP.STOP = yyV4; exp->SLICE_EXP.INC = yyV7;  
+/* line 293 "TempExp.puma" */
    yyV2 = CombineACF (yyV2, CombineACF (yyV5, yyV8));
-# line 294 "TempExp.puma"
+/* line 294 "TempExp.puma" */
    yyV3 = CombineACF (yyV9, CombineACF (yyV6, yyV3));
   }
    * yyP12 = exp;
@@ -424,7 +434,7 @@ static void LocalizeExp
  }
 
   case kDO_EXP:
-# line 304 "TempExp.puma"
+/* line 304 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -433,23 +443,23 @@ static void LocalizeExp
   tTree yyV5;
   tTree yyV6;
   {
-# line 306 "TempExp.puma"
+/* line 306 "TempExp.puma" */
    LocalizeExp (exp->DO_EXP.RANGE, home_vard, & yyV1, & yyV2, & yyV3);
-# line 308 "TempExp.puma"
+/* line 308 "TempExp.puma" */
    IncParNesting (exp);
-# line 310 "TempExp.puma"
+/* line 310 "TempExp.puma" */
    LocalizeExp (exp->DO_EXP.BODY, home_vard, & yyV4, & yyV5, & yyV6);
-# line 312 "TempExp.puma"
+/* line 312 "TempExp.puma" */
  exp->DO_EXP.BODY = yyV4; exp->DO_EXP.RANGE = yyV1; 
-# line 314 "TempExp.puma"
+/* line 314 "TempExp.puma" */
    DecParNesting (exp);
-# line 316 "TempExp.puma"
+/* line 316 "TempExp.puma" */
    ExpandNewStatements (yyV5, exp, 0);
-# line 317 "TempExp.puma"
+/* line 317 "TempExp.puma" */
    ExpandNewStatements (yyV6, exp, 0);
-# line 319 "TempExp.puma"
+/* line 319 "TempExp.puma" */
    yyV5 = CombineACF (yyV2, yyV5);
-# line 320 "TempExp.puma"
+/* line 320 "TempExp.puma" */
    yyV6 = CombineACF (yyV6, yyV3);
   }
    * yyP12 = exp;
@@ -460,22 +470,22 @@ static void LocalizeExp
 
   case kVAR_EXP:
   if (exp->VAR_EXP.V->Kind == kREMOTE_VAR) {
-# line 323 "TempExp.puma"
+/* line 323 "TempExp.puma" */
    * yyP12 = exp;
    * yyP11 = NoTree;
    * yyP10 = NoTree;
    return;
 
   }
-# line 336 "TempExp.puma"
+/* line 336 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 338 "TempExp.puma"
+/* line 338 "TempExp.puma" */
    if (! ((IsLocalVarRead (exp->VAR_EXP.V, home_vard)))) goto yyL15;
   {
-# line 340 "TempExp.puma"
+/* line 340 "TempExp.puma" */
    LocalizeVarIndexes (exp->VAR_EXP.V, home_vard, & yyV1, & yyV2);
   }
   }
@@ -486,47 +496,15 @@ static void LocalizeExp
  }
 yyL15:;
 
-# line 343 "TempExp.puma"
- {
-  var_descriptor yyV1;
-  tTree yyV2;
-  tTree yyV3;
-  tTree yyV4;
-  tTree yyV5;
-  {
-# line 345 "TempExp.puma"
-   if (! ((IsSpreadAssign (home_vard, exp)))) goto yyL16;
-  {
-# line 347 "TempExp.puma"
-   MakeSpreadDescriptor (home_vard, exp, & yyV1);
-# line 349 "TempExp.puma"
-   SetExpType (exp, & yyV1);
-# line 351 "TempExp.puma"
-   tree_protocol ("non local spread assign, temp = ", PrintableDescriptorVar (& yyV1));
-# line 354 "TempExp.puma"
-   MyMakeTemporaryStmt (exp, & yyV1, & yyV2, & yyV3, & yyV4, & yyV5);
-# line 356 "TempExp.puma"
-   yyV3 = CombineACF (yyV3, yyV4);
-# line 358 "TempExp.puma"
-   tree_protocol ("non local spread assign : \n", yyV3);
-  }
-  }
-   * yyP12 = yyV2;
-   * yyP11 = yyV3;
-   * yyP10 = yyV5;
-   return;
- }
-yyL16:;
-
-# line 371 "TempExp.puma"
+/* line 354 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 373 "TempExp.puma"
+/* line 356 "TempExp.puma" */
    MakeLocalVarAccess (exp->VAR_EXP.V, home_vard, IntentIn, & yyV1, & yyV2, & yyV3);
-# line 376 "TempExp.puma"
+/* line 359 "TempExp.puma" */
  exp->VAR_EXP.V = yyV1; 
   }
    * yyP12 = exp;
@@ -537,11 +515,11 @@ yyL16:;
 
   }
 
-# line 379 "TempExp.puma"
+/* line 362 "TempExp.puma" */
   {
-# line 381 "TempExp.puma"
+/* line 364 "TempExp.puma" */
    error_protocol ("LocalizeExp failed, could not localize computation");
-# line 382 "TempExp.puma"
+/* line 365 "TempExp.puma" */
    tree_protocol ("illegal term  : ", exp);
   }
    * yyP12 = exp;
@@ -564,12 +542,12 @@ static void LocalizeVarIndexes
 # endif
 {
   if (var->Kind == kREMOTE_VAR) {
-# line 394 "TempExp.puma"
+/* line 377 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 396 "TempExp.puma"
+/* line 379 "TempExp.puma" */
    LocalizeVarIndexes (var->REMOTE_VAR.VAR, home_vard, & yyV1, & yyV2);
   }
    * yyP14 = yyV1;
@@ -579,12 +557,12 @@ static void LocalizeVarIndexes
 
   }
   if (var->Kind == kSELECTED_VAR) {
-# line 399 "TempExp.puma"
+/* line 382 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 401 "TempExp.puma"
+/* line 384 "TempExp.puma" */
    LocalizeVarIndexes (var->SELECTED_VAR.SELEC_VAR, home_vard, & yyV1, & yyV2);
   }
    * yyP14 = yyV1;
@@ -594,7 +572,7 @@ static void LocalizeVarIndexes
 
   }
   if (var->Kind == kINDEXED_VAR) {
-# line 404 "TempExp.puma"
+/* line 387 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -602,11 +580,11 @@ static void LocalizeVarIndexes
   tTree yyV4;
   tTree yyV5;
   {
-# line 407 "TempExp.puma"
+/* line 390 "TempExp.puma" */
    LocalizeVarIndexes (var->INDEXED_VAR.IND_VAR, home_vard, & yyV1, & yyV2);
-# line 408 "TempExp.puma"
+/* line 391 "TempExp.puma" */
    LocalizeExp (var->INDEXED_VAR.IND_EXPS, home_vard, & yyV3, & yyV4, & yyV5);
-# line 410 "TempExp.puma"
+/* line 393 "TempExp.puma" */
  var->INDEXED_VAR.IND_EXPS = yyV3; 
   }
    * yyP14 = CombineACF (yyV1, yyV4);
@@ -616,7 +594,7 @@ static void LocalizeVarIndexes
 
   }
   if (var->Kind == kSUBSTRING_VAR) {
-# line 413 "TempExp.puma"
+/* line 396 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -624,11 +602,11 @@ static void LocalizeVarIndexes
   tTree yyV4;
   tTree yyV5;
   {
-# line 416 "TempExp.puma"
+/* line 399 "TempExp.puma" */
    LocalizeVarIndexes (var->SUBSTRING_VAR.IND_VAR, home_vard, & yyV1, & yyV2);
-# line 417 "TempExp.puma"
+/* line 400 "TempExp.puma" */
    LocalizeExp (var->SUBSTRING_VAR.IND_EXP, home_vard, & yyV3, & yyV4, & yyV5);
-# line 419 "TempExp.puma"
+/* line 402 "TempExp.puma" */
  var->SUBSTRING_VAR.IND_EXP = yyV3; 
   }
    * yyP14 = CombineACF (yyV1, yyV4);
@@ -638,20 +616,20 @@ static void LocalizeVarIndexes
 
   }
   if (var->Kind == kUSED_VAR) {
-# line 422 "TempExp.puma"
+/* line 405 "TempExp.puma" */
    * yyP14 = NoTree;
    * yyP13 = NoTree;
    return;
 
   }
   if (var->Kind == kLOOP_VAR) {
-# line 425 "TempExp.puma"
+/* line 408 "TempExp.puma" */
    * yyP14 = NoTree;
    * yyP13 = NoTree;
    return;
 
   }
-# line 428 "TempExp.puma"
+/* line 411 "TempExp.puma" */
    * yyP14 = NoTree;
    * yyP13 = NoTree;
    return;
@@ -671,20 +649,20 @@ void MakeLocalParams
 # endif
 {
   if (args->Kind == kBTP_LIST) {
-# line 440 "TempExp.puma"
+/* line 423 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   tTree yyV4;
   {
-# line 442 "TempExp.puma"
+/* line 425 "TempExp.puma" */
    MakeLocalParams (args->BTP_LIST.Elem, home_vard, & yyV1, & yyV2);
-# line 443 "TempExp.puma"
+/* line 426 "TempExp.puma" */
    MakeLocalParams (args->BTP_LIST.Next, home_vard, & yyV3, & yyV4);
-# line 445 "TempExp.puma"
+/* line 428 "TempExp.puma" */
    yyV1 = CombineACF (yyV1, yyV3);
-# line 446 "TempExp.puma"
+/* line 429 "TempExp.puma" */
    yyV2 = CombineACF (yyV4, yyV2);
   }
    * yyP16 = yyV1;
@@ -694,7 +672,7 @@ void MakeLocalParams
 
   }
   if (args->Kind == kBTP_EMPTY) {
-# line 449 "TempExp.puma"
+/* line 432 "TempExp.puma" */
    * yyP16 = NoTree;
    * yyP15 = NoTree;
    return;
@@ -702,15 +680,15 @@ void MakeLocalParams
   }
   if (args->Kind == kVAR_PARAM) {
   if (args->VAR_PARAM.V->Kind == kADDR) {
-# line 452 "TempExp.puma"
+/* line 435 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 454 "TempExp.puma"
+/* line 437 "TempExp.puma" */
    LocalizeExp (args->VAR_PARAM.V->ADDR.E, home_vard, & yyV1, & yyV2, & yyV3);
-# line 455 "TempExp.puma"
+/* line 438 "TempExp.puma" */
    SetParamExp (args, yyV1);
   }
    * yyP16 = yyV2;
@@ -719,15 +697,15 @@ void MakeLocalParams
  }
 
   }
-# line 458 "TempExp.puma"
+/* line 441 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 461 "TempExp.puma"
+/* line 444 "TempExp.puma" */
    MakeLocalVarParam (args->VAR_PARAM.V, args->VAR_PARAM.intent, home_vard, & yyV1, & yyV2, & yyV3);
-# line 464 "TempExp.puma"
+/* line 447 "TempExp.puma" */
  args->VAR_PARAM.V = yyV1; 
   }
    * yyP16 = yyV2;
@@ -737,31 +715,31 @@ void MakeLocalParams
 
   }
   if (args->Kind == kNO_PARAM) {
-# line 467 "TempExp.puma"
+/* line 450 "TempExp.puma" */
    * yyP16 = NoTree;
    * yyP15 = NoTree;
    return;
 
   }
   if (args->Kind == kFUNC_PARAM) {
-# line 470 "TempExp.puma"
+/* line 453 "TempExp.puma" */
    * yyP16 = NoTree;
    * yyP15 = NoTree;
    return;
 
   }
   if (args->Kind == kPROC_PARAM) {
-# line 473 "TempExp.puma"
+/* line 456 "TempExp.puma" */
    * yyP16 = NoTree;
    * yyP15 = NoTree;
    return;
 
   }
-# line 476 "TempExp.puma"
+/* line 459 "TempExp.puma" */
   {
-# line 478 "TempExp.puma"
+/* line 461 "TempExp.puma" */
    error_protocol ("MakeLocalParams failed, could not localize arguments");
-# line 479 "TempExp.puma"
+/* line 462 "TempExp.puma" */
    tree_protocol ("illegal arg  : ", args);
   }
    * yyP16 = NoTree;
@@ -785,15 +763,15 @@ void MakeLocalVarParam
 # endif
 {
   if (var->Kind == kDO_VAR) {
-# line 493 "TempExp.puma"
+/* line 476 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 495 "TempExp.puma"
+/* line 478 "TempExp.puma" */
    tree_protocol ("I/O argument will be localized : ", var);
-# line 499 "TempExp.puma"
+/* line 482 "TempExp.puma" */
    MakeLocalVarAccess (var, home_vard, intent, & yyV1, & yyV2, & yyV3);
   }
    * yyP19 = yyV1;
@@ -803,18 +781,18 @@ void MakeLocalVarParam
  }
 
   }
-# line 505 "TempExp.puma"
+/* line 488 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   tTree yyV3;
   {
-# line 507 "TempExp.puma"
+/* line 490 "TempExp.puma" */
    if (! ((TreeRank (var) == 0))) goto yyL2;
   {
-# line 509 "TempExp.puma"
+/* line 492 "TempExp.puma" */
    tree_protocol ("scalar argument will be localized : ", var);
-# line 511 "TempExp.puma"
+/* line 494 "TempExp.puma" */
    MakeLocalVarAccess (var, home_vard, intent, & yyV1, & yyV2, & yyV3);
   }
   }
@@ -825,15 +803,15 @@ void MakeLocalVarParam
  }
 yyL2:;
 
-# line 517 "TempExp.puma"
+/* line 500 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 519 "TempExp.puma"
+/* line 502 "TempExp.puma" */
    if (! ((TreeRank (var) > 0))) goto yyL3;
   {
-# line 523 "TempExp.puma"
+/* line 506 "TempExp.puma" */
    LocalizeVarIndexes (var, home_vard, & yyV1, & yyV2);
   }
   }
@@ -861,7 +839,7 @@ void MakeLocalVarAccess
 # endif
 {
   if (var->Kind == kDO_VAR) {
-# line 549 "TempExp.puma"
+/* line 543 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -870,23 +848,23 @@ void MakeLocalVarAccess
   tTree yyV5;
   tTree yyV6;
   {
-# line 551 "TempExp.puma"
+/* line 545 "TempExp.puma" */
    LocalizeExp (var->DO_VAR.RANGE, home_vard, & yyV1, & yyV2, & yyV3);
-# line 553 "TempExp.puma"
+/* line 547 "TempExp.puma" */
    IncParNesting (var);
-# line 555 "TempExp.puma"
+/* line 549 "TempExp.puma" */
    MakeLocalVarAccess (var->DO_VAR.BODY, home_vard, intent, & yyV4, & yyV5, & yyV6);
-# line 558 "TempExp.puma"
+/* line 552 "TempExp.puma" */
    DecParNesting (var);
-# line 560 "TempExp.puma"
+/* line 554 "TempExp.puma" */
  var->DO_VAR.BODY = yyV4; var->DO_VAR.RANGE = yyV1; 
-# line 562 "TempExp.puma"
+/* line 556 "TempExp.puma" */
    ExpandNewStatements (yyV5, var, 0);
-# line 563 "TempExp.puma"
+/* line 557 "TempExp.puma" */
    ExpandNewStatements (yyV6, var, 0);
-# line 565 "TempExp.puma"
+/* line 559 "TempExp.puma" */
    yyV5 = CombineACF (yyV2, yyV5);
-# line 566 "TempExp.puma"
+/* line 560 "TempExp.puma" */
    yyV6 = CombineACF (yyV6, yyV3);
   }
    * yyP22 = var;
@@ -897,7 +875,7 @@ void MakeLocalVarAccess
 
   }
   if (var->Kind == kBTV_LIST) {
-# line 569 "TempExp.puma"
+/* line 563 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -906,17 +884,17 @@ void MakeLocalVarAccess
   tTree yyV5;
   tTree yyV6;
   {
-# line 571 "TempExp.puma"
+/* line 565 "TempExp.puma" */
    MakeLocalVarAccess (var->BTV_LIST.Elem, home_vard, intent, & yyV1, & yyV2, & yyV3);
-# line 573 "TempExp.puma"
+/* line 567 "TempExp.puma" */
    MakeLocalVarAccess (var->BTV_LIST.Next, home_vard, intent, & yyV4, & yyV5, & yyV6);
-# line 576 "TempExp.puma"
+/* line 570 "TempExp.puma" */
  var->BTV_LIST.Elem = yyV1;
      var->BTV_LIST.Next = yyV4; 
    
-# line 580 "TempExp.puma"
+/* line 574 "TempExp.puma" */
    yyV2 = CombineACF (yyV2, yyV5);
-# line 581 "TempExp.puma"
+/* line 575 "TempExp.puma" */
    yyV3 = CombineACF (yyV6, yyV3);
   }
    * yyP22 = var;
@@ -927,7 +905,7 @@ void MakeLocalVarAccess
 
   }
   if (var->Kind == kBTV_EMPTY) {
-# line 584 "TempExp.puma"
+/* line 578 "TempExp.puma" */
    * yyP22 = var;
    * yyP21 = NoTree;
    * yyP20 = NoTree;
@@ -935,12 +913,12 @@ void MakeLocalVarAccess
 
   }
   if (var->Kind == kREMOTE_VAR) {
-# line 591 "TempExp.puma"
+/* line 585 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 593 "TempExp.puma"
+/* line 587 "TempExp.puma" */
    LocalizeVarIndexes (var->REMOTE_VAR.VAR, home_vard, & yyV1, & yyV2);
   }
    * yyP22 = var;
@@ -950,18 +928,18 @@ void MakeLocalVarAccess
  }
 
   }
-# line 600 "TempExp.puma"
+/* line 594 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 602 "TempExp.puma"
+/* line 596 "TempExp.puma" */
    if (! ((intent == IntentIn))) goto yyL5;
   {
-# line 603 "TempExp.puma"
+/* line 597 "TempExp.puma" */
    if (! ((IsLocalVarRead (var, home_vard)))) goto yyL5;
   {
-# line 605 "TempExp.puma"
+/* line 599 "TempExp.puma" */
    LocalizeVarIndexes (var, home_vard, & yyV1, & yyV2);
   }
   }
@@ -973,22 +951,18 @@ void MakeLocalVarAccess
  }
 yyL5:;
 
-# line 612 "TempExp.puma"
+/* line 606 "TempExp.puma" */
  {
   tTree pre_stmts;
   tTree post_stmts;
   {
-# line 614 "TempExp.puma"
-
-# line 615 "TempExp.puma"
-
-# line 617 "TempExp.puma"
+/* line 611 "TempExp.puma" */
    if (! ((intent == IntentIn))) goto yyL6;
   {
-# line 618 "TempExp.puma"
+/* line 612 "TempExp.puma" */
    if (! ((IsIndShadowVariable (var, home_vard)))) goto yyL6;
   {
-# line 622 "TempExp.puma"
+/* line 616 "TempExp.puma" */
    MakeIndShadowGet (var, & pre_stmts, & post_stmts);
   }
   }
@@ -1000,22 +974,37 @@ yyL5:;
  }
 yyL6:;
 
-# line 629 "TempExp.puma"
+/* line 623 "TempExp.puma" */
+ {
+  tTree yyV1;
+  tTree yyV2;
+  {
+/* line 625 "TempExp.puma" */
+   if (! ((IsLocalVarWrite (var, home_vard)))) goto yyL7;
+  {
+/* line 627 "TempExp.puma" */
+   LocalizeVarIndexes (var, home_vard, & yyV1, & yyV2);
+  }
+  }
+   * yyP22 = var;
+   * yyP21 = yyV1;
+   * yyP20 = yyV2;
+   return;
+ }
+yyL7:;
+
+/* line 634 "TempExp.puma" */
  {
   tTree pre_stmts;
   tTree post_stmts;
   {
-# line 631 "TempExp.puma"
-
-# line 632 "TempExp.puma"
-
-# line 634 "TempExp.puma"
-   if (! ((intent == IntentOut))) goto yyL7;
+/* line 639 "TempExp.puma" */
+   if (! ((intent == IntentOut))) goto yyL8;
   {
-# line 635 "TempExp.puma"
-   if (! ((IsIndShadowVariable (var, home_vard)))) goto yyL7;
+/* line 640 "TempExp.puma" */
+   if (! ((IsIndShadowVariable (var, home_vard)))) goto yyL8;
   {
-# line 637 "TempExp.puma"
+/* line 642 "TempExp.puma" */
    MakeIndShadowPut (var, & pre_stmts, & post_stmts);
   }
   }
@@ -1025,46 +1014,23 @@ yyL6:;
    * yyP20 = post_stmts;
    return;
  }
-yyL7:;
-
-# line 644 "TempExp.puma"
- {
-  tTree yyV1;
-  tTree yyV2;
-  {
-# line 646 "TempExp.puma"
-   if (! ((IsLocalVarWrite (var, home_vard)))) goto yyL8;
-  {
-# line 648 "TempExp.puma"
-   LocalizeVarIndexes (var, home_vard, & yyV1, & yyV2);
-  }
-  }
-   * yyP22 = var;
-   * yyP21 = yyV1;
-   * yyP20 = yyV2;
-   return;
- }
 yyL8:;
 
-# line 655 "TempExp.puma"
+/* line 649 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 657 "TempExp.puma"
+/* line 651 "TempExp.puma" */
    if (! ((IsLocalVarGlobal (var, home_vard)))) goto yyL9;
   {
-# line 658 "TempExp.puma"
-   if (! ((IsHostDescriptor (home_vard)))) goto yyL9;
-  {
-# line 662 "TempExp.puma"
+/* line 657 "TempExp.puma" */
    if (! ((IsFullBroadcast (var)))) goto yyL9;
   {
-# line 664 "TempExp.puma"
+/* line 659 "TempExp.puma" */
    LocalizeVarIndexes (var, home_vard, & yyV1, & yyV2);
   }
   }
-  }
   }
    * yyP22 = var;
    * yyP21 = yyV1;
@@ -1073,35 +1039,27 @@ yyL8:;
  }
 yyL9:;
 
-# line 674 "TempExp.puma"
+/* line 669 "TempExp.puma" */
  {
   tTree tmp_var;
   tTree pre_stmts;
   tTree post_stmts;
   var_descriptor new_vard;
   {
-# line 676 "TempExp.puma"
-
-# line 677 "TempExp.puma"
-
-# line 678 "TempExp.puma"
-
-# line 680 "TempExp.puma"
+/* line 675 "TempExp.puma" */
    if (! ((IsDescriptorVar (var)))) goto yyL10;
   {
-# line 682 "TempExp.puma"
-
-# line 684 "TempExp.puma"
+/* line 679 "TempExp.puma" */
    tree_protocol ("communication temporary needed for : ", var);
-# line 686 "TempExp.puma"
-   HomeTempVar (var, home_vard, & new_vard);
-# line 687 "TempExp.puma"
+/* line 681 "TempExp.puma" */
+   HomeTempVar (var, home_vard, intent, & new_vard);
+/* line 683 "TempExp.puma" */
    ParallelExpansion (& new_vard);
-# line 691 "TempExp.puma"
+/* line 687 "TempExp.puma" */
    SetExpType (var, & new_vard);
-# line 693 "TempExp.puma"
+/* line 689 "TempExp.puma" */
    tree_protocol ("new var access for localization ", PrintableDescriptorVar (& new_vard));
-# line 696 "TempExp.puma"
+/* line 692 "TempExp.puma" */
    MakeTempVarStmt (var, intent, & new_vard, & tmp_var, & pre_stmts, & post_stmts);
   }
   }
@@ -1113,7 +1071,7 @@ yyL9:;
 yyL10:;
 
   if (var->Kind == kINDEXED_VAR) {
-# line 700 "TempExp.puma"
+/* line 696 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -1122,27 +1080,19 @@ yyL10:;
   tTree post_stmts;
   var_descriptor vard;
   {
-# line 702 "TempExp.puma"
+/* line 698 "TempExp.puma" */
    MakeFullLocalIndexes (var->INDEXED_VAR.IND_EXPS, home_vard, & yyV1, & yyV2);
-# line 704 "TempExp.puma"
-
-# line 705 "TempExp.puma"
-
-# line 706 "TempExp.puma"
-
-# line 708 "TempExp.puma"
-
-# line 712 "TempExp.puma"
+/* line 708 "TempExp.puma" */
    FullHomeTemp (var, home_vard, & vard);
-# line 713 "TempExp.puma"
+/* line 709 "TempExp.puma" */
    ParallelExpansion (& vard);
-# line 715 "TempExp.puma"
+/* line 711 "TempExp.puma" */
    tree_protocol ("new irregular var access for localization ", PrintableDescriptorVar (& vard));
-# line 717 "TempExp.puma"
+/* line 713 "TempExp.puma" */
    MakeTempVarStmt (var, intent, & vard, & tmp_var, & pre_stmts, & post_stmts);
-# line 720 "TempExp.puma"
+/* line 716 "TempExp.puma" */
    pre_stmts = CombineACF (yyV1, pre_stmts);
-# line 721 "TempExp.puma"
+/* line 717 "TempExp.puma" */
    post_stmts = CombineACF (post_stmts, yyV2);
   }
    * yyP22 = tmp_var;
@@ -1152,11 +1102,11 @@ yyL10:;
  }
 
   }
-# line 724 "TempExp.puma"
+/* line 720 "TempExp.puma" */
   {
-# line 726 "TempExp.puma"
+/* line 722 "TempExp.puma" */
    error_protocol ("MakeLocalVarAccess failed, could not localize computation");
-# line 727 "TempExp.puma"
+/* line 723 "TempExp.puma" */
    tree_protocol ("illegal term  : ", var);
   }
    * yyP22 = var;
@@ -1179,12 +1129,12 @@ void MakeFullLocalIndexes
 # endif
 {
   if (var->Kind == kINDEXED_VAR) {
-# line 738 "TempExp.puma"
+/* line 734 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
   {
-# line 740 "TempExp.puma"
+/* line 736 "TempExp.puma" */
    MakeFullLocalIndexes (var->INDEXED_VAR.IND_EXPS, home_vard, & yyV1, & yyV2);
   }
    * yyP24 = yyV1;
@@ -1194,7 +1144,7 @@ void MakeFullLocalIndexes
 
   }
   if (var->Kind == kBTE_LIST) {
-# line 743 "TempExp.puma"
+/* line 739 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -1202,11 +1152,11 @@ void MakeFullLocalIndexes
   tTree yyV4;
   tTree yyV5;
   {
-# line 746 "TempExp.puma"
+/* line 742 "TempExp.puma" */
    MakeLocalIndex (var->BTE_LIST.Elem, home_vard, & yyV1, & yyV2, & yyV3);
-# line 747 "TempExp.puma"
+/* line 743 "TempExp.puma" */
  var->BTE_LIST.Elem = yyV1; 
-# line 748 "TempExp.puma"
+/* line 744 "TempExp.puma" */
    MakeFullLocalIndexes (var->BTE_LIST.Next, home_vard, & yyV4, & yyV5);
   }
    * yyP24 = CombineACF (yyV2, yyV4);
@@ -1216,15 +1166,15 @@ void MakeFullLocalIndexes
 
   }
   if (var->Kind == kBTE_EMPTY) {
-# line 751 "TempExp.puma"
+/* line 747 "TempExp.puma" */
    * yyP24 = NoTree;
    * yyP23 = NoTree;
    return;
 
   }
-# line 754 "TempExp.puma"
+/* line 750 "TempExp.puma" */
   {
-# line 756 "TempExp.puma"
+/* line 752 "TempExp.puma" */
    failure_protocol (MODULE, "MakeFullLocalIndexes", var);
   }
    * yyP24 = NoTree;
@@ -1234,7 +1184,7 @@ void MakeFullLocalIndexes
 ;
 }
 
-static bool IsSpreadAssign
+static rbool IsSpreadAssign
 # if defined __STDC__ | defined __cplusplus
 (pvar home_vard, register tTree exp)
 # else
@@ -1243,87 +1193,12 @@ static bool IsSpreadAssign
  register tTree exp;
 # endif
 {
-  if (exp->Kind == kVAR_EXP) {
-# line 772 "TempExp.puma"
+/* line 768 "TempExp.puma" */
   {
-# line 774 "TempExp.puma"
-   if (! ((! IsFullParLoopVar (exp->VAR_EXP.V)))) goto yyL1;
-  {
-# line 775 "TempExp.puma"
-   if (! ((! IsIndShadowVariable (exp->VAR_EXP.V, home_vard)))) goto yyL1;
-  {
-# line 776 "TempExp.puma"
-   if (! ((HasOuterParallelLoop ()))) goto yyL1;
+/* line 770 "TempExp.puma" */
+   return rfalse;
   }
-  }
-  }
-   return true;
-yyL1:;
 
-  }
-  return false;
-}
-
-static void MakeSpreadDescriptor
-# if defined __STDC__ | defined __cplusplus
-(pvar home, register tTree exp, var_descriptor * yyP25)
-# else
-(home, exp, yyP25)
- pvar home;
- register tTree exp;
- var_descriptor * yyP25;
-# endif
-{
-  if (exp->Kind == kVAR_EXP) {
-# line 792 "TempExp.puma"
- {
-  var_descriptor vard;
-  {
-# line 794 "TempExp.puma"
-
-# line 796 "TempExp.puma"
- int k, rank, pos;
-
-     vard = *home;
-
-     pos = 1;
- 
-     while (pos != 0)
- 
-       { pos = 0;
- 
-         rank = vard.formal_rank;
- 
-         for (k=0; k < rank; k++)
-            { if (FreeLoopIndex (vard.actual_shape[k][0], exp->VAR_EXP.V))
-                 pos = k+1;
-            }
- 
-        if (pos > 0) RemDimDescriptor (&vard, pos);
- 
-       } 
-
-  
-  }
-   * yyP25 = vard;
-   return;
- }
-
-  }
-# line 820 "TempExp.puma"
- {
-  var_descriptor vard;
-  {
-# line 822 "TempExp.puma"
-
-# line 824 "TempExp.puma"
-   failure_protocol (MODULE, "MakeSpreadDescriptor", exp);
-  }
-   * yyP25 = vard;
-   return;
- }
-
-;
 }
 
 static void SetParamExp
@@ -1337,34 +1212,34 @@ static void SetParamExp
 {
   if (param->Kind == kVAR_PARAM) {
   if (exp->Kind == kVAR_EXP) {
-# line 835 "TempExp.puma"
+/* line 788 "TempExp.puma" */
   {
-# line 836 "TempExp.puma"
+/* line 789 "TempExp.puma" */
  param->VAR_PARAM.V = exp->VAR_EXP.V; 
   }
    return;
 
   }
   if (param->VAR_PARAM.V->Kind == kADDR) {
-# line 839 "TempExp.puma"
+/* line 792 "TempExp.puma" */
   {
-# line 840 "TempExp.puma"
+/* line 793 "TempExp.puma" */
  param->VAR_PARAM.V->ADDR.E = exp; 
   }
    return;
 
   }
-# line 843 "TempExp.puma"
+/* line 796 "TempExp.puma" */
   {
-# line 844 "TempExp.puma"
+/* line 797 "TempExp.puma" */
  param->VAR_PARAM.V = mADDR (exp); 
   }
    return;
 
   }
-# line 847 "TempExp.puma"
+/* line 800 "TempExp.puma" */
   {
-# line 848 "TempExp.puma"
+/* line 801 "TempExp.puma" */
    failure_protocol (MODULE, "SetParamExp", param);
   }
    return;
@@ -1374,34 +1249,34 @@ static void SetParamExp
 
 static void MakeLocalIndex
 # if defined __STDC__ | defined __cplusplus
-(register tTree index, pvar home, register tTree * yyP28, register tTree * yyP27, register tTree * yyP26)
+(register tTree index, pvar home, register tTree * yyP27, register tTree * yyP26, register tTree * yyP25)
 # else
-(index, home, yyP28, yyP27, yyP26)
+(index, home, yyP27, yyP26, yyP25)
  register tTree index;
  pvar home;
- register tTree * yyP28;
  register tTree * yyP27;
  register tTree * yyP26;
+ register tTree * yyP25;
 # endif
 {
   if (index->Kind == kVAR_EXP) {
-# line 878 "TempExp.puma"
+/* line 831 "TempExp.puma" */
   {
-# line 880 "TempExp.puma"
+/* line 833 "TempExp.puma" */
    if (! ((IsFullLoopVar (index->VAR_EXP.V)))) goto yyL1;
   {
-# line 881 "TempExp.puma"
+/* line 834 "TempExp.puma" */
    if (! ((IsLocalVarWrite (index->VAR_EXP.V, home)))) goto yyL1;
   }
   }
-   * yyP28 = index;
-   * yyP27 = NoTree;
+   * yyP27 = index;
    * yyP26 = NoTree;
+   * yyP25 = NoTree;
    return;
 yyL1:;
 
   }
-# line 887 "TempExp.puma"
+/* line 840 "TempExp.puma" */
  {
   tTree yyV1;
   tTree yyV2;
@@ -1412,22 +1287,20 @@ yyL1:;
   tTree yyV6;
   tTree yyV7;
   {
-# line 891 "TempExp.puma"
+/* line 844 "TempExp.puma" */
    LocalizeExp (index, home, & yyV1, & yyV2, & yyV3);
-# line 893 "TempExp.puma"
-
-# line 895 "TempExp.puma"
+/* line 848 "TempExp.puma" */
    FullHomeTemp (index, home, & vard);
-# line 897 "TempExp.puma"
+/* line 850 "TempExp.puma" */
    MyMakeTemporaryStmt (yyV1, & vard, & yyV4, & yyV5, & yyV6, & yyV7);
-# line 899 "TempExp.puma"
+/* line 852 "TempExp.puma" */
    yyV2 = CombineACF (yyV2, CombineACF (yyV5, yyV6));
-# line 900 "TempExp.puma"
+/* line 853 "TempExp.puma" */
    yyV3 = CombineACF (yyV7, yyV3);
   }
-   * yyP28 = yyV4;
-   * yyP27 = yyV2;
-   * yyP26 = yyV3;
+   * yyP27 = yyV4;
+   * yyP26 = yyV2;
+   * yyP25 = yyV3;
    return;
  }
 
@@ -1436,48 +1309,40 @@ yyL1:;
 
 static void MyMakeTemporaryStmt
 # if defined __STDC__ | defined __cplusplus
-(register tTree exp, pvar vard, register tTree * yyP32, register tTree * yyP31, register tTree * yyP30, register tTree * yyP29)
+(register tTree exp, pvar vard, register tTree * yyP31, register tTree * yyP30, register tTree * yyP29, register tTree * yyP28)
 # else
-(exp, vard, yyP32, yyP31, yyP30, yyP29)
+(exp, vard, yyP31, yyP30, yyP29, yyP28)
  register tTree exp;
  pvar vard;
- register tTree * yyP32;
  register tTree * yyP31;
  register tTree * yyP30;
  register tTree * yyP29;
+ register tTree * yyP28;
 # endif
 {
-# line 911 "TempExp.puma"
+/* line 864 "TempExp.puma" */
  {
   tTree post_stmt;
   tTree pre_stmt;
   tTree assign;
   tTree tmp_var;
   {
-# line 913 "TempExp.puma"
-
-# line 914 "TempExp.puma"
-
-# line 915 "TempExp.puma"
-
-# line 916 "TempExp.puma"
-
-# line 918 "TempExp.puma"
+/* line 871 "TempExp.puma" */
    ParallelExpansion (vard);
-# line 919 "TempExp.puma"
+/* line 872 "TempExp.puma" */
    MakeTemporaryStmt (exp, vard, & pre_stmt, & tmp_var, & assign, & post_stmt);
   }
-   * yyP32 = mVAR_EXP (tmp_var);
-   * yyP31 = pre_stmt;
-   * yyP30 = assign;
-   * yyP29 = post_stmt;
+   * yyP31 = mVAR_EXP (tmp_var);
+   * yyP30 = pre_stmt;
+   * yyP29 = assign;
+   * yyP28 = post_stmt;
    return;
  }
 
 ;
 }
 
-static bool IsFullBroadcast
+static rbool IsFullBroadcast
 # if defined __STDC__ | defined __cplusplus
 (register tTree var)
 # else
@@ -1485,46 +1350,48 @@ static bool IsFullBroadcast
  register tTree var;
 # endif
 {
-# line 935 "TempExp.puma"
+/* line 888 "TempExp.puma" */
   {
-# line 937 "TempExp.puma"
+/* line 890 "TempExp.puma" */
    if (! ((ImpliedDos () == 0))) goto yyL1;
   }
-   return true;
+   return rtrue;
 yyL1:;
 
   if (var->Kind == kUSED_VAR) {
-# line 940 "TempExp.puma"
-   return true;
+/* line 893 "TempExp.puma" */
+   return rtrue;
 
   }
   if (var->Kind == kINDEXED_VAR) {
-# line 943 "TempExp.puma"
+/* line 896 "TempExp.puma" */
   {
-# line 945 "TempExp.puma"
-   if (! ((TreeListLength (var->INDEXED_VAR.IND_EXPS) <= ImpliedDos ()))) goto yyL3;
+/* line 900 "TempExp.puma" */
+   if (! ((TreeListLength (var->INDEXED_VAR.IND_EXPS) == GeneralIndexListLength (var)))) goto yyL3;
+  {
+/* line 902 "TempExp.puma" */
+   if (! ((GeneralIndexListLength (var) <= ImpliedDos ()))) goto yyL3;
   }
-   return true;
+  }
+   return rtrue;
 yyL3:;
 
   }
-  return false;
+  return rfalse;
 }
 
 static int ImpliedDos
 # if defined __STDC__ | defined __cplusplus
-()
+(void)
 # else
 ()
 # endif
 {
-# line 958 "TempExp.puma"
+/* line 925 "TempExp.puma" */
  {
   int n;
   {
-# line 960 "TempExp.puma"
-
-# line 962 "TempExp.puma"
+/* line 929 "TempExp.puma" */
  int loop;
 
      n = 0;
@@ -1535,14 +1402,12 @@ static int ImpliedDos
 
    
   }
-  {
    return n;
-  }
  }
 
 }
 
-static bool ImpliedDoLoop
+static rbool ImpliedDoLoop
 # if defined __STDC__ | defined __cplusplus
 (register tTree loop)
 # else
@@ -1551,22 +1416,22 @@ static bool ImpliedDoLoop
 # endif
 {
   if (loop->Kind == kDO_EXP) {
-# line 977 "TempExp.puma"
-   return true;
+/* line 944 "TempExp.puma" */
+   return rtrue;
 
   }
   if (loop->Kind == kDO_VAR) {
-# line 980 "TempExp.puma"
-   return true;
+/* line 947 "TempExp.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
-void BeginTempExp ()
+void BeginTempExp ARGS ((void))
 {
 }
 
-void CloseTempExp ()
+void CloseTempExp ARGS ((void))
 {
 }
