@@ -1,6 +1,13 @@
 # include "TempVars.h"
-# include "yyTempVars.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,27 +15,24 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 46 "TempVars.puma"
+/* line 47 "TempVars.puma" */
 
 
 # undef DEBUG
@@ -45,7 +49,7 @@
 # include "ChangeDefs.h"   /* MakeObjAllocatable              */
 # include "Expressions.h"  /* MakeConstant                    */
 # include "Descriptor.h"   /* IsTemplateDescriptor            */
-# include "SemMap.h"       /* EvalObjDistribution             */
+# include "SemMap.h"       /* EvalObjSpecMapping              */
 
 # include "Nesting.h"      /* access to outer loops           */
 
@@ -89,15 +93,29 @@ static int template_add    [MAX_DIMENSIONS];
     /* data global needed for alignment directive */
 
 static tTree global_var;
-static bool global_error;
+static rbool global_error;
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* TempVars_Exit) () = yyExit;
+# include "yyTempVars.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* TempVars_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -106,29 +124,36 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module TempVars, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module TempVars, routine %s failed\n",
+  yyFunction);
  TempVars_Exit ();
 }
 
-void MakeTempVar ARGS((tIdent name, pvar vard, tTree * yyP5, tTree * yyP4, tTree * yyP3, tTree * yyP2, tTree * yyP1));
-static tTree MakeVarDeclaration ARGS((tIdent name, int rank, pvar vard));
-static tTree MakeDeferredShape ARGS((int n));
-static tTree MakeVDType ARGS((pvar vard));
-static void MakeMemoryStmts ARGS((tIdent id, int rank, pvar vard));
-static tTree MakeExplicitShape ARGS((tTree exp));
-static tTree MakeUsedVar ARGS((tIdent id, int rank));
-static void PreCompute ARGS((pvar vard, int * temp_rank));
-static tTree TranslateActual ARGS((tTree exp));
-static void SetActual ARGS((int pos, tTree low, tTree up, tTree inc));
-static void SetFormal ARGS((int pos, tTree low, tTree up));
-static tTree MakeDistributeDecl ARGS((tIdent name, int new_rank, pvar vard));
-static tTree MakeAlignDecl ARGS((tIdent name, int new_rank, pvar vard));
-static void SetTemplateObject ARGS((tTree t, tDefinitions template));
-static tTree MakeUse ARGS((tIdent id));
-static void SetReachingInfo ARGS((tTree used_var, pvar vard));
-static void GetDistribution ARGS((tDefinitions obj, tDefinitions * yyP6));
-static void GetTemplateInfo ARGS((tTree reach_info, tTree * yyP8, tDefinitions * yyP7));
-static bool IsAlignedDescriptor ARGS((pvar vard));
+void MakeTempVar ARGS ((tIdent name, pvar vard, tTree * yyP5, tTree * yyP4, tTree * yyP3, tTree * yyP2, tTree * yyP1));
+void MakeAttTempVar ARGS ((tIdent name, pvar vard, rbool is_pointer, tTree * yyP10, tTree * yyP9, tTree * yyP8, tTree * yyP7, tTree * yyP6));
+static tTree MakeVarDeclaration ARGS ((tIdent name, int line, int rank, pvar vard));
+static tTree MakeDeferredShape ARGS ((int n));
+static tTree MakeVDType ARGS ((pvar vard));
+static void MakeMemoryStmts ARGS ((tIdent id, int rank, pvar vard, rbool is_pointer));
+static void MakeDspStmts ARGS ((tIdent id, int kind, tTree indexes, tTree * yyP12, tTree * yyP11));
+static tTree MakeExplicitShape ARGS ((tTree exp));
+static tTree MakeUsedVar ARGS ((tIdent id, int rank));
+static void PreCompute ARGS ((pvar vard, int * temp_rank));
+static tTree TranslateActual ARGS ((tTree exp));
+static void SetActual ARGS ((int pos, tTree low, tTree up, tTree inc));
+static void SetFormal ARGS ((int pos, tTree low, tTree up));
+static tTree MakeDistributeDecl ARGS ((tIdent name, int line, int new_rank, pvar vard));
+static tTree MakeAlignDecl ARGS ((tIdent name, int line, int new_rank, pvar vard));
+static void SetTemplateObject ARGS ((tTree t, tDefinitions template));
+static tTree MakeUse ARGS ((tIdent id));
+static void SetReachingInfo ARGS ((tTree used_var, pvar vard));
+static void GetSpecMapping ARGS ((tDefinitions obj, tDefinitions * yyP13));
+static void GetTemplateInfo ARGS ((tTree reach_info, tTree * yyP15, tDefinitions * yyP14));
+static rbool IsAlignedDescriptor ARGS ((pvar vard));
+static void SetObjAllocatable ARGS ((tDefinitions new_obj));
+static void SetObjPointer ARGS ((tDefinitions new_obj));
+static void MakeObjReplicated ARGS ((tDefinitions t));
+static void MakeObjNoRange ARGS ((tDefinitions t));
 
 void MakeTempVar
 # if defined __STDC__ | defined __cplusplus
@@ -144,37 +169,71 @@ void MakeTempVar
  register tTree * yyP1;
 # endif
 {
-# line 126 "TempVars.puma"
+/* line 127 "TempVars.puma" */
+ {
+  tTree yyV1;
+  tTree yyV2;
+  tTree yyV3;
+  tTree yyV4;
+  tTree yyV5;
+  {
+/* line 129 "TempVars.puma" */
+   MakeAttTempVar (name, vard, rfalse, & yyV1, & yyV2, & yyV3, & yyV4, & yyV5);
+  }
+   * yyP5 = yyV1;
+   * yyP4 = yyV2;
+   * yyP3 = yyV3;
+   * yyP2 = yyV4;
+   * yyP1 = yyV5;
+   return;
+ }
+
+;
+}
+
+void MakeAttTempVar
+# if defined __STDC__ | defined __cplusplus
+(register tIdent name, pvar vard, register rbool is_pointer, register tTree * yyP10, register tTree * yyP9, register tTree * yyP8, register tTree * yyP7, register tTree * yyP6)
+# else
+(name, vard, is_pointer, yyP10, yyP9, yyP8, yyP7, yyP6)
+ register tIdent name;
+ pvar vard;
+ register rbool is_pointer;
+ register tTree * yyP10;
+ register tTree * yyP9;
+ register tTree * yyP8;
+ register tTree * yyP7;
+ register tTree * yyP6;
+# endif
+{
+/* line 136 "TempVars.puma" */
  {
   tTree var_decl;
   tTree dist_decl;
   tDefinitions new_obj;
   tTree used_var;
+  int line;
   int yyV1;
   {
-# line 128 "TempVars.puma"
-
-# line 129 "TempVars.puma"
-
-# line 130 "TempVars.puma"
-
-# line 131 "TempVars.puma"
-
-# line 133 "TempVars.puma"
- global_error = false;
+/* line 144 "TempVars.puma" */
+ global_error = rfalse;
 
       global_var = vard->var_tree;
 
+      
+
+      line = get_protocol_line ();
+
 #ifdef DEBUG
       printf ("MakeTempVar for the following var descriptor:\n");
       PrintVarDescriptor (vard);
 #endif
 
     
-# line 144 "TempVars.puma"
+/* line 159 "TempVars.puma" */
    PreCompute (vard, & yyV1);
-# line 146 "TempVars.puma"
- var_decl  = MakeVarDeclaration (name, yyV1, vard);
+/* line 161 "TempVars.puma" */
+ var_decl  = MakeVarDeclaration (name, line, yyV1, vard);
 
 #ifdef DEBUG
       FileUnparse (stdout, var_decl); printf (" is the complete decl\n");
@@ -185,8 +244,13 @@ void MakeTempVar
       new_obj = MakeNewObject (var_decl, GetCurrentUnitObject ());
       InsertEntry (new_obj);
 
-      if (yyV1 > 0)
-         MakeObjAllocatable (new_obj);   
+      if (is_pointer)
+
+         SetObjPointer (new_obj);       
+
+       else if (yyV1 > 0)
+
+         SetObjAllocatable (new_obj);   
 
       if (yyV1 == 0) 
 
@@ -198,7 +262,7 @@ void MakeTempVar
 
          { 
 
-           dist_decl  = MakeDistributeDecl (name, yyV1, vard);
+           dist_decl  = MakeDistributeDecl (name, line, yyV1, vard);
 
 #ifdef DEBUG
            printf ("Distribution: ");
@@ -207,7 +271,7 @@ void MakeTempVar
 #endif
 
            SetDistributions (dist_decl, GetCurrentUnitObject());
-           EvalObjDistribution (new_obj);
+           EvalObjSpecMapping  (new_obj);
 
          }
 
@@ -215,7 +279,7 @@ void MakeTempVar
 
          { 
 
-           dist_decl  = MakeDistributeDecl (name, yyV1, vard);
+           dist_decl  = MakeDistributeDecl (name, line, yyV1, vard);
 
 #ifdef DEBUG
            printf ("Distribution: ");
@@ -224,7 +288,7 @@ void MakeTempVar
 #endif
 
            SetDistributions (dist_decl, GetCurrentUnitObject());
-           EvalObjDistribution (new_obj);
+           EvalObjSpecMapping  (new_obj);
 
          }
 
@@ -232,7 +296,7 @@ void MakeTempVar
 
          { 
 
-           dist_decl  = MakeAlignDecl (name, yyV1, vard);
+           dist_decl  = MakeAlignDecl (name, line, yyV1, vard);
 
 #ifdef DEBUG
            printf ("Alignment: ");
@@ -241,11 +305,13 @@ void MakeTempVar
 #endif
 
            SetAlignments (dist_decl, GetCurrentUnitObject ());
-           EvalObjDistribution (new_obj);
+           EvalObjSpecMapping  (new_obj);
 
          }
 
-      MakeMemoryStmts (name, yyV1, vard);
+      MakeObjNoRange (new_obj);
+
+      MakeMemoryStmts (name, yyV1, vard, is_pointer);
 
       used_var = MakeUsedVar (name, yyV1);
 
@@ -258,11 +324,11 @@ void MakeTempVar
 
     
   }
-   * yyP5 = used_var;
-   * yyP4 = var_decl;
-   * yyP3 = dist_decl;
-   * yyP2 = allocate_stmt;
-   * yyP1 = deallocate_stmt;
+   * yyP10 = used_var;
+   * yyP9 = var_decl;
+   * yyP8 = dist_decl;
+   * yyP7 = allocate_stmt;
+   * yyP6 = deallocate_stmt;
    return;
  }
 
@@ -271,32 +337,33 @@ void MakeTempVar
 
 static tTree MakeVarDeclaration
 # if defined __STDC__ | defined __cplusplus
-(register tIdent name, register int rank, pvar vard)
+(register tIdent name, register int line, register int rank, pvar vard)
 # else
-(name, rank, vard)
+(name, line, rank, vard)
  register tIdent name;
+ register int line;
  register int rank;
  pvar vard;
 # endif
 {
-# line 241 "TempVars.puma"
+/* line 264 "TempVars.puma" */
   {
-# line 243 "TempVars.puma"
+/* line 266 "TempVars.puma" */
    if (! ((rank == 0))) goto yyL1;
   }
-   return mVAR_DECL (name, 0, MakeVDType (vard));
+   return mVAR_DECL (name, line, MakeVDType (vard));
 yyL1:;
 
-# line 248 "TempVars.puma"
+/* line 271 "TempVars.puma" */
   {
-# line 250 "TempVars.puma"
+/* line 273 "TempVars.puma" */
    if (! ((IsTemplateDescriptor (vard)))) goto yyL2;
   }
-   return mTEMPLATE_DECL (name, 0, MakeDeferredShape (rank));
+   return mTEMPLATE_DECL (name, line, MakeDeferredShape (rank));
 yyL2:;
 
-# line 255 "TempVars.puma"
-   return mVAR_DECL (name, 0, mARRAY_TYPE (MakeDeferredShape (rank), MakeVDType (vard)));
+/* line 278 "TempVars.puma" */
+   return mVAR_DECL (name, line, mARRAY_TYPE (MakeDeferredShape (rank), MakeVDType (vard)));
 
 }
 
@@ -308,30 +375,23 @@ static tTree MakeDeferredShape
  register int n;
 # endif
 {
-# line 269 "TempVars.puma"
+/* line 292 "TempVars.puma" */
  {
   tTree type_decl;
   tTree type_index;
   {
-# line 271 "TempVars.puma"
-
-# line 272 "TempVars.puma"
-
-# line 274 "TempVars.puma"
+/* line 297 "TempVars.puma" */
  int i;
  
      type_decl = mSHAPE_EMPTY ();
  
      for (i=0; i<n; i++)
        {  type_index = mDEFERRED_SHAPE ();
-          type_index->DEFERRED_SHAPE.Overlap = mOVERLAP_SPEC (0, 0, 0, 0);
           type_decl = mSHAPE_LIST (type_index, type_decl);
        }
    
   }
-  {
    return type_decl;
-  }
  }
 
 }
@@ -344,13 +404,11 @@ static tTree MakeVDType
  pvar vard;
 # endif
 {
-# line 296 "TempVars.puma"
+/* line 318 "TempVars.puma" */
  {
   tTree result;
   {
-# line 298 "TempVars.puma"
-
-# line 300 "TempVars.puma"
+/* line 322 "TempVars.puma" */
  if (vard->type_kind == kINTEGER_TYPE)
        result = MakeIntegerType (vard->type_size);
      else if (vard->type_kind == kREAL_TYPE)
@@ -365,52 +423,47 @@ static tTree MakeVDType
        result = MakeIntegerType (default_int_size);
   
   }
-  {
    return result;
-  }
  }
 
 }
 
 static void MakeMemoryStmts
 # if defined __STDC__ | defined __cplusplus
-(register tIdent id, register int rank, pvar vard)
+(register tIdent id, register int rank, pvar vard, register rbool is_pointer)
 # else
-(id, rank, vard)
+(id, rank, vard, is_pointer)
  register tIdent id;
  register int rank;
  pvar vard;
+ register rbool is_pointer;
 # endif
 {
-# line 324 "TempVars.puma"
+/* line 349 "TempVars.puma" */
   {
-# line 326 "TempVars.puma"
+/* line 351 "TempVars.puma" */
    if (! ((rank == 0))) goto yyL1;
   {
-# line 328 "TempVars.puma"
+/* line 353 "TempVars.puma" */
    allocate_stmt = NoTree;
-# line 329 "TempVars.puma"
+/* line 354 "TempVars.puma" */
    deallocate_stmt = NoTree;
-# line 330 "TempVars.puma"
+/* line 355 "TempVars.puma" */
    reaching_allocate = NoTree;
   }
   }
    return;
 yyL1:;
 
-# line 333 "TempVars.puma"
+/* line 358 "TempVars.puma" */
  {
   tTree indexes;
   tTree stmt;
   {
-# line 335 "TempVars.puma"
+/* line 360 "TempVars.puma" */
    if (! ((IsTemplateDescriptor (vard)))) goto yyL2;
   {
-# line 337 "TempVars.puma"
-
-# line 339 "TempVars.puma"
-
-# line 341 "TempVars.puma"
+/* line 366 "TempVars.puma" */
  int i;
 
 #ifdef DEBUG
@@ -458,33 +511,50 @@ yyL1:;
  }
 yyL2:;
 
-# line 384 "TempVars.puma"
+/* line 409 "TempVars.puma" */
+ {
+  tTree yyV1;
+  tTree yyV2;
+  {
+/* line 411 "TempVars.puma" */
+   if (! ((is_pointer))) goto yyL3;
+  {
+/* line 413 "TempVars.puma" */
+   MakeDspStmts (id, kALLOCATE_DSP, MakeDeferredShape (rank), & yyV1, & yyV2);
+/* line 416 "TempVars.puma" */
+ allocate_stmt   = CombineACF (yyV1, NoTree);
+     deallocate_stmt = CombineACF (yyV2, NoTree);
+
+     reaching_allocate = NoTree;    
+   
+  }
+  }
+   return;
+ }
+yyL3:;
+
+/* line 423 "TempVars.puma" */
  {
   tTree indexes;
   tTree stmt;
+  int idim;
+  tTree yyV1;
+  tTree yyV2;
   {
-# line 386 "TempVars.puma"
-
-# line 388 "TempVars.puma"
-
-# line 390 "TempVars.puma"
- int i;
-
+/* line 429 "TempVars.puma" */
+   MakeDspStmts (id, kALLOCATE_DSP, MakeDeferredShape (rank), & yyV1, & yyV2);
+/* line 432 "TempVars.puma" */
+ 
 #ifdef DEBUG
      printf ("generate allocate/deallocate, rank = %d\n", rank);
 #endif
 
-     stmt = MakeUse (id);
-     stmt = mFREE_DSP_STMT (stmt, kALLOCATE_DSP);
-     stmt = mACF_BASIC (stmt);
-
-     deallocate_stmt = mACF_LIST (stmt, NoTree);
-
      stmt = MakeUse (id);
      stmt = mBTP_LIST (mVAR_PARAM (stmt), mBTP_EMPTY());
      stmt = mDEALLOCATE_STMT (stmt, mDUMMY_VAR ());
      stmt = mACF_BASIC (stmt);
 
+     deallocate_stmt = CombineACF (yyV2, NoTree);
      deallocate_stmt = CombineACF (stmt, deallocate_stmt);
       
 #ifdef DEBUG
@@ -494,8 +564,8 @@ yyL2:;
 #endif
 
      indexes = mBTE_EMPTY ();
-     for (i=rank-1; i>=0; i--)
-        indexes = mBTE_LIST (temp_formals[i], indexes);
+     for (idim=rank-1; idim>=0; idim--)
+        indexes = mBTE_LIST (temp_formals[idim], indexes);
 
      reaching_allocate = indexes;
 
@@ -506,12 +576,7 @@ yyL2:;
      stmt = mACF_BASIC (stmt);
 
      allocate_stmt = mACF_LIST (stmt, NoTree);
-
-     stmt = MakeUse (id);
-     stmt = mCREATE_DSP_STMT (stmt, MakeDeferredShape (rank), kALLOCATE_DSP);
-     stmt = mACF_BASIC (stmt);
-
-     allocate_stmt = CombineACF (stmt, allocate_stmt);
+     allocate_stmt = CombineACF (yyV1, allocate_stmt);
 
 #ifdef DEBUG
      printf ("allocate : ");
@@ -527,6 +592,44 @@ yyL2:;
 ;
 }
 
+static void MakeDspStmts
+# if defined __STDC__ | defined __cplusplus
+(register tIdent id, register int kind, register tTree indexes, register tTree * yyP12, register tTree * yyP11)
+# else
+(id, kind, indexes, yyP12, yyP11)
+ register tIdent id;
+ register int kind;
+ register tTree indexes;
+ register tTree * yyP12;
+ register tTree * yyP11;
+# endif
+{
+/* line 486 "TempVars.puma" */
+ {
+  tTree create_stmt;
+  tTree free_stmt;
+  {
+/* line 491 "TempVars.puma" */
+   free_stmt = MakeUse (id);
+/* line 492 "TempVars.puma" */
+   free_stmt = mFREE_DSP_STMT (free_stmt, kind);
+/* line 493 "TempVars.puma" */
+   free_stmt = mACF_BASIC (free_stmt);
+/* line 495 "TempVars.puma" */
+   create_stmt = MakeUse (id);
+/* line 496 "TempVars.puma" */
+   create_stmt = mCREATE_DSP_STMT (create_stmt, indexes, kind);
+/* line 497 "TempVars.puma" */
+   create_stmt = mACF_BASIC (create_stmt);
+  }
+   * yyP12 = create_stmt;
+   * yyP11 = free_stmt;
+   return;
+ }
+
+;
+}
+
 static tTree MakeExplicitShape
 # if defined __STDC__ | defined __cplusplus
 (register tTree exp)
@@ -536,26 +639,20 @@ static tTree MakeExplicitShape
 # endif
 {
   if (exp->Kind == kSLICE_EXP) {
-# line 452 "TempVars.puma"
+/* line 508 "TempVars.puma" */
  {
   tTree shape;
   {
-# line 454 "TempVars.puma"
-
-# line 456 "TempVars.puma"
- shape = mEXPLICIT_SHAPE (CopyTree(exp->SLICE_EXP.START), CopyTree(exp->SLICE_EXP.STOP));
-     shape->EXPLICIT_SHAPE.Overlap = mOVERLAP_SPEC (0, 0, 0, 0);
-   
+/* line 512 "TempVars.puma" */
+ shape = mEXPLICIT_SHAPE (CopyTree(exp->SLICE_EXP.FIRST), CopyTree(exp->SLICE_EXP.STOP)); 
   }
-  {
    return shape;
-  }
  }
 
   }
-# line 463 "TempVars.puma"
+/* line 517 "TempVars.puma" */
   {
-# line 464 "TempVars.puma"
+/* line 518 "TempVars.puma" */
    failure_protocol (MODULE, "MakeExplicitShape", exp);
   }
    return NoTree;
@@ -571,21 +668,19 @@ static tTree MakeUsedVar
  register int rank;
 # endif
 {
-# line 476 "TempVars.puma"
+/* line 530 "TempVars.puma" */
   {
-# line 477 "TempVars.puma"
+/* line 531 "TempVars.puma" */
    if (! ((rank == 0))) goto yyL1;
   }
    return MakeUse (id);
 yyL1:;
 
-# line 481 "TempVars.puma"
+/* line 535 "TempVars.puma" */
  {
   tTree result;
   {
-# line 483 "TempVars.puma"
-
-# line 485 "TempVars.puma"
+/* line 539 "TempVars.puma" */
  int i;
 
      result = mBTE_EMPTY ();
@@ -602,9 +697,7 @@ yyL1:;
 
    
   }
-  {
    return result;
-  }
  }
 
 }
@@ -618,23 +711,21 @@ static void PreCompute
  register int * temp_rank;
 # endif
 {
-# line 516 "TempVars.puma"
+/* line 570 "TempVars.puma" */
 
 int i, loop;
 tTree a_low, a_up, a_inc;       /* actual   extension    */
 tTree t_low, t_up, t_inc;       /* extension of temporay */
 
-# line 522 "TempVars.puma"
+/* line 576 "TempVars.puma" */
  {
   int new_rank;
   {
-# line 524 "TempVars.puma"
-
-# line 526 "TempVars.puma"
+/* line 580 "TempVars.puma" */
  int   dist;
      tTree size;
      int   tdim;
-     bool  is_aligned;
+     rbool  is_aligned;
 
      new_rank = 0;
 
@@ -731,20 +822,20 @@ static tTree TranslateActual
  register tTree exp;
 # endif
 {
-# line 622 "TempVars.puma"
+/* line 676 "TempVars.puma" */
   {
-# line 623 "TempVars.puma"
+/* line 677 "TempVars.puma" */
    if (! ((exp == NoTree))) goto yyL1;
   }
    return exp;
 yyL1:;
 
   if (exp->Kind == kLOOP_VAR) {
-# line 627 "TempVars.puma"
+/* line 681 "TempVars.puma" */
    return mVAR_EXP (exp);
 
   }
-# line 631 "TempVars.puma"
+/* line 685 "TempVars.puma" */
    return exp;
 
 }
@@ -760,25 +851,23 @@ static void SetActual
  register tTree inc;
 # endif
 {
-# line 645 "TempVars.puma"
+/* line 699 "TempVars.puma" */
   {
-# line 646 "TempVars.puma"
+/* line 700 "TempVars.puma" */
    if (! ((low == up))) goto yyL1;
   {
-# line 647 "TempVars.puma"
+/* line 701 "TempVars.puma" */
  temp_actuals[pos] = CopyTree (low); 
   }
   }
    return;
 yyL1:;
 
-# line 650 "TempVars.puma"
+/* line 704 "TempVars.puma" */
  {
   tTree inc1;
   {
-# line 652 "TempVars.puma"
-
-# line 654 "TempVars.puma"
+/* line 708 "TempVars.puma" */
  if (inc == NoTree)
          inc1 = mDUMMY_EXP ();
         else
@@ -803,19 +892,19 @@ static void SetFormal
  register tTree up;
 # endif
 {
-# line 675 "TempVars.puma"
+/* line 729 "TempVars.puma" */
   {
-# line 676 "TempVars.puma"
+/* line 730 "TempVars.puma" */
    if (! ((low == up))) goto yyL1;
   {
-# line 677 "TempVars.puma"
+/* line 731 "TempVars.puma" */
    if (! ((low != NoTree))) goto yyL1;
   {
-# line 678 "TempVars.puma"
+/* line 732 "TempVars.puma" */
    if (! ((TreeRank (low) > 0))) goto yyL1;
   {
-# line 679 "TempVars.puma"
- global_error = true; 
+/* line 733 "TempVars.puma" */
+ global_error = rtrue; 
   }
   }
   }
@@ -823,19 +912,13 @@ static void SetFormal
    return;
 yyL1:;
 
-# line 682 "TempVars.puma"
+/* line 736 "TempVars.puma" */
  {
   tTree temp_low;
   tTree temp_up;
   int local;
   {
-# line 684 "TempVars.puma"
-
-# line 685 "TempVars.puma"
-
-# line 687 "TempVars.puma"
-
-# line 689 "TempVars.puma"
+/* line 743 "TempVars.puma" */
  local = 0;
 
       if (low == NoTree)
@@ -861,40 +944,37 @@ yyL1:;
 
 static tTree MakeDistributeDecl
 # if defined __STDC__ | defined __cplusplus
-(register tIdent name, register int new_rank, pvar vard)
+(register tIdent name, register int line, register int new_rank, pvar vard)
 # else
-(name, new_rank, vard)
+(name, line, new_rank, vard)
  register tIdent name;
+ register int line;
  register int new_rank;
  pvar vard;
 # endif
 {
-# line 718 "TempVars.puma"
+/* line 774 "TempVars.puma" */
   {
-# line 720 "TempVars.puma"
+/* line 776 "TempVars.puma" */
    if (! ((vard -> topology_rank == - 1))) goto yyL1;
   }
-   return mDISTRIBUTE_DECL (name, 0, mREPL_DISTRIBUTION (0), mDEFAULT_PROCESSOR_ARRAY (mTOP_OBJ (DefaultId ())));
+   return mDISTRIBUTE_DECL (name, line, mREPL_DISTRIBUTION (0), mDEFAULT_PROCESSOR_ARRAY ());
 yyL1:;
 
-# line 729 "TempVars.puma"
+/* line 785 "TempVars.puma" */
   {
-# line 731 "TempVars.puma"
+/* line 787 "TempVars.puma" */
    if (! ((vard -> topology_rank <= 0))) goto yyL2;
   }
-   return mDISTRIBUTE_DECL (name, 0, mREPL_DISTRIBUTION (0), mDEFAULT_PROCESSOR_ARRAY (mTOP_OBJ (DefaultId ())));
+   return mDISTRIBUTE_DECL (name, line, mREPL_DISTRIBUTION (0), mDEFAULT_PROCESSOR_ARRAY ());
 yyL2:;
 
-# line 740 "TempVars.puma"
+/* line 796 "TempVars.puma" */
  {
   tTree mapping;
   tTree target;
   {
-# line 742 "TempVars.puma"
-
-# line 743 "TempVars.puma"
-
-# line 745 "TempVars.puma"
+/* line 801 "TempVars.puma" */
  int i, rank, dist;
      tTree size;
 
@@ -922,33 +1002,30 @@ yyL2:;
        } 
   
    
-# line 773 "TempVars.puma"
-   target = mDEFAULT_PROCESSOR_ARRAY (mTOP_OBJ (DefaultId ()));
-  }
-  {
-   return mDISTRIBUTE_DECL (name, 0, mNODE_DISTRIBUTION (0, mapping), target);
+/* line 829 "TempVars.puma" */
+   target = mDEFAULT_PROCESSOR_ARRAY ();
   }
+   return mDISTRIBUTE_DECL (name, line, mNODE_DISTRIBUTION (0, mapping), target);
  }
 
 }
 
 static tTree MakeAlignDecl
 # if defined __STDC__ | defined __cplusplus
-(register tIdent name, register int new_rank, pvar vard)
+(register tIdent name, register int line, register int new_rank, pvar vard)
 # else
-(name, new_rank, vard)
+(name, line, new_rank, vard)
  register tIdent name;
+ register int line;
  register int new_rank;
  pvar vard;
 # endif
 {
-# line 792 "TempVars.puma"
+/* line 852 "TempVars.puma" */
  {
   tTree align_decl;
   {
-# line 794 "TempVars.puma"
-
-# line 796 "TempVars.puma"
+/* line 856 "TempVars.puma" */
  tTree source_list, spec_list, item;
     tIdent item_id, template_id;
     int i, vdim, template_rank;
@@ -985,13 +1062,11 @@ static tTree MakeAlignDecl
 
     spec_list = mALIGN_SPEC (0, mTEMP_OBJ (DefaultId()), spec_list);
     SetTemplateObject (spec_list, template_obj);
-    align_decl = mALIGN_DECL (name, 0, source_list, spec_list);
+    align_decl = mALIGN_DECL (name, line, source_list, spec_list);
      
   
   }
-  {
    return align_decl;
-  }
  }
 
 }
@@ -1005,19 +1080,20 @@ static void SetTemplateObject
  register tDefinitions template;
 # endif
 {
+ yyRecursion:
   if (t->Kind == kALIGN_SPEC) {
-# line 841 "TempVars.puma"
+/* line 901 "TempVars.puma" */
   {
-# line 843 "TempVars.puma"
-   SetTemplateObject (t->ALIGN_SPEC.TEMPNAME, template);
+/* line 903 "TempVars.puma" */
+   t = t->ALIGN_SPEC.TEMPNAME;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kTEMP_OBJ) {
-# line 846 "TempVars.puma"
+/* line 906 "TempVars.puma" */
   {
-# line 848 "TempVars.puma"
+/* line 908 "TempVars.puma" */
  t->TEMP_OBJ.Object = template;
       t->TEMP_OBJ.Ident  = template->Object.Ident;
     
@@ -1036,20 +1112,16 @@ static tTree MakeUse
  register tIdent id;
 # endif
 {
-# line 863 "TempVars.puma"
+/* line 923 "TempVars.puma" */
  {
   tTree var;
   {
-# line 865 "TempVars.puma"
-
-# line 867 "TempVars.puma"
+/* line 927 "TempVars.puma" */
  var = mVAR_OBJ (0, id);
     var->VAR_OBJ.Object = GetGlobalObject (id);
   
   }
-  {
    return mUSED_VAR (var);
-  }
  }
 
 }
@@ -1063,47 +1135,48 @@ static void SetReachingInfo
  pvar vard;
 # endif
 {
+ yyRecursion:
   if (used_var->Kind == kINDEXED_VAR) {
-# line 881 "TempVars.puma"
+/* line 941 "TempVars.puma" */
   {
-# line 883 "TempVars.puma"
-   SetReachingInfo (used_var->INDEXED_VAR.IND_VAR, vard);
+/* line 943 "TempVars.puma" */
+   used_var = used_var->INDEXED_VAR.IND_VAR;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (used_var->Kind == kUSED_VAR) {
-# line 886 "TempVars.puma"
+/* line 946 "TempVars.puma" */
   {
-# line 888 "TempVars.puma"
-   SetReachingInfo (used_var->USED_VAR.VARNAME, vard);
+/* line 948 "TempVars.puma" */
+   used_var = used_var->USED_VAR.VARNAME;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (used_var->Kind == kVAR_OBJ) {
-# line 891 "TempVars.puma"
+/* line 951 "TempVars.puma" */
   {
-# line 893 "TempVars.puma"
+/* line 953 "TempVars.puma" */
    if (! ((VarRank (used_var->VAR_OBJ.Object) == 0))) goto yyL3;
   {
-# line 895 "TempVars.puma"
+/* line 955 "TempVars.puma" */
  used_var->VAR_OBJ.Reaching = NoTree; 
   }
   }
    return;
 yyL3:;
 
-# line 898 "TempVars.puma"
+/* line 958 "TempVars.puma" */
  {
   tDefinitions yyV1;
   {
-# line 900 "TempVars.puma"
+/* line 960 "TempVars.puma" */
    if (! ((vard -> topology_rank < 1))) goto yyL4;
   {
-# line 902 "TempVars.puma"
-   GetDistribution (used_var->VAR_OBJ.Object, & yyV1);
-# line 904 "TempVars.puma"
+/* line 962 "TempVars.puma" */
+   GetSpecMapping (used_var->VAR_OBJ.Object, & yyV1);
+/* line 964 "TempVars.puma" */
  used_var->VAR_OBJ.Reaching = mREACHING_INFO (reaching_allocate, reaching_allocate, yyV1, yyV1); 
   }
   }
@@ -1111,16 +1184,16 @@ yyL3:;
  }
 yyL4:;
 
-# line 907 "TempVars.puma"
+/* line 967 "TempVars.puma" */
  {
   tDefinitions yyV1;
   {
-# line 909 "TempVars.puma"
+/* line 969 "TempVars.puma" */
    if (! ((IsTemplateDescriptor (vard)))) goto yyL5;
   {
-# line 911 "TempVars.puma"
-   GetDistribution (used_var->VAR_OBJ.Object, & yyV1);
-# line 913 "TempVars.puma"
+/* line 971 "TempVars.puma" */
+   GetSpecMapping (used_var->VAR_OBJ.Object, & yyV1);
+/* line 973 "TempVars.puma" */
  used_var->VAR_OBJ.Reaching = mREACHING_INFO (reaching_allocate, reaching_allocate, yyV1, yyV1); 
   }
   }
@@ -1128,26 +1201,26 @@ yyL4:;
  }
 yyL5:;
 
-# line 916 "TempVars.puma"
+/* line 976 "TempVars.puma" */
  {
   tTree yyV1;
   tDefinitions yyV2;
   tDefinitions yyV3;
   {
-# line 918 "TempVars.puma"
+/* line 978 "TempVars.puma" */
    GetTemplateInfo (vard -> reach_info, & yyV1, & yyV2);
-# line 919 "TempVars.puma"
-   GetDistribution (used_var->VAR_OBJ.Object, & yyV3);
-# line 921 "TempVars.puma"
+/* line 979 "TempVars.puma" */
+   GetSpecMapping (used_var->VAR_OBJ.Object, & yyV3);
+/* line 981 "TempVars.puma" */
  used_var->VAR_OBJ.Reaching = mREACHING_INFO (reaching_allocate, yyV1, yyV3, yyV2); 
   }
    return;
  }
 
   }
-# line 925 "TempVars.puma"
+/* line 985 "TempVars.puma" */
   {
-# line 927 "TempVars.puma"
+/* line 987 "TempVars.puma" */
    failure_protocol (MODULE, "SetReachingInfo", used_var);
   }
    return;
@@ -1155,33 +1228,33 @@ yyL5:;
 ;
 }
 
-static void GetDistribution
+static void GetSpecMapping
 # if defined __STDC__ | defined __cplusplus
-(register tDefinitions obj, register tDefinitions * yyP6)
+(register tDefinitions obj, register tDefinitions * yyP13)
 # else
-(obj, yyP6)
+(obj, yyP13)
  register tDefinitions obj;
- register tDefinitions * yyP6;
+ register tDefinitions * yyP13;
 # endif
 {
   if (obj->Kind == kVarObject) {
-# line 936 "TempVars.puma"
-   * yyP6 = obj->VarObject.Dist;
+/* line 996 "TempVars.puma" */
+   * yyP13 = obj->VarObject.Dist->Mapping.spec;
    return;
 
   }
   if (obj->Kind == kTemplateObject) {
-# line 939 "TempVars.puma"
-   * yyP6 = obj->TemplateObject.Dist;
+/* line 999 "TempVars.puma" */
+   * yyP13 = obj->TemplateObject.Dist->Mapping.spec;
    return;
 
   }
-# line 942 "TempVars.puma"
+/* line 1002 "TempVars.puma" */
   {
-# line 944 "TempVars.puma"
+/* line 1004 "TempVars.puma" */
    failure_protocol (MODULE, "GetDistribuiton", obj->Object.decl);
   }
-   * yyP6 = NoDefinitions;
+   * yyP13 = NoDefinitions;
    return;
 
 ;
@@ -1189,44 +1262,44 @@ static void GetDistribution
 
 static void GetTemplateInfo
 # if defined __STDC__ | defined __cplusplus
-(register tTree reach_info, register tTree * yyP8, register tDefinitions * yyP7)
+(register tTree reach_info, register tTree * yyP15, register tDefinitions * yyP14)
 # else
-(reach_info, yyP8, yyP7)
+(reach_info, yyP15, yyP14)
  register tTree reach_info;
- register tTree * yyP8;
- register tDefinitions * yyP7;
+ register tTree * yyP15;
+ register tDefinitions * yyP14;
 # endif
 {
-# line 953 "TempVars.puma"
+/* line 1013 "TempVars.puma" */
   {
-# line 955 "TempVars.puma"
+/* line 1015 "TempVars.puma" */
    if (! ((reach_info == NoTree))) goto yyL1;
   }
-   * yyP8 = NoTree;
-   * yyP7 = NoDefinitions;
+   * yyP15 = NoTree;
+   * yyP14 = NoDefinitions;
    return;
 yyL1:;
 
   if (reach_info->Kind == kREACHING_INFO) {
-# line 958 "TempVars.puma"
-   * yyP8 = reach_info->REACHING_INFO.temp_allocate;
-   * yyP7 = reach_info->REACHING_INFO.temp_distribution;
+/* line 1018 "TempVars.puma" */
+   * yyP15 = reach_info->REACHING_INFO.temp_allocate;
+   * yyP14 = reach_info->REACHING_INFO.temp_distribution;
    return;
 
   }
-# line 961 "TempVars.puma"
+/* line 1021 "TempVars.puma" */
   {
-# line 963 "TempVars.puma"
+/* line 1023 "TempVars.puma" */
    failure_protocol (MODULE, "GetTemplateInfo", reach_info);
   }
-   * yyP8 = NoTree;
-   * yyP7 = NoDefinitions;
+   * yyP15 = NoTree;
+   * yyP14 = NoDefinitions;
    return;
 
 ;
 }
 
-static bool IsAlignedDescriptor
+static rbool IsAlignedDescriptor
 # if defined __STDC__ | defined __cplusplus
 (pvar vard)
 # else
@@ -1234,32 +1307,144 @@ static bool IsAlignedDescriptor
  pvar vard;
 # endif
 {
-# line 978 "TempVars.puma"
+/* line 1038 "TempVars.puma" */
   {
-# line 980 "TempVars.puma"
+/* line 1040 "TempVars.puma" */
    if (! ((IsTemplateDescriptor (vard)))) goto yyL1;
   {
-# line 981 "TempVars.puma"
-   return false;
+/* line 1041 "TempVars.puma" */
+   return rfalse;
   }
   }
 yyL1:;
 
-# line 984 "TempVars.puma"
+/* line 1044 "TempVars.puma" */
   {
-# line 985 "TempVars.puma"
+/* line 1045 "TempVars.puma" */
    if (! ((vard -> template_obj != NoObject))) goto yyL2;
   }
-   return true;
+   return rtrue;
 yyL2:;
 
-  return false;
+  return rfalse;
 }
 
-void BeginTempVars ()
+static void SetObjAllocatable
+# if defined __STDC__ | defined __cplusplus
+(register tDefinitions new_obj)
+# else
+(new_obj)
+ register tDefinitions new_obj;
+# endif
+{
+  if (new_obj->Kind == kVarObject) {
+/* line 1060 "TempVars.puma" */
+  {
+/* line 1062 "TempVars.puma" */
+   MakeObjAllocatable (new_obj);
+  }
+   return;
+
+  }
+;
+}
+
+static void SetObjPointer
+# if defined __STDC__ | defined __cplusplus
+(register tDefinitions new_obj)
+# else
+(new_obj)
+ register tDefinitions new_obj;
+# endif
+{
+  if (new_obj->Kind == kVarObject) {
+/* line 1067 "TempVars.puma" */
+  {
+/* line 1069 "TempVars.puma" */
+   MakeObjPointer (new_obj);
+  }
+   return;
+
+  }
+;
+}
+
+static void MakeObjReplicated
+# if defined __STDC__ | defined __cplusplus
+(register tDefinitions t)
+# else
+(t)
+ register tDefinitions t;
+# endif
+{
+  if (t->Kind == kVarObject) {
+/* line 1083 "TempVars.puma" */
+  {
+/* line 1085 "TempVars.puma" */
+ t->VarObject.Dist->Mapping.spec = mMapReplicated (); 
+  }
+   return;
+
+  }
+  if (t->Kind == kTemplateObject) {
+/* line 1088 "TempVars.puma" */
+  {
+/* line 1090 "TempVars.puma" */
+ t->TemplateObject.Dist->Mapping.spec = mMapReplicated (); 
+  }
+   return;
+
+  }
+/* line 1093 "TempVars.puma" */
+  {
+/* line 1094 "TempVars.puma" */
+   failure_protocol (MODULE, "MakeObjReplicated", t->Object.decl);
+  }
+   return;
+
+;
+}
+
+static void MakeObjNoRange
+# if defined __STDC__ | defined __cplusplus
+(register tDefinitions t)
+# else
+(t)
+ register tDefinitions t;
+# endif
+{
+  if (t->Kind == kVarObject) {
+/* line 1099 "TempVars.puma" */
+  {
+/* line 1101 "TempVars.puma" */
+ t->VarObject.Dist->Mapping.range = mRangeNone (); 
+  }
+   return;
+
+  }
+  if (t->Kind == kTemplateObject) {
+/* line 1104 "TempVars.puma" */
+  {
+/* line 1106 "TempVars.puma" */
+ t->TemplateObject.Dist->Mapping.range = mRangeNone (); 
+  }
+   return;
+
+  }
+/* line 1109 "TempVars.puma" */
+  {
+/* line 1110 "TempVars.puma" */
+   failure_protocol (MODULE, "MakeObjNoRange", t->Object.decl);
+  }
+   return;
+
+;
+}
+
+void BeginTempVars ARGS ((void))
 {
 }
 
-void CloseTempVars ()
+void CloseTempVars ARGS ((void))
 {
 }
