@@ -1,6 +1,13 @@
 # include "UnitDefs.h"
-# include "yyUnitDefs.w"
-# include "System.h"
+# ifdef __cplusplus
+extern "C" {
+# include "General.h"
+# include "rSystem.h"
+}
+# else
+# include "General.h"
+# include "rSystem.h"
+# endif
 # include <stdio.h>
 # include "Tree.h"
 # include "Definitions.h"
@@ -8,32 +15,29 @@
 # ifndef NULL
 # define NULL 0L
 # endif
-# ifndef false
-# define false 0
+# ifndef rfalse
+# define rfalse 0
 # endif
-# ifndef true
-# define true 1
+# ifndef rtrue
+# define rtrue 1
 # endif
 
 # ifdef yyInline
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
-  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
-  free += nodesize [kind]; \
-  ptr->yyHead.yyMark = 0; \
-  ptr->Kind = kind;
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = (free -= yyAlignedSize (sizeof (type))) >= start ? \
+   (tree) free : alloc (sizeof (type)); \
+  init (ptr, kind);
 # else
-# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
+# define yyALLOC(tree, free, start, alloc, type, make, ptr, kind, init) \
+  ptr = make (kind);
 # endif
 
-# define yyWrite(s) (void) fputs (s, yyf)
-# define yyWriteNl (void) fputc ('\n', yyf)
-
-# line 22 "UnitDefs.puma"
+/* line 22 "UnitDefs.puma" */
 
 
 # include "Idents.h"
 
-# include "StringMem.h"
+# include "StringM.h"
 
 # include "protocol.h"
 
@@ -55,11 +59,25 @@ static int ProgramCounter = 0;     /* counter for MAIN programs */
 
 
 
-static void yyExit () { Exit (1); }
+# ifndef yyWrite
+# define yyWrite(s) (void) fputs (s, yyf)
+# endif
+# ifndef yyWriteNl
+# define yyWriteNl (void) fputc ('\n', yyf)
+# endif
 
-void (* UnitDefs_Exit) () = yyExit;
+# include "yyUnitDefs.h"
 
+static void yyExit ARGS ((void)) { rExit (1); }
+
+void (* UnitDefs_Exit) ARGS ((void)) = yyExit;
+
+# ifdef UNIX
 static FILE * yyf = stdout;
+# else
+static FILE * yyf;
+# endif
+static rbool yyb;
 
 static void yyAbort
 # ifdef __cplusplus
@@ -68,25 +86,26 @@ static void yyAbort
  (yyFunction) char * yyFunction;
 # endif
 {
- (void) fprintf (stderr, "Error: module UnitDefs, routine %s failed\n", yyFunction);
+ (void) fprintf (stderr, "Error: module UnitDefs, routine %s failed\n",
+  yyFunction);
  UnitDefs_Exit ();
 }
 
-void UseModule ARGS((tDefinitions obj, tTree renamings));
-void UseOnlyModule ARGS((tDefinitions obj, tTree names));
-static void CheckModuleDefinitions ARGS((tDefinitions obj));
-static void EnterAllDeclarations ARGS((tDefinitions all, tTree renamings));
-static void EnterNamedDeclarations ARGS((tDefinitions all, tTree names));
-static void MakeVisible ARGS((tDefinitions obj, tIdent old_id, tIdent new_id));
-static void EnterRenameEntry ARGS((tDefinitions obj, tTree renamings, bool done));
-static void DoNotHideDummy ARGS((tDefinitions obj, tDefinitions hide_obj));
-static tDefinitions MyCopyObject ARGS((tDefinitions obj, tIdent new_id));
-void DeclareUnits ARGS((tTree t, tDefinitions father));
-static bool IsGeneric ARGS((tDefinitions obj));
-static void CheckProgram ARGS((tTree t));
-static tDefinitions MakeDummyObject ARGS((tTree t, tDefinitions unit));
-static bool IsDummyRoutine ARGS((tDefinitions obj, tTree new_decl));
-static bool IsDummyFunction ARGS((tDefinitions obj, tTree new_decl));
+void UseModule ARGS ((tDefinitions obj, tTree renamings));
+void UseOnlyModule ARGS ((tDefinitions obj, tTree names));
+static void CheckModuleDefinitions ARGS ((tDefinitions obj));
+static void EnterAllDeclarations ARGS ((tDefinitions all, tTree renamings));
+static void EnterNamedDeclarations ARGS ((tDefinitions all, tTree names));
+static void MakeVisible ARGS ((tDefinitions obj, tIdent old_id, tIdent new_id));
+static void EnterRenameEntry ARGS ((tDefinitions obj, tTree renamings, rbool done));
+static void DoNotHideDummy ARGS ((tDefinitions obj, tDefinitions hide_obj));
+static tDefinitions MyCopyObject ARGS ((tDefinitions obj, tIdent new_id));
+void DeclareUnits ARGS ((tTree t, tDefinitions father));
+static rbool IsGeneric ARGS ((tDefinitions obj));
+static void CheckProgram ARGS ((tTree t));
+static tDefinitions MakeDummyObject ARGS ((tTree t, tDefinitions unit));
+static rbool IsDummyRoutine ARGS ((tDefinitions obj, tTree new_decl));
+static rbool IsDummyFunction ARGS ((tDefinitions obj, tTree new_decl));
 
 void UseModule
 # if defined __STDC__ | defined __cplusplus
@@ -97,26 +116,26 @@ void UseModule
  register tTree renamings;
 # endif
 {
-# line 58 "UnitDefs.puma"
+/* line 58 "UnitDefs.puma" */
   {
-# line 60 "UnitDefs.puma"
+/* line 60 "UnitDefs.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   }
    return;
 yyL1:;
 
   if (obj->Kind == kModuleObject) {
-# line 63 "UnitDefs.puma"
+/* line 63 "UnitDefs.puma" */
   {
-# line 65 "UnitDefs.puma"
+/* line 65 "UnitDefs.puma" */
    CheckModuleDefinitions (obj);
-# line 66 "UnitDefs.puma"
+/* line 66 "UnitDefs.puma" */
    EnterAllDeclarations (obj->ModuleObject.Declarations, renamings);
   }
    return;
 
   }
-# line 69 "UnitDefs.puma"
+/* line 69 "UnitDefs.puma" */
    return;
 
 ;
@@ -131,26 +150,26 @@ void UseOnlyModule
  register tTree names;
 # endif
 {
-# line 80 "UnitDefs.puma"
+/* line 80 "UnitDefs.puma" */
   {
-# line 82 "UnitDefs.puma"
+/* line 82 "UnitDefs.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   }
    return;
 yyL1:;
 
   if (obj->Kind == kModuleObject) {
-# line 85 "UnitDefs.puma"
+/* line 85 "UnitDefs.puma" */
   {
-# line 87 "UnitDefs.puma"
+/* line 87 "UnitDefs.puma" */
    CheckModuleDefinitions (obj);
-# line 88 "UnitDefs.puma"
+/* line 88 "UnitDefs.puma" */
    EnterNamedDeclarations (obj->ModuleObject.Declarations, names);
   }
    return;
 
   }
-# line 91 "UnitDefs.puma"
+/* line 91 "UnitDefs.puma" */
    return;
 
 ;
@@ -165,30 +184,28 @@ static void CheckModuleDefinitions
 # endif
 {
   if (obj->Kind == kModuleObject) {
-# line 104 "UnitDefs.puma"
+/* line 104 "UnitDefs.puma" */
   {
-# line 106 "UnitDefs.puma"
+/* line 106 "UnitDefs.puma" */
    if (! ((obj->ModuleObject.decl == NoTree))) goto yyL1;
   }
    return;
 yyL1:;
 
-# line 109 "UnitDefs.puma"
+/* line 109 "UnitDefs.puma" */
  {
   tTree SaveUnit;
   {
-# line 111 "UnitDefs.puma"
+/* line 111 "UnitDefs.puma" */
    if (! ((obj->ModuleObject.uses == 0))) goto yyL2;
   {
-# line 115 "UnitDefs.puma"
-
-# line 117 "UnitDefs.puma"
+/* line 117 "UnitDefs.puma" */
    SaveUnit = GetCurrentUnit ();
-# line 119 "UnitDefs.puma"
+/* line 119 "UnitDefs.puma" */
    NestCloseUnit (SaveUnit);
-# line 120 "UnitDefs.puma"
+/* line 120 "UnitDefs.puma" */
    MakeUnitDefs (obj->ModuleObject.decl);
-# line 121 "UnitDefs.puma"
+/* line 121 "UnitDefs.puma" */
    NestOpenUnit (SaveUnit);
   }
   }
@@ -196,12 +213,12 @@ yyL1:;
  }
 yyL2:;
 
-# line 124 "UnitDefs.puma"
+/* line 124 "UnitDefs.puma" */
   {
-# line 126 "UnitDefs.puma"
+/* line 126 "UnitDefs.puma" */
    if (! ((obj->ModuleObject.uses == 1))) goto yyL3;
   {
-# line 130 "UnitDefs.puma"
+/* line 130 "UnitDefs.puma" */
    error_protocol ("cycle in USE of modules");
   }
   }
@@ -222,24 +239,24 @@ static void EnterAllDeclarations
 # endif
 {
   if (all->Kind == kENTRY_LIST) {
-# line 141 "UnitDefs.puma"
+/* line 141 "UnitDefs.puma" */
   {
-# line 143 "UnitDefs.puma"
+/* line 143 "UnitDefs.puma" */
    EnterAllDeclarations (all->ENTRY_LIST.Next, renamings);
-# line 144 "UnitDefs.puma"
-   EnterRenameEntry (all->ENTRY_LIST.Elem, renamings, false);
+/* line 144 "UnitDefs.puma" */
+   EnterRenameEntry (all->ENTRY_LIST.Elem, renamings, rfalse);
   }
    return;
 
   }
   if (all->Kind == kENTRY_EMPTY) {
-# line 147 "UnitDefs.puma"
+/* line 147 "UnitDefs.puma" */
    return;
 
   }
-# line 150 "UnitDefs.puma"
+/* line 150 "UnitDefs.puma" */
   {
-# line 151 "UnitDefs.puma"
+/* line 151 "UnitDefs.puma" */
    failure_protocol (MODULE, "EnterAllDeclarations", NoTree);
   }
    return;
@@ -256,43 +273,44 @@ static void EnterNamedDeclarations
  register tTree names;
 # endif
 {
+ yyRecursion:
   if (names->Kind == kDECL_LIST) {
-# line 162 "UnitDefs.puma"
+/* line 162 "UnitDefs.puma" */
   {
-# line 164 "UnitDefs.puma"
+/* line 164 "UnitDefs.puma" */
    EnterNamedDeclarations (all, names->DECL_LIST.Elem);
-# line 165 "UnitDefs.puma"
-   EnterNamedDeclarations (all, names->DECL_LIST.Next);
+/* line 165 "UnitDefs.puma" */
+   names = names->DECL_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (names->Kind == kDECL_EMPTY) {
-# line 168 "UnitDefs.puma"
+/* line 168 "UnitDefs.puma" */
    return;
 
   }
   if (names->Kind == kNAME_DECL) {
-# line 171 "UnitDefs.puma"
+/* line 171 "UnitDefs.puma" */
   {
-# line 173 "UnitDefs.puma"
+/* line 173 "UnitDefs.puma" */
    MakeVisible (GetDeclEntry (names->NAME_DECL.Ident, all), names->NAME_DECL.Ident, names->NAME_DECL.Ident);
   }
    return;
 
   }
   if (names->Kind == kRENAME_DECL) {
-# line 176 "UnitDefs.puma"
+/* line 176 "UnitDefs.puma" */
   {
-# line 178 "UnitDefs.puma"
+/* line 178 "UnitDefs.puma" */
    MakeVisible (GetDeclEntry (names->RENAME_DECL.oldname, all), names->RENAME_DECL.oldname, names->RENAME_DECL.Ident);
   }
    return;
 
   }
-# line 181 "UnitDefs.puma"
+/* line 181 "UnitDefs.puma" */
   {
-# line 182 "UnitDefs.puma"
+/* line 182 "UnitDefs.puma" */
    failure_protocol (MODULE, "EnterNamedDeclarations", names);
   }
    return;
@@ -310,57 +328,57 @@ static void MakeVisible
  register tIdent new_id;
 # endif
 {
-# line 193 "UnitDefs.puma"
+/* line 193 "UnitDefs.puma" */
 
 char string[60], msg[100];
 
-# line 197 "UnitDefs.puma"
+/* line 197 "UnitDefs.puma" */
   {
-# line 199 "UnitDefs.puma"
+/* line 199 "UnitDefs.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   {
-# line 201 "UnitDefs.puma"
+/* line 201 "UnitDefs.puma" */
    GetString (old_id, string);
-# line 202 "UnitDefs.puma"
+/* line 202 "UnitDefs.puma" */
    sprintf (msg, "%s not available in the MODULE", string);
-# line 203 "UnitDefs.puma"
+/* line 203 "UnitDefs.puma" */
    error_protocol (msg);
   }
   }
    return;
 yyL1:;
 
-# line 206 "UnitDefs.puma"
+/* line 206 "UnitDefs.puma" */
   {
-# line 208 "UnitDefs.puma"
+/* line 208 "UnitDefs.puma" */
    if (! ((obj->Object.private == Private))) goto yyL2;
   {
-# line 209 "UnitDefs.puma"
+/* line 209 "UnitDefs.puma" */
    GetString (old_id, string);
-# line 210 "UnitDefs.puma"
+/* line 210 "UnitDefs.puma" */
    sprintf (msg, "%s is private in the MODULE", string);
-# line 211 "UnitDefs.puma"
+/* line 211 "UnitDefs.puma" */
    error_protocol (msg);
   }
   }
    return;
 yyL2:;
 
-# line 214 "UnitDefs.puma"
+/* line 214 "UnitDefs.puma" */
   {
-# line 216 "UnitDefs.puma"
+/* line 216 "UnitDefs.puma" */
    if (! ((old_id == new_id))) goto yyL3;
   {
-# line 217 "UnitDefs.puma"
+/* line 217 "UnitDefs.puma" */
    InsertEntry (obj);
   }
   }
    return;
 yyL3:;
 
-# line 220 "UnitDefs.puma"
+/* line 220 "UnitDefs.puma" */
   {
-# line 222 "UnitDefs.puma"
+/* line 222 "UnitDefs.puma" */
    InsertEntry (MyCopyObject (obj, new_id));
   }
    return;
@@ -370,31 +388,32 @@ yyL3:;
 
 static void EnterRenameEntry
 # if defined __STDC__ | defined __cplusplus
-(register tDefinitions obj, register tTree renamings, register bool done)
+(register tDefinitions obj, register tTree renamings, register rbool done)
 # else
 (obj, renamings, done)
  register tDefinitions obj;
  register tTree renamings;
- register bool done;
+ register rbool done;
 # endif
 {
-# line 237 "UnitDefs.puma"
+ yyRecursion:
+/* line 237 "UnitDefs.puma" */
   {
-# line 239 "UnitDefs.puma"
+/* line 239 "UnitDefs.puma" */
    if (! ((obj->Object.private == Private))) goto yyL1;
   }
    return;
 yyL1:;
 
   if (renamings->Kind == kDECL_EMPTY) {
-# line 242 "UnitDefs.puma"
+/* line 242 "UnitDefs.puma" */
   {
-# line 244 "UnitDefs.puma"
+/* line 244 "UnitDefs.puma" */
    if (! ((! done))) goto yyL2;
   {
-# line 246 "UnitDefs.puma"
+/* line 246 "UnitDefs.puma" */
    DoNotHideDummy (obj, GetLocalObject (obj->Object.Ident));
-# line 248 "UnitDefs.puma"
+/* line 248 "UnitDefs.puma" */
    InsertEntry (obj);
   }
   }
@@ -404,20 +423,21 @@ yyL2:;
   }
   if (renamings->Kind == kDECL_LIST) {
   if (renamings->DECL_LIST.Elem->Kind == kRENAME_DECL) {
-# line 251 "UnitDefs.puma"
+/* line 251 "UnitDefs.puma" */
   {
-# line 253 "UnitDefs.puma"
+/* line 253 "UnitDefs.puma" */
    if (! ((obj->Object.Ident == renamings->DECL_LIST.Elem->RENAME_DECL.oldname))) goto yyL3;
   {
-# line 255 "UnitDefs.puma"
+/* line 255 "UnitDefs.puma" */
    DoNotHideDummy (obj, GetLocalObject (obj->Object.Ident));
-# line 257 "UnitDefs.puma"
+/* line 257 "UnitDefs.puma" */
    InsertEntry (MyCopyObject (obj, renamings->DECL_LIST.Elem->RENAME_DECL.Ident));
-# line 261 "UnitDefs.puma"
-   EnterRenameEntry (obj, renamings->DECL_LIST.Next, true);
+/* line 261 "UnitDefs.puma" */
+   renamings = renamings->DECL_LIST.Next;
+   done = rtrue;
+   goto yyRecursion;
   }
   }
-   return;
 yyL3:;
 
   }
@@ -434,20 +454,20 @@ static void DoNotHideDummy
  register tDefinitions hide_obj;
 # endif
 {
-# line 272 "UnitDefs.puma"
+/* line 272 "UnitDefs.puma" */
   {
-# line 274 "UnitDefs.puma"
+/* line 274 "UnitDefs.puma" */
    if (! ((hide_obj == NoObject))) goto yyL1;
   }
    return;
 yyL1:;
 
-# line 277 "UnitDefs.puma"
+/* line 277 "UnitDefs.puma" */
   {
-# line 279 "UnitDefs.puma"
+/* line 279 "UnitDefs.puma" */
    if (! ((IsVarDummy (hide_obj)))) goto yyL2;
   {
-# line 281 "UnitDefs.puma"
+/* line 281 "UnitDefs.puma" */
  error_protocol ("USE hides dummy argument");
      tree_protocol ("dummy = ", hide_obj->Object.decl);
      tree_protocol ("used  = ", obj->Object.decl);
@@ -470,30 +490,30 @@ static tDefinitions MyCopyObject
 # endif
 {
   if (obj->Kind == kVarObject) {
-# line 297 "UnitDefs.puma"
-   return mVarObject (new_id, obj->VarObject.decl, obj->VarObject.private, obj->VarObject.in, obj->VarObject.Kind, obj->VarObject.trace, obj->VarObject.tree, obj->VarObject.sequence, CopyTree (obj->VarObject.select), obj->VarObject.uses, obj->VarObject.Dist);
+/* line 297 "UnitDefs.puma" */
+   return mVarObject (new_id, obj->VarObject.decl, obj->VarObject.private, obj->VarObject.in, obj->VarObject.Kind, obj->VarObject.arr_kind, obj->VarObject.target, obj->VarObject.trace, obj->VarObject.tree, obj->VarObject.dsp_kind, obj->VarObject.sequence, CopyTree (obj->VarObject.select), CopyTree (obj->VarObject.shadow), obj->VarObject.uses, obj->VarObject.Dist);
 
   }
   if (obj->Kind == kFuncObject) {
-# line 303 "UnitDefs.puma"
+/* line 305 "UnitDefs.puma" */
    return mFuncObject (new_id, obj->FuncObject.decl, obj->FuncObject.private, obj->FuncObject.in, obj->FuncObject.Kind, obj->FuncObject.Declarations);
 
   }
   if (obj->Kind == kProcObject) {
-# line 308 "UnitDefs.puma"
+/* line 310 "UnitDefs.puma" */
    return mProcObject (new_id, obj->ProcObject.decl, obj->ProcObject.private, obj->ProcObject.in, obj->ProcObject.Kind, obj->ProcObject.Declarations);
 
   }
   if (obj->Kind == kGenericObject) {
-# line 313 "UnitDefs.puma"
+/* line 315 "UnitDefs.puma" */
    return mGenericObject (new_id, obj->GenericObject.decl, obj->GenericObject.private, obj->GenericObject.in, obj->GenericObject.Interfaces);
 
   }
-# line 318 "UnitDefs.puma"
+/* line 320 "UnitDefs.puma" */
   {
-# line 320 "UnitDefs.puma"
+/* line 322 "UnitDefs.puma" */
    error_protocol ("cannot use an entity");
-# line 321 "UnitDefs.puma"
+/* line 323 "UnitDefs.puma" */
    obj_protocol ("this entity cannot be used", obj);
   }
    return obj;
@@ -509,35 +529,36 @@ void DeclareUnits
  register tDefinitions father;
 # endif
 {
-# line 344 "UnitDefs.puma"
+/* line 346 "UnitDefs.puma" */
  
-char s[MAXID_LENGTH], msg[MAX_LINE_LENGTH]; 
+char s[MAX_ID_LENGTH], msg[MAX_LINE_LENGTH]; 
 
+ yyRecursion:
   if (t->Kind == kUNIT_LIST) {
-# line 348 "UnitDefs.puma"
+/* line 350 "UnitDefs.puma" */
   {
-# line 350 "UnitDefs.puma"
+/* line 352 "UnitDefs.puma" */
    set_protocol_stmt (t->UNIT_LIST.Elem);
-# line 351 "UnitDefs.puma"
+/* line 353 "UnitDefs.puma" */
    DeclareUnits (t->UNIT_LIST.Elem, father);
-# line 352 "UnitDefs.puma"
-   DeclareUnits (t->UNIT_LIST.Next, father);
+/* line 354 "UnitDefs.puma" */
+   t = t->UNIT_LIST.Next;
+   goto yyRecursion;
   }
-   return;
 
   }
   if (t->Kind == kUNIT_EMPTY) {
-# line 355 "UnitDefs.puma"
+/* line 357 "UnitDefs.puma" */
    return;
 
   }
   if (t->Kind == kMODULE_PROC_DECL) {
-# line 358 "UnitDefs.puma"
+/* line 360 "UnitDefs.puma" */
   {
-# line 360 "UnitDefs.puma"
+/* line 362 "UnitDefs.puma" */
    if (! ((father == NoObject))) goto yyL3;
   {
-# line 361 "UnitDefs.puma"
+/* line 363 "UnitDefs.puma" */
    error_protocol ("MODULE PROCEDURE only in generic interfaces allowed");
   }
   }
@@ -545,15 +566,13 @@ char s[MAXID_LENGTH], msg[MAX_LINE_LENGTH];
 yyL3:;
 
   if (father->Kind == kGenericObject) {
-# line 364 "UnitDefs.puma"
+/* line 366 "UnitDefs.puma" */
  {
   tDefinitions obj;
   {
-# line 368 "UnitDefs.puma"
-
-# line 370 "UnitDefs.puma"
+/* line 372 "UnitDefs.puma" */
    obj = GetGlobalObject (t->MODULE_PROC_DECL.Ident);
-# line 372 "UnitDefs.puma"
+/* line 374 "UnitDefs.puma" */
  if (obj == NoObject)
 
         { GetString (t->MODULE_PROC_DECL.Ident, s);
@@ -573,27 +592,25 @@ yyL3:;
  }
 
   }
-# line 388 "UnitDefs.puma"
+/* line 390 "UnitDefs.puma" */
   {
-# line 390 "UnitDefs.puma"
+/* line 392 "UnitDefs.puma" */
    error_protocol ("MODULE PROCEDURE only in generic interfaces allowed");
   }
    return;
 
   }
   if (Tree_IsType (t, kUNIT_NODE)) {
-# line 393 "UnitDefs.puma"
+/* line 395 "UnitDefs.puma" */
  {
   tDefinitions old_obj;
   {
-# line 395 "UnitDefs.puma"
-
-# line 397 "UnitDefs.puma"
+/* line 399 "UnitDefs.puma" */
    old_obj = GetDeclEntry (t->UNIT_NODE.Ident, GetCurrentScope ());
-# line 399 "UnitDefs.puma"
+/* line 401 "UnitDefs.puma" */
    if (! ((old_obj != NoObject))) goto yyL6;
   {
-# line 401 "UnitDefs.puma"
+/* line 403 "UnitDefs.puma" */
  if (IsDummyRoutine (old_obj, t) || IsDummyFunction (old_obj, t))
 
        { 
@@ -624,11 +641,11 @@ yyL3:;
  }
 yyL6:;
 
-# line 427 "UnitDefs.puma"
+/* line 429 "UnitDefs.puma" */
   {
-# line 429 "UnitDefs.puma"
+/* line 431 "UnitDefs.puma" */
    CheckProgram (t);
-# line 431 "UnitDefs.puma"
+/* line 433 "UnitDefs.puma" */
  t->UNIT_NODE.Object = MakeNewObject (t, father);
      InsertEntry (t->UNIT_NODE.Object);
    
@@ -636,9 +653,9 @@ yyL6:;
    return;
 
   }
-# line 436 "UnitDefs.puma"
+/* line 438 "UnitDefs.puma" */
   {
-# line 438 "UnitDefs.puma"
+/* line 440 "UnitDefs.puma" */
    failure_protocol (MODULE, "DeclareUnits", t);
   }
    return;
@@ -646,7 +663,7 @@ yyL6:;
 ;
 }
 
-static bool IsGeneric
+static rbool IsGeneric
 # if defined __STDC__ | defined __cplusplus
 (register tDefinitions obj)
 # else
@@ -654,23 +671,23 @@ static bool IsGeneric
  register tDefinitions obj;
 # endif
 {
-# line 443 "UnitDefs.puma"
+/* line 445 "UnitDefs.puma" */
   {
-# line 444 "UnitDefs.puma"
+/* line 446 "UnitDefs.puma" */
    if (! ((obj == NoObject))) goto yyL1;
   {
-# line 445 "UnitDefs.puma"
-   return false;
+/* line 447 "UnitDefs.puma" */
+   return rfalse;
   }
   }
 yyL1:;
 
   if (obj->Kind == kGenericObject) {
-# line 448 "UnitDefs.puma"
-   return true;
+/* line 450 "UnitDefs.puma" */
+   return rtrue;
 
   }
-  return false;
+  return rfalse;
 }
 
 static void CheckProgram
@@ -681,14 +698,14 @@ static void CheckProgram
  register tTree t;
 # endif
 {
-# line 459 "UnitDefs.puma"
+/* line 461 "UnitDefs.puma" */
  
 char s[50], msg[156]; 
 
   if (t->Kind == kPROGRAM_DECL) {
-# line 463 "UnitDefs.puma"
+/* line 465 "UnitDefs.puma" */
   {
-# line 465 "UnitDefs.puma"
+/* line 467 "UnitDefs.puma" */
  if (GetCurrentScope () != GetUnitEntries ())
 
        { 
@@ -727,25 +744,25 @@ static tDefinitions MakeDummyObject
 # endif
 {
   if (t->Kind == kPROC_DECL) {
-# line 490 "UnitDefs.puma"
+/* line 492 "UnitDefs.puma" */
    return mProcObject (t->PROC_DECL.Ident, t, Default, unit, DummyRoutine, mENTRY_EMPTY ());
 
   }
   if (t->Kind == kFUNC_DECL) {
-# line 496 "UnitDefs.puma"
+/* line 498 "UnitDefs.puma" */
    return mFuncObject (t->FUNC_DECL.Ident, t, Default, unit, DummyRoutine, mENTRY_EMPTY ());
 
   }
-# line 502 "UnitDefs.puma"
+/* line 504 "UnitDefs.puma" */
   {
-# line 504 "UnitDefs.puma"
+/* line 506 "UnitDefs.puma" */
    failure_protocol (MODULE, "MakeDummyObject", t);
   }
    return NoObject;
 
 }
 
-static bool IsDummyRoutine
+static rbool IsDummyRoutine
 # if defined __STDC__ | defined __cplusplus
 (register tDefinitions obj, register tTree new_decl)
 # else
@@ -758,17 +775,17 @@ static bool IsDummyRoutine
   if (obj->VarObject.decl->Kind == kVAR_PARAM_DECL) {
   if (obj->VarObject.decl->VAR_PARAM_DECL.VAL->Kind == kDUMMY_TYPE) {
   if (new_decl->Kind == kPROC_DECL) {
-# line 517 "UnitDefs.puma"
-   return true;
+/* line 519 "UnitDefs.puma" */
+   return rtrue;
 
   }
   }
   }
   }
-  return false;
+  return rfalse;
 }
 
-static bool IsDummyFunction
+static rbool IsDummyFunction
 # if defined __STDC__ | defined __cplusplus
 (register tDefinitions obj, register tTree new_decl)
 # else
@@ -781,20 +798,20 @@ static bool IsDummyFunction
   if (obj->VarObject.decl->Kind == kVAR_PARAM_DECL) {
   if (obj->VarObject.decl->VAR_PARAM_DECL.VAL->Kind == kDUMMY_TYPE) {
   if (new_decl->Kind == kFUNC_DECL) {
-# line 522 "UnitDefs.puma"
-   return true;
+/* line 524 "UnitDefs.puma" */
+   return rtrue;
 
   }
   }
   }
   }
-  return false;
+  return rfalse;
 }
 
-void BeginUnitDefs ()
+void BeginUnitDefs ARGS ((void))
 {
 }
 
-void CloseUnitDefs ()
+void CloseUnitDefs ARGS ((void))
 {
 }
