@@ -18,7 +18,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include "ratc.h"             /* bool, true, false */
+#include "ratc.h"             /* rbool, rtrue, rfalse */
 #include "global.h"
 #include <string.h>
 
@@ -36,25 +36,25 @@
 *                                                               *
 *  INPUT:       string for filename                             *
 *                                                               *
-*  OUTPUT:      true, if string is an allowed fortran filename  *
-*               false, if string isn`t a fortran filename       *
+*  OUTPUT:      rtrue, if string is an allowed fortran filename  *
+*               rfalse, if string isn`t a fortran filename       *
 *                                                               *
 ****************************************************************/
 
-bool is_source_file (string)
+static rbool is_source_file (string)
 char *string;
 {  int length;
    
    length = strlen (string);
 
-   if (strcmp(string+length-2,".f") == 0) return (true);
-   if (strcmp(string+length-3,".f9") == 0) return (true);
-   if (strcmp(string+length-4,".f90") == 0) return (true);
-   if (strcmp(string+length-4,".fcm") == 0) return (true);
-   if (strcmp(string+length-4,".hpf") == 0) return (true);
-   if (strcmp(string+length-4,".h90") == 0) return (true);
+   if (strcmp(string+length-2,".f") == 0) return (rtrue);
+   if (strcmp(string+length-3,".f9") == 0) return (rtrue);
+   if (strcmp(string+length-4,".f90") == 0) return (rtrue);
+   if (strcmp(string+length-4,".fcm") == 0) return (rtrue);
+   if (strcmp(string+length-4,".hpf") == 0) return (rtrue);
+   if (strcmp(string+length-4,".h90") == 0) return (rtrue);
 
-   return(false);
+   return(rfalse);
 }
 
 /****************************************************************
@@ -63,7 +63,7 @@ char *string;
 *                                                               *
 ****************************************************************/
 
-void print_version ()
+static void print_version ()
 
 {  printf (VERSION_STRING);
    printf ("\n");
@@ -78,7 +78,8 @@ void print_version ()
 *                                                               *
 ****************************************************************/
 
-void print_help ()
+static void print_help ()
+
 { int i;
  printf ("\n");
 
@@ -108,6 +109,11 @@ void print_help ()
    printf (" %15s     vectorization = %s\n", vec_options[i], vec_items[i]);
  printf ("  \n");
 
+ for (i=0; i<no_loopo; i++)
+   printf (" %15s     loop_parallelization = %s\n", loopo_options[i], 
+                                                    loopo_items[i]);
+ printf ("  \n");
+
  for (i=0; i<no_ddefault; i++)
    printf (" %15s     default distribution = %s\n", ddefault_options[i], 
                                                     ddefault_items[i]);
@@ -124,6 +130,10 @@ void print_help ()
  printf ("  [-G]               abbreviation for -v -debug -list\n");
  printf ("\n");
 
+ printf ("  [-w]               suppresses warning messages\n");
+ printf ("  [-noinfo]          suppresses info messages\n");
+ printf ("\n");
+
  printf ("  [-interface]       Generate Interface Specification\n");
  printf ("  [-call]            Generate Call Graph\n");
 
@@ -146,7 +156,15 @@ void print_help ()
  printf ("\n");
 } /* print_help */
 
-void print_settings ()
+/****************************************************************
+*                                                               *
+*  void print_settings ()                                       *
+*                                                               *
+*    - prints values of all global variables used for fadapt    *
+*                                                               *
+****************************************************************/
+
+static void print_settings ()
 
 { int i;
 
@@ -171,6 +189,9 @@ void print_settings ()
   printf ("Vectorization        : %s [%s]\n", vec_items[vectorization],
                                               vec_options[vectorization]);
 
+  printf ("Loop Parallelization : %s [%s]\n", loopo_items[loop_parallelization],
+                                          loopo_options[loop_parallelization]);
+
   printf ("\n");
 
   printf ("default integer size : %d\n", default_int_size);
@@ -211,7 +232,7 @@ void print_settings ()
 *                                                                        *
 *************************************************************************/
 
-int find_item (item_list, item_no, item_string)
+static int find_item (item_list, item_no, item_string)
 char *item_list[], *item_string;
 int  item_no;
 
@@ -241,6 +262,7 @@ static void set_defaults ()
   target_model          = predefined_target_model;
   sm_parallelization    = predefined_sm_parallelization;
   vectorization         = predefined_vectorization;
+  loop_parallelization  = predefined_loop_parallelization;
   ddefault_kind         = predefined_ddefault_kind;
   MaxDistributedDims    = predefined_MaxDistributedDims;
   optimization          = predefined_optimization;
@@ -259,10 +281,123 @@ static void set_defaults ()
   debug_flag      = 0;
   list_flag       = 0;
  
+  no_warnings     = 0;
+  no_messages     = 0;
+
   adapt_command   = FULL_ADAPT;
  
 } /* set_defaults */
 
+/*************************************************************************
+*                                                                        *
+*  int get_source_argument (int argc, char **argv)                       *
+*                                                                        *
+*    - find interpretation of arguments                                  *
+*    - argc is number of available arguments in argv                     *
+*    - returns number of interpreted arguments (usually 1, some 2)       *
+*                                                                        *
+*  ATTENTION: arguments here can be used for fadapt -<arg> or            *
+*             directly in the source file                                *
+*             !ADP$ FLAGS -<arg>                                         *
+*                                                                        *
+*************************************************************************/
+
+int get_source_argument (argc, argv)
+
+int argc;
+char **argv;
+
+{ char *comm;
+  int  pos;
+
+  if (argc < 1) return 0;   /* nothing  to interpret */
+
+  comm = argv[0];
+
+  if (strcmp(comm,"-free") == 0)
+
+     { fix_length = 0; return 1; }
+
+  if (strcmp(comm,"-O") == 0)
+
+     { optimization = 4; return 1; }
+
+  if (strcmp(comm,"-O1") == 0)
+
+     { optimization = 1; return 1; }
+
+  if (strcmp(comm,"-O2") == 0)
+
+     { optimization = 2; return 1; }
+
+  if (strcmp(comm,"-O3") == 0)
+
+     { optimization = 3; return 1; }
+
+  if (strcmp(comm,"-O4") == 0)
+
+     { optimization = 4; return 1; }
+
+  if (strcmp(comm,"-noopt") == 0)
+
+     { optimization = 0; return 1; }
+
+  if ( (pos=find_item (loopo_options, no_loopo, comm)) >= 0)
+
+     { loop_parallelization = pos; return 1; }
+
+  if (strcmp(comm,"-test") == 0)
+
+     { test_flag = 1; return 1; }
+
+  if (strcmp(comm,"-w") == 0)
+
+     { no_warnings = 1; return 1; }
+
+  if (strcmp(comm,"-noinfo") == 0)
+
+     { no_messages = 1; return 1; }
+
+  if ( (pos=find_item (smp_options, no_smp, comm)) >= 0)
+
+     { sm_parallelization   = pos; return 1; }
+
+  if ( (pos=find_item (vec_options, no_vec, comm)) >= 0)
+
+     { vectorization        = pos; return 1; }
+
+  if ( (pos=find_item (slanguage_options, no_slanguage, comm)) >= 0)
+
+     { source_language = pos; return 1; }
+
+  if (strcmp(comm,"-nostrict") == 0)
+
+     { semantic_check = 0; return 1; }
+
+  if (strcmp(comm,"-static") == 0)
+
+     { static_flag = 1; return 1; }
+
+  if (strcmp(comm,"-shared") == 0)
+
+     { shared_flag = 1; return 1; }
+
+  if (strcmp(comm,"-safety") == 0)
+
+     { if ((argc > 1) && (sscanf (argv[1], "%d", &safety))) return 2;
+       fprintf(stderr,"second integer argument needed for -safety\n");
+     }
+
+   if (strcmp(comm,"-fix") == 0)
+
+     { if ((argc > 1) && (sscanf (argv[1], "%d", &fix_length))) return 2;
+       fprintf(stderr,"second integer argument needed for -fix\n");
+     }
+
+  return 0;
+
+} /* get_source_argument */
+
 /*************************************************************************
 *                                                                        *
 *  eval_arg (argc, argv)                                                 *
@@ -341,50 +476,6 @@ char **argv;
 	       exit(-1);
 	     }
          }
-      else if (strcmp(comm,"-fix") == 0)
-         { if ((argc >= (i+1)) && (sscanf (argv[i+1], "%d", &fix_length)))
-             	i++;
-	     else
-	     { fprintf(stderr,"argument missing or argument type wrong\n");
-	       fprintf(stderr,"needed for argument -fix\n");
-	       exit(-1);
-	     }
-         }
-
-      else if (strcmp(comm,"-free") == 0)
-           fix_length = 0;
-
-      else if (strcmp(comm,"-test") == 0)
-           test_flag = 1;
-
-      else if (strcmp(comm,"-O") == 0)
-           optimization = 4;
-      else if (strcmp(comm,"-O1") == 0)
-           optimization = 1;
-      else if (strcmp(comm,"-O2") == 0)
-           optimization = 2;
-      else if (strcmp(comm,"-O3") == 0)
-           optimization = 3;
-      else if (strcmp(comm,"-O4") == 0)
-           optimization = 4;
-      else if (strcmp(comm,"-noopt") == 0)
-           optimization = 0;
-
-      else if (strcmp(comm,"-safety") == 0)
-         { if ((argc >= (i+1)) && (sscanf (argv[i+1], "%d", &safety)))
-             	i++;
-	     else
-	     { fprintf(stderr,"argument missing or argument type wrong\n");
-	       fprintf(stderr,"needed for argument -safety\n");
-	       exit(-1);
-	     }
-         }
-
-      else if (strcmp(comm,"-nostrict") == 0)
-           semantic_check = 0;
-
-      else if ( (pos=find_item (slanguage_options, no_slanguage, comm)) >= 0)
-           source_language = pos;
 
       else if ( (pos=find_item (tlanguage_options, no_tlanguage, comm)) >= 0)
            target_language = pos;
@@ -392,12 +483,6 @@ char **argv;
       else if ( (pos=find_item (model_options, no_model, comm)) >= 0)
            target_model         = pos;
 
-      else if ( (pos=find_item (smp_options, no_smp, comm)) >= 0)
-           sm_parallelization   = pos;
-
-      else if ( (pos=find_item (vec_options, no_vec, comm)) >= 0)
-           vectorization        = pos;
-
       else if (strcmp(comm,"-i4") == 0)
            default_int_size = 4;
       else if (strcmp(comm,"-i8") == 0)
@@ -463,12 +548,110 @@ char **argv;
            }
       else if (comm[1] == 'I')   /* is -I */
            SetIncludeDir (comm+2);
-      else { /* not recognized option */
-             no_arg += 1;
-             argv[no_arg] = argv[i];
+
+      else {int no;
+            no = get_source_argument (argc-i, argv+i);
+            if (no == 2) i++;
+             else if (no == 0)
+              { /* not recognized option */
+                no_arg += 1;
+                argv[no_arg] = argv[i];
+              }
            }
       i++;
     }
 
     return (no_arg);   /* not evaluated arguments */
 }
+
+/*************************************************************************
+*                                                                        *
+*   void GetOptions (char option_string[], int *argc, char **argv)       *
+*                                                                        *
+*************************************************************************/
+
+void GetOptions (option_string, argc, argv)
+
+char option_string [];
+int  *argc;
+char *argv[];
+
+{  int  i;
+   int  len;
+   char *str, c;
+
+   int  no;
+   int  new;
+
+   len = strlen (option_string);
+   str = option_string;
+
+   no  = 0;   /* no options found so far    */
+   new = 1;   /* start to find a new option */
+
+   for (i=0; i<len; i++, str++)
+
+     { c = *str;
+
+       if ((c == ' ') && (!new))
+
+         { /* an old option ends here */
+
+           *str = '\0'; new = 1;
+         }
+
+       else if ((c != ' ') && (new))
+
+         { /* a new option starts here */
+
+           argv[no++] = str; new = 0;
+
+         }
+
+     } /* for loop */
+
+   *argc = no;
+
+} /* GetOptions */
+
+# include "StringM.h"
+
+void EvalOptions (opt_ref)
+
+tStringRef opt_ref;
+
+{ char string [2048];
+  int i, nopt;
+  char *options [64];
+
+  StGetString (opt_ref, string);
+  printf ("new flags for fadapt : %s\n", string);
+
+  GetOptions (string, &nopt, options);
+
+  if (nopt > 64)
+
+     { printf ("too many options in source file\n");
+       exit (-1);
+     }
+
+  i = 0;
+
+  while (i < nopt)
+
+    { int no;
+ 
+      no = get_source_argument (nopt - i, options + i);
+
+      if (no > 0)
+ 
+        { i += no;  /* we could identify the argument */ }
+
+       else
+
+        { printf ("unrecognized option in source file : %s\n", options[i]);
+          i++;
+        }
+    }
+
+} /* EvalOptions */
