@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* AdaptAnalysis_Exit) ();
+extern void (* AdaptAnalysis_Exit) ARGS ((void));
 
-extern void AdaptAnalysis ARGS((tTree t));
+extern void AdaptAnalysis ARGS ((tTree t));
 
-extern void BeginAdaptAnalysis ();
-extern void CloseAdaptAnalysis ();
+extern void BeginAdaptAnalysis ARGS ((void));
+extern void CloseAdaptAnalysis ARGS ((void));
 
 # endif
