@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* AnaOverlap_Exit) ();
+extern void (* AnaOverlap_Exit) ARGS ((void));
 
-extern void OverlapAnalysis ARGS((tTree t));
+extern void OverlapAnalysis ARGS ((tTree t));
 
-extern void BeginAnaOverlap ();
-extern void CloseAnaOverlap ();
+extern void BeginAnaOverlap ARGS ((void));
+extern void CloseAnaOverlap ARGS ((void));
 
 # endif
