@@ -7,25 +7,25 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
-# line 25 "CheckHome.puma"
+/* line 25 "CheckHome.puma" */
 
 
 #include "VarDescriptor.h"
 
 
 
-extern void (* CheckHome_Exit) ();
+extern void (* CheckHome_Exit) ARGS ((void));
 
-extern void CheckHome ARGS((tTree t));
+extern void CheckHome ARGS ((tTree t));
 
-extern void BeginCheckHome ();
-extern void CloseCheckHome ();
+extern void BeginCheckHome ARGS ((void));
+extern void CloseCheckHome ARGS ((void));
 
 # endif
