@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* Classification_Exit) ();
+extern void (* Classification_Exit) ARGS ((void));
 
-extern void Classification ARGS((tTree t));
+extern void Classification ARGS ((tTree t));
 
-extern void BeginClassification ();
-extern void CloseClassification ();
+extern void BeginClassification ARGS ((void));
+extern void CloseClassification ARGS ((void));
 
 # endif
