@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* CodeGeneration_Exit) ();
+extern void (* CodeGeneration_Exit) ARGS ((void));
 
-extern void NodeAdapt ARGS((tTree t));
+extern void NodeAdapt ARGS ((tTree t));
 
-extern void BeginCodeGeneration ();
-extern void CloseCodeGeneration ();
+extern void BeginCodeGeneration ARGS ((void));
+extern void CloseCodeGeneration ARGS ((void));
 
 # endif
