@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* CodeMovement_Exit) ();
+extern void (* CodeMovement_Exit) ARGS ((void));
 
-extern tTree CodeMovement ARGS((tTree t));
+extern tTree CodeMovement ARGS ((tTree t));
 
-extern void BeginCodeMovement ();
-extern void CloseCodeMovement ();
+extern void BeginCodeMovement ARGS ((void));
+extern void CloseCodeMovement ARGS ((void));
 
 # endif
