@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* ControlFlow_Exit) ();
+extern void (* ControlFlow_Exit) ARGS ((void));
 
-extern void ControlFlow ARGS((tTree t));
+extern void ControlFlow ARGS ((tTree t));
 
-extern void BeginControlFlow ();
-extern void CloseControlFlow ();
+extern void BeginControlFlow ARGS ((void));
+extern void CloseControlFlow ARGS ((void));
 
 # endif
