@@ -7,14 +7,14 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
-# line 41 "Dependences.puma"
+/* line 53 "Dependences.puma" */
 
 
 #include "pred.h"
@@ -25,11 +25,11 @@ typedef PredVector* pPredVector;
 
 
 
-extern void (* Dependences_Exit) ();
+extern void (* Dependences_Exit) ARGS ((void));
 
-extern void Dependences ARGS((tTree e1, ptree loops1, int n1, tTree e2, ptree loops2, int n2, int CommonLoops, int ConstLoops, pPredVector p));
+extern void Dependences ARGS ((tTree e1, ptree loops1, int n1, tTree e2, ptree loops2, int n2, int CommonLoops, int ConstLoops, ptree params, int np, pPredVector p));
 
-extern void BeginDependences ();
-extern void CloseDependences ();
+extern void BeginDependences ARGS ((void));
+extern void CloseDependences ARGS ((void));
 
 # endif
