@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* FinalCode_Exit) ();
+extern void (* FinalCode_Exit) ARGS ((void));
 
-extern void FinalCode ARGS((tTree t));
+extern void FinalCode ARGS ((tTree t));
 
-extern void BeginFinalCode ();
-extern void CloseFinalCode ();
+extern void BeginFinalCode ARGS ((void));
+extern void CloseFinalCode ARGS ((void));
 
 # endif
