@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* GlobalComm_Exit) ();
+extern void (* GlobalComm_Exit) ARGS ((void));
 
-extern tTree GlobalCommunication ARGS((tTree stmt));
+extern tTree GlobalCommunication ARGS ((tTree stmt));
 
-extern void BeginGlobalComm ();
-extern void CloseGlobalComm ();
+extern void BeginGlobalComm ARGS ((void));
+extern void CloseGlobalComm ARGS ((void));
 
 # endif
