@@ -7,14 +7,14 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
-# line 15 "HomeTraverse.puma"
+/* line 15 "HomeTraverse.puma" */
 
 
 # include "VarDescriptor.h"
@@ -23,11 +23,11 @@ typedef tTree UserFunction1 ();
 
 
 
-extern void (* HomeTraverse_Exit) ();
+extern void (* HomeTraverse_Exit) ARGS ((void));
 
-extern tTree HomeReplaceAST ARGS((tTree t, pvar home, UserFunction1 visit));
+extern tTree HomeReplaceAST ARGS ((tTree t, pvar home, UserFunction1 visit));
 
-extern void BeginHomeTraverse ();
-extern void CloseHomeTraverse ();
+extern void BeginHomeTraverse ARGS ((void));
+extern void CloseHomeTraverse ARGS ((void));
 
 # endif
