@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* ImplMapping_Exit) ();
+extern void (* ImplMapping_Exit) ARGS ((void));
 
-extern void ImplMapping ARGS((tTree t));
+extern void ImplMapping ARGS ((tTree t));
 
-extern void BeginImplMapping ();
-extern void CloseImplMapping ();
+extern void BeginImplMapping ARGS ((void));
+extern void CloseImplMapping ARGS ((void));
 
 # endif
