@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* Independence_Exit) ();
+extern void (* Independence_Exit) ARGS ((void));
 
-extern void CheckIndependence ARGS((tTree t));
+extern void CheckIndependence ARGS ((tTree t));
 
-extern void BeginIndependence ();
-extern void CloseIndependence ();
+extern void BeginIndependence ARGS ((void));
+extern void CloseIndependence ARGS ((void));
 
 # endif
