@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* InitAllocate_Exit) ();
+extern void (* InitAllocate_Exit) ARGS ((void));
 
-extern void InitAllocate ARGS((tTree t, tTree unit));
+extern void InitAllocate ARGS ((tTree t, tTree unit));
 
-extern void BeginInitAllocate ();
-extern void CloseInitAllocate ();
+extern void BeginInitAllocate ARGS ((void));
+extern void CloseInitAllocate ARGS ((void));
 
 # endif
