@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* Inquiry_Exit) ();
+extern void (* Inquiry_Exit) ARGS ((void));
 
-extern tTree TranslateInquiryCall ARGS((tTree f));
+extern tTree TranslateInquiryCall ARGS ((tTree f));
 
-extern void BeginInquiry ();
-extern void CloseInquiry ();
+extern void BeginInquiry ARGS ((void));
+extern void CloseInquiry ARGS ((void));
 
 # endif
