@@ -7,8 +7,8 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
@@ -16,11 +16,11 @@
 # include "CallGraph.h"
 
 
-extern void (* InterProc_Exit) ();
+extern void (* InterProc_Exit) ARGS ((void));
 
-extern void Interprocedural ARGS((tTree t));
+extern void Interprocedural ARGS ((tTree t));
 
-extern void BeginInterProc ();
-extern void CloseInterProc ();
+extern void BeginInterProc ARGS ((void));
+extern void CloseInterProc ARGS ((void));
 
 # endif
