@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* Interchange_Exit) ();
+extern void (* Interchange_Exit) ARGS ((void));
 
-extern bool CanBeInterchanged ARGS((tTree loop1));
+extern rbool CanBeInterchanged ARGS ((tTree loop1));
 
-extern void BeginInterchange ();
-extern void CloseInterchange ();
+extern void BeginInterchange ARGS ((void));
+extern void CloseInterchange ARGS ((void));
 
 # endif
