@@ -7,25 +7,25 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
-# line 28 "MakeArguments.puma"
+/* line 28 "MakeArguments.puma" */
 
 
 #include "VarDescriptor.h"
 
 
 
-extern void (* MakeArguments_Exit) ();
+extern void (* MakeArguments_Exit) ARGS ((void));
 
-extern void MakeArguments ARGS((tTree t));
+extern void MakeArguments ARGS ((tTree t));
 
-extern void BeginMakeArguments ();
-extern void CloseMakeArguments ();
+extern void BeginMakeArguments ARGS ((void));
+extern void CloseMakeArguments ARGS ((void));
 
 # endif
