@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* MakeCalls_Exit) ();
+extern void (* MakeCalls_Exit) ARGS ((void));
 
-extern void MakeCalls ARGS((tTree t));
+extern void MakeCalls ARGS ((tTree t));
 
-extern void BeginMakeCalls ();
-extern void CloseMakeCalls ();
+extern void BeginMakeCalls ARGS ((void));
+extern void CloseMakeCalls ARGS ((void));
 
 # endif
