@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* MakeForall_Exit) ();
+extern void (* MakeForall_Exit) ARGS ((void));
 
-extern void MakeForall ARGS((tTree t));
+extern void MakeForall ARGS ((tTree t));
 
-extern void BeginMakeForall ();
-extern void CloseMakeForall ();
+extern void BeginMakeForall ARGS ((void));
+extern void CloseMakeForall ARGS ((void));
 
 # endif
