@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* MakeInterface_Exit) ();
+extern void (* MakeInterface_Exit) ARGS ((void));
 
-extern void MakeInterface ARGS((tTree t));
+extern void MakeInterface ARGS ((tTree t));
 
-extern void BeginMakeInterface ();
-extern void CloseMakeInterface ();
+extern void BeginMakeInterface ARGS ((void));
+extern void CloseMakeInterface ARGS ((void));
 
 # endif
