@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* MakeLocal_Exit) ();
+extern void (* MakeLocal_Exit) ARGS ((void));
 
-extern void MakeLocal ARGS((tTree t));
+extern void MakeLocal ARGS ((tTree t));
 
-extern void BeginMakeLocal ();
-extern void CloseMakeLocal ();
+extern void BeginMakeLocal ARGS ((void));
+extern void CloseMakeLocal ARGS ((void));
 
 # endif
