@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* MakeOn_Exit) ();
+extern void (* MakeOn_Exit) ARGS ((void));
 
-extern void MakeOn ARGS((tTree t));
+extern void MakeOn ARGS ((tTree t));
 
-extern void BeginMakeOn ();
-extern void CloseMakeOn ();
+extern void BeginMakeOn ARGS ((void));
+extern void CloseMakeOn ARGS ((void));
 
 # endif
