@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* MakeOvUpdates_Exit) ();
+extern void (* MakeOvUpdates_Exit) ARGS ((void));
 
-extern void MakeOvUpdates ARGS((tTree t));
+extern void MakeOvUpdates ARGS ((tTree t));
 
-extern void BeginMakeOvUpdates ();
-extern void CloseMakeOvUpdates ();
+extern void BeginMakeOvUpdates ARGS ((void));
+extern void CloseMakeOvUpdates ARGS ((void));
 
 # endif
