@@ -7,25 +7,25 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
-# line 28 "MakeParLoops.puma"
+/* line 28 "MakeParLoops.puma" */
 
 
 #include "VarDescriptor.h"
 
 
 
-extern void (* MakeParLoops_Exit) ();
+extern void (* MakeParLoops_Exit) ARGS ((void));
 
-extern void MakeParLoops ARGS((tTree t));
+extern void MakeParLoops ARGS ((tTree t));
 
-extern void BeginMakeParLoops ();
-extern void CloseMakeParLoops ();
+extern void BeginMakeParLoops ARGS ((void));
+extern void CloseMakeParLoops ARGS ((void));
 
 # endif
