@@ -7,23 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
-# line 19 "MakeReductions.puma"
 
+extern void (* MakeReductions_Exit) ARGS ((void));
 
+extern void MakeReductions ARGS ((tTree t));
 
-
-extern void (* MakeReductions_Exit) ();
-
-extern void MakeReductions ARGS((tTree t));
-
-extern void BeginMakeReductions ();
-extern void CloseMakeReductions ();
+extern void BeginMakeReductions ARGS ((void));
+extern void CloseMakeReductions ARGS ((void));
 
 # endif
