@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* MakeRemapping_Exit) ();
+extern void (* MakeRemapping_Exit) ARGS ((void));
 
-extern void MakeRemapping ARGS((tTree t));
+extern void MakeRemapping ARGS ((tTree t));
 
-extern void BeginMakeRemapping ();
-extern void CloseMakeRemapping ();
+extern void BeginMakeRemapping ARGS ((void));
+extern void CloseMakeRemapping ARGS ((void));
 
 # endif
