@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* MakeSerial_Exit) ();
+extern void (* MakeSerial_Exit) ARGS ((void));
 
-extern void MakeSerial ARGS((tTree t));
+extern void MakeSerial ARGS ((tTree t));
 
-extern void BeginMakeSerial ();
-extern void CloseMakeSerial ();
+extern void BeginMakeSerial ARGS ((void));
+extern void CloseMakeSerial ARGS ((void));
 
 # endif
