@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* Normalization_Exit) ();
+extern void (* Normalization_Exit) ARGS ((void));
 
-extern void Normalization ARGS((tTree t));
+extern void Normalization ARGS ((tTree t));
 
-extern void BeginNormalization ();
-extern void CloseNormalization ();
+extern void BeginNormalization ARGS ((void));
+extern void CloseNormalization ARGS ((void));
 
 # endif
