@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* Optimization_Exit) ();
+extern void (* Optimization_Exit) ARGS ((void));
 
-extern void Optimization ARGS((tTree t));
+extern void Optimization ARGS ((tTree t));
 
-extern void BeginOptimization ();
-extern void CloseOptimization ();
+extern void BeginOptimization ARGS ((void));
+extern void CloseOptimization ARGS ((void));
 
 # endif
