@@ -7,25 +7,25 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
-# line 15 "ReductionComm.puma"
+/* line 15 "ReductionComm.puma" */
 
 
 #include "VarDescriptor.h"
 
 
 
-extern void (* ReductionComm_Exit) ();
+extern void (* ReductionComm_Exit) ARGS ((void));
 
-extern void ReductionCommunication ARGS((tTree body));
+extern void ReductionCommunication ARGS ((tTree body));
 
-extern void BeginReductionComm ();
-extern void CloseReductionComm ();
+extern void BeginReductionComm ARGS ((void));
+extern void CloseReductionComm ARGS ((void));
 
 # endif
