@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* RemoteAccess_Exit) ();
+extern void (* RemoteAccess_Exit) ARGS ((void));
 
-extern tTree TranslateRMA ARGS((tTree stmt));
+extern tTree TranslateRMA ARGS ((tTree stmt));
 
-extern void BeginRemoteAccess ();
-extern void CloseRemoteAccess ();
+extern void BeginRemoteAccess ARGS ((void));
+extern void CloseRemoteAccess ARGS ((void));
 
 # endif
