@@ -7,19 +7,19 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
 
-extern void (* Semantic_Exit) ();
+extern void (* Semantic_Exit) ARGS ((void));
 
-extern void Semantic ARGS((tTree t));
+extern void Semantic ARGS ((tTree t));
 
-extern void BeginSemantic ();
-extern void CloseSemantic ();
+extern void BeginSemantic ARGS ((void));
+extern void CloseSemantic ARGS ((void));
 
 # endif
