@@ -7,25 +7,25 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
-# line 20 "SpecialMap.puma"
+/* line 20 "SpecialMap.puma" */
 
 
 #include "VarDescriptor.h"
 
 
 
-extern void (* SpecialMap_Exit) ();
+extern void (* SpecialMap_Exit) ARGS ((void));
 
-extern bool IsSpecialisation ARGS((pvar actual_vard, pvar dummy_vard));
+extern rbool IsSpecialisation ARGS ((pvar actual_vard, pvar dummy_vard));
 
-extern void BeginSpecialMap ();
-extern void CloseSpecialMap ();
+extern void BeginSpecialMap ARGS ((void));
+extern void CloseSpecialMap ARGS ((void));
 
 # endif
