@@ -7,25 +7,25 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
-# line 25 "StrUnparse.puma"
+/* line 25 "StrUnparse.puma" */
 
 
 typedef char* String;
 
 
 
-extern void (* StrUnparse_Exit) ();
+extern void (* StrUnparse_Exit) ARGS ((void));
 
-extern void StrUnparse ARGS((String str, int len, tTree t));
+extern void StrUnparse ARGS ((String str, int len, tTree t));
 
-extern void BeginStrUnparse ();
-extern void CloseStrUnparse ();
+extern void BeginStrUnparse ARGS ((void));
+extern void CloseStrUnparse ARGS ((void));
 
 # endif
