@@ -7,22 +7,22 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
-# line 26 "TempFunctions.puma"
+/* line 26 "TempFunctions.puma" */
 
 
 
-extern void (* TempFunctions_Exit) ();
+extern void (* TempFunctions_Exit) ARGS ((void));
 
-extern void MakeTempFunctions ARGS((tTree t));
+extern void MakeTempFunctions ARGS ((tTree t));
 
-extern void BeginTempFunctions ();
-extern void CloseTempFunctions ();
+extern void BeginTempFunctions ARGS ((void));
+extern void CloseTempFunctions ARGS ((void));
 
 # endif
