@@ -7,25 +7,25 @@
 # define ARGS(parameters)	()
 # endif
 
-# ifndef bool
-# define bool char
+# ifndef rbool
+# define rbool char
 # endif
 
 # include "Tree.h"
 # include "Definitions.h"
 
-# line 20 "FUnparse.puma"
+/* line 20 "FUnparse.puma" */
 
 
 void FileUnparse ARGS((FILE *f, tTree t));
 
 
 
-extern void (* Unparse_Exit) ();
+extern void (* Unparse_Exit) ARGS ((void));
 
-extern void Unparse ARGS((tTree t));
+extern void Unparse ARGS ((tTree t));
 
-extern void BeginUnparse ();
-extern void CloseUnparse ();
+extern void BeginUnparse ARGS ((void));
+extern void CloseUnparse ARGS ((void));
 
 # endif
