@@ -4,7 +4,7 @@
 *                                                                        *
 *  Purpose : Data Structures for Label Tables                            *
 *                                                                        *
-*  Author : Dr. Thomas Brandes, GMD, Z1.HR                               *
+*  Author : Dr. Thomas Brandes, GMD, SCAI.WR                             *
 *                                                                        *
 *  Date   : 15. 4. 1993                                                  *
 *                                                                        *
@@ -25,3 +25,11 @@ extern void LabelDo ARGS((int label));
 extern void LabelDefine ARGS((int label));
            
 extern void LabelUse ARGS((int label));
+
+extern int GetNoLabels ();
+
+extern void GetLabelInfo ARGS((int no, int *label, int *used, int *defined));
+
+extern int GetLabelPos ARGS((int label));
+
+extern int GetNewLabel ();
