@@ -1,9 +1,19 @@
 # ifndef yyratc
 # define yyratc
 
-/* $Id: ratc.h,v 1.4 1992/08/07 14:35:05 grosch rel $ */
+/* $Id: ratc.h,v 1.7 1997/05/27 13:25:04 grosch rel $ */
 
-/* $Log: ratc.h,v $
+/*
+ * $Log: ratc.h,v $
+ * Revision 1.7  1997/05/27 13:25:04  grosch
+ * truncation of filenames to 8.3 format
+ *
+ * Revision 1.6  1994/04/05 09:00:35  grosch
+ * added prefix 'r' to true and false for improved portability
+ *
+ * Revision 1.5  1994/01/29  22:13:50  grosch
+ * renamed bool to rbool
+ *
  * Revision 1.4  1992/08/07  14:35:05  grosch
  * changed tString from # define to typedef
  *
@@ -23,17 +33,17 @@
 
 /* Ich, Doktor Josef Grosch, Informatiker, Sept. 1987 */
 
-# ifndef bool
-# define bool		int
+# ifndef rbool
+# define rbool		char
 # endif
-# ifndef true
-# define true		1
+# ifndef rtrue
+# define rtrue		1
 # endif
-# ifndef false
-# define false		0
+# ifndef rfalse
+# define rfalse		0
 # endif
 # ifndef cardinal
-# define cardinal	short
+# define cardinal	/* unsigned */ short
 # endif
 
 typedef char * tString;
