@@ -14,10 +14,18 @@
 #include <ctype.h>
 #include <stdlib.h>
 
+#ifdef WIN32
+#include <process.h>       /* getpid           */
+#else
+#include <unistd.h>        /* getpid           */
+#endif
+
 #include "global.h"
 #include "protocol.h"      /* protocol_errors  */
+#include "translate.h"
 
 #include "MySource.h"
+#include "Scanner.h"       /* BeginFile        */
 #include "Parser.h"
 #include "Tree.h"
 #include "MakeDefs.h"      /* SEMANTIC Phase 1 */
@@ -28,12 +36,16 @@
 
 #include "CallGraphFns.h"
 #include "Calling.h"
+#include "MakeInterface.h"
 
 #include "ImplMapping.h"     /* PARALLEL Phase 1 */
-#include "Normalization.h"
-#include "AnaOverlap.h"
-#include "InterProc.h"
-#include "AdaptAnalysis.h"
+#include "Normalization.h"   /* PARALLEL Phase 2 */
+#include "LooPo.h"           /* PARALLEL Phase 3 */
+#include "AnaOverlap.h"      /* PARALLEL Phase 4 */
+#include "InterProc.h"       /* PARALLEL Phase 5 */
+#include "AdaptAnalysis.h"   /* PARALLEL Phase 6 */
+
+/* include main routines of ADAPTOR PHASEs */
 
 #include "MakeArguments.h"
 #include "MakeParLoops.h"
@@ -42,6 +54,7 @@
 #include "Classification.h"
 #include "Optimization.h"
 #include "CodeGeneration.h"
+#include "Addressing.h"
 #include "FinalCode.h"
 
 /*************************************************************************
@@ -168,12 +181,13 @@ int strip_length;
 
 #ifdef WIN32
   sprintf (buffer, "copy %s %s", outfile_name, infile_name);
+  error = system (buffer);
   sprintf (buffer, "del %s", outfile_name);
+  error = system (buffer);
 #else
   sprintf (buffer, "mv -f %s %s", outfile_name, infile_name);
-#endif
-
   error = system (buffer);
+#endif
 
   if (error != 0)
 
@@ -262,51 +276,82 @@ int SourceCheck ()
      }
 }
 
+/*************************************************************************
+*                                                                        *
+*                                                                        *
+*                                                                        *
+*************************************************************************/
+
+int DoPhase (route, ph_id, ph_name, ph_routine, suffix, print_flag)
+
+char route[];
+int  ph_id;
+char ph_name [];
+void ph_routine ();
+char suffix [];
+int  print_flag;
+
+{ int errors;
+  char filename [30];
+
+  sprintf (filename, "adaptor.%s", suffix);
+
+  if (verbose_flag)
+
+    printf ("%s PHASE %d : %s\n", route, ph_id, ph_name);
+
+  open_protocol (filename);
+  ph_routine (TreeRoot);
+  close_protocol ();
+
+  errors = protocol_errors ();
+
+  if (errors > 0)
+
+    { sprintf (last_message, "%d Errors in %s %d : %s (see adaptor.%s)",
+               errors, route, ph_id, ph_name, suffix);
+      return errors;
+    }
+
+  if (!CheckTree (TreeRoot))
+
+    { sprintf (last_message, "Illegal Tree after %s %d : %s",
+               route, ph_id, ph_name);
+      return 1;
+    }
+
+  if (debug_flag && print_flag)
+
+     { sprintf (filename, "%s_unparse.f", suffix);
+       UnparseTree (filename, TreeRoot);
+     }
+
+  return 0;  /* no errors */
+
+}  /* DoPhase */
+
+/*************************************************************************
+*                                                                        *
+*                                                                        *
+*                                                                        *
+*************************************************************************/
+
 int SourceSemantic ()
 
 { int phase;
   int errors;
 
-  if (verbose_flag) printf ("SEMANTIC PHASE 1 : Make Definitions\n");
   phase = 1;
-  MakeDefs (TreeRoot); 
-  errors = protocol_errors ();
-  if (errors > 0)
-    { sprintf (last_message, 
-        "%d Errors in SEMANTIC 1 : MakeDefs (see adaptor.def)", errors);
-      goto Ende;
-    }
+  errors = DoPhase ("SEMANTIC", phase, "MakeDefs", MakeDefs, "def", 0);
+  if (errors) goto Ende;
 
-  if (!CheckTree (TreeRoot))
-    { sprintf (last_message, "Illegal Tree after PHASE 1 : MakeDefs");
-      goto Ende;
-    }
-
-  if (verbose_flag) printf ("SEMANTIC PHASE 2 : Checking\n");
   phase = 2;
-  BeginSemantic (); 
-  Semantic (TreeRoot); 
-  CloseSemantic (); 
-  errors = protocol_errors ();
-  if (errors > 0)
-    { sprintf (last_message, 
-         "%d Errors in SEMANTIC 2 : Checks (see adaptor.sem)", errors);
-      goto Ende;
-    }
-  if (!CheckTree (TreeRoot))
-    { sprintf (last_message, "Illegal Tree after PHASE 2 : Semantic");
-      goto Ende;
-    }
+  errors = DoPhase ("SEMANTIC", phase, "SemChecks", Semantic, "sem", 0);
+  if (errors) goto Ende;
 
-  if (verbose_flag) printf ("SEMANTIC PHASE 3 : Control Flow\n");
   phase = 3;
-  ControlFlow (TreeRoot); 
-  errors = protocol_errors ();
-  if (errors > 0)
-    { sprintf (last_message, 
-         "%d Errors in SEMANTIC 3 : Control Flow (see adaptor.cf)", errors);
-      goto Ende;
-    }
+  errors = DoPhase ("SEMANTIC", phase, "ControlFlow", ControlFlow, "cf", 1);
+  if (errors) goto Ende;
 
   sprintf (last_message, "Semantic Analysis was successful");
 
@@ -315,9 +360,10 @@ int SourceSemantic ()
   phase = 0;
 
 Ende: return (phase);
+
 }
 
-int SourceWrite ()
+void SourceWrite ()
 
 {  FILE *myFile;
    printf ("Write Tree on File test.out \n");
@@ -333,12 +379,10 @@ int SourceWrite ()
    sprintf (last_message, "Abtract Tree written to file test.out");
 }
 
-int SourceCalling (filename)
+void SourceCalling (filename)
 char *filename;
 
-{  FILE *myFile;
-
-   BeginCalling ();
+{  BeginCalling ();
    printf ("Starting Calling Analysis \n");
    Calling (TreeRoot); 
    printf ("Ending Calling Analysis\n");
@@ -399,272 +443,127 @@ int SourceParallelization ()
 { int phase;
   int errors;
 
-  /* PHASE 1 : DEFAULT DISTRIBUTION */
+  phase = 1;
+  errors = DoPhase ("PARALLELIZATION", phase, "Distribution",
+                     ImplMapping, "dist", 0);
+  if (errors) goto Ende;
 
-     if (verbose_flag) printf ("PARALLELIZATION 1 : Distribution\n");
-     phase = 1;
-     BeginImplMapping ();
-     ImplMapping (TreeRoot); 
-     CloseImplMapping ();
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message, 
-            "%d Errors in PHASE 1 : Distribution (see adaptor.dist)", errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message, "Illegal Tree after PHASE 1 : Distribution");
-         goto Ende;
-       }
+  phase = 2;
+  errors = DoPhase ("PARALLELIZATION", phase, "Normalization",
+                     Normalization, "normal", 1);
+  if (errors) goto Ende;
 
-  /* PHASE 2 : Normalization  */
+  phase = 3;
+  errors = DoPhase ("PARALLELIZATION", phase, "Loop Parallelization",
+                     LooPo, "auto", 0);
+  if (errors) goto Ende;
 
-     if (verbose_flag) printf ("PARALLELIZATION 2 : Normalization\n");
-     phase = 2;
-     BeginNormalization ();
-     Normalization (TreeRoot); 
-     CloseNormalization ();
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message, 
-            "%d Errors in PHASE 2 : Normalizations (see adaptor.normal)", 
-            errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message, "Illegal Tree after PHASE 2 : Normalization ");
-         goto Ende;
-       }
+  phase = 4;
+  errors = DoPhase ("PARALLELIZATION", phase, "Shadows",
+                     OverlapAnalysis, "ovl", 0);
+  if (errors) goto Ende;
 
-  /* PHASE 3 : Overlapping */
+  phase = 6;
+  errors = DoPhase ("PARALLELIZATION", phase, "ADAPTOR Analysis",
+                     AdaptAnalysis, "ana");
+  if (errors) goto Ende;
 
-     if (verbose_flag) printf ("PARALLELIZATION 3 : Overlapping\n");
-     phase = 3;
-     BeginAnaOverlap ();
-     OverlapAnalysis (TreeRoot); 
-     CloseAnaOverlap ();
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message, 
-            "%d Errors in PHASE 3 : Overlapping (see adaptor.ovl)", 
-            errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message, "Illegal Tree after PHASE 3 : Overlapping ");
-         goto Ende;
-       }
-
-  /* PHASE 4 : Interprocedural Analysis */
-
-     if (verbose_flag) 
-        printf ("PARALLELIZATION 4 : Interprocedural Analysis\n");
-     phase = 4;
-     BeginInterProc ();
-     Interprocedural (TreeRoot); 
-     CloseInterProc ();
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message, 
-            "%d Errors in PHASE 4 : Overlapping (see adaptor.ipa)", 
-            errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message, "Illegal Tree after PHASE 4 : Interprocedural");
-         goto Ende;
-       }
-
-     /* make an intermediate output */
-
-     if (debug_flag)
-        UnparseTree ("normal_unparse.f", TreeRoot);
-
-  /* PHASE 5 : ADAPTOR Analysis */
-
-     if (verbose_flag) 
-        printf ("PARALLELIZATION 5 : ADAPTOR Analysis\n");
-     phase = 5;
-     BeginAdaptAnalysis ();
-     AdaptAnalysis (TreeRoot); 
-     CloseAdaptAnalysis ();
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message, 
-            "%d Errors in PHASE 5 : Analysis (see adaptor.ana)", 
-            errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message, "Illegal Tree after PHASE 5 : Analysis");
-         goto Ende;
-       }
-
-     sprintf (last_message, "Parallelization without errors");
-     phase = 0;   /* no errors happened */
+  sprintf (last_message, "Parallelization without errors");
+  phase = 0;   /* no errors happened */
 
 Ende: return(phase);
 
 } /* SourceParallelization */
 
-int SourceAdapt (node_file)
-char * node_file;
+/*************************************************************************
+*                                                                        *
+*                                                                        *
+*                                                                        *
+*************************************************************************/
 
-{  FILE *f;
-   tTree HostRoot;
-   int errors;
-   int phase;
+int SourceAdapt ()
 
-   /* PHASE 1 : ADAPTOR Arguments */
+{ int errors;
+  int phase;
 
-     if (verbose_flag) printf ("ADAPTOR PHASE 1 : Arguments\n");
-     phase = 1;
-     MakeArguments (TreeRoot); 
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message, 
-            "%d Errors in PHASE 1 : Arguments (see adaptor.args)", errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message, "Illegal Tree after PHASE 1 : Arguments ");
-         goto Ende;
-       }
+  phase = 1;
+  errors = DoPhase ("ADAPTOR", phase, "Arguments",
+                     MakeArguments, "arg", 1);
+  if (errors) goto Ende;
 
-     /* make an intermediate output */
+  phase = 2;
+  errors = DoPhase ("ADAPTOR", phase, "Make Loops",
+                     MakeParLoops, "loop", 1);
+  if (errors) goto Ende;
 
-     if (debug_flag)
-        UnparseTree ("arg_unparse.f", TreeRoot);
+  phase = 3;
+  errors = DoPhase ("ADAPTOR", phase, "Find Homes",
+                     MakeHome, "home", 1);
+  if (errors) goto Ende;
 
-   /* PHASE 2 : ADAPTOR Make Loops */
+  phase = 4;
+  errors = DoPhase ("ADAPTOR", phase, "Extract Communication",
+                     MakeLocal, "local", 1);
+  if (errors) goto Ende;
 
-     if (verbose_flag) printf ("ADAPTOR PHASE 2 : Make Loops \n");
-     phase = 2;
-     MakeParLoops (TreeRoot); 
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message,
-            "%d Errors in PHASE 2 : MakeLoops (see adaptor.loops)", errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message, "Illegal Tree after PHASE 2 : Make Loops");
-         goto Ende;
-       }
+  phase = 5;
+  errors = DoPhase ("ADAPTOR", phase, "Classification",
+                     Classification, "class", 0);
+  if (errors) goto Ende;
 
-     /* make an intermediate output */
+  phase = 1;
+  errors = DoPhase ("OPTIMIZATION", phase, "Intraprocedural",
+                     Optimization, "opt", 1);
+  if (errors) goto Ende;
 
-     if (debug_flag)
-        UnparseTree ("loop_unparse.f", TreeRoot);
+  phase = 2;
+  errors = DoPhase ("OPTIMIZATION", phase, "Interprocedural Analysis",
+                     Interprocedural, "ipa", 0);
+  if (errors) goto Ende;
 
-   /* PHASE 3 : ADAPTOR Find Homes */
-
-     if (verbose_flag) printf ("ADAPTOR PHASE 3 : Find Homes \n");
-     phase = 3;
-     MakeHome (TreeRoot);  
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message, 
-            "%d Errors in PHASE 3 : FindHomes (see adaptor.home)", errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message, "Illegal Tree after PHASE 3 : Find Homes ");
-         goto Ende;
-       }
-
-     /* make an intermediate output */
-
-     if (debug_flag)
-        UnparseTree ("home_unparse.f", TreeRoot);
-
-   /* PHASE 4 : ADAPTOR Extract Communication */
-
-     if (verbose_flag) printf ("ADAPTOR PHASE 4 : Extract Communication \n");
-     phase = 4;
-     MakeLocal (TreeRoot);  
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message, 
-            "%d Errors in PHASE 4 : ExtractComm (see adaptor.local)", errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message, "Illegal Tree after PHASE 4 : ExtractComm ");
-         goto Ende;
-       }
-
-     /* make an intermediate output */
-
-     if (debug_flag)
-        UnparseTree ("loc_unparse.f", TreeRoot);
-
-   /* PHASE 5 : ADAPTOR Classification */
-
-     if (verbose_flag) printf ("ADAPTOR PHASE 5 : Classification\n");
-     phase = 5;
-     Classification (TreeRoot); 
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message, 
-            "%d Errors in PHASE 5 : Serial (see adaptor.class)", errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message, "Illegal Tree after PHASE 5 : Classification");
-         goto Ende;
-       }
-
-   /* PHASE 6 : ADAPTOR Optimization */
-
-     if (verbose_flag) printf ("ADAPTOR PHASE 6 : Optimization\n");
-     phase = 6;
-     Optimization (TreeRoot); 
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message, 
-            "%d Errors in PHASE 6 : Optimization (see adaptor.opt)", errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message, "Illegal Tree after PHASE 6 : Optimization");
-         goto Ende;
-       }
-
-     /* make an intermediate output */
-
-     if (debug_flag)
-        UnparseTree ("opt_unparse.f", TreeRoot);
-
-   /* PHASE 7 : ADAPTOR Code Generation */
-
-     if (verbose_flag) printf ("ADAPTOR PHASE 7 : Code Generation\n");
-     phase = 7;
-
-     NodeAdapt (TreeRoot); 
-
-     errors = protocol_errors ();
-     if (errors > 0)
-       { sprintf (last_message, 
-          "%d Errors in PHASE 7 : Code Generation (see adaptor.code)", errors);
-         goto Ende;
-       }
-     if (!CheckTree (TreeRoot))
-       { sprintf (last_message,
-                  "Illegal Node Tree after PHASE 7 : Code Generation");
-         goto Ende;
-       }
-
-     FinalCode (TreeRoot); 
-
-     UnparseTree (node_file, TreeRoot);
-
-     sprintf (last_message, "Translation successful, %s generated", node_file);
-     phase = 0;
+  sprintf (last_message, "Translation without errors");
+  phase = 0;   /* no errors happened */
 
 Ende: return(phase);
 }
 
+/*************************************************************************
+*                                                                        *
+*                                                                        *
+*                                                                        *
+*************************************************************************/
+
+int SourceCodeGen (node_file)
+char * node_file;
+
+{ int errors;
+  int phase;
+
+  phase = 1;
+  errors = DoPhase ("CODEGEN", phase, "Code Generation",
+                     NodeAdapt, "code", 1);
+  if (errors) goto Ende;
+
+  phase = 2;
+  errors = DoPhase ("CODEGEN", phase, "Addressing",
+                     Addressing, "addr", 1);
+  if (errors) goto Ende;
+
+  phase = 3;
+  errors = DoPhase ("CODEGEN", phase, "Final Code",
+                     FinalCode, "final", 0);
+  if (errors) goto Ende;
+
+  UnparseTree (node_file, TreeRoot);
+
+  sprintf (last_message,
+           "CodeGeneration successful, %s generated", node_file);
+  phase = 0;
+
+Ende: return(phase);
+
+} /* Code
+
 /*************************************************************************
 *                                                                        *
 *  translate_file (source_file, target_file)                             *
@@ -705,6 +604,15 @@ char *target_filename;
 
   ErrorCount = SourceParallelization ();
  
+  if (ErrorCount > 0)
+    { printf ("%s\n", last_message);
+      exit (-1);
+    }
+  else if (verbose_flag)
+      printf ("%s\n", last_message);
+
+  ErrorCount = SourceAdapt ();
+ 
   if (ErrorCount > 0)
     { printf ("%s\n", last_message);
       exit (-1);
@@ -715,12 +623,12 @@ char *target_filename;
   if (strcmp (target_filename, "") == 0)
 
    {  strcpy (node_file, model_srcfile [target_model]);
-      ErrorCount = SourceAdapt (node_file);
+      ErrorCount = SourceCodeGen (node_file);
    }
 
    else
  
-      ErrorCount = SourceAdapt (target_filename);
+      ErrorCount = SourceCodeGen (target_filename);
 
   if (ErrorCount > 0)
     { printf ("%s\n", last_message);
@@ -731,6 +639,12 @@ char *target_filename;
 
 }
 
+/*************************************************************************
+*                                                                        *
+*  interface_file (source_file, target_file)                             *
+*                                                                        *
+*************************************************************************/
+
 void interface_file (source_filename, target_filename)
 
 char *source_filename;
@@ -738,8 +652,6 @@ char *target_filename;
 
 { int ErrorCount;
 
-  char node_file [10];  /* name of file for node/cube/node1 program */
-
   ErrorCount = SourceParser (source_filename);
  
   if (ErrorCount > 0)
@@ -779,8 +691,6 @@ char *target_filename;
 
 { int ErrorCount;
 
-  char node_file [10];  /* name of file for node/cube/node1 program */
-
   ErrorCount = SourceParser (source_filename);
  
   if (ErrorCount > 0)
@@ -800,9 +710,9 @@ char *target_filename;
       printf ("%s\n", last_message);
 
   if (strcmp (target_filename, "") == 0)
-      ErrorCount = SourceInterface ("interface.h");
+      SourceInterface ("interface.h");
    else
-      ErrorCount = SourceCalling ("test.call");
+      SourceCalling ("test.call");
 
   if (ErrorCount > 0)
     { printf ("%s\n", last_message);
